/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ErrorPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.string");
dojo.requireLocalization("jazz.ui", "ErrorPageMessages");

(function() {

var messages;

/**
 * Constructs a new ErrorPage. All fields may be initialized in the constructor, or
 * changed later by using dijit get/set API
 * @name jazz.ui.ErrorPage
 * @class <p>An Error page widget for Web UIs</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 */ 
dojo.declare("jazz.ui.ErrorPage", [dijit._Widget, dijit._Templated], {
	
	/**
	 * Specify the summary to display on the error page. Optionally, may include an operational message id at the start.
	 * @type String
	 * @default ""
	 */
	summary: "",
	
	/**
	 * Specify the details of the error message. 
	 * @type String
	 * @default ""
	 */
	details: "",
	
	/**
	 * Specify the actions to display on the error page. 
	 * 
	 * Accepts an array with the following properties:
	 *  - iconClass
	 *  - icon
	 *  - onClick
	 *  - href
	 *  - label 
	 * @type Array
	 * @default null
	 */
	actions: null,
	
	/**
	 * Specify whether a "Go back to the previous page" action should be included. This will programmatically call window.history.back(); 
	 * @type String
	 * @default ""
	 */
	includeBackLink: true,
	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/ErrorPage.html"),
	
	constructor: function(args) {
		if (!messages) messages = dojo.i18n.getLocalization("jazz.ui", "ErrorPageMessages");
		this.messages = messages; 
	},
	_setActionsAttr: function(actions) {
		if (this._actionLinks) {
			while(this._actionLinks.length) {
				this._actionLinks.pop().destroyRecursive();
			}
		} else {
			this._actionLinks = [];
		}
		if (actions && dojo.isArray(actions)) {
			for (var i = 0; i < actions.length; i++) {
				this._actionLinks.push(new jazz.ui.errorPage.ActionLink(actions[i]).placeAt(this._actionLinksDiv));
			}
		} else {
			actions = null;
		}
		var totalActions = (actions && actions.length) || 0;
		if (this.includeBackLink) {
			totalActions++;
			this._actionLinks.push(new jazz.ui.errorPage.ActionLink({
				iconClass: "backIcon",
				label: this.messages.goBack, 
				onClick: function(e) {
					e.preventDefault();
					history.back();
				}
			}).placeAt(this._actionLinksDiv, "first"));
		}
		dojo.style(this._nextMessage,{display: totalActions > 1 ? "" : "none"});
		this.actions = actions;
	},
	_setIncludeBackLinkAttr: function(includeBackLink) {
		this.includeBackLink = includeBackLink;
		if (this._created || !this.actions) {
			this.set("actions", this.actions);
		}
	},
	_setSummaryAttr: function(summary) {
		if (summary.substring(0,4) === "@ID@") {
			var messageId = summary.substring(0,summary.indexOf(" "));
		} else {
			messageId = /^[A-Z]{5}[0-9]{3,5}[EWI]/.exec(summary);
		}
		if (messageId) {
			summary = summary.substring(summary.indexOf(" ") + 1) + "&nbsp;<span class='message-id'>" + dojo.string.substitute(messages.id, [messageId]) + "</span>";
		}
		this._errorMessage.innerHTML = summary;
		this.summary = summary;
	},
	_setDetailsAttr: function(details) {
		dojo.style(this._errorDetails,{display: details ? "block" : ""});
		this._errorDetails.innerHTML = details;
		this.details = details;
	},
	uninitialize: function() {
		while(this._actionLinks.length) {
			this._actionLinks.pop().destroyRecursive();
		}
	}
});

dojo.provide("jazz.ui.errorPage.ActionLink");

dojo.declare("jazz.ui.errorPage.ActionLink", [dijit._Widget, dijit._Templated], {
	icon:"",
	iconClass:"",
	label:"",
	href:"",
	templateString: "<div class=\"jazz-ui-errorPage-ActionLink\"><a dojoattachpoint=\"focusNode\" href=\"#\"><img class=\"error-icon\" src=\"${_blankGif}\" dojoattachpoint=\"_img\"><span class=\"error-label\" dojoattachpoint=\"_label\"></span></a></div>",
	_setIconAttr: function(icon) {
		this.icon = icon;
		this._img.src = icon || this._blankGif;
	},
	_setIconClassAttr: function(iconClass) {
		this.iconClass = iconClass;
		this._img.className = "error-icon" + (iconClass ? " " + iconClass : "");
	},
	_setLabelAttr: function(label) {
		this.label = label;
		this._label.innerHTML = label; 
	},
	_setHrefAttr: function(href) {
		this.href = href;
		this.focusNode.href = href;
	}
});

})();
