/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.StyledBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");

dojo.require("jazz.util.html");
dojo.require("jazz.util.ResizeNotifier");

dojo.require("jazz.ui.Menu");
dojo.require("jazz.ui.menu.MenuItem");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.MenuPopup");

dojo.requireLocalization("jazz.ui", "StyledBoxMessages");

(function() {
var MenuItem = jazz.ui.menu.MenuItem;
/**
 * @name jazz.ui.StyledBox.TrimStyles
 * @namespace <p>A set of constants for the {@link jazz.ui.StyledBox#trim} field.</p>
 * <p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.</p>
 */ 
var TrimStyles = /** @lends jazz.ui.StyledBox.TrimStyles */{
	/** 
	 * Blue trim.
	 * @constant
	 */
	BLUE: "sbBlue sbLight shadow",
	/** 
	 * Blue trim with an alpha channel drop shadow.
	 * @constant
	 */
	BLUE_ALPHA_SHADOW: "sbBlue sbDark shadow",
	/** 
	 * Gray trim.
	 * @constant
	 */
	GRAY: "sbGray sbLight shadow",
	/** 
	 * Gold trim.
	 * @constant
	 */
	GOLD: "sbGold sbLight shadow",
	/** 
	 * Green trim.
	 * @constant
	 */
	GREEN: "sbGreen sbLight shadow",
	/** 
	 * Teal trim.
	 * @constant
	 */
	TEAL: "sbTeal sbLight shadow",
	/** 
	 * Purple trim.
	 * @constant
	 */
	PURPLE: "sbPurple sbLight shadow",
	/** 
	 * Orange trim.
	 * @constant
	 */
	ORANGE: "sbOrange sbLight shadow",
	/** 
	 * No trim.
	 * @constant
	 */
	TRIMLESS: "sbTrimless",
	/** 
	 * Wizard style trim.
	 * @constant
	 */
	WIZARD: "wizard sbDark shadow no-separator",
	/** 
	 * Editor style trim.
	 * @constant
	 */
	EDITOR: "sbEditor sbLight shadow no-separator",
	/** 
	 * Editor style trim with a focus state.
	 * @constant
	 */
	EDITOR_FOCUS: "sbEditor editor-focus sbLight shadow tweak-border no-separator",
	/** 
	 * Yellow editor style trim.
	 * @constant
	 */
	EDITOR_YELLOW: "sbYellow sbEditor sbLight shadow tweak-border no-separator",
	/** 
	 * Login dialog trim.
	 * @constant
	 */
	LOGIN: "login sbDark border"
};
var messages;

/**
 * Constructs a new StyledBox. All fields may be initialized in the widget constructor, but the provided
 * setters must be used to change them for existing instances.
 * @name jazz.ui.StyledBox
 * @class <p>A box widget for web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 * @extends dijit._Container
 * @since 0.7
 */
dojo.declare("jazz.ui.StyledBox",
	[dijit._Widget, dijit._Templated, dijit._Container], /** @lends jazz.ui.StyledBox.prototype */{

	/** @private */
	templatePath: dojo.moduleUrl("jazz.ui", "templates/StyledBox.html"),
	widgetsInTemplate: true,
	
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
        primaryTitle : [{node: "_headerPrimarySpan", type: "innerText"},
                        {node: "_headerPrimaryAnchor", type: "innerText"}]
    }),
	
	/**#@+ @field */
	/**
	 * DOM node for contents of StyledBox.
	 * @type DomNode
	 * @required
	 */
	contentNode: null,
	/**
	 * Specify the width (eg. "400px", "40%"). This is the outer width of the StyledBox (including trim).
	 * Zero and empty string result in natural width (width: auto).
	 * @type String
	 * @default ""
	 */
	width: "",
	/**
	 * Specify the height. This is the inner height of the content area of the StyledBox (not including trim).
	 * Zero and empty string result in natural height (height: auto).
	 * @type String
	 * @default ""
	 */
	height: "",
	/**
	 * Specify the title of the box. 
	 * @type String
	 * @default ""
	 */
	primaryTitle: "",
	/**
	 * Specify the secondary title of the box. Appears beside primary title, in a lighter color 
	 * @type String
	 * @default ""
	 */
	secondaryTitle: "",
	/**
	 * Specify whether the title should be truncated with ellipsis' to fit in the header of the StyledBox. 
	 * @type Boolean
	 * @default true
	 */
	truncateTitle: true,
	/**
	 * Specifies a url for the title of the StyledBox to link to.
	 * @type String
	 * @default ""
	 */
	titleURL: "",
	/**
	 * Specify the trim style of the StyledBox. Valid values are available under {@link jazz.ui.StyledBox.TrimStyles}.
	 * @type String
	 * @default TRIMLESS
	 */
	trim: TrimStyles.TRIMLESS,
	/**
	 * Specify whether the header color should get darker on hover. 
	 * @type Boolean
	 * @default true
	 */
	headerHoverState: true,				//causes viewlet to change its rendering on mouse over
	/**
	 * Specify whether the header should default to hover state color. This is a slightly darker rendering for the header color.
	 * If true, {@link jazz.ui.StyledBox#headerHoverState} is ignored.
	 * @type Boolean
	 * @default false
	 */
	headerDefaultToHoverState: false,  	//causes box to default to its hover state rendering (if true, headerHoverState value is ignored)
	/**
	 * Specify whether the StyledBox should have a solid background color. 
	 * @type Boolean
	 * @default false
	 */
	showBackground: false,
	/**
	 * Specify a class for a 16x16 icon to appear in the header of the box to the left of the title.
	 * @type String
	 * @default ""
	 */
	iconClass: "",
	/**
	 * Specify the path to a 16x16 icon to appear in the header of the box to the left of the title. 
	 * @type String
	 * @default ""
	 */
	iconPath: "",
	/**
	 * Specify menu entries for the StyledBox drop down menu. Array of objects in the form
	 * {label: "", iconClass: "", onClick: function(){}, index: 0}
	 * @type Object[]
	 */
	menuItems: null,
	/**
	 * Specify a function that will be called when the user tries to open the menu (if specified, a menu arrow is added to the toolbar
	 * in the StyledBox header). The function must instantiate and return a dijit.Menu. If specified, menuProvider supersedes {@link jazz.ui.StyledBox#menuItems}. 
	 * @type Function
	 */
	menuProvider: null,
	/**
	 * Specify whether the StyledBox can be closed by the user. 
	 * @type Boolean
	 * @default true
	 */
	closable: true,
	/**
	 * Specify whether the StyledBox can be collapsed. 
	 * @type Boolean
	 * @default false
	 */
	collapsible: false,
	/**
	 * Specify whether the StyledBox is collapsed by default. 
	 * @type Boolean
	 * @default false
	 */
	collapsed: false,
	/**
	 * Hides the header title, icon and bgcolor. Header buttons appear if defined. No effect when using TRIMLESS. Ignores {@link jazz.ui.StyledBox#collapsible}. 
	 * @type Boolean
	 * @default false
	 */
	headerless: false,
	/**
	 * Causes the box to destroy the widget passed in 'contentNode' on close (if it's a widget or the domNode for a widget).
	 * @type Boolean
	 * @default false
	 */
	destroyContents: false,

	/**
	 * Called when the StyledBox is closed. Return false in your function to veto the close. 
	 * @type Function
	 */
	onClose: null,		//in your handlers, return boolean false to veto action (close, expand, collapse).
	/**
	 * Called when the StyledBox is opened. 
	 * @type Function
	 */
	onOpen: null,
	/**
	 * Called when the StyledBox is collapsed. Return false in your function to veto collapsing. 
	 * @type Function
	 */
	onCollapse: null,
	/**
	 * Called when the StyledBox is expanded. Return false in your function to veto expanding. 
	 * @type Function
	 */
	onExpand: null,
	/**#@-*/

	/** @private */
	constructor: function(args) {
		this._messages = messages = dojo.i18n.getLocalization("jazz.ui", "StyledBoxMessages");
		if (args.trim == TrimStyles.WIZARD)
			args.truncateTitle = false;
		setTimeout(dojo.hitch(this, function() {
			if (this._started)
				return;
			if (dojo.isDescendant(this.domNode, document.body)) {
				console.log("startup() has not been called on widget:" + this.id + " after it was inserted into the DOM.\nSee Foundation Workitem 231682");
				this.startup();
			}
		}));
	},
	/** @private */
	postCreate: function() {
		if (!this.serverTemplated) {
			this.setWidth();
			this.setHeight();
			if (!this.contentNode && this.getChildren().length == 0)
				this.contentNode = document.createElement("div");
			this.containerNode.appendChild(this.contentNode);
			this.setTrim(this.trim);
			this.setBackground(this.showBackground);
			if (this.menuItems || this.menuProvider)
				this._createMenuButton();
			this._updateCollapsedState();
			this.setClosable(this.closable);
			if (dojo.isFunction(this.onOpen))
				this.onOpen();
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#collapsed }. Controls whether or not the StyledBox is collapsed. 
	 * @param {Boolean} collapsed The new value.
	 * @param {Boolean} animate Indicates if the styled box should animate to the new state. Defaults to true if undefined.
	 */
	setCollapsed: function(collapsed, animate) {
		if (this.collapsed != collapsed) {
			this._toggleCollapse(null, animate);
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#closable}. Controls whether the close icon is present in the top right corner of the StyledBox. 
	 * @param {Boolean} closable The new value.
	 */
	setClosable: function(closable) {
		if (closable && !this._closeButton)
			this._closeButton = this.toolbar.addButton(this,this.close,{
					label: messages.close,
					className: "close"
				});
		else {
			this.toolbar.removeItem(this._closeButton);
			delete this._closeButton;
		}
		this.closable = closable;
	},
	_toggleCollapse: function(event, animate) {
		if (event) {
			if (!this.collapsible)
				return;
			dojo.stopEvent(event);
		}
		if (this._collapseAnimation)
			return;
		animate = animate !== false;
		var args = {
				node: this._collapsible,
				onEnd: dojo.hitch(this, this._onCollapseAnimationEnd),
				duration: 200};
		if (this.collapsed) {
			if (dojo.isFunction(this.onExpand) && this.onExpand() === false)
				return;
			if (animate)
				this._collapseAnimation = dojo.fx.wipeIn(args).play();
		} else {
			//call onCollapse callback, if exists
			if (dojo.isFunction(this.onCollapse) && this.onCollapse() === false)
				return;
			if (animate)
				this._collapseAnimation = dojo.fx.wipeOut(args).play();
		}
		if (!animate)
			this._onCollapseAnimationEnd();
	},
	_refreshStyles: function() {
		this._applyClass("hover",
				(this.headerHoverState && this._headerMouseOver || this._menuShowing)
						|| this._headerVisible
						|| this.headerDefaultToHoverState,
				this._header);
		this._applyClass("j-hide",
				!this.collapsed && (this.headerless || this.trim == TrimStyles.TRIMLESS) && !this._headerVisible,
				this._header, this._separator);
		this._applyClass("j-hide",
				this.trim != TrimStyles.TRIMLESS || !this._mouseOver,
				this._toggleHeader);
	},
	_onCollapseAnimationEnd: function() {
		this.collapsed = !this.collapsed;
		this._updateCollapsedState();
		delete this._collapseAnimation;
	},
	_updateCollapsedState: function() {
		var collapsed = this.collapsed;
		if (this._expandCollapseButton) {
			var index = this.toolbar.getIndex(this._expandCollapseButton);
			this.toolbar.removeItem(this._expandCollapseButton);
		}
		dijit.setWaiState(this._headerText, "expanded", !collapsed);
		this._collapsible.style.display = this.collapsed ? "none" : "";
		if (this.collapsible) {
			this._expandCollapseButton = this.toolbar.addButton(this, this._toggleCollapse, {
				className: collapsed ? "expand" : "collapse",
				label: collapsed ? messages.expand : messages.collapse,
				index: index == null ? 1 : index
			});
		}
	},
	/**
	 * Adds a menu item to the drop down.  
	 * @param {Object} args Object in the form {label: "", iconClass: "", onClick: function(), index: 0} where index is optional
	 */
	addMenuItem: function(args) {
		// args format: {/* string */ label, /* function */ onClick, /* string */ iconClass, /* int (optional)*/ index}
		if (this.menuProvider) {
			console.error("Cannot use addMenuItem when using a menuProvider.");
			return;
		}
		if (!this.menuItems) {
			this.menuItems = [];
			this._createMenuButton();
		}
		if (typeof args.index == "number") {
			this.menuItems.splice(args.index,0,args);
		} else {
			this.menuItems.push(args);	
		}
		// if the menu has already been created, insert it
		if (this._menu) {
			if (typeof args.index == "number") {
				this._menu.addChild(new MenuItem(args), args.index);	
			} else {
				this._menu.addChild(new MenuItem(args));
			}
		}
	},
	_createMenuButton: function() {
		if (!this._openMenuButton) {
			var button = this._openMenuButton = this.toolbar.addButton(null, null, {
				label: messages.menu,
				className: "menu",
				index: 0
			});
			this._menuPopup = new jazz.ui.MenuPopup({
				align: "right",
				menuProvider: dojo.hitch(this, this._createMenu)
			}, button.domNode);
		}
	},
	/*
	 * Bust-out hack - add styles to make the box look right when all the
	 * overflow: hidden rules are removed from the container nodes.
	 */
	bustOutHandler: function(isBustOut) {
		dojo[isBustOut ? "addClass" : "removeClass"](this.domNode, "busted-out");
	},
	/**
	 * Closes the StyledBox and removes it from the DOM.
	 * @noparams
	 */
	close: function(event) {
		if (event)
			dojo.stopEvent(event);
		
		// call onClose callback function
		if (dojo.isFunction(this.onClose) && this.onClose(this) === false)
			return false;
		
		if (dojo.isIE == 7)
			this._onClose(); //IE7 does not fadeOut alpha PNG drop shadows properly (shows ugly black bars) - so just close it
		else {
			this._fadeOutAnim = dojo.fadeOut({
				node: this.domNode,
				duration: 200,
				onEnd: dojo.hitch(this, this._onClose)
			});
			this._fadeOutAnim.play();
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#iconPath}. Adds/updates the icon in the header of the StyledBox. 
	 * @param {String} iconPath The path to to the new icon.
	 */
	setIcon: function(iconPath) {
		this.set('iconPath', iconPath);
	},
	_setIconPathAttr: function(iconPath) {
		this.iconPath = iconPath;
		this._applyClass("j-hide", !iconPath, this._iconImage);
		if (iconPath)
			this._iconImage.src = iconPath;
	},
	_setIconClassAttr: function(iconClass) {
		if (iconClass) {
			dojo.addClass(this._iconImage, iconClass);
			this.set('iconPath', this._blankGif);
		} else if (iconClass != this.iconClass) {
			dojo.removeClass(this._iconImage, this.iconClass);
			this.set('iconPath', null);
		}
		this.iconClass = iconClass;
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#height}. 
	 * @param {String} height A string representing the new height, including units.
	 */
	setHeight: function(/*String with unit*/ height) {
		if (height) {
			this.containerNode.style.height = height;
			this.height = height;
		} else if (this.height) {
			this.containerNode.style.height = this.height;
		} else {
			this.containerNode.style.height = "";
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#width}. 
	 * @param {String} width A string representing the new width, including units.
	 */
	setWidth: function(/*String with unit*/ width){
		if (width) {
			this.domNode.style.width = width;
			this.width = width;
		} else if (this.width){
			this.domNode.style.width = this.width;
		} else {
			this.domNode.style.width = "";
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#primaryTitle}. 
	 * @param {String} title The new primary title.
	 */
	setTitle: function(/* string */ title) {
		this.set('primaryTitle', title);
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#secondaryTitle}. 
	 * @param {String} title The new secondary title.
	 */
	setSecondaryTitle: function(/* string */ title) {
		this._setSecondaryTitleAttr(title);
	},
	_setSecondaryTitleAttr: function(title) {
		if (dojo.isString(title) && title.length)
			title = "&nbsp;" + title;
		this._headerSecondary.innerHTML = title;
		this.secondaryTitle = title;
		this._firefoxWorkaround();
	},
	_firefoxWorkaround: function() {
		var self = this;
		if (dojo.isFF && self._started) {
			function workaround() {
				var correction = self._headerSecondary.clientWidth + "px";
				self._headerSecondary.style.marginLeft = "-" + correction;
				self._primaryHeaderText.style.marginRight = correction;
			}
			workaround();
			setTimeout(workaround);
		}
	},
	startup: function() {
		if (!dojo.isDescendant(this.domNode, document.body)) {
			console.log("ERROR: Startup was called prematurely on widget " + this.id);
			return;
		}
		this.inherited(arguments);
		this._firefoxWorkaround();
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#titleURL}. The title will be shown
	 * as a hyperlink only when a valid URL is set.
	 * @param {String} url The new url for the title.
	 */
	setTitleURL: function(/* string */ url) {
		this.set('titleURL', url);
	},
	_setTitleURLAttr: function(url) {
		this.titleURL = url ? url : "";
		this._headerPrimaryAnchor.href = url;
		if (url) {
			dojo.addClass(this._headerPrimarySpan,"j-hide");
			dojo.removeClass(this._headerPrimaryAnchor,"j-hide");
		} else {
			dojo.addClass(this._headerPrimaryAnchor,"j-hide");
			dojo.removeClass(this._headerPrimarySpan,"j-hide");
		}
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#trim}. See {@link jazz.ui.StyledBox.TrimStyles} for trim constants.
	 * @param {String} url The new trim style.
	 */
	setTrim: function(trim) {
		if (this.trim != null && this.trim !== trim)
			dojo.removeClass(this.domNode,this.trim);
		this.trim = trim;
		dojo.addClass(this.domNode,trim);
		this._refreshStyles();
	},
	/**
	 * Setter for {@link jazz.ui.StyledBox#showBackground}.
	 * @param {Boolean} background The new showBackground state.
	 */
	setBackground: function(value) {
		this._applyClass("showBackground", value, this.domNode);
	},
	_toggleTrimlessHeader: function(event) {
		if (event) event.preventDefault();
		this._headerVisible = !this._headerVisible;
		this._applyClass("headerVisible", this._headerVisible, this._toggleHeader);
		this._refreshStyles();
	},
	_createMenu: function (){
		if (this._menu && !this._menu._destroyed)
			return this._menu;
		var menu;
		if (this.menuProvider) {
			menu = this.menuProvider();
		} else {
			menu = new jazz.ui.Menu();
			// add known menu contributions
			if (this.menuItems) {
				for (var i=0;i<this.menuItems.length;i++) {
					if (this.menuItems[i].separator) {
						menu.addChild(new dijit.MenuSeparator());
						continue;
					}
					if (!this.menuItems[i].label) {
						console.error("Error creating menu:  Menu item at index "+i+" is missing a label.");
						continue;
					}
					menu.addChild(new MenuItem(this.menuItems[i]));
				}
			}
		}
		if (menu instanceof dijit._Widget) {
			dojo.addClass(menu.domNode, "jazz-ui-StyledBox-menu");
			menu._connects.push([
				dojo.connect(menu, "onOpen", this, this._onMenuOpen),
				dojo.connect(menu, "onClose", this, this._onMenuClose)]);
			return this._menu = menu;
		} else {
			throw new Error("Error creating menu.");
		}
	},
	_onMenuOpen: function(event) {
		this._menuShowing = true;
		this._refreshStyles();
	},
	_onMenuClose: function(event) {
		this._menuShowing = false;
		this._refreshStyles();
	},
	_onMouseOver: function() {
		this._mouseOver = true;
		this._refreshStyles();
	},
	_onMouseOut: function() {
		this._mouseOver = false;
		this._refreshStyles();
	},
	_onHeaderMouseOver: function() {
		this._headerMouseOver = true;
		this._refreshStyles();
		var node = this._primaryHeaderText;
		if (node.scrollWidth > node.clientWidth)
			node.title = this.primaryTitle;
		else
			node.title = "";
	},
	_onHeaderMouseOut: function() {
		this._headerMouseOver = false;
		this._refreshStyles();
	},
	_applyClass: function(clazz, condition, node /*...*/) {
		for (var i = 2; i < arguments.length; i++)
			if (condition)
				dojo.addClass(arguments[i], clazz);
			else
				dojo.removeClass(arguments[i], clazz);
	},
	_onClose: function() {
		if (this._closed) return;
		this._closed = true;
		if (this._fadeOutAnim) {
			this._fadeOutAnim.stop();
			delete this._fadeOutAnim;
		}
		var widget = dijit.byId(this.contentNode.id);
		if (widget) {
			if (this.destroyContents) { // Destroy the content node widget for the dialog
				if (dojo.isFunction(widget.destroyRecursive)) {
				    widget.destroyRecursive();
				}
			} else {
				// remove widget from dom to ensure it isn't destroyed.
				var dom = widget.domNode;
				dom && dom.parentNode && dom.parentNode.removeChild(dom);
			}
		}
		this.destroy();
	},
	destroy: function() {
		if (this._collapseAnimation)
			this._collapseAnimation.stop();
		if (this._menuPopup) {
			this._menuPopup.destroy();
			delete this._menuPopup;
		}
		if (this._menu) {
			this._menu.destroy();
			delete this._menu;
		}
		this.inherited(arguments);
	}
});

jazz.ui.StyledBox.TrimStyles = TrimStyles;
})();