/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {

dojo.provide("jazz.jsp._ServerTemplated");

dojo.require("dijit._Templated");

var _Templated = dijit._Templated;

var buildDijitRendering = _Templated.prototype.buildRendering;

function _attachPoint(values, widget, toAttach) {
	if (values) {
		values = values.split(/\s*,\s*/);
		for (i = 0; i < values.length; i++) {
			var tokens = values[i].split(":");
			var point = tokens[0];
			if (tokens[1])
				widget = dijit.byId(tokens[1]);
			if (dojo.isArray(widget[point]))
				widget[point].push(toAttach);
			else
				widget[point] = toAttach;
			widget._attachPoints.push(point);
		}
	}
}

function _attachEvent(values, widget, eventSource) {
	if (values) {
		// NOTE: we want to support attributes that have the form
		// "domNode: nativeElement; ..."
		values = values.split(/\s*,\s*/);
		for(i = 0; i < values.length; i++) {
			var tokens = values[i].split(":");
			var event = tokens[0];
			var functionName = tokens[1] || tokens[0];
			if (tokens[2])
				widget = dijit.byId(tokens[2]);
			widget._attachEvents.push(
					widget.connect(eventSource, event, functionName));
		}
	}	
}

function buildServerRendering(widget) {
	var node = widget.domNode = widget.srcNodeRef;
	delete widget.srcNodeRef;

	/* dijit.getEnclosingWidget is broken without this because
	 * even though the widget has registered itself by its "id" attribute,
	 * the lookup requires the "widgetId" attribute to be set.
	 */
	node.setAttribute("widgetId", widget.id);
	
	var i, currentNode, stack = [node], values, foundNestedWidgets;
	
	while (currentNode = stack.pop()) {
		// Process it's attach points
		_attachPoint(currentNode.getAttribute("dojoAttachPoint"), widget, currentNode);
		_attachEvent(currentNode.getAttribute("dojoAttachEvent"), widget, currentNode);

		// waiRole, waiState
		// TODO: remove this in 2.0, templates are now using role=... and aria-XXX=... attributes directly
		var role = currentNode.getAttribute("waiRole");
		if (role)
			dijit.setWaiRole(currentNode,role);
		values = currentNode.getAttribute("waiState");
		if (values) {
			values = values.split(/\s*,\s*/);
			for (i = 0; i < values.length; i++) {
				var tokens = values[i].split('-');
				if (tokens.length == 2)
					dijit.setWaiState(tokens[0], tokens[1]);
			}
		}
		
		if (currentNode.children) {
			for (i = 0; i < currentNode.children.length; i++) {
				var child = currentNode.children[i];
				if (child.getAttribute("dojoType")) {
					foundNestedWidgets = true;
					continue;
				}
            	stack.push(child);
        	}
    	}
	}

	if(foundNestedWidgets) {
		// Store widgets that we need to start at a later point in time
		var cw = (widget._startupWidgets = dojo.parser.parse(node, {
			noStart: !widget._earlyTemplatedStartup,
//			template: false,
//			inherited: {dir: widget.dir, lang: widget.lang},
//			scope: "dojo",	// even in multi-version mode templates use dojoType/data-dojo-type
			propsThis: widget	// so data-dojo-props of widgets in the template can reference "this" to refer to me
		}));

		widget._supportingWidgets = dijit.findWidgets(node);
		
		for (i = 0; i < cw.length; i++) {
			var child = cw[i];
			_attachPoint(child.jazzAttachPoint, widget, child);
			_attachEvent(child.jazzAttachEvent, widget, child);
		}
	}
}

dojo.extend(dijit._Templated, {
	stopParser : true,
	serverTemplated : false,
	
	buildRendering: function () {
		if (this.serverTemplated)
			buildServerRendering(this);
		else
			buildDijitRendering.apply(this);
	}
});

dojo.extend(dijit._Widget, {
	jazzAttachEvent: "",
	jazzAttachPoint: ""
});

})();
