/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.client.internal.ParameterRegistry"); //$NON-NLS-1$

dojo.require("jazz.core.RegistryFactory"); //$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$

(function() { // module container

var RegistryFactory = jazz.core.RegistryFactory;
var loader = jazz.core.loader;

com.ibm.team.reports.web.client.internal.ParameterRegistry = {
	
	// ---- API ------------------------------------------------------------------------------------------------------------	
	
	getContributedParameterHandler: function(parameterType) {
		if (this._contributedHandlers == null) {
			this._loadContributedParameterHandlers();
		}
		var candidates = [];
		for (var key in this._contributedHandlers) {
			if (this._matches(key, parameterType)) {
				candidates.push({key: key, value: this._contributedHandlers[key]});
			}
		}
		if (candidates.length == 0) {
			// No matches. Return the default handler, if one exists.
			return this._defaultHandler;
		}
		if (candidates.length == 1) {
			return candidates[0].value;
		}
		// Sort by length, longest first
		var sortFunction = function(a, b) {
			return b.key.length - a.key.length;
		};
		candidates.sort(sortFunction);
		
		for (var i = 0; i < candidates.length; i++) {
			loader.load_sync(candidates[i].value);
			var obj = dojo.getObject(candidates[i].value);
			if (obj.accept(parameterType)) {
				return candidates[i].value;
			}
		}
		// No matches. Return the default handler, if one exists.
		return this._defaultHandler;
	},
	

	// ---- implementation -------------------------------------------------------------------------------------------------

	_contributedHandlers: null,
	_defaultHandler: null,

	_matches: function(extension, parameterType) {
		var exp = "^";
		if (extension.indexOf("*") == (extension.length - 1)) {
			var strippedExtension = extension.substring(0, extension.indexOf("*"));
			exp += strippedExtension;
		} else {
			exp += extension;
			exp += "$";
		}
		var regexp = new RegExp(exp);
		
		return regexp.test(parameterType);
	},
	
	_loadContributedParameterHandlers: function() {
		var extensionPoint = RegistryFactory.getRegistry().getExtensionPoint("com.ibm.team.reports.web.parameter"); //$NON-NLS-1$

		var contributedHandlers = [];
		var defaultHandler = null;
		
		dojo.forEach(extensionPoint.getExtensions(), function(extension) {
			dojo.forEach(extension.getConfigurationElements(), function(configElement) {
				if (configElement.getAttribute("default") == "true") {
					defaultHandler = configElement.getAttribute("widgetId");
				} else {
					contributedHandlers[configElement.getAttribute("parameterType")] = configElement.getAttribute("widgetId");
				}
			});
		});
		
		this._contributedHandlers = contributedHandlers;
		this._defaultHandler = defaultHandler;
	},

	__sentinel: null // terminates this class definition
	
};

})();