/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007, 2012,
 * 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("jazz.app.context");

(function() { // module container

	var session = com.ibm.team.repository.web.client.session;
	var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;

	com.ibm.team.reports.web.client.internal.ReportsClient = {

		ALL : 0,
		SHARED : 1 << 0,
		PRIVATE : 1 << 2,
		EXCLUDE_MICRO : 1 << 4,
		LOCAL : 1 << 0,
		REMOTE : 1 << 1,

		SERVICE_URI : "com.ibm.team.reports.common.internal.service.IReportRestService",

		/**
		 * Fetch whether the current user can deploy queries.
		 */
		getCanDeployQuery : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getCanDeployQuery");
		},

		/**
		 * Fetch whether the current user can manage folders.
		 */
		getCanManageFolder : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getCanManageFolder");
		},

		/**
		 * Fetch whether the current user can see reports UI.
		 */
		getCanSeeReports : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getCanSeeReports");
		},

		/**
		 * Fetch whether the current user can see reports UI.
		 */
		getShowArchived : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getShowArchived");
		},
		
		/**
		 * Fetch all visible root folder descriptors. The client does not need
		 * to supply arguments. This method will supply the
		 * currently-authenticated user and current projectArea.
		 */
		getSystemRootFolderDescriptors : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"getSystemRootFolderDescriptors");
		},

		/**
		 * Fetch query descriptors. This method will supply the projectAreaUUID.
		 */
		getQueryDescriptors : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler, "getQueryDescriptors",
					args);
		},

		/**
		 * Fetch report descriptors. This method will supply the
		 * projectAreaUUID.
		 */
		getReportDescriptors : function(responseHandler, args) {
			if (args == undefined) {
				args = {};
			}
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler, "getReportDescriptors",
					args);
		},

		/**
		 * Fetch the report descriptor identified by the argument. Caller must
		 * supply reportUUID. This method will supply the projectAreaUUID.
		 */
		getReportDescriptor : function(responseHandler, args) {
			if (args == undefined) {
				args = {};
			}
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler, "getReportDescriptor",
					args);
		},

		/**
		 * Fetch folders. This method will supply the projectAreaUUID.
		 */
		getFolderDescriptors : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler, "getFolderDescriptors",
					args);
		},

		/**
		 * Fetch a single folder.
		 */
		getFolderDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "getFolderDescriptor",
					args);
		},

		/**
		 * Fetches report descriptors missing from the given process area.
		 */
		getMissingSharedReportDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"getMissingSharedReportDescriptors", args);
		},

		/**
		 * Fetch parameters
		 */
		getParameters : function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "getParameters", args);
		},

		/**
		 * Fetch parameter values
		 */
		getParameterValues : function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "getParameterValues",
					args);
		},

		/**
		 * Fetch parameter group values
		 */
		getParameterGroupValues : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"getParameterGroupValues", args);
		},
		
		getCustomParams: function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "getCustomParams", args);
		},

		/**
		 * Get the parameters section for the given query.
		 */
		getQueryParametersSection : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"getQueryParametersSection", args);
		},

		/**
		 * Get the rendered report for the given query.
		 */
		getRenderedQuery : function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "getRenderedQuery",
					args);
		},
		
		/**
		 * Check whether we should display the legacy snapshot ui or not
		 */
		getShouldHideLegacy : function(responseHandler){
			this._invokeReportsService(responseHandler, "getShouldHideLegacy");
		},
		
		/**
		 * Get the current data warehouse provider
		 */
		getDataWarehouseProvider : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getDataWarehouseProvider");
		},
		
		/**
		 * Get the current XDC authentication type
		 */
		getXdcAuthType : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getXdcAuthType");
		},
		
		/**
		 * Get the list of etl job history
		 */
		getEtlJobHistory : function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "getEtlJobHistory",
					args);
		},
		
		/**
		 * Get the list of running etl jobs
		 */
		getRunningEtlJobs : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getRunningEtlJobs");
		},

		/**
		 * Get the rendered report for the given query.
		 */
		postRenderQuery : function(responseHandler, args) {
			this
					._invokeReportsService(responseHandler, "postRenderQuery",
							args);
		},

		/**
		 * Get the rendered report for the given report.
		 */
		getRenderedReport : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler, "getRenderedReport",
					args);
		},

		/**
		 * Get the rendered report for the given report.
		 */
		postRenderReport : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler, "postRenderReport",
					args);
		},
		
		/**
		 * Fetch parameter values
		 */
		postComputeParameterValues : function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "postComputeParameterValues",
					args);
		},

		/**
		 * Get whether the given report exists
		 */
		getDoesReportDescriptorExist : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler,
					"getDoesReportDescriptorExist", args);
		},

		/**
		 * Get the default report for the project area
		 */
		getDefaultReportDescriptor : function(responseHandler) {
			var currentProjectArea = jazz.app.context.get();
			var args = {
				projectAreaUUID : currentProjectArea.itemId
			};
			this._invokeReportsService(responseHandler,
					"getDefaultReportDescriptor", args);
		},

		/**
		 * Helper function. ProcessClient doesn't provide this.
		 */
		getProcessAreaByUUID : function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "getProcessAreaByUUID",
					args);
		},

		/**
		 * Get the query descriptor for the queryUUID provided
		 */
		getQueryDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler, "getQueryDescriptor",
					args);
		},

		/**
		 * Delete the given folder.
		 */
		postDeleteFolderDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postDeleteFolderDescriptor", args);
		},

		/**
		 * Delete the given folders.
		 */
		postDeleteFolderDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postDeleteFolderDescriptors", args);
		},

		/**
		 * Delete the given query.
		 */
		postDeleteQueryDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postDeleteQueryDescriptor", args);
		},

		/**
		 * Delete the given queries.
		 */
		postDeleteQueryDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postDeleteQueryDescriptors", args);
		},

		/**
		 * Delete the given report descriptor.
		 */
		postDeleteReportDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postDeleteReportDescriptor", args);
		},

		/**
		 * Delete the given report descriptor.
		 */
		postDeleteReportDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postDeleteReportDescriptors", args);
		},

		/**
		 * Rename the given folder.
		 */
		postRenameFolderDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postRenameFolderDescriptor", args);
		},

		/**
		 * Rename the given query.
		 */
		postRenameQueryDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postRenameQueryDescriptor", args);
		},

		/**
		 * Reset the given report descriptor to its factory defaults.
		 */
		postResetReportDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postResetReportDescriptor", args);
		},

		/**
		 * Reset the given report descriptors to their factory defaults.
		 */
		postResetReportDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postResetReportDescriptors", args);
		},

		/**
		 * Save a folder with the given arguments. This could be creating a new
		 * one or saving an existing one.
		 */
		postSaveFolderDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postSaveFolderDescriptor", args);
		},

		/**
		 * Save a query with the given arguments. This could be creating a new
		 * one or saving an existing one.
		 */
		postSaveQueryDescriptor : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler,
					"postSaveQueryDescriptor", args);
		},
		
		/**
		 * Save a template with the given arguments. This could be creating a
		 * new one or saving an existing one.
		 */
		postSaveReportDescriptor : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId;
			}
			this._invokeReportsService(responseHandler,
					"postSaveReportDescriptor", args);
		},

		/**
		 * Save report descriptors with the given information. This is a
		 * shortcut so the web UI doesn't have to call
		 * postSaveReportDescriptor() in a loop for every report descriptor to
		 * create.
		 */
		postSaveMissingReportDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postSaveMissingReportDescriptors", args);
		},

		/**
		 * Set the given report resource as the default template for viewing in
		 * the APT editor
		 */
		postSetDefaultReportDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postSetDefaultReportDescriptor", args);
		},

		/**
		 * Get whether the current user is a data warehouse administrator
		 */
		getIsDWAdmin : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getIsDWAdmin");
		},

		/**
		 * Get the list of snapshot names
		 */
		getAllSnapshotNames : function(responseHandler) {
			this._invokeReportsService(responseHandler, "getAllSnapshotNames");
		},

		/**
		 * Get the list of snapshot descriptors
		 */
		getSnapshotDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"getSnapshotDescriptors", args);
		},

		/**
		 * Get the snapshot descriptor
		 */
		getSnapshotDescriptor : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"getSnapshotDescriptor", args);
		},

		getSnapshotSchedulingDefaults : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"getSnapshotSchedulingDefaults");
		},

		/**
		 * Data Warehouse Administration functions.
		 */
		postUpdateAllSnapshotData : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"postUpdateAllSnapshotData");
		},

		postUpdateSomeSnapshotData : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postUpdateSomeSnapshotData", args);
		},
		
		postUpdateAllRemoteSnapshotData : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"postUpdateAllRemoteSnapshotData");
		},

		postUpdateSnapshotData : function(responseHandler, args) {
			if(args.delta == undefined || args.delta == null){
				args.delta = true;
			}
			this._invokeReportsService(responseHandler,
					"postUpdateSnapshotData", args);
		},

		postPingRemoteSnapshot : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postPingRemoteSnapshot", args);
		},

		postExportAllSnapshotData : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postExportAllSnapshotData", args);
		},

		postExportSnapshotData : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postExportSnapshotData", args);
		},

		postImportSnapshotData : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postImportSnapshotData", args);
		},

		postInitializeDataWarehouse : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"postInitializeDataWarehouse");
		},

		getDataWarehouseInitializationStatus : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"getDataWarehouseInitializationStatus");
		},

		postCreateAllSnapshotSchemas : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"postCreateAllSnapshotSchemas");
		},

		postCreateSnapshotSchema : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postCreateSnapshotSchema", args);
		},

		postUpdateParameterValues : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"postUpdateParameterValues");
		},

		postSaveQueryParameters : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postSaveQueryParameters", args);
		},
		
		getPerformDataWarehouseDiagnostics : function(responseHandler) {
			this._invokeReportsService(responseHandler,
					"getPerformDataWarehouseDiagnostics");
		},

		postMoveQueryDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postMoveQueryDescriptors", args);
		},

		postCopyQueryDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postCopyQueryDescriptors", args);
		},
		postMoveFolderDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postMoveFolderDescriptors", args);
		},

		postCopyFolderDescriptors : function(responseHandler, args) {
			this._invokeReportsService(responseHandler,
					"postCopyFolderDescriptors", args);
		},
		
		/**
		 * Invokes the requested method on the Reports REST service.
		 */
		_invokeReportsService : function(responseHandler, method, args) {
			// args is optional, so test whether or not it was sent
			var serviceRequest;
			if (arguments.length == 2) {
				serviceRequest = new ServiceRequest(this.SERVICE_URI, method);
			} else {
				serviceRequest = new ServiceRequest(this.SERVICE_URI, method,
						args);
			}
			TeamServerClient.invokeService(serviceRequest, responseHandler);
		}
	};

})();