/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.common.internal.PracticeNavigatorData"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("dojox.uuid.Uuid"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	dojo.declare("com.ibm.team.process.authoring.web.common.common.internal.PracticeNavigatorData", null, { //$NON-NLS-1$
		practiceJsonData: null,
		practiceETag: null,
		constructor: function(args) {
			this.practiceETag = args.ETag;
			this.init(args.practiceXMLData);
		},
		
		init: function(practiceXMLData) {
			var practiceResource = UIHelper.getPracticeUrl(practiceXMLData);
			var languages = UIHelper.getLanguages(practiceXMLData.documentElement);
			var languagesObject = {};
			var defaultLanguage;
			if (languages) {
				languages = languages.split(','); //$NON-NLS-1$
				if(languages.length > 0) {
					defaultLanguage = languages[0];
				}
				dojo.forEach(languages, function(language) {
					if (language) {
						languagesObject[language] = {};
						languagesObject[language].persisted = true;
					}
				});
			}
			
			var ETagGroup = UIHelper.extractETag(this.practiceETag);
			var currentLanguage = UIHelper.getLanguage(practiceXMLData.documentElement);
			var id = UIHelper.getResourceId(practiceResource);
			
			var attachmentsUrl = UIHelper.getAttachmentsUrl(practiceXMLData.documentElement);
			
			languagesObject[currentLanguage].ETag = ETagGroup.textContentETag;
			languagesObject[currentLanguage].textContent = UIHelper.getPracticeTextContent(practiceXMLData);
			languagesObject[currentLanguage].basicInformation = {};
			languagesObject[currentLanguage].basicInformation.label = UIHelper.getPracticeTitle(practiceXMLData);
			languagesObject[currentLanguage].basicInformation.description = UIHelper.getPracticeDescription(practiceXMLData);
			var model = {};
			model.ETag = ETagGroup.modelETag;
			model.structure = UIHelper.getPresentation(practiceXMLData);;
			
			this.practiceJsonData = {
				identifier: 'id', //$NON-NLS-1$
				label: 'label', //$NON-NLS-1$
				resource: practiceResource,
				id: id,
				title: languagesObject[currentLanguage].basicInformation.label,
				practiceId: id,
				model: model,
				languages: languagesObject,
				attachmentsUrl: attachmentsUrl,
				defaultLanguage: defaultLanguage,
				items: []
			};
			var presentation = UIHelper.getPracticePresentation(practiceXMLData);
			
			if (presentation) {
				dojo.forEach(presentation.childNodes, function(node) {
					if (UIHelper.isPracticeElement(node)) {
						this.practiceJsonData.items.push(this.getPracticeElement(node));
					} else if (UIHelper.isCategoryNode(node)) {
						this.practiceJsonData.items.push(this.getCategoryNode(node));
					}
				}, this);
			}
			
		},
		
		getPracticeElement: function(practiceElement) {
			var practiceResource = UIHelper.getNodeResource(practiceElement);
			var jsonElement = {
				type: ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT,
				id: UIHelper.getResourceId(practiceResource),
				label: UIHelper.getNodeLabel(practiceElement),
				icon: UIHelper.getNodeIcon(practiceElement),
				resource: practiceResource
			};
			return jsonElement;
		},
		
		getCategoryNode: function(categoryNode) {
			var jsonNode = {
				type: ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY,
				id: UIHelper.getNodeId(categoryNode),
				label: UIHelper.getNodeLabel(categoryNode),
				children: []
			};
			dojo.forEach(categoryNode.childNodes, function(node) {
				if (UIHelper.isPracticeElement(node)) {
					jsonNode.children.push(this.getPracticeElement(node));
				} else if (UIHelper.isCategoryNode(node)) {
					jsonNode.children.push(this.getCategoryNode(node));
				}
			}, this);
			
			return jsonNode;
		},
		
		getJsonData: function() {
			return this.practiceJsonData;
		}
		
	});
	com.ibm.team.process.authoring.web.common.common.internal.PracticeNavigatorData.getEmptyPracticeJsonData = function(practiceTitle, practiceDescription, processDescriptionId, currentLanguage) {
		var uuid = dojox.uuid.generateTimeBasedUuid();
		var languages = {};
		languages[currentLanguage] = {};
		languages[currentLanguage].textContent = {};
		languages[currentLanguage].basicInformation = {};
		languages[currentLanguage].basicInformation.label = practiceTitle;
		languages[currentLanguage].basicInformation.description = practiceDescription;
		var model = {};
		model.structure = UIHelper.getEmptyPresentation();
		var result = {
			identifier: 'id', //$NON-NLS-1$
			label: 'label', //$NON-NLS-1$
			resource: uuid,
			practiceId: uuid,
			id: uuid,
			processDescriptionId: processDescriptionId,
			title: practiceTitle,
			languages: languages, //$NON-NLS-1$
			model: model,
			items: [],
			isNew: true
		};
		return result;
	};
})();
