/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.common.internal.WorkingCopy"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.PracticeHelper");//$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient");//$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var PracticeHelper = com.ibm.team.process.authoring.web.common.common.internal.PracticeHelper;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.common.internal.WorkingCopy", null, { //$NON-NLS-1$
		practiceList: null,
		practiceIndexMap: null,
		processDescription: null,
		currentPractice: null,
		results: null,
		currentLanguage: null,
		currentItem: null,
		lastModified: null,
		useLocalServices: false,
		sharedSupport: false,
		
		constructor: function(args) {
			this.practiceList = [];
			this.results = [];
			if(args && args.useLocalServices) {
				this.useLocalServices = useLocalServices;
			}
		},
		
		addCategory: function(practiceResource, parentId, id, label) {
			var parentNode = null;
			var nodePair = this.getNodeById(parentId);
			var ownedPractice = nodePair.practice;
			parentNode = nodePair.node;
			var isRoot = false;
			if (!parentNode) {
				parentNode = ownedPractice;
				isRoot = true;
			}
			
			var newCategory = {
				type: ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY,
				id: id,
				label: label,
				languages: {},
				children: []
			};
			newCategory.languages[this.currentLanguage] = {};
			newCategory.languages[this.currentLanguage].basicInformation = {
				label: label
			};
			if (parentNode.children) {
				parentNode.children.push(newCategory);
			} else {
				parentNode.items.push(newCategory);
			}
			this.safeUpdatePracticeLanguage(ownedPractice);
			UIHelper.addCategory(ownedPractice, newCategory, this.currentLanguage, isRoot ? null : parentNode.id);
			this.updateRelatedPracticeStatus(ownedPractice);
		},
		
		updateCategory: function(id, label) {
			var nodePair = this.getNodeById(id);
			var ownedPractice = nodePair.practice;
			this.safeUpdatePracticeLanguage(ownedPractice);
			UIHelper.updateCategory(ownedPractice, this.currentLanguage, id, label);
			this.updateRelatedPracticeStatus(ownedPractice);
		},
		
		deleteCategory: function(id) {
			var nodePair = this.getNodeById(id);
			var ownedPractice = nodePair.practice;
			var categoryNode = nodePair.node;
			this.safeUpdatePracticeLanguage(ownedPractice);
			UIHelper.deleteCategory(ownedPractice, id);
			categoryNode.status = this.calculateStatus(categoryNode.status, ProcessAuthoringConstants.STATUS_DELETE);
			this.updateRelatedPracticeStatus(ownedPractice);
		},
		
		createPractice: function(practice) {
			practice.status = ProcessAuthoringConstants.STATUS_NEW;
			practice.languages[this.currentLanguage].status = ProcessAuthoringConstants.STATUS_UPDATE;
			practice.languages[this.currentLanguage].descriptionUpdate = true;
			this.addPractice(practice);
			if (this.processDescription) {
				this.addPracticeToPracticeList({
					url: practice.resource,
					name: practice.title,
					shared: "false", //$NON-NLS-1$
					unsaved: true
				}, true);
			}
		},
		
		updateRelatedPracticeStatus: function(practice) {
			var language = this.currentLanguage;
			
			practice.status = this.calculateStatus(practice.status, ProcessAuthoringConstants.STATUS_UPDATE);
			practice.languages[language].status = ProcessAuthoringConstants.STATUS_UPDATE;
			practice.languages[language].structureUpdate = true;
		},
		
		/**
		* Update practice with new data. 
		* 
		* @param {String} practiceResource the practice url. 
		* E.g. https://127.0.0.1:9444/jazz/process-authoring/practices/_wBvGPwBoEeGi5ZyM9Oq7iQ
		* @param {String} data the new practice data in String.
		* 
		*/
		updatePractice: function(practiceResource, data) {
			var practiceNode = this.getPractice(practiceResource);
			
			if (practiceNode) {
				var doc = UIHelper.parseXmlTextToDOMDocument(data);
				var presentation = UIHelper.getPracticeElementPresentation(doc);
				var title = UIHelper.getPracticeElementTitle(doc.documentElement);
				var description = UIHelper.getPracticeElementDescription(doc.documentElement);
				var language = UIHelper.getLanguage(doc.documentElement);
				practiceNode.model.data = presentation;
				// update text content, we should not simply replace text content, 
				// because that will lose the category changes in working copy, see bug 202692.
				var updatedTextContent = UIHelper.getPracticeElementTextContent(doc);
				if (!practiceNode.languages[language].textContent) {
					practiceNode.languages[language].textContent = {};
				}
				var textContent = practiceNode.languages[language].textContent;
				for (var key in updatedTextContent) {
					textContent[key] = updatedTextContent[key];
				}
				practiceNode.languages[language].basicInformation.label = title;
				practiceNode.languages[language].basicInformation.description = description;
				practiceNode.languages[language].status = ProcessAuthoringConstants.STATUS_UPDATE;
				practiceNode.languages[language].descriptionUpdate = true;
				practiceNode.status = this.calculateStatus(practiceNode.status, ProcessAuthoringConstants.STATUS_UPDATE);
			}
		},
		
		updatePracticeSummary: function(title, description, practiceResource) {
			var practiceNode = this.getPractice(practiceResource);
			if (!practiceNode) {
				return;
			}
			
			practiceNode.status = this.calculateStatus(practiceNode.status, ProcessAuthoringConstants.STATUS_UPDATE);
			practiceNode.languages[this.currentLanguage].basicInformation.label = title;
			practiceNode.languages[this.currentLanguage].basicInformation.description = description;
			practiceNode.languages[this.currentLanguage].descriptionUpdate = true;
			practiceNode.languages[this.currentLanguage].status = ProcessAuthoringConstants.STATUS_UPDATE;
		},
		
		addPractice: function(practice) {
			this.practiceList.push(practice);
		},
		
		deletePractice: function(practiceResource) {
			var practiceNode = this.getPractice(practiceResource);
			practiceNode.status = this.calculateStatus(practiceNode.status, ProcessAuthoringConstants.STATUS_DELETE);
			if (this.processDescription) {
				this.removePracticeFromPracticeList(practiceResource);
			}
		},
		
		/**
		* Add practice element into practice. 
		* 
		* @param {String} practiceResource the practice url. 
		* E.g. https://127.0.0.1:9444/jazz/process-authoring/practices/_wBvGPwBoEeGi5ZyM9Oq7iQ
		* @param {String} parentId parent's Id of practice element.
		* @param {String} id the practice element Id.
		* @param {String} data the new practice element data in String.
		* 
		*/
		addPracticeElement: function(practiceResource, parentId, id, data) {
			var parentNode = null;
			var nodePair = this.getNodeById(parentId);
			var ownedPractice = nodePair.practice;
			parentNode = nodePair.node;
			var isRoot = false;
			if (!parentNode) {
				parentNode = ownedPractice;
				isRoot = true;
			}
			
			var doc = UIHelper.parseXmlTextToDOMDocument(data);
			var presentation = UIHelper.getPracticeElementPresentation(doc);
			var title = UIHelper.getPracticeElementTitle(doc.documentElement);
			var description = UIHelper.getPracticeElementDescription(doc.documentElement);
			
			var newPracticeElement = {
				type: ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT,
				id: id,
				label: title,
				languages: {},
				model: {},
				status: ProcessAuthoringConstants.STATUS_NEW
			};
			
			newPracticeElement.model.data = presentation;
			newPracticeElement.languages[this.currentLanguage] = {};
			newPracticeElement.languages[this.currentLanguage].status = ProcessAuthoringConstants.STATUS_UPDATE;
			newPracticeElement.languages[this.currentLanguage].textContent = {};
			newPracticeElement.languages[this.currentLanguage].basicInformation = {};
			newPracticeElement.languages[this.currentLanguage].basicInformation.label = title;
			newPracticeElement.languages[this.currentLanguage].basicInformation.description = description;
			
			if (parentNode.children) {
				parentNode.children.push(newPracticeElement);
			} else {
				parentNode.items.push(newPracticeElement);
			}
			this.safeUpdatePracticeLanguage(ownedPractice);
			UIHelper.addPracticeElement(ownedPractice, newPracticeElement, this.currentLanguage, isRoot ? null : parentNode.id);
			this.updateRelatedPracticeStatus(ownedPractice);
		},
		
		/**
		* Update practice element with new data. 
		* 
		* @param {String} id the practice element Id.
		* @param {String} data the new practice element data in String.
		* 
		*/
		updatePracticeElement: function(id, data) {
			var nodePair = this.getNodeById(id);
			var ownedPractice = nodePair.practice;
			var practiceElementNode = nodePair.node;
			if (practiceElementNode) {
				var doc = UIHelper.parseXmlTextToDOMDocument(data);
				var presentation = UIHelper.getPracticeElementPresentation(doc);
				var title = UIHelper.getPracticeElementTitle(doc.documentElement);
				var description = UIHelper.getPracticeElementDescription(doc.documentElement);
				var language = UIHelper.getLanguage(doc.documentElement);
				practiceElementNode.model.data = presentation;
				practiceElementNode.languages[language].textContent = UIHelper.getPracticeElementTextContent(doc);
				practiceElementNode.languages[language].basicInformation.label = title;
				practiceElementNode.languages[language].basicInformation.description = description;
				practiceElementNode.languages[language].status = ProcessAuthoringConstants.STATUS_UPDATE;
				practiceElementNode.status = this.calculateStatus(practiceElementNode.status, ProcessAuthoringConstants.STATUS_UPDATE);
				this.safeUpdatePracticeLanguage(ownedPractice);
				UIHelper.updatePracticeElement(ownedPractice, language, id, title);
				if (ownedPractice.languages[language].persisted !== true) {
					this.updateRelatedPracticeStatus(ownedPractice);
				}
			}
		},
		
		deletePracticeElement: function(id) {
			var nodePair = this.getNodeById(id);
			var ownedPractice = nodePair.practice;
			var practiceElementNode = nodePair.node;
			if (practiceElementNode.status !== ProcessAuthoringConstants.STATUS_NEW) {
				id = practiceElementNode.resource;
			}
			practiceElementNode.status = this.calculateStatus(practiceElementNode.status, ProcessAuthoringConstants.STATUS_DELETE);
			this.safeUpdatePracticeLanguage(ownedPractice);
			UIHelper.deletePracticeElement(ownedPractice, id);
			this.updateRelatedPracticeStatus(ownedPractice);
		},
		
		safeUpdatePracticeLanguage: function(practice) {
			this.addLanguage(practice, this.currentLanguage);
			if (!practice.languages[this.currentLanguage].textContent) {
				practice.languages[this.currentLanguage].textContent = {};
			}
			if (!practice.languages[this.currentLanguage].basicInformation.label) {
				practice.languages[this.currentLanguage].basicInformation.label = practice.title;
			}
		},
		
		updateProcessDescription: function(args) {
			//first visit, add it directly
			this.processDescription.languages[this.currentLanguage].basicInformation.label = args.title;
			this.processDescription.languages[this.currentLanguage].basicInformation.description = args.description;
			this.processDescription.model.owner = args.owner;
			this.processDescription.model.isPLMShared= args.isPLMShared;
			this.processDescription.languages[this.currentLanguage].status = ProcessAuthoringConstants.STATUS_UPDATE;
			this.processDescription.status = this.calculateStatus(this.processDescription.status, ProcessAuthoringConstants.STATUS_UPDATE);
		},

		/**
		 *  Move a node in a tree, from source node to target node.
		 *  
		 *  @param {Object} node  the node that is to be moved.
		 *  @param {Object} sourceParent  the parent from which a node will be removed.
		 *  @param {Object} targetParent  the parent to which a node will be added.
		 *  @param {int} position  the position of the moved node in its new parent.
		 * 
		 */		
		moveNode: function(node, sourceParent, targetParent, position) {
			var sourceChildren = sourceParent.type === ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY? sourceParent.children : sourceParent.items;
			var targetChildren = targetParent.type === ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY? targetParent.children : targetParent.items;
			
			for (var i = 0; i < sourceChildren.length; i++) {
				var child = sourceChildren[i];
				if (child === node) {
					sourceChildren.splice(i, 1);
					targetChildren.splice(position, 0, node);
					break;
				}
			}
			var nodePair = this.getNodeById(node.id);
			var ownedPractice = nodePair.practice;
			UIHelper.moveNode(ownedPractice.model.structure, node, targetParent, position);
			ownedPractice.status = this.calculateStatus(ownedPractice.status, ProcessAuthoringConstants.STATUS_UPDATE);
			// ownedPractice may not have the same language as current language.
			// e.g. this.currentLanguage = "en-us", ownedPractice only contains default language "en" .
			var ownedPracticeLanguage = ownedPractice.languages[this.currentLanguage] ? ownedPractice.languages[this.currentLanguage] : ownedPractice.languages[ownedPractice.defaultLanguage];
			ownedPracticeLanguage.status = ProcessAuthoringConstants.STATUS_UPDATE;
			ownedPracticeLanguage.structureUpdate = true;
		},
		
		updatePracticeOrderInPracticeList: function(practiceResource, position) {
			if (!this.sharedSupport) {
				var count = -1;
				for (var i = 0; i < this.processDescription.model.practiceList.length; i++) {
					if (this.processDescription.model.practiceList[i].shared !== "true") { //$NON-NLS-1$
						count++;
					}
					if (count == position) {
						position = i;
						break;
					}
				}
			}
			for (var i = 0; i < this.processDescription.model.practiceList.length; i++) {
				var practice = this.processDescription.model.practiceList[i];
				if (practice.url === practiceResource) {
					this.processDescription.model.practiceList.splice(i, 1);
					this.processDescription.model.practiceList.splice(position, 0, practice);
					this.processDescription.status = this.calculateStatus(this.processDescription.status, ProcessAuthoringConstants.STATUS_UPDATE);
					break;
				}
			}
		},
		
		getPracticeList: function() {
			if (this.processDescription) {
				return this.processDescription.model.practiceList;
			}
			return null;
		},
		
		getPrivatePracticeList: function() {
			if (this.processDescription) {
				return dojo.filter(this.processDescription.model.practiceList, function(practice){ return (practice.shared === "false" && !practice.unsaved) }); //$NON-NLS-1$
			}
			return null;
		},
		
		// Methods for PracticeList
		updatePracticeList: function(oldUrl, newUrl) {
			dojo.some(this.processDescription.model.practiceList, function(practice) {
				if (practice.url === oldUrl) {
					practice.url = newUrl;
					return true;
				}
				return false;
			}, this);
		},
		
		addPracticeToPracticeList: function(practice, isProcessDescritionUpdated) {
			if (!this.practiceIndexMap[practice.url]) {
				this.practiceIndexMap[practice.url] = true;
				this.processDescription.model.practiceList.push(practice);
				if (isProcessDescritionUpdated) {
					this.processDescription.status = this.calculateStatus(this.processDescription.status, ProcessAuthoringConstants.STATUS_UPDATE);
				}
			}
		},
		
		removePracticeFromPracticeList: function(practiceUrl) {
			var removedPractice = null;
			if (this.practiceIndexMap[practiceUrl]) {
				delete this.practiceIndexMap[practiceUrl];
				
				var practiceList = this.processDescription.model.practiceList;
				for (var i = 0; i < practiceList.length; i++) {
					if (practiceList[i].url === practiceUrl) {
						removedPractice = practiceList[i];
						practiceList.splice(i, 1);
						this.processDescription.status = this.calculateStatus(this.processDescription.status, ProcessAuthoringConstants.STATUS_UPDATE);
						break; 
					}
				}
			}
			return removedPractice;
		},
		
		getPracticeInPracticeList: function(practiceUrl) {
			var practice = null;
			if (this.practiceIndexMap[practiceUrl]) {
				var practiceList = this.processDescription.model.practiceList;
				for (var i = 0; i < practiceList.length; i++) {
					if (practiceList[i].url === practiceUrl) {
						practice = practiceList[i];
						break;
					}
				}
			}
			
			return practice;
		},
		
		addProcessDescription: function(args) {
			var id = UIHelper.getResourceId(args.resourceURL);
			this.processDescription = {
				resourceURL: args.resourceURL,
				id: id
			};
			
			var ETagGroup = UIHelper.extractETag(args.ETag);
			var model = {};
			model.ETag = ETagGroup.modelETag;
			model.practiceList = args.practiceList;
			this.practiceIndexMap = {};
			dojo.forEach(args.practiceList, function(practice) {
				this.practiceIndexMap[practice.url] = true;
			}, this);
			
			model.owner = args.owner;
			model.isPLMShared = args.isPLMShared;
			var languagesObject = {};
			if (args.languages) {
				var languages = args.languages.split(','); //$NON-NLS-1$
				dojo.forEach(languages, function(language) {
					if (language) {
						languagesObject[language] = {};
					}
				});
				if(languages.length > 0) {
					this.processDescription.defaultLanguage = languages[0];
				}
			}
			languagesObject[this.currentLanguage].ETag = ETagGroup.textContentETag;
			languagesObject[this.currentLanguage].basicInformation = {};
			languagesObject[this.currentLanguage].basicInformation.label = args.title;
			languagesObject[this.currentLanguage].basicInformation.description = args.description;
			this.processDescription.model = model;
			this.processDescription.languages = languagesObject;
		},
		
		getNodeInformationByLanguage: function(node, language) {
			if (!language) {
				for (var key in node.languages) {
					if (node.languages[key].ETag) {
						language = key;
						break;
					}
				}
			}
			if (language) {
				var ETag = node.model.ETag + ' ' + node.languages[language].ETag; //$NON-NLS-1$
				return {
					'language': language, //$NON-NLS-1$
					'ETag': ETag //$NON-NLS-1$
				};
			} else {
				return null;
			}
		},
		
		setNodeETag: function(node, language, ETag) {
			var ETagGroup = UIHelper.extractETag(ETag);
			node.model.ETag = ETagGroup.modelETag;
			if (!node.languages[language]) {
				node.languages[language] = {};
			}
			node.languages[language].ETag = ETagGroup.textContentETag;
		},
		
		getLanguageArray: function(node) {
			var languages = [];
			for (var language in node.languages) {
				if (language && node.languages[language].status === ProcessAuthoringConstants.STATUS_UPDATE) {
					languages.push({
						language: language,
						data: node.languages[language]
					});
				}
			}
			return languages;
		},
		
		getLanguageArrayForStructureUpdate: function(node) {
			var languages = [];
			for (var language in node.languages) {
				if (language && node.languages[language].status === ProcessAuthoringConstants.STATUS_UPDATE && node.languages[language].structureUpdate === true) {
					languages.push({
						language: language,
						data: node.languages[language]
					});
				}
			}
			return languages;
		},
		
		getLanguageArrayForDescriptionUpdate: function(node) {
			var languages = [];
			for (var language in node.languages) {
				if (language && node.languages[language].status === ProcessAuthoringConstants.STATUS_UPDATE && node.languages[language].descriptionUpdate === true) {
					languages.push({
						language: language,
						data: node.languages[language]
					});
				}
			}
			return languages;
		},
		
		updateCurrentItem: function(oldId, newId) {
			if (oldId === this.currentItem) {
				this.currentItem = newId;
			}
		},
		
		getNodeById: function(nodeId) {
			var node = null;
			var rootPractice = null;
			nodeId = nodeId + ""; //$NON-NLS-1$
			if (this.processDescription && this.processDescription.id == nodeId) {
				return {
					node: this.processDescription
				};
			}
			
			dojo.some(this.practiceList, function(practice) {
				if ((practice.resource == nodeId) || (practice.id == nodeId)) {
					rootPractice = practice;
					return true;
				} else {
					node = this.searchNode(practice.items, nodeId);
					if (node) {
						rootPractice = practice;
						return true;
					}
				}
				
				return false;
			}, this);
			
			return {
				practice: rootPractice,
				node: node
			};
		},
		
		isParent: function(practiceResource, parentId, childId) {
			if (!childId) {
				return false;
			}
			if (parentId == childId) {
				return true;
			}
			var parentNode = null;
			var subNodes = null;
			if (parentId) {
				parentNode = this.getNodeById(parentId).node;
			}
			if (!parentNode) {
				parentNode = this.getPractice(practiceResource);
				if (!parentNode) {
					return false;
				}
				subNodes = parentNode.items;
			} else {
				subNodes = parentNode.children;
			}
			
			return this.searchChildNode(subNodes, childId);
		},
		
		searchChildNode: function(subNodes, childId) {
			var found = false;
			dojo.some(subNodes, function(node) {
				if (node.id == childId) {
					found = true;
					return true;
				}
				if (node.children) {
					found = this.searchNode(node.children, childId);
					if (found) {
						return true;
					}
				}
				return false;
			}, this);
			return found;
		},
		
		searchNode: function(nodeList, nodeId) {
			var selNode = null;
			dojo.some(nodeList, function(node) {
				if (node.id == nodeId) {
					selNode = node;
					return true;
				}
				if (node.children) {
					selNode = this.searchNode(node.children, nodeId);
					if (selNode) {
						return true;
					}
				}
				return false;
			}, this);
			return selNode;
		},
		
		getPractice: function(practiceResource) {
			var practiceId = practiceResource.substring(practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
			var node = null;
			dojo.some(this.practiceList, function(practice) {
				if (practice.resource == practiceResource || practice.practiceId == practiceId) {
					node = practice;
					return true;
				}
				return false;
			}, this);
			
			return node;
		},
		
		getIndexofPractice: function(practiceResource) {
			var practiceId = practiceResource.substring(practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
			var index = -1;
			dojo.some(this.practiceList, function(practice, i) {
				if (practice.resource == practiceResource || practice.practiceId == practiceId) {
					index = i;
					return true;
				}
				return false;
			}, this);
			return index;
		},
		
		updatePracticeElementETag: function(id, ETag, language) {
			var practiceElementNode = this.getNodeById(id).node;
			if (practiceElementNode) {
				var ETags = ETag.split(' '); //$NON-NLS-1$
				practiceElementNode.modelETag = ETags[0];
				if (!practiceElementNode.languages) {
					practiceElementNode.languages = {};
				}
				if (!practiceElementNode.languages[language]) {
					practiceElementNode.languages[language] = {};
				}
				practiceElementNode.languages[language].contentETag = ETags[1];
			}
		},
		
		calculateStatus: function(oldStatus, newStatus) {
			if (newStatus === ProcessAuthoringConstants.STATUS_DELETE) {
				if (oldStatus) {
					if (oldStatus === ProcessAuthoringConstants.STATUS_NEW) {
						return ProcessAuthoringConstants.STATUS_IGNORE;
					} else if (oldStatus === ProcessAuthoringConstants.STATUS_UPDATE) {
						return ProcessAuthoringConstants.STATUS_DELETE;
					} else {
						return oldStatus;
					}
				} else {
					return ProcessAuthoringConstants.STATUS_DELETE;
				}
			} else if (newStatus === ProcessAuthoringConstants.STATUS_UPDATE) {
				if (oldStatus) {
					if (oldStatus === ProcessAuthoringConstants.STATUS_NEW) {
						return ProcessAuthoringConstants.STATUS_NEW;
					}
				} else {
					return ProcessAuthoringConstants.STATUS_UPDATE;
				}
			}
			return oldStatus;
		},
		
		handleResult: function(value) {
			this.results.push(value);
			
			if (value.lastModified) {
				if (this.lastModified) {
					var latestLastModified = new Date(this.lastModified);
					var currentLastModified = new Date(value.lastModified);
					if (currentLastModified > latestLastModified) {
						this.lastModified = value.lastModified;
					}
				} else {
					this.lastModified = value.lastModified;
				}
			}
			
			//stop when there's error(sometimes can't continue traverse because of the error)
			if (!value.success) {
				dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_UPDATEDONE, [this.results]);
				if (this.resultNoticeHandle) {
					dojo.unsubscribe(this.resultNoticeHandle);
					this.resultNoticeHandle = null;
				}
			}
		},
		
		resetPractices: function() {
			dojo.forEach(this.practiceList, dojo.hitch(this, function(practice) {
				delete practice.scanedChildNum;
				if (practice.items) {
					this.resetNodes(practice.items);
				}
			}));
		},
		
		resetNodes: function(nodes) {
			dojo.forEach(nodes, dojo.hitch(this, function(node) {
				delete node.scanedChildNum;
				if ((node.children)) {
					this.resetNodes(node.children);
				}
			}));
		},
		
		scanPractices: function() {
			var done = true;
			for (var i = this.practiceIdx; i < this.practiceList.length; i++) {
				var practice = this.practiceList[i];
				if (practice.status === ProcessAuthoringConstants.STATUS_IGNORE) {
					continue;
				} else //undefined type for practice.scanedChildNum means the first time to scan
				if ((practice.status === ProcessAuthoringConstants.STATUS_NEW || practice.status === ProcessAuthoringConstants.STATUS_UPDATE) && typeof(practice.scanedChildNum) === 'undefined') { //$NON-NLS-1$
					practice.scanedChildNum = 0;
					this.practiceIdx = i;
					this.handlePractice(practice);
					done = false;
					break;
				} else if (practice.status === ProcessAuthoringConstants.STATUS_DELETE) {
					this.practiceIdx = i + 1;
					this.handlePractice(practice);
					done = false;
					break;
				} else {
					if (typeof(practice.scanedChildNum) === 'undefined') { //$NON-NLS-1$
						practice.scanedChildNum = 0;
					}
					this.practiceIdx = i + 1;
					this.currentPractice = practice;
					if (!this.traverseNode(practice)) {
						done = false;
						break;
					}
				}
				
			}
			//it's the end of scan
			if (done) {
				this.handleProcessDescription(dojo.hitch(this, function() {
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_UPDATEDONE, [this.results]);
					if (this.resultNoticeHandle) {
						dojo.unsubscribe(this.resultNoticeHandle);
						this.resultNoticeHandle = null;
					}
				}));
			}
		},
		
		traverseNode: function(node) {
			var parent = node.parent;
			var children = node === this.currentPractice ? node.items : node.children;
			var tempNode = null;
			
			while (node === this.currentPractice || !node.status || node.status === ProcessAuthoringConstants.STATUS_IGNORE || node.type !== ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT || typeof(node.scanedChildNum) !== 'undefined') { //$NON-NLS-1$
				if (typeof(node.scanedChildNum) === 'undefined') { //$NON-NLS-1$
					node.scanedChildNum = 0;
				}
				//check children
				if (node.status === ProcessAuthoringConstants.STATUS_DELETE && node.type === ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
					// if current category is deleted, skip scanning its children 
					node = parent;
					parent = node.parent;
					children = node === this.currentPractice ? node.items : node.children;
				} else if (node.status !== ProcessAuthoringConstants.STATUS_IGNORE && children && children.length > node.scanedChildNum) {
					tempNode = children[node.scanedChildNum++];
					tempNode.parent = node;
					parent = node;
					node = tempNode;
					children = node.children;
				} else if (parent) {
					node = parent;
					parent = node.parent;
					children = node === this.currentPractice ? node.items : node.children;
				} else {
					node = null;
					break;
				}
			}
			
			//there's node to be updated
			if (node) {
				if (typeof(node.scanedChildNum) === 'undefined') { //$NON-NLS-1$
					node.scanedChildNum = 0;
				}
				if (node.type === ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
					this.handlePracticeElement(node);
				}
				
				return false;
			} else {
				return this.checkPracticeStructureUpdate(this.currentPractice);
			}
		},
		
		checkPracticeStructureUpdate: function (practice) {
			var languageArray = this.getLanguageArrayForStructureUpdate(practice);
			if (languageArray.length > 0) {
				this.handlePracticeStructureUpdate(practice, languageArray);
				return false;
			}
			
			return true;
		},
		
		getNodePath: function(nodeId) {
			var path = [];
			dojo.some(this.practiceList, function(practice) {
				path.push(practice.resource + ''); //$NON-NLS-1$
				if (practice.resource == nodeId) {
					return true;
				} else {
					var found = this.getSubNodePath(practice.items, nodeId, path);
					if (found) {
						return true;
					}
				}
				path.pop();
				return false;
			}, this);
			
			return path;
		},
		
		getSubNodePath: function(nodeList, nodeId, path) {
			var found = false;
			dojo.some(nodeList, function(node) {
				path.push(node.id + ''); //$NON-NLS-1$
				if (node.id == nodeId) {
					found = true;
					return true;
				}
				if (node.children) {
					found = this.getSubNodePath(node.children, nodeId, path);
					if (found) {
						return true;
					}
				}
				path.pop();
				return false;
			}, this);
			return found;
		},
		
		handleProcessDescription: function(callback) {
			if (this.processDescription && this.processDescription.status === ProcessAuthoringConstants.STATUS_UPDATE) {
				this.processDescription.languages[this.currentLanguage].status = ProcessAuthoringConstants.STATUS_UPDATE;
				
				var processDescriptionInformation;
				var languageArray;
				var currentLanguagePosition;
				var title, description, resource, owner, practiceList, reload;
				
				languageArray = this.getLanguageArray(this.processDescription);
				currentLanguagePosition = 0;
				processDescriptionInformation = this.getNodeInformationByLanguage(this.processDescription, languageArray[currentLanguagePosition].language);
				reload = (!languageArray[currentLanguagePosition].data.ETag) ? true : false;
				
				owner = this.processDescription.model.owner;
				resource = this.processDescription.resourceURL;
				practiceList = this.processDescription.model.practiceList;
				
				title = languageArray[currentLanguagePosition].data.basicInformation ? languageArray[currentLanguagePosition].data.basicInformation.label : null;
				description = languageArray[currentLanguagePosition].data.basicInformation ? languageArray[currentLanguagePosition].data.basicInformation.description : null;
				
				var languageUpdateCallback = dojo.hitch(this, function(ETag) {
					this.setNodeETag(this.processDescription, languageArray[currentLanguagePosition++].language, ETag);
					if (languageArray.length === currentLanguagePosition) {
						callback();
					} else {
						reload = (!languageArray[currentLanguagePosition].data.ETag) ? true : false;
						title = languageArray[currentLanguagePosition].data.basicInformation ? languageArray[currentLanguagePosition].data.basicInformation.label : null;
						description = languageArray[currentLanguagePosition].data.basicInformation ? languageArray[currentLanguagePosition].data.basicInformation.description : null;
						processDescriptionInformation = this.getNodeInformationByLanguage(this.processDescription, languageArray[currentLanguagePosition].language);
						var isDefault = this.processDescription.defaultLanguage == processDescriptionInformation.language;
						PracticeHelper.updateProcessDescription(resource, processDescriptionInformation.language, isDefault, processDescriptionInformation.ETag, title, description, 
							owner, practiceList, this.processDescription.model.isPLMShared, languageUpdateCallback, this.useLocalServices, reload);
					}
				});
				
				var isDefault = this.processDescription.defaultLanguage == processDescriptionInformation.language;
				PracticeHelper.updateProcessDescription(resource, processDescriptionInformation.language, isDefault, processDescriptionInformation.ETag, title, description, owner, practiceList, this.processDescription.model.isPLMShared, languageUpdateCallback, this.useLocalServices, reload);
			} else {
				callback();
			}
		},
		
		/**
		 * Saves the practice content for a given language.
		 * 
		 * @param {Object} practice
		 * @param {Object} languageObject
		 * @param {Array} newSections array of new sections
		 * @param {Array} newFields array of new fields
		 * @param {Function} languageUpdateCallback
		 */
		saveLanguage: function(practice, languageObject, newSections, newFields, languageUpdateCallback) {
			var practiceData, label, description, reload;
			var practiceInformation = this.getNodeInformationByLanguage(practice, languageObject.language);
			var isDefault = practice.defaultLanguage == practiceInformation.language;
			label = languageObject.data.basicInformation ? languageObject.data.basicInformation.label : null;
			description = languageObject.data.basicInformation ? languageObject.data.basicInformation.description : null;
			reload = !languageObject.data.basicInformation ? true : false;
			if (languageObject.data.isDescriptionLoaded || practice.status === ProcessAuthoringConstants.STATUS_NEW) {
				UIHelper.updatePracticeElementPresentation(practice.model, languageObject.data.textContent);
				UIHelper.updatePracticeElementModel(practice, newSections, newFields);
				practiceData = UIHelper.buildPracticeXML(label, description, practice.model.data, practiceInformation.language, "description", isDefault); //$NON-NLS-1$
			} else {
				// save only the basic information (title, description)
				practiceData = UIHelper.buildBasicPracticeXML(label, description, languageObject.language, isDefault);
			}
			PracticeHelper.updatePractice(practice, practiceData, practiceInformation.language, practiceInformation.ETag, languageUpdateCallback, this.useLocalServices, reload);
		},
		
		handlePractice: function(practice) {			
			var practiceInformation;
			var languageArray;
			var currentLanguagePosition;
			var newSections = [];
			var newFields = [];
			
			var languageUpdateCallback = dojo.hitch(this, function(newElements, ETag) {
				this.setNodeETag(practice, languageArray[currentLanguagePosition++].language, ETag);
				if (newElements.newSections || newElements.newFields) {
					UIHelper.updateNewElements(practice, newElements, newSections, newFields);
				}
				newSections = [];
				newFields = [];
				if (languageArray.length === currentLanguagePosition) {
					this.scanPractices();
				} else {
					if (languageArray[currentLanguagePosition].data.ETag) {
						this.saveLanguage(practice, languageArray[currentLanguagePosition], newSections, newFields, languageUpdateCallback);
					} else {
						ProcessAuthoringClient.getPractice({
							'lastModified': this.lastModified, //$NON-NLS-1$
							'url': practice.resource + "?lang=" + languageArray[currentLanguagePosition].language, //$NON-NLS-1$ //$NON-NLS-2$
							'load': dojo.hitch(this, function(data, ioArgs) { //$NON-NLS-1$
								var ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
								languageArray[currentLanguagePosition].data.ETag = ETag.split(" ")[1]; //$NON-NLS-1$
								this.saveLanguage(practice, languageArray[currentLanguagePosition], newSections, newFields, languageUpdateCallback);
							})
						});
					}
				}
			});
			
			if (practice.status === ProcessAuthoringConstants.STATUS_UPDATE) {
				languageArray = this.getLanguageArrayForDescriptionUpdate(practice);
				currentLanguagePosition = 0;
				if (languageArray.length === currentLanguagePosition) {
					this.scanPractices();
				} else {
					this.saveLanguage(practice, languageArray[currentLanguagePosition], newSections, newFields, languageUpdateCallback);
				}
			} else if (practice.status === ProcessAuthoringConstants.STATUS_DELETE) {
				var processDescriptionETag = null;
				if (this.processDescription) {
					processDescriptionETag = this.processDescription.model.ETag;
				}
				PracticeHelper.deletePractice(practice.practiceId, processDescriptionETag, dojo.hitch(this, function(ETag) {
					if (this.processDescription) {
						this.processDescription.model.ETag = ETag;
					}
					this.scanPractices();
				}));
			} else if (practice.status === ProcessAuthoringConstants.STATUS_NEW) {
				languageArray = this.getLanguageArrayForDescriptionUpdate(practice);
				currentLanguagePosition = 0;
				UIHelper.updatePracticeElementPresentation(practice.model, languageArray[currentLanguagePosition].data.textContent);
				UIHelper.updatePracticeElementModel(practice, newSections, newFields);
				var practiceData = UIHelper.buildPracticeXML(UIHelper.escapeXML(languageArray[currentLanguagePosition].data.basicInformation.label), UIHelper.escapeXML(languageArray[currentLanguagePosition].data.basicInformation.description), practice.model.data, languageArray[currentLanguagePosition].language);
				var processDescriptionETag = null;
				if (this.processDescription) {
					processDescriptionETag = this.processDescription.model.ETag;
				}
				PracticeHelper.createPractice(practice, processDescriptionETag, practiceData, dojo.hitch(this, function(newElements, resource, ETag) {
					practice.resource = resource;
					if (this.processDescription) {
						var ETags = ETag.split(" "); //$NON-NLS-1$
						ETag = ETags[0] + " " + ETags[1]; //$NON-NLS-1$
						this.processDescription.model.ETag = ETags[2];
					}
					var oldId = practice.id;
					var newId = resource;
					if (this.processDescription) {
						this.updatePracticeList(oldId, newId);
					}
					this.updateCurrentItem(oldId, newId);
					practice.practiceId = UIHelper.getResourceId(resource); //$NON-NLS-1$
					practice.Id = practice.practiceId; //$NON-NLS-1$
					this.currentPracticeResource = practice.resource;
					languageUpdateCallback(newElements, ETag);
				}));
			}
		},
		
		handlePracticeStructureUpdate: function(practice, languageArray) {
			var practiceInformation;
			var practiceData;
			var currentLanguagePosition = 0;
			var newGroups = [];
			
			var languageUpdateCallback = dojo.hitch(this, function(data, ETag) {
				this.setNodeETag(practice, languageArray[currentLanguagePosition++].language, ETag);
				
				if (languageArray.length === currentLanguagePosition) {
					this.scanPractices();
				} else {
					UIHelper.updatePracticeStructurePresentation(practice.model, languageArray[currentLanguagePosition].data.textContent);
					practiceInformation = this.getNodeInformationByLanguage(practice, languageArray[currentLanguagePosition].language);
					practiceData = UIHelper.buildPracticeXML(UIHelper.escapeXML(languageArray[currentLanguagePosition].data.basicInformation.label), UIHelper.escapeXML(languageArray[currentLanguagePosition].data.basicInformation.description), practice.model.structure, languageArray[currentLanguagePosition].language, "structure"); //$NON-NLS-1$
					PracticeHelper.updatePractice(practice, practiceData, practiceInformation.language, practiceInformation.ETag, languageUpdateCallback, this.useLocalServices);
				}
			});
			
			UIHelper.updatePracticeStructurePresentation(practice.model, languageArray[currentLanguagePosition].data.textContent);
			UIHelper.updatePracticeStructureModel(practice, newGroups);
			practiceData = UIHelper.buildPracticeXML(UIHelper.escapeXML(languageArray[currentLanguagePosition].data.basicInformation.label), UIHelper.escapeXML(languageArray[currentLanguagePosition].data.basicInformation.description), practice.model.structure, languageArray[currentLanguagePosition].language, "structure"); //$NON-NLS-1$
			practiceInformation = this.getNodeInformationByLanguage(practice, languageArray[currentLanguagePosition].language);
		
			PracticeHelper.updatePractice(practice, practiceData, practiceInformation.language, practiceInformation.ETag, dojo.hitch(this, function(newElements, ETag) {
				UIHelper.updateNewGroups(practice, newElements, newGroups, dojo.hitch(this, this.updateCurrentItem));
				languageUpdateCallback({}, ETag);
			}), this.useLocalServices);
		},
		
		handlePracticeElement: function(practiceElement) {
			var parent = practiceElement.parent;
			var practiceElementInformation;
			var languageArray;
			var currentLanguagePosition;
			var pracriceElementData;
			var newSections = [];
			var newFields = [];
			
			var languageUpdateCallback = dojo.hitch(this, function(newElements, ETag) {
				this.setNodeETag(practiceElement, languageArray[currentLanguagePosition++].language, ETag);
				if (newElements.newSections || newElements.newFields) {
					UIHelper.updateNewElements(practiceElement, newElements, newSections, newFields);
				}
				if (languageArray.length === currentLanguagePosition) {
					if (this.traverseNode(practiceElement)) {
						this.scanPractices();
					}
				} else {
					if (languageArray[currentLanguagePosition].data.ETag) {
						practiceElementInformation = this.getNodeInformationByLanguage(practiceElement, languageArray[currentLanguagePosition].language);
						UIHelper.updatePracticeElementPresentation(practiceElement.model, languageArray[currentLanguagePosition].data.textContent);
						pracriceElementData = UIHelper.buildPracticeElementXML(languageArray[currentLanguagePosition].data.basicInformation.label, languageArray[currentLanguagePosition].data.basicInformation.description, practiceElement.model.data, practiceElementInformation.language);
						PracticeHelper.updatePracticeElement(practiceElement.resource, pracriceElementData, practiceElementInformation.ETag, practiceElementInformation.language, languageUpdateCallback); //$NON-NLS-1$
					} else {
						ProcessAuthoringClient.getPracticeElement({
							'lastModified': this.lastModified, //$NON-NLS-1$
							'url': practiceElement.resource + "?lang=" + languageArray[currentLanguagePosition].language, //$NON-NLS-1$ //$NON-NLS-2$
							'load': dojo.hitch(this, function(data, ioArgs) { //$NON-NLS-1$
								var ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
								languageArray[currentLanguagePosition].data.ETag = ETag.split(" ")[1]; //$NON-NLS-1$
								practiceElementInformation = this.getNodeInformationByLanguage(practiceElement, languageArray[currentLanguagePosition].language);
								UIHelper.updatePracticeElementPresentation(practiceElement.model, languageArray[currentLanguagePosition].data.textContent);
								pracriceElementData = UIHelper.buildPracticeElementXML(languageArray[currentLanguagePosition].data.basicInformation.label, languageArray[currentLanguagePosition].data.basicInformation.description, practiceElement.model.data, practiceElementInformation.language);
								PracticeHelper.updatePracticeElement(practiceElement.resource, pracriceElementData, practiceElementInformation.ETag, practiceElementInformation.language, languageUpdateCallback); //$NON-NLS-1$
							})
						});
					}
				}
			});
			
			if (practiceElement.status === ProcessAuthoringConstants.STATUS_NEW) {
				languageArray = this.getLanguageArray(practiceElement);
				currentLanguagePosition = 0;
				
				UIHelper.updatePracticeElementPresentation(practiceElement.model, languageArray[currentLanguagePosition].data.textContent);
				UIHelper.updatePracticeElementModel(practiceElement, newSections, newFields);
				pracriceElementData = UIHelper.buildPracticeElementXML(languageArray[currentLanguagePosition].data.basicInformation.label, languageArray[currentLanguagePosition].data.basicInformation.description, practiceElement.model.data, languageArray[currentLanguagePosition].language);
				
				PracticeHelper.createPracticeElement(this.currentPractice.resource, this.currentPractice.model.ETag, null, pracriceElementData, dojo.hitch(this, function(newElements, resource, ETag) {
					practiceElement.resource = resource;
					var ETags = ETag.split(" "); //$NON-NLS-1$
					var practiceElementId = resource.substring(resource.lastIndexOf('/') + 1); //$NON-NLS-1$
					var practiceETag = ETags[0] + " " + ETags[1]; //$NON-NLS-1$
					this.currentPractice.model.ETag = ETags[2];
					var oldId = practiceElement.id;
					var newId = practiceElementId;
					this.updateCurrentItem(oldId, newId);
					UIHelper.updatePracticeElementResource(this.currentPractice, oldId, resource);
					languageUpdateCallback(newElements, practiceETag);
				}));
				
			} else if (practiceElement.status === ProcessAuthoringConstants.STATUS_DELETE) {
				if (this.traverseNode(practiceElement)) {
					this.scanPractices();
				}
			} else if (practiceElement.status === ProcessAuthoringConstants.STATUS_UPDATE) {
				languageArray = this.getLanguageArray(practiceElement);
				currentLanguagePosition = 0;
				practiceElementInformation = this.getNodeInformationByLanguage(practiceElement, languageArray[currentLanguagePosition].language);
				UIHelper.updatePracticeElementPresentation(practiceElement.model, languageArray[currentLanguagePosition].data.textContent);
				UIHelper.updatePracticeElementModel(practiceElement, newSections, newFields);
				pracriceElementData = UIHelper.buildPracticeElementXML(languageArray[currentLanguagePosition].data.basicInformation.label, languageArray[currentLanguagePosition].data.basicInformation.description, practiceElement.model.data, practiceElementInformation.language);
				PracticeHelper.updatePracticeElement(practiceElement.resource, pracriceElementData, practiceElementInformation.ETag, practiceElementInformation.language, languageUpdateCallback); //$NON-NLS-1$
			}
		},
		
		executeUpdate: function() {
			this.resultNoticeHandle = dojo.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, this, this.handleResult);
			this.results = [];
			this.resetPractices();
			this.practiceIdx = 0;
			this.scanPractices();
		},
		
		setPracticeElementDataById: function(id, data) {
			var node = this.getNodeById(id).node;
			if (node) {
				node.data = data;
			}
		},
		
		getPracticeElementDataById: function(id) {
			var node = this.getNodeById(id).node;
			if (node && node.languages && node.languages[node.currentLanguage]) {
				return node.languages[node.currentLanguage].data;
			}
			return null;
		},
		
		setCurrentLanguage: function(id, language) {
			var node = this.getNodeById(id).node;
			if (node) {
				node.currentLanguage = language;
			}
		},
		
		updateLanguagesById: function(id, languages) {
			var nodePair = this.getNodeById(id);
			var node = nodePair.node ? nodePair.node : nodePair.practice;
			if (node) {
				if (!node.languages) {
					node.languages = {};
				}
				dojo.forEach(languages, function(language) {
					if (!node.languages[language]) {
						node.languages[language] = {};
						node.languages[language].persisted = true;
					}
				});
			}
		},
		
		getLanguagesById: function(id) {
			var node = this.getNodeById(id);
			var element = null;
			var languages = [];
			
			if (node && node.node) {
				element = node.node;
			} else if (node && node.practice) {
				element = node.practice;
			}
			
			if (element && element.languages) {
				for (var i in element.languages) {
					languages.push(i);
				}
			}
			return languages;
		},
		
		getDefaultLanguageById: function(id) {
			var node = this.getNodeById(id);
			if (node) {
				if(node.practice) {
					return node.practice.defaultLanguage;
				} else if(node.node) {
					return node.node.defaultLanguage;
				}
			}
			return null;
		},
		
		getPracticeElementETagById: function(id) {
			var node = this.getNodeById(id).node;
			if (node && (node.status === ProcessAuthoringConstants.STATUS_UPDATE)) {
				return node.modelETag + " " + node.languages[node.currentLanguage].contentETag; //$NON-NLS-1$
			}
			return null;
		},
		
		getPracticeElementLanguageETagById: function(id, language) {
			var node = this.getNodeById(id).node;
			if (node && (node.status === ProcessAuthoringConstants.STATUS_UPDATE)) {
				return node.modelETag + " " + node.languages[language].contentETag; //$NON-NLS-1$
			}
			return null;
		},
		
		getPracticeElementLanguageDataById: function(id, language) {
			var node = this.getNodeById(id).node;
			if (node && node.languages && node.languages[language]) {
				return node.languages[language].data;
			}
			return null;
		},
		
		addLanguage: function(node, language) {
			if (!node.languages) {
				node.languages = {};
			}
			
			if (!node.languages[language]) {
				node.languages[language] = {};
			}
			
			if (!node.languages[language].basicInformation) {
				node.languages[language].basicInformation = {};
			}
		},
		
		getProcessDescriptionData: function(processDescription, callback) {
			this.addLanguage(processDescription, this.currentLanguage);
			
			ProcessAuthoringClient.getPractice({
				'lastModified': this.lastModified, //$NON-NLS-1$
				'url': processDescription.resourceURL + "?lang=" + this.currentLanguage, //$NON-NLS-1$ //$NON-NLS-2$
				'load': dojo.hitch(this, function(data, ioArgs) { //$NON-NLS-1$
					var ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
					processDescription.languages[this.currentLanguage].ETag = UIHelper.extractETag(ETag).textContentETag;
					if (typeof(processDescription.languages[this.currentLanguage].basicInformation.label) === 'undefined') { //$NON-NLS-1$
						processDescription.languages[this.currentLanguage].basicInformation.label = UIHelper.getProcessDescriptionTitle(data);
					}
					if (typeof(processDescription.languages[this.currentLanguage].basicInformation.description) === 'undefined') { //$NON-NLS-1$
						processDescription.languages[this.currentLanguage].basicInformation.description = UIHelper.getProcessDescriptionDescription(data);
					}
					
					callback();
				})
			});
		},
		
		/**
		 * Gets the JSON object for a practice.
		 * 
		 * @param {Object} practice the practice object that might or might not be loaded yet
		 * @param {Function} callback the callback function that will be passed in the practice JSON object that has the following structure:
		 *                   
         *					practiceJson = {
         *						identifier: the unique identifier
         *						label: //TODO why are both label and title needed?
         *						title: the practice title
         *						resource: the URL of the practice
         *						practiceId: the UUID of the practice
         *						id: //TODO the practice ID again, why is it still needed when there is practiceId already?
         *						items: [], the child items, see the structure of a child item below
         *						isNew: indicates whether this is a new practice that has not been saved yet
         *					};
         *
         *         			item = {
         *						type: the item type that must be one of the following constants: ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY, ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT 
         *						id: the item unique ID
         *						label: the item label which is the title of a category or practice element
         *						children: [] the child items
         *					};
         *
		 * @param {Object} isLanguageChanged
		 */
		getTreeJsonArray: function(practice, callback, isLanguageChanged) {
			var practiceJson = null;
			if (practice.status != ProcessAuthoringConstants.STATUS_DELETE) {
				this.addLanguage(practice, this.currentLanguage);
				var func = dojo.hitch(this, function(textContent) {
					if (typeof(practice.languages[this.currentLanguage].basicInformation.label) === 'undefined') { //$NON-NLS-1$
						practice.languages[this.currentLanguage].basicInformation.label = practice.title;
					}
					if (typeof(practice.languages[this.currentLanguage].basicInformation.description) === 'undefined') { //$NON-NLS-1$
						practice.languages[this.currentLanguage].basicInformation.description = ""; //$NON-NLS-1$
					}
					practiceJson = {
						identifier: 'id', //$NON-NLS-1$
						label: 'label', //$NON-NLS-1$
						title: practice.languages[this.currentLanguage].basicInformation.label,
						resource: practice.resource,
						practiceId: practice.id,
						id: practice.id,
						items: [],
						isNew: practice.isNew
					};
					
					this.buildNodes(practice.items, practiceJson.items, textContent);
					return practiceJson;
				});
				
				if (practice.status === ProcessAuthoringConstants.STATUS_NEW || !isLanguageChanged) {
					if (!practice.languages[this.currentLanguage].textContent) {
						practice.languages[this.currentLanguage].textContent = {};
					}
					callback(func(practice.languages[this.currentLanguage].textContent));
				} else {
					if (!practice.languages[this.currentLanguage]) {
						practice.languages[this.currentLanguage] = {};
					}
					
					if (practice.languages[this.currentLanguage].textContent) {
						callback(func(practice.languages[this.currentLanguage].textContent));
					} else {
						ProcessAuthoringClient.getPractice({
							'lastModified': this.lastModified, //$NON-NLS-1$
							'url': practice.resource + "?lang=" + this.currentLanguage, //$NON-NLS-1$ //$NON-NLS-2$
							'load': dojo.hitch(this, function(data, ioArgs) { //$NON-NLS-1$
								var ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
								practice.languages[this.currentLanguage].ETag = UIHelper.extractETag(ETag).textContentETag;
								if (typeof(practice.languages[this.currentLanguage].basicInformation.label) === 'undefined') { //$NON-NLS-1$
									practice.languages[this.currentLanguage].basicInformation.label = UIHelper.getPracticeTitle(data);
								}
								if (typeof(practice.languages[this.currentLanguage].basicInformation.description) === 'undefined') { //$NON-NLS-1$
									practice.languages[this.currentLanguage].basicInformation.description = UIHelper.getPracticeDescription(data);
								}
								var textContent = UIHelper.getPracticeTextContent(data);
								practice.languages[this.currentLanguage].textContent = textContent;
								callback(func(textContent));
							})
						});
					}
						
				}
			}
		},
		
		buildNodes: function(nodeList, copyList, practiceTextContent) {
			dojo.forEach(nodeList, function(node) {
				if (node.status != ProcessAuthoringConstants.STATUS_DELETE) {
					var jsonNode;
					var label = this.calculateLabel(node, practiceTextContent);
					if (node.type === ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
						jsonNode = {
							type: ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT,
							id: node.id,
							label: label,
							icon: node.icon,
							resource: node.resource
						};
					} else if (node.type === ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
						jsonNode = {
							type: ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY,
							id: node.id,
							label: label,
							children: []
						};
					}
					copyList.push(jsonNode);
					if (node.children) {
						this.buildNodes(node.children, jsonNode.children, practiceTextContent);
					}
				}
			}, this);
		},
		
		getNodeModel: function(node) {
			if (!node.model) {
				node.model = {};
			}
			
			if (typeof node.model.data === "undefined") { //$NON-NLS-1$
				return null;
			} else 
				return node.model.data;
		},
		
		getNodeTextContent: function(node, language) {
			if (!node.languages) {
				node.languages = {};
			}
			
			if (!node.languages[language]) {
				node.languages[language] = {};
			}
			
			return node.languages[language].textContent;
		},
		
		calculateLabel: function(node, textConetent) {
			if (textConetent[node.id]) {
				return textConetent[node.id].label;
			} else {
				return node.label;
			}
		},
		
		getPracticeBasicInformationForCurrentLangugage: function(practice) {
			return practice.languages[this.currentLanguage].basicInformation;
		},
		
		getProcessDescriptionBasicInformationForCurrentLangugage: function() {
			return this.processDescription.languages[this.currentLanguage].basicInformation;
		},
		
		setProcessDescriptionDefaultLanguage: function(newDefaultLanguage) {
			this._setDefaultLanguage(this.processDescription, newDefaultLanguage);
		},
		
		setPracticeDefaultLanguage: function(practice, newDefaultLanguage) {
			if(this._setDefaultLanguage(practice, newDefaultLanguage)) {
				practice.status = this.calculateStatus(practice.status, ProcessAuthoringConstants.STATUS_UPDATE);
			}
		},
			
		/**
		 * Sets default language for the given object.
		 * 
		 * @param {Object} obj
		 * @param {Object} newDefaultLanguage
		 * @return true if new default language has been set, false otherwise
		 */	
		_setDefaultLanguage: function(obj, newDefaultLanguage) {
			if(obj.defaultLanguage == newDefaultLanguage || (!obj.defaultLanguage && !newDefaultLanguage)) {
				return false;
			}
			if (newDefaultLanguage) {
				if(obj.languages[newDefaultLanguage]) {
					obj.languages[newDefaultLanguage].status = ProcessAuthoringConstants.STATUS_UPDATE;
					obj.languages[newDefaultLanguage].descriptionUpdate = true;
					if (obj.languages[obj.defaultLanguage]) {
						obj.languages[obj.defaultLanguage].status = ProcessAuthoringConstants.STATUS_UPDATE;
						obj.languages[obj.defaultLanguage].descriptionUpdate = true;
					}
					obj.defaultLanguage = newDefaultLanguage;
					return true;
				}
			} else {
				if (obj.languages[obj.defaultLanguage]) {
					obj.languages[obj.defaultLanguage].status = ProcessAuthoringConstants.STATUS_UPDATE;
					obj.languages[obj.defaultLanguage].descriptionUpdate = true;
				}
				obj.defaultLanguage = null;
				return true;
			}
			return false;
		},

		clean: function() {
			this.practiceList = [];
			this.sharedSupport = false;
			if (this.resultNoticeHandle) {
				dojo.unsubscribe(this.resultNoticeHandle);
				this.resultNoticeHandle = null;
			}
			
		}
	});
	
})();
