/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2005, 2012,
 * 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("jazz.ui.internal.Drawer");

dojo.require("jazz.ui.MegaMenu");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.MenuButton");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.internal.ScrollableView");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("jazz.ui.internal", "DrawerMessages");

(function(){

var COLLAPSED_WIDTH 	= 16,
	MIN_WIDTH			= 200,
	MAX_WIDTH 			= 600,
	ANIM_DURATION		= 300,
	DRAWER_VERT_PAD 	= 8,
	LOCAL_STORAGE_KEY	= "jazz-ui-internal-Drawer";

dojo.setObject("jazz.ui.internal.Drawer.state",{
	UNDOCKED  		: "undocked",
	CLOSED    		: "closed",
	OPEN    		: "open",
	PINNED    		: "pinned",
	SLIDING_OPEN	: "closed sliding",
	SLIDING_CLOSED	: "open sliding"
});

var state = jazz.ui.internal.Drawer.state;

var _messages;

dojo.declare("jazz.ui.internal.Drawer", [dijit._Widget, dijit._Container, dijit._Templated], {
	templatePath		: dojo.moduleUrl("jazz.ui", "internal/templates/Drawer.html"),
	headerTitle			: "",
	tooltip				: "",
	width				: 250,
	widgetsInTemplate	: true,
	leftAnchor 			: null,
	rightAnchor 		: null,
	currentAnchor       : null,
	buttons             : null,
	menuItems			: null, //array of objects in the form {label (string), iconClass (string), onClick (function), index (int, optional)}
	navbarNode			: null,
	_toolbar			: null,
	_isOpen				: false,
	_isPinned			: false,
	_slideCloseFn       : null,  // cache the hitch
	_slideOpenFn        : null,  // cache the hitch
	_openAnim           : null,
	_closeAnim          : null,
	_openMenuButton     : null,
	
	constructor: function() {
		if (!_messages) _messages = dojo.i18n.getLocalization("jazz.ui.internal", "DrawerMessages");
		this._messages = _messages;
	},
	
	postCreate: function(){
		this._scrollableView.set("overNode", this.domNode);
		try {
			// each context root (server) used to get a separate state, check for them and delete them
			var contextRoot = dojo.getObject("net.jazz.ajax.contextRoot")||"";
			if (contextRoot)
				contextRoot = contextRoot();
			var legacyLocalStorageKey = "jazz-ui-internal-Drawer" + (contextRoot).replace(/[^a-z0-9]/gi, "-");
			var legacyStateJson = localStorage[legacyLocalStorageKey];
			var anchorName = null;
			var stateJson = localStorage[LOCAL_STORAGE_KEY];
			if (legacyStateJson) {
				// use the legacy state if we have no proper state defined yet
				if (!stateJson)
					stateJson = localStorage[LOCAL_STORAGE_KEY] = legacyStateJson;
				delete localStorage[legacyLocalStorageKey];
			}
			// read pinned & anchor state from local storage
			if (stateJson) {
				var state = dojo.fromJson(stateJson); 
				this._isPinned = this._isOpen = state.isPinned;
				if (state.width) this.width = state.width;
				anchorName = state.anchor;
			}
		}
		catch (e) {
			// browser doesn't support localStorage; ignore
		}
		
		this._headerText.appendChild(document.createTextNode(this.headerTitle));
		
		this._menuButton.set("menuProvider", dojo.hitch(this, this._menuProvider));
		this._pinButton.set("pressed", this._isPinned);
		/*
		 * cache the slide close function so we don't create a hitch each time.
		 */
		this._slideCloseFn = dojo.hitch(this, "_slideClosed");
		this._slideOpenFn  = dojo.hitch(this, "_slideOpen");
		
		this._backgroundIframe = new dijit.BackgroundIframe(this.domNode);
		dijit.setWaiState(this.domNode, "expanded", this._isOpen);
		
		this._initAnchor(anchorName);
		if (dojo.isIE) this.connect(this.domNode, "onselectstart", this._onSelectStart);
	},
	_onSelectStart: function(e) {
		var ca = this.currentAnchor;
		if (ca) switch(ca.getState()) {
			case state.CLOSED:
			case state.SLIDING_OPEN:
				e.preventDefault();
		}
	},
	startup: function() {
		if (this._started) return;
		if (!this.currentAnchor && this._initAnchor() === false) {
			console.error("Error creating Drawer: no anchor provided.");
			return;
		}
		if (this._isPinned) {
			this.domNode.tabIndex="-1";
			this.currentAnchor.setState(state.PINNED);
			this.onOpen();
			this.onPinChange(this._isPinned);
			this.domNode.title = "";
		}
		this._updateWidth();
		this.checkHeight();
		this.inherited(arguments);
	},
	_initAnchor: function(/* string */ anchorName) {
		if (!(this.leftAnchor && this.rightAnchor)) {
			return false;
		}
		if (anchorName == "left" && this.leftAnchor) {
			var anchor = this.leftAnchor;
		} else if (anchorName == "right" && this.rightAnchor) {
			anchor = this.rightAnchor;
		}
		this._attachAnchor(anchor||this.leftAnchor||this.rightAnchor);
	},
	onClose: function() {
		// override to do work on close
	},
	onOpen: function() {
		// override to do work on open
	},
	onPinChange: function(pinned) {
		
	},
	swapTo: function (anchor) {
		this._noBlur = true;
		var state = this.currentAnchor ? this.currentAnchor.getState() : state.PINNED;
		this._attachAnchor(anchor, state);
		this.onPinChange(this._isPinned);
		this._saveState();
		var item = this._menuButton;
		if (item && item.domNode) item.focus();
		this._noBlur = false;
	},
	_attachAnchor: function (anchor, newState) {
		if (!anchor || anchor == this.currentAnchor) return;
		this._menu && this._menu.destroy();
		this._menu = null;
		this.currentAnchor && this.currentAnchor.setState(state.UNDOCKED);  //hide empty anchor if exists
		anchor.attachDrawer(this, newState);	//attach without removing to reduce reflows
		if (anchor.getState() == state.CLOSED) this.domNode.title = this.tooltip;
		this.currentAnchor = anchor;
		this._menuButton.align = (anchor == this.rightAnchor) ? "right" : "left";
		if (this._started) this.checkHeight();
	},
	onClick: function(e) {
		if (this._isPinned)	return;
		!this._isOpen && dojo.stopEvent(e);
		this._slideOpenFn();
	},
	_pin: function() {
		this._isPinned = !this._isPinned;
		
		this.onPinChange(this._isPinned);
		
		if (this._isPinned) {
			this.domNode.tabIndex="-1";
			this.currentAnchor.setState(state.PINNED);
		} else {
			this.domNode.tabIndex="0";
			this.currentAnchor.setState(state.OPEN);
			if (!this._bodyClick) this._bodyClick = this.connect(document.body, "onclick", this.onBlur);
		}
		this._saveState();
		this.checkHeight();
	},
	getWidth: function() {
		return this._isPinned ? this.width : COLLAPSED_WIDTH;
	},
	_fixed: false,
	_height: 0,
	checkHeight: function(force) {
		if (this._innerWrapper) { //if _innerWrapper is null, drawer has been destroyed.
			var viewport = dijit.getViewport(),
				scrollTop = document.documentElement.scrollTop || document.body.scrollTop,
				navPos = this.navbarNode ? dojo.position(this.navbarNode) : {y: 0, h: 0},
				domNode = this.domNode;
			if (navPos.y <= 0) {
				if (!this._fixed||force===true) {
					domNode.style.position = "fixed";
					domNode.style.top = navPos.h + 2 + "px";
					this._fixed = true;
				}
				var height = viewport.h - navPos.h - 8;
			} else {
				if (this._fixed||force===true) {
					domNode.style.position = "absolute";
					domNode.style.top = "";
					this._fixed = false;
				}
				height = viewport.h - (navPos.y + navPos.h) - 7;
			}
			if (height != this._height && height >= 0) {
				this._innerWrapper.style.height = height + "px";
				this._height = height;
			}
			//delay scrollbar computations until scrolling is done
			if (this._timeout) clearTimeout(this._timeout);
			this._timeout = setTimeout(dojo.hitch(this, function() {
				this._scrollableView.checkHeight();
				delete this._timeout;
			}),300);
		}
	},
	_slideOpen: function() {
		if (this._isOpen) return;
		this._isOpen = true;

		if (this._closeAnim) {
			this._closeAnim.stop();
			this._closeAnim = null;
		}
		
		this.currentAnchor.setState(state.SLIDING_OPEN);
		
		this.domNode.title = "";
		
		this._openAnim = dojo.animateProperty({
			node       : this.domNode,
			duration   : ANIM_DURATION,
			properties : { width: { start: COLLAPSED_WIDTH, end: this.width, unit: "px" }}
		});
		this.domNode.style.overflow = "hidden";
		
		this.connect(this._openAnim, "onEnd", function() {
			this.currentAnchor.setState(state.OPEN);
			this.domNode.style.overflow = "";
			this._openAnim = null;
			dijit.setWaiState(this.domNode, "expanded", true);
			setTimeout(dojo.hitch(this,function() {
				this.onOpen();
				this.checkHeight();
			}),0);
		});
		this._openAnim.play();
		if (!this._bodyClick) this._bodyClick = this.connect(document.body, "onclick", this.onBlur);
	},
	_onFocus: function(event) {
		this.currentAnchor.addHover();
	},
	_setNavbarNodeAttr: function(navbarNode) {
		this.navbarNode = navbarNode;
		if (this._started) this.checkHeight(true);
	},
	onBlur: function(event) {
		// if there is no event or the target is not the body, this is an onBlur - skip if closed
		if (this.currentAnchor.getState() !== state.CLOSED && (!event || event.currentTarget != document.body)) {
			var focus = dijit.getFocus();
			// if the focus node is the Drawer, and focus is not moving to another stack of widgets in the page
			// then the click is probably in an iframe or outside the viewport - don't close
			if (!focus.node || (focus.node == this.domNode && !dijit._activeStack.length)) {
				return; 
			}
			// FF gets confused when you tab to iframes for remote viewlets, wire up a fake event so we can check if it's a descendant
			if ((!dojo.isIE || dojo.isIE > 8) && focus.node.nodeName === "IFRAME") {
				event = {target: focus.node};
			}
		}
		
		// remove the hover class for all blurs
		this.currentAnchor.removeHover();
		//do not close if focus changed to a child node of the drawer, or if we're swapping sides
		if (this._noBlur || this._isPinned || (focus && focus.node && dojo.hasClass(focus.node, "dijitMenuItem")) || (event && event.target && (dojo.isDescendant(event.target, this.domNode) || !dojo.isDescendant(event.target, document.body)))) return;
		
		//do not close if a hover popup is open
		var mp = dojo.query("body > #jazz_ui_internal__MasterPopup_0");
		if (mp && mp[0] && dojo.style(mp[0],"display") != "none") return;
		
		//do not close if a dialog is open
		var dialog = dojo.query("body > .jazz-ui-Dialog-absolute");
		if (dialog.length > 0) return;
		
		this._slideClosed();
	},
	onKeyPress: function(e) {
		var k = dojo.keys;
		if (e.charCode == k.SPACE) e.charOrCode = k.SPACE;
		if (e.target == e.currentTarget && (e.charOrCode === k.ENTER || e.charOrCode === k.SPACE)) {
			this._isOpen ? this._slideClosed() : this._slideOpen();
			dojo.stopEvent(e);
		} else if (e.charOrCode == k.ESCAPE) {
			this.domNode.focus();
			this._slideClosed();
			this._onFocus();
		} else if (e.charOrCode == k.TAB && this.currentAnchor.getState() !== state.CLOSED) {
			//get tab order - wrap around tab focus within drawer
			var elems = dijit._getTabNavigable(this.domNode);
			var _lastFocusItem = elems.last || elems.highest || this._pinButton;
			// if we're focusing on the last or first items, wrap around.
			if (e.shiftKey && e.target == this.domNode && _lastFocusItem) {
				dijit.focus(_lastFocusItem);
			} else if (!e.shiftKey && e.target == _lastFocusItem) {
				dijit.focus(this.domNode);
			} else {
				return;
			}
			dojo.stopEvent(e);
		}
	},
	_slideClosed: function() { 
		if (!this._isOpen) return;
		this._isOpen = false;
		
		if (this._openAnim) {
			this._openAnim.stop();
			this._openAnim = null;
		} 
		
		this.currentAnchor.setState(state.SLIDING_CLOSED);
		
		this._closeAnim = dojo.animateProperty({
			node       : this.domNode,
			duration   : ANIM_DURATION,
			properties : { width: {	start: this.width, end: COLLAPSED_WIDTH, unit: "px" } }
		});
		this.onClose();
		this.domNode.style.overflow = "hidden";
		
		this.connect(this._closeAnim, "onEnd", function() {
			this.domNode.style.overflow = "";
			this.domNode.title = this.tooltip;
			this.currentAnchor.setState(state.CLOSED);
			this._closeAnim = null;
			dijit.setWaiState(this.domNode, "expanded", false);
		});
		
		this._closeAnim.play();
	},
	
	addMenuItem: function(args) {
		if (!this.menuItems) {
			this.menuItems = [];
		}
		if (typeof args.index == "number") {
			this.menuItems.splice(args.index,0,args);
		} else {
			this.menuItems.push(args);	
		}
		// if the menu has already been created, insert it
		if (this._menu) {
			if (typeof args.index == "number") {
				this._menu.addItemByArgs(args, args.index);	
			} else {
				this._menu.addItemByArgs(args);
			}
		}
	},
	_createMenu: function (){
		this._menu && this._menu.destroy();
		this._menu = new jazz.ui.MegaMenu({"class": "jazz-ui-Drawer-menu"});

		if (this.leftAnchor) {
			var args = {label:_messages.dockOnLeft, onClick: dojo.hitch(this, "swapTo", this.leftAnchor)};
			var item = this._menu.addItemByArgs(args);
			if (this.leftAnchor == this.currentAnchor) item.set("selected", true);
		}
		if (this.rightAnchor) {
			args = {label:_messages.dockOnRight, onClick: dojo.hitch(this, "swapTo", this.rightAnchor)};
			item = this._menu.addItemByArgs(args);
			if (this.rightAnchor == this.currentAnchor) item.set("selected", true);
		}

		// add known menu contributions
		if (this.menuItems) {
			this._menu.addSeparator();
			dojo.forEach(this.menuItems, function(item) {
				if (item.separator) {
					this._menu.addSeparator();
				} else if (!item.label) {
					console.error("Error creating menu:  Menu item at index "+i+" is missing a label.");
				} else {
					this._menu.addItemByArgs(item);
				}
			}, this);
		}
	},
	_menuProvider: function() {
		if (!this._menu) this._createMenu();
		return this._menu;
	},
	_saveState: function() {
		try {
			// save in local storage for now until we implement
			// 121162: Dashboard mini forgets about position
			localStorage[LOCAL_STORAGE_KEY] = dojo.toJson({
				isPinned: this._isPinned,
				anchor: this.currentAnchor == this.rightAnchor ? "right" : "left",
				width: this.width
			});
		} catch (e) {
			// browser doesn't support localStorage; ignore and don't save
		}
	},
	_drag: function(e) {
		if (!this._events) this._events = [];
		this._events.push(this.connect(document.body, "onmouseup", this._mouseUp));
		this._events.push(this.connect(document.body, "onmousemove", this._mouseMove));
		// Prevent text selection while dragging the scrollbar in IE9-.
		// For IE10, FF and WebKit, this is done in CSS with user-select rule.
		if (dojo.isIE < 10) {
			this._events.push(this.connect(document, "onselectstart", this._cancelSelect));
			this._events.push(this.connect(document, "ondragstart", this._cancelSelect));
		}
		this._initX = e.pageX;
		this._initWidth = this.width;
		jazz.ui.underlay.show({opacity: 1, color: "transparent", zIndex: 10000});
		dojo.addClass(document.body, "jazz-ui-Drawer-resizing");
	},
	_cancelSelect: function(e) {
		dojo.stopEvent(e);
		return false;
	},

	_mouseMove: function(e) {
		if (this.currentAnchor == this.rightAnchor) {
			var newWidth =  this._initX - e.pageX + this._initWidth;			
		} else {
			newWidth = e.pageX - this._initX + this._initWidth;
		}
		// defer updating the DOM to prevent reflows from stacking up if the mouse is moving fast. 
		clearTimeout(this._resizeTimeout);
		this._resizeTimeout = setTimeout(dojo.hitch(this, this._onMouseMoveDelayed, newWidth),0);
	},
	_onMouseMoveDelayed: function(width) {
		if (this._resizeTimeout) {
			clearTimeout(this._resizeTimeout);
			delete this._resizeTimeout;
			width = Math.max(MIN_WIDTH, Math.min(width, MAX_WIDTH));
			this.width = width;
			this._updateWidth();
			this._scrollableView.checkHeight();		
		}
	},
	_mouseUp: function(e) {
		if (this._events) {
			while(this._events.length){
				this.disconnect(this._events.pop());
			}
			delete this._events;
		}
		delete this._initX;
		delete this._initWidth;
		jazz.ui.underlay.hide();
		dojo.removeClass(document.body, "jazz-ui-Drawer-resizing");
		this._saveState();
		
		//notify things that care about resizing to check their width (eg. dashboard viewlet truncated titles)
		jazz.util.ResizeNotifier.poke();
		
		//defer the call to checkHeight to wait for ResizeNotifier async events to fire
		setTimeout(dojo.hitch(this,this.checkHeight),0);
	},
	_updateWidth: function() {
		if (this.currentAnchor.getState() != state.CLOSED) {
			this.domNode.style.width = this.width + "px";
		}
		if (this._isPinned) {
			this.currentAnchor.domNode.style.width = this.getWidth() + "px";
		} else {
			this.currentAnchor.domNode.style.width = "";
		}
	},
	destroy: function() {
		this._menu && this._menu.destroy();
		this._backgroundIframe.destroy();
		this.inherited(arguments);
	}
});

dojo.declare("jazz.ui.internal.DrawerAnchor", [dijit._Widget, dijit._Templated], {
	templateString: "<div class='jazz-ui-DrawerAnchor undocked' dojoattachevent='onmouseover: addHover, onmouseout: removeHover'></div>",
	_state: state.UNDOCKED,
	_drawer: null,
	
	postCreate: function (){
		this.setState(state.CLOSED);
	},
	attachDrawer: function (drawer, attachState){
		this._drawer = drawer;
		attachState = attachState || state.CLOSED;
		this.setState(attachState);
		this.domNode.appendChild(drawer.domNode);
	},
	removeDrawer: function (drawer){
		this._drawer = null;
		this.domNode.removeChild(drawer.domNode);
		this.setState(state.UNDOCKED);
	},
	addHover: function(event) {
		if (this._state == state.CLOSED || this._state == state.SLIDING_CLOSED) dojo.addClass(this.domNode, "hover");
	},
	removeHover: function(event) {
		if (this._state == state.CLOSED) dojo.removeClass(this.domNode, "hover");
	},
	setState: function (newState){
		var classes = this.domNode.className.split(" ");
		var curStates = this._state.split(" ");
		if (this._state == state.CLOSED) curStates.push("hover");  //add "hover" to list of classes to be removed
		for (var i = 0; i < curStates.length; i++) {
			var curStateIndex = dojo.indexOf(classes,curStates[i]);
			if (curStateIndex >= 0) {
				classes.splice(curStateIndex,1);
			}			
		}
		classes.push(newState);
		this.domNode.className = classes.join(" "); 
		this.domNode.style.width = (newState == state.PINNED && this._drawer) ? this._drawer.getWidth() + "px" : "";
		this._state = newState;
	},
	getState: function (){
		return this._state;
	}
});

dojo.declare("jazz.ui.internal.DrawerAnchorLeft", [jazz.ui.internal.DrawerAnchor], {
	postCreate: function(){dojo.addClass(this.domNode,"left");}
});

dojo.declare("jazz.ui.internal.DrawerAnchorRight", [jazz.ui.internal.DrawerAnchor], {
	postCreate: function(){dojo.addClass(this.domNode,"right");}
});

})();