/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.LogoutArea");

dojo.require("net.jazz.ajax.viewletServiceRoot");
dojo.require("dojo.cookie");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
dojo.require("jazz.util.html");
dojo.require("jazz.app.internal.UserMenu");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.MenuPopup");
dojo.requireLocalization("jazz.app.internal", "LoginLogoutMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

dojo.declare("jazz.app.internal.LogoutArea", [dijit._Widget, dijit._Templated], {
	allProfilesUrl: "",
	userProfileUrl: "",
	smallPreviewUrl: "",
	personalDashboardUrl: "",
	logoutUrl: "",
	isAdmin: false,
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("jazz.app.internal", "LoginLogoutMessages");
		this.allProfilesUrl = getJtsRoot() + "/admin#action=jazz.viewPage&id=com.ibm.team.repository.users";
	},
	
	postCreate: function() {
		new jazz.ui.MenuPopup({
			align: "right",
			menuProvider: dojo.hitch(this, this._menuProvider),
			title: this.messages.userProfile,
			destroyOnClose: true,
			group: "jazz-app-BannerButtons"
		}, this.anchor);
	},
	
	uninitialize: function() {
		this._resetState();
	},
	
	_showLoggedInUserAndLogoutControl: function() {
		new jazz.ui.MenuPopup({
			align: "right",
			menuProvider: dojo.hitch(this, this._menuProvider),
			title: this.messages.userProfile,
			destroyOnClose: true,
			group: "jazz-app-BannerButtons"
		},node).placeAt(this.domNode);
	},
	
	_menuProvider: function() {
		var items = this._getDefaultMenuItems();	
		var args = {
			items: items,
			config: {logout: this.logoutUrl},
			userSmallPreviewUrl: this.smallPreviewUrl
		};
		return new jazz.app.internal.UserMenu(args);
	},
	_getDefaultMenuItems: function() {
		var items = [ {
			label : this.messages.viewMyProfile,
			href : this.userProfileUrl
		} ];
		
		if (this.isAdmin)
			items.push({
				label: this.messages.viewOtherProfiles,
				href: this.allProfilesUrl
			});
		items.push({
			label: this.messages.openPersonalDash,
			href: this.personalDashboardUrl
		});
		return items;
	}
});

function getJtsRoot() {
	try {
		var session = dojo.getObject("com.ibm.team.repository.web.client.session");
		if (session) {
			var userUri = session.getAuthenticatedContributorUri();
		}
		if (userUri) {
			var jts = userUri.split("/users/");
			if (jts.length) {
				return jts[0];
			}
		}
	} catch(e) {
		// eat the error
	}
	return net.jazz.ajax.viewletServiceRoot();
}

})();