/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.ProjectsUtil");

dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("jazz.app.internal.util.XhrUtil");

var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var XhrUtil = jazz.app.internal.util.XhrUtil;

var NAMESPACES = {
	dc: "http://purl.org/dc/terms/",
	jd: "http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
	ju: "http://jazz.net/ns/ui#",
	oslc: "http://open-services.net/ns/core#",
	rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
};

var HOME_MENU_BINDING = {
	namespaces: NAMESPACES,
	applicationAbout: ["string", "oslc:publisher/@rdf:resource", "1"],
	contextRoot: ["string", "jd:contextRoot", "1"],
	defaultIcon: ["string", "ju:defaultIcon/@rdf:resource", "1"],
	menuSection: ["string", "ju:menuSection/@rdf:resource", "1"],
	publisher: ["string", "dc:publisher", "1"],
	service: ["string", "jd:service/@rdf:resource", "1"]
};
var HOME_MENU_ITEM_BINDING = {
	namespaces: NAMESPACES,
	icon: ["string", "ju:icon/@rdf:resource", "1"],
	link: ["string", "ju:link/@rdf:resource", "1"],
	resource: ["string", "ju:resource/@rdf:resource", "1"],
	title: ["string", "dc:title", "1"],
	usage: ["string", "ju:usage/@rdf:resource", "1"]
};
var HOME_MENU_DISCOVERY_BINDING = {
	namespaces: NAMESPACES,
	HomeMenu: [HOME_MENU_BINDING, "ju:HomeMenu", "*"]
};

var HOME_MENU_SERVICE_BINDING = {
	namespaces: NAMESPACES,
	MenuItems: [HOME_MENU_ITEM_BINDING, "ju:MenuItem", "*"]
};
var MENU_SECTION_PROJECT   = "http://jazz.net/ns/ui/menuSection#project";

jazz.app.internal.ProjectsUtil = {
	getProjectHomeLink: function (uri, publisher) {
		var deferred = new dojo.Deferred();
		var serviceDeferred = DiscoveredService.loadDiscoveryServices(NAMESPACES.ju + "HomeMenu", HOME_MENU_DISCOVERY_BINDING, true);
		serviceDeferred.addCallback(dojo.hitch(this, function(services){
			if (!services || !services.HomeMenu || services.HomeMenu.length == 0){
				throw new Error("No HomeMenu entries found.");	//throw to the errback code
				deferred.errback();
				return services;
			}
			var about = jazz.app.internal.applicationAbout.get()[publisher]; 
			if (!about) {
				deferred.errback();
				return services;
			}
			
			var serviceDef;
			dojo.forEach(services.HomeMenu, function(service){
				if (service.menuSection && service.menuSection != MENU_SECTION_PROJECT) return;
				if (publisher != service.applicationAbout) return;
				var servUrl = service.service;
				servUrl += ((servUrl.indexOf("?") >= 0) ? "&" : "?") + "relatedUri="+uri;
				
				serviceDef = XhrUtil.getXml(servUrl, {schema: HOME_MENU_SERVICE_BINDING, passive: true});
				serviceDef.addCallback(dojo.hitch(this, function (doc){
					var items = doc.MenuItems;
					var link;
					for (var i = 0; i < items.length; i++) {
						if(items[i].resource == uri) {
							link = items[i].link;
							break;
						}
					}
					deferred.callback(link);
				}));
				serviceDef.addErrback(function() {
					deferred.errback();
				})
			});
			if (!serviceDef) {
				deferred.errback();
				return services;
			}
		}));
		
		serviceDeferred.addErrback(dojo.hitch(this, function(err){
			console.warn("[ProjectsUtil] Discovery service error: ", dojo.toJson(err));
		}));
		
		return deferred;
	}
}
})();