/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.ProxyRequestInterceptor");

dojo.require("net.jazz.ajax.oauthDomains");
dojo.require("net.jazz.ajax.friendDomains");
dojo.require("net.jazz.ajax.hasLocalRepository");
dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("net.jazz.ajax.contextRoot");

var encodeProxiedURIFunction = null;
var oauthDomains = net.jazz.ajax.oauthDomains();
var friendDomains = net.jazz.ajax.friendDomains();

/**
 * Checks to see if the incoming request contains a URL with a different origin
 * than the current page (that is either a different scheme, host, or port). If
 * the request URL has a different origin, this request will rewrite the URL to
 * go to the Jazz Web UI Server proxy, or to the current server if the client
 * accessed the server via an alias (i.e. not the public URL).
 * 
 * @param {Object} request the request to check for different origin
 */
jazz.app.internal.ProxyRequestInterceptor = function(request){
	var url = dojo.getObject("args.url", false, request);
	if(!url)
		return;
	var urlObj = new dojo._Url(request.args.url);
	dojo.setObject('headers.X-jazz-downstream-auth-client-level',
			'4.0', request.args);
	if (useAlias(urlObj)) {
		request.args.url = urlObj.path;
		if(urlObj.query)
			request.args.url += "?" + urlObj.query;
	} else if (useProxy(urlObj)) {
		request.args.url = encodeProxiedURIFunction(request.args.url);
	}
	if (encodeProxiedURIFunction !== jazz.app.proxy._originalEncodeProxiedURI) {
		dojo.setObject('headers.X-jazz-proxy-chained-callback',
				LOCAL_URL.scheme + '://' + LOCAL_URL.host + ':' + LOCAL_URL.port + net.jazz.ajax.contextRoot() + '/web/net.jazz.ajax.service/oauth/internal/OAuthRedirectPopup.html?bogus=param',
				request.args);
	} else if (!net.jazz.ajax.hasLocalRepository()) {
		dojo.setObject('headers.X-jazz-proxy-chained-callback',
				LOCAL_URL.scheme + '://' + LOCAL_URL.host + ':' + LOCAL_URL.port + net.jazz.ajax.contextRoot() + '/oauthCallback',
				request.args);
	}
};

/**
 * Sets the function that will be used to construct proxy URIs.
 * 
 * @param {Function} paramencodeProxiedURIFunction the function that will be used to
 *  construct proxy URIs.
 */
jazz.app.internal.ProxyRequestInterceptor.setEncodeProxiedURIFunction = function(paramEncodeProxiedURIFunction) {
	encodeProxiedURIFunction = paramEncodeProxiedURIFunction;
};

/**
 * Checks the URL object's port property and if the port property is not set,
 * sets it to either "80" (HTTP) or "443" (HTTPS).
 * 
 * @param {Object} url the URL object to check and perhaps normalize
 */
function normalizePort(url) {
	var scheme = url.scheme || "http";
	if(!url.port) {
		url.port = scheme === "http" ? "80" : "443";
	}
}

// normalize the port property of the current page for later comparison with
// incoming URLs
var LOCAL_URL = new dojo._Url(location.href); 
normalizePort(LOCAL_URL);

// get frontside URL and and check if we're using an alias
var FRONTSIDE_URL = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL;
var IS_LOCAL_URL_ALIAS = false;
if (FRONTSIDE_URL) {
	FRONTSIDE_URL = new dojo._Url(FRONTSIDE_URL);
	normalizePort(FRONTSIDE_URL);
	IS_LOCAL_URL_ALIAS = !originsAreEquivalent(LOCAL_URL, FRONTSIDE_URL);
}

/**
 * Returns true if the URL object is to the public URL of this server and that
 * the client has accessed the server via an alias (i.e. not the public URL).
 * 
 * @param {Object} url the URL object to check
 */
function useAlias(url) {
	// not needed unless we're using an alias
	if (!IS_LOCAL_URL_ALIAS) {
		return false;
	}
	// optimize for the most common case (request to a non-complete URL)
	if(!url.host)
		// if host isn't set, scheme and port don't even make sense
		return false;
	if (FRONTSIDE_URL && FRONTSIDE_URL.path != net.jazz.ajax.contextRoot())
		// if context root doesn't match frontside URL path, then it's not a
		// simple case of using an alias; there must be a proxy
		return false;
 	
	normalizePort(url);
	
	// if we're trying to talk to the current server via frontside URL,
	// then use the alias
	return originsAreEquivalent(url, FRONTSIDE_URL) && matchesAuthDomain(url);
}

/**
 * Returns true if the URL object is from a different origin than the current
 * page, otherwise false.
 * 
 * @param {Object} url the URL object to check
 */
function useProxy(url) {
	// optimize for the most common case (request to a non-complete URL)
	if(!url.host)
		// if host isn't set, scheme and port don't even make sense
		return false;
	
	normalizePort(url);
	
	// Compare the origin of the target URL vs. the web page's origin. If it's
	// different then proxy the request
	if(!originsAreEquivalent(url, LOCAL_URL))
		return true;
	
	if (net.jazz.ajax.hasLocalRepository()) 
		return !matchesAuthDomain(url) && matchesFriendDomain(url);
	else 
		return matchesFriendDomain(url) || !matchesAuthDomain(url);
}

function matchesFriendDomain(url) {
	for (var i=0; i < friendDomains.length; i++) {
		var curFriend = new dojo._Url(friendDomains[i]);
		normalizePort(curFriend);
		var curFriendPath = curFriend.path;
		if (originsAreEquivalent(url, curFriend) && (url.path === curFriend.path || url.path.indexOf(curFriendPath) === 0))
			return true;
	}
	return false;
}

function matchesAuthDomain(url) {
	for (var i = 0; i < oauthDomains.length; i++) {
		var curDomain = new dojo._Url(oauthDomains[i]);
		normalizePort(curDomain);
		var curDomainPath = curDomain.path;
		if (originsAreEquivalent(url, curDomain) && (url.path.indexOf(curDomainPath) === 0))
			return true;
	}
	return false;
}

/**
 * Compare the two dojo._Url objects' scheme, host, and port. Returns 'true' if
 * all parts are the same (disregarding case differences), otherwise false.
 * 
 * @param {Object} url1
 * @param {Object} url2
 */
function originsAreEquivalent(url1, url2) {
	var props = ["scheme", "host", "port"];
	for(var i=0; i<props.length; i++) {
		var prop = props[i];
		if(url1[prop].toLowerCase() !== url2[prop].toLowerCase()) {
			return false;
		}
	}
	return true;
}

})();