/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.adapter.WidgetGadgetUriDialog");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ensemble.internal.adapter.WidgetGadgetUri");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

(function() {

var _messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");
var WidgetGadgetUri = jazz.ensemble.internal.adapter.WidgetGadgetUri;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

dojo.declare("jazz.ensemble.internal.adapter.WidgetGadgetUriDialog", jazz.ui.Dialog, {
	
	iconPath: dojo.moduleUrl("jazz.ensemble", "internal/graphics/widget-catalog/images/icon_gadget.gif").toString(),
	primaryTitle: _messages.gagdetUrlDialog,
	width: "900px",
	
	constructor: function(args) {
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		this._createContent();
	},
	
	_createContent: function() {
		this.contentNode = dojo.create("div", {"class": "jazz-ensemble-internal-adapter-WidgetGadgetUriDialog"}, this.containerNode);
		if (this.viewlet && this.viewletDefinition) {
			this._initSettings();
			this._openSocialUriContent = new WidgetGadgetUri({
				frontsizeUrl : this.viewlet.getWebURL(),
				viewletDefinitionId : this.viewletDefinition.id,
				settings: this._settings
			});
		} else {
			this._openSocialUriContent = new WidgetGadgetUri({
				frontsizeUrl : this.frontsizeUrl,
				viewletEntryId: this.viewletEntryId
			});			
		}
		this._openSocialUriContent.placeAt(this.contentNode);
		dojo.create("input", {
			type: "submit",
			value: _messages.dialogClose,
			"class": "close",
			onclick: dojo.hitch(this, this.close)
		}, dojo.create("div", {"class": "button-bar"}, this.contentNode));		
	},
	
	_initSettings: function() {
		var prefs = this.context.getPreferences();
		var prefDefs = this.viewletDefinition.preferenceDefinitions;
		this._settings = [];

		this._settings.push({
			id: "_viewletTitle",
			value: encodeURIComponent(this.context.getTitle())
		});
		
		if (this._hasScope()) {
			this._settings.push({
				id: "_scope",
				value: encodeURIComponent(this.viewlet.getScope())
			});
			var scopeItemId = this.viewlet.getScopeItem() ? this.viewlet.getScopeItem().itemId : null;
			this._settings.push({
				id: "_scopeItem",
				value: encodeURIComponent(scopeItemId)
			});
			
			if (this._hasScopeSubteams()) {
				this._settings.push({
					id: "_scopeSubteams",
					value: this.viewlet.getScopeSubteams() ? "true" : "false"
				});
			}
		}
		if (this._hasPrefs()) {
			for (var i = 0; i < prefDefs.length; i++) {
				var value =  this.viewlet.getPreference(prefDefs[i].id);
				if (typeof value === "undefined" || value === null) continue;
				var setting = {
					id: prefDefs[i].id,
					value: encodeURIComponent(value)
				};
				this._settings.push(setting);
			}
		}
		
		var memento = this.viewlet.getViewletState();
		if (memento != undefined) {
			this._settings.push({
				id: "_viewletContentMemento",
				value: encodeURIComponent(Util.toJson(memento.getState()))
			});
		}
	},
	
	_hasScope: function() {
		return this.viewletDefinition.scopeSensitive;
	},
	
	_hasScopeSubteams: function() {
		return this.viewletDefinition.scopeSubteams == "yes" || this.viewletDefinition.scopeSubteams == "yes-and-check";
	},
	
	_hasPrefs: function() {
		var defs = this.viewletDefinition.preferenceDefinitions;
		if (!defs || defs.length==0) return false;

		// test for hidden
		for (var i=0; i<defs.length; i++) {
			if (defs[i].preferenceType!="HIDDEN")
				return true;
		}
		return false;
	},
	
	destroy: function() {
		this._openSocialUriContent.destroy();
		this.inherited(arguments);		
	}
});
})();
