/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.menu.MenuItem");

dojo.require("dijit.Menu");
dojo.require("jazz.util.moduleResource");

(function() {


/**
 * A more versatile version of dijit.MenuItem that:
 *   - For its icon, accepts either a className with background image or an image src (in order to support images that come from other servers).
 *   - For its onClick behavior, accepts either an onClick handler or an href (to support CTRL+Click and right click to open in new window).
 * @name jazz.ui.menu.MenuItem
 * @class <p>A versatile menu item for web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit.MenuItem
 * @extends dijit._Container
 */
dojo.declare("jazz.ui.menu.MenuItem", dijit.MenuItem, {
	
	/**
	 * URL of a 16x16 pixel image to use as the menu item's icon.  The URL may be absolute, relative or 
	 * module-relative. The value is passed through jazz.util.moduleResource, so, for 
	 * example, jazz.ui/graphics/myimage.png will resolve properly.
	 * @type String
	 * @default ""
	 */
	icon: "",
	
	/**
	 * Class that specifies a background image to use as the MenuItem's icon.
	 * @type String
	 * @default ""
	 */
	iconClass: "",
	
	/**
	 * URL to follow when the user clicks the menu item.
	 * @type String
	 * @default ""
	 */
	href: "",

	/**
	 * Specify whether this menu item is selected or not. If true, shows a black dot.
	 * @type boolean
	 * @default false
	 */
	selected: false,
	
	// internals
	
	templateString: "<tr class=\"jazz-ui-menu-MenuItem dijitReset dijitMenuItem\" dojoAttachPoint=\"focusNode\" waiRole=\"menuitem\" tabIndex=\"-1\"\n\t\tdojoAttachEvent=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\n\t<td class=\"dijitReset dijitMenuItemLabel\" dojoattachpoint=\"_td\" colspan=\"3\"><span class=\"MenuItemContent\" dojoattachpoint=\"_wrapper\"><img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuItemIcon ${iconClass}\" dojoAttachPoint=\"iconNode\"><span dojoAttachPoint=\"containerNode\"></span></span></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" dojoAttachPoint=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" waiRole=\"presentation\">\n\t\t<div dojoAttachPoint=\"arrowWrapper\" style=\"visibility: hidden\"></div>\n\t</td>\n</tr>\n",
	postCreate: function() {
		this.inherited(arguments);
		this.focusNode.style.cursor="pointer";
		if (this.href) {
			var a = dojo.create("a", {"class": "itemHref", tabIndex: "-1", href: this.href});
			dojo.place(this._wrapper, a);
			dojo.place(a, this._td);
		}
	},
	_setIconAttr: function(icon) {
		icon = jazz.util.moduleResource(icon);
		this.icon = icon;
		this.iconNode.src = icon;
	},
	_setSelectedAttr: function(selected) {
		if (selected) {
			if (!this._bullet) this._bullet = dojo.create("span", {"class": "bullet"}, this._td, "first");
		} else if (this._bullet){
			this._td.removeChild(this._bullet);
			delete this._bullet;
		}
		dijit.setWaiState(this.domNode, "checked", selected);
		dojo.toggleClass(this.domNode, "selected", selected);
		this.selected = selected;
	},
	_onClick: function(evt){
		var parent = this.getParent();
		if (this.href && !this.disabled) {
			if (evt.type == "keyup" || (evt.target && evt.target.parentNode == this.domNode)) {
				var PlatformUI = dojo.getObject("net.jazz.ajax.ui.PlatformUI");
				if (!PlatformUI || !PlatformUI.getWorkbench().shouldVetoNavigation(this.href)) {
					location.href = this.href;
				}
				parent.onItemClick(this, evt);
			} else {
				setTimeout(dojo.hitch(parent, parent.onItemClick, this, evt), 0);	
			}
		} else {
			parent.onItemClick(this, evt);
			dojo.stopEvent(evt);
		}
	},
	_setStateClass: function() {
		if (!this.domNode) return;
		// prevent selected from being used by _CssStateMixin
		var sel = this.selected;
		this.selected = false;
		this.inherited(arguments);
		this.selected = sel;
	}
});
})();