/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.ui.menu.NodeMenuItem");

dojo.require("dijit.MenuItem");

/**
 * An MenuItem that accepts a DOM node or HTML as its content.
 * @name jazz.ui.menu.NodeMenuItem
 * @class <p>A MenuItem that accepts a DOM node or HTML as its content.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit.MenuItem
 * @extends dijit._Container
 */
dojo.declare("jazz.ui.menu.NodeMenuItem", [dijit.MenuItem, dijit._Container], {
	
	/**
	 * HTML or text to use as the content of the menu item.  To add dom nodes, 
	 * use this.addChild instead (Do not use both APIs).
	 * @field
	 * @type String
	 */	
	label: "",
	
	/**
	 * Specify whether or not this menu item is focusable/selectable.
	 * @field
	 * @type Boolean
	 */
	focusable: false,

	/**
	 * Specify whether or not this menu item is a menu section title. 
	 * Adds the "menu-title" className, which makes the text small and gray.
	 * @field
	 * @type Boolean
	 */
	isTitle: false,
	
	// internals
	
	templateString: "<tr class=\"jazz-ui-menu-NodeMenuItem dijitMenuItem\" waiRole=\"menuitem\"><td colspan=\"4\"><div dojoAttachPoint=\"containerNode, focusNode\" tabIndex=\"-1\"></div></td></tr>",
	
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		label: { node: "containerNode", type: "innerHTML" },
		className: { node: "containerNode", type: "class"}
	}),
	_cssMouseEvent: function() {
		// NodeMenuItems don't need hover states
		return;
	},
	postCreate: function(){
		this.inherited(arguments);
		if (!this.focusable) {
			dojo.setSelectable(this.domNode, false);
			this.focusNode.removeAttribute("tabIndex");
		}
	},
	focus: function(){
		if (!this.domNode || !this.focusable) return;
		this.inherited(arguments);
	},
	isFocusable: function(){
		return !!this.focusable;
	},
	_setIsTitleAttr: function(isTitle) {
		dojo.toggleClass(this.containerNode, "menu-title", isTitle);
		this.isTitle = isTitle;
	},
	_setSelected: function(){
		if (this.focusable) this.inherited(arguments);
	}
});
})();
