SET CURRENT SCHEMA $CALM_SCHEMA;

CREATE TABLE TEAM_CAPACITY (
		TEAM_CAPACITY_ID INTEGER NOT NULL,
		TEAM_ID INTEGER NOT NULL,
		PROJECT_ID INTEGER NOT NULL,
		TIMELINE_ID INTEGER NOT NULL,
		CAPACITY BIGINT,
		REC_DATETIME TIMESTAMP DEFAULT NOT NULL,
		SOURCE_ID VARCHAR (36) DEFAULT '-1' NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
) IN DATABASE RIDW;

CREATE TABLE GROUP_NAME_FACET (
		GROUP_NAME_FACET_ID INTEGER NOT NULL,
		GROUP_NAME VARCHAR(255) DEFAULT 'Group Missing' NOT NULL,
		NAME VARCHAR(255) DEFAULT 'Name Missing' NOT NULL,
		FACET VARCHAR(255) DEFAULT 'Facet Missing' NOT NULL
) IN DATABASE RIDW;

CREATE TABLE SCALAR_COUNTER (
		SCALAR_COUNTER_ID INTEGER NOT NULL,
		REC_DATETIME TIMESTAMP DEFAULT NOT NULL,
		GROUP_NAME_FACET_ID INTEGER DEFAULT -1 NOT NULL,
		VAL BIGINT,
		MIN_VALUE BIGINT,
		MAX_VALUE BIGINT,
		SOURCE_ID VARCHAR (36) DEFAULT '-1' NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT NOT NULL
) IN DATABASE RIDW;

CREATE TABLE RANGE_COUNTER (
		RANGE_COUNTER_ID INTEGER NOT NULL,
		REC_DATETIME TIMESTAMP DEFAULT NOT NULL,
		GROUP_NAME_FACET_ID INTEGER DEFAULT -1 NOT NULL,
		COUNT_VALUE BIGINT,
		TOTAL_VALUE DECIMAL(20, 2),
		MIN_VALUE DECIMAL(20, 2),
		MAX_VALUE DECIMAL(20, 2),
		AVERAGE_VALUE DECIMAL(20, 2),
		STD_DEV_VALUE DECIMAL(20, 2),
		SOURCE_ID VARCHAR (36) DEFAULT '-1' NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT NOT NULL
) IN DATABASE RIDW;

CREATE TABLE INTEGRAL_RANGE_COUNTER (
		INTEGRAL_RANGE_COUNTER_ID INTEGER NOT NULL,
		REC_DATETIME TIMESTAMP DEFAULT NOT NULL,
		GROUP_NAME_FACET_ID INTEGER DEFAULT -1 NOT NULL,
		COUNT_VALUE BIGINT,
		TOTAL_VALUE BIGINT,
		MIN_VALUE BIGINT,
		MAX_VALUE BIGINT,
		AVERAGE_VALUE BIGINT,
		STD_DEV_VALUE BIGINT,
		SOURCE_ID VARCHAR (36) DEFAULT '-1' NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT NOT NULL
) IN DATABASE RIDW;

CREATE TABLE ITEM_TYPE_STORAGE (
		ITEM_TYPE_STORAGE_ID INTEGER NOT NULL,
		REC_DATETIME TIMESTAMP DEFAULT NOT NULL,
		NAMESPACE VARCHAR(255) DEFAULT 'Item Type Namespace Missing' NOT NULL,
		NAME VARCHAR(255) DEFAULT 'Item Type Name Missing' NOT NULL,
		NUM_STATES BIGINT,
		NUM_ITEMS BIGINT,
		SIZE_VAL BIGINT,
		CONTENT_SIZE BIGINT,
		ORM_SIZE BIGINT,
		PERSISTED_CONTENT_SIZE BIGINT,
		SOURCE_ID VARCHAR (36) DEFAULT '-1' NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT NOT NULL
) IN DATABASE RIDW;

CREATE INDEX GRNMFC_IDX1 ON GROUP_NAME_FACET (GROUP_NAME ASC, NAME ASC, FACET ASC);

ALTER TABLE TEAM_CAPACITY ADD PRIMARY KEY (TEAM_CAPACITY_ID);

ALTER TABLE GROUP_NAME_FACET ADD PRIMARY KEY (GROUP_NAME_FACET_ID);

ALTER TABLE SCALAR_COUNTER ADD PRIMARY KEY (SCALAR_COUNTER_ID);

ALTER TABLE RANGE_COUNTER ADD PRIMARY KEY (RANGE_COUNTER_ID);

ALTER TABLE INTEGRAL_RANGE_COUNTER ADD PRIMARY KEY (INTEGRAL_RANGE_COUNTER_ID);

ALTER TABLE ITEM_TYPE_STORAGE ADD PRIMARY KEY (ITEM_TYPE_STORAGE_ID);

ALTER TABLE TEAM_CAPACITY ADD CONSTRAINT TEAMCAP_FK1 FOREIGN KEY (TEAM_ID)
	REFERENCES $NF_SCHEMA.TEAM (TEAM_ID);

ALTER TABLE TEAM_CAPACITY ADD CONSTRAINT TEAMCAP_FK2 FOREIGN KEY (PROJECT_ID)
	REFERENCES $NF_SCHEMA.PROJECT (PROJECT_ID);

ALTER TABLE TEAM_CAPACITY ADD CONSTRAINT TEAMCAP_FK3 FOREIGN KEY (TIMELINE_ID)
	REFERENCES $NF_SCHEMA.TIMELINE (TIMELINE_ID);

ALTER TABLE SCALAR_COUNTER ADD CONSTRAINT SCALCNT_FK1 FOREIGN KEY (GROUP_NAME_FACET_ID)
	REFERENCES GROUP_NAME_FACET (GROUP_NAME_FACET_ID);
	
ALTER TABLE RANGE_COUNTER ADD CONSTRAINT RANGECNT_FK1 FOREIGN KEY (GROUP_NAME_FACET_ID)
	REFERENCES GROUP_NAME_FACET (GROUP_NAME_FACET_ID);

ALTER TABLE INTEGRAL_RANGE_COUNTER ADD CONSTRAINT INTRGCNT_FK1 FOREIGN KEY (GROUP_NAME_FACET_ID)
	REFERENCES GROUP_NAME_FACET (GROUP_NAME_FACET_ID);

CREATE VIEW VW_TEAM_CAPACITY AS
 (SELECT TC.TEAM_CAPACITY_ID, TC.TEAM_ID, T.NAME AS TEAM_NAME, TC.PROJECT_ID, P.NAME AS PROJECT_NAME, TC.TIMELINE_ID, 
  TL.NAME AS TIMELINE_NAME, TC.CAPACITY, TC.REC_DATETIME, TC.SOURCE_ID, TC.ISSOFTDELETED
  FROM $CALM_SCHEMA.TEAM_CAPACITY TC
  JOIN $NF_SCHEMA.PROJECT P   ON P.PROJECT_ID   = TC.PROJECT_ID
  JOIN $NF_SCHEMA.TEAM T      ON T.TEAM_ID      = TC.TEAM_ID 
  JOIN $NF_SCHEMA.TIMELINE TL ON TL.TIMELINE_ID = TC.TIMELINE_ID);
  
CREATE VIEW VW_SCALAR_COUNTER AS
 (SELECT A.SCALAR_COUNTER_ID, A.REC_DATETIME, B.GROUP_NAME, B.NAME, B.FACET,
 A.VAL, A.MIN_VALUE, A.MAX_VALUE, A.SOURCE_ID, A.ISSOFTDELETED 
 FROM $CALM_SCHEMA.SCALAR_COUNTER A 
 JOIN $CALM_SCHEMA.GROUP_NAME_FACET B ON A.GROUP_NAME_FACET_ID = B.GROUP_NAME_FACET_ID);

CREATE VIEW VW_RANGE_COUNTER AS
 (SELECT A.RANGE_COUNTER_ID, A.REC_DATETIME, B.GROUP_NAME, B.NAME, B.FACET,
 A.COUNT_VALUE, A.TOTAL_VALUE, A.MIN_VALUE, A.MAX_VALUE, A.AVERAGE_VALUE, A.STD_DEV_VALUE, A.SOURCE_ID, 
 A.ISSOFTDELETED 
 FROM $CALM_SCHEMA.RANGE_COUNTER A 
 JOIN $CALM_SCHEMA.GROUP_NAME_FACET B ON A.GROUP_NAME_FACET_ID = B.GROUP_NAME_FACET_ID);

CREATE VIEW VW_INTEGRAL_RANGE_COUNTER AS
 (SELECT A.INTEGRAL_RANGE_COUNTER_ID, A.REC_DATETIME, B.GROUP_NAME,
 B.NAME, B.FACET, A.COUNT_VALUE, A.TOTAL_VALUE, A.MIN_VALUE, A.MAX_VALUE, A.AVERAGE_VALUE, A.STD_DEV_VALUE,
 A.SOURCE_ID, A.ISSOFTDELETED 
 FROM $CALM_SCHEMA.INTEGRAL_RANGE_COUNTER A 
 JOIN $CALM_SCHEMA.GROUP_NAME_FACET B ON A.GROUP_NAME_FACET_ID = B.GROUP_NAME_FACET_ID);

CREATE VIEW VW_ITEM_TYPE_STORAGE AS
(SELECT ITEM_TYPE_STORAGE_ID, REC_DATETIME, NAMESPACE, NAME, NUM_STATES,
NUM_ITEMS, SIZE_VAL, CONTENT_SIZE, ORM_SIZE, PERSISTED_CONTENT_SIZE, SOURCE_ID, ISSOFTDELETED 
FROM $CALM_SCHEMA.ITEM_TYPE_STORAGE);

GRANT SELECT ON $CALM_SCHEMA.VW_TEAM_CAPACITY TO REPUSER;
GRANT SELECT ON $CALM_SCHEMA.VW_SCALAR_COUNTER TO REPUSER;
GRANT SELECT ON $CALM_SCHEMA.VW_RANGE_COUNTER TO REPUSER;
GRANT SELECT ON $CALM_SCHEMA.VW_INTEGRAL_RANGE_COUNTER TO REPUSER;
GRANT SELECT ON $CALM_SCHEMA.VW_ITEM_TYPE_STORAGE TO REPUSER;

UPDATE $CONFIG_SCHEMA.SYSCONFIG_INFO SET INT_VALUE=5, CHAR_VALUE='4.0.1' WHERE INFO_ID=16;
UPDATE $CONFIG_SCHEMA.SYSCONFIG_INFO SET INT_VALUE=20, CHAR_VALUE='4.0.1 M1' WHERE INFO_ID=17;

INSERT INTO $CALM_SCHEMA.GROUP_NAME_FACET (GROUP_NAME_FACET_ID, GROUP_NAME, NAME, FACET) VALUES (-1,'Info not available', 'Info not available', 'Info not available');

