/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.Dashboard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.contextRoot");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("jazz.ui.ErrorPage");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.bindings");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");

//require this here to save a round-trip inside WidgetPanel
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletAdapterFactory");

dojo.require("jazz.core.loader");
dojo.require("jazz.ensemble.internal.client.HttpConstants");
dojo.require("jazz.ensemble.internal.client.RestClient");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.util.binding");
dojo.require("jazz.util.html");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var binding = jazz.util.binding;
var bindings = com.ibm.team.dashboard.web.ui.internal.util.bindings;
var HttpConstants = jazz.ensemble.internal.client.HttpConstants;
var html = jazz.util.html;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardInitializationData = com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardContainer = com.ibm.team.dashboard.web.ui.internal.DashboardContainer;
var MessageBox = jazz.ui.MessageBox;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var RestClient = jazz.ensemble.internal.client.RestClient;

var messages;

/**
 * An embeddable dashboard widget which allows any page to contain a dashboard. The
 * widget is given a URL to a dashboard resource, where it will GET and PUT to load
 * and persist the state of the dashboard.
 * 
 * @see https://jazz.net/wiki/bin/view/Main/DashboardResourceAPI
 * @see https://jazz.net/wiki/bin/view/Main/DashboardServiceAPI
 * @since 0.7
 * 
 * NOTE: This is provisional API. Clients using this API do so with the awareness that
 * this class may be removed or substantially changed at future times without warning.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.Dashboard",
	[Observable, dijit._Widget, dijit._Templated, dijit._Container], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/Dashboard.html"),
	widgetsInTemplate: true,
	
	/**
	 * (Required)
	 * The URL of the dashboard resource containing the state of the dashboard
	 * to display. e.g. "https://example.com/jts/dashboards/123"
	 * 
	 * @see https://jazz.net/wiki/bin/view/Main/DashboardResourceAPI
	 * @see https://jazz.net/wiki/bin/view/Main/DashboardServiceAPI
	 */
	url: null,
	
	/**
	 * An event fired after the dashboard's dirty state has changed. This does not fire in auto-save
	 * mode. Use dojo/this.connect to listen for this event.
	 * 
	 * @param {boolean} isDirty whether the dashboard is currently dirty or not.
	 * @since 4.0.1
	 */
	onDirtyStateChange: function(isDirty) {},
	
	/**
	 * An event fired after the dashboard container has loaded the dashboard state and the widgets
	 * have begun loading. Use dojo/this.connect to listen for this event.
	 * 
	 * @since 4.0.1
	 */
	onLoad: function() {},
	
	/**
	 * An event fired after the user has navigated to another page of widgets in the dashboard.
	 * Use dojo/this.connect to listen for this event.
	 * 
	 * @param {string} pageId the id of the page the user has navigated to.
	 * @since 4.0.1
	 */
	onPageChange: function(pageId) {},
	
	/**
	 * An event fired after the dashboard's title has changed. This is not fired on load. Use
	 * dojo/this.connect to listen for this event.
	 * 
	 * @param {string} newTitle the new dashboard title.
	 * @since 4.0.1
	 */
	onTitleChange: function(newTitle) {},
	
	/**
	 * (Optional) A widget to include on the first tab of the dashboard that includes
	 * context information.  Pass the widget id or a widget instance. If you pass a
	 * widget instance, you are responsible for destroying it.
	 * 
	 * @type String or _Widget
	 */	
	contextInfoWidget: null,
	
	/**
	 * (Optional)
	 * The unique id of the widget page to show initially upon loading.
	 * 
	 * @type String
	 */
	page: null,
	
	/**
	 * (Required for SCOPE_PROJECT_AREA, SCOPE_TEAM_AREA, Optional for SCOPE_CONTRIBUTOR)
	 * The name of the project area containing the dashboard, e.g. "Jazz Foundation".
	 * 
	 * @deprecated use the url parameter to specify which dashboard resource to load
	 */
	projectArea: null,
	
	/**
	 * (Required) The scope of the dashboard; one of the following properties from
	 * DashboardConstants: SCOPE_PROJECT_AREA, SCOPE_TEAM_AREA.
	 * 
	 * @deprecated use the url parameter to specify which dashboard resource to load
	 */
	scope: null,
	
	/**
	 * (Required for SCOPE_TEAM_AREA) This is required if scope is SCOPE_TEAM_AREA,
	 * where it is an array of team names; a path from a root team to the target team,
	 * e.g. ["Jazz Foundation Development", "Web UI", "Dashboard"]. The use of this
	 * param may be	expanded for other scopes in the future.
	 * 
	 * @deprecated use the url parameter to specify which dashboard resource to load
	 */
	scopeItem: null,
	
	/**
	 * Constructs a new dashboard widget. The required parameters listed above must be
	 * passed in as properties to the first argument, as well as any optional arguments.
	 * 
	 * e.g.
	 * var myDashboard = new Dashboard({
	 *    url: "https://example.com/jts/dashboards/123"
	 * });
	 * myDomNode.appendChild(myDashboard.domNode);
	 */
	constructor: function(args) {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	/**
	 * Destroys the dashboard widget. Should be called when the dashboard is removed
	 * from a page and no longer needed.
	 */
	destroy: function() {
		if (this._redirectTimeout) {
			clearTimeout(this._redirectTimeout);
			delete this._redirectTimeout;
		}
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		if (this.navigateAwayHandle) {
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
		}
		if (this.onBeforeUnloadHandle){
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);	
		}
		if (this._contentWidget) {
			this._contentWidget.destroy();
			delete this._contentWidget;
		}
		this.inherited(arguments);
	},
	
	/**
	 * Returns the XML DOM Document containing the state of the dashboard that was loaded.
	 * If the document is modified, you must call resourceChanged() to notify the dashboard
	 * that its resource has changed. You should not modify the document in a way that would
	 * change the visible content of the dashboard, i.e. this should only be used for
	 * modifying application-specific annotations, etc.
	 * 
	 * @return {Document} an XML Document object, or undefined if the dashboard has not yet been loaded
	 * @since 4.0
	 */
	getResource: function() {
		var bundle = this.getBundle();
		if (bundle) {
			return bundle.dashboard.content;
		}
	},
	
	/**
	 * Returns a javascript object containing key value pairs that can be used to store additional
	 * properties on the dashboard. These values will be persisted in the dashboard state on save.
	 */
	getProperties: function() {
		if (this._contentWidget && this._contentWidget instanceof DashboardContainer) {
			return this._contentWidget.getProperties();
		}
	},
	
	setProperties: function(properties) {
		if (this._contentWidget && this._contentWidget instanceof DashboardContainer) {
			return this._contentWidget.setProperties(properties);
		}
	},
	
	/**
	 * Shows the widget page with the given string id or index.
	 * 
	 * @param {string || number} page the unique id of the widget page, or page index.
	 * @since 4.0.1
	 */
	showPage: function(page) {
		this.showTab(page);
	},
	
	/**
	 * Returns a boolean indicating if the dashboard is currently dirty.
	 */
	isDirty: function() {
		if (this._contentWidget && this._contentWidget.isDirty) return this._contentWidget.isDirty();
		return false;
	},
	
	/**
	 * Returns a boolean indicating if the dashboard is currently unsaved. Will be true if the dashboard
	 * is dirty or if a save is in progress.
	 */
	isUnsaved: function() {
		if (this._contentWidget && this._contentWidget.isUnsaved) return this._contentWidget.isUnsaved();
		return false;
	},
	
	/**
	 * Refreshes the dashboard. The latest state will be retrieved from the server, and the viewlet
	 * content will be reloaded.
	 * 
	 * @since 4.0
	 */
	refresh: function() {
		if (this._contentWidget && this._contentWidget instanceof DashboardContainer) {
			this._contentWidget.reload();
		}
	},
	
	/**
	 * Notifies the dashboard widget that its underlying resource has changed. You should not modify
	 * the document in a way that would change the visible content of the dashboard, i.e. this should
	 * only be used for modifying application-specific annotations, etc.
	 * 
	 * The dashboard widget will react as if the user changed something in the dashboard (e.g. it will
	 * become dirty or will auto-save).
	 * 
	 * @since 4.0
	 */
	resourceChanged: function() {
		if (this._contentWidget && this._contentWidget instanceof DashboardContainer) {
			this._contentWidget.fireDashboardChange();
		}
	},
	
	/*------------------------------------------------------------------------
	 * INTERNAL CODE BELOW - DO NOT CALL THIS CODE.
	 * 
	 * If you need additional functionality, please request API via work item.
	 *-----------------------------------------------------------------------*/
	
	// only applicable to personal dashboard
	allowDelete: true,
	
	/**
	 * Only applicable to contributor scope dashboards. If false, the "Sharing Options"
	 * button will be hidden.
	 */
	allowSharing: false,
	
	allowSetDefault: false,

	showBreadcrumbs: true,
	
	/* void */ showTab: function(/* string || number */ tab) {
		if (this._contentWidget instanceof DashboardContainer && this._isLoaded) {
			var type = typeof(tab);
			if (type === "string") {
				this._contentWidget.showViewletPageById(tab);
			}
			else if (type === "number") {
				this._contentWidget.showViewletPage(tab);
			}
			else {
				this._contentWidget.showViewletPage(0);
			}
			this.tab = this._contentWidget.selectedTabId;
			this.setActive(true);
		}
	},
	
	/* void */ setActive: function(active) {
		if (this._contentWidget instanceof DashboardContainer) {
			this._contentWidget.setActive(active);
		}
	},
	
	/* void */ postCreate: function() {
		// attach listeners
		this.addListener(dojo.hitch(this, function() {this.onLoad()}), "onload");
		this.addListener(dojo.hitch(this, function(property, tab) {this.onPageChange(tab)}), "tab");
		this.addListener(dojo.hitch(this, function(property, newTitle) {this.onTitleChange(newTitle)}), "title");
		
		if (this.serverTemplated) {
			var cdata = this.cdata;
			var source = cdata.innerHTML;
			cdata.parentNode.removeChild(cdata);
		} else {
			var args = {tab: this.page || this.tab};
			if (this.url) {
				args.id = this.url.substring(this.url.lastIndexOf("/") + 1);
			}
			else {
				switch (this.scope) {
					case DashboardConstants.SCOPE_TEAM_AREA:
						args.team = this.scopeItem.join("/");
						break;
					case DashboardConstants.SCOPE_CONTRIBUTOR:
						// internal usage only
						for (var i in this.scopeItem) {
							args[i] = this.scopeItem[i];
						}
						break;
				}
			}
			this.viewDashboard(args, this.projectArea);
		}
	},
	
	/* object */ getBundle: function() {
		return this._contentWidget && this._contentWidget.dashboardBundle;
	},
	
	/* ServiceTracker */ getServiceTracker: function() {
		if (!this._serviceTracker) {
			this._serviceTracker = new ServiceTracker();
		}
		return this._serviceTracker;
	},
	
	/* string */ getTitle: function(includeTab) {
		if (this._contentWidget && this._contentWidget instanceof DashboardContainer) {
			var title = this._contentWidget.getTitle();
			if (includeTab && this._contentWidget.currentPage && this._contentWidget.currentPage.page) {
				var tabTitle = this._contentWidget.currentPage.page.getTitle();
				return dojo.string.substitute(messages.dashboardTitleWithTab, [title, tabTitle]);
			}
			return title;
		}
		return "";
	},
	
	/* boolean */ isVisible: function() {
		return this._isVisible;
	},
	
	/* String */ handleOnBeforeUnload: function(event) {
		if (this._redirectTimeout) {
			clearTimeout(this._redirectTimeout);
			delete this._redirectTimeout;
		}
		// if the current content widget is a dashboard, ask it for an appropriate message
		if (this._contentWidget && this._contentWidget.dashboardResource)
			return this._contentWidget.handleOnBeforeUnload();
		return false;
	},
	
	/* boolean */ promptNavigateAway: function() {
		var doNavigate = true;
		if (this._contentWidget && this._contentWidget instanceof DashboardContainer && this._contentWidget.dashboardResource) {
			doNavigate = this._contentWidget.promptNavigateAway();
		}
		if (doNavigate) {
			this._isVisible = false;
		}
		return doNavigate;	
	},
	
	viewDashboard: function(args, projectArea) {
		var dfd = DashboardInitializationData.load();
		dfd.addCallback(dojo.hitch(this, this.viewDashboard2, args, projectArea));
	},
	
	viewDashboard2: function(args, projectArea) {
		this._isVisible = true;
		this._checkNavigateAwayHandle();
		
		var tab = args.tab;
		delete args.tab;
		delete args.action;
		
		var serviceObj = {
			dashboard: this,
			args: args,
			tab: tab,
			success: function(/* ResponseDTO */ responseDTO) {
				var actions = [];
				if (this.dashboard.isPersonal) {
					actions.push({
						label: messages.errorGotoDefaultPersonal,
						iconClass: "default-dash",
						href: PlatformUI.getWorkbench().getAppPath()
					});
				} else if (this.dashboard.scope == DashboardConstants.SCOPE_TEAM_AREA || this.dashboard.scope == DashboardConstants.SCOPE_PROJECT_AREA) {
					var workbench = PlatformUI.getWorkbench();
					actions.push({
						label: messages.errorVisitProjectDash,
						iconClass: "project-icon",
						href: workbench ? workbench.getAppPath() : net.jazz.ajax.contextRoot()
					});
				}
				switch(responseDTO.status) {
					case "MIGRATION_REQUIRED":
					case "MIGRATION_IN_PROGRESS":
						this.dashboard._showMigrationPage(args, projectArea);
						break;
					case "NOT_FOUND":
						// internal usage only
						this.dashboard._createError({
							summary: messages.errorNotFound,
							actions: actions
						});
						break;
					case "MOVED":
						var linkMarkup = [
							"<a href=\"",
							responseDTO.msg,
							"\">",
							responseDTO.msg,
							"</a>"
						].join("");
						this.dashboard._createError({
							summary: dojo.string.substitute(messages.movedMsg, [linkMarkup]),
							actions: actions
						});
						// redirect in 10 seconds
						this.dashboard._redirectTimeout = setTimeout(dojo.hitch(this, function() {
							// clearTimeout doesn't always work on IE7
							if (this.dashboard._redirectTimeout) {
								location.replace(responseDTO.msg);
							}
						}), 10000);
						break;
					case "ERROR":
						// invalid request or server error
						this.dashboard._createError({summary: responseDTO.msg, actions: actions});
						break;
					case "READ_ONLY":
						// server is in read-only mode, can't create dashboard now
						this.dashboard._createError({
							summary: messages.errorCreateReadOnly,
							details: html.escape(responseDTO.msg),
							actions: actions
						});
						break;
					case "OK":
					default:
						// store license permissions as global
						dojo.setObject("com.ibm.team.dashboard.internal.permissions", responseDTO.data.permissions);
						var dashboardUrl = this.dashboard.url = responseDTO.data.dashboardUrl;
						var contentWidget = this.dashboard._contentWidget;
						if (contentWidget.dashboardUrl == dashboardUrl) {
							if (contentWidget.tabs) {
								contentWidget.showViewletPageById(this.tab);
							}
							dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
							this.dashboard.propertyChanged("onload");
							this.dashboard._isLoaded = true;
						}
						else {
							if (contentWidget && contentWidget.dashboardUrl) {
								this.dashboard._createContentWidget(this.tab);
							}
							var headers = {};
							headers[HttpConstants.ACCEPT] = HttpConstants.CT_APPLICATION_RDF_XML;
							// Add a bogus query param to make the RDF URL distinct from the HTML page
							//    See: 130601
							if (dashboardUrl.indexOf("?") > -1)
								dashboardUrl += "&";
							else
								dashboardUrl += "?";
							dashboardUrl += "contentType=application%2Frdf%25xml";
							var dfd = RestClient.getResource(dashboardUrl, RestClient.HANDLE_AS_XML, headers);
							dfd.addCallback(dojo.hitch(this, function(/* Resource */ resource) {
								var bundle = responseDTO.data;
								bundle.dashboard = resource;
								bundle.revision = _getRevision();
								resource.binding = binding.bind(resource.content.documentElement, bindings.Dashboard);
								this.dashboard._initializeContentWidget(bundle);
								dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
								this.dashboard.propertyChanged("onload");
								this.dashboard._isLoaded = true;
								return resource;
							}));
							dfd.addErrback(dojo.hitch(this, function(/* Error */ e) {
								this.dashboard._createError({summary: messages.errorRetrieving + "<br/><br/>" + e.message, actions: actions});
								return e;
							}));
						}
						break;
				}
			},
			failure: function(/* Error */ error) {
				var actions = [];
				var msg;
				if (error) {
					switch (error.status) {
						case 0:
							error.message = messages.connectionFailure;
							break;
						case 401:
							// assume it's the popup blocker
							actions.push({
								label: messages.tryAgain,
								iconClass: "com-ibm-team-dashboard-content-refresh",
								onClick: dojo.hitch(this, function() {
									this.dashboard.viewDashboard(args, projectArea);
								})
							});
							msg = error.message;
							break;
						case 403:
							msg = messages.errorNoPermission;
							var details = messages.errorNoPermission_explanation + "<br/>" + messages.errorNoPermission_useraction;
							break;
					}
				}
				if (!msg) {
					msg = messages.errorRetrieving;
				}
				if (!details && error) {
					var buf = [];
					// response
					error.status && error.ioArgs && error.ioArgs.xhr && buf.push(
						"HTTP/1.1 ", html.escape(error.status), " ", html.escape(error.ioArgs.xhr.statusText), "<br/>", html.escape(error.ioArgs.url), "<br/><br/>"
					);
					// exception
					var stacktrace = _getExceptionStackTrace(error);
					if (stacktrace) {
						buf.push("<pre>", html.escape(stacktrace), "</pre>");
					}
					else {
						// just message
						error.message && buf.push(html.escape(error.message));
					}
					details = buf.join("");
				}
				var args = {
					summary: msg,
					actions: actions
				};
				if (details) {
					args.details = details;
				}
				this.dashboard._createError(args);
			}
		};
		this._createContentWidget(tab);
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getDashboard(args.id, args.team, projectArea, srh, this.getServiceTracker(), _getRevision());
	},
	
	setCurrentDeleted: function () {
		this._contentWidget.deleted = true;
	},
	// ensure onload callbacks are fired if they are added after load completes
	// see work item 73132
	addListener: function(callback, property) {
		this.inherited(arguments);
		if (property == "onload" && this._isLoaded)
			callback("onload");
	},
	_createContentWidget: function(/* string */ dashboardTabId) {
		if (this._contentWidget){
			this._destroyContentWidget();
		}
		if (!dashboardTabId)
			dashboardTabId = "tab1";
		this._contentWidget = new DashboardContainer({dashboardWidget: this, selectedTabId: dashboardTabId, allowDelete: this.allowDelete, allowSharing: this.allowSharing, allowSetDefault: this.allowSetDefault, contextInfoWidget: this.contextInfoWidget, showBreadcrumbs: this.showBreadcrumbs});
		this._contentWidget.addListener(dojo.hitch(this, this._onTabChange), "tab");
		this._contentWidget.addListener(dojo.hitch(this, this._bubblePropertyChange), "title");
		this._contentWidget.addListener(dojo.hitch(this, this.propertyChanged), "onsave");
		this._contentWidget.addListener(dojo.hitch(this, this.propertyChanged), "ondelete");
		this._setContent(this._contentWidget.domNode);
		this._setDirtyConnect = this.connect(this._contentWidget, "onDirtyStateChange", function(isDirty) {
			this.onDirtyStateChange(isDirty);
		});
	},
	_onTabChange: function(/* string */ property, /* string */ tab) {
		this.tab = tab;
		this.propertyChanged(property, tab);
	},
	_bubblePropertyChange: function(/* string */ property, /* string */ arg) {
		this.propertyChanged(property, arg);
	},
	_hideMessage: function() {
		this.messageBox.hideMessage(false);
		this.messageArea.style.display = "none";
	},
	_initializeContentWidget: function(/* DashboardBundleDTO */ bundle) {
		if (this._contentWidget && this._contentWidget.setContent) {
			this._contentWidget.setContent(bundle);
		}
	},
	_createError: function(/* object */ args) {
		// clear old content first
		if (this._contentWidget) {
			this._destroyContentWidget();	
		}
		this._contentWidget = new jazz.ui.ErrorPage(args);
		this._setContent(this._contentWidget.domNode);
	},
	_setContent: function(/* Node */ node) {
		if (this._contentNode && this._contentNode.parentNode === this.domNode) {
			this.domNode.replaceChild(node, this._contentNode);
		}
		else {
			this.domNode.appendChild(node);
		}
		this._contentNode = node;
	},
	_showMessage: function(/* object */ args, /* boolean */ showIcon) {
		this.messageArea.style.display = "";
		this.messageBox.showMessage(args);
		dojo[showIcon ? "removeClass" : "addClass"](this.messageBox.domNode, "noIcon");
		this.messageBox.onClose = dojo.hitch(this, function() {
			this.messageArea.style.display = "none";
			args.onClose && args.onClose();
		});
	},
	_destroyContentWidget: function() {
		if (!this._contentWidget) return;
		if (this._setDirtyConnect) {
			this.disconnect(this._setDirtyConnect);
			delete this._setDirtyConnect;
		}
		this._contentWidget.destroy();
		this._contentWidget = null;
	},
	_checkNavigateAwayHandle: function() {
		// don't assume there is a workbench (e.g. RM project dashboard in embedded mode)
		if (!this.onBeforeUnloadHandle && PlatformUI.getWorkbench()){
			this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.handleOnBeforeUnload), /*priority*/ 10000);	
		}
	},
	_showMigrationPage: function(/* object */ args, /* string */ projectArea) {
		// lazy load
		var className = "com.ibm.team.dashboard.web.ui.internal.MigrationStatusPage";
		var MigrationStatusPage = dojo.getObject(className);
		if (!MigrationStatusPage) {
			jazz.core.loader.load_async(className, dojo.hitch(this, this._showMigrationPage, args, projectArea));
			return;
		}
		
		// clear old content first
		if (this._contentWidget) {
			this._destroyContentWidget();	
		}
		this._contentWidget = new MigrationStatusPage({
			handleRefresh: dojo.hitch(this, this.viewDashboard, args, projectArea)
		});
		this._setContent(this._contentWidget.domNode);
	}
});

function _getRevision() {
	try {
		// first check hash
		var params = dojo.hash();
		if (params) {
			var revision = dojo.queryToObject(params).revision;
			if (revision) {
				return revision;
			}
		}
		// then check query
		params = location.search;
		if (params) {
			return dojo.queryToObject(params.substring(1)).revision;
		}
	}
	catch (/* Error */ e) {
		console.error(e);
	}
}

function _getExceptionStackTrace(/* Error */ error) {
	if (error.errorClass) {
		var buf = [html.escape(error.errorClass)];
		var msg = error.message || error.errorMessage;
		msg && buf.push(": ", msg);
		// stack trace
		var stacktrace = error.exception || error.errorTrace;
		if (stacktrace && dojo.isArray(stacktrace)) {
			var len = stacktrace.length;
			for (var i=0;i<len;++i) {
				buf.push("\n\tat ", stacktrace[i]);
			}
			if (error.nestedError) {
				buf.push("\nCaused by: ", _getExceptionStackTrace(error.nestedError));
			}
		}
		return buf.join("");
	}
}

})();
