/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function() {
	dojo.provide("com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.declare("com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/configdata/templates/AbstractConfigurationDataEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args){
			this._processAreaEditor = args.processAreaEditor; // hold the outer project area editor
			this._editorContainer = args.editorContainer; // hold the editor container
			this._editorId = args.editorId;
			this._configurationDataContentList = args.configurationDataContentList;
		},

		postCreate: function(){

		},
		
		/**
		 * This method will be called by the Process framework to check whether the editor is dirty.
		 * return true if there are data changes in the editor, otherwise, return false.
		 */
		changesPending: function() {
		
		},
		
		/**
		 * This method is called by Process framework when the project area editor is saved,
		 * it is commonly used to do some cleanup/state clearup job when the save is done.
		 *
		 * @param {Object} saveResponse
		 */
		saved: function(saveResponse) {

		},
		
		/**
		 * Return an array of configuration data that has latest modified configuration data content
		 * Each element in the array is a JSON object, in the format:
		 * { "configurationDataId": <configurationDataId>, 
		 *   "configurationDataContent": <configurationDataContent>
		 *  }
		 */
		getUnsavedConfigurationDataList: function() {

		},
		
		/**
		 * This is a callback method that will be called by the Process framework before the editor is shown
		 * It gives the editor a chance to populate the data in the editor
		 * @param {Object} contextArguments context object that is passed into the editor to be shown.
		 */
		aboutToShowEditor: function(contextArguments) {
				
		},
		
		/**
		 * This method is called by Process framework to determine whether the project area editor is available to save or not.
		 * If this method returns false, it will prevent the user from saving the project area editor ("Save" button will be disabled).
		 * Normally, this method should return false if there is any validation error in this configuration data editor. Otherwise, return true.
		 *
		 * This method will be invoked whenever the project area editor is being asked to set dirty status to true.
		 *
		 * Derived should override this method to determine whether the save of project area editor should be prevented.
		 *
		 * @return {Boolean} true if this configuration data editor can be saved, otherwise, return false
		 */
		canSave: function() {
			return true;
		},
		
		/**
		 * This method is called by Process framework to validate the configuration data editor. If validation fails, 
		 * this method should return an error message, which could be displayed in the message area of the project area editor.
		 * Since only one error message can be shown in the project area editor at one time, so it's NOT guaranteed that 
		 * the error message will always be shown if there are some other error messages thrown by other pages.
		 * 
		 * This method will be invoked at two places if this configuration data editor has pending changes: 
		 * 1. before saving the project area editor (when "Save" button is clicked);
		 * 2. before activating this configuration data editor.
		 * 
		 * Derived should override this method to provide custom validation logic to validate the configuration data editor.
		 * 		
		 * @return {String} the error message if fails, else return null|""|undefined
		 */
		validate: function() {

		},

		/**
		 * Set this configuration data editor in error status. This will
		 * show the given error message in current project area editor. 
		 * 
		 * @param {String} message the error message
		 */
		setErrorStatus: function(message) {
			this._editorContainer.setErrorStatus(message, this._editorId);
		},
		
		/**
		 * Clear the error status for this configuration data editor. This will
		 * clear the message of current project area editor
		 */
		clearErrorStatus: function() {
			this._editorContainer.clearErrorStatus(this._editorId);
		},

		/**
		 * Show the given configuration data editor,using the contextArguments to populate the editor
		 * This is a final method that is NOT intended to be overridden
		 * 
		 * @param {String} editorId the id of the editor that will be shown
		 * @param {Object} contextArguments the context parameters that will be passed into aboutToShowEditor method by Process framework
		 */
		showConfigurationDataEditor: function(editorId, contextArguments) {
			if (editorId === this._editorId) {
				return;
			}
			this._editorContainer.showConfigurationDataEditor(editorId, contextArguments);
		},
		
		/**
		 * Get configuration data that doesn't belong to this editor.
		 * This is a final method that is NOT intended to be overridden
		 * 
		 * The return value is an array, each element is a JSON object with format:
		 * { "configurationDataId": <configurationDataId>, 
		 *   "configurationDataContent": <configurationDataContent>
		 *  }
		 * 
		 * @param {Object} configurationDataIds an array of configuration data id
		 * @param {Object} callbackContext the object context on which defines the successCallbackName and errorCallbackName
		 * @param {String} successCallbackName method name that will be called when the configuration data are fetched successfully
		 * @param {String} errorCallbackName   method name that will be called when the configuration data are fetched failed
		 */		
		getConfigurationDataListNotOwnedByThisEditor: function(configurationDataIds, callbackContext, successCallbackName, errorCallbackName) {
			// Besides the configuration data specified in editor extension,
			// sometimes the editor also need to know other configuration data,
			// especially the latest modified configuration data from other editor.
			this._editorContainer.getConfigurationDataContentList(configurationDataIds, callbackContext, successCallbackName, errorCallbackName);
		},
		
		uninitialize: function() {
			this.inherited(arguments);
		}
		
	});
})();