/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.details.MemberInfo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");

dojo.require("dojo.string");
dojo.require("jazz.util.i18n");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "MemberInfoMessages");

(function() {

var messages;
var IMG_SIZE = 26;
var MAX_IMGS = 18;

dojo.declare("com.ibm.team.dashboard.web.ui.details.MemberInfo", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/MemberInfo.html"),
	
	constructor: function(args) {
		if (!messages) messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "MemberInfoMessages");
		this.messages = messages;
	},
	
	postCreate: function() {
		if (this.dashboard.scope === com.ibm.team.dashboard.web.ui.DashboardConstants.SCOPE_PROJECT_AREA) {
			this.projectInit();
		} else if (this.dashboard.getBundle()) {
			this.teamInit();
		} else {
			this.dashboard.addListener(dojo.hitch(this,this.teamInit), "onload");
		}
	},
	
	projectInit: function() {
		var context = jazz.app.context.get();
		this._titleLink.href = net.jazz.ajax.contextRoot() + "/admin#action=com.ibm.team.process.editProjectArea&itemId=" + context.itemId;
		var dfd = jazz.client.xhrGet({
			url: net.jazz.ajax.contextRoot() + "/service/com.ibm.team.dashboard.common.internal.service.IProjectMemberService/members?projectItemId=" + context.itemId + "&count=" + MAX_IMGS,
			headers: {
				accept: "text/json"
			},
			handleAs: "json"
		});
		dfd.addCallback(dojo.hitch(this,this.handleMembers));
	},
	
	teamInit: function() {
		var context = jazz.app.context.get();
		var bundle = this.dashboard.getBundle();
		this._titleLink.href = net.jazz.ajax.contextRoot() + "/admin#action=com.ibm.team.process.editTeamArea&itemId=" + bundle.scopeItem.itemId + "&projectAreaItemId=" + context.itemId;
		var dfd = jazz.client.xhrGet({
			url: net.jazz.ajax.contextRoot() + "/service/com.ibm.team.dashboard.common.internal.service.IProjectMemberService/members?teamItemId=" + bundle.scopeItem.itemId + "&count=" + MAX_IMGS,
			headers: {
				accept: "text/json"
			},
			handleAs: "json"
		});
		dfd.addCallback(dojo.hitch(this,this.handleMembers));
	},
	
	handleMembers: function(result) {
		var members = result.members;
		this._totalCount.innerHTML = dojo.string.substitute(messages.totalCount, [result.total]);
		this._bottom.innerHTML = "";
		dojo.create("div",{innerHTML: dojo.string.substitute(messages.showingMembers,[Math.min(result.total, MAX_IMGS), result.total])},this._bottom);
		var editorLink = jazz.util.i18n.addMessage(dojo.create("div",{},this._bottom), messages.toProcessArea)[0];
		editorLink.href = this._titleLink.href;
		editorLink.innerHTML = this.dashboard.scope === com.ibm.team.dashboard.web.ui.DashboardConstants.SCOPE_PROJECT_AREA ? messages.projectArea : messages.teamArea;
		var imgUrl = net.jazz.ajax.contextRoot() + "/service/com.ibm.team.dashboard.common.internal.service.IProjectMemberService/photos";
		for (var i = 0; i < members.length; i++) {
			imgUrl += (i === 0 ? "?" : "&") + "userItemId=" + members[i].itemId;
		}
		this._contentArea.innerHTML = "";
		for (i = 0; i < members.length; i++) {
			var m = members[i];
			var a = dojo.create("a", {
					className: "user",
					title: dojo.string.substitute(messages.tooltip, [m.name, m.userId]),
					href: "#action=com.ibm.team.repository.viewUserProfile&itemId=" + m.itemId
			}, this._contentArea);
			// 201134: IE8 shows a security dialog if you set a background img on an off-DOM node.
			a.innerHTML = "<div class='photo' style='background: url(" + imgUrl + ") -" + (IMG_SIZE * i) + "px 0;'></div>";
		}
	}
});

})();