/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.util.i18n");
dojo.require("jazz.util.html");
dojo.require("jazz.app.context");
dojo.require("jazz.util.i18n");
dojo.require("net.jazz.ajax.dataServerRoot");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var HTML = jazz.util.html;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ResourceLink = jazz.ui.ResourceLink;

var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllTeamAreas.html"),
	widgetsInTemplate: true,
	
	constructor: function() {
		if (!messages) messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.allProjectDash = dojo.string.substitute(messages.allProjectDash, [jazz.app.context.get().name]);
		this.messages = messages;
		this.tableTrim = jazz.ui.StyledTable.TrimStyles.SECONDARY_HEADLESS;
	},
	postCreate: function() {
		var context = jazz.app.context.get();
		var projectName = context.name;
		this.uriRoot = (net.jazz.ajax.dataServerRoot() || Util.getContextPath(true)) + "/process/project-areas/" + context.itemId;
		var nodes = jazz.util.i18n.addMessage(this._secondary, dojo.string.substitute(messages.allTopLevel, [context.name]));
		new ResourceLink({
			uri: this.uriRoot,
			presentationUri: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, projectName),
			label: HTML.escape(context.name),
			retainLinkText: true
		}).placeAt(nodes[0]);
		nodes = jazz.util.i18n.addMessage(this._footer, messages.allProjectFooter);
		nodes[0].href = Util.getAllProjectsHref();
		nodes[1].className = "home-menu-icon";
		nodes[2].className = "all-dash-icon";
		var serviceObject = {
			success: dojo.hitch(this,function(result) {
				if (this.domNode) {
					this._roots = result.children || [];
					this._populate();
				}
			}),
			failure: dojo.hitch(this,function(error) {
				if (this.domNode) {
					this._messageBox.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: messages.allTeamsLoadError,
						details: [ {message: messages.allTeamsLoadError_explanation, code: jazz.ui.MessageBox.STATUS_TYPES.ERROR},
								{message: messages.allTeamsLoadError_useraction, code: jazz.ui.MessageBox.STATUS_TYPES.ERROR} ],
						closable: true
					});
				}
			})
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		ProcessClient.getProjectHierarchy({uuid: context.itemId}, srh);
		
		// Accessibility
		dijit.setWaiRole(this._resultsArea, "tree");
		dijit.setWaiState(this._resultsArea, "expanded", "true");
		dijit.setWaiState(this._resultsArea, "disabled", "true");
	},
	
	_populate: function() {
		if (!this._roots.length) {
			this._createRow(messages.allTeamsNoTeams);
			dojo.addClass(this.domNode, "disabled");
			return;
		}
		this._roots = this._sortAreas(this._roots);
		this._parseTeams(this._roots, 0, []);
		this._filterBox.setEnabled(true);
	},
	
	_parseTeams: function(teams, indent, pathSegs, parent) {
		for (var i = 0; i < teams.length; i++) {
			var team = teams[i];
			if (parent) team.parent = parent;
			team.uri = this.uriRoot + "/team-areas/" + team.itemId;
			if (i > 0)
				team.previousSibling = teams[i-1];
			if (i < teams.length - 1)
				team.nextSibling = teams[i+1];
			var thisPath = dojo.clone(pathSegs);
			thisPath.push(team.name);
			team.node = this._createRow(team, DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, thisPath), indent);
			team.name = team.name.toLowerCase();
			if (team.children)
				this._parseTeams(team.children, indent+1, thisPath, team);
		}
	},
	
	_createRow: function(area, href, indent) {
		area.expanded = false;
		var result = dojo.create("tr",args,this._resultsArea.tbody);
		var className = "resultRow";
		if (!indent) {
			className += " headerColumn";
			if (href) {
				className += " rootRow";
			}
		}
		var args = {className: className};
		var div = dojo.create("div",args,dojo.create("td",{},result));
		this.connect(div,"onkeypress",dojo.partial(this._onKeyPress,area));
		area.div = div;
		dijit.setWaiRole(div, "treeitem");
		dijit.setWaiState(div, "expanded", "false");
		dijit.setWaiState(div, "disabled", "false");
		var level = indent + 1;
		dijit.setWaiState(div, "level", level);
		
		div.style.paddingLeft = indent ? (1 + (indent * 2)) + "em" : "1em";
		if (href) {
			area.twistie = dojo.create("a",{href: "#", className: "twistie", innerHTML: "<span class='twistie-icon'></span>", tabindex: "-1"},div);
			if (area.children && area.children.length > 0) {
				this.connect(area.twistie, "onclick", dojo.partial(this._handleTwistie, area));
				this.connect(area.twistie, "onfocus", function() {
					area.rl.domNode.focus();
				});
			} else {
				area.twistie.style.visibility = "hidden";
			}
			dojo.create("img", {
				alt: messages.allTeamDashboard,
				title: messages.allTeamDashboard,
				src: this._blankGif,
				className: "icon team-icon"
			}, div);
			area.rl = new ResourceLink({
				uri: area.uri,
				presentationUri: href,
				label: HTML.escape(area.name),
				retainLinkText: true
			}).placeAt(div);
		} else {
			div.appendChild(document.createTextNode(area));
		}
		if (indent) {
			result.style.display = "none";
		}
		return result;
	},
	
	_onKeyPress: function(area, e) {
		switch(e.keyCode) {
		case dojo.keys.LEFT_ARROW:
			if (area.expanded) {
				this._setExpanded(area, false);
			} else if (area.parent) {
				area.parent.rl.domNode.focus();
			}
			break;
		case dojo.keys.RIGHT_ARROW:
			if (!area.children) break;
			if (!area.expanded) {
				this._setExpanded(area, true);
			} else {
				area.children[0].rl.domNode.focus();
			}
			break;
		case dojo.keys.UP_ARROW:
			if (area.previousSibling) {
				var current = area.previousSibling;
				while (current.expanded) {
					current = current.children[current.children.length-1];
				}
				current.rl.domNode.focus();
			} else if (area.parent) {
				area.parent.rl.domNode.focus();
			}
			break;
		case dojo.keys.DOWN_ARROW:
			if (area.expanded) {
				area.children[0].rl.domNode.focus();
			} else if (area.nextSibling) {
				area.nextSibling.rl.domNode.focus();
			} else {
				var current = area.parent;
				while (current && !current.nextSibling) {
					current = current.parent;
				}
				if (current)
					current.nextSibling.rl.domNode.focus();
			}
			break;
		}
	},
	
	_handleTwistie: function(area, e) {
		e.preventDefault();
		if (this._twistiesDisabled) return;
		this._setExpanded(area, !area.expanded);
	},
	
	_setExpanded: function(area, expanded) {
		area.expanded = expanded;
		dojo.toggleClass(area.twistie, "expanded", expanded);
		dijit.setWaiState(area.div, "expanded", expanded ? "true" : "false");
		this._setAreasVisible(area.children, expanded);
	},
	
	_setAreasVisible: function(areas, visible) {
		for (var i = 0; i < areas.length; i++) {
			var area = areas[i];
			area.node.style.display = visible ? "" : "none";
			if (area.expanded && area.children) {
				this._setAreasVisible(area.children, visible);
			}
		}
	},
	
	_doFilter: function(filter) {
		if (this._currentFilter && this._currentFilter == filter)
			return;
		this._currentFilter = filter;
		if (this._roots) {
			if (this._currentFilter.length == 0) {
				this._twistiesDisabled = false;
				dojo.removeClass(this.domNode, "disabled");
				this._clearFilter(this._roots);
			} else {
				this._twistiesDisabled = true;
				dojo.addClass(this.domNode, "disabled");
				var hits = this._filterAreas(this._roots);
				if (!hits) {
					if (!this._noHitsRow) {
						this._noHitsRow = this._createRow(messages.allTeamsNoneMatch);
					}
					this._noHitsRow.style.display = "";
				}
				else 
					if (this._noHitsRow) 
						this._noHitsRow.style.display = "none";
			} 
		}
	},
	
	_filterAreas: function(areas) {
		var hitCount = 0;
		var result = false;
		for (var i = 0; i < areas.length; i++) {
			dijit.setWaiState(areas[i].div, "disabled", "true");
			var childHits = false;
			if (areas[i].children && areas[i].children.length > 0) {
				childHits = this._filterAreas(areas[i].children);
				dijit.setWaiState(areas[i].div, "expanded", childHits ? "true" : "false");
				dojo.toggleClass(areas[i].twistie, "expanded", childHits === "all");
				dojo.toggleClass(areas[i].twistie, "partial", childHits === "some");
			}
			var isHit = this._testFilterRow(areas[i], childHits);
			if (isHit) {
				hitCount++;
				result = (hitCount === areas.length) ? "all" : "some";
			}
		}
		for (var i = 0; i < areas.length; i++) {
		}
		return result;
	},
	
	_testFilterRow: function(area, childHits) {
		var result = (childHits || area.name.indexOf(this._currentFilter) != -1) ? true : false;
		area.node.style.display = result ? "" : "none";
		return result;
	},
	
	_clearFilter: function(areas, visible) {
		if (typeof visible === "undefined") visible = true;
		if (this._noHitsRow)
			this._noHitsRow.style.display = "none";
		for (var i = 0; i < areas.length; i++) {
			dijit.setWaiState(areas[i].div, "disabled", "false");
			areas[i].node.style.display = visible ? "" : "none";
			dojo.removeClass(areas[i].twistie, "partial");
			dojo.toggleClass(areas[i].twistie, "expanded", areas[i].expanded);
			dijit.setWaiState(areas[i].div, "expanded", areas[i].expanded ? "true" : "false");
			if (areas[i].children && areas[i].children.length > 0)
				this._clearFilter(areas[i].children, areas[i].expanded);
		}
	},
	
	_sortAreas: function(areas) {
		areas = dojo.filter(areas, function(area) {
			return !area.archived;
		});
		areas.sort(function(a,b) {
			return a.name < b.name ? -1 : 1;
		});
		for (var i = 0; i < areas.length; i++) {
			if (areas[i].children && areas[i].children.length > 0) 
				areas[i].children = this._sortAreas(areas[i].children);
		}
		return areas;
	},
	
	_collapseAll: function(e) {
		e.preventDefault();
		if (this._twistiesDisabled) return;
		this._collapseAreas(this._roots, false);
	},
	
	_collapseAreas: function(areas, hide) {
		for (var i = 0; i < areas.length; i++) {
			var area = areas[i];
			if (!area.node) continue;
			area.node.style.display = hide ? "none" : "";
			if (!area.children) continue;
			area.expanded = false;
			dojo.removeClass(area.twistie, "expanded");
			dijit.setWaiState(area.div, "expanded", "false");
			this._collapseAreas(area.children, true);
		}
	},
	
	_expandAll: function(e) {
		e.preventDefault();
		if (this._twistiesDisabled) return;
		this._expandAreas(this._roots);
	},
	
	_expandAreas: function(areas) {
		for (var i = 0; i < areas.length; i++) {
			var area = areas[i];
			if (!area.node) continue;
			area.node.style.display = "";
			if (!area.children) continue;
			area.expanded = true;
			dojo.addClass(area.twistie, "expanded");
			dijit.setWaiState(area.div, "expanded", "true");
			this._expandAreas(area.children);
		}
	}
	
});

})();
