/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.DeployTemplatesViewer");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");
dojo.require("com.ibm.team.reports.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var MAX_MESSAGE_LENGTH = 500;
var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;

dojo.declare("com.ibm.team.reports.web.ui.internal.DeployTemplatesViewer", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/DeployTemplatesViewer.html"),
						
		constructor: function(props) {
			dojo.mixin(this, props);
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		},
		
		postCreate: function() {
			this.metanav = new com.ibm.team.reports.web.ui.internal.MetaNav({DeployTemplatesViewer: this}, this.metanav);
			this.metanav.setTitle(this.messages.deployNewTemplates);
			this.typeIcon = document.createElement("img");
			this.typeIcon.style.paddingTop = "3px";
			this.typeIcon.src = ImagePool.REPORTTEMPLATE_OBJ;
			this.typeIcon.className = "typeIcon";
			this.metanav.insertTypeIcon(this.typeIcon);
			
			this.connect(this.saveChanges, "onsubmit", this._saveChangesHandler);
			var responseHandler = new ServiceResponseHandler(
					this,
					"_setReportTemplates",
					"_remoteError");
			ReportsClient.getMissingSharedReportDescriptors(responseHandler, {processAreaUUID: this.processAreaUUID});
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			if(this.displayBreadcrumbs) this.metanav.populateResourcesBreadcrumbs();

		},

		_setReportTemplates: function(reportTemplates) {
			// Let's sort them alphabetically first
			var sortFunc = function(a,b) {
				return a.name.localeCompare(b.name);
			}
			if (reportTemplates) {
				reportTemplates.sort(sortFunc);
			}
			
			// In case there was already some content, remove it and null the list out
			if (this.reportTemplates && this.reportTemplates.length > 0) {
				this.templatesTable.innerHTML = "";
			}
			if (!reportTemplates || reportTemplates.length == 0) {
				this.setHeaderMessage({
					message: this.messages.noTemplatesFound,
					showOnlyMessage: true,
					severity: 4
				});
				return;
			}
			this.reportTemplates = reportTemplates;
			// Populate the checkbox table
			for (var i = 0; i < reportTemplates.length; i++) {
				this.templatesTable.innerHTML += "<input type=\"checkbox\" id=\"" + reportTemplates[i].reportId + "\">" + reportTemplates[i].name + "</input><br/>";
			}
		},
		
		_saveChangesHandler: function(e) {
			e.preventDefault();
			selectedTemplateIds = [];
			this.messageBox.hideMessage();
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
			if (this.reportTemplates != null) {
				for (var i = 0; i < this.reportTemplates.length; i++) {
					var item = this.reportTemplates[i];
					if (item != null) {
						var id = item.reportId;
						var input = document.getElementById(id);
						if (input == null) {
							continue;
						}
						if (input.checked == true) {
							selectedTemplateIds.push(id);
						}
					}
				}
			}
			if (selectedTemplateIds.length > 0) {
				var handler = new ServiceResponseHandler(
						this,
						"_templatesCreated",
						"_remoteError");
				this.setHeaderMessage({
					message: this.messages.deployingNewTemplates,
					showOnlyMessage: true,
					severity: 1
				});
				var args = {
					reportIds: selectedTemplateIds,
					processAreaUUID: this.processAreaUUID
				};
				ReportsClient.postSaveMissingReportDescriptors(handler, args);
			} else {
				var status = {
					type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary : this.messages.noTemplatesSelected,
					closable : false
				};
				dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
				this.messageBox.showMessage(status);
			}
		},
		
		_templatesCreated: function() {
			this.setHeaderMessage({
				message: this.messages.deployingTemplatesSuccess,
				showOnlyMessage: true,
				severity: 1
			});
			if (this.reportTemplates != null) {
				for (var i = 0; i < this.reportTemplates.length; i++) {
					var item = this.reportTemplates[i];
					if (item != null) {
						var input = document.getElementById(item.reportId);
						if (input == null) {
							continue;
						}
						if (input.checked == true) {
							input.disabled = true;
						}
					}
				}
			}
		},
						
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this.templatesTable, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this.setHeaderMessage({
				message: error.message,
				showOnlyMessage: true,
				severity: 8
			});
		},
		
		// Status message area
		setHeaderMessage: function(message) {
			this.clearValidationMessage();
			if (!message) {
				return;
			}
			
			if (!message.message || message.message.length == 0) {
				message.message = this.messages.errorUnhelpful;
			} else {
				if(message.message.lastIndexOf(".") != (message.message.length-1)) {
					message.message += ".";
				}
			}
			
            if (!message.details) {
                message.details = "";
            } else {
                if (message.details.lastIndexOf(".") != (message.details.length - 1)) {
                    message.details += ".";
                }
            }
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            } else {
            	if (message.showOnlyMessage == true) {
            		messageText = message.message
            	} else {
                	messageText = message.message + " " + message.details;            		
            	}
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this.metanav.validationMessage.__jazzMessage = message;
            if (message.severity==8) {
				this.typeIcon.src = ImagePool.ERROR;
                this.metanav.validationMessageSpan.className = "validationMessageError";
            } else {
				if (message.severity==4) {
					this.typeIcon.src = ImagePool.WARNING;
					this.metanav.validationMessageSpan.className = "validationMessageWarning";
				} else {
					this.typeIcon.src = ImagePool.INFO;
					this.metanav.validationMessageSpan.className = "validationMessageInfo";
				}
            }
            this.metanav.validationMessage.title = message.details || message.message;
//            dojox.data.dom.replaceChildren(this.validationMessage, document.createTextNode(messageText));
			this.metanav.validationMessage.innerHTML = messageText;
			window.scrollTo(0,0); // show message
		},
		
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if (!e || !e.target || !e.target.__jazzMessage) return;

			var message = e.target.__jazzMessage;
			var text = message.message;
			if (message.details) {
				text += "\n\n";
				text += message.details;
			}
			alert(text);
		},

		clearValidationMessage: function(force) {
			this.metanav.validationMessageSpan.className = "validationMessageEmpty";
		}
	}
);

})();