/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007, 2011,
 * 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.EtlStatus");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date");

dojo.require("jazz.util.html");
dojo.require("jazz.util.date");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("jazz.app.ua.PageHoverHelp");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal",
		"ReportsManagementMessages");

(function() { // module container
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
	var JobHistoryTable = null;
	var RunningJobsTable = null;
	var DefaultHistorySize = 10;
	var NumberOfJobsInput = null;
	var PageHoverHelp = jazz.app.ua.PageHoverHelp;

	var EtlStatus = dojo.declare(
					"com.ibm.team.reports.web.ui.internal.EtlStatus",
					[ dijit._Widget, dijit._Templated ],
					{	
						_csPageHelpWidgets: null,
						csHelpNode: null,
						
						templatePath : dojo.moduleUrl(
								"com.ibm.team.reports.web",
								"ui/internal/templates/EtlStatus.html"),
								
						constructor : function() {
							this.messages = dojo.i18n.getLocalization(
									"com.ibm.team.reports.web.ui.internal",
									"ReportsManagementMessages");
							this._fetchingRunningJobs = false;
							this._fetchingJobHistory = false;
							this._csPageHelpWidgets= [];
						},

						postCreate : function() {
							com.ibm.team.reports.web.ui.internal.EtlStatus.superclass.postCreate.apply(this);
							this.header = new jazz.ui.MessageBox();
							this.headerArea.appendChild(this.header.domNode);
							this._createHistoryTable();
							this._fetchProviders();
							this._createHistoryInput();
							this._createContextSensitiveHelp();
							dojo.connect(this._runQueryButton, "onclick", this,
							"_runQuery");
						},
						
						_createContextSensitiveHelp: function() {
							dojo.empty(this.csHelpNode);
							this._createCSHWidget(EtlStatus.CSH_DATA_COLLECTION_JOB_STATUS);
						},

						_createCSHWidget: function(uaKey) {
							var parms = {};
							parms.uaKey = uaKey;
							var pageHelp = new PageHoverHelp(parms);
							this.csHelpNode.appendChild(pageHelp.domNode);
							this._csPageHelpWidgets.push(pageHelp);
						},
						
						_createHistoryInput : function() {
							NumberOfJobsInput = new com.ibm.team.reports.web.ui.internal.ValidatingTextBox();
							NumberOfJobsInput.inputNode.value = DefaultHistorySize;
							NumberOfJobsInput
									.setErrorMessage(this.messages.numberOfJobsValidationError);
							var self = this;
							NumberOfJobsInput.validate = function(value) {
								var valid = value - 0 == value && value.length > 0
										&& value <= 1000 && value >= 1 && value % 1 == 0;
								if (valid) {
									self._runQueryButton.disabled = false;
								} else {
									self._runQueryButton.disabled = true;
								}
								return valid;
							};
							dojo.addClass(NumberOfJobsInput.inputNode, "smallTextBox");
							this._numberOfJobsContainer
									.appendChild(NumberOfJobsInput.domNode);
						},
						
						_runQuery: function(){
							this.header.hideMessage();
							NumberOfJobsInput._checkValidity();
							if (!NumberOfJobsInput.isValid()) {
								this._displayErrorMessage(
										this.messages.invalidSaveValues, true);
								return;
							}
							this._runQueryButton.disabled = true;
							this._fetchJobs(NumberOfJobsInput.inputNode.value);
							this._fetchRunningJobs();
						},
						
						_createHistoryTable : function() {
							JobHistoryTable = new jazz.ui.StyledTable( {
								columns : [ {
									label : this.messages.status,
									sortable : false,
									width : "8%"
								}, {
									label : this.messages.dataCollectionJob,
									sortable : false,
									width : "25%"
								}, {
									label : this.messages.startTime,
									sortable : false,
									width : "15%"
								}, {
									label : this.messages.endTime,
									sortable : false,
									width : "15%"
								}, {
									label : this.messages.totalTime,
									sortable : false,
									width : "100px"
								}  ],
								sortBy : 1,
								sortByAscending : false,
								handleSort : function() {
								},
								trim : jazz.ui.styledTable.TrimStyles.BLUE
							});

							JobHistoryTable.domNode.style.width = "100%";

							this.jobList
									.appendChild(JobHistoryTable.domNode);
						},
						
						_createCurrentlyRunningJobsTable : function() {
							RunningJobsTable = new jazz.ui.StyledTable( {
								columns : [{
									label : this.messages.dataCollectionJob,
									sortable : false,
									width : "25%"
								}, {
									label : this.messages.startTime,
									sortable : false,
									width : "15%"
								}, {
									label : this.messages.runningTime,
									sortable : false,
									width : "100px"
								}  ],
								sortBy : 2,
								sortByAscending : false,
								handleSort : function() {
								},
								trim : jazz.ui.styledTable.TrimStyles.BLUE
							});

							RunningJobsTable.domNode.style.width = "100%";
							if(this.runningJobList){
								this.runningJobList
										.appendChild(RunningJobsTable.domNode);
							}
						},
						
						_displayErrorMessage : function(message, closeable) {
							this._displayMessage(message,
									jazz.ui.MessageBox.STATUS_TYPES.ERROR,
									closeable);
						},
						

						_fetchProviders : function() {
							var providerHandler = new ServiceResponseHandler(
									this, "_fetchProvidersHandler",
									"_remoteError");

							ReportsClient
									.getDataWarehouseProvider(providerHandler);
						},

						_fetchProvidersHandler : function(provider) {
							this.SavedProvider = provider;
							if (provider.toLowerCase() == "local") {
								this._displayInfoMessage(this.messages.remoteHistoryWarning, false);
							}else{
								this._fetchJobs(DefaultHistorySize);
								this._fetchRunningJobs();
							}
						},
						
						_fetchRunningJobs : function() {
							this._fetchingRunningJobs = true;
							this._getWorkbench().showMessage(this.messages.loadingRunningJobs);
							var runningJobsHandler = new ServiceResponseHandler(
									this, "_fetchRunningJobsHandler",
									"_fetchRunningJobsError");

							ReportsClient
									.getRunningEtlJobs(runningJobsHandler);
						},

						_fetchRunningJobsHandler : function(jobs) {
							this._fetchingRunningJobs = false;
							this._getWorkbench().hideMessageArea();
							if(this._fetchingJobHistory == true){
								this._getWorkbench().showMessage(this.messages.loadingJobHistory);
							}
							if(this.runningJobList){
								while(this.runningJobList.hasChildNodes()){
									this.runningJobList.removeChild(this.runningJobList.firstChild);
								}
								if(jobs == null || jobs.length == 0){
									this.runningJobList.appendChild(document.createTextNode(this.messages.noRunningJobs));
								}else{
									this._createCurrentlyRunningJobsTable();
									for(var i = 0; i < jobs.length; i++){
										RunningJobsTable.tbody.appendChild(this._createRunningJobRow(jobs[i]));
									}
								}
							}
						},
						
						_fetchJobs: function(jobSize){
							this._fetchingJobHistory = true;
							this._getWorkbench().showMessage(this.messages.loadingJobHistory);
							var jobsHandler = new ServiceResponseHandler(
									this, "_fetchJobsHandler",
									"_remoteError");
							
							ReportsClient
									.getEtlJobHistory(jobsHandler, {size:jobSize});
						},
						
						_fetchJobsHandler: function(items) {
							this._fetchingJobHistory = false;
							this._getWorkbench().hideMessageArea();
							if(this._fetchingRunningJobs == true){
								this._getWorkbench().showMessage(this.messages.loadingRunningJobs);
							}
							while (JobHistoryTable.tbody.hasChildNodes()) {
								JobHistoryTable.tbody
										.removeChild(JobHistoryTable.tbody.firstChild);
							}
							if(this.jobListMessage){
								while(this.jobListMessage.hasChildNodes()){
									this.jobListMessage.removeChild(this.jobListMessage.firstChild);
								}
							}
							if (items === null) {
								return;
							}
							items.sort(this._sortItems);
							for(var i = 0; i < items.length; i++){
								JobHistoryTable.tbody.appendChild(this._createJobRow(items[i]));
							}
							if(items.length == 0 && this.jobListMessage){
								this.jobListMessage.appendChild(document.createTextNode(this.messages.noJobHistory));
							}
						},
						
						_sortItems: function(a,b) {
							return a.startTime < b.startTime;
						},
						
						_createJobRow: function(job){
							var row = document.createElement("tr");
							var rowStatus = job.successful ? "succeeded" : "failed";
							if(job.endTime == null || job.endTime.length == 0 || job.endTime < 1){
								rowStatus = "interrupted";
							}
							dojo.addClass(row, rowStatus);
							var status = document.createElement("td");
							dojo.addClass(status, "center-column");
							status.appendChild(document.createTextNode(this.messages[rowStatus]));
							var name = document.createElement("td");
							name.appendChild(document.createTextNode(job.name));
							var startTime = document.createElement("td");
							var startDate = new Date(job.startTime);
							startTime.appendChild(document.createTextNode(jazz.util.date.formatDif(startDate, true, false)));
							var endTime = document.createElement("td");
							var endDate = null;
							if(job.endTime != null && job.endTime > 0){
								endDate = new Date(job.endTime);
								endTime.appendChild(document.createTextNode(jazz.util.date.formatDif(endDate, true, false)));
							}else{
								endTime.appendChild(document.createTextNode(this.messages.didNotFinish));
							}
							var totalTime = document.createElement("td");
							if(endDate != null){
								var timeTaken = endDate.getTime() - startDate.getTime();
								totalTime.appendChild(document.createTextNode(jazz.util.date.formatDuration(timeTaken)));
							}else{
								totalTime.appendChild(document.createTextNode(this.messages.na));
							}
							
							row.appendChild(status);
							row.appendChild(name);
							row.appendChild(startTime);
							row.appendChild(endTime);
							row.appendChild(totalTime);
							return row;
						},
						
						_createRunningJobRow: function(job){
							var row = document.createElement("tr");
							var name = document.createElement("td");
							name.appendChild(document.createTextNode(job.name));
							var startTime = document.createElement("td");
							var startDate = new Date(job.startTime);
							startTime.appendChild(document.createTextNode(jazz.util.date.formatDif(startDate, true, false)));
							var totalTime = document.createElement("td");
							var endDate = new Date(job.endTime);
							var timeTaken = endDate.getTime() - startDate.getTime();
							totalTime.appendChild(document.createTextNode(jazz.util.date.formatDuration(timeTaken)));
							
							row.appendChild(name);
							row.appendChild(startTime);
							row.appendChild(totalTime);
							return row;
						},

						_remoteError : function(e) {
							//possible to have two messages displaying depending on timing
							this._getWorkbench().hideMessageArea();
							this._getWorkbench().hideMessageArea();
							this._displayErrorMessage(this.messages.etlHistoryError);
						},
						
						_fetchRunningJobsError: function(){
							this._displayErrorMessage(this.messages.runningJobsError);
							if(this.runningJobList){
								while(this.runningJobList.hasChildNodes()){
									this.runningJobList.removeChild(this.runningJobList.firstChild);
								}
								this.runningJobList.appendChild(document.createTextNode(this.messages.noRunningJobs));
							}
						},
						
						_displaySuccessMessage : function(message, closeable) {
							this._displayMessage(message,
									jazz.ui.MessageBox.STATUS_TYPES.OK,
									closeable);
						},

						_displayInfoMessage : function(message, closeable) {
							this._displayMessage(message,
									jazz.ui.MessageBox.STATUS_TYPES.INFO,
									closeable);
						},

						_displayMessage : function(message, messageType,
								closeable) {
							if (!closeable) {
								closeable = false;
							}
							if (this.header) {
								var status = {
									type : messageType,
									summary : message,
									closeable : closeable
								};
								this.header.showMessage(status);
							}
						},

					    _getWorkbench : function(){
					        return net.jazz.ajax.ui.PlatformUI.getWorkbench();
					    }
					});
	EtlStatus.CSH_DATA_COLLECTION_JOB_STATUS = "com.ibm.jazz.reports.web.cshelp.data_collection_job_status";

})();
