/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.FilteredSelectionDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.reports.web.ui.internal.ListBox");
dojo.require("com.ibm.team.reports.web.ui.internal.SelectionDialog");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

(function(){

/**
 * A dialog box that allows the user to select from a list of values, using a filter.  Values for the selection
 * list are fetched when the user enters the filter.
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.FilteredSelectionDialog", [com.ibm.team.reports.web.ui.internal.SelectionDialog], {

	_filterSection: null,
	
	//mixin
	setFilter: null, //function.  Takes a filter string as parameter; to be called when the user sets a filter

	postCreate: function() {
		this._filterSection = new com.ibm.team.reports.web.ui.internal._FilterSection({
			onFilter : dojo.hitch(this, this.onFilter),
			messages:this.messages			
		});
		this.topSection.appendChild(this._filterSection.domNode);
		this.inherited(arguments);
		
		//for this kind of dialog, don't show "loading" until the user has actually entered a filter
		this._listBox.showLoading(false);
	},
	
	/**
	 * Called when the filter button is selected.  Does nothing if no filter text was entered.
	 */
	onFilter: function() {
		var filterString = this._filterSection.filterText.value;
		
		if (filterString.length > 0) {
			this._resetList();
			this._listBox.showLoading(true);
			this.setFilter(filterString);
		}
	},
	
});	

dojo.declare("com.ibm.team.reports.web.ui.internal._FilterSection", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/FilteredSelectionDialog.html")
	
});
	
})();
