/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.FolderEditor");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");
dojo.require("com.ibm.team.reports.web.ui.internal.ResourceDialog");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.app.context");
dojo.require("jazz.util.html");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var REPORTS_URI_PREFIX = "#action=com.ibm.team.reports.viewAllReports&shared=";
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var MAX_MESSAGE_LENGTH = 500;

/**
 * Widget to create a new report query
 */
var FolderEditor = dojo.declare("com.ibm.team.reports.web.ui.internal.FolderEditor", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/FolderEditor.html"),
		_csPageHelpWidgets: null,
		csHelpNode: null,
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			this.dirty = false;
			this._shared = args.shared;
			this._csPageHelpWidgets= [];
		},
				
		postCreate: function() {
			this._folderUUID = this.folderUUID;
			this.typeIcon.src = ImagePool.FOLDER;
			
			// The folder. We may not have one initially.
			this._folder = null;
			
			this._editableResources = [];
			if (this.folderUUID) {
				var responseHandler = new ServiceResponseHandler(
						this,
						"_setFolder",
						"_remoteError");
				ReportsClient.getFolderDescriptor(responseHandler, {folderUUID: this.folderUUID});
			} else {
				this._setEmptyFolder();
			}
			
			var self = this;
			this.connect(this._processAreaAnchor, "onclick", this._handleProcessAreaClick);
			this._createContextSensitiveHelp();
			
			if (!this.navigateAwayHandle) {
				this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
			}
			if (!this.onBeforeUnloadHandle) {
				this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleOnBeforeUnload), /*priority*/ 10000);
			}
		},
		
		_createContextSensitiveHelp: function() {
			dojo.empty(this.csHelpNode);
			this._createCSHWidget(FolderEditor.CSH_FOLDER_EDITOR);
		},
	
		_createCSHWidget: function(uaKey) {
			var parms = {};
			parms.uaKey = uaKey;
			var pageHelp = new PageHoverHelp(parms);
			this.csHelpNode.appendChild(pageHelp.domNode);
			this._csPageHelpWidgets.push(pageHelp);
		},
		
		/**
		 * Callback function to populate the page with the given folder.
		 */
		_setFolder: function(folder) {
			this._folder = folder;
			
			// Set the widgets up
			this._nameInput.value = folder.name;
			if (folder.description) {
				this._descriptionText.value = folder.description;
			}
			if (folder.processAreaUUID) {
				// Make a request for the process area name, which we'll fill in later
				var responseHandler = new ServiceResponseHandler(
					this,
					"_setProcessArea",
					"_remoteError");
				// Try team area first; if that fails look for project area
				var args = { uuid: folder.processAreaUUID };
				ReportsClient.getProcessAreaByUUID(responseHandler, args);
			}
			
			// If the folder has a parent, we can't change the sharing checkbox
			// nor the process area. Disable them now.
			if (folder.parentUUID) {
				this._sharedCheckbox.checked = true;
				this._sharedCheckbox.disabled = true;
			}
			
			this._sharedCheckbox.checked = folder.shared;
		},
			
		/**
		 * Populate the page with no initial folder.
		 */
		_setEmptyFolder: function() {
			this._folder = new Object();
			// Check if a processArea was supplied
			if (this.processAreaUUID) {
				this._folder.processAreaUUID = this.processAreaUUID;
			} else {
				// Assume we want the project area for sharing
				var currentProjectArea = jazz.app.context.get();
				this._folder.processAreaUUID = currentProjectArea.itemId;
			}
			if (this._folder.processAreaUUID) {
				// Make a request for the process area name, which we'll fill in later
				var responseHandler = new ServiceResponseHandler(
					this,
					"_setProcessArea",
					"_remoteError");
				var args = { uuid: this._folder.processAreaUUID };
				ReportsClient.getProcessAreaByUUID(responseHandler, args);
			}
			// Check if sharing was specified
			if (this.shared == true) {
				this._sharedCheckbox.checked = true;
				this._sharedCheckbox.disabled = true;
			} else {
				this._sharedCheckbox.checked = false;
				this._sharedCheckbox.disabled = true;
			}
			// Check if a parent folder was specified
			if (this.parentUUID) {
				this._folder.parentUUID = this.parentUUID;
			}
		},
					
		_setProcessArea: function(result) {
			if (this.processAreaUUID || this._folder.parentUUID) {
				this._processAreaParent.innerHTML = result.name;
			} else {
				this._processAreaAnchor.innerHTML = result.name;
			}
		},
		
		_handleSave: function() {
	        var args = {
		        name: this._getName()
	        }
	        if (this.folderUUID) {
	        	args.folderUUID = this.folderUUID;
	        }
	        if (this._folder.parentUUID) {
	        	args.parentUUID = this._folder.parentUUID;
	        }
	        var description = this._getDescription();
	        if (description && description.length > 0) {
	        	args.description = description;
	        }
	        args.shared = this._getShared();
	        args.processAreaUUID = this._folder.processAreaUUID;
	        
	        var responseHandler = new ServiceResponseHandler(
					this,
					"_saveComplete",
					"_remoteError");
			ReportsClient.postSaveFolderDescriptor(responseHandler, args);
		},
		
		destroy: function() {
			if (this.navigateAwayHandle) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				this.navigateAwayHandle = null;
			}
			if (this.onBeforeUnloadHandle) {
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				this.onBeforeUnloadHandle = null;
			}	
		},
		
		_handleCancel: function() {
			if (this.dirty) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
					this.destroy();
					document.location.href = REPORTS_URI_PREFIX + this.shared;
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.destroy();
				document.location.href = REPORTS_URI_PREFIX + this.shared;
				return false;
			}
		},
		
		_handleNavigateAway: function(url) {
			var currentLocation = document.location.href;
			var badLink = currentLocation.substring(0, currentLocation.indexOf("#")+1);
			if (url == badLink) {
				return true;
			}
			
			if (this.dirty) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
					this.destroy();
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.destroy();
				return false;
			}
		},
		
		_handleOnBeforeUnload: function() {
			return (this.dirty) ? this.messages.unsavedChanges + "\n\n" : false;
		},
		
		_getName: function() {
			return this._nameInput.value;
		},
		
		_getDescription: function() {
			return this._descriptionText.value;
		},
		
		_getShared: function() {
			return this._sharedCheckbox.checked;
		},
				
		_saveComplete: function(args) {
			this.folderUUID = args;
			this.destroy();
			document.location.href = "#action=com.ibm.team.reports.editFolder&folderUUID=" + args;
			var responseHandler = new ServiceResponseHandler(
					this,
					"_setFolder",
					"_remoteError");
			ReportsClient.getFolderDescriptor(responseHandler, {folderUUID: this.folderUUID});
			this._setDirty(false);
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this.setHeaderMessage({
				message: error.message,
				showOnlyMessage: true,
				severity: 8
			});
		},
		
		_handleNameChange: function() {
			this._setDirty(true);
		},
		
		_handleIdChange: function() {
			this._setDirty(true);
		},
		
		_handleCheckboxChange: function() {
			this._setDirty(true);
		},
		
		_handleDescriptionChange: function() {
			this._setDirty(true);
		},
		
		_handleProcessAreaClick: function(e) {
			e.preventDefault();

			var self = this;
			var params = {
				projects: true,
				teams: true,
				allowsMultiple: false,
				selection: self._folder.processAreaUUID,
				filterProjectArea: self._folder.projectAreaUUID,
				onOk: function(selectedItem, action) {
					self._onProcessOk(selectedItem);
				}
			};
			new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);
		},
		
		_onProcessOk: function(item) {
			this._folder.processAreaUUID = item.itemId;
			this._processAreaAnchor.innerHTML = item.name;
			this._setDirty(true);
		},
				
		_updateEnablement: function() {
			var name = this._getName();
			var hasName = (name && (name.length > 0));
			this._saveButton.disabled = (!this.dirty || !hasName);
		},
		
		_setDirty: function(isDirty) {
			if (isDirty === this.dirty) {
				this._updateEnablement();
				return;
			}
		
			if (isDirty) {
				this.folderTitle.innerHTML = this.messages.folderHeading + "*";
				this.folderTitle.style.color = "#A83300";
			} else {
				this.folderTitle.innerHTML = this.messages.folderHeading;
				this.folderTitle.style.color = "#000000";
			}
	
			this.dirty = isDirty;
			this._updateEnablement();
		},
				
		setHeaderMessage: function(message) {
			this.clearValidationMessage();
			if (!message) {
				return;
			}
			
			if (!message.message || message.message.length == 0) {
				message.message = this.messages.errorUnhelpful;
			} else {
				if(message.message.lastIndexOf(".") != (message.message.length-1)) {
					message.message += ".";
				}
			}
			
            if (!message.details) {
                message.details = "";
            } else {
                if (message.details.lastIndexOf(".") != (message.details.length - 1)) {
                    message.details += ".";
                }
            }
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            } else {
            	if (message.showOnlyMessage == true) {
            		messageText = message.message
            	} else {
                	messageText = message.message + " " + message.details;            		
            	}
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this.validationMessage.__jazzMessage = message;
            if (message.severity==8) {
				this.typeIcon.src = ImagePool.ERROR;
                this.validationMessageSpan.className = "validationMessageError";
            } else {
				if (message.severity==4) {
					this.typeIcon.src = ImagePool.WARNING;
					this.validationMessageSpan.className = "validationMessageWarning";
				} else {
					this.typeIcon.src = ImagePool.INFO;
					this.validationMessageSpan.className = "validationMessageInfo";
				}
            }
            this.validationMessage.title = message.details || message.message;
            dojox.data.dom.replaceChildren(this.validationMessage, document.createTextNode(messageText));
			window.scrollTo(0,0); // show message
		},
		
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if (!e || !e.target || !e.target.__jazzMessage) return;

			var message = e.target.__jazzMessage;
			var text = message.message;
			if (message.details) {
				text += "\n\n";
				text += message.details;
			}
			alert(text);
		},
		
		clearValidationMessage: function(force) {
			this.validationMessageSpan.className = "validationMessageEmpty";
		}		
	}
);

FolderEditor.CSH_FOLDER_EDITOR = "com.ibm.jazz.reports.web.cshelp.folder_editor";

})();