/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.FolderWidget");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.ui.internal.QueryWidget");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;

var TWISTIE_OPEN   = ImagePool.BLU_TWISTIE_OPEN;
var TWISTIE_CLOSED = ImagePool.BLU_TWISTIE_CLOSED;
var DROPDOWN_HOVER = ImagePool.DD_ARROW_HOV;
var DROPDOWN       = ImagePool.DD_ARROW;
var FOLDER         = ImagePool.FOLDER;
var EDIT           = ImagePool.EDIT;

/**
 * Widget to display a single folder, with its contents including subfolders.
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.FolderWidget", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/FolderWidget.html"),
		
		NEW_FOLDER_ICON_CLASS: "newReportFolderIcon",
		DELETE_ICON_CLASS: "deleteReportIcon",
		EDIT_ICON_CLASS: "editFolderIcon",
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		},

		postCreate: function(){
			this._dropDownImage.src = DROPDOWN;
			this._folderImage.src = FOLDER;
			this._folder = this.folder;
			this._expand = this.expand;
			this._parentWidget = this.parentWidget;
			this._controller = this.controller;
			this._shared = this.shared;
			this._canDeployQuery = this.canDeployQuery;
			this._canManageFolder = this.canManageFolder;
			this._defaultQuery = this.defaultQuery;
			this._childFolders = [];
			this._childQueries = [];
			this._hasChildQueries = true;
			this._hasChildFolders = true;
			this._fetchChildren();
			
			// Insert the folder name
			this._folderLinkTd.appendChild(document.createTextNode(this.folder.name));
			
			// Expand or collapse as appropriate
			if (this._expand == true) {
				this._runAction("maximize");
			} else {
				this._runAction("minimize");
			}
			
			// Hook the event for the triangle twisty
			this.connect(this._togglerLink, "onclick", this._togglerLinkClickHandler);
			
			if (!dojo.isIE) {
				var itemRowDivTopLeft = document.createElement("span");
				itemRowDivTopLeft.className = "itemRowDivTopLeft";
				var itemRowDivTopRight = document.createElement("span");
				itemRowDivTopRight.className = "itemRowDivTopRight";
				this._treeItem.insertBefore(itemRowDivTopRight, this._treeItem.firstChild);
				this._treeItem.insertBefore(itemRowDivTopLeft, this._treeItem.firstChild);
			}

			// Hide the drop-down indicator initially
			this._dropDown.style.visibility = "hidden";

			// Hook the events for the drop-down indicator
			if (this._canManageFolder) {
				this.connect(this._dropDownImage, "onmouseover", this._dropDownImageMouseOverHandler);
				this.connect(this._dropDownImage, "onmouseout", this._dropDownImageMouseOutHandler);
				this.connect(this._dropDown, "onclick", this._dropDownClickHandler);
			}
			
			if (!dojo.isIE) {
				var itemRowDivBottomLeft = document.createElement("span");
				itemRowDivBottomLeft.className = "itemRowDivBottomLeft";
				var itemRowDivBottomRight = document.createElement("span");
				itemRowDivBottomRight.className = "itemRowDivBottomRight";
				this._treeItem.appendChild(itemRowDivBottomLeft);
				this._treeItem.appendChild(itemRowDivBottomRight);
			}			
			
			// Blue hover support for the whole item
			this.connect(this._treeItem, "onmouseover", this._treeItemMouseOverHandler);
			this.connect(this._treeItem, "onmouseout", this._treeItemMouseOutHandler);
		},
		
		setSelection: function(queryUUID) {
			var selected = false;
			for (var i = 0; i < this._childQueries.length; i++) {
				var queryWidget = this._childQueries[i];
				if (queryWidget.query.queryUUID == queryUUID) {
					queryWidgetToSelect = queryWidget;
					queryWidget.setSelected(true);
					selected = true;
				} else {
					queryWidget.setSelected(false);
				}
			}
			if (!selected) {
				for (var i = 0; i < this._childFolders.length; i++) {
					var folderWidget = this._childFolders[i];
					if (folderWidget.setSelection(queryUUID)) {
						selected = true;
					}
				}
			}
			if (selected) {
				this._runAction("maximize");
			} else {
				//this._runAction("minimize");
			}
			return selected;
		},
		
		setDefaultQuery: function(defaultQuery) {
			this._defaultQuery = defaultQuery;
			for (var i = 0; i < this._childQueries.length; i++) {
				this._childQueries[i].setDefaultQuery(defaultQuery);
			}
			for (var i = 0; i < this._childFolders.length; i++) {
				this._childFolders[i].setDefaultQuery(defaultQuery);
			}
		},
		
		_togglerLinkClickHandler: function(e) {
			e.preventDefault();
			this._togglerClicked(e);
		},
		
		_dropDownImageMouseOverHandler: function(e) {
			e.preventDefault();
			this._dropDownImage.src = DROPDOWN_HOVER;
		},
		
		_dropDownImageMouseOutHandler: function(e) {
			e.preventDefault();
			this._dropDownImage.src = DROPDOWN;
		},
		
		_dropDownClickHandler: function(e) {
			e.preventDefault();
			this._openMenu(e);
		},
		
		_treeItemMouseOverHandler: function(e) {
			e.preventDefault();
			if (this._menu != null) {
				// We're already in the menu, ignore this
				this._mouseOutOccurred = false;
				return;
			}
			this._treeItem.className = "treeItemSelected";
			if (this._canManageFolder) {
				this._dropDown.style.visibility = "visible";
				this._dropDown.style.display = "inline";
			}
		},
		
		_treeItemMouseOutHandler: function(e) {
			e.preventDefault();
			if (this._menu != null) {
				// We're in the menu, don't remove the hover
				// but remember that we left
				this._mouseOutOccurred = true;
				return;
			}
			this._treeItem.className = "treeItem"
			if (this._canManageFolder) {
				this._dropDown.style.visibility = "hidden";
			}
		},
		
		removeChild: function(child) {
			for (var i = 0; i < this._childFolders.length; i++) {
				if (this._childFolders[i] == child) {
					this._childFolders.splice(i, 1);
				}
			}
			for (var i = 0; i < this._childQueries.length; i++) {
				if (this._childQueries[i] == child) {
					this._childQueries.splice(i, 1);
				}
			}
			child.domNode.parentNode.removeChild(child.domNode);
			child.destroy();			
		},
		
		/**
		 * Is this used any more?
		 */
		_refresh: function() {
			this._childQueries = [];
			this._childFolders = [];
			while (this.queriesDiv.childNodes[0] != null) {
				this.queriesDiv.removeChild(this.queriesDiv.childNodes[0]);
			}
			while (this.foldersDiv.childNodes[0] != null) {
				this.foldersDiv.removeChild(this.foldersDiv.childNodes[0]);
			}
			this._hasChildQueries = true;
			this._hasChildFolders = true;
			this._toggler.style.visibility = "visible";
			this._fetchChildren();
		},

		_addQuery: function(query) {
			var queryWidget = new com.ibm.team.reports.web.ui.internal.QueryWidget({query: query, parentWidget: this, controller: this._controller, canDeployQuery: this._canDeployQuery, defaultQuery: this._defaultQuery});
			this.queriesDiv.appendChild(queryWidget.domNode);
			this._childQueries[this._childQueries.length] = queryWidget;
		},
		
		_addFolder: function(folder) {
			var folderWidget = new com.ibm.team.reports.web.ui.internal.FolderWidget({folder: folder, parentWidget: this, shared: this._shared, expand: false, controller: this._controller, canDeployQuery: this._canDeployQuery, canManageFolder: this._canManageFolder, defaultQuery: this._defaultQuery});
			this.foldersDiv.appendChild(folderWidget.domNode);
			this._childFolders[this._childFolders.length] = folderWidget;
		},
		
		_fetchChildren: function() {
			var args = {
				// Project Area will be added by ReportsClient
				// Owner is intentionally empty
        		parentUUID: this._folder.folderUUID,
        		flags: ReportsClient.ALL
        	};
			var responseHandler = new ServiceResponseHandler(
					this,
					"_addChildFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);
			var args2 = {
				// Project Area will be added by ReportsClient
				// Owner is intentionally empty
        		folderUUID: this._folder.folderUUID,
        		flags: ReportsClient.EXCLUDE_MICRO
        	};
			var responseHandler2 = new ServiceResponseHandler(
					this,
					"_addChildQueries",
					"_remoteError");
			ReportsClient.getQueryDescriptors(responseHandler2, args2);
		},
		
		/**
		 * Helper function to sort a list of named items
		 * Note: Assume that names are unique
		 */
		_sort: function(items) {
			var names = [];
			for (var i = 0; i < items.length; i++) {
				names[i] = items[i].name;
			}
			names = names.sort();
			var result = [];
			for (var i = 0; i < names.length; i++) {
				result[i] = this._find(names[i], items);
			}
			return result;
		},
		
		_find: function(name, items) {
			for (var i = 0; i < items.length; i++) {
				if (items[i].name == name) {
					return items[i];
				}
			}
			return null;
		},		
		
		_addChildQueries: function(args) {
			var hasChildren = false;
			if (args != null) {
				// Sort them first.
				var sorted = this._sort(args);
				for (var i = 0; i < sorted.length; i++) {
					hasChildren = true;
					this._addQuery(sorted[i]);
				}
			}
			this._hasChildQueries = hasChildren;
			if (hasChildren == false && this._hasChildFolders == false) {
				this._hideTwistie();
			}
		},
		
		_addChildFolders: function(args) {
			var hasChildren = false;
			if (args != null) {
				// Sort them first.
				var sorted = this._sort(args);
				for (var i = 0; i < sorted.length; i++) {
					hasChildren = true;
					this._addFolder(sorted[i]);
				}
			}
			this._hasChildFolders = hasChildren;
			if (hasChildren == false && this._hasChildQueries == false) {
				this._hideTwistie();
			}
		},
		
		_hideTwistie: function() {
			this._toggler.style.visibility = "hidden";
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._itemDiv, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			alert(error.message);
		},
		
		_runAction: function(action) {
			if (action == "minimize") {
				this._setToggler(TWISTIE_CLOSED, "/maximize");
		        this.foldersDiv.style.display = "none";
		        this.queriesDiv.style.display = "none";
				if (this._parentWidget && this._parentWidget._runAction && typeof this._parentWidget._runAction == 'function') {
					this._parentWidget._runAction("minimize");
				}
			} else if (action == "maximize") {
				this._setToggler(TWISTIE_OPEN, "/minimize");
		        this.foldersDiv.style.display = "";
		        this.queriesDiv.style.display = "";
				if (this._parentWidget && this._parentWidget._runAction && typeof this._parentWidget._runAction == 'function') {
					this._parentWidget._runAction("maximize");
				}
			}
		},
		
		_setToggler: function(currentStateImg, actionUri) {
			this._toggler.src = currentStateImg;
			this._togglerLink.href = actionUri;
		},
		
		_togglerClicked: function(e) {
			// extract the action we're going to take from the link's href
			// (something like "minimize", "maximize", etc.)
			var node = this._togglerLink;
			var action = node.href.slice(node.href.lastIndexOf("/") + 1);
			this._runAction(action);
		},
		
		_openMenu: function(e) {
			if (this._menu) {
				this._closeMenu();
				return;
			}
			var menu = new dijit.Menu();
			menu.addChild(new dijit.MenuItem({label: this.messages.newFolderDotDotDot, iconClass: this.NEW_FOLDER_ICON_CLASS, onClick: dojo.hitch(this,this._newFolder)}));
			menu.addChild(new dijit.MenuSeparator());
			menu.addChild(new dijit.MenuItem({label: this.messages.edit, iconClass: this.EDIT_ICON_CLASS, onClick: dojo.hitch(this,this._edit)}));
			menu.addChild(new dijit.MenuItem({label: this.messages.deleteMsg, iconClass: this.DELETE_ICON_CLASS, onClick: dojo.hitch(this,this._delete)}));
			var coords = dojo.coords(e.target, true);
			dijit.popup.open({ popup: menu, parent: this, x: coords.x + 10, y: coords.y + 10, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
			//register event handler that causes menu to close
			this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
			this._menu = menu;
			menu.focus();			
		},
		
		_closeMenu: function(e){
			if (e) {
				dojo.stopEvent(e);
			}
			if (this._mouseOutOccurred) {
				this._treeItem.className = "treeItem";
				this._dropDown.style.visibility = "hidden";
			}
			this.disconnect(this._menuCloseOnBody);
			dijit.popup.close(this._menu);
			this._menu = null;
		},
		
		_edit: function() {
			document.location.href = "#action=com.ibm.team.reports.editFolder&folderUUID=" + this._folder.folderUUID;
		},
			
		_delete: function() {
			if (!confirm(this.messages.confirmDelFolder)) {
				return;
			}
			var args = {
        		folderUUID: this._folder.folderUUID
        	};
			var responseHandler = new ServiceResponseHandler(
					this,
					"_folderDeleted",
					"_remoteError");
			ReportsClient.postDeleteFolderDescriptor(responseHandler, args);
		},
		
		_newFolder: function() {
			document.location.href = "#action=com.ibm.team.reports.editFolder&parentUUID=" + this._folder.folderUUID + "&processAreaUUID=" + this._folder.processAreaUUID + "&shared=" + this._shared;
		},
		
		_folderDeleted: function() {
			if (this._parentWidget != null) {
				this._parentWidget.removeChild(this);
			}
		},
				
		_trim: function(string) {
			return this._trimLeft(this._trimRight(string));
		},
		_trimLeft: function(string) {
			return string.replace(new RegExp("^[\\s]+", "g"), "");
		},
		_trimRight: function(string) {
			return string.replace(new RegExp("[\\s]+$", "g"), "");
		}
	}
);

})();