/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.MetaNav");

dojo.require("dijit.InlineEditBox");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.xml");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");

dojo.require("jazz.app.context");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.app.events");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.TwoStateButton");
dojo.require("jazz.ui.toolbar.Separator");
dojo.require("jazz.util.html");
dojo.require("jazz.util.i18n");

dojo.require("net.jazz.ajax.BootstrapProperties");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

//
// Aliases
//
var BootstrapProperties = net.jazz.ajax.BootstrapProperties;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardInitializationData = com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData;
var HTML = jazz.util.html;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var xml = com.ibm.team.dashboard.web.ui.internal.util.xml;

var messages;

/**
 * The widget for the content area (right side) of the dashboard.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.MetaNav", [dijit._Widget, dijit._Templated, Observable], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/MetaNav.html"),
	widgetsInTemplate: true,
	dashboardContainer: null,
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		//initialize publish API for showing/hiding the message area (info, error, warning)
		this.subscribe(Constants.serverError, "_showErrorMessage");
		this.subscribe(Constants.hideServerError, "hideMessage");
	},
	init: function(args) {		
		if (this.isInitialized) {
			return;
		}
		
		//set dashboardBundle and setup other useful pointers
		this.dashboardBundle = args.dashboardBundle;
		this.dashboardBinding = this.dashboardBundle.dashboard.binding;
		this.allowDelete = args.allowDelete;
		this.allowSharing = args.allowSharing;
		this.allowSetDefault = args.allowSetDefault;
		
		this._dashboardTitle.innerHTML = "";
		dojo.removeClass(this._dashboardTitle, "loading-title");
		
		this.setEditMode(args.editMode);
		
		var perms = this.dashboardBundle.permissions;
		if (args.showBreadcrumbs && (perms.viewTeam || perms.viewPersonal))
			this._addBreadcrumbs();
		else {
			this._breadcrumbs.domNode.style.display = "none";
		}
		
		// show a notice if scope item is archived
		if (this.dashboardBundle.isScopeArchived) {
			var msg;
			var scope = this.dashboardBundle.scope;
			switch(scope) {
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					msg = messages.archivedContributor;
					break;
				case DashboardConstants.SCOPE_TEAM_AREA:
					msg = messages.archivedTeam;
					break;
				case DashboardConstants.SCOPE_PROJECT_AREA:
					msg = messages.archivedProject;
					break;
			}
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: msg,
				closable: true
			});
		}
		
		this._helpHover = new jazz.app.ua.PageHoverHelp({
			"class": "dashboard-hover-help", 
			uaKey: jazz.app.context.get().name ? 'com.ibm.team.dashboard.web.dash_proj_team' : 'com.ibm.team.dashboard.web.dash_personal'
		}).placeAt(this._dashboardTitle.parentNode, 'last');
		this._toolbar.domNode.style.display="";
		this._toolbar.startup();
	},
	_showErrorMessage: function(errorObj) {
		this.dashboardContainer.errorState = true;
		var args = {
			type: errorObj.type ? errorObj.type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			closable: errorObj.closeable
		};
		if (errorObj.status == 409 || errorObj.status == 412) {
			var isContributorScope = (this.dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR);
			var msg = errorObj.customMessage;
			// both summary and/or details can have refresh links
			var refreshLinkId = "com-ibm-team-dashboard-web-ui-internal-MetaNav-refreshLink";
			var refreshLinkCount = 0;
			if (msg) {
				msg = _formatRefreshLink(msg, refreshLinkId + (refreshLinkCount++));
			}
			else {
				msg = isContributorScope ? messages.errorConflictSelf : messages.errorConflict;
			}
			args.summary = msg;
			
			// details
			var buf = [isContributorScope ? messages.errorConflictSelf_explanation : messages.errorConflict_explanation];
			buf.push(" ");
			buf.push(_formatRefreshLink(messages.errorConflict_userAction, refreshLinkId + (refreshLinkCount++)));
			args.details = [{message: buf.join(""), code: jazz.ui.MessageBox.STATUS_TYPES.OK}];
			this.showMessage(args);
			for (var i=0;i<refreshLinkCount;++i) {
				var a = dojo.byId(refreshLinkId + i);
				this.connect(a, "onclick", dojo.hitch(this.dashboardContainer, this.dashboardContainer.refresh));
			}
		}
		else if (errorObj.status == 403) {
			args.summary = messages.errorNoPermissionSave;
			args.details = [{message: errorObj.responseText}];
			this.showMessage(args);
		}
		else if (errorObj.status == 503) {
			args.summary = messages.errorSaveReadOnly;
			args.details = [{message: errorObj.responseText}];
			this.showMessage(args);
		}
		else if (errorObj.status > 0) {
			var buf = ["HTTP/1.1 ", errorObj.status];
			if (errorObj.statusText) {
				buf.push(" ");
				buf.push(errorObj.statusText);
			}
			args.summary = dojo.string.substitute(messages.errorSave, [buf.join("")]);
			var responseText = errorObj.responseText;
			if (responseText && responseText.length > 0) {
				responseText = responseText.replace(/\r?\n/g, "<br/>");
				responseText = responseText.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
				args.details = [{message: responseText}];
			}
			args.errorObject = errorObj;
			this.showMessage(args);
		}
		else {
			args.errorObject = errorObj;
			this.showMessage(args);
		}
	},
	_addBreadcrumbs: function() {
		var breadcrumbs = [];
		var defaults = this.dashboardBundle.defaults,
			context = jazz.app.context.get();
		if (context.name) {
			breadcrumbs.push({label: dojo.string.substitute(messages.allProjectDash,[context.name]), href: Util.getAllProjectDashboardsHref(context.name)});			
		} else {
			breadcrumbs.push({label: messages.allPersonalDashboards, href: Util.getAllPersonalDashboardsHref()});
		}

		if (this.dashboardBundle.scope == DashboardConstants.SCOPE_TEAM_AREA) {
			breadcrumbs.push({label: defaults.project.name,	href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, defaults.project.name)});
			var path = defaults.team.path;
			var curPath = [];
			for (var i=0;i<path.length-1;i++) {
				curPath.push(path[i]);
				breadcrumbs.push({
					label: path[i],
					href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, curPath)
				});
			}
		}
		
		this._breadcrumbs.segments = breadcrumbs;
		this._breadcrumbs.refresh();
	},
	destroy: function() {
		if (this._title) {
			this._title.destroy();
			delete this._title;
		}
		if (this._helpHover) {
			this._helpHover.destroy();
			delete this._helpHover;
		}
		this.inherited(arguments);
	},	
	setEditMode: function(editMode) {
		var beforeMode = this.editMode;
		this.editMode = editMode;
		if (editMode != beforeMode) {
			if (editMode == Constants.EDIT_AUTO_SAVE || editMode == Constants.EDIT_DEFERRED_SAVE) {
				if (!this._title) {
					//create editable dashboard title
					var titleStr = this.dashboardBinding.getTitle();
					var node = document.createElement("span");
					node.appendChild(document.createTextNode(titleStr));
					this._dashboardTitle.appendChild(node);
					// value param is workaround for:
					// http://bugs.dojotoolkit.org/ticket/10653
			        this._title = new dijit.InlineEditBox({title:messages.clickToEdit, autoSave: true, width: "400px", noValueIndicator: "?", editorParams: {maxLength: "60"}, onChange: dojo.hitch(this, this._onTitleChange), value: titleStr}, node);
			        this._title.startup();					
				}
				this._toolbar.addChild(new jazz.ui.toolbar.Separator(), 1);
				this.showSaveControls(true);
				this._setAutoSave();
			}
			else {
				if (this._title) {
					this._title.destroy();
					delete this._title;
				}
				this._dashboardTitle.appendChild(document.createTextNode(this.dashboardBinding.getTitle()));
				this.showSaveControls(false);
				if (typeof(this.beforeMode) != "undefined" && this.beforeMode != Constants.READ_ONLY) 
					this.dashboardContainer.setDirty(false);
			}
		}
	},
	showSaveControls: function (showControls){
		if(!this._editEvents) {
			this._editEvents = new Array();
		}
		this._autoSaveArea.style.display = showControls ? "" : "none";
		if (showControls) {
			if (typeof this.editMode != "undefined" && this.editMode == Constants.EDIT_AUTO_SAVE) {
				this._saveButton.style.display = "none";
			}
		} else {
			while(this._editEvents && this._editEvents.length > 0) {
				this.disconnect(this._editEvents.pop());	
			}
		}
		var dashboardBundle = this.dashboardBundle;
		if (!this._separator && (dashboardBundle.permissions["modify"] || dashboardBundle.permissions["modify"]))
			this._separator = this._toolbar.addChild(new jazz.ui.toolbar.Separator(), 0);
		if (this.allowSetDefault && !this._defaultButton && dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR && dashboardBundle.permissions["modify"]) {
			var isDefault = dashboardBundle.isDefaultDashboard;
			this._defaultButton = new jazz.ui.toolbar.Button({onClick: dojo.hitch(this, this._setDefaultDashboard), imgClass: "default-dash", title: messages[isDefault ? "defaultDisabled" : "default"]});
			this._toolbar.addChild(this._defaultButton,0);
			if (isDefault) {
				this._defaultButton.set("enabled", false);
			}
		}
		if (this.allowSharing && !this._shareButton && dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR && dashboardBundle.permissions["modify"]) {
			var resource = this.dashboardContainer.dashboardResource;
			var contributorUri = DashboardInitializationData.getAuthenticatedContributorUri();
			var isShared = resource.securityContext != contributorUri;
			this._shareButton = new jazz.ui.toolbar.TwoStateButton({pressed: isShared, onClick: dojo.hitch(this, this._shareDashboard), imgClass: "shared-dash", imgClassPressed: "unshared-dash", title: messages.sharing},0);
			this._toolbar.addChild(this._shareButton,0); 
		}
		if (!this._pasteButton && dashboardBundle.permissions["modify"]) {
			this._pasteButton = new jazz.ui.toolbar.Button({
				onClick: dojo.hitch(this, this._paste),
				imgClass: "paste-button",
				title: messages.paste
			});
			this._toolbar.addChild(this._pasteButton, 0);
		}
		if (this.allowDelete && !this._deleteButton && dashboardBundle.scope == DashboardConstants.SCOPE_CONTRIBUTOR && dashboardBundle.permissions["delete"]) {
			this._deleteButton = new jazz.ui.toolbar.Button({onClick: dojo.hitch(this, this._deleteDashboard), imgClass: "delete-button", title: messages.del});
			this._toolbar.addChild(this._deleteButton, 0);
		}
		if (!this._restoreButton && dashboardBundle.scope != DashboardConstants.SCOPE_CONTRIBUTOR && this.dashboardBundle.permissions["modify"] && DashboardInitializationData.getAllowRestore()) {
			this._restoreButton = new jazz.ui.toolbar.Button({onClick: dojo.hitch(this, this._restoreDashboard), imgClass: "restore-button", title: messages.restoreDefault});
			this._toolbar.addChild(this._restoreButton, 0);
		}
		// internal options
		var showInternalOptions = (location.search.indexOf("internal") !== -1);
		if (showInternalOptions) {
			if (!this._historyButton) {
				this._historyButton = new jazz.ui.toolbar.Button({onClick: dojo.hitch(this, this._showHistory), imgClass: "history-button", title: "Show History"});
				this._toolbar.addChild(this._historyButton, 0);
			}
		}
	},
	_refresh: function() {
		this.dashboardContainer.refresh();
	},
	_paste: function(event) {
		if (event) 
			dojo.stopEvent(event);
		try {
			var data = localStorage["net-jazz-ajax-clipboard"];
		} catch(e) {
			alert(messages.featureNotSupported);
			return;
		}
		if (!data) {
			alert(messages.clipboardEmpty);
			return;
		}
		var document = xml.parse(data);
		var elem = document.documentElement;
		var ownerDocument = this.dashboardBinding.elem.ownerDocument;
		if (ownerDocument.importNode) {
			elem = ownerDocument.importNode(elem, true);
		}
		if (elem.nodeName.indexOf("Page") != -1) {
			this.dashboardContainer.addPage(elem);
		} else {
			this.dashboardContainer.currentPage.addWidgetFromElement(elem);
		}
	},
	_setAutoSave: function(event) {
		var dc = this.dashboardContainer;
		if((event && event.target.checked) || (!event && dc.autoSave)) {
			if (event) {
				dc.setEditMode(Constants.EDIT_AUTO_SAVE, true);
				this.editMode = Constants.EDIT_AUTO_SAVE;
			}
			var wasDirty = dc.isDirty();
			this._saveButton.style.display = "none";
			dc.setDirty(false);
			dc.autoSave = true;
			this._autoSave.checked = "checked";
			if (wasDirty) {
				this.propertyChanged(Constants.PROPERTY_DASHBOARD);
			}
		} else {
			if (event) {
				dc.setEditMode(Constants.EDIT_DEFERRED_SAVE, true);
				this.editMode = Constants.EDIT_DEFERRED_SAVE;
			}
			this._saveButton.style.display = "";
			dc.autoSave = false;
		}
	},
	_setDefaultDashboard: function() {
		if (confirm(messages.defaultConfirm)) {
			var dashboardUrl = this.dashboardBundle.dashboardUrl;
			// keep the flag in the dashboard bundle up to date, so we can check it for navigate away logic
			this.dashboardBundle.isDefaultDashboard = true;
			var headers = {};
			headers["Content-Type"] = "text/plain";
			jazz.client.xhrPut({
				url: Util.getServiceRoot() + "/dashboard-user-prefs/default-dashboard",
				headers: headers,
				putData: dashboardUrl,
				error: dojo.hitch(this, this._showErrorMessage)
			});
			this._defaultButton.set("enabled", false);
			this._defaultButton.set("title", messages.defaultDisabled);
			dojo.publish(jazz.app.events.HOME_MENU_REFRESH);
			dojo.publish("/jazz/dashboard/setDefault", [dashboardUrl]);
		}
	},
	_saveEditingHandler: function(event){
		if (event) 
			dojo.stopEvent(event);
		this.dashboardContainer.saveDashboard();
		this.dashboardContainer.setDirty(false);
		if (this.editMode == Constants.EDIT_DEFERRED_SAVE) {
			this.dashboardContainer.propertyChanged("title", this.dashboardBinding.getTitle());
		}
	},
	_shareDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var isShared = !this._shareButton.get("pressed");
		var dashboardContainer = this.dashboardContainer;
		if (this.editMode == Constants.EDIT_DEFERRED_SAVE && dashboardContainer.isDirty()) {
			alert(messages.alertShareDirty);
			// undo the toggle
			this._shareButton.set("pressed", isShared);
			return;
		}
		var msg = isShared ? messages.confirmUnshare : messages.confirmShare;
		if (confirm(msg)) {
			var newSecurityContext = isShared ? DashboardInitializationData.getAuthenticatedContributorUri() : Constants.CONTEXT_PUBLIC;
			dashboardContainer.dashboardResource.securityContext = newSecurityContext;
			if (this.editMode == Constants.EDIT_AUTO_SAVE) {
				this.propertyChanged(Constants.PROPERTY_DASHBOARD);
				if (!isShared) {
					this._autoSave.checked = false;
					this._setAutoSave({target: this._autoSave});
				}
			} else if (this.editMode == Constants.EDIT_DEFERRED_SAVE) {
				if (!isShared) {
					dashboardContainer.setEditMode(Constants.EDIT_AUTO_SAVE, true);
					this.propertyChanged(Constants.PROPERTY_DASHBOARD);
					dashboardContainer.setEditMode(Constants.EDIT_DEFERRED_SAVE, true);
				} else {
					this._autoSave.checked = "checked";
					this._setAutoSave({ target: this._autoSave });
					this.propertyChanged(Constants.PROPERTY_DASHBOARD);
				}
			}
			// broadcast change
			dashboardContainer.dashboardWidget.propertyChanged("shared", !isShared);
		} else {
			// undo the toggle
			this._shareButton.set("pressed", isShared);
		}
	},
	_showHistory: function(event) {
		if (event) 
			dojo.stopEvent(event);
		// compose history service URL
		var discoveryUrl = BootstrapProperties.DISCOVERYSERVICE_URL;
		var jtsUrl = discoveryUrl.substring(0, discoveryUrl.lastIndexOf("/"));
		var dashboardUrl = this.dashboardBundle.dashboardUrl;
		var dashboardId = dashboardUrl.substring(dashboardUrl.lastIndexOf("/") + 1);
		var index = dashboardId.indexOf("?");
		if (index !== -1) {
			dashboardId = dashboardId.substring(0, index);
		}
		// url param specifies the external URL of the dashboard; to be used in the feed
		// strip out the ?internal param required to show this hidden option
		var url = location.href;
		var urlObj = new dojo._Url(url);
		var query = urlObj.query;
		var queryObj = dojo.queryToObject(query);
		delete queryObj.internal;
		urlObj.query = dojo.objectToQuery(queryObj) || null;
		
		// reconstruct url
		var buf = [urlObj.scheme, "://", urlObj.host];
		urlObj.port && buf.push(":" + urlObj.port);
		urlObj.path && buf.push(urlObj.path);
		urlObj.query && buf.push("?" + urlObj.query);
		urlObj.fragment && buf.push("#" + urlObj.fragment);
		url = buf.join("");
		// assemble history URL, send the user there
		var historyUrl = jtsUrl + "/dashboard-history/" + dashboardId + "?url=" + encodeURIComponent(url);
		location.href = historyUrl;
	},
	_deleteDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var message = dojo.string.substitute(messages.delConfirmPrivate, [this.dashboardBinding.getTitle() || ""]);
		if (!confirm(message))
			return;
		this.dashboardContainer.propertyChanged("ondelete");	
	},
	_restoreDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var message = dojo.string.substitute(messages.restoreDashboard);
		if (!confirm(message))
			return;
		this.dashboardContainer.restoreDashboard();
	},
	/*
	 * Called when the user changes the dashboard title.
	 */
	_onTitleChange: function(/* string */ newTitle) {
		this.dashboardBinding.setTitle(newTitle);
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
		if (this.editMode != Constants.EDIT_DEFERRED_SAVE) {
			this.dashboardContainer.propertyChanged("title", newTitle);	
		}
	},
	showMessage: function(args) {
		this._messageBox.showMessage(args);
	},
	hideMessage: function(/* boolean */ fadeAnimation) {
		this._messageBox.hideMessage(fadeAnimation);
		this.dashboardContainer.errorState = false;
	},
	_onChange: function(/* string */ property) {
		// notify listeners that the content has changed
		this.propertyChanged(property);
	},
	setDirty: function (isDirty) {
		if (isDirty && !this.dashboardContainer.autoSave) {
			if (!this.dirtyIcon) {
				this.dirtyIcon = dojo.create("span", {
					innerHTML: "*", 
					"class": "dirtyStar", 
					title: messages.dashboardModified
				}, this._dashboardTitle);
			}
			this._saveButton.disabled = "";
		} else {
			if (this.dirtyIcon) {
				this._dashboardTitle.removeChild(this.dirtyIcon);
				delete this.dirtyIcon;
			}
			this._saveButton.disabled = "disabled";
		}
	}
});

/*
 * Formats a message which may or may not contain a <0>Refresh</0> link embedded.
 * If it does, it will expand the link into HTML with the given id.
 */
function _formatRefreshLink(/* string */ msg, /* string */ id) {
	var buf = [];
	var parts = jazz.util.i18n.split(msg);
	var len = parts.length;
	for (var i=0;i<len;++i) {
		var part = parts[i];
		if (part.id == 0) {
			buf.push("<a href=\"#\" id=\"" + id + "\">");
			buf.push(part.text);
			buf.push("</a>");
		} else {
			buf.push(part.text);
		}
	}
	return buf.join("");
}
})();