/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.MonitoredTaskProgressWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date");
dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");
dojo.require("dojo.number");
dojo.require("dojo.string");
dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "MonitoredTaskProgressWidgetMessages");

var MessageBox = jazz.ui.MessageBox;
var messages;

dojo.declare("com.ibm.team.repository.web.ui.internal.MonitoredTaskProgressWidget", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/MonitoredTaskProgressWidget.html"),
	
	_disableMessageBox: false,
	
	_successImgSrc: dojo.moduleUrl("com.ibm.team.repository.web.ui.internal", "images/ProgressWidget/checkmark.png"),
	_failureImgSrc: dojo.moduleUrl("com.ibm.team.repository.web.ui.internal", "images/ProgressWidget/error_obj.gif"),
	_pendingImgSrc: dojo.moduleUrl("com.ibm.team.repository.web.ui.internal", "images/ProgressWidget/arrow.png"),
	_blankImgSrc: 	dojo.moduleUrl("com.ibm.team.repository.web.ui.internal", "images/ProgressWidget/blank.gif"),
	_progressImgSrc: dojo.moduleUrl("com.ibm.team.repository.web.ui.internal", "images/ProgressWidget/progress.gif"),
	
	_currentStep: null,
	
	/* Job Monitor */
	monitor: null,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "MonitoredTaskProgressWidgetMessages");
	},
	
	postCreate: function() {
		if(!this.monitor) {
			throw new Error("A job monitor must be provided");
		}
		this.monitor.addListener(dojo.hitch(this, this._updateStatus));
		
		dojo.addClass(this.domNode, "com-ibm-team-repository-web-MonitoredTaskProgressWidget");
	
		if(this._disableMessageBox === false) {
			this.msgBox = new MessageBox();
			this.messageBoxArea.appendChild(this.msgBox.domNode);
			this._inProgressMessage = this.inProgressMessage || this.messages.inProgressMessage;
			this._failureMessage = this.failureMessage || this.messages.failureMessage;
			this._successMessage = this.successMessage || this.messages.successMessage;
			
			this.msgBox.showMessage({
				summary: this.messages.loadingMessage,
				type: MessageBox.STATUS_TYPES.INFO
			});
		}
	},
	
	_updateStatus: function(job) {
		if(this._beingDestroyed || !this.isJobUpdated(job)) {
			return;
		}
		
		if (this.msgBox) {
			var summaryText, messageType;
			if (job.state === "FINISHED") {
				if (job.resolution === "SUCCESS") {
					summaryText = this._successMessage;
					messageType = MessageBox.STATUS_TYPES.OK;
				} else {
					summaryText = this._failureMessage;
					messageType = MessageBox.STATUS_TYPES.ERROR;
				}
			} else {
				var percentComplete = this.getProgressPercentage(job.steps);
				summaryText = this._inProgressMessage + " " + dojo.string.substitute(this.messages.percentCompleteMessage, [percentComplete]);
				messageType = MessageBox.STATUS_TYPES.INFO;
			}
			this.msgBox.showMessage({
				summary: summaryText,
				type: messageType,
				closable: job.state === "FINISHED"
			});
		}
		
		var content = dojo.create("div");
		this.stepListDOM = dojo.create("ul", {}, content);
		this.processSteps(job.steps);
		dojo.place(content, this.taskProgressArea, "only");
	},
	
	processSteps: function(stepList, level) {
		if(!level) var level = 0; //keep track of our depth as we navigate the various levels of nested steps
		this.currentIsSet = false;
		this.isError = false;
		for (var i = 0; i < stepList.length; i++) {
			var step = stepList[i];
			var stepListElement = dojo.create("li", {
				className: "step "
			}, this.stepListDOM);
			dojo.addClass(stepListElement, "level" + level);
			var stepIconSrc = this._blankImgSrc;
			if (step.state === "FINISHED") {
				if (step.resolution === "SUCCESS") {
					stepIconSrc = this._successImgSrc;
				} else {
					stepIconSrc = this._failureImgSrc;
					this.isError = true;
				}
			}
			else {
				if (!this.currentIsSet && !this.isError) {
					stepIconSrc = this._pendingImgSrc;
					this.currentIsSet = true;
					dojo.addClass(stepListElement, "current");
				} else {
					dojo.addClass(stepListElement, "future");
				}
			}
			var stepIcon = dojo.create("img", {
				src: stepIconSrc.uri,
				height: 16,
				width: 16
			}, stepListElement);
			if (step.resultMessage) {
				stepIcon.title = step.resultMessage;
			}
			stepListElement.appendChild(document.createTextNode(step.name));
			
			if (dojo.hasClass(stepListElement, "current")) {
				dojo.create("img", {
					className: "progressImg",
					src: this._progressImgSrc.uri,
					height: 16,
					width: 16
				}, stepListElement);
			}
			
			//Indent each nested step by 25px
			stepListElement.style.marginLeft = 25 * level + "px";

			if (step.subtasks) {
				var percentComplete = dojo.string.substitute(this.messages.percentCompleteMessage, [ this.getProgressPercentage(step.subtasks) ]);
				dojo.create("span", { className: "progressText", innerHTML: percentComplete }, stepListElement);
				level++;
				this.processSteps(step.subtasks, level);
				level--;
			}
		}
	},

	getProgressPercentage: function(stepList){
		var numSteps = stepList.length;
		if(numSteps <= 0) return 0;
		var completedSteps = 0;
		for (var i = 0; i < numSteps; i++) {
			if (stepList[i].state === "FINISHED") {
				completedSteps++;
			}
		}
		var percentComplete = (completedSteps / numSteps) * 100;
		return dojo.number.round(percentComplete);
	},
	
	isJobUpdated: function(job){
		var jobLength = job.steps.length
		
		if(jobLength <= 0) {
			return false;
		}
		
		if (job.steps[jobLength - 1].state === "FINISHED" || (job.state === "FINISHED")) {
			//the final step is finished, no need to do anything else
			return true;
		}
		
		var currentStep = this._currentStep;
		if (!currentStep){
			//initialize the first step of the job
			this._currentStep = dojo.date.stamp.fromISOString(job.steps[0].start);
			return true;
		}
		
		this._getLatestStartTime(job.steps);
		if(currentStep != this._currentStep) {
			return true;
		} 
		
		return false;
	},
	
	_getLatestStartTime: function(stepList) {
		for(var i=0; i < stepList.length; i++) {
			var stepStartTime =  dojo.date.stamp.fromISOString(stepList[i].start);
			if(!stepStartTime) break;
			if (dojo.date.compare(stepStartTime, this._currentStep) > 0){
				//the job has updated, update the current step
				this._currentStep = stepStartTime;
				break;
			} else if(stepList[i].subtasks) {
				//Still on the same step, but maybe this step has substeps
				this._getLatestStartTime(stepList[i].subtasks);
			}
		}
	},
	
	destroy: function() {
		this.inherited(arguments);
		this._beingDestroyed = true;
	},
});
