/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.PersonalDashboardAjaxModule");

dojo.require("com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData");
dojo.require("com.ibm.team.dashboard.web.ui.internal.PersonalDashboard");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.ErrorPage");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var DashboardInitializationData = com.ibm.team.dashboard.web.ui.internal.DashboardInitializationData;
var PersonalDashboard = com.ibm.team.dashboard.web.ui.internal.PersonalDashboard;
var PersonalDashboardClient = com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var APPLICATION_ID = "com.ibm.team.dashboard.personal";

var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.PersonalDashboardAjaxModule",
		[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-PersonalDashboardAjaxModule\"></div>",
	
	constructor: function() {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	postCreate: function() {
		// dashboard requires a workbench; create one
		PlatformUI.runWorkbench(APPLICATION_ID);
		
		// load dashboard init data
		var dfd = DashboardInitializationData.load();
		dfd.addCallback(dojo.hitch(this, function() {
			// to support com.ibm.team.repository.web.client.session API where no
			// initialization data is loaded (this is an ajax module)
			var contributor = DashboardInitializationData.getAuthenticatedContributor();
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", contributor);
			
			// process request
			var path = location.pathname;
			if (path.search(/\/dashboards\/?$/) != -1) {
				// no dashboard specified; show default
				this._viewDefaultPersonalDashboard();
			}
			else {
				// derive a dashboard resource URL from the current URL
				// (current URL minus query and hash)
				var href = location.href;
				var index = href.indexOf("?");
				if (index === -1) {
					index = href.indexOf("#") || href.length;
				}
				this._viewPersonalDashboard(href.substring(0, index));
			}
		}));
		dfd.addErrback(dojo.hitch(this, function(/* Error */ e) {
			this._showErrorPage(messages.errorRetrieving, e);
		}));
	},
	
	uninitialize: function() {
		if (this._dashboard) {
			this._dashboard.destroyRecursive();
			delete this._dashboard;
		}
		if (this._errorPage) {
			this._errorPage.destroyRecursive();
			delete this._errorPage;
		}
	},
	
	_showErrorPage: function(/* string */ summary, /* Error */ e) {
		var errorPage = this._errorPage = jazz.ui.ErrorPage({
			summary: summary,
			details: e.message || (e.errorObj ? e.errorObj.message : null)
		});
		this.domNode.appendChild(errorPage.domNode);
	},
	
	_viewDefaultPersonalDashboard: function() {
		var deferred = PersonalDashboardClient.get();
		deferred.addCallback(dojo.hitch(this, function(info) {
			if (info && info.personalDashboards && info.personalDashboards[0]) {
				location.href = Util.addUIParams(info.personalDashboards[0].url);
			}
			else {
				// no dashboards; create one automatically
				var dfd = PersonalDashboardClient.createDashboard();
				dfd.addCallback(dojo.hitch(this, function(/* string */ url) {
					location.href = Util.addUIParams(url);
				}));
				dfd.addErrback(this, function(/* Error */ e) {
					this._showErrorPage(messages.errorRetrieving, e);
				});
			}
		}));
		deferred.addErrback(dojo.hitch(this, function(e) {
			this._showErrorPage(messages.errorRetrieving, e);
		}));
	},
	
	_viewPersonalDashboard: function(/* string */ url) {
		this._dashboard = new PersonalDashboard({
			allowDelete: false,
			allowSetDefault: false,
			allowSharing: false,
			showBreadcrumbs: false,
			url: url
		});
		this.domNode.appendChild(this._dashboard.domNode);
	}
});

})();
