/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportsComponentMenu");

dojo.require("jazz.app._ComponentMenu");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("jazz.util.html");
dojo.require("jazz.util.format");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var REPORTS_WELCOME_URI = "#action=com.ibm.team.reports.defaultAction";
var MY_REPORTS_URI = "#action=com.ibm.team.reports.viewAllReports&shared=false";
var SHARED_REPORTS_URI = "#action=com.ibm.team.reports.viewAllReports&shared=true";
var REPORT_TEMPLATES_URI = "#action=com.ibm.team.reports.viewAllReportTemplates";
var NEW_REPORT_URI = "#action=com.ibm.team.reports.newReport";
var ADDITIONAL_TASKS = "#action=com.ibm.team.reports.showAdditionalTasks";

dojo.declare("com.ibm.team.reports.web.ui.internal.ReportsComponentMenu", jazz.app._ComponentMenu, {
	
	postCreate: function() {
		var srh = new ServiceResponseHandler(this, "_storeCustomReportingConfigured", "_handleError");
		ServerStatusClient.getServiceConfiguration(srh, "com.ibm.team.reports.service.cognos.internal.CognosReportService");
		
		this.addItemByArgs({
			label: messages.welcomeToReports,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/report-obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: REPORTS_WELCOME_URI
		});
		this.addTitle(messages.browse);
		this.addItemByArgs({
			label: messages.myReports,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/reports-obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: MY_REPORTS_URI
		});
		this.addItemByArgs({
			label: messages.sharedReports,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/reports-obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: SHARED_REPORTS_URI
		});
		this.addItemByArgs({
			label: messages.reportTemplates,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/report-template-group_obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: REPORT_TEMPLATES_URI
		});
		
		var div = document.createElement("div");
		this.archivedCheckbox = document.createElement("input");
		this.archivedCheckbox.type = "checkbox";
		this.archivedCheckbox.onclick = dojo.hitch(this, this._onCheckBoxClick);
		this.archivedCheckbox.style.marginRight = "7px";
		
		div.appendChild(this.archivedCheckbox);
		var span = document.createElement("span");
		span.className = "com-ibm-team-reports-archivedCheckbox";
		span.appendChild(document.createTextNode(messages.includeArchived));
		div.style.color = "#000000";
		div.appendChild(span);
		this.add(div);
		
		this.addTitle(messages.create);
		this.addItemByArgs({
			label: messages.createReportFromTemplate,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/create-report_obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: NEW_REPORT_URI
		});
		
		srh = new ServiceResponseHandler(this, "_handleShowArchived", "_handleError");
		ReportsClient.getShowArchived(srh);
	},
	
	_onCheckBoxClick: function() {
		dojo.publish("/jazz/reports/includeArchived", [this.archivedCheckbox.checked]);
	},
	
	// Called by the super class. It will put a black dot in front of the
	// returned item to denote it as "selected"
	getSelected: function() {
		var items = this.getItems();
		var href = location.href;
		if (href.search(REPORTS_WELCOME_URI) != -1) {
			return items[0];
		} else if (href.search(MY_REPORTS_URI) != -1) {
			return items[2];
		} else if (href.search(SHARED_REPORTS_URI) != -1) {
			return items[3];
		} else if (href.search(REPORT_TEMPLATES_URI) != -1) {
			return items[4];
		} else if (href.search(NEW_REPORT_URI) != -1) {
			return items[7];
		} else {
			return null;
		}
	},
	
	_storeCustomReportingConfigured: function(response) {
		for(var i = 0; i < response.properties.length; i++){
			if(response.properties[i].name == "cognos.hostAndPort"){
				if(response.properties[i].overrideValue && response.properties[i].overrideValue.length > 0){
					this.addTitle(messages.customizeReport);
					var customReportingUrl = response.properties[i].overrideValue;
					this.addItemByArgs({
						label: messages.performAdditional,
						icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/report-obj.gif"),
						wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
						//href: "#action=com.ibm.team.reports.authorCustomReport&customReportingUrl=" + customReportingUrl
						href: ADDITIONAL_TASKS
					});
					return;
				}
			}
		}
	},
	
	_handleShowArchived: function(response) {
		this.archivedCheckbox.checked = (response == "true");
	},
	
	_handleError: function() {
	}
});
})();