/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportsController");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("net.jazz.ajax.ui.PlatformUI");

(function() { // module container

var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;

var session	= com.ibm.team.repository.web.client.session;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var includeArchived = false;

dojo.subscribe("/jazz/reports/includeArchived", null,
	function(include) { includeArchived = include; });

/**
 * Main application controller for Reports perspective.
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.ReportsController", null, {

	/**
	 * Internal constructor. To be called only from ReportsView.
	 */
	constructor: function(reportsView) {
		this._view = reportsView;
		this._teamAreas = [];
		this._selectedQuery = null;
		this._selectedAdminLink = null;
		
		this._lastReportArgs = null;
		this._lastQueryArgs = null;
		
		this._defaultQuery = null;
	},
	
	/**
	 * Called by Jazz web framework to provide default view when the URL
	 * contains no 'action' request parameter.
	 */
	init: function() {
	},
	
	getSafeUrlActions: function() {
		return ["viewQuery", "viewReport", "getSharedQueries", "admin", 
		"newReport", "editTemplate", "editCustomTemplate", "deployNewTemplates", "editFolder", "editReport"];
	},

	// -------------- action API methods ----------------------------------------------------------
	
	/**
	 * Display the welcome page
	 */
	defaultAction: function() {
		this._view.showWelcome();
	},
	
	/**
	 * Display additional Tasks
	 */
	showAdditionalTasks: function() {
		this._view.showAdditionalTasks();
	},
	
	/**
	 * View a report query.
	 */
	viewQuery: function(args) {
		checkNotNull(args.queryUUID, "Query id");
		this._selectedQuery = args.queryUUID;
		this._selectedAdminLink = null;
		this._view.showQuery(args);
		this._lastQueryArgs = args;
		this._lastReportArgs = null;
	},
	
	/**
	 * View a report.
	 */
	viewReport: function(args) {
		checkNotNull(args.reportUUID, "Report id");
		this._selectedQuery = null;
		this._selectedAdminLink = null;
		this._view.showReport(args);
		this._lastQueryArgs = null;
		this._lastReportArgs = args;
	},
	
	/**
	 * Administer the data warehouse.
	 */
	admin: function(args) {
		this._view.admin(args);
	},
	
	/**
	 * Return the UUID of the selected query or
	 * null if no query is selected.
	 */
	getSelectedQuery: function() {
		return this._selectedQuery;
	},
	
	/**
	 * Return the selected admin link
	 * or null if no admin link is selected.
	 */
	getSelectedAdminLink: function() {
		return this._selectedAdminLink;
	},
	
	viewWorkItems: function(args) {
		this._view.viewWorkItems(args);
	},

	/**
	 * Return whether archived things should be
	 * included in reports
	 */
	getIncludeArchived: function() {
		return includeArchived;
	},
	
	/**
	 * Set whether archived things should be
	 * included in reports
	 */
	setIncludeArchived: function(include) {
		includeArchived = include;
	},
	
	setDefaultQuery: function(defaultQuery) {
		this._defaultQuery = defaultQuery;
	},
	
	getDefaultQuery: function() {
		return this._defaultQuery;
	},
	
	/**
	 * Marks the start of a long-running operation (needs more documentation).
	 * May be called by code within the com.ibm.team.workitem.web bundle.
	 */
	startOperation: function(message) {
		PlatformUI.getWorkbench().showMessage(message);
	},

	/**
	 * Marks the end of a long-running operation (needs more documentation).
	 * May be called by code within the com.ibm.team.workitem.web bundle.
	 */
	finishOperation: function(operation) {
		PlatformUI.getWorkbench().hideMessageArea();
	},
	
	/**
	 * Reload the currently-displayed query or report.
	 */
	reload: function() {
		if (this._lastQueryArgs == null && this._lastReportArgs == null) {
			// Nothing to reload
			return;
		}
		var messageString;
		if (includeArchived) {
			messageString = "Do you wish to reload the current report to show archived artifacts?";
		} else {
			messageString = "Do you wish to reload the current report to hide archived artifacts?";
		}
		var answer = confirm(messageString);
		if (answer) {
			if (this._lastQueryArgs != null) {
				this._view.showQuery(this._lastQueryArgs);
			} else if (this._lastReportArgs != null) {
				this._view.showReport(this._lastReportArgs);
			}
		}
	},
	
	newReport: function(args) {
		this._view.newReport(args);
	},
	
	editTemplate: function(args) {
		this._view.editTemplate(args);
	},
	
	editCustomTemplate: function(args) {
		this._view.editCustomTemplate(args);
	},

	editFolder: function(args) {
		this._view.editFolder(args);
	},

	editReport: function(args) {
		this._view.editReport(args);
	},
	
	deployNewTemplates: function(args) {
		this._view.deployNewTemplates(args);
	},
	
	// --------------- internal helper methods -----------------------------------------------------
	
	_handleDefaultReport: function(result) {
		if (result != null) {
			this.viewQuery(result);
		} else {
			this._view.showWelcome();
		}
	},
	
	_remoteError: function(error) {
		alert(error.message);
	}
});

})();