/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.SelectableStyledTable"); //$NON-NLS-1$
dojo.require("dijit.MenuSeparator"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.ui.SimpleToolbar"); //$NON-NLS-1$
(function(){
	
	/**
	 * Extends the base StyledTable class and adds support for permanently selecting a row
	 * 
	 * @name com.ibm.team.process.web.ui.internal.SelectableStyledTable
	 * @constructor
	 * @extends jazz.ui.StyledTable
	 */ 
    dojo.declare("com.ibm.team.process.web.ui.internal.SelectableStyledTable", jazz.ui.StyledTable, { //$NON-NLS-1$
    	/**
    	 * What item position the list starts at, can be changed at initialization using this.start
    	 * To change after initialization, use this.changeTablePosition(start, length)
    	 * 
    	 * @private
    	 * @type Positive_Integers
    	 * @default 0
    	 */ 
		_start : 0,
		
		/**
    	 * How long the list should be, can be changed at initialization using this.length
    	 * To change after initialization, use this.changeTablePosition(start, length)
    	 * Note: Negative numbers denote infinite length
    	 * 
    	 * @private
    	 * @type Integers
    	 * @default -1
    	 */ 
		_length : -1,
		
		/**
    	 * An array which contains all the rows in the order they are displayed
    	 * 
    	 * @private
    	 * @type Array
    	 */ 
		_rowsByOrder: null,
		
		/**
    	 * An object which stores all the rows in a map by their IDs
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_rowsById : null,
		
		/**
    	 * An object which stores all the selected rows in a map by their IDs
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_rowsBySelected : null,
		
		/**
    	 * An Integer which counts the size of _rowsBySelected
    	 * 
    	 * @private
    	 * @type Positive_Integers
    	 */ 
		_selectedCount : 0,
		
		/**
    	 * An Object which contains the information of the clickable button next to the checkbox
    	 * imgSrc: The location of the image to use for the button
    	 * label: The label of the button that is seen on hover
    	 * 
    	 * @public
    	 * @type Object
    	 */ 
		button: null,
		
		/**
    	 * An Array of objects which contains the information of an element of the dropdown menu.
    	 * label: The hover over value
		 * title: The display value in the menu
		 * isForMany: Is this button for when many items are selected
		 * isForOne: Is this button for when only one item is selected
		 * isDivider: Determines if the current element is a Divider or is a field
    	 * 
    	 * @public
    	 * @type Array
    	 */ 
		menu: null,
		
		/**
    	 * An object of the actual dropdown menu
    	 * 
    	 * @private
    	 * @type Object
    	 */ 
		_menu: null,
		
		/**
    	 * An array which contains each object in the dropdown menu
    	 * 
    	 * @private
    	 * @type Array
    	 */ 
		_menuChildren: null,
		
		/**
    	 * If this variable is true, _updateTable will not perform any actions
		 * If false, _updateTable will function as intended
    	 * 
    	 * @private
    	 * @type Boolean
    	 */ 
		_suspendUpdate : false,
		
		/**
    	 * The checkbox in the header of the table
    	 * 
    	 * @private
    	 * @type Checkbox
    	 */ 
		_selectAllCheckBox : null,
		
		/**
    	 * The pop up menu for the checkbox in the header of the table
    	 * 
    	 * @private
    	 * @type Menu
    	 */
		_selectAllPopUpMenu : null,
		
		/**
    	 * The last row in which the action menu was clicked
    	 * 
    	 * @private
    	 * @type TR_Element
    	 */ 
		_lastFocusedRow : null,
		
		
		/**
    	 * Occurs before anything else is allowed to happen
		 * This is required in order to insert the checkbox column in the table.
    	 */ 
		preamble: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._selectAllCheckBox = new jazz.ui.SimpleToolbar();//$NON-NLS-1$
       		
       		var button = {
       			imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/spacer.gif"), //$NON-NLS-1$  //$NON-NLS-2$
   				label: this.messages.SelectableStyledTable_SelectAllCheckBoxHover
		    };
			button = this._selectAllCheckBox.addButton(null, null, button);
			
			this._selectAllCheckBox.dropdownMenu = new jazz.ui.MenuPopup({
				align: "left", //$NON-NLS-1$
				menuProvider: dojo.hitch(this, function(e){return this._handleSelectAllPopupMenuAction(e);}) //$NON-NLS-1$
			}, button.domNode);
			
			this._setCheckBox(this._selectAllCheckBox, false);
			
			args.columns.unshift({
				label: this._selectAllCheckBox.domNode,
				className: "com-ibm-team-process-web-ui-internal-SelectableStyledTable-checkbox", //$NON-NLS-1$
				sortable: false
			});
		},
		
		/**
		 * @constructor
		 * @param {Object} args An object which contains multiple arguments
		 * @param {Object} args.button An object which contains the following
		 * @param {String} args.button.imgSrc The location of the image to use for the button
    	 * @param {String} args.button.label The label of the button that is seen on hover
		 * @param {Object[]} args.menu An array which contains objects setup in the following
		 * @param {String|Function}	args.menu[].label The hover over value (Can be a string or a function as follows)
		 *   parameters Object The row's callback data which the popup menu was opened on
		 *   returns String the string to be outputed
		 * @param {String|Function}	args.menu[].title The display value in the menu (Can be a string or a function as follows)
		 *   parameters Object The row's callback data which the popup menu was opened on
		 *   returns String the string to be outputed
		 * @param {Function} [args.menu[].isEnabled = true] Is a function which is called to see if the current item is currently enabled (default function always returns true)
		 *   returns Boolean (true means the item is enabled, false mean the item is disabled)
		 * @param {Boolean} [args.menu[].isDivider = false] Determines if the current element is a Divider or is a field (default is false)
		 * 
		 * See jazz.ui.StyledTable for all other input parameters
    	 */
		constructor: function(args){
			if(args.start != null){
				this._start = args.start;
			}
			if(args.length != null){
				this._length = args.length;
			}
			this._intializeVariables();
		},
		
		/**
    	 * A function responsible for setting up the objects with their initial values
    	 * 
    	 * @private
    	 */ 
		_intializeVariables: function(){
        	this._rowsById = {};
        	this._rowsByOrder = [];
        	this._rowsBySelected = {};
        	this._selectedCount = 0;
		},
		
		/**
    	 * A function responsible for setting up the initial table
    	 */ 
        postCreate: function(){       	
            this.inherited(arguments);
			dojo.addClass(this.tbody, "com-ibm-team-process-web-ui-internal-SelectableStyledTable"); //$NON-NLS-1$
			
			this.connect(this._selectAllCheckBox, "onclick", "_handleSelectAllCheckboxClick"); //$NON-NLS-1$ //$NON-NLS-2$
        },
        
        /**
    	 * An extendable function which is called before a row's selection is changed
    	 * This function allows the change to be cancelable
    	 * 
    	 * @public
    	 * @param {Object} row the callback data of the row that's selection is changing
    	 * @returns {Boolean} true allows the change to occur, false will not
    	 */ 
        onSelectionChange: function(row) { return true; },
        
        /**
    	 * Get all the rows from the table
    	 * 
    	 * @public
    	 * @returns {Array} The callback data of all the rows
    	 */ 
        getAllRows: function(){
        	var rows = [];
            var i;
            for (i = 0; i < this._rowsByOrder.length; i++) {
                rows.push(this._rowsByOrder[i].callbackData);
            }
            return rows;
        },
        
        /**
    	 * Get all the selected rows from the table
    	 * 
    	 * @public
    	 * @returns {Array} The callback data of all the selected rows
    	 */ 
        getAllSelectedRows: function(){
        	var selectedRows = [];
        	
        	if(this._lastFocusedRow != null && !this._lastFocusedRow.rowSelected){
        		selectedRows.push(this._lastFocusedRow.callbackData);
        	}
        	
            for (var property in this._rowsBySelected) {
            	if(this._rowsBySelected[property] != null){
            		selectedRows.push(this._rowsBySelected[property].callbackData);
            	}
            }
            return selectedRows;
        },
        
        /**
    	 * Get the number of rows in the table
    	 * 
    	 * @public
    	 * @returns {Positive_Integer} The number of rows
    	 */ 
        getCount: function(){
        	return this._rowsByOrder.length;
        },
        
        /**
    	 * Get the number of selected rows in the table
    	 * 
    	 * @public
    	 * @returns {Positive_Integer} The number of selected rows
    	 */ 
        getSelectedCount: function(){
       		if(this._lastFocusedRow != null && !this._lastFocusedRow.rowSelected){ //$NON-NLS-1$
        		return this._selectedCount + 1;
        	}
        	return this._selectedCount;
        },
        
        /**
    	 * Determines if a specific row is selected
    	 * 
    	 * @public
    	 * @param {String} rowId The ID of the row in question
    	 * @returns {Boolean} True if the row is selected, false otherwise
    	 */ 
        isRowSelected: function(rowId){
        	var row = this._rowsById[rowId];
        	if(row == undefined){
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	return row.rowSelected;
        },
        
        /**
    	 * Determines if a specific row is selected
    	 * 
    	 * @public
    	 * @param {String} rowId The ID of the row in question
    	 * @param {String} className Name of class to check if the row has
    	 * @returns {Boolean} True if the row has class, false otherwise
    	 */ 
        hasClass: function(rowId, className){
        	var row = this._rowsById[rowId];
        	if(row == undefined){
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	
        	return this._hasClass(row, className);
        },
        
        /**
    	 * Determines if a specific row is selected
    	 * 
    	 * @private
    	 * @param {Object} row The row to check on
    	 * @param {String} className Name of class to check if the row has
    	 * @returns {Boolean} True if the row has class, false otherwise
    	 */ 
        _hasClass: function(row, className){
        	return dojo.hasClass(row, className);
        },
        
        /**
    	 * Selects all the rows in the table
    	 * 
    	 * @public
    	 * @param {Boolean} selection True selects all the rows, false deselects all the rows
    	 */ 
        selectAll: function(selection){
        	for(var i = 0; i < this._rowsByOrder.length; i++){
        		this._selectRow(this._rowsByOrder[i], selection);
        	}
        },
        
        /**
    	 * Selects a specific row in the table
    	 * 
    	 * @public
    	 * @param {String} rowId The ID of the row to change
    	 * @param {Boolean} selection True to select the row, false to deselect the row
    	 * @returns {Boolean} True if row was successfully changed or if the row's
    	 * value was already equal to selection, false if unable to change row's value
    	 * @throws {Error} When the row ID is not found
    	 */ 
        selectRow: function(rowId, selection){
        	var row = this._rowsById[rowId];
        	if(row == undefined){
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	return this._selectRow(row, selection);
        },
        
        /**
    	 * Selects a specific row in the table
    	 * 
    	 * @private
    	 * @param {Object} row The row to change
    	 * @param {Boolean} selection True to select the row, false to deselect the row
    	 * @returns {Boolean} True if row was successfully changed or if the row's
    	 * value was already equal to selection, false if unable to change row's value
    	 */ 
        _selectRow: function(row, selection){
        	if(selection == row.rowSelected){
        		return true;
        	}
        	if(this.onSelectionChange(row.callbackData)){
	        	if(selection && !row.rowSelected){
	        		this._rowsBySelected[row.id] = row;
	        		this._selectedCount++;
	        	}else if(!selection && row.rowSelected){
	        		delete this._rowsBySelected[row.id];
	        		this._selectedCount--;
	        	}
	        	
	        	dojo.toggleClass(row, "checked", selection); //$NON-NLS-1$
	        	row.rowSelected = selection;
	        	row.checkbox.checked = selection;
	        	
    			this._setCheckBox(this._selectAllCheckBox, 
    					this._selectedCount > 0 && this._selectedCount < this.getCount() 
	        			? null 
	        			: this._selectedCount !== 0);
	        	
	        	return true;
        	}
        	return false;
        },

        /**
    	 * Creates a row to be inserted into the table
    	 * 
    	 * @private
    	 * @param {Object} arguments An object which contains the following row information
    	 * @param {String} arguments.id a unique identifier for the row, user is responsible for making sure this id is unique.
    	 * @param {String} arguments.className the class for this row 
         * @param {Object} arguments.callbackData the variable which will be returned on call back
         * @param {Object[]} arguments.columnsData an array of all the columns
         * @param {Boolean} arguments.rowSelected determines if this row should be in the selected state
    	 * @returns {Object} the row to be inserted into the table
    	 * @throws {Error} ID already exists
    	 */ 
        _createRow: function(arguments){
        	//cannot create a row that already exists
        	if(this._rowsById[arguments.id] != null){
        		throw new Error(this.messages.SelectableStyledTable_ErrorRepeatID);
        	}
        	
           	//initialization of row
        	var row = document.createElement("tr"); //$NON-NLS-1$
        	if(arguments.className){
        		dojo.addClass(row, arguments.className); //$NON-NLS-1$
        	}
        	
    		row.id = arguments.id;
			row.callbackData = arguments.callbackData;
			row.columnsData = arguments.columnsData;
       		row.rowSelected = false;
       		
        	//initialization of the toolbar
       		row.toolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();//$NON-NLS-1$
       		
       		var button = {};
       		for (var property in this.button) {
            	button[property] = this.button[property];
            }
			button = row.toolbar.addButton(null, null, button);
			
			row.dropdownMenu = new jazz.ui.MenuPopup({
				align: "left", //$NON-NLS-1$
				menuProvider: dojo.hitch(this, function(){return this._handlePopupMenuAction(row);})
			}, button.domNode);
			
        	//initialization of the checkbox
        	row.checkbox = document.createElement("input"); //$NON-NLS-1$
        	row.checkbox.type = "checkbox"; //$NON-NLS-1$
        	this.connect(row.checkbox, "onclick", "_handleRowCheckboxClick"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(row.checkbox, "onfocus", "_handleFocus"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(row.checkbox, "onblur", "_handleBlur"); //$NON-NLS-1$ //$NON-NLS-2$
			
			//begin the creation of the checkbox column
			var td = document.createElement("td"); //$NON-NLS-1$
			dojo.addClass(td, "com-ibm-team-process-web-ui-internal-SelectableStyledTable-checkbox", true); //$NON-NLS-1$
			td.appendChild(row.checkbox);
			td.appendChild(row.toolbar.domNode);
			row.appendChild(td);
			
			//create the rest of the columns
			var i;
			for(i = 0; i < row.columnsData.length; i++){
				td = document.createElement("td"); //$NON-NLS-1$
				td.appendChild(row.columnsData[i]);
				row.appendChild(td);
			}
			
			this._selectRow(row, arguments.rowSelected === true);
			
			this.connect(row, "onmouseout", "_handleBlur"); //$NON-NLS-1$ //$NON-NLS-2$
			
			//return the current row
			return row;
        },
               
        /**
    	 * Creates a row to be inserted at the end of the table
    	 * 
    	 * @public
    	 * @param {Object} An object which contains the following row information
    	 * @param {String} arguments.id a unique identifier for the row, user is responsible for making sure this id is unique.
    	 * @param {String} arguments.className the class for this row 
         * @param {Object} arguments.callbackData the variable which will be returned on call back
         * @param {Object[]} arguments.columnsData an array of all the columns
         * @param {Boolean} arguments.rowSelected determines if this row should be in the selected state
    	 * @returns {String} the row's ID
    	 * @throws {Error} ID already exists
    	 */ 
        insertRowAtEnd: function(arguments){
        	//create the row
        	var row = this._createRow(arguments);
        	
			//append to the end of the table
        	row.position = this._rowsByOrder.push(row) - 1;
        	this._rowsById[row.id] = row;
        	
        	//update the layout of the table
        	this._updateTable();
        	
        	//return the row
        	return row.id;
        },
        
        /**
    	 * Deletes a row in the table
    	 * 
    	 * @public
    	 * @param {String} rowId The ID of the row to be deleted
    	 * @returns {Object} The callback information of the row
    	 * @throws {Error} ID does not exist
    	 */ 
        remove: function(rowId){
        	//find row to remove
        	var removeRow = this._rowsById[rowId];
        	if(removeRow == undefined){
        		throw new Error(this.messages.SelectableStyledTable_NoID);
        	}
        	
        	return this._remove(removeRow);
        },
        
        /**
    	 * Deletes a row in the table
    	 * 
    	 * @private
    	 * @param {Object} row The row to be deleted
    	 * @returns {Object} The callback information of the row
    	 */ 
        _remove: function(row){
        	//row the row from the IDs
        	this._rowsByOrder.splice(row.position, 1);
        	
        	if(this._lastFocusedRow != null && this._lastFocusedRow.id == row.id){
        		this._lastFocusedRow = null;
        	}
        	
        	if(row.rowSelected){
        		delete this._rowsBySelected[row.id];
        		this._selectedCount--;
        		
        		//reset the select all checkbox, because now none may be selected
        		this._setCheckBox(this._selectAllCheckBox, 
    					this._selectedCount > 0 && this._selectedCount < this.getCount() 
	        			? null 
	        			: this._selectedCount !== 0);
        	}
        	
        	//remove the row
        	delete this._rowsById[row.id];
        	
        	row.toolbar.destroyRecursive();
        	row.toolbar = null;
        	row.dropdownMenu.destroyRecursive();
        	row.dropdownMenu = null;
        	
        	//update the layout of the table
        	this._updateOrder(row.position);
        	this._updateTable();
        	
        	//return the arguments
        	return row.callbackData;
        },
        
        /**
    	 * Removes all the rows from the table
    	 * 
    	 * @public
    	 */ 
        clear: function(){
        	var previousUpdateValue = this._suspendUpdate;
        	this._suspendUpdate = true;
        	for(var i = 0; i < this._rowsByOrder.length; i++){
        		this._remove(this._rowsByOrder[i]);
        	}
        	
        	this._intializeVariables();
        	
        	this._suspendUpdate = previousUpdateValue;
        	this._updateTable();
        },
        
        /**
    	 * Determines if the table is empty
    	 * 
    	 * @public
    	 * @returns {Boolean} True if the table is empty, false if the table is not empty
    	 */ 
        empty: function(){
        	return this._rowsByOrder.length < 1;
        },
        
        /**
    	 * Updates the order of the table
         * (the .position elements attached to each row)
    	 * 
    	 * @private
    	 * @param {Positive_Integer} startPosition the position in which to start the update
    	 */ 
        _updateOrder: function(startPosition){
        	for(var i = startPosition; i < this._rowsByOrder.length; i++){
        		this._rowsByOrder[i].position = i;
        	}
        },
        
        /**
    	 * Changes the number of elements to be displayed and the
         * starting position of the elements to be displayed
    	 * 
    	 * @public
    	 * @param {Positive_Integer} startPosition the first element to be shown in the table
    	 * @param {Integer} length the number of elements to be shown in the table
    	 */ 
        changeTablePosition: function(startPosition, length){
        	this._start = startPosition;
        	this._length = length;
        	this._updateTable();
        },
        
        /**
    	 * This function allows an external person to prevent the table from
         * being updated until they are ready for the table to be updated
    	 * 
    	 * @public
    	 * @param {Boolean} suspendUpdate true will prevent the table from update its display elements
         * false will allow the table to update its display elements, and the table will
         * immediately update its elements when this function is called
    	 */ 
        suspendTableUpdate: function(suspendUpdate){
        	this._suspendUpdate = suspendUpdate;
        	this._updateTable();
        },
        
        /**
    	 * This function update what the table is currently displaying
    	 * 
    	 * @private
    	 */ 
        _updateTable: function(){
        	
        	if(!this._suspendUpdate){
	        	//clear all of tbody
	        	this.clearBody();
	        
	        	//update table contents
	        	if(this._length < 0){
	        		for(var i = this._start; i < this._rowsByOrder.length; i++){
	            		this.tbody.appendChild(this._rowsByOrder[i]);
	            	}
	        	} else {
	        		for(var i = this._start; i < this._length + this._start && i < this._rowsByOrder.length; i++){
	            		this.tbody.appendChild(this._rowsByOrder[i]);
	            	}
	        	}
        	}
        },

        /**
    	 * This function gets a row from an event
    	 * 
    	 * @private
    	 * @param {Object} The event data
    	 * @returns {Object} The row data or null if it could not find the row
    	 */ 
        _getRow: function(evt){
            var e = evt.target;
            while (e.parentNode) {
                if (e.tagName === "TR") { //$NON-NLS-1$
                    return e;
                }
                e = e.parentNode;
            }
            return null;
        },
        
        /**
    	 * Sets a checkbox to one of three states, checked, unchecked, inbetween
    	 * 
    	 * @private
    	 * @param {Object} checkBox the checkbox to be effected
    	 * @param {Nullable_Boolean} value the value to set the checkbox (null for inbetween)
    	 */ 
        _setCheckBox: function(checkBox, value){
			dojo.toggleClass(checkBox.domNode, "checked", value === true); //$NON-NLS-1$
			dojo.toggleClass(checkBox.domNode, "unchecked", value === false); //$NON-NLS-1$
			dojo.toggleClass(checkBox.domNode, "undetermined", value === null); //$NON-NLS-1$
			
			checkBox.checked = value !== false;
			checkBox.value = value === true ? "checked" : "undetermined"; //$NON-NLS-1$ //$NON-NLS-2$
        },
        
        /**
    	 * This function handles the dropdown menu for the select all checkbox
    	 * 
    	 * @private
    	 * @param {Object} event the event variable
    	 * @returns {Object} the menu to be displayed
    	 */ 
        _handleSelectAllPopupMenuAction: function(event){
        	if(this._selectAllPopUpMenu == null){
	        	this._selectAllPopUpMenu = new jazz.ui.Menu();
				
				//This is needed because the menu element is not part of the main table
				dojo.addClass(this._selectAllPopUpMenu, "com-ibm-team-process-web-ui-internal-SelectableStyledTable"); //$NON-NLS-1$
				
				this._selectAllPopUpMenu.addChild(new dijit.MenuItem({
					label: this.messages.SelectableStyledTable_SelectAllPopUpMenuSelectAll,
					title: this.messages.SelectableStyledTable_SelectAllPopUpMenuSelectAll,
					onClick: dojo.hitch(this, function(e){this.selectAll(true);})
				}));
				this._selectAllPopUpMenu.addChild(new dijit.MenuItem({
					label: this.messages.SelectableStyledTable_SelectAllPopUpMenuSelectNone,
					title: this.messages.SelectableStyledTable_SelectAllPopUpMenuSelectNone,
					onClick: dojo.hitch(this, function(e){this.selectAll(false);})
				}));
        	}
			
			return this._selectAllPopUpMenu;
        },
        
        /**
    	 * Handles a Popup Menu Close Event on each row's action menu
    	 * 
    	 * @private
    	 */ 
        _handlePopupMenuClose: function(){
        	if(this._lastFocusedRow != null){
        		dojo.toggleClass(this._lastFocusedRow, "menu-open", false); //$NON-NLS-1$
        	}
        },
        
        /**
    	 * This function handles the dropdown menu
    	 * 
    	 * @private
    	 * @param {Object} row the event variable
    	 * @returns {Object} the menu to be display on success, null on failure
    	 */ 
        _handlePopupMenuAction: function(row){
        	if(!this.onSelectionChange(row.callbackData)){
        		return null;
        	}
        	
        	//set the last focused row
        	this._lastFocusedRow = row;
        	dojo.toggleClass(row, "menu-open", true); //$NON-NLS-1$
        	
        	//create the menu items
        	if(this._menu == null){
				this._menu = new jazz.ui.Menu();
				this._menuChildren = [];
				this.connect(this._menu, "onClose", "_handlePopupMenuClose"); //$NON-NLS-1$ //$NON-NLS-2$
				
				//This is needed because the menu element is not part of the main table
				dojo.addClass(this._menu.domNode, "com-ibm-team-process-web-ui-internal-SelectableStyledTable"); //$NON-NLS-1$

				//populate the menu items
				var child;
				for(var i = 0; i < this.menu.length; i++){
					if(this.menu[i].isDivider === true){
						child = new dijit.MenuSeparator();
					} else {
						child = new dijit.MenuItem(this.menu[i]);
						
						//Check if label is a function, if not assume string
						if(this.menu[i].label instanceof Function){ //$NON-NLS-1$
							child.set('label', this.menu[i].label(row.callbackData)); //$NON-NLS-1$
						} else {
							child.set('label', dojo.string.substitute(this.menu[i].label, [this.getSelectedCount()])); //$NON-NLS-1$
						}
						
						//Check if title is a function, if not assume string
						if(this.menu[i].title instanceof Function){ //$NON-NLS-1$
							child.set('title', this.menu[i].title(row.callbackData)); //$NON-NLS-1$
						} else {
							child.set('title', dojo.string.substitute(this.menu[i].title, [this.getSelectedCount()])); //$NON-NLS-1$
						}
					}
					
					var isEnabled = this.menu[i].isEnabled === undefined ? true : this.menu[i].isEnabled(row.callbackData);
					child.set('disabled', !isEnabled); //$NON-NLS-1$
					
					this._menu.addChild(child);
					this._menuChildren.push(child);
				}
			} else {
				for(var i = 0; i < this.menu.length; i++){
					if(this.menu[i].isDivider !== true){
						//Check if label is a function, if not assume string
						if(this.menu[i].label instanceof Function){ //$NON-NLS-1$
							this._menuChildren[i].set('label', this.menu[i].label(row.callbackData)); //$NON-NLS-1$
						} else {
							this._menuChildren[i].set('label', dojo.string.substitute(this.menu[i].label, [this.getSelectedCount()])); //$NON-NLS-1$
						}
						
						//Check if title is a function, if not assume string
						if(this.menu[i].title instanceof Function){ //$NON-NLS-1$
							this._menuChildren[i].set('title', this.menu[i].title(row.callbackData)); //$NON-NLS-1$
						} else {
							this._menuChildren[i].set('title', dojo.string.substitute(this.menu[i].title, [this.getSelectedCount()])); //$NON-NLS-1$
						}
					}
					
					//if no isEnabled function, assume always true
					var isEnabled = this.menu[i].isEnabled === undefined ? true : this.menu[i].isEnabled(row.callbackData);
					this._menuChildren[i].set('disabled', !isEnabled); //$NON-NLS-1$
				}
			}
        	
			return this._menu;
        },
        
        /**
    	 * This function handles when the row obtains focus
    	 * 
    	 * @private
    	 * @param {Object} e the event variable
    	 */ 
        _handleFocus: function(e){
        	var row = this._getRow(e);
        	dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
        },
        
        /**
    	 * This function handles when the row looses focus
    	 * 
    	 * @private
    	 * @param {Object} e the event variable
    	 */ 
        _handleBlur: function(e){
        	var row = this._getRow(e);
        	dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
        },
        
        /**
    	 * This function handles when the checkbox is clicked
    	 * 
    	 * @private
    	 * @param {Object} e the event variable
    	 */ 
        _handleRowCheckboxClick: function(e){
        	var row = this._getRow(e);
        	this._selectRow(row, e.target.checked);
        },
        
        /**
    	 * This function handles when the select all checkbox is clicked
    	 * 
    	 * @private
    	 * @param {Object} e the event variable
    	 */ 
        _handleSelectAllCheckboxClick: function(e){
        	this.selectAll(e.target.checked);
        },
        
        /**
    	 * This function is responsible for destroying this object
    	 */ 
		uninitialize: function(){
			this.clear();
			
        	delete this._rowsById;
        	delete this._rowsByOrder;
        	delete this._rowsBySelected;
        	
        	if(this._lastFocusedRow != null){
        		delete this._lastFocusedRow;
        	}
        	
        	if(this._menu != null){
        		for(var i = 0; i < this._menuChildren.length; i++){
					this._menuChildren[i].destroyRecursive();
					delete this._menuChildren[i];
				}
        		delete this._menuChildren;
        		
        		this._menu.destroyRecursive();
        		delete this._menu;
        	}
        	
        	if(this._selectAllPopUpMenu != null){
        		this._selectAllPopUpMenu.destroyRecursive();
        		delete this._selectAllPopUpMenu;
        	}
        	
        	this._selectAllCheckBox.dropdownMenu.destroyRecursive();
        	this._selectAllCheckBox.destroyRecursive();
        	delete this._selectAllCheckBox;
        	
			this.inherited(arguments);
		}
    });
})();
