/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.team.repository.web.ui.internal.SystemAlertsWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.client.xhr");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.ui.HoverPopup");

var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;

dojo.declare("com.ibm.team.repository.web.ui.internal.SystemAlertsWidget", [dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-repository-web-SystemAlerts\" dojoattachevent=\"onmousemove: _pause, onmouseout: _play\"><span class=\"alerts\" dojoattachpoint=\"_alerts\"></span></div>",
	messages: null,
	rotateTime: 5000,
	_alertWidgets: null,
	_licenseInfo: null,
	_currentIndex: 0,
	
	constructor: function(args) {
		if(args && args.messages) {
			this.messages = args.messages;
		} else {
			this.messages = [];
			this._queueXhr = true;
		}
		this._alertWidgets = [];
	},
	
	postCreate: function() {
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		this.headers[HttpConstants.ACCEPT] = HttpConstants.CT_TEXT_JSON;

		this._licenseInfo = new com.ibm.team.repository.web.ui.internal.LicenseInfoWidget();
		this.connect(this._licenseInfo, "notify", function() {
			this._addLicenseMessage();
		});
		
		if(this.messages && !this._queueXhr) {
			this._displayMessages();	
		} else {
			//Unable to get the messages from the init data (could be RM...)
			//Try to get them directly from the JTS.
			this._getMessageService();
		}
	},
	
	_getMessageService: function() {
		var d = DiscoveredService.loadDiscoveryServices(NAMESPACES.jfs + "SystemMessages", SYSTEM_ALERTS_BINDING, true);
		d.addCallback(dojo.hitch(this, function(services){
			if(services && services.SystemMessages && services.SystemMessages.length > 0) {
				this.messagesEndpoint = services.SystemMessages[0].service;
			}
			if(this.messagesEndpoint) {
				this._loadMessages();
			} else {
				console.error("Unable to locate the system messages service.");
			}
		}));
	},
	
	_loadMessages: function() {
		jazz.client.xhrGet({
			url: this.messagesEndpoint,
			handleAs: "json",
			headers: this.headers,
			load: dojo.hitch(this, function(response) {
				this.messages = response.items;
				this._displayMessages();
			}),
			error: function(errObject) {
				console.error("Unable to load the system messages.");
			}
		});	
	},
	
	_addLicenseMessage: function(message) {
		this._showLicenseMessage = true;
		this._displayMessages();
	},
	
	_displayMessages: function() {
		this._cleanup();
		if(this._showLicenseMessage || this.messages.length) {
			for (var i = 0; i < this.messages.length; i++) {
				this._alertWidgets.push(new com.ibm.team.repository.web.ui.internal.SystemAlert({message: this.messages[i].description, style: {display: (i==0) ? "" : "none"}}).placeAt(this._alerts));
			}
			var count = this.messages.length;
			if (this._showLicenseMessage) {
				count++;
				this._licenseInfo.domNode.style.display = "none";
				this._alertWidgets.push(this._licenseInfo.placeAt(this._alerts));
				if (this._alertWidgets.length == 1) this._licenseInfo.domNode.style.display = "";
			}
			if (count > 1) {
				this._badge = dojo.create("span", { innerHTML: count, className: "badge" }, this.domNode);
				if (!this._timeout) this._timeout = setTimeout(dojo.hitch(this, this._rotateNext), this.rotateTime);
			}
			this._hp = new jazz.ui.HoverPopup({orientation:"V", createContents: dojo.hitch(this,function() {
				var content = dojo.create("div");
				for (var i = 0; i < this._alertWidgets.length; i++) {
					var node = dojo.clone(this._alertWidgets[i].domNode);
					node.style.display = "block";
					dojo.addClass(node,"com-ibm-team-repository-web-alert-hover-item");
					dojo.place(node, content);
				}
				return content;
			}), around: this.domNode, anchor: count > 1 ? this._badge : null });
		}
	},
	_cleanup: function() {
		dojo.empty(this._alerts);
		while(this._alertWidgets.length) {
			this._alertWidgets.pop().destroy();
		}
		if (this._hp) {
			this._hp.destroy();
			delete this._hp;
		}
		if (this._badge) {
			this.domNode.removeChild(this._badge);
			delete this._badge;
		}		
	},
	_rotateNext: function() {
		if (this._paused) {
			this._missedRotation = true;
			return;
		}
		var widgets = this._alertWidgets, 
			currentIndex = this._currentIndex,
			nextIndex = (currentIndex >= widgets.length - 1) ? 0 : currentIndex + 1,
			currentWidget = widgets[currentIndex],
			nextWidget = widgets[nextIndex];

		dojo.style(nextWidget.domNode, {opacity: 0, display: "", zIndex: "1"});
		
		if(!this.animOut) this.animOut = dojo.animateProperty({node: currentWidget.domNode, duration: 500, 
			properties: {
				opacity: {start: 1, end: 0, unit: "px"}
			},
			onEnd: dojo.hitch(this, function() {
				delete this.animOut;
				dojo.style(currentWidget.domNode, {opacity: "", display:"none", zIndex: "0"});
			})
		}).play();
		if (!this.animIn) this.animIn = dojo.animateProperty({node: nextWidget.domNode, duration: 1000, 
			properties: {
				opacity: {start: 0, end: 1, unit: "px"}
			},
			onEnd: dojo.hitch(this, function() {
				delete this.animIn;
			})
		}).play();
		
		this._currentIndex = nextIndex;
		
		this._timeout = setTimeout(dojo.hitch(this, this._rotateNext), this.rotateTime);
	},
	_pause: function(event) {
		this._paused = true;
	},
	_play: function(event) {
		this._paused = false;
		if (this._missedRotation) {
			setTimeout(dojo.hitch(this, this._rotateNext), 1200);
			delete this._missedRotation;
		}
	},
	uninitialize: function() {
		this.inherited(arguments);
	}
});

dojo.declare("com.ibm.team.repository.web.ui.internal.SystemAlert", [dijit._Widget, dijit._Templated], {
	templateString: "<span class='com-ibm-team-repository-web-SystemAlert'>${message}</span>",
	message: "",
	_setMessageAttr: function(message) {
		this.domNode.innerHTML = message;
	}
});

	
var NAMESPACES = {
	jd: "http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
	jfs: "http://jazz.net/xmlns/foundation/1.0/",
	rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
};
var MESSAGE_INFO = {
	namespaces: NAMESPACES,
	service: ["string", "jd:service/@rdf:resource", "1"]
};
var SYSTEM_ALERTS_BINDING = {
	namespaces: NAMESPACES,
	SystemMessages: [MESSAGE_INFO, "jfs:SystemMessages", "*"]
};

})();