/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.TRLink"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
(function(){

	/**
	 * Widget designed to include an <A> element within a <TR> element.  It
	 * optionally supports creating a <code>jazz.ui.ResourceLink</code>
	 * wrapper for the anchor.  It adds selection support for the <TR> element
	 * it's included in as the anchor gains and loses focus.
	 * 
	 * @name com.ibm.team.process.web.ui.internal.TRLink
	 * @extends dijit._Widget
	 */ 
	dojo.declare("com.ibm.team.process.web.ui.internal.TRLink", [dijit._Widget], { //$NON-NLS-1$	
		
		/*
		 * The <code>jazz.ui.ResourceLink</code> that was optionally created
		 * using the <code>args.resourceLink</code> object supplied in the constructor
		 * @private
		 */
		_resourceLink: null,
		
		/**
		 * Creates a new <code>TRLink<code> 
		 *
		 * @constructor 
		 * @param {Object} args The arguments to the constructor
		 * @param {String} args.href The the URL of the page the link refers to
		 * @param {DomNode|String} args.linkItem: the "label" of the anchor
		 * @param {Object} [args.resourceLink] The args for creating a resource link wrapper 
		 *        for the anchor.  See @link {jazz.ui.ResourceLink}.
    	 */
		constructor: function(args){
			
		},
		
		postCreate: function(){
			var link = document.createElement("a"); //$NON-NLS-1$
			link.href = this.href;
			if(dojo.isString(this.linkItem)){ //$NON-NLS-1$
				this.linkItem = document.createTextNode(this.linkItem);
			}
			link.appendChild(this.linkItem);
			
			this.connect(link, "onfocus", function(e){ //$NON-NLS-1$
				var row = e.target;
				while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
					row = row.parentNode;
				}
				if (row) {
					dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
				}
			});
			this.connect(link, "onblur", function(e){ //$NON-NLS-1$
				var row = e.target;
				while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
					row = row.parentNode;
				}
				if (row) {
					dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
				}
			});
			
			if(this.resourceLink != null){
				this._resourceLink = new jazz.ui.ResourceLink(this.resourceLink, link);
			}
			
			this.domNode.appendChild(link);
		},
		
		uninitialize: function(){
			if(this._resourceLink != null){
				this._resourceLink.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
})();
