/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
(function(){

	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var PROCESSING_BATCH_SIZE = 10;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog", com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog, { //$NON-NLS-1$
		constructor: function(){
			//this is called messages2 to avoid replacing the messages field in the parent class
			this.messages2 = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Overridden to add a list of the names of the team areas the given
		 * user is a member of to their display.
		 */
		getDispValue: function(elem){
			var processAreas = [];
			if (elem.processAreas) {
				for (var j = 0; j < elem.processAreas.length; j++) {
					processAreas.push(elem.processAreas[j].name);
				}
			}
			var disp;
			if (processAreas.length > 0) {
				disp = dojo.string.substitute(this.messages2.TeamContributorSelectionDialog_2, [elem.name, elem.userId, processAreas.sort().join(", ")]);//$NON-NLS-1$
			} else if (elem.doneLoading) {
				disp = dojo.string.substitute(this.messages2.TeamContributorSelectionDialog_1, [elem.name, elem.userId]);
			} else {
				disp = dojo.string.substitute(this.messages2.TeamContributorSelectionDialog_3, [elem.name, elem.userId]);
			}
			return disp;
		},
		
		/**
		 * Overridden so as to do processing (add team area names) on the
		 * results (contributors) of the query after it returns.
		 */
		_loadQueryResults: function (resultSet) {
			this.inherited(arguments);
			if (resultSet != null && resultSet.elements != null) {
				this._processQueryResults(0, PROCESSING_BATCH_SIZE);
			}
		},
		
		/**
		 * Public. Sets the default focus for the dialog.
		 */
		setDefaultFocus: function(){
			this.searchText.focus();
		},
		
		/**
		 * Adds a list of the names of the team areas each user in the results
		 * is a member of. Does so for a particular batch of the results, then
		 * calls itself recursively for the next batch.
		 * 
		 * @param index {int} where in the results to start processing
		 * @param batchSize {int} how many to process in this call
		 */
		_processQueryResults: function(index, batchSize) {
			var self = this;
			var params = {
				userItemIds: []
			};
			for (var i = index; i < index+batchSize && i < this.userSelector.options.length; i++) {
				params.userItemIds.push(this.userSelector.options[i]._itemId);
			}
			var serviceObj = {
				index: index,
				batchSize: batchSize,
				closureSearchCount: self.searchCount,
				success: function (resultSet) {
					// ensure the dialog is still there and that a new query has
					// not been started before processing results
					if (this.closureSearchCount === self.searchCount && self.userSelector) {
						for (var i = 0; i < this.batchSize; i++) {
							var user = resultSet.elements[i];
							var userRow = self.userSelector.options[this.index+i];
							user.name = userRow.userName;
							user.userId = userRow.userId;
							user.doneLoading = true;
							userRow.innerHTML = self.getDispValue(user);
							if (this.index+i === self.userSelector.options.length-1) {
								// just processed the last result
								return;
							}
						}
						// process the next batch
						self._processQueryResults(this.index+this.batchSize, this.batchSize);
					}
				},
				failure: function (errorObj) {
					self._queryError(obj);
				}
			};
			var srh = new com.ibm.team.repository.web.transport.ServiceResponseHandler(serviceObj, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getTeamAreasForUsers(params, srh);
		}
	});
	
})();
