/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.ViewHistoryViewlet");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.web.ui.PagedTableViewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.util.format");
dojo.require("jazz.util.html");
dojo.require("jazz.util.i18n");
dojo.require("jazz.app.viewHistory");
dojo.require("jazz.ui.ResourceLink");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ViewHistoryMessages");

(function() {

/*
 * Aliases
 */
var PagedTableViewlet = com.ibm.team.dashboard.web.ui.PagedTableViewlet;
var HTML = jazz.util.html;
var I18nUtil = jazz.util.i18n;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var viewHistory = jazz.app.viewHistory;
var ResourceLink = jazz.ui.ResourceLink;

/*
 * Constants
 */
var PREFERENCE_ITEMS_TO_SHOW = "items";

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ViewHistoryMessages");

/*
 * A viewlet that displays a news feed.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.ViewHistoryViewlet",
	PagedTableViewlet, {
	
	/*
	 * the css class for this viewlet; consumed by superclass.
	 */
	contentClass: "com-ibm-team-dashboard-viewlets-web-ui-internal-ViewHistoryViewlet",
	
	constructor: function() {
		this._entryCount = -1;
	},
	
	/*
	 * Initializes the view history viewlet.
	 */
	init: function() {
		this.inherited(arguments);
	},
	
	uninitialize: function() {
	},
	
	/*
	 * Called by super class when entries are needed.
	 */
	fetch: function (/* ServiceResponseHandler */ srh, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details) {
		delete this._errorMsgNode;

		var items = this.getPreference(PREFERENCE_ITEMS_TO_SHOW);
		if (!items || items < 1) {
			this._errorMsgNode = document.createTextNode(messages.errorItems);
		}
		
		// display error msg if there was an error
		if (this._errorMsgNode) {
			this._entryCount = 0;
			this._updateSecondaryTitle();
			srh.doCallback();
		}
		else {
			var serviceObj = {
				_srh: srh,
				_viewlet: this,
				_entries: entries,
				_details: details,
				success: function(response) {
					this._viewlet.handleFetch(response, this._entries, this._details, this._srh);
				},
				error: function(/* Error */ error) {
					this._srh.doErrorCallback(error);
				}
			};
		    var query = {
				application: "%MY_APPLICATION%",
				size: count,
				startIndex: startIndex
			};
			var deferred = viewHistory.get(query, true);
			deferred.addCallback(dojo.hitch(this, function(result) {
				this._handleFetch(result, entries, details, srh);
			}));
			deferred.addErrback(dojo.hitch(this, function(error) {
				srh.doErrorCallback(error);
			}));
		}
	},
	
	/*
	 * Returns the number of items to show per page. Called by superclass, and to
	 * be overriden by subclasses.
	 */
	/* number */ getItemsPerPage: function() {
		return this.getPreference(PREFERENCE_ITEMS_TO_SHOW);
	},
	
	/*
	 * Returns the message to display when there are no feed items to show.
	 */
	/* string */ getNoItemsMessage: function() {
		return messages.noItems;
	},
	
	/*
	 * Returns the message to display when there is no data available,
	 * including error messages.
	 */
	/* Node */ getNoResultsMessage: function() {
		if (this._errorMsgNode) {
			return this._errorMsgNode;
		}
		return document.createTextNode(this.getNoItemsMessage());
	},
	
	/*
	 * Returns the id of the latest reference feed item. The reference item is the
	 * first item of a feed at a given time (index 0).
	 */
	/* string */ getReferenceItemId: function() {
		if (this._trackReadMarks) {
			var readState = this.getContributorState().get("read");
			if (readState && readState[0]) {
				return readState[0];
			}
		}
	},
	
	/*
	 * Returns the total number of results in the feed; called by superclass.
	 */
	/* number */ getResultCount: function() {
		return this._entryCount;
	},
	
	/*
	 * Handles a feed request's response. Subclasses should use this handler
	 * for feed responses.
	 */
	_handleFetch: function(response, /* boolean */ entries, /* boolean */ details, /* ServiceResponseHandler */ srh) {
		// was there an error?
		if (response.errorMsg) {
			// show multi-line plain text error msg
			this._errorMsgNode = document.createElement("span");
			this._errorMsgNode.className = "errorText";
			this._errorMsgNode.innerHTML = HTML.escape(response.errorMsg).replace(/\n/g, "<br/>");
			this._entryCount = 0;
			this._updateSecondaryTitle();
			srh.doCallback();
		}
		else {
			// update result count
			this._entryCount = response.count;
			this._updateSecondaryTitle();

			// package results
			if (response.count > 0) {
				var result = new Object();
				var items = response.items;

				if (entries) {
					result.entries = items;
				}
				if (details) {
					result.details = items;
				}
				srh.doCallback(result);
			}
			else {
				// superclass will show no entries msg
				srh.doCallback();
			}
		}
	},
	
	/*
	 * Renders an entry for the given entry.
	 */
	/* Node[] */ renderEntry: function(entry, entryDetails, /* boolean */ zoomed) {
		var columns = new Array();
		var td = dojo.create("td", {className: "iconCell"});
		var div = dojo.create("div", {className: "iconContainer"}, td);
		var iconNode = dojo.create("img", {src: entry.iconUri, className: "icon"}, div);
		columns.push(td);

		td = dojo.create("td");
		var rl = new ResourceLink({
				uri: entry.uri,
				label: entry.label,
				iconNode: iconNode
		}).placeAt(td);
		columns.push(td);
		return columns;
	},
	
	/*
	 * Set number of entries back to initial state on each refresh.
	 */
	refresh: function() {
		this._entryCount = -1;
		this.inherited(arguments);
	},
	
	/*
	 * Updates the viewlet's secondary title based on the number of total
	 * items and unread items.
	 */
	_updateSecondaryTitle: function() {
		this.updateTotalPageFooter();
		var heading = "(" + this._entryCount + ")";
		this.getSite().setSecondaryTitle(heading);
	},	
	
	/*
	 * Refresh or update depending on which settings changed; called by platform.
	 */
	settingsChanged: function(oldSettings, newSettings) {
		if (newSettings.preferences.items != oldSettings.preferences.items) {
			this.reset(false);
		} else {
			this.update();
		}
	},
	
	/*
	 * Update secondary title on update.
	 */
	update: function() {
		this.inherited(arguments);
		this._updateSecondaryTitle();		
	}
});
})();