/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletPage");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");

dojo.require("dojo.i18n"); 
dojo.require("dojo.string");

dojo.require("jazz.ensemble.internal.util.bindings");
dojo.require("jazz.ensemble.WidgetPanel");
dojo.require("jazz.util.binding");
dojo.require("jazz.util.i18n");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

//
// Aliases
//
var binding = jazz.util.binding;
var bindings = jazz.ensemble.internal.util.bindings;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;

var messages;

/**
 * A page of viewlets (contents of one dashboard tab).
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletPage",
	[dijit._Widget, dijit._Templated, Observable], {
	
	// Variables passed into constructor
	page: null,					//required
	pageId: null,				//required
	dashboardContainer: null,	//required
	dashboardBundle: null,		//required
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletPage.html"),
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	
	postCreate: function() {
		if (this.serverTemplated) {
			
		} else {
			this._emptyTabAddViewletLink = jazz.util.i18n.addMessage(this._showInEditMode, dojo.string.substitute(messages.noViewletsAdd, [ messages.addViewletText ]))[0];
			this._emptyTabAddViewletEvent = dojo.connect(this._emptyTabAddViewletLink,"onclick",this.dashboardContainer,this.dashboardContainer.openCatalog);
			this.editing = (this.dashboardContainer.editMode != Constants.READ_ONLY) ? true : false;
			if (!this.editing)
				this._showInEditMode.style.display = "none"; 
		}
	},
	init: function() {
		this._yieldTimeout = setTimeout(dojo.hitch(this, this.init2), 0);
		this.isActive = true;
	},
	init2: function() {
		if (!this._isDestroyed) {
			delete this._yieldTimeout;
			this._createWidgetPanel();
			this._loadingText.style.display ="none";
		}
	},
	_createWidgetPanel: function() {
		var properties = {
			"com.ibm.team.dashboard.scope": this.dashboardBundle.scope,
			"com.ibm.team.dashboard.scopeItem": this.dashboardBundle.scopeItem,
			"com.ibm.team.dashboard.defaults": this.dashboardBundle.defaults
		};
		// only contributor dashboards with the current user owner are not shared
		var shared = (this.dashboardBundle.scope != DashboardConstants.SCOPE_CONTRIBUTOR) || (this.dashboardBundle.contributorItemId != this.dashboardBundle.scopeItem.itemId);
		this._widgetPanel = new jazz.ensemble.WidgetPanel({
			allowMaximize: true,
			properties: properties,
			readOnly: !this.dashboardBundle.permissions.modify,
			shared: shared,
			state: this.page.elem,
			onStateChange: dojo.hitch(this, this._onStateChange),
			uuid: this.dashboardBundle.dashboardUrl
		});
		this._tabArea.appendChild(this._widgetPanel.domNode);
		this._widgetPanel.startup();
		this.updateEmptyPageMessage();
	},
	isEditing: function() {
		return this._widgetPanel && this._widgetPanel.isDirty();
	},
	isVisible: function() {
		return (this.dashboardContainer.currentPage == this) && this.dashboardContainer.isVisible();
	},
	setEditMode: function(editing) {
		this.editing = editing;
		this._showInEditMode.style.display = editing ? "" : "none";
	},
	setActive: function(/* boolean */ active) {
		if (this.isActive != active) {
			this.isActive = active;
			if (this._widgetPanel) {
				if (active) {
					this._widgetPanel.onShow();
				}
				else {
					this._widgetPanel.onHide();
				}
			}
		}
	},
	setLayout: function(/* string */ type) {
		this._widgetPanel.setLayout(type);
	},
	addWidget: function(/* object */ data) {
		var widgetElem = this._widgetPanel.createWidget();
		var widget = binding.bind(widgetElem, bindings.Widget);
		
		widget.setType(data.type);
		widget.setDefinition(data.widgetUri);
		widget.setTitle(data.title);
		if (data.icon) {
			widget.setIcon(data.icon);
		}
		
		var preferences = data.preferences;
		if (preferences instanceof Array && preferences.length > 0) {
			var prefs = [];
			widget.setPreferencesParseType("Collection");
			var len = preferences.length;
			for (var i=0;i<len;++i) {
				var pref = preferences[i];
				prefs.push({name: pref.name, value: pref.value});
			}
			widget.setPreferences(prefs);
		}		
		this._widgetPanel.addWidget(widgetElem);
		this.updateEmptyPageMessage();
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
	},
	addWidgetFromElement: function(/* Element */ elem) {
		this._widgetPanel.addWidget(elem);
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
	},
	removeWidget: function(/* string */ id) {
		this._widgetPanel.removeWidget(id);
		this.updateEmptyPageMessage();
	},
	
	updateEmptyPageMessage: function() {
		// show msg if no widgets
		this._setEmptyTabMessage(this._widgetPanel.isEmpty());
	},
	
	_onStateChange: function() {
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
		this.updateEmptyPageMessage();
	},
	
	_setEmptyTabMessage: function(isVisible) {
		if (isVisible) {
			this._emptyTabMessage.style.display = "block";	
		} else {
			this._emptyTabMessage.style.display = "none";
		}	
	},
	/*
	 * Destroys all viewlet widgets so that they can be garbage collected.
	 */
	destroy: function() {
		this._isDestroyed = true;
		// are we in the middle of loading?
		if (this._yieldTimeout) {
			// cancel loading
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		if (this._emptyTabAddViewletEvent) dojo.disconnect(this._emptyTabAddViewletEvent);
		if (this._widgetPanel) this._widgetPanel.destroy();
		this.inherited(arguments);
	},
	refreshPage: function() {
		this.dashboardContainer._refreshCurrentTab();
	}
});

})();
