/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.accessgroup.AccessGroupManagementPage"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.accessgroup.ManageAccessGroups"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var PREFIX = "com.ibm.team.process."; //$NON-NLS-1$
	var AccessGroupManagementPageMessages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var SERVER_PACKAGE = "com.ibm.team.process.web.ui.internal.admin.accessgroup."; //$NON-NLS-1$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	var ACTION_MAP = {};
	var ACTION_SETS = [{
		label: AccessGroupManagementPageMessages.AccessGroupManagementPage_Administration,
		actions: [{
			label: AccessGroupManagementPageMessages.AccessGroupManagementPage_AccessGroups,
			shortId: "manageAccessGroups", //$NON-NLS-1$
			widget: "ManageAccessGroups", //$NON-NLS-1$
			showInNav: true
		}]
	}];
	
	function getWorkbench(){
		return PlatformUI.getWorkbench();
	}
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.accessgroup.AccessGroupManagementPage", dijit._Widget, { //$NON-NLS-1$

		_actionRegistry: null,
		
		constructor: function(){
			// Get owning application key from the URL. Embedding fronting
			// application must provide it's application key.
			// If not present, then it means that request has been made by JTS 
			// itself. In that case, set the application key of JTS.
			var _queryParams = dojo.queryToObject(window.location.search.slice(1));
			if (_queryParams.owningApplicationKey && _queryParams.owningApplicationKey !== ProcessConstants.JTS_OWNING_APPLICATION_KEY) {
				this._owningApplicationKey = _queryParams.owningApplicationKey;
				this._isEmbedded = true;
			} else {
				this._owningApplicationKey = ProcessConstants.JTS_OWNING_APPLICATION_KEY;
			}
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.accessgroup", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			this.inherited(arguments);
			this._allLinks = [];
			
			dojo.addClass(this.domNode, "com-ibm-team-repository-admin"); //$NON-NLS-1$
			
			// register actions and create hyperlinks
			this._actionRegistry = getWorkbench().getActionRegistry();
			dojo.forEach(ACTION_SETS, function(actionSet){
				this._registerActions(actionSet);
			}, this);
		},
		
		_registerActions: function(actionSet){
			dojo.forEach(actionSet.actions, function(action){
				var id = PREFIX + action.shortId;
				ACTION_MAP[action.shortId] = action;
				this._actionRegistry.registerAction(id, this, "_handleAction"); //$NON-NLS-1$
			}, this);
		},
		
		_handleAction: function(args){
			if (!args.action) {
				args.action = PREFIX + ACTION_SETS[0].actions[0].shortId;
			}
			
			// cache the previous entry to see if the widget should be destroyed or
			// not
			this._previousActionEntry = this._currentActionEntry || null;
			
			// parse out the short id from the fully-qualified id
			var shortId = args.action.substring(args.action.lastIndexOf(".") + 1); //$NON-NLS-1$
			this._currentActionEntry = ACTION_MAP[shortId];
			this._currentArgs = args;
			this._currentArgs.owningApplicationKey = this._owningApplicationKey;
			this._currentArgs.parent = this;
			this._currentArgs.hideTeams = this._hideTeams;
			if (this._isEmbedded) {
				this._currentArgs.isEmbedded = this._isEmbedded;
			}
			
			// set the title for the action
			getWorkbench().setTitle(this._currentActionEntry.label);
			
			// highlight the selected link and unhighlight the others
			dojo.forEach(this._allLinks, function(link){
				if (link.__jazzActionId === shortId) {
					dojo.toggleClass(link, "selected", true); //$NON-NLS-1$
				} else {
					if (link.aliases) {
						var found = false;
						for (var i = 0; i < link.aliases.length; i++) {
							if (link.aliases[i] === shortId) {
								dojo.toggleClass(link, "selected", true); //$NON-NLS-1$
								found = true;
								break;
							}
						}
						if (!found) {
							dojo.toggleClass(link, "selected", false); //$NON-NLS-1$
						}
					} else {
						dojo.toggleClass(link, "selected", false); //$NON-NLS-1$
					}
				}
			}, this);
			
			// call the actual method implementation
			if (!this._currentActionEntry.simpleAction || !this._currentContentWidget ||
				this._currentContentWidget.id.replace(/_\d+$/, "").replace(/_/g, ".") !== SERVER_PACKAGE + this._currentActionEntry.widget) { //$NON-NLS-1$ //$NON-NLS-2$
				this._doShowCurrentWidget(false);
			}
			if (this._currentActionEntry.simpleAction) {
				this._currentActionEntry.simpleAction(args, this._currentContentWidget);
			}
		},
		
		reloadCurrent: function(){
			this._previousActionEntry = this._currentActionEntry || null;
			this._doShowCurrentWidget(/*isReload*/true);
		},
		
		_doShowCurrentWidget: function(isReload){
			// save a reference to the prior content widget
			this._previousContentWidget = this._currentContentWidget || null;
			
			if (this._currentActionEntry.cached) {
				this._showCachedWidget(isReload);
			} else {
				this._showUncachedWidget();
			}
			
			// help the garbage collector do its job
			if (this._previousContentWidget && !this._previousActionEntry.cached) {
				this._previousContentWidget.destroyRecursive();
			}
		},
		
		_showCachedWidget: function(isReload){
			var className = this._currentActionEntry.widget;
			if (this._widgetCache[className]) {
				// get widget from the cache if it exists
				this._currentContentWidget = this._widgetCache[className];
			} else {
				// otherwise instantiate it and add it to the cache
				var constructor = dojo.getObject(SERVER_PACKAGE + className);
				this._currentContentWidget = new constructor(this._currentArgs);
				dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea"); //$NON-NLS-1$
				this._widgetCache[className] = this._currentContentWidget;
			}
			if (isReload) {
				this._currentContentWidget.reload(this._currentArgs);
			} else {
				this._currentContentWidget.show(this._currentArgs);
				this._setMainContent(this._currentContentWidget.domNode);
			}
		},
		
		_showUncachedWidget: function(){
			var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
			this._currentContentWidget = new constructor(this._currentArgs);
			dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea"); //$NON-NLS-1$
			this._setMainContent(this._currentContentWidget.domNode);
		},
		
		_setMainContent: function(widgetNode) {
			dojox.xml.parser.removeChildren(this.domNode);
			this.domNode.appendChild(widgetNode);
		}		
	});
})();
