/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.process_templates.ExportTemplateProgressDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.ProgressBar"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
(function(){

	var _WIDTH_IN_EM = 50;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var EXPORT_PREPARATION_STATUS_SERVICE_URL = dojo.baseUrl + "../../service/com.ibm.team.process.internal.service.web.IProcessDefinitionFileService/checkExportPreparationProgress";//$NON-NLS-1$
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.process_templates.ExportTemplateProgressDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/process_templates/templates/ExportTemplateProgressDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
		widgetsInTemplate: true,
		constructor: function(args){
			if (!args) {
				args = {};
			}
			this.width = _WIDTH_IN_EM + "em"; //$NON-NLS-1$
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_templates", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			this.processExportKey = dojox.uuid.generateTimeBasedUuid();
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.contentNode,
				primaryTitle: this.messages.ExportTemplateProgressDialog_1,
				width: this.width,
				destroyContents: true,
				closable: false // force user to click Cancel button to close the dialog
			});
			
			// show progress bar
			this._showProgressBar();

			this._doExportTemplate();

			this.connect(this.cancelButton, "onclick", this._onCancelClick); //$NON-NLS-1$
		},
		
		/**
		 * Event handler for Cancel button.
		 * 
		 * @param {Object} e
		 */
		_onCancelClick: function(e) {
			dojo.stopEvent(e);
			
			// stop checking progress 
			if (this.intervalId) {
				clearInterval(this.intervalId);
			}
			
			if (window.stop) {
				window.stop();
			}
			
			this.dialog.close();

			// Send a request to cancel service without callback methods.
			// Server will do the clean job.
			dojo.xhrGet({
				url: this.serviceUrl + "/cancelExport", //$NON-NLS-1$,
				handleAs: "json", //$NON-NLS-1$
				content: {
					exportKey: this.processExportKey
				},
				preventCache: true
			});
		},
		
		/**
		 * Show progress bar in indeterminate style since we are unable to know the 
		 * accurate progress data for export. 
		 */
		_showProgressBar: function() {
			this._progressBar.update({
				indeterminate: true
			});
		},
		
		/**
		 * Use window.location to download the template zip file from server.
		 * Also set the interval to check export progress.
		 */
		_doExportTemplate: function() {
			window.location = this.serviceUrl + "?itemId=" + this.itemId + "&exportKey=" + this.processExportKey; //$NON-NLS-1$ //$NON-NLS-2$
			this.intervalId = setInterval(dojo.hitch(this, "_checkExportPreparationStatus"), 1000); //$NON-NLS-1$
			
		},
		
		/**
		 * Check the export preparation progress. It might take long time to prepare 
		 * the template zip file on server side. So, we need to check this progress.
		 * If the progress status is 'completed' or 'error', this progress dialog will
		 * be closed. 
		 */
		_checkExportPreparationStatus: function() {
			dojo.xhrGet({
				url: EXPORT_PREPARATION_STATUS_SERVICE_URL,
				handleAs: "json", //$NON-NLS-1$
				content: {
					exportKey: this.processExportKey
				},
				preventCache: true,
				load: dojo.hitch(this, function(response, ioArgs){
					if (response.status === "completed" || response.status === "error") { //$NON-NLS-1$ //$NON-NLS-2$
						clearInterval(this.intervalId);
						if (this.dialog) {
							this.dialog.close();
						}
					}
					return response;
				}),
				error: function(response, ioArgs){
					return response;
				}
			});
		}
	});
})();
