/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2012. All
 * Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.AddMembersWizard"); //$NON-NLS-1$	

	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.Wizard"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWizardStep"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.SelectRolesWizardStep"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

	var WIZARD_WIDTH = "66.1em"; //$NON-NLS-1$

	/**
	 * Provides a jazz.ui.Wizard for the SelectUsersWizardStep and
	 * SelectRolesWizardStep.
	 * 
	 * @constructor new new
	 *              com.ibm.team.process.web.ui.internal.admin.projects.AddMembersWizard({arguments});
	 * @arguments: disableUsers:[] :An array of objects containing the itemId of
	 *             users that should appear disabled in the list. Example:{
	 *             itemId="_zQADspleEd-joaYjvz6g2w"}
	 *             <p>
	 *             allRoles : [] : Array of role objects.
	 *             <p>
	 *             defaultRole:The default role object.
	 * @since 4.0.1
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.AddMembersWizard", null, { //$NON-NLS-1$

		constructor : function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

			this._usersStep = new com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWizardStep(args);
			this._rolesStep = new com.ibm.team.process.web.ui.internal.admin.projects.SelectRolesWizardStep(args);

			this._wizard = new jazz.ui.Wizard({
				primaryTitle : this.messages.AddMembersWizard_Title,
				width : WIZARD_WIDTH,
				steps : [this._usersStep, this._rolesStep],
				customProcessFinish : dojo.hitch(this, this._onFinish),
				customProcessCancel : dojo.hitch(this, this._onCancel)
			});
		},

		/**
		 * Passes the selected users and roles to the callback function provided
		 * by the user and performs cleanup.
		 */
		_onFinish : function() {
			this.onFinish(this._usersStep.getSelectedUsers(), this._rolesStep.getChosenRoles());

			this._destroyWizard();
		},

		/**
		 * Override this method to get and process selected new members with
		 * selected roles.
		 */
		onFinish : function(newMemebers, chosenRoles) {
		},

		_onCancel : function() {
			this._wizard.close();
			this._destroyWizard();
		},

		/**
		 * Clean up the resources and widgets used by this wizard.
		 */
		_destroyWizard : function() {
			this._wizard.destroyRecursive();
			delete this._wizard;
		}
	});
})();
