/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeEditor"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("jazz.ui.SimpleToolbar"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeModel"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeTree"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;

(function(){
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/IterationTypeEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(arg){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			
			this._actionsBar = new jazz.ui.SimpleToolbar();
			
			this._newIterationTypeButton = this._actionsBar.addButton(this, this._handleNewIterationTypeClick, {
				label: this.messages.IterationTypeEditor_NewIterationTypeAction,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/new-iteration-type_co.gif") //$NON-NLS-1$ //$NON-NLS-2$
			});
			if (this.editor._isArchived){
				this._actionsBar.disableButton(this._newIterationTypeButton);
			};
			
			this._multiActionButton = this._actionsBar.addButton(this, this._handleMultiActionClick, this._getDeleteActionArgs());
			this._actionsBar.disableButton(this._multiActionButton);
			
			this._showArchivedButton = this._actionsBar.addButton(this, this._handleShowArchivedClick, {
				label: this.messages.IterationTypeEditor_ShowArchivedAction,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/iteration_type_arch_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				buttonType: jazz.ui.SimpleToolbar.ButtonTypes.TWO_STATE
			});
			
			this.actionsBarDiv.appendChild(this._actionsBar.domNode);
			
			this.refresh();
			
			this.connect(this._identifier, "onkeyup", "_handleIterationTypeDataChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._name, "onkeyup", "_handleIterationTypeDataChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._summary, "onkeyup", "_handleIterationTypeDataChange"); //$NON-NLS-1$ //$NON-NLS-2$

		},
				
		refresh: function(){
			this._clearState();			
			
			this.editor.addSaveParticipant(this, "com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeEditor"); //$NON-NLS-1$	
			this._iterationTypeTree = this._createIterationTypeTree();
			dojox.xml.parser.removeChildren(this.iterationTypeTreeDiv);
			this.iterationTypeTreeDiv.appendChild(this._iterationTypeTree.domNode);
						
		},
		
		_clearState: function(){
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeEditor"); //$NON-NLS-1$	
			delete this._model;
			if (this._iterationTypeTree) {
				this._iterationTypeTree.destroyRecursive();
				delete this._iterationTypeTree;
			}
			this._identifier.disabled = true;
			this._identifier.value = ProcessConstants.EMPTY;
			this._name.disabled = true;
			this._name.value = ProcessConstants.EMPTY;
			this._summary.disabled = true;
			this._summary.value = ProcessConstants.EMPTY;
			
		},
		
		changesPending: function(){
			return this._model.store.isDirty();
		},
		
		_createIterationTypeTree: function(){
		
			var store = this.editor._getProcessAreaStore();
			this._model = new com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeModel({
				store: store
			});
			
			this._updateArchivedQuery();
			
			var self = this;
			var tree = new com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeTree({
				model: this._model,
				editorContext: ProcessConstants.EditorContext.PROJECT,
				
				focusChanged: function(node, showValidationMsg){
					this._updateDetails(node.item);
					this._updateMultiActionButton(node.item);
					self._validateIterationType(node.item, showValidationMsg === undefined ? true : showValidationMsg);
				},
								
				_updateDetails: function(item) {
					self._identifier.disabled = false;
					self._identifier.value = this.model.store.getValue(item, "contextId");//$NON-NLS-1$	
					
					self._name.disabled = false;
					var name = this.model.store.getValue(item, "name"); //$NON-NLS-1$	
					if (name) {
						self._name.value = name;
					} else {
						self._name.value = ""; //$NON-NLS-1$	
					}
					
					self._summary.disabled = false;
					var summary = this.model.store.getValue(item, "summary"); //$NON-NLS-1$	
					if (summary) {
						self._summary.value = summary;
					} else {
						self._summary.value = ""; //$NON-NLS-1$	
					}
				},
				
                _updateMultiActionButton: function(item){
                	var actionArgs = self._getArchiveActionArgs();
                	if (this.isNewItem(item)) {
                		actionArgs = self._getDeleteActionArgs();
                	} else if (this.isArchivedItem(item)) {
                		actionArgs = self._getRestoreActionArgs();
                	}
                	self._actionsBar.updateButton(self._multiActionButton, actionArgs);
                	self._actionsBar.enableButton(self._multiActionButton);
                }
			});
			return tree;
			
		},
		
		_handleShowArchivedClick: function(event) {
			event.preventDefault();
			this._updateArchivedQuery();
			this._iterationTypeTree.update();
			if (this._iterationTypeTree.isArchivedItemSelected()) {
				this._clearFocus();
			}
		},
			
		_handleNewIterationTypeClick: function(event) {
			event.preventDefault();
            this._iterationTypeTree.addNewIterationType({
            	name: this._computeInitialIterationTypeAttributeValue("name", this.messages.IterationTypeEditor_NewIterationTypeDefaultName, this.messages.IterationTypeEditor_NewIterationTypeDefaultName_1), //$NON-NLS-1$
				contextId: this._computeInitialIterationTypeAttributeValue("contextId", this.messages.IterationTypeEditor_NewIterationTypeDefaultId, this.messages.IterationTypeEditor_NewIterationTypeDefaultId_1), //$NON-NLS-1$
				summary: "" //$NON-NLS-1$
            });
			this.editor.setDirty(this.editor.changesPending());
		},
						
		_handleMultiActionClick: function(event){
			event.preventDefault();
			var item = this._iterationTypeTree.getFocusedItem();
			if (item !== null) {
				if (this._iterationTypeTree.isNewItem(item)) {
					this._handleDelete(item);
				} else if (this._iterationTypeTree.isArchivedItem(item)) {
					this._handleUnarchive(item);
				} else {
					this._handleArchive(item);
				}
			}
		},
		
		_handleDelete: function(item){
			this._iterationTypeTree.deleteSelectedItem();
			this._clearFocus();
			this.editor.setDirty(this.editor.changesPending());
		},

		_handleArchive: function(item){
			var question = dojo.string.substitute(this.messages.IterationTypeEditor_ArchiveIterationTypeConfirmationMessage, [this._iterationTypeTree.getLabel(item)]);
			if (confirm(question)) {
				this._iterationTypeTree.lastFocused.setSelected(false);
				this._iterationTypeTree.archiveSelectedItem();
				this.editor.setDirty(this.editor.changesPending());
				if (this._actionsBar.isPressed(this._showArchivedButton)) {
					this._iterationTypeTree.focusItem(item);
				} else {
					this._clearFocus();
				}
			}
		},
		
		_handleUnarchive: function(item){
			this._iterationTypeTree.unarchiveSelectedItem();
			this.editor.setDirty(this.editor.changesPending());
			this._iterationTypeTree.focusItem(item);
		},

		_handleIterationTypeDataChange: function(event) {
			var selectedNode = this._iterationTypeTree.lastFocused;
			this._iterationTypeTree.updateSelectedIterationType(this._getIterationTypeData());
			this.editor.setDirty(this.editor.changesPending());
			this._validateIterationType(this._iterationTypeTree.getFocusedItem(), true);
			this._iterationTypeTree.update();
			if (this._iterationTypeTree.lastFocused.getIdentity() !== selectedNode.getIdentity()) {
				// after the tree updates, the selected node may have changed 
				// due to reordering; set it back to the original selection
				this._iterationTypeTree.setFocusedNode(selectedNode);
			}
		},
		
		_clearFocus: function() {
			try {
				this._iterationTypeTree.lastFocused.setSelected(false);
			} catch(error) {
				// In some instances(like delete) the last focused no longer is valid
				// even though it still is defined...just ignore the error since it's
				// irrelevant
			}
			this._iterationTypeTree.lastFocused = null;
            this._actionsBar.disableButton(this._multiActionButton);
			
			this._identifier.value = ProcessConstants.EMPTY;
			this._identifier.disabled = true;
			this._name.disabled = true;
			this._name.value = ProcessConstants.EMPTY;
			this._summary.disabled = true;
			this._summary.value = ProcessConstants.EMPTY;
			
		},
						
		canSave: function(){
			var msg = this._validateRequired();
			return msg === undefined; // If no error message we can save.
		},
		
		aboutToBeSaved: function(saveObj){
			var dirtyIterationTypes = [];
			this._model.store.fetch({
				query: {
					enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this._model.store, function(items){
					for (var i = 0; i < items.length; i++) {
						var item = items[i];
						if (this.isDirty(item)) {
							var dirtyIterationType = {};
							dirtyIterationType.itemId = this.getIdentity(item);
							dirtyIterationType.stateId = this.getValue(item, "stateId"); //$NON-NLS-1$
							dirtyIterationType.id = this.getValue(item, "contextId"); //$NON-NLS-1$
							dirtyIterationType.name = this.getValue(item, "name"); //$NON-NLS-1$
							dirtyIterationType.summary = this.getValue(item, "summary"); //$NON-NLS-1$
							dirtyIterationType.archived = this.getValue(item, "archived"); //$NON-NLS-1$
							dirtyIterationTypes.push(dojo.toJson(dirtyIterationType));
						}
					}
				})
			});
			saveObj.jsonIterationTypes = dirtyIterationTypes;
		},
		
		saved: function(){
			this._model.store.save();
		},
				
		_validateRequired: function(){
			var dirtyIterationTypes = [];
			this._model.store.fetch({
				query: {
					enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this._model.store, function(items){
					if (items && items.length > 0) {
						dirtyIterationTypes = items;
					}
				})
			});
			
			for (var i = 0; i < dirtyIterationTypes.length; i++) {
				var valid = this._validateIterationType(dirtyIterationTypes[i], false);
				if (valid.isError) {
					return valid.msg;
				}
			}
	},
		
		_validateIterationType: function(iterationTypeItem, displayMessage){
			
			// Only set the error state to false if it has already been set
			// otherwise it will dirty the model triggering a false pending changes
			if (this._model.store.hasAttribute(iterationTypeItem, "errorState")) {  //$NON-NLS-1$
				this._model.store.setValue(iterationTypeItem, "errorState", false); //$NON-NLS-1$
			}

			var msg;
			var isError;
			var id = this._model.store.getValue(iterationTypeItem, "contextId"); //$NON-NLS-1$
            
            if (id.length === 0 || id === this.messages.IterationTypeEditor_IdentifierValueWhenEmpty) {
            	msg = this.messages.IterationTypeEditor_IdentifierRequired;
            	isError = true;
            	this._model.store.setValue(iterationTypeItem, "errorState", true); //$NON-NLS-1$
            } else if (this._isDuplicateIterationType(id) === true) {
            	msg = dojo.string.substitute(this.messages.IterationTypeEditor_DuplicateIDMessage, [id]);
            	isError = true;
            	this._model.store.setValue(iterationTypeItem, "errorState", true); //$NON-NLS-1$
            } else if (!this._model.store.isNewItem(iterationTypeItem) && this._model.store.getValue(iterationTypeItem, "savedContextId") !== id) { //$NON-NLS-1$
            	// Only warn on iteration types that had been previously saved
            	msg = this.messages.IterationTypeEditor_ChangeIterationTypeIDMessage;
            	isError = false;
            }
			
			if (displayMessage === true) {
				if (msg) {
					if (isError === true) {
						this.editor._displayLongErrorMessage(msg);
					} else {
						this.editor._displayLongWarningMessage(msg);
					}
				} else {
					this.editor._clearMessage();
				}
			}
			
			return {
				msg: msg,
				isError: isError
			};
			
		},
		
		_isDuplicateIterationType: function(identifier){
			var count = 0;
			this._model.store.fetch({
				query: {
					enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this._model.store, function(items){
					for (var i = 0; i < items.length; i++) {
						var itemIdentifier = this.getValue(items[i], "contextId"); //$NON-NLS-1$
						if (identifier === itemIdentifier) {
							count++;
						}
					}
				})
			});
			return count > 1;
		},
		
		_updateArchivedQuery: function(){
			if (!this._actionsBar.isPressed(this._showArchivedButton)) {
				this._model.query.archived = false;
			} else {
				delete this._model.query.archived;
			}
		},
		
		_getIterationTypeData: function() {
			var data = {
				contextId : dojo.string.trim(this._identifier.value),
				name : dojo.string.trim(this._name.value),
				summary: this._summary.value
			};
			
			if (data.contextId.length === 0 && data.name.length ===0) {
				// Need to show something in the tree
				data.contextId = this.messages.IterationTypeEditor_IdentifierValueWhenEmpty;
			}
			return data;
		},
		
		_getDeleteActionArgs: function() {
			return {
				label: this.messages.IterationTypeEditor_DeleteIterationTypeAction,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/delete_co.gif") //$NON-NLS-1$ //$NON-NLS-2$
			};
		},
		
		_getArchiveActionArgs: function() {
			return {
				label: this.messages.IterationTypeEditor_ArchiveIterationTypeAction,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/archive_hov.gif") //$NON-NLS-1$ //$NON-NLS-2$
			};
		},
		
		_getRestoreActionArgs: function() {
			return {
				label: this.messages.IterationTypeEditor_RestoreIterationTypeAction,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/unarchive_hov.gif") //$NON-NLS-1$ //$NON-NLS-2$
			};
		},
				
		_computeInitialIterationTypeAttributeValue: function(attributeName, defaultValue, defaultValueIncremented) {
			var initialIterationTypeValue = defaultValue;
			var i = 0;
			var store = this._model.store;
			var containsValue = function(initialIterationTypeValue, items) {
				if (initialIterationTypeValue && items && items.length > 0) {
					for (var k = 0; k < items.length; k++) {
						var attributeValue = store.getValue(items[k], attributeName);
						if (initialIterationTypeValue === attributeValue) {
							return true;
						}
					}
				}
				return false;
			};
			
			var fetchIterationTypesComplete = function(items, request) {
				while (containsValue(initialIterationTypeValue, items)) {
					i++;
					initialIterationTypeValue = dojo.string.substitute(defaultValueIncremented, [i]);
				}
			};
			
			store.fetch({
				query: { parent: ProcessConstants.PermissionsScopeContext.ITERATION_TYPES},
				queryOptions: {deep: true},
				onComplete: fetchIterationTypesComplete
			});
			
			return initialIterationTypeValue;
		},
				
		uninitialize: function(){
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeEditor"); //$NON-NLS-1$
			if (this._iterationTypeTree) {
				this._iterationTypeTree.destroyRecursive();
				delete this._iterationTypeTree;
			}
			if (this._actionsBar) {
				this._actionsBar.destroyRecursive();
				delete this._actionsBar;
			}
			delete this.editor;
			delete this._model;
			this.inherited(arguments);
		}
		
	});
	
})();
