/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorConfigurationStore"); //$NON-NLS-1$

	dojo.require("jazz.util.html"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorStore"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	/**
	 * This store handles populating itself with operation behavior
	 * configuration information from the server, and provides convenience
	 * methods for interacting with that information.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorConfigurationStore", com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorStore, { //$NON-NLS-1$
		
		/*
		 * Allows close() to be called to clear out the store
		 */
		clearOnClose: true,
		
		/**
		 * Creates the store, but does not load any data, to allow for lazy
		 * loading. To load, call initialize().
		 */
		constructor: function (args) {
			this._processAreaItemId = args.processAreaItemId;
			this._processProviderUrl = args.processProviderUrl;
			this._processAreaContext = args.processAreaContext;
			this._owningApplicationKey = args.owningApplicationKey;
			this._extensionStore = args.extensionStore;
			// these stores have to be manually kept up to date as of now...
			this.rolesStore = args.rolesStore;
			this.contextStore = args.contextStore;
			this.connect(this.contextStore, "onSet", "_onSetContext"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this.data = {
				// don't declare an "identifier": the "id" value will not be
				// unique, just let dojo use the default identifier
				items: []
			};
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Loads the store. Calling more than once without calling uninitialize
		 * in between does nothing (but still calls successCallback).
		 */
		initialize: function(successCallback, errorCallback) {
			if (!this._isInitialized) {
				if (this.contextStore && !this.contextStore.isInitialized()) {
					// contexts need to be loaded to continue
					setTimeout(dojo.hitch(this, function() {
						this.initialize(successCallback, errorCallback);
					}), 100);
					return;
				}
				var args = {
					processAreaContext: this._processAreaContext,
					context: ProcessConstants.BehaviorContext.ALL,
					roleId: "", //$NON-NLS-1$
					owningApplicationKey: this._owningApplicationKey	
				};
				if (this._processAreaItemId) {
					args.processAreaItemId = this._processAreaItemId;
				} else if (this._processProviderUrl) {
					args.processProviderUrl = this._processProviderUrl;
				}
				ProcessClient.getOperationBehavior(args, new ServiceResponseHandler({
					success: dojo.hitch(this, function(data) {
						this._populateConfigurations(data, dojo.hitch(this, function() {
							this._isInitialized = true;
							successCallback();
						}));
					}),
					failure: errorCallback
				}, "success", "failure")); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				successCallback();
			}
		},
		
		/**
		 * Callback after operation behavior configurations are retrieved from
		 * the server. Runs in asynchronous chunks so as not to tie up the
		 * browser's UI thread too long (this can take a while when there are
		 * many behavior configurations).
		 * 
		 * @param configurations {OperationBehaviorConfigurationDTO[]}
		 *            collection of behavior configurations
		 * @param callback {Function} function to call once configurations are
		 *            populated
		 */
		_populateConfigurations: function(configurations, callback) {
			// pair each configuration with the extension it represents to do
			// additional processing
			if (configurations && configurations.length > 0) {
				var i = 0;
				var populateConfiguration = dojo.hitch(this,  function() {
					var configuration = configurations[i];
					var query = {};
					query[ProcessConstants.ExtensionDTO.ID] = "*" + configuration[ProcessConstants.ExtensionDTO.ID]; //$NON-NLS-1$
					this._extensionStore.fetch({
						query: query,
						queryOptions: {
							deep: true
						},
						onItem: dojo.hitch(this, function(item) {
							this._populateConfiguration(item, configuration);
						})
					});
					i++;
					if (i < configurations.length) {
						setTimeout(populateConfiguration);
					} else {
						// mark the store saved, subsequent change will dirty it
						this.save();
						callback();
					}
				});
				// run the query and processing for each configuration in
				// sequential async chunks
				setTimeout(populateConfiguration);
			} else {
				// no configurations, just call callback
				callback();
			}
		},
		
		/**
		 * Performs additional processing on configuration items from the server
		 * and then puts them in the configuration store. Populates them with
		 * info from their corresponding extension item where necessary.
		 */
		_populateConfiguration: function(extension, configuration) {
			configuration[ProcessConstants.ExtensionDTO.ID] = this._extensionStore.getValue(extension, ProcessConstants.ExtensionDTO.ID);
			if (configuration[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] === undefined || configuration[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] === null) {
				configuration[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = ""; //$NON-NLS-1$
			} else if (configuration[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID].indexOf(ProcessConstants.BEHAVIOR_PATH_SEPARATOR) !== -1) {
				// check if this configuration is for an iteration that moved
				this.contextStore.fetch({
					query: {
						originalContextId: configuration[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID]
					},
					queryOptions: {
						deep: true
					},
					scope: this,
					onItem: function(item) {
						configuration[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = this.contextStore.getValue(item, "contextId"); //$NON-NLS-1$
					}
				});
			}
			configuration[ProcessConstants.BehaviorConfigurationDTO.CONFIGURED] = true;
			if (configuration[ProcessConstants.ExtensionDTO.ID].split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length === 2) {
				// configuration is for an operation
				configuration[ProcessConstants.ExtensionDTO.NAME] = this._extensionStore.getValue(extension, ProcessConstants.ExtensionDTO.NAME);
				configuration[ProcessConstants.ExtensionDTO.DESCRIPTION] = this._extensionStore.getValue(extension, ProcessConstants.ExtensionDTO.DESCRIPTION);
			} else {
				// configuration is for a precondition or follow-up action
				if (configuration[ProcessConstants.ExtensionDTO.DESCRIPTION] === undefined || configuration[ProcessConstants.ExtensionDTO.DESCRIPTION] === null) {
					configuration[ProcessConstants.ExtensionDTO.DESCRIPTION] = ""; //$NON-NLS-1$
				}
				if (configuration[ProcessConstants.BehaviorConfigurationDTO.CONTENT] === undefined) {
					configuration[ProcessConstants.BehaviorConfigurationDTO.CONTENT] = null;
				}
			}
			// create new item
			this.newItem(configuration, {
				parent: this.getParent(configuration, true),
				attribute: ProcessConstants.ExtensionDTO.ITEMS
			});
		},
		
		/**
		 * Gets the item from the configuration store, or creates a new item,
		 * which will be the given configuration's parent.
		 * 
		 * @param configuration {OperationBehaviorConfigurationDTO} if its an
		 * 			  item that's not in the store yet, {dojo.data.Item} if its
		 * 			  not a new item
		 * @param create {Boolean} whether or not to create the parent if its
		 * 			  not in the store yet
		 * @returns {dojo.data.Item} configuration store item, or null
		 */
		getParent: function(configuration, create) {
			var getValue = dojo.hitch(this, function(name) {
				// utility function to abstract away the fact that variables
				// have to be retrieved differently from objects and store items
				if (this.isItem(configuration)) {
					return this.getValue(configuration, name);
				} else {
					return configuration[name];
				}
			});
			var id = getValue(ProcessConstants.ExtensionDTO.ID);
			if (id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length === 2) {
				// configuration is for an operation, its at the top of the tree
				return null;
			} else {
				// configuration is for a precondition or follow-up
				var parentId = id.substring(0, id.lastIndexOf(ProcessConstants.BEHAVIOR_PATH_SEPARATOR));
				var args = {};
				args[ProcessConstants.ExtensionDTO.ID] = parentId;
				args[ProcessConstants.BehaviorConfigurationDTO.CONTEXT] = getValue(ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
				args[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = getValue(ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
				args[ProcessConstants.BehaviorConfigurationDTO.ROLE_ID] = getValue(ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
				var parent = null;
				this.fetch({
					query: args,
					queryOptions: {
						deep: true
					},
					onItem: function(item) {
						parent = item;
					}
				});
				if (!parent && create === true) {
					// create parent, assume parent's parent (an operation) is
					// already in the store
					parent = this.newItem(args, {
						parent: this.getParent(args),
						attribute: ProcessConstants.ExtensionDTO.ITEMS
					});
				}
				return parent;
			}
		},
		
		/**
		 * Returns the configuration item matching the given arguments or null
		 * if it does not exist. Ignores 'unconfigured' items by default.
		 * 
		 * @param includeUnconfigured {Boolean} (optional) specify true to 
		 * 			  return an item even if it has 'configured' = false
		 */
		getConfiguration: function(operationId, context, contextId, roleId, includeUnconfigured) {
			var query = {};
			query[ProcessConstants.ExtensionDTO.ID] = operationId;
			query[ProcessConstants.BehaviorConfigurationDTO.CONTEXT] = context;
			query[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = contextId === undefined || contextId === null ? "" : contextId; //$NON-NLS-1$
			query[ProcessConstants.BehaviorConfigurationDTO.ROLE_ID] = roleId;
			if (!includeUnconfigured) {
				query[ProcessConstants.BehaviorConfigurationDTO.CONFIGURED] = true;
			}
			var configuration = null;
			this.fetch({
				query: query,
				onItem: function(item) {
					configuration = item;
				}
			});
			return configuration;
		},
		
		/**
		 * Adds a new item to the configuration store, and notifies consumers that are connected to the onNewConfiguration event.
		 */
		newConfiguration: function(operationId, context, contextId, roleId) {
			var newItem = this._internalNewConfiguration(operationId, context, contextId, roleId);
			
			// notify and return
			this.onNewConfiguration(newItem);
			return newItem;
		},
		
		/**
		 * Adds a new item to the configuration store. Internal helper function that is used to create new
		 * configurations and defer the notification to consumers (to modify the item first, for example)
		 */
		_internalNewConfiguration: function(operationId, context, contextId, roleId) {
			// if there is an item in the store with the same attributes
			// already, delete it. There could be an item that was marked for
			// deletion and has BehaviorConfigurationDTO.CONFIGURED=false
			this._deleteExistingItem(operationId, context, contextId, roleId);
			// fill in new item's fields
			var newItem = {};
			newItem[ProcessConstants.ExtensionDTO.ID] = operationId;
			newItem[ProcessConstants.BehaviorConfigurationDTO.CONTEXT] = context;
			newItem[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = contextId !== undefined ? contextId : ""; //$NON-NLS-1$
			newItem[ProcessConstants.BehaviorConfigurationDTO.ROLE_ID] = roleId;
			newItem[ProcessConstants.BehaviorConfigurationDTO.CONFIGURED] = true;
			newItem[ProcessConstants.BehaviorConfigurationDTO.FINAL_OP] = false;
			// get additional fields from the operation's extension
			var extension = this._extensionStore.fetchItemByIdentity(operationId);
			newItem[ProcessConstants.ExtensionDTO.NAME] = this._extensionStore.getValue(extension, ProcessConstants.ExtensionDTO.NAME);
			newItem[ProcessConstants.ExtensionDTO.DESCRIPTION] = this._extensionStore.getValue(extension, ProcessConstants.ExtensionDTO.DESCRIPTION);
			// create the item
			newItem = this.newItem(newItem);
			// add parent nodes for child items (preconditions and follow-ups),
			// important for the case of creating a configuration with the same
			// values as an existing one that has been deleted
			this._createBehaviorParent(newItem, ProcessConstants.PRECONDITIONS);
			this._createBehaviorParent(newItem, ProcessConstants.FOLLOWUP_ACTIONS);
			
			return newItem;
		},
		
		/**
		 * Updates one or more attribute values of an item, creating a new item
		 * and marking the old one for deletion if roleId/context/contextId are
		 * set. For these attributes, this method is the only way to change them
		 * after a configuration has been created.
		 * 
		 * @param item {dojo.data.Item} item in the store that is to be modified
		 * @param keywordArgs {Object} contains the attributes to be changed,
		 * 			  and the values they will be set to
		 */
		updateConfiguration: function(item, keywordArgs) {
			var oldItem = null;
			for (var attribute in keywordArgs) {
				var value = keywordArgs[attribute];
				if (attribute === ProcessConstants.BehaviorConfigurationDTO.CONTEXT || attribute === ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID || attribute === ProcessConstants.BehaviorConfigurationDTO.ROLE_ID) {
					// delete the existing configuration and create a new
					// one with the new role/context/contextId under the covers
					if (oldItem === null) {
						// if there is already an item in the store with the attributes you
						// are about to update to, delete it. There could be an item that
						// has BehaviorConfigurationDTO.CONFIGURED=false
						var getValue = dojo.hitch(this, function(attribute) {
							return keywordArgs[attribute] !== undefined ? keywordArgs[attribute] : this.getValue(item, attribute);
						});
						
						this._deleteExistingItem(
							getValue(ProcessConstants.ExtensionDTO.ID),
							getValue(ProcessConstants.BehaviorConfigurationDTO.CONTEXT),
							getValue(ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID),
							getValue(ProcessConstants.BehaviorConfigurationDTO.ROLE_ID)
						);
						
						// create a new configuration and mark the current item
						// for deletion (set BehaviorConfigurationDTO.CONFIGURED=false)
						var newItem = this._copyAndDelete(item);
						
						// make further updates based on the new item
						oldItem = item;
						item = newItem;
					}
				}
				this._setProtectedValue(item, attribute, value);
			}
			if (oldItem !== null) {
				//Now that the new item has role/context/contextId values that
				//are unique in the store, copy over the behaviors from the
				//original item
				this._copyBehaviors(item, oldItem, this);
			}
			this.onUpdateConfiguration(oldItem || item, item);
			return item;
		},
		
		/**
		 * If there is an item whose attributes match the arguments, permanently
		 * delete it. This is necessary when the user marks a configuration as
		 * unconfigured, then wants to create/modify another configuration with
		 * the same attributes.
		 */
		_deleteExistingItem: function(operationId, context, contextId, roleId) {
			var item = this.getConfiguration(operationId, context, contextId, roleId, true);
			if (item !== null) {
				// recursively delete the child items, and then the item itself
				var behaviorContainerItems = this.getValues(item, ProcessConstants.ExtensionDTO.ITEMS);
				for (var i = 0; i < behaviorContainerItems.length; i++) {
					var behaviorContainer = behaviorContainerItems[i];
					var behaviorItems = this.getValues(behaviorContainer, ProcessConstants.ExtensionDTO.ITEMS);
					for (var j = 0; j < behaviorItems.length; j++) {
						var behaviorItem = behaviorItems[j];
						this.deleteItem(behaviorItem);
					}
					this.deleteItem(behaviorContainer);
				}
				this.deleteItem(item);
			}
		},
		
		/**
		 * Copies all of the relevant attributes of the given item into a new
		 * item, and marks the given item for deletion.
		 * 
		 * Note that neither the preconditions nor follow-up actions, nor
		 * their containers are copied to the new item that is created and
		 * returned: it's 'item' property is empty, in other words. Callers
		 * that want the behaviors to be copied over should invoke
		 * this._copyBehaviors after calling this function, when ready.
		 * 
		 * @param existingItem {dojo.data.Item} item to copy from
		 * @returns {dojo.data.Item} new item
		 */
		_copyAndDelete: function(existingItem) {
			var newItem = {};
			var copyValue = dojo.hitch(this, function(attribute, defaultValue) {
				var values = this.getValues(existingItem, attribute);
				if (values.length > 1) {
					newItem[attribute] = values;
				} else if (values.length === 1) {
					newItem[attribute] = values[0];
				} else {
					newItem[attribute] = defaultValue;
				}
			});
			copyValue(ProcessConstants.ExtensionDTO.ID);
			copyValue(ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
			copyValue(ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
			copyValue(ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
			copyValue(ProcessConstants.BehaviorConfigurationDTO.CONFIGURED);
			copyValue(ProcessConstants.BehaviorConfigurationDTO.FINAL_OP);
			copyValue(ProcessConstants.ExtensionDTO.NAME);
			copyValue(ProcessConstants.ExtensionDTO.DESCRIPTION);
			
			newItem = this.newItem(newItem);
			
			// set the existing item to "unconfigured"
			this.setValue(existingItem, ProcessConstants.BehaviorConfigurationDTO.CONFIGURED, false);
			return newItem;
		},
		
		/**
		 * Copies the behavior specified in a given configuration to an item in the configuration store. If an item with the
		 * specified operation, context/contextId, and role exists in the store, the item is updated. The only notification
		 * that listeners will receive in that case is for the onSet event. If an item does not exist, a new item is created
		 * and listeners connected to onNewConfiguration will be notified via that function.
		 */
		copyConfiguration: function(operationId, context, contextId, roleId, configurationItemToCopy, configurationItemToCopyStore) {
			var localItem = this.getConfiguration(operationId, context, contextId, roleId);
			
			if (localItem === null) {
				//We need to create a new item
				localItem = this._internalNewConfiguration(operationId, context, contextId, roleId);
				this._copyBehaviors(localItem, configurationItemToCopy, configurationItemToCopyStore);
				this.onNewConfiguration(localItem);
			} else {
				// merge copied behaviors into existing item 
				this._copyBehaviors(localItem, configurationItemToCopy, configurationItemToCopyStore);
				this.onMergeConfiguration(localItem);
			}
			return localItem;
		},
		
		/**
		 * Helper function to create copies of all of the preconditions and
		 * follow-up actions in the configuration item being copied and add them
		 * to this store and to the precondition/follow-up action nodes in the
		 * configuration item being edited (a.k.a destItem)
		 */
		_copyBehaviors: function(destItem, sourceItem, sourceItemConfigurationStore) {
			var behaviorParents = {};
			//_createBehaviorParent will return the existing behavior container if there isn't one already, which will be the case
			//if a new item was created
			behaviorParents[ProcessConstants.PRECONDITIONS] = this._createBehaviorParent(destItem, ProcessConstants.PRECONDITIONS);
			behaviorParents[ProcessConstants.FOLLOWUP_ACTIONS] = this._createBehaviorParent(destItem, ProcessConstants.FOLLOWUP_ACTIONS);
			
			//Create copies of the behavior items, add them to the store and into local arrays
			var updatedBehaviorLists = {};
			var behaviorParentsToCopy = sourceItemConfigurationStore.getValues(sourceItem, ProcessConstants.ExtensionDTO.ITEMS);
			dojo.forEach(behaviorParentsToCopy, dojo.hitch(this, function(behaviorParentToCopy) {
				var behaviorType = sourceItemConfigurationStore.getValue(behaviorParentToCopy, ProcessConstants.ExtensionDTO.ID).split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR)[2];
				//Add copied behavior items to the list of behavior items of the appropriate type. After the following assignment
				//this list will be empty if we created a new item, and will have the existing item's current behavior otherwise
				updatedBehaviorLists[behaviorType] = this.getValues(behaviorParents[behaviorType], ProcessConstants.ExtensionDTO.ITEMS);
				var behaviorsToCopy = sourceItemConfigurationStore.getValues(behaviorParentToCopy, ProcessConstants.ExtensionDTO.ITEMS);
				dojo.forEach(behaviorsToCopy, dojo.hitch(this, function(behaviorToCopy) {
					if (sourceItemConfigurationStore.getValue(behaviorToCopy, ProcessConstants.BehaviorConfigurationDTO.CONFIGURED) === true) {
						// only copy over configured ones
						updatedBehaviorLists[behaviorType].push(this.newBehavior(behaviorParents[behaviorType], behaviorToCopy, sourceItemConfigurationStore));
					}
				}));
			}));
			
			//Add the copied behavior items to the containers in the local configuration item
			var addChildrenToParentContainer = dojo.hitch(this, function(type) {
				if (updatedBehaviorLists[type]) {
					this.setValues(behaviorParents[type], ProcessConstants.ExtensionDTO.ITEMS, updatedBehaviorLists[type]);
				}
			});
			addChildrenToParentContainer(ProcessConstants.PRECONDITIONS);
			addChildrenToParentContainer(ProcessConstants.FOLLOWUP_ACTIONS);
		},
		
		/**
		 * Creates a new precondition/follow-up based on the given item, under
		 * the given parent.
		 * 
		 * @param parent {dojo.data.Item} an operation configuration, or a
		 *            preconditions/follow-ups node, to create behavior under
		 * @param copyFromItem {dojo.data.Item} extension item if creating from
		 *            scratch, another behavior configuration item if copying an
		 *            existing configuration
		 * @param copyFromStore {dojo.data.Store} store containing copyFromItem
		 * @returns {dojo.data.Item} new behavior
		 */
		newBehavior: function(parent, copyFromItem, copyFromStore) {
			var newItem = {};
			// set defaults
			newItem[ProcessConstants.BehaviorConfigurationDTO.CONFIGURED] = true;
			newItem[ProcessConstants.BehaviorConfigurationDTO.NOT_OPTIONAL] = false;
			newItem[ProcessConstants.BehaviorConfigurationDTO.CONTENT] = null;
			// copy certain values from the parent
			var getParentValue = dojo.hitch(this, function(attribute) {
				newItem[attribute] = this.getValue(parent, attribute);
			});
			getParentValue(ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
			getParentValue(ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
			getParentValue(ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
			// copy any values the item-to-copy-from contains
			var copyValue = dojo.hitch(this, function(attribute) {
				var store = copyFromStore || this._extensionStore;
				if (store.hasAttribute(copyFromItem, attribute)) {
					newItem[attribute] = store.getValue(copyFromItem, attribute);
				}
			});
			copyValue(ProcessConstants.ExtensionDTO.ID);
			copyValue(ProcessConstants.ExtensionDTO.NAME);
			copyValue(ProcessConstants.ExtensionDTO.DESCRIPTION);
			copyValue(ProcessConstants.BehaviorConfigurationDTO.NOT_OPTIONAL);
			copyValue(ProcessConstants.BehaviorConfigurationDTO.CONTENT);
			
			if (this.getValue(parent, ProcessConstants.ExtensionDTO.ID).split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length === 2) {
				// an operation item was passed in as "parent", this isn't
				// quite right, update the "parent" reference to point to a
				// preconditions/follow-up actions item
				var type = newItem[ProcessConstants.ExtensionDTO.ID].split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR)[2];
				parent = this._createBehaviorParent(parent, type);
			}
			
			return this.newItem(newItem, {
				parent: parent,
				attribute: ProcessConstants.ExtensionDTO.ITEMS
			});
		},
		
		/**
		 * Creates a behavior parent (preconditions/follow-ups node) of the
		 * given type for the given operation configuration item, if it does not
		 * already exist. The existing or newly created parent item is returned.
		 */
		_createBehaviorParent: function(item, type) {
			var behaviorParent = {};
			behaviorParent[ProcessConstants.ExtensionDTO.ID] = this.getValue(item, ProcessConstants.ExtensionDTO.ID) + ProcessConstants.BEHAVIOR_PATH_SEPARATOR + type;
			behaviorParent[ProcessConstants.BehaviorConfigurationDTO.CONTEXT] = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
			behaviorParent[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
			behaviorParent[ProcessConstants.BehaviorConfigurationDTO.ROLE_ID] = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
			// fetch parent node if it already exists
			var parentItem = null;
			this.fetch({
				query: behaviorParent,
				queryOptions: {
					deep: true
				},
				onItem: function(item) {
					parentItem = item;
				}
			});
			if (parentItem === null) {
				// if not, create it
				parentItem = this.newItem(behaviorParent, {
					parent: item,
					attribute: ProcessConstants.ExtensionDTO.ITEMS
				});
			}
			return parentItem;
		},
		
		setValue: function(item, attribute, value) {
			this._checkProtectedAttributes(attribute, arguments);
			this.inherited(arguments);
		},
		
		setValues: function(item, attribute, values) {
			this._checkProtectedAttributes(attribute, arguments);
			this.inherited(arguments);
		},
		
		_setProtectedValue: function(item, attribute, values) {
			this._canSetProtected = true;
			this.setValue(item, attribute, values);
		},
		
		/**
		 * Prevents certain fields from being set from public methods.
		 */
		_checkProtectedAttributes: function(attribute, args) {
			if (attribute === ProcessConstants.BehaviorConfigurationDTO.CONTEXT || attribute === ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID || attribute === ProcessConstants.BehaviorConfigurationDTO.ROLE_ID) {
				// the attribute is one of the protected attributes
				if (!this._canSetProtected) {
					throw new Error("The '" + attribute + "' attribute cannot be changed using setValue(). Use updateConfiguration()."); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			delete this._canSetProtected;
		},
		
		onNewConfiguration: function(newItem) {
			// implementors can connect to this function to be notified when a
			// new item is added to the store via newConfiguration()
		},
		
		onUpdateConfiguration: function(oldItem, newItem) {
			// implementors can connect to this function to be notified when an
			// item is updated via the updateConfiguration() method
			//
			// oldItem - item in the store that was modified, or marked for
			// 		deletion if a new item was created instead
			// newItem - oldItem may have been replaced by a new item as a
			//		result of the update, so this may be a different item, or it
			//		may be the same
		},
		
		onMergeConfiguration: function(iItem) {
			// implementors can connect to this function to be notified when
			// behaviors are merged into an item via copyConfiguration()
		},
		
		/**
		 * Returns the name of the given operation configuration's category.
		 */
		getCategoryLabel: function(item) {
			return this._extensionStore.getValue(this._getCategory(item), ProcessConstants.ExtensionDTO.NAME);
		},
		
		/**
		 * Returns a URL that can be used as the src of an img for the given
		 * operation configuration's category, or null if none specified.
		 */
		getCategoryIconUrl: function(item) {
			var category = this._getCategory(item);
			if (this._extensionStore.hasAttribute(category, ProcessConstants.ExtensionDTO.ICON_URL)) {
				return this._extensionStore.getValue(category, ProcessConstants.ExtensionDTO.ICON_URL);
			}
			return null;
		},
		
		/**
		 * Returns the extension for the given operation configuration's
		 * category.
		 */
		_getCategory: function(item) {
			var id = this.getValue(item, ProcessConstants.ExtensionDTO.ID);
			return this._extensionStore.fetchItemByIdentity(id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR)[0]);
		},
		
		/**
		 * Returns the display name of the role associated with the given
		 * configuration.
		 * 
		 * @param item {dojo.data.Item} operation configuration
		 * @returns {String} label
		 */
		getRoleLabel: function(item) {
			var roleId = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
			var role = this.rolesStore.fetchRoleById(roleId);
			if (role !== null) {
				return this.rolesStore.getRoleLabel(role);
			}
			return null;
		},
		
		/**
		 * Returns the name of the iteration type/timeline/iteration that the
		 * given operation is configured for. 
		 */
		getContextLabel: function(item) {
			var context = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
			var contextId = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
			if (contextId === undefined || contextId === null || dojo.string.trim(contextId).length === 0) {
				return this.messages.OperationBehaviorConfigurationStore_0;
			} else if (!this.contextStore && this._processProviderUrl !== undefined) {
				// trying to get the label of an iteration type in a remote
				// provider (they don't support dev lines/iterations), just
				// return the ID for now
				return jazz.util.html.escape(contextId);
			}
			var contextItem = this._getContextItem(context, contextId);
			if (!contextItem) {
				// this configuration is invalid
				return null;
			}
			var label = this.contextStore.getValue(contextItem, "name"); //$NON-NLS-1$
			if (label === undefined || label === null || dojo.string.trim(label).length === 0) {
				label = this.contextStore.getValue(contextItem, "id"); //$NON-NLS-1$
			}
			return jazz.util.html.escape(label);
		},
		
		/**
		 * Return whether the item's time period is an iteration type,
		 * iteration, or timeline that is archived.
		 */
		usesArchivedContext: function(item) {
			//We will not have a context store in the case of copying a
			//configuration from a remote provider. We will consider the
			//iteration type un-archived for now, since the user will have
			//to select a different context to use
			if (this.contextStore) {
				var context = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
				var contextId = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
				if (contextId !== undefined && contextId !== null && dojo.string.trim(contextId).length !== 0) {
					var contextItem = this._getContextItem(context, contextId);
					if (contextItem) {
						return this.contextStore.getValue(contextItem, "archived"); //$NON-NLS-1$
					}
				}
			}
			return false;
		},
		
		/**
		 * Returns false if the given configuration is for a team area and is
		 * scoped to an iteration which is not a child of the team's current
		 * development line, return true otherwise.
		 */
		usesCurrentDevelopmentLine: function(item) {
			if (this.contextStore) {
				var context = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
				var contextId = this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
				var currentDevLine = this.contextStore.getCurrentDevelopmentLineItemId();
				if (context === ProcessConstants.BehaviorContext.ITERATION && currentDevLine !== undefined) {
					var contextItem = this._getContextItem(context, contextId);
					if (contextItem) {
						while (!this.contextStore.hasAttribute(contextItem, "parentItemId")) { //$NON-NLS-1$
							// only top-level iterations store their development
							// line's id
							this.contextStore.fetchItemByIdentity({
								identity: this.contextStore.getValue(contextItem, "parent"), //$NON-NLS-1$
								onItem: function(item) {
									contextItem = item;
								}
							});
						}
						return this.contextStore.getValue(contextItem, "parentItemId") === currentDevLine; //$NON-NLS-1$
					}
				}
			}
			return true;
		},
		
		/**
		 * Helper to fetch the item from the context store that matches a
		 * context and contextId.
		 */
		_getContextItem: function(context, contextId) {
			var contextItem = null;
			this.contextStore.fetch({
				query: {
					enabledContext : context,
					contextId : contextId
				},
				queryOptions: {
					deep: true
				},
				onItem: function(item) {
					contextItem = item;
				}
			});
			return contextItem;
		},
		
		/**
		 * Overridden to filter out unconfigured behaviors.
		 */
		getBehavior: function(operation, type) {
			return dojo.filter(this.inherited(arguments), dojo.hitch(this, function(item) {
				return this.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONFIGURED) !== false;
			}));
		},
		
		/**
		 * TODO: remove after workitem 232520 is done, contextStore will only 
		 * need to be passed in in the constructor.
		 */
		setContextStore: function(store) {
			this.contextStore = store;
			this.connect(store, "onSet", "_onSetContext"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_onSetContext: function(item, attribute, oldValue, newValue) {
			if (attribute === "contextId") { //$NON-NLS-1$
				var oldParent = oldValue.substring(0, oldValue.lastIndexOf(ProcessConstants.BEHAVIOR_PATH_SEPARATOR));
				var newParent = newValue.substring(0, newValue.lastIndexOf(ProcessConstants.BEHAVIOR_PATH_SEPARATOR));
				if (oldParent !== newParent) {
					// this iteration's parent changed (it was moved), update
					// any configurations using it
					query = {};
					query[ProcessConstants.BehaviorConfigurationDTO.CONTEXT] = this.contextStore.getValue(item, "enabledContext"); //$NON-NLS-1$
					query[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = oldValue;
					this.fetch({
						query: query,
						queryOptions: {
							deep: true
						},
						scope: this,
						onItem: function(configuration) {
							this._setProtectedValue(configuration, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID, newValue);
						}
					});
				}
			}
		},
		
		/**
		 * Call this to empty the store, then call initialize() again to reload.
		 */
		uninitialize: function() {
			if (this._isInitialized) {
				this.save();
				this.data = { items: [] };
				this.close();
				this._isInitialized = false;
			}
		}
		
	});
	
})();