/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCreateConfigurationWizard"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.Wizard"); //$NON-NLS-1$
	dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizard"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectOperationWidget"); //$NON-NLS-1$
	
	/**
	 * Invoking the constructor will present a wizard to create a new operation
	 * behavior configuration from scratch. Three pages are presented in
	 * sequence, to...
	 * 
	 * 1. Select an operation
	 * 2. Select a role. The widget is presented, and then a message stating
	 *    that the user may click Finish if desired to have the configuration
	 *    always apply.
	 * 3. Select a time period
	 * 
	 * If the user selects values that match an existing operation behavior
	 * configuration, a warning message is displayed and the user is not able to
	 * click Finish until unique values are selected.
	 * 
	 * When Finish or Cancel is clicked, the wizard is closed and all child
	 * widgets are destroyed recursively.
	 * 
	 * The argument to the constructor must be an object with the following
	 * properties:
	 * 
	 * - model	[required]  An object which provides information about the
	 *                      general process model that is not operation-behavior
	 *                      specific. The following fields are required:
	 * 
	 * 		- rolesStore			[required]
	 *          A dojo.data.ItemFileReadStore with roles available to select 
	 *          from, with fields id, name, and description
	 * 
	 * 		- contextStore			[required]
	 *          A dojo.data.ItemFileReadStore for iteration types and timelines/
	 *          iterations for the process area
	 * 
	 * 		- configurationStore	[required]
	 *          A dojo.data.ItemFileReadStore which contains operation
	 *          configurations defined
	 * 
	 * 		- extensionStore		[required]
	 *          A dojo.data.ItemFileReadStore which contains categories and
	 *          operations for which operation behavior can be configured
	 * 
	 * 		- editorContext			[required]
	 *          The editor context that the widget is being used in: a value in
	 *          ProcessConstants.EditorContext
	 * 
	 *  	- developmentLineItemId	[optional]
	 *          Required if editorContext is ProcessConstants.EditorContext.TEAM
	 *          A string that is used offer child iterations of the team area's
	 *          selected development line
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCreateConfigurationWizard", //$NON-NLS-1$
		[com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizard], {
		
		createSteps: function(model) {
			var onFocusCallback = dojo.hitch(this, function() {
				this._wizardPageSelectionChanged();
			});
			
			var hasValidationProblemCallback = dojo.hitch(this, function() {
				return this._isShowingValidationError;
			});
			
			this._whatStep = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWhatStep({
				model: model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues) {
					this._selectedOperationId = selectedValues.operationId;
					this._selectedOperationContext = selectedValues.operationContext;
					
					this._whoStep.notifyOperationSelectionChanged(selectedValues);
					
					this._wizardPageSelectionChanged();
				}),
				onFocusCallback: onFocusCallback
			});
			
			this._whoStep = this._newWhoStep({
				model: model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues) {
					this._selectedRoleId = selectedValues.roleId;
					
					this._wizardPageSelectionChanged();
				}),
				onFocusCallback: onFocusCallback,
				hasValidationProblemCallback: hasValidationProblemCallback
			});
			
			this._whenStep = this._newWhenStep({
				model: model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues) {
					this._selectedContext = selectedValues.context;
					this._selectedContextId = selectedValues.contextId;
					
					this._wizardPageSelectionChanged();
				}),
				onFocusCallback: onFocusCallback,
				hasValidationProblemCallback: hasValidationProblemCallback
			});
			
			this._whoStep.whenStepCanFinish = dojo.hitch(this._whenStep, this._whenStep.canFinish);
			
			return [this._whatStep, this._whoStep, this._whenStep];
		},
		
		getPrimaryTitle: function() {
			return this.messages.OperationBehaviorCreateConfigurationWizard_0;
		},
		
		/**
		 * Helper function to change the flag that tracks whether an error is being shown, and notify the current
		 * step, so they can change their presentation if necessary.
		 */
		_changeValidationErrorState: function(showError) {
			this._isShowingValidationError = showError;
			
			var currentStep = this.wizard.contentWidget.getSelectedStep();
			
			currentStep.triggerButtonStateUpdate();
			
			if (currentStep.notifyValidationErrorChanged && dojo.isFunction(currentStep.notifyValidationErrorChanged)) {
				currentStep.notifyValidationErrorChanged(showError);
			}
		},
		
		/**
		 * Hides the validation message, updates state, and notifies current step.
		 */
		_clearValidationError: function() {
			this._changeValidationErrorState(false);
			this._messageBox.hideMessage();
		},
		
		/**
		 * Shows a validation message, (if desired), updates state, and notifies
		 * current step. If argument "silentError" is false, a message is not
		 * displayed, but the rest of the wizard will behave as if there is
		 * a validation problem (i.e. not allow Finish).
		 */
		_addValidationError: function(errorMessage, errorType, silentError) {
			if (!this._isShowingValidationError) {
				this._changeValidationErrorState(true);
			}
				
			if (silentError) {
				this._messageBox.hideMessage();
			} else {
				this._messageBox.showMessage({
					type: errorType,
					summary: errorMessage
				});
			}
		},
		
		/**
		 * Callback when the user makes a change to what's selected in any
		 * wizard page. If the user would be able to finish, perform validation
		 * and prevent finishing if there's a problem.
		 */
		_wizardPageSelectionChanged: function() {
			if (this.wizard === undefined) {
				//A page reported a change (for an initial value) before we've
				//created the wizard. We don't need to validate yet
				return;
			}
			
			var currentStep = this.wizard.contentWidget.getSelectedStep();
			if (currentStep.canFinish() || this._isShowingValidationError) {
				//See if there is an existing configuration that matches the
				//selected options
				var selectedValues = this._getSelectedValues();
				
				var existing = this.configurationStore.getConfiguration(selectedValues.operationId, selectedValues.context, selectedValues.contextId, selectedValues.roleId);
				
				if (existing !== null) {
					//There is a configuration with the same properties as the
					//user selected. Prohibit finish, and display an error
					//if either:
					//- the current step is the 'When' step
					//- a project operation has been selected (in which case)
					//  there is no 'When' step
					//We don't want to show a validation problem about
					//duplicating an existing item until the user has had a
					//chance to specify all values, but adding a "silent"
					//error will prohibit clicking Finish, which is the
					//behavior we want.
					this._addValidationError(this._getConfigurationDuplicationMessage(selectedValues), jazz.ui.MessageBox.STATUS_TYPES.WARNING, currentStep !== this._whenStep && !this._whoStep._projectOperationSelected);
				} else if (this._isShowingValidationError) {
					//There is not a configuration with the same properties as
					//the user selected. Clear the error and allow finish.
					this._clearValidationError();
				}
			}
		},
		
		/**
		 * Report the selections made by the user to the editor.
		 */
		onFinish: function() {
			var selectedValues = this._getSelectedValues();			
			this.configurationStore.newConfiguration(selectedValues.operationId, selectedValues.context, selectedValues.contextId, selectedValues.roleId);
			
			this.wizard.close();
			this._uninitialize();
		}
	});
	
	/**
	 * Wizard step where user selects the operation they are going to add
	 * preconditions and follow-up actions to.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWhatStep", //$NON-NLS-1$
			[jazz.ui.wizard.WizardStep, dijit._Widget], {
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stepTitle = this.messages.OperationBehaviorWizard_0;
		},
		
		postCreate: function() {
			this.domNode = dojo.create("div", {}); //$NON-NLS-1$
			
			var contents = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			dojo.addClass(contents, "wizardPageContents"); //$NON-NLS-1$
			
			this._operationBehaviorSelectOperationWidget = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectOperationWidget({
				model: this.model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues) {
					//See if an operation was selected, so we know whether to
					//enable Next...
					this._selectedOperationId = selectedValues.operationId;
					
					//...and then tell the creator of this step that the
					//selection changed  
					this.selectionChangedCallback(selectedValues);
					
					this.triggerButtonStateUpdate();
				})
			});
			
			contents.appendChild(this._operationBehaviorSelectOperationWidget.domNode);
		},
		
		/**
		 * Allow the user to advance if they have selected an operation in the tree
		 */
		canFlipToNextPage: function() {
			return this._selectedOperationId !== undefined;
		},
		
		focusStep: function() {
			this.onFocusCallback();
		},
		
		uninitialize: function(){
			this._operationBehaviorSelectOperationWidget.destroyRecursive();
		}
	});
})();
