/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditor"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.MenuButton"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.TwoStateButton"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTable"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorInlineEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCustomEditorManager"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCreateConfigurationWizard"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCopyConfigurationWizard"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorExtensionStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorConfigurationStore"); //$NON-NLS-1$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	var CATEGORY_COLUMN = 0;
	var ROLE_COLUMN = 2;
	var CONTEXT_COLUMN = 3;
	var SUMMARY_COLUMN = 4;
	var ACTION_COLUMN = 5;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditor", com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTable, { //$NON-NLS-1$
		
		constructor: function(args) {
			this.extensionStore = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorExtensionStore({
				processAreaItemId: args.editor.itemId,
				owningApplicationKey: args.owningApplicationKey
			});
			this.configurationStore = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorConfigurationStore({
				processAreaItemId: args.editor.itemId,
				processAreaContext: args.editor.context,
				owningApplicationKey: args.owningApplicationKey,
				extensionStore: this.extensionStore,
				rolesStore: args.rolesStore,
				contextStore: args.contextStore
			});
			
			// manages the life cycle of the custom editors for any
			// preconditions/follow-ups that have them
			this.customEditorManager = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCustomEditorManager({
				processAreaEditor: args.editor,
				extensionStore: this.extensionStore,
				configurationStore: this.configurationStore,
				configurationDataModel: args.configurationDataModel,
				rolesStore: args.rolesStore
			});
			
			// add Actions column
			this.columns.push({ label: this.messages.OperationBehaviorEditor_4, sortable: false, className: ProcessConstants.CSSClass.ACTION_COLUMN });
		},
		
		postCreate: function() {
			this.inherited(arguments);
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-OperationBehaviorEditor toolbar-table"); //$NON-NLS-1$
			this._createHeaderActions();
			this.refresh();
		},
		
		/**
		 * Will be called by the process editor to add our actions to the header
		 * above our editor
		 */
		getHeaderActionArea: function() {
			return this._headerActionArea;
		},
		
		/**
		 * Can be called by the owning editor to tell this tab to refresh.
		 */
		refresh: function() {
			if (this._loading) {
				// already in the middle of a refresh...
				return;
			}
			this._startLoading();
			this._clearState();
			
			this.editor.addSaveParticipant(this, "com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditor"); //$NON-NLS-1$
			
			// rolesStore reference no longer valid after refresh right now...
			this.rolesStore = this.editor.getRolesStore();
			this.configurationStore.rolesStore = this.rolesStore;
			this.customEditorManager.rolesStore = this.rolesStore;
			
			// contextStore reference no longer valid after refresh right now...
			this.contextStore = this.editor._getProcessAreaStore();
			this.configurationStore.setContextStore(this.contextStore);
			
			this.extensionStore.initialize(dojo.hitch(this, function() {
				// now that extensions are loaded, load configurations
				this.configurationStore.initialize(dojo.hitch(this, this._configurationsLoaded), dojo.hitch(this, this._handleConfigurationFetchError));
			}), dojo.hitch(this, this._handleExtensionsFetchError));
		},
		
		/**
		 * Can be called by the owning editor to tell this tab when it is
		 * switched to again by the user.
		 */
		activateTab: function() {
			if (this._loading) {
				// tab started loading while it was not the active tab, show the
				// loading message now
				this._startLoading();
			}
			for (var i in this._editorRows) {
				if (this._editorRows[i].parentNode === this.tbody) {
					// editor row is showing (since it has a parent in document)
					this._editorRows[i]._editor.aboutToShowEditor();
				}
			}
			if (this._mostRecentError !== undefined) {
				this.editor._displayLongErrorMessage(this._mostRecentError);
			}
		},
		
		/**
		 * Called by the owning editor when switching away from this tab.
		 */
		deactivateTab: function() {
			//we dont want to hinder tab deactivation because of an error in a
			//contributed widget
			UIHelper.runSafe(dojo.hitch(this, function() {
				this.customEditorManager.saveContent();
			}));
			return true;
		},
		
		getSummaryTextForNoBehaviorSpecified: function(preconditionsPossible, followupsPossible) {
			if (preconditionsPossible && !followupsPossible) {
				return this.messages.OperationBehaviorEditor_15;
			} else if (!preconditionsPossible && followupsPossible) {
				return this.messages.OperationBehaviorEditor_16;
			} else {
				return this.messages.OperationBehaviorEditor_17;
			}
		},
		
		_clearState: function() {
			this.clearTable();
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditor"); //$NON-NLS-1$
			this.configurationStore.uninitialize();
			
			if (this._editorConnects) {
				dojo.forEach(this._editorConnects, dojo.disconnect);
				delete this._editorConnects;
			}
			
			if (this._actionToolbars) {
				for (var i = 0; i < this._actionToolbars.length; i++) {
					this._actionToolbars[i].destroyRecursive();
				}
			}
			this._actionToolbars = [];
			
			if (this._editorRows) {
				for (var i in this._editorRows) {
					this._editorRows[i]._editor.destroyRecursive();
				}
			}
			this._editorRows = {};
			
			this.customEditorManager.destroyEditors();
			
			this._errors = [];
			delete this._mostRecentError;
		},
		
		_startLoading: function() {
			this._loading = true;
			if (this.editor.isTheActiveTab(ProcessConstants.TabEnum.OPERATION_BEHAVIOR)) {
				this.editor.startLoading(this.messages.OperationBehaviorEditor_6);
			}
		},
		
		_stopLoading: function() {
			delete this._loading;
			if (this.editor.isTheActiveTab(ProcessConstants.TabEnum.OPERATION_BEHAVIOR)) {
				this.editor.stopLoading();
			}
		},
		
		_handleExtensionsFetchError: function(error) {
			this.editor.handleServerResponseError(error, this.messages.OperationBehaviorEditor_7);
			this._stopLoading();
		},
		
		/**
		 * Called once all operation behavior configurations have come back from
		 * the server and have been fully processed.
		 */
		_configurationsLoaded: function() {
			this._editorConnects = [
				dojo.connect(this.configurationStore, "onNewConfiguration", this, "_onNewConfiguration"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.configurationStore, "onMergeConfiguration", this, "_onMergeConfiguration"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.configurationStore, "onUpdateConfiguration", this, "_onUpdateConfiguration"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.configurationStore, "onSet", this, "_onSetConfiguration"), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.connect(this.configurationStore, "onNew", this, "_onNewBehavior") //$NON-NLS-1$ //$NON-NLS-2$
			];
			this.populateTable();
			this._stopLoading();
		},
		
		_handleConfigurationFetchError: function(error) {
			this.editor.handleServerResponseError(error, this.messages.OperationBehaviorEditor_8);
			this._stopLoading();
		},
		
		/**
		 * Overridden to add an additional column for the Actions toolbar.
		 */
		getRowInfo: function(operation) {
			var info = this.inherited(arguments);
			if (info) {
				info.push(this._getActions(operation, info));
			}
			return info;
		},
		
		/**
		 * Returns a jazz.ui.Toolbar with all of the available actions for the 
		 * given operation configuration.
		 */
		_getActions: function(operation, rowInfo) {
			var self = this;
			var toolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();
			dojo.addClass(toolbar.domNode, "actionsToolbar"); //$NON-NLS-1$
			// utility function for buttons to get their row
			var getParentRow = function(widget) {
				var row = widget.domNode;
				while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
					row = row.parentNode;
				}
				return row;
			};
			// edit button
			toolbar._editButton = new jazz.ui.toolbar.TwoStateButton({
				title: this.messages.OperationBehaviorEditor_13,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/edit.gif").uri, //$NON-NLS-1$ //$NON-NLS-2$
				onClick: function() {
					var row = getParentRow(this);
					self._toggleEditing(row._rowInfo._item, row, this.pressed);
				}
			});
			toolbar.selectRowOnFocus(toolbar._editButton.focusNode);
			toolbar.addWidget(toolbar._editButton);
			
			// delete button
			var deleteButton = new jazz.ui.toolbar.Button({
				title: this.messages.OperationBehaviorEditor_14,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/delete_co.gif").uri, //$NON-NLS-1$ //$NON-NLS-2$
				onClick: function() {
					var row = getParentRow(this);
					var editorRow = self._editorRows[self.configurationStore.getIdentity(row._rowInfo._item)];
					if (!editorRow || editorRow.parentNode !== self.tbody || confirm(dojo.string.substitute(self.messages.OperationBehaviorEditor_3, [self.configurationStore.getValue(row._rowInfo._item, ProcessConstants.ExtensionDTO.NAME)]))) {
						// clear the corresponding editor row if its there
						self._toggleEditing(row._rowInfo._item, row, false);
						if (editorRow) {
							editorRow._editor.destroyRecursive();
							delete editorRow;
						}
						// delete the operation configuration and its row
						self._delete(row._rowInfo._item, row);
					}
				}
			});
			toolbar.selectRowOnFocus(deleteButton.focusNode);
			toolbar.addWidget(deleteButton);
			
			toolbar.selectRowOnFocus(rowInfo[CATEGORY_COLUMN])
			
			this._actionToolbars.push(toolbar);
			return toolbar;
		},
		
		/**
		 * Toggles whether or not the inline behavior editor is showing for the
		 * given item.
		 */
		_toggleEditing: function(item, row, editing) {
			var editorRow = this._editorRows[this.configurationStore.getIdentity(item)];
			if (editing) {
				var newEditor = false;
				if (!editorRow) {
					// an inline editor needs to be created
					// create the table row to hold it
					editorRow = document.createElement("tr"); //$NON-NLS-1$
					dojo.addClass(editorRow, "editorRow"); //$NON-NLS-1$
					var td = document.createElement("td"); //$NON-NLS-1$
					td.setAttribute("colspan", this.rowInfo[0].length); //$NON-NLS-1$
					editorRow.appendChild(td);
					// create the editor
					editorRow._editor = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorInlineEditor({
						item: item,
						editor: this,
						extensionStore: this.extensionStore,
						configurationStore: this.configurationStore,
						rolesStore: this.rolesStore,
						contextStore: this.contextStore,
						customEditorManager: this.customEditorManager,
						onClose: dojo.hitch(this, function() {
							editorRow.parentNode.removeChild(editorRow);
							delete this._editorRows[this.configurationStore.getIdentity(item)];
							row._toolbar._editButton.set("pressed", false); //$NON-NLS-1$
						})
					});
					td.appendChild(editorRow._editor.domNode);
					this._editorRows[this.configurationStore.getIdentity(item)] = editorRow;
					newEditor = true;
				}
				// add the editor to the document
				if (row.nextSibling) {
					this.tbody.insertBefore(editorRow, row.nextSibling);
				} else {
					this.tbody.appendChild(editorRow);
				}
				if (newEditor) {
					editorRow._editor.startup();
				}
			} else if (editorRow && this.tbody === editorRow.parentNode) {
				// toggle edit off
				this.tbody.removeChild(editorRow);
			}
			// make sure the button state is accurate
			row._toolbar._editButton.set("pressed", editing); //$NON-NLS-1$
			
			//Unfocus the edit button, so that the 'selected' class is
			//removed from the row after the editor is closed
			row._toolbar._editButton.focusNode.blur();
		},
		
		/**
		 * Removes the given item's row from the table, closing its editor if it
		 * was open.
		 */
		_delete: function(item, row) {
			// remove this row from rowInfo
			var index = this.pageNum * this.pageSize;
			for (var i = 0; i < row.parentNode.childNodes.length; i++) {
				var tr = row.parentNode.childNodes[i];
				if (tr === row) {
					break;
				}
				if (!tr._editor) { // ignore editor rows
					index++;
				}
			}
			var rowInfo = this.rowInfo.splice(index, 1)[0];
			
			// remove this row from allRowInfo, if necessary
			// allRowInfo is used when we're filtering based on some text, so
			// all items can be shown when the filter is removed
			if (this.allRowInfo !== undefined && this.rowInfo !== this.allRowInfo) {
				for (var index = 0; index < this.allRowInfo.length; index++) {
					if (this.allRowInfo[index] === rowInfo) {
						this.allRowInfo.splice(index, 1);
						break;
					}
				}
			}
			
			// remove this row from the table
			row.parentNode.removeChild(row);
			// add the first row on the next page if its there
			var lastOnPage = (this.pageNum + 1) * this.pageSize - 1;
			if (lastOnPage < this.rowInfo.length) {
				this.addRow(null, this.rowInfo[lastOnPage]);
			}
			// update the pager and extra info
			this.numConfigurationsChanged();
			
			// update the store
			this.configurationStore.setValue(item, ProcessConstants.BehaviorConfigurationDTO.CONFIGURED, false);
			this.editor.setDirty(this.editor.changesPending());
		},
		
		/**
		 * Overridden to reset the edit button of the toolbar of any rows that
		 * are currently showing, before clearing the page and re-populating it.
		 * If the editor for that row was showing, then the button will be in
		 * the pressed state, and next time that row shows we don't want it to
		 * still be in that state.
		 */
		showPage: function(pageNum) {
			dojo.forEach(this.tbody.childNodes, function(row) {
				if (row._toolbar) {
					row._toolbar._editButton.set("pressed", false); //$NON-NLS-1$
				}
			});
			this.inherited(arguments);
		},
		
		/**
		 * Overridden from StyledTableItemList to do extra processing on the row
		 * after its created.
		 */
		addRow: function(actionHref, columnInfo) {
			var row = this.inherited(arguments);
			row._toolbar = columnInfo[ACTION_COLUMN];
			if (columnInfo.isNew) {
				dojo.addClass(row, "newRow"); //$NON-NLS-1$
			}
			return row;
		},
		
		/**
		 * Create a section that will be displayed to the right-side of the
		 * header area, and add appropriate controls, (eg. a button to create a
		 * configuration).
		 */
		_createHeaderActions: function() {
			//This node will be placed in the "header action area" of the
			//ProcessAreaEditorPageHeaderSection. It will be retrieved by a call
			//to getHeaderActionArea() 
			this._headerActionArea = dojo.create("div", {}); //$NON-NLS-1$
			
			this._addConfigurationMenuButton = new jazz.ui.toolbar.MenuButton({
				title: this.messages.OperationBehaviorEditor_5,
				label : this.messages.OperationBehaviorEditor_5,
				menuProvider: dojo.hitch(this, this._addConfigurationMenuProvider),				
				imgSrc :  "com.ibm.team.process.web/ui/internal/admin/projects/images/new-config_co.gif", //$NON-NLS-1$				
				align: "right" //$NON-NLS-1$
			}).placeAt(this._headerActionArea, "only"); //$NON-NLS-1$
		},
		
		/**
		 * Callback for the 'menuProvider' for the drop-down button to add an
		 * operation behavior configuration. Returns a menu with items that
		 * launch the appropriate wizards to create new/copy existing
		 * configurations.
		 */		
		_addConfigurationMenuProvider: function() {
			this._createAddConfigurationDropDownMenu();
			return this._addConfigurationMenu;
		},
		
		/**
		 * If this function has not already been invoked, creates a menu with
		 * items to either create a new operation behavior configuration or copy
		 * an existing one.
		 */
		_createAddConfigurationDropDownMenu: function (){
			if (this._addConfigurationMenu === undefined) {
				var menu = new jazz.ui.Menu();
				dojo.addClass(menu.domNode, "com-ibm-team-process-web-ui-internal-OperationBehaviorEditor-addConfigurationDropDown"); //$NON-NLS-1$
				
				//Item to create a new configuration
				menu.addChild(new dijit.MenuItem({
					label: this.messages.OperationBehaviorEditor_9,
					title: this.messages.OperationBehaviorEditor_9,
					iconClass: 'createConfigurationIcon', //$NON-NLS-1$
					onClick: dojo.hitch(this, this._createNewConfigurationWizard)
				}));
				
				//Item to copy an existing configuration				
				menu.addChild(new dijit.MenuItem({
					label: this.messages.OperationBehaviorEditor_10,
					title: this.messages.OperationBehaviorEditor_10,
					iconClass: 'copyConfigurationIcon', //$NON-NLS-1$
					onClick: dojo.hitch(this, this._createCopyConfigurationWizard)
				}));
				
				menu.startup();
				
				this._addConfigurationMenu = menu;
			}
		},
		
		_createNewConfigurationWizard: function() {
			var model = {
				rolesStore: this.rolesStore,
				contextStore: this.contextStore,
				configurationStore: this.configurationStore,
				extensionStore: this.extensionStore,
				editorContext: this.editor.context
			};
			if (this.editor.isTeamArea) {
				model.developmentLineItemId = this.editor.getSelectedDevelopmentLineItemId();
			}
			new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCreateConfigurationWizard({
				model: model
			});
		},
		
		_createCopyConfigurationWizard: function() {
			var model = {
				rolesStore: this.rolesStore,
				contextStore: this.contextStore,
				configurationStore: this.configurationStore,
				extensionStore: this.extensionStore,
				editorContext: this.editor.context,
				owningApplicationKey: this.owningApplicationKey,
				processAreaItemId: this.editor.itemId
			};
			if (this.editor.isTeamArea) {
				model.developmentLineItemId = this.editor.getSelectedDevelopmentLineItemId();
			}
			new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCopyConfigurationWizard({
				model: model
			});
		},
		
		// =====================================================================
		// React to events from the stores
		
		/**
		 * When a new item is added the configuration store, adds a new row for
		 * it at the top of the table, and expands its editor.
		 */
		_onNewConfiguration: function(newItem) {
			this.editor.setDirty(this.editor.changesPending());
			// create info needed to create a row for this item
			var info = this.getRowInfo(newItem);
			info.isNew = true;
			this._promoteRow(info);
			// update the pager and extra info
			this.numConfigurationsChanged();
		},
		
		/**
		 * Move the merged item's row to the top of the table and expand it.
		 */
		_onMergeConfiguration: function(item) {
			this.editor.setDirty(this.editor.changesPending());
			var rowInfo = this.findRowInfo(item);
			this._promoteRow(rowInfo);
		},
		
		/**
		 * Updates the UI based on changes made to a configuration item. 
		 */
		_onUpdateConfiguration: function(oldItem, newItem) {
			this.editor.setDirty(this.editor.changesPending());
			var editorRowForItem = this._findEditorRow(oldItem);
			var rowForItem = editorRowForItem.previousSibling;
			if (this.configurationStore.getIdentity(oldItem) !== this.configurationStore.getIdentity(newItem)) {
				// a new item was created for this update, update _editorRows
				delete this._editorRows[this.configurationStore.getIdentity(oldItem)];
				this._editorRows[this.configurationStore.getIdentity(newItem)] = editorRowForItem;
				// update references to the old item
				rowForItem._rowInfo._item = newItem;
			}
			// update the information in the table row that might have changed
			rowForItem._rowInfo[ROLE_COLUMN] = this.configurationStore.getRoleLabel(newItem);
			rowForItem.childNodes[ROLE_COLUMN].innerHTML = jazz.util.html.escape(rowForItem._rowInfo[ROLE_COLUMN]);
			rowForItem._rowInfo[CONTEXT_COLUMN] = this.configurationStore.getContextLabel(newItem);
			rowForItem.childNodes[CONTEXT_COLUMN].innerHTML = jazz.util.html.escape(rowForItem._rowInfo[CONTEXT_COLUMN]);
			
			// if a new item was created for the update, create a new summary,
			// since the existing one will be pointing to the old item
			if (oldItem !== newItem) {
				this._updateSummary(newItem);
			}
		},
		
		/**
		 * Triggered when setValue(s) is called on the configuration store, to
		 * check if the summary of a configuration needs updating.
		 */
		_onSetConfiguration: function(item, attribute, oldValue, newValue) {
			var isPlaceholder = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID).split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length === 3;
			var isBehavior = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID).split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length === 4;
			if (isPlaceholder && attribute === ProcessConstants.ExtensionDTO.ITEMS || 
					isBehavior && (attribute === ProcessConstants.ExtensionDTO.NAME || attribute === ProcessConstants.BehaviorConfigurationDTO.CONFIGURED)) {
				// if the children of a placeholder (preconditions/follow-ups
				// node) changed, or other changes were made to a
				// precondition/follow-up, the summary might need updating
				if (isPlaceholder) {
					item = this.configurationStore.getParent(item);
				} else if (isBehavior) {
					item = this.configurationStore.getParent(this.configurationStore.getParent(item));
				}
				this._updateSummary(item);
				if (isBehavior && attribute === ProcessConstants.ExtensionDTO.NAME) {
					// currently the only time we need to validate
					this._validate(item);
				}
			}
			// set dirty after validating
			this.editor.setDirty(this.editor.changesPending());
		},
		
		/**
		 * Dojo stores don't send out an "onSet" notification when the children
		 * of a parent change for a new child, so listen to "onNew"
		 * notifications to detect the case when a new precondition/follow-up
		 * is added to an operation's configuration, and update its summary.
		 */
		_onNewBehavior: function(newItem, parentInfo) {
			if (parentInfo && parentInfo.item && this.configurationStore.getValue(parentInfo.item, ProcessConstants.ExtensionDTO.ID).split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length === 3) {
				// a new item was just created as a child of a placeholder
				// (preconditions/follow-ups node)
				var operation = this.configurationStore.getParent(parentInfo.item);
				this._updateSummary(operation);
			}
		},
		
		onCurrentDevelopmentLineChange: function() {
			this.inherited(arguments);
			// show a loading message the next time the tab is opened if the
			// context store is still re-initializing
			if (this._loadingDevLineInterval) {
				clearInterval(this._loadingDevLineInterval);
			} else {
				this._startLoading();
			}
			this._loadingDevLineInterval = setInterval(dojo.hitch(this, function() {
				if (this.contextStore.isInitialized()) {
					clearInterval(this._loadingDevLineInterval);
					this._stopLoading();
				}
			}), 100);
		},
		
		// =====================================================================
		
		/**
		 * Moves the given rowInfo to the top of the rowInfo collections if its
		 * not already there (adds it if necessary), displays its row at the top
		 * of the table, and expands its editor.
		 */
		_promoteRow: function(rowInfo) {
			// move rowInfo to the top of the collection(s)
			var moveToTop = function(collection) {
				if (!dojo.some(collection, function(info, index) {
					if (info === rowInfo) {
						if (index > 0) { // not already at the top
							collection.splice(0, 0, collection.splice(index, 1)[0]);
						}
						return true;
					}
				})) {
					// add it to the top if not already in the collection
					collection.splice(0, 0, rowInfo);
				}
			};
			moveToTop(this.rowInfo);
			if (this.allRowInfo !== undefined && this.allRowInfo !== this.rowInfo) {
				// move row to the top of allRowInfos, which is used when we are
				// filtering
				moveToTop(this.allRowInfo);
			}
			// show the new row
			if (this.pageNum === 0) {
				var existingRow = this.findRow(rowInfo._item);
				if (existingRow) {
					if (existingRow === this.tbody.firstChild) {
						// the row is already the top one, no need to move
					} else {
						// move row from where it is to the top
						this.tbody.insertBefore(existingRow, this.tbody.firstChild);
					}
				} else {
					// not already showing, add new row to the top of the page
					var newRow = this.addRow(null, this.rowInfo[0]);
					this.tbody.insertBefore(newRow, this.tbody.firstChild);
					if (this.rowInfo.length > this.pageSize) {
						if (this.tbody.lastChild._editor) {
							// remove last row's editor if it was open
							this.tbody.removeChild(this.tbody.lastChild);
						}
						// remove the last row if it should now be on next page
						this.tbody.removeChild(this.tbody.lastChild);
					}
				}
			} else {
				// switch to first page, new row will be at the top
				this.showPage(0);
			}
			// expand the editor
			this._toggleEditing(rowInfo._item, this.tbody.firstChild, true);
		},
		
		_findEditorRow: function(item) {
			for (var id in this._editorRows) {
				var row = this._editorRows[id];
				if (item && this.configurationStore.getIdentity(item) == id) {
					return row;
				}
			}
			return null;
		},
		
		/**
		 * Create a new summary for the given item's row in the table, so that
		 * its text and hover content will be up-to-date.
		 */
		_updateSummary: function(item) {
			// update the appropriate row info, this will cover updating the
			// summary of rows that are not being displayed currently
			var rowInfo = this.findRowInfo(item);
			if (rowInfo) {
				rowInfo[SUMMARY_COLUMN] = this._getSummary(item);
				// update the row associated with the item
				var rowForItem = this.findRow(item);
				if (rowForItem) {
					dojo.empty(rowForItem.childNodes[SUMMARY_COLUMN]);
					rowForItem.childNodes[SUMMARY_COLUMN].appendChild(rowForItem._rowInfo[SUMMARY_COLUMN]);
				}
			}
		},
		
		/**
		 * Overridden to keep open any editors for rows still on the current
		 * page, and to keep in place any new rows that were at the top of the
		 * table.
		 */
		resetTable: function() {
			// record which editors were open
			var showingItems = {};
			for (var id in this._editorRows) {
				if (this._editorRows[id].parentNode === this.tbody) {
					showingItems[id] = true;
				}
			}
			// record which new rows were at the top of the table, if any
			var newRows = [];
			for (var i = 0; i < this.rowInfo.length && this.rowInfo[i].isNew; i++) {
				newRows.unshift(this.rowInfo[i]);
			}
			// temporarily connect to the next sort call (to be called in
			// inherited below), to move the recorded new rows back to the top
			var handle = dojo.connect(this, "sortRowInfo", this, dojo.hitch(this, function() { //$NON-NLS-1$
				dojo.forEach(newRows, dojo.hitch(this, function(newRow) {
					for (var i = 0; i < this.rowInfo.length; i++) {
						if (this.rowInfo[i] === newRow){
							this.rowInfo.splice(i, 1);
							this.rowInfo.unshift(newRow);
							break;
						}
					}
				}));
			}));
			this.inherited(arguments);
			// re-open the editors that were open before resetting
			dojo.forEach(this.tbody.childNodes, dojo.hitch(this, function(row) {
				var item = row._rowInfo ? row._rowInfo._item : null;
				if (item && showingItems[this.configurationStore.getIdentity(item)]) {
					this._toggleEditing(item, row, true);
				}
			}));
			// disconnect the temporary connect
			dojo.disconnect(handle);
		},
		
		/**
		 * Part of the AbstractProcessAreaEditor saveParticipant framework,
		 * used to indicate whether this tab has any changes so the editor can
		 * display as dirty. Also checks whether any custom behavior editors
		 * have returned an error from their validate() methods.
		 */
		changesPending: function() {
			var customEditorError = this.customEditorManager.validate();
			if (this._errors.length > 0 && (this._editorError !== this._errors[0] || customEditorError === null)) {
				// there is a new error, or no custom editor error to show
				this._mostRecentError = this._errors[0].error;
				this.editor._displayLongErrorMessage(this._mostRecentError);
			} else if (customEditorError !== null && (this._customEditorError !== customEditorError || this._errors.length === 0)) {
				// there is a new custom editor error, or no other error to show
				this._mostRecentError = customEditorError.error;
				this.editor._displayLongErrorMessage(this._mostRecentError);
			} else if (this._errors.length > 0 || customEditorError !== null) {
				// there is an old error
				this.editor._displayLongErrorMessage(this._mostRecentError);
			} else {
				// there is no error
				delete this._mostRecentError;
				this.editor._clearMessage();
			}
			this._editorError = this._errors.length > 0 ? this._errors[0] : null;
			this._customEditorError = customEditorError;
			
			return this.configurationStore.isDirty() || this.customEditorManager.changesPending();
		},
		
		/**
		 * Checks with the inline editor for the given item to see if it is
		 * returning an error message. Maintains a stack of errors, so the most
		 * recent error can be found without having to validate every editor.
		 */
		_validate: function(item) {
			var editor = this._findEditorRow(item)._editor;
			// remove the previous error for this editor from the stack
			var id = this.configurationStore.getIdentity(item);
			for (var i = 0; i < this._errors.length; i++) {
				if (this._errors[i].id === id) {
					this._errors.splice(i, 1);
					break;
				}
			}
			// if there is still an error, put it on top of the stack
			var error = editor.validate();
			if (error !== undefined && error !== null && dojo.string.trim(error).length !== 0) {
				this._errors.splice(0, 0, {
					id: id,
					error: error
				});
			}
		},
		
		/**
		 * Part of the AbstractProcessAreaEditor saveParticipant framework,
		 * used to indicate whether this tab is in a suitable state to save, so
		 * the editor can disable the save button if needed.
		 */
		canSave: function() {
			for (var i in this._editorRows) {
				if (!this._editorRows[i]._editor.canSave()) {
					return false;
				}
			}
			return this.customEditorManager.canSave();
		},
		
		/**
		 * Part of the AbstractProcessAreaEditor saveParticipant framework,
		 * participants add any info they want sent to the server to saveObj.
		 */
		aboutToBeSaved: function(saveObj) {
			// push pending custom editor changes into the configuration store
			this.customEditorManager.saveContent();
			
			// create a list for each configuration context, to add to later
			var configurations = {};
			configurations[ProcessConstants.BehaviorContext.PROJECT] = [];
			configurations[ProcessConstants.BehaviorContext.TEAM] = [];
			configurations[ProcessConstants.BehaviorContext.ITERATION_TYPE] = [];
			configurations[ProcessConstants.BehaviorContext.DEV_LINE] = [];
			configurations[ProcessConstants.BehaviorContext.ITERATION] = [];
			
			this.configurationStore.fetch({
				// get all items in the store
				queryOptions: {
					deep: true
				},
				onItem: dojo.hitch(this, function(item) {
					var id = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
					if (!this.configurationStore.isDirty(item)) {
						// nothing needs to be saved for this item
						return;
					}
					
					// remove category part from id
					id = id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).slice(1).join(ProcessConstants.BEHAVIOR_PATH_SEPARATOR);
					var isOperation = id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length === 1 && id.length > 0;
					if (isOperation) {
						// item is an operation, process it
					} else if (id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).length === 3) {
						// item is a precondition/follow-up, process the whole
						// parent preconditions/follow-ups node instead
						item = this.configurationStore.getParent(item);
						id = id.substring(0, id.lastIndexOf(ProcessConstants.BEHAVIOR_PATH_SEPARATOR));
					} else {
						// item is a preconditions/follow-ups parent. Do not
						// ignore, because it could be dirty because order of
						// children changed
					}
					
					// initialize
					var configuration = null;
					var context = this.configurationStore.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
					var contextId = this.configurationStore.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
					for (var i = 0; i < configurations[context].length; i++) {
						if (configurations[context][i].contextId === contextId) {
							// a configuration has already been started for this
							// context, add to it
							configuration = configurations[context][i];
							break;
						}
					}
					if (configuration === null) {
						// first time this context encountered this save
						configuration = {};
						configuration.contextId = contextId;
						configuration.updated = {};
						configurations[context].push(configuration);
					}
					var roleId = this.configurationStore.getValue(item, ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
					configuration.updated[roleId] = configuration.updated[roleId] || {};
										
					if (isOperation) {
						// updated operation
						configuration.updated[roleId][id] = configuration.updated[roleId][id] || {};
						configuration.updated[roleId][id][ProcessConstants.BehaviorConfigurationDTO.CONFIGURED] = this.configurationStore.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONFIGURED);
						configuration.updated[roleId][id][ProcessConstants.BehaviorConfigurationDTO.FINAL_OP] = this.configurationStore.getValue(item, ProcessConstants.BehaviorConfigurationDTO.FINAL_OP);
					} else {
						// updated preconditions/follow-ups node
						var operationId = id.substring(0, id.lastIndexOf(ProcessConstants.BEHAVIOR_PATH_SEPARATOR));
						configuration.updated[roleId][operationId] = configuration.updated[roleId][operationId] || {};
						
						var type = id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).pop();
						if (configuration.updated[roleId][operationId][type]) {
							// already updated, we fully construct the state of
							// precondition/follow-up lists for each update, so
							// no need to reconstruct
							return;
						} else {
							// new preconditions/follow-ups element
							configuration.updated[roleId][operationId][type] = [];
						}
						
						dojo.forEach(this.configurationStore.getValues(item, ProcessConstants.ExtensionDTO.ITEMS), function(child) {
							// send info for all child elements across
							if (this.configurationStore.getValue(child, ProcessConstants.BehaviorConfigurationDTO.CONFIGURED)) {
								var subConfiguration = this._createOperationBehaviorSubConfiguration(child);
								configuration.updated[roleId][operationId][type].push(subConfiguration);
							}
						}, this);
					}
				})
			});
			
			// if something needs to be saved, convert it to JSON and store it
			// in the appropriate variable in saveObj
			if (configurations[ProcessConstants.BehaviorContext.PROJECT].length > 0) {
				saveObj.jsonProjectConfigurationBehavior = [dojo.toJson(configurations[ProcessConstants.BehaviorContext.PROJECT][0])];
			}
			if (configurations[ProcessConstants.BehaviorContext.TEAM].length > 0) {
				saveObj.jsonTeamConfigurationBehavior = [dojo.toJson(configurations[ProcessConstants.BehaviorContext.TEAM][0])];
			}
			if (configurations[ProcessConstants.BehaviorContext.ITERATION_TYPE].length > 0) {
				saveObj.jsonIterationTypeBehavior = [];
				dojo.forEach(configurations[ProcessConstants.BehaviorContext.ITERATION_TYPE], function(configuration) {
					saveObj.jsonIterationTypeBehavior.push(dojo.toJson(configuration));
				});
			}
			if (configurations[ProcessConstants.BehaviorContext.DEV_LINE].length > 0) {
				saveObj.jsonDevelopmentLineBehavior = [];
				dojo.forEach(configurations[ProcessConstants.BehaviorContext.DEV_LINE], function(configuration) {
					saveObj.jsonDevelopmentLineBehavior.push(dojo.toJson(configuration));
				});
			}
			if (configurations[ProcessConstants.BehaviorContext.ITERATION].length > 0) {
				saveObj.jsonIterationBehavior = [];
				dojo.forEach(configurations[ProcessConstants.BehaviorContext.ITERATION], function(configuration) {
					saveObj.jsonIterationBehavior.push(dojo.toJson(configuration));
				});
			}
		},
		
		/**
		 * Turns the precondition/follow-up action represented by the given item
		 * into an object with the fields necessary for the server to interpret
		 * it.
		 * 
		 * @param item {dojo.data.Item} precondition or follow-up
		 * @returns {Object} all info the server needs for preconditions/
		 * 			follow-up actions
		 */
		_createOperationBehaviorSubConfiguration: function(item) {
			var configuration = {};
			var childId = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
			var operationBehaviorId = childId.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR).pop();
			configuration[ProcessConstants.ExtensionDTO.ID] = operationBehaviorId;
			configuration[ProcessConstants.ExtensionDTO.NAME] = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.NAME);
			if (this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.DESCRIPTION) !== "") { //$NON-NLS-1$
				configuration[ProcessConstants.ExtensionDTO.DESCRIPTION] = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.DESCRIPTION);
			}
			if (this.configurationStore.getValue(item, ProcessConstants.BehaviorConfigurationDTO.NOT_OPTIONAL)) {
				configuration[ProcessConstants.BehaviorConfigurationDTO.NOT_OPTIONAL] = true;
			}
			var content = this.customEditorManager.getContent(item);
			if (content === null || content === undefined) {
				content = this.configurationStore.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTENT);
			}
			if (content !== null) {
				configuration[ProcessConstants.BehaviorConfigurationDTO.CONTENT] = content;
			}
			return configuration;
		},
		
		/**
		 * Part of the AbstractProcessAreaEditor saveParticipant framework,
		 * called after saving is done.
		 */
		saved: function() {
			// nothing needs to be done
		},
		
		uninitialize: function() {
			this._clearState();
			this.configurationStore.destroy();
			this._addConfigurationMenuButton.destroyRecursive();
			
			if (this._addConfigurationMenu !== undefined) {
				this._addConfigurationMenu.destroyRecursive();
			}

			this.inherited(arguments);
		}
		
	});
	
})();
