/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectContextWidget"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TimelineModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TimelineTree"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	/**
	 * A widget that includes series of radio buttons to specify a context for a
	 * new or existing operation behavior configuration: Always, during an
	 * Iteration Type, or during a specific Timeline/Iteration. When the latter
	 * two buttons are selected, content to select the iteration type/iteration/
	 * timeline appears below the buttons.
	 * 
	 * The constructor accepts an argument "args" with the following properties:
	 * 
	 * - model	[required]	An object which provides information about the
	 * 						general process model that is not operation-behavior
	 * 						specific. The following fields are required:
	 * 		- contextStore				A dojo.data.ItemFileReadStore for
	 * 									iteration types and timelines/iterations
	 * 									for the process area
	 * 		- editorContext				The editor context that the widget is
	 * 									being used in: a value in
	 * 									ProcessConstants.EditorContext
	 * 		- developmentLineItemId		A string that is used if editorContext
	 * 									is ProcessConstants.EditorContext.TEAM
	 * 									to offer child iterations of the team
	 * 									area's selected development line
	 *		- configurationStore		A dojo.data.ItemFileReadStore which
	 * 									contains operation configurations
	 * 									defined for the current process area
	 * 
	 * - behaviorConfigurationItem	[optional]
	 *     A dojo.data store item that represents an existing operation behavior
	 *     configuration that is to be updated
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectContextWidget", //$NON-NLS-1$
			[dijit._Widget, dijit._Templated], {
		
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/OperationBehaviorSelectContextWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._buttonIds = [
				"buttonConfigAlways", //$NON-NLS-1$
				"buttonConfigIterationType", //$NON-NLS-1$
				"buttonConfigTimeline" //$NON-NLS-1$
			];
		},
		
		postCreate: function() {
			this._initIterationTypes();
			this._initTimelines();
			
			//Initialize the _selectedContext for the iteration type and
			//iteration buttons. These values may change when values in the
			//respective trees are clicked, but giving an initial value will
			//allow our listener to know that no iteration type or development
			//line was selected; eg. if there are no iteration types or
			//development lines/iterations available. Otherwise, the listener
			//could not distinguish between that case and the case of 'Always'
			//being selected
			this.buttonConfigIterationType._selectedContext = ProcessConstants.BehaviorContext.ITERATION_TYPE;
			this.buttonConfigTimeline._selectedContext = ProcessConstants.BehaviorContext.DEV_LINE;
			
			if (this.model.editorContext === ProcessConstants.EditorContext.TEAM) {
				this.buttonConfigTimelineText.innerHTML = this.messages.OperationBehaviorSelectContextWidget_2;
			} else {
				this.buttonConfigTimelineText.innerHTML = this.messages.OperationBehaviorSelectContextWidget_1;
			}
			
			if (this.behaviorConfigurationItem === undefined) {
				//Creating a new configuration. Select 'Always' by default
				this.buttonConfigAlways.checked = true;
				this._primButtonSelected(this.buttonConfigAlways);
			} else {
				//Editing an existing configuration. If the current scope is
				//"always", select radio "Always" and grey it out. Otherwise,
				//code in the focusChanged() functions of the iteration type and
				//timeline tree initialization below will take care of the
				//initial selection.
				if (this.model.configurationStore.getValue(this.behaviorConfigurationItem, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID) === "") { //$NON-NLS-1$
					this.buttonConfigAlways.checked = true;
					this._primButtonSelected(this.buttonConfigAlways);
					dojo.addClass(this.buttonConfigAlways.parentNode, "disabled"); //$NON-NLS-1$
				}
			}
			
			dojo.forEach(this._buttonIds, dojo.hitch(this, function(buttonId) {
				this.connect(this[buttonId], "onclick", "_buttonSelected"); //$NON-NLS-1$ //$NON-NLS-2$
			}));
		},
		
		getSummaryTextString: function() {
			return this.summaryText.innerHTML;
		},
		
		hideSummaryTextNode: function() {
			dojo.addClass(this.summaryText, "hidden"); //$NON-NLS-1$
		},
		
		/**
		 * Create a tree to display the iteration types that available for
		 * selection. Selecting an iteration type will update the state in this
		 * page: the identifier of the iteration type, and display the summary
		 * of the iteration type in the page.
		 */
		_initIterationTypes: function() {
			this._iterationTypesModel = new com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeModel({
				store: this.model.contextStore
			});
			
			var self = this;
			this._iterationTypeTree = new com.ibm.team.process.web.ui.internal.admin.projects.IterationTypeTree({
				model: this._iterationTypesModel,
				editorContext: this.model.editorContext,
				
				focusChanged: function(node, showValidationMsg){
					var item = node.item;
					
					//If necessary, select a specific iteration type initially
					//(eg. when editing an existing behavior configuration)
					if (self.behaviorConfigurationItem !== undefined && !self._initializedDefaultIterationTypeSelection) {
						self._initializedDefaultIterationTypeSelection = true;
						
						var configurationContext = self.model.configurationStore.getValue(self.behaviorConfigurationItem, ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
						
						if (ProcessConstants.PermissionContext.ITERATION_TYPE === configurationContext) {
							var configurationContextId = self.model.configurationStore.getValue(self.behaviorConfigurationItem, "contextId"); //$NON-NLS-1$
							item = self._selectIterationTypeIfAvailable(configurationContextId);
							
							//grey out the selection, since the user will have
							//to pick a different context
							var node = self._iterationTypeTree.dndController.getSelectedTreeNodes()[0].contentNode;
							dojo.addClass(node, "disabled"); //$NON-NLS-1$
						}
					}
					
					var contextId = self.model.contextStore.getValue(item, "contextId"); //$NON-NLS-1$
					
					//Update local state in button, in case another radio is
					//selected and then the iteration type radio
					self.buttonConfigIterationType._selectedContextId = contextId;
					
					//Update state
					if (self.buttonConfigIterationType.checked) {
						self._selectedContextId = self.buttonConfigIterationType._selectedContextId;
						self._selectedContext = self.buttonConfigIterationType._selectedContext;
					}
					
					dojo.removeClass(self.iterationTypeTreeContainer, "hidden"); //$NON-NLS-1$
					dojo.addClass(self.iterationTypeNoIterationsMsgContainer, "hidden"); //$NON-NLS-1$
					
					self._hasIterationTypes = true;
					var description = self.model.contextStore.getValue(item, "summary"); //$NON-NLS-1$
					dojo.empty(self.iterationTypeDescriptionContainer);
					if (description) {
						dojo.removeClass(self.iterationTypeDescriptionContainer, "placeholder"); //$NON-NLS-1$
					} else {
						dojo.addClass(self.iterationTypeDescriptionContainer, "placeholder"); //$NON-NLS-1$
						description = this.messages.OperationBehaviorSelectContextWidget_0;
					}
					self.iterationTypeDescriptionContainer.appendChild(document.createTextNode(description));
					
					self.selectionChangedCallback({
						context: self._selectedContext,
						contextId: self._selectedContextId
					}, false);
				},
				onDblClick: function(/* dojo.data */ item, /*TreeNode*/ node, /*Event*/ evt) {
					self.selectionChangedCallback({
						context: self._selectedContext,
						contextId: self._selectedContextId
					}, true);
				}
			});
			
			dojox.xml.parser.removeChildren(this.iterationTypeTreeDiv);
			this.iterationTypeTreeDiv.appendChild(this._iterationTypeTree.domNode);
		},
		
		/**
		 * Attempt to select an iteration type with a given id. If there is such
		 * a type available, it is selected as if clicked by the user, and an
		 * object (the store item) is returned to indicate success. Otherwise,
		 * null is returned.
		 */
		_selectIterationTypeIfAvailable: function(contextId) {
			var queryArg = {};
			queryArg[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = contextId;
			var selectedItem = null;
			this.model.contextStore.fetch({
				query: queryArg,
				queryOptions: {
					deep: true
				},
				onItem: dojo.hitch(this, function(theItem, request) {
					selectedItem = theItem;
					
					this.buttonConfigIterationType.checked = true;
					this.buttonConfigIterationType._selectedContextId = contextId;
					this._primButtonSelected(this.buttonConfigIterationType);
					
					//focusItem() can be used because all nodes in the tree are
					//displayed.
					this._iterationTypeTree.focusItem(theItem);
				})
			});
			return selectedItem;
		},
		
		/**
		 * Create a tree to display the timelines and iterations types that are
		 * available for selection. In the context of a project area, all
		 * development lines and iterations will be displayed. For a team area,
		 * only the child iterations of the development line that is configured
		 * for the team area are shown. Selecting a development line or
		 * iteration will update the state in this page: the identifier of the
		 * selected scope.
		 */
		_initTimelines: function() {
			if (this.model.editorContext === ProcessConstants.EditorContext.TEAM) {
				//Team areas require a different model. See the comment for
				//function _createIterationsModelForTeamArea for details
				this._timelinesModel = this._createIterationsModelForTeamArea();
				
				//The call to self._timelineTree.set("path", ...) in code below
				//that expands the tree to an item and selects it fails if the
				//identifier of the root item is undefined, which is the case by
				//default. Set it to the identifier of the root item.
				this._timelinesModel.root.id = ProcessConstants.PermissionsScopeContext.ITERATIONS;
			} else {
				this._timelinesModel = new com.ibm.team.process.web.ui.internal.admin.projects.TimelineModel({
					store: this.model.contextStore
				});
				
				//Set root identifier
				this._timelinesModel.root.id = ProcessConstants.PermissionsScopeContext.DEV_LINES;
			}
			
			var self = this;
			this._timelineTree = new com.ibm.team.process.web.ui.internal.admin.projects.TimelineTree({
				model: this._timelinesModel,
				editorContext: this.model.editorContext,
				
				focusChanged: function(node, showValidationMsg){
					var item = node.item;
					
					//If necessary, select a specific iteration type initially
					//(eg. when editing an existing behavior configuration) 
					if (self.behaviorConfigurationItem !== undefined && !self._initializedDefaultTimelineSelection) {
						self._initializedDefaultTimelineSelection = true;

						var configurationContext = self.model.configurationStore.getValue(self.behaviorConfigurationItem, ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
						
						if (ProcessConstants.PermissionContext.ITERATION === configurationContext ||
							ProcessConstants.PermissionContext.DEV_LINE === configurationContext) {
							var configurationContextId = self.model.configurationStore.getValue(self.behaviorConfigurationItem, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
							item = self._selectDevelopmentLineOrIterationIfAvailable(configurationContextId);
							
							//We want to make grey out the selected tree item,
							//but if it's nested then attempts to access the
							//node with getNodesByItem fail, because the node is
							//not been added to the tree until all of its
							//parents have been expanded, which is in progress
							//asynchronously. Keep trying until we can do it.
							var makeDisabled = function() {
								var nodes = self._timelineTree.getNodesByItem(item);
								if (nodes[0] === undefined) {
									setTimeout(makeDisabled, 30);
									return;
								}
								dojo.addClass(nodes[0].contentNode, "disabled"); //$NON-NLS-1$
							};
							setTimeout(makeDisabled, 30);
						}
					}
					
					var contextId = self.model.contextStore.getValue(item, "contextId"); //$NON-NLS-1$
					
					//Update local state in button, in case another radio is
					//selected and then the iteration radio
					self.buttonConfigTimeline._selectedContextId = contextId;
					self.buttonConfigTimeline._selectedContext = self.model.contextStore.isTimeline(item) || self.model.contextStore.isProjectTimeline(item)
						? ProcessConstants.BehaviorContext.DEV_LINE
						: ProcessConstants.BehaviorContext.ITERATION;
						
					//Update state
					if (self.buttonConfigTimeline.checked) {
						self._selectedContextId = self.buttonConfigTimeline._selectedContextId;
						self._selectedContext = self.buttonConfigTimeline._selectedContext;
					}
					
					self.selectionChangedCallback({
						context: self._selectedContext,
						contextId: self._selectedContextId
					}, false);
				},
				onDblClick: function(/* dojo.data */ item, /*TreeNode*/ node, /*Event*/ evt) {
					self.selectionChangedCallback({
						context: self._selectedContext,
						contextId: self._selectedContextId
					}, true);
				}
			});
			
			dojox.xml.parser.removeChildren(this.timelineTreeDiv);
			this.timelineTreeDiv.appendChild(this._timelineTree.domNode);
		},
		
		/**
		 * Attempt to reveal and a development line/iteration with a given id.
		 * If there is such a line/iteration available, it is selected as if
		 * clicked by the user, and an object (the store item) is returned to
		 * indicate success. Otherwise, null is returned.
		 */
		_selectDevelopmentLineOrIterationIfAvailable: function(contextId) {
			var queryArg = {};
			queryArg[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = contextId;
			var selectedItem = null;
			this.model.contextStore.fetch({
				query: queryArg,
				queryOptions: {
					deep: true
				},
				onItem: dojo.hitch(this, function(theItem, request) {
					selectedItem = theItem;
					
					this.buttonConfigTimeline.checked = true;
					this.buttonConfigTimeline._selectedContextId = contextId;
					this.buttonConfigTimeline._selectedContext = this.model.contextStore.isTimeline(theItem) || this.model.contextStore.isProjectTimeline(theItem)
						? ProcessConstants.BehaviorContext.DEV_LINE
						: ProcessConstants.BehaviorContext.ITERATION;
					this._primButtonSelected(this.buttonConfigTimeline);
					
					//Expand the tree to the configuration's current item and
					//select it. This is done via set("path", [id1, id2]) on the
					//tree where the array is the ids from the root to the item
					var parentItem = theItem;
					var expandPath = [];
					while (parentItem !== null) {
						//Add identifier to the beginning of the array
						expandPath.unshift(this.model.contextStore.getIdentity(parentItem));
						parentItem = this.model.contextStore.getParent(parentItem);
					}
					
					this._timelineTree.set("path", expandPath); //$NON-NLS-1$
					
					//Verify that a node was selected. In the case that we're
					//displaying iterations for a team area and the team's
					//current development line is changed, the context store
					//could have an entry for an iteration that the tree
					//does not include
					var path = this._timelineTree.get("path"); //$NON-NLS-1$
					if (path.length === 0) {
						selectedItem = null;
					}
				})
			});
			
			return selectedItem;
		},
		
		/**
		 * In a team area,
		 * com.ibm.team.process.web.ui.internal.admin.projects.TimelineModel
		 * cannot be used because there is no development line parent in the
		 * context store. We only want to show the iterations that are children
		 * of the team area's selected development line. This function returns a
		 * ForestStoreModel that only has those items and their descendents.
		 */
		_createIterationsModelForTeamArea: function() {
			var iterations = [];
			var query = {};
			query.archived = false;
			query.parentItemId = this.model.developmentLineItemId;
			this.model.contextStore.fetch({
				query: query,
				queryOptions: {
					deep: true
				},
				onItem: function(item) {
					iterations.push(item);
				}
			});
			var iterationsModel = new dojo.declare(dijit.tree.ForestStoreModel, {
			})({
				childrenAttrs: ["items"], //$NON-NLS-1$
				store: this.model.contextStore,
				rootChildren: iterations
			});
			return iterationsModel;
		},
		
		/**
		 * Callback when any of the radio buttons are selected. Separate from
		 * _primButtonSelected so that the latter function can be called
		 * programatically.
		 */
		_buttonSelected: function(event) {
			this._primButtonSelected(event.target);
		},
		
		/**
		 * If there is hidden content associated with the button, as declared by
		 * the existence of a widget whose dojoAttachPoint is "custom-" followed
		 * by the identifier of the button, that content is displayed.
		 * 
		 * The state of this page: the selected context type (a value in
		 * ProcessConstants.BehaviorContext) and the identifier of the selected
		 * context, is updated to the values stored in the buttons, to handle
		 * cases where the user selects one button and another.
		 */
		_primButtonSelected: function(button) {
			//Reveal hidden associated with button, if there is content to reveal
			dojo.forEach(this._buttonIds, dojo.hitch(this, function(buttonId) {
				var contentForButton = this["custom-" + buttonId]; //$NON-NLS-1$
				if (contentForButton) {
					if (button.id === buttonId) {
						dojo.removeClass(contentForButton, "hidden"); //$NON-NLS-1$
					} else {
						dojo.addClass(contentForButton, "hidden"); //$NON-NLS-1$
					}
				}
			}));
			if (button.id === "buttonConfigAlways") { //$NON-NLS-1$
				this._selectedContextId = undefined;
				this._selectedContext = undefined;
			} else {
				this._selectedContextId = button._selectedContextId;
				this._selectedContext = button._selectedContext;
			}
			
			this.selectionChangedCallback({
				context: this._selectedContext,
				contextId: this._selectedContextId
			}, false);
		},
		
		/**
		 * If a context with the specified identifier is listed, select it.
		 * Whether an item was selected is returned as a boolean. 'False' would
		 * indicate the the provided context ID does not match the identifier of
		 * an iteration type/timeline/iteration in this widget.
		 */
		selectContextIfAvailable: function(context, contextId) {
			var selectDefault = dojo.hitch(this, function() {
				this.buttonConfigAlways.checked = true;
				this._primButtonSelected(this.buttonConfigAlways);
			});
			var valueSet = true;
			if (ProcessConstants.PermissionContext.ITERATION === context ||
				ProcessConstants.PermissionContext.DEV_LINE === context) {
				if (this._selectDevelopmentLineOrIterationIfAvailable(contextId) === null) {
					valueSet = false;
					selectDefault();
				}
			} else if (ProcessConstants.PermissionContext.ITERATION_TYPE === context) {
				if (this._selectIterationTypeIfAvailable(contextId) === null) {
					valueSet = false;
					selectDefault();
				}
			} else {
				selectDefault();
			}
			return valueSet;
		},
		
		uninitialize: function(){
			if (this._iterationTypeTree !== undefined) {
				this._iterationTypeTree.destroyRecursive();
				delete this._iterationTypeTree;
			}
			delete this._iterationTypesModel;
			
			if (this._timelineTree !== undefined) {
				this._timelineTree.destroyRecursive();
				delete this._timelineTree;
			}
			delete this._timelinesModel;
			
			this.inherited(arguments);
		}
	});
})();
