/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore", dojo.data.ItemFileWriteStore, { //$NON-NLS-1$
		/*
		 * Constructs the PermissionsScopeStore instance
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    editorContext: the editor context tree is being used in... a value in ProcessConstants.EditorContext
		 *    processAreaItemId: the item id of the process area to load the timeline model for
		 *    errorContext: the object to invoke the error call back on
		 *    errorCallback: the function to call if an error occurs loading the model
		 *  }
		 */
		constructor: function(args){
			// A little dangerous here...setting the private value 
			// that is normally taken from the args in the constructor
			// for ItemFileReadStore.  Don't want to force the client
			// to have to pass that in since it's pre-determined for this
			// model
			this._jsonData = {
				identifier: 'id', //$NON-NLS-1$
				label: 'name', //$NON-NLS-1$
				items: []
			};
			
			this.comparatorMap = {
				name : ProcessClient.stringCompare, 
				contextId : ProcessClient.stringCompare
			};
			
			this._loadedDevelopmentLines = {};
			
			this._editorContext = args.editorContext;
			this._processAreaItemId = args.processAreaItemId;
			this._currentDevelopmentLineItemId = args.currentDevelopmentLineItemId;
			this._errorContext = args.errorContext;
			this._errorCallback = args.errorCallback;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._initialize();
		},
		
		_initialize: function(){
			this._addProjectRootItem();
			this._addTeamRootItem();
			this._addIterationTypesRootItem();
			this._addTimelinesRootItem();
			this._addIterationsRootItem();
			this.save();
			this.operationsLeft = 2;
			
			var srh = new ServiceResponseHandler(this, "_populateIterationTypesData", "_handleIterationTypesFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getIterationTypes({
				processAreaItemId: this._processAreaItemId,
				processAreaContext: this._editorContext,
				includeArchived: true
			}, srh);
			
			if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
				srh = new ServiceResponseHandler(this, "_populateDevelopmentLinesData", "_handleDevelopmentLinesFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getProjectDevelopmentLines({
					uuid: this._processAreaItemId,
					includeArchived: true
				}, srh);
			} else {
				// The editor context is team area.
				if (this._currentDevelopmentLineItemId) {
					// Populate iterations with the passed-in development line if this development line is not undefined/null.
					// This will be true in most cases.
					this._populateIterationsRootItemWithDevLine(this._currentDevelopmentLineItemId);
				} else {
					// Otherwise populate iterations with the current team area.
					// This will be used when refresh button is clicked on permissions tab.
					srh = new ServiceResponseHandler(this, "_populateIterationsData", "_handleIterationsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessClient.getTeamAreaIterations({
						uuid: this._processAreaItemId,
						includeArchived: true
					}, srh);
				}
			}
			
		},
		
		_populateDevelopmentLinesData: function(data){
			var self = this;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.DEV_LINES,
				onItem: function(item, request){
					dojo.forEach(data, function(developmentLine){
						self.newItem({
							name: developmentLine.label,
							id: developmentLine.itemId,
							stateId: developmentLine.stateId,
							parent: self.getIdentity(item),
							context: ProcessConstants.ExtensionContext.TEAM,
							contextId: developmentLine.id,
							enabledContext: ProcessConstants.PermissionContext.DEV_LINE,
							overrideAllowed: true,
							projectTimeline: developmentLine.projectTimeline,
							startDate: dojo.date.stamp.fromISOString(developmentLine.startDate),
							endDate: dojo.date.stamp.fromISOString(developmentLine.endDate),
							currentIterationItemId: developmentLine.currentIterationItemId ? developmentLine.currentIterationItemId : "", //$NON-NLS-1$
							archived: developmentLine.archived
						}, {
							parent: item,
							attribute: "items" //$NON-NLS-1$
						});
						
					});
				}
			});
			
			dojo.forEach(data, function(developmentLine){
				this.operationsLeft++;
				var srh = new ServiceResponseHandler(self, "_populateIterationData", "_handleIterationsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getIterations({
					uuid: developmentLine.itemId,
					includeArchived: true
				}, srh);
			}, this);
			this.save();
			this.operationsLeft--;
		},
		
		_populateIterationsData: function(data){
			var self = this;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.ITERATIONS,
				onItem: function(item, request){
					dojo.forEach(data, function(iteration){
						self.newItem({
							name: iteration.label,
							id: iteration.itemId,
							stateId: iteration.stateId,
							parent: self.getIdentity(item),
							context: ProcessConstants.ExtensionContext.TEAM,
							contextId: iteration.id,
							enabledContext: ProcessConstants.PermissionContext.ITERATION,
							overrideAllowed: true,
							current: iteration.current,
							completed: iteration.completed,
							leaf: iteration.leaf,
							startDate: dojo.date.stamp.fromISOString(iteration.startDate),
							endDate: dojo.date.stamp.fromISOString(iteration.endDate),
							iterationTypeItemId: iteration.iterationTypeItemId ? iteration.iterationTypeItemId : ProcessConstants.ITERATION_TYPE_NONE,
							hasDeliverable: iteration.hasDeliverable,
							archived: iteration.archived,
							parentItemId: iteration.parentItemId // this attribute will be used to filter out the iterations that do not belong to current development line
						}, {
							parent: item,
							attribute: "items" //$NON-NLS-1$
						});
						
					});
				}
			});
			
			dojo.forEach(data, function(iteration){
				if (iteration.iterations) {
					this.operationsLeft++;
					this._populateIterationData(iteration.iterations);
				}
			}, this);
			
			if (data && data.length > 0) {
				// Set the current development line while populating the iterations.
				this._currentDevelopmentLineItemId = data[0].parentItemId;
				// Mark the development line as loaded, then we don't need to load it again in future.
				this._loadedDevelopmentLines[this._currentDevelopmentLineItemId] = true;
			}
			
			this.save();
			this.operationsLeft--;
		},
		
		_populateIterationData: function(iterations){
			if (iterations) {
				var self = this;
				for (var i = 0; i < iterations.length; i++) {
					var iteration = iterations[i];
					this.fetchItemByIdentity({
						identity: iteration.parentItemId,
						onItem: function(item, request){
							self.newItem({
								name: iteration.label,
								id: iteration.itemId,
								stateId: iteration.stateId,
								parent: iteration.parentItemId,
								context: ProcessConstants.ExtensionContext.TEAM,
								contextId: self.getValue(item, "contextId") + ProcessConstants.PERMISSION_PATH_SEPARATOR + iteration.id, //$NON-NLS-1$
								originalContextId: self.getValue(item, "contextId") + ProcessConstants.PERMISSION_PATH_SEPARATOR + iteration.id, //$NON-NLS-1$
								enabledContext: ProcessConstants.PermissionContext.ITERATION,
								overrideAllowed: true,
								current: iteration.current,
								completed: iteration.completed,
								leaf: iteration.leaf,
								startDate: dojo.date.stamp.fromISOString(iteration.startDate),
								endDate: dojo.date.stamp.fromISOString(iteration.endDate),
								iterationTypeItemId: iteration.iterationTypeItemId ? iteration.iterationTypeItemId : ProcessConstants.ITERATION_TYPE_NONE,
								hasDeliverable: iteration.hasDeliverable,
								archived: iteration.archived
							}, {
								parent: item,
								attribute: "items" //$NON-NLS-1$
							});
						}
					});
					this.operationsLeft++;
					this._populateIterationData(iteration.iterations);
				}
				this.save();
			}
			this.operationsLeft--;
		},
		
		_populateIterationTypesData: function(data){
			var self = this;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.ITERATION_TYPES,
				onItem: function(item, request){
					var items = {};
					var i = 0;
					dojo.forEach(data, function(iterationType){
						var newChildItem = self.newItem({
							name: iterationType[ProcessConstants.IterationTypeDTO.LABEL],
							id: iterationType[ProcessConstants.IterationTypeDTO.ITEM_ID],
							stateId: iterationType[ProcessConstants.IterationTypeDTO.STATE_ID],
							parent: self.getValue(item, ProcessConstants.IterationTypeDTO.ID),
							context: ProcessConstants.ExtensionContext.TEAM,
							contextId: iterationType.id,
							savedContextId: iterationType.id,
							enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE,
							overrideAllowed: true,
							archived: iterationType.archived,
							summary: iterationType.summary
						}, {
							parent: item,
							attribute: "items" //$NON-NLS-1$
						});
						items[i] = newChildItem;
						i++;
					});
				}
			});
			this.save();
			this.operationsLeft--;
		},
		
		_handleDevelopmentLinesFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.DEV_LINES);
		},
		
		_handleIterationTypesFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.ITERATION_TYPES);
		},
		
		_handleIterationsFetchError: function(error){
			this._errorCallback.call(this._errorContext, error, ProcessConstants.PermissionsScopeContext.ITERATIONS);
		},
		
		_addProjectRootItem: function(){
			if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
				this.newItem({
					id: ProcessConstants.PermissionsScopeContext.PROJECT,
					name: this.messages.PermissionsScopeStore_ProjectConfigurationNodeName,
					context: ProcessConstants.ExtensionContext.PROJECT,
					contextId: "", //$NON-NLS-1$
					enabledContext: ProcessConstants.PermissionContext.PROJECT,
					overrideAllowed: false,
					archived: false
				});
			}
		},
		
		_addTeamRootItem: function(){
			var teamItem = {
				id: ProcessConstants.PermissionsScopeContext.TEAM,
				name: this.messages.PermissionsScopeStore_TeamConfigurationNodeName,
				context: ProcessConstants.ExtensionContext.TEAM,
				contextId: "", //$NON-NLS-1$
				enabledContext: ProcessConstants.PermissionContext.TEAM,
				overrideAllowed: false,
				archived: false
			};
			if (this._editorContext === ProcessConstants.EditorContext.TEAM) {
				teamItem.name = this.messages.PermissionsScopeStore_PermissionsNodeName;
				teamItem.overrideAllowed = true;
			}
			this.newItem(teamItem);
		},
		
		_addIterationTypesRootItem: function(){
			this.newItem({
				id: ProcessConstants.PermissionsScopeContext.ITERATION_TYPES,
				name: this.messages.PermissionsScopeStore_IterationTypesNodeName,
				archived: false
			});
		},
		
		_addTimelinesRootItem: function(){
			if (this._editorContext === ProcessConstants.EditorContext.PROJECT) {
				this.newItem({
					id: ProcessConstants.PermissionsScopeContext.DEV_LINES,
					name: this.messages.PermissionsScopeStore_TimelinesNodeName,
					archived: false
				});
			}
		},
		
		_addIterationsRootItem: function(){
			if (this._editorContext === ProcessConstants.EditorContext.TEAM) {
				this.newItem({
					id: ProcessConstants.PermissionsScopeContext.ITERATIONS,
					name: this.messages.PermissionsScopeStore_IterationsNodeName,
					archived: false
				});
			}
		},
		
		/**
		 * Populate the iterations structure with the given development line.
		 * This method will actually add the iterations hierarchy to IterationsRootItem.
		 * The original iterations under IterationsRootItem will not be removed.
		 * Model can filter out the iterations that do not belong to current development line.
		 * 
		 * @param {Object} developmentLineItemId
		 */
		_populateIterationsRootItemWithDevLine: function(developmentLineItemId) {
			if (developmentLineItemId === ProcessConstants.TIMELINE_SELECTOR_EMPTY_OPTION_ID) {
				this.operationsLeft--;
				return;
			}
			var srh = new ServiceResponseHandler(this, "_populateIterationsData", "_handleIterationsFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getIterations({
				uuid: developmentLineItemId,
				includeArchived: true
			}, srh);
		},
		
		isIterationType: function(item){
			var context = this.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (context === ProcessConstants.PermissionContext.ITERATION_TYPE) {
				return true;
			}
			return false;
		},
		
		isTimeline: function(item){
			var context = this.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (context === ProcessConstants.PermissionContext.DEV_LINE) {
				return true;
			}
			return false;
		},
		
		isProjectTimeline: function(item){
			return this.getValue(item, "projectTimeline"); //$NON-NLS-1$
		},
		
		isIteration: function(item){
			if (this.isItem(item)) {
				var context = this.getValue(item, "enabledContext"); //$NON-NLS-1$
				if (context === ProcessConstants.PermissionContext.ITERATION) {
					return true;
				}
			}
			return false;
		},
		
		isCurrentIteration: function(item){
			return this.getValue(item, "current") === true && this.getValue(item, "leaf") === true; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		getTimelineForIteration: function(iteration){
			var self = this;
			var timeline;
			this.fetchItemByIdentity({
				identity: iteration.parent,
				onItem: function(item, request){
					if (self.isTimeline(item)) {
						timeline = item;
					} else {
						timeline = self.getTimelineForIteration(item);
					}
				}
			});
			return timeline;
			
		},
		
		getParent: function(item){
			var parent;
			this.fetchItemByIdentity({
				identity: item.parent,
				onItem: function(item, request){
					parent = item;
				}
			});
			return parent;
		},
		
		deleteItem: function(item){
			/*
			 * TODO: temporary, see workitem 235332
			 * 
			 * Send out a new event, "aboutToDelete", that implementors can
			 * connect to to find out when deleteItem is called on an item,
			 * *before* that item is removed from the store.
			 */
			this.aboutToDelete(item);
			
			var enabledContext = this.getValue(item, "enabledContext"); //$NON-NLS-1$
			if (enabledContext === ProcessConstants.PermissionContext.ITERATION_TYPE) {
				// Eliminate any references from the iterations to the iteration type
				// being deleted
				var iterationTypeItemId = this.getIdentity(item);
				this.fetch({
					query: {
						enabledContext: ProcessConstants.PermissionContext.ITERATION,
						iterationTypeItemId: iterationTypeItemId
					},
					queryOptions: {
						deep: true
					},
					onComplete: dojo.hitch(this, function(items){
						for (var i = 0; i < items.length; i++) {
							var item = items[i];
							if (this.isDirty(item)) {
								this.setValue(item, "iterationTypeItemId", ProcessConstants.ITERATION_TYPE_NONE); //$NON-NLS-1$
							}
						}
					})
				});
			}
			
			// Get the children of the item
			var children = this.getValues(item, "items"); //$NON-NLS-1$
			// Now destroy the children recursively
			if (children) {
				dojo.forEach(children, function(child){
					this.deleteItem(child);
				}, this);
			}
			// Now get rid of the parent
			this.inherited(arguments);
		},
		
		aboutToDelete: function(item) {
			// implementors can connect to this method to find out about a
			// delete event while the item is in the store (so it can be read)
		},
		
		archiveItem: function(item) {
			this.setValue(item, "archived", true); //$NON-NLS-1$	
			// Archive the children of the item
			var children = this.getValues(item, "items"); //$NON-NLS-1$			
			if (children) {
				dojo.forEach(children, function(child){
					if (this.isNewItem(child)) {
						this.deleteItem(child);
					} else {
						this.archiveItem(child);
					}
				}, this);
			}
		},
		
		unarchiveItem: function(item) {
			this.setValue(item, "archived", false); //$NON-NLS-1$	
			// Unarchive the children of the item
			var children = this.getValues(item, "items"); //$NON-NLS-1$			
			if (children) {
				dojo.forEach(children, function(child){
					this.unarchiveItem(child);
				}, this);
			}
		},
		
		isNewItem: function(item){
			var id = this.getIdentity(item);
			if (id.indexOf(ProcessConstants.EditorMode.NEW) === 0) {
				return true;
			}
			return false;
		},
		
		isArchivedItem: function(item){
			return this.hasAttribute(item, "archived") && this.getValue(item, "archived"); //$NON-NLS-1$	//$NON-NLS-2$	
		},
		
		/**
		 * Set the current development line in this store.
		 * Will also load the iterations if the development line has not been loaded.
		 * 
		 * @param {Object} developmentLineItemId
		 */
		setCurrentDevelopmentLineItemId: function(developmentLineItemId) {
			var previousDevelopmentLineItemId = this._currentDevelopmentLineItemId; 
			this._currentDevelopmentLineItemId = developmentLineItemId;
			if (developmentLineItemId && !this._loadedDevelopmentLines[developmentLineItemId] && previousDevelopmentLineItemId !== developmentLineItemId) {
				if (!this.operationsLeft) {
					this.operationsLeft = 1;
				} else {
					this.operationsLeft++;
				}
				this._populateIterationsRootItemWithDevLine(developmentLineItemId);
			}
			this.onCurrentDevelopmentLineChange();
		},
		
		onCurrentDevelopmentLineChange: function() {
			// implementors can connect to be informed when this store's current
			// development line changes
		},
		
		/**
		 * Get current development line.
		 * 
		 * @return current development line
		 */
		getCurrentDevelopmentLineItemId: function() {
			return this._currentDevelopmentLineItemId;
		},
		
		isInitialized: function() {
			return this.operationsLeft === 0;
		}
		
	});
	
})();
