/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessMessageBox"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	
	/**
	 * Extends jazz.ui.MessageBox to add the additional functionality of
	 * being able to specify a "fixHandler" function and a "fixText" string
	 * when calling showMessage(). If these are provided then an additional
	 * link with the given text will appear, and the given function will run
	 * when it is clicked. The additional link will appear next to the "hide
	 * details" link at the end of the message summary.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessMessageBox", jazz.ui.MessageBox, { //$NON-NLS-1$
		
		showMessage: function(args){
			this.inherited(arguments);
			
			if (args.fixHandler && typeof args.fixHandler == "function") { //$NON-NLS-1$
				this.fixLink = document.createElement("a"); //$NON-NLS-1$
				this.fixLink.innerHTML = args.fixText;
				this.connect(this.fixLink, "onclick", function(e){ //$NON-NLS-1$
					dojo.stopEvent(e);
					if (args.fixHandler()) {
						this.messageArea.removeChild(this.fixLink);
						this.messageArea.removeChild(this.separator);
					}
				});
				dojo.addClass(this.fixLink, "fixLink"); //$NON-NLS-1$
				dojo.place(this.fixLink, this.detailsLink, "after"); //$NON-NLS-1$
				
				this.separator = document.createElement("span"); //$NON-NLS-1$
				dojo.addClass(this.separator, "separator"); //$NON-NLS-1$
				dojo.place(this.separator, this.fixLink, "before"); //$NON-NLS-1$
			}
		},
		
		onToggle: function(){
			if (this.fixLink) {
				if (this.getToggleState() === "visible"){ //$NON-NLS-1$
					this.fixLink.style.display = "inline"; //$NON-NLS-1$
					this.separator.style.display = "inline-block"; //$NON-NLS-1$
				} else {
					this.fixLink.style.display = "none"; //$NON-NLS-1$
					this.separator.style.display = "none"; //$NON-NLS-1$
				}
			}
		}
	
	});
	
})();