/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessProviderDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.SearchBox"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

(function(){
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessProviderDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ProcessProviderDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages");//$NON-NLS-1$ //$NON-NLS-2$	
			this._owningApplicationKey = args.owningApplicationKey;
			this._projectAreaItemId = args.projectAreaItemId;
			this._isEmbedded = args.isEmbedded;
		},
		
		postCreate: function(){
			this.connect(this.processProviderContainerSelector, "onchange", this._handleProcessProviderContainerSelection); //$NON-NLS-1$
			this.connect(this.processSelector, "onchange", this._handleProcessProviderSelection); //$NON-NLS-1$
			this._createSearchBox();
			this._getProcessProviderContainers("", true); //$NON-NLS-1$
		},
		
		startup: function(){
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			// not ideal, but need to do this to make the search box focusable
			this._searchBox._setScope({
				scope: this._searchBox.scopes[0],
				focusText: true
			});
			
			this.inherited(arguments);
		},
		
		_setDialogEnabled: function(enable, message){
			this.processSelector.disabled = !enable;
			this.dialogHeading.innerHTML = message;
			this.okButton.disabled = !this._shouldEnableOkButton(enable);
		},
		
		_shouldEnableOkButton: function(enable){
			if (enable) {
				return !this.processSelector.disabled && this.processSelector.selectedIndex > -1;
			}
			return false;
		},

		/**
		 * Gets the list of applications registered as friends with this
		 * application, whose title or rootservices location's hostname contain
		 * the given filter. Returns all friends if an undefined/empty filter is
		 * given. Always filters out internal friends and this application's
		 * configured JTS.
		 * 
		 * @param filter {String} the string to query with
		 * @param initial {boolean} if populating provider containers initially,
		 *            selects "Local" by default
		 */
		_getProcessProviderContainers: function(filter, initial){
			this._setDialogEnabled(false, this.messages.ProcessProviderDialog_FetchingProcessProviderContainers);
			var args = {};
			if (filter && filter !== "") { //$NON-NLS-1$
				args.filter = filter;
			}
			var callback = initial ? "_populateProcessProviderContainersInitial" : "_populateProcessProviderContainers"; //$NON-NLS-1$ //$NON-NLS-2$
			var srh = new ServiceResponseHandler(this, callback, "_handleFetchError"); //$NON-NLS-1$
            ProcessClient.getProcessProviderContainers(args, srh);
		},
		
		/*
		 * see _populateProcessProviderContainers: function(containers, initial)
		 */
		_populateProcessProviderContainersInitial: function(containers) {
			this._populateProcessProviderContainers(containers, true);
		},

		/**
		 * Populates the application list with options for each of the given
		 * containers.
		 * 
		 * @param containers {ProcessProviderContainerDTO[]} 
		 * @param initial {boolean} if <code>true</code>, assumes the first
		 *            object in containers is the "Local" option and selects it
		 */
		_populateProcessProviderContainers: function(containers, initial) {
			dojo.empty(this.processProviderContainerSelector);
			
			if (containers) {
				var newOptions = dojo.map(containers, function(container){
					var containerName = dojo.string.substitute(this.messages.ProcessProviderDialog_ProviderContainerName, [container.name, container.hostName]);
	                var option = new Option(containerName, containerName);
	                option.container = container;
	                return option;
				}, this);
				
				newOptions.sort(function(a, b){
					if (a.container.local) {
						return -1;
					}
					if (b.container.local) {
						return 1;
					}
					return a.text.localeCompare(b.text);
				});
				
				dojo.forEach(newOptions, function(option){	
				    // Do not show remote providers if this page is running on the JTS - Task 204695			             
			        if (option.container.local || !this._isEmbedded) {
					    if (dojo.isIE) {
						    this.processProviderContainerSelector.options[this.processProviderContainerSelector.options.length] = option;
					    } else {
						    this.processProviderContainerSelector.appendChild(option);
					    }
				    }
				}, this);
			}
			
			if (initial) {
				// start with "Local" selected
				this.processProviderContainerSelector.selectedIndex = 0;
			} else {
				this.processProviderContainerSelector.selectedIndex = -1;
			}
			this._handleProcessProviderContainerSelection();
		},

		/**
		 * Populates the project area list based on the currently selected
		 * application. If a remote application is selected, uses the
		 * rootservices URL for that application to get its process providers
		 * service URL, and then makes a GET to that URL to get any process
		 * providing projects it has. Asks ProcessWebUIService to query the
		 * database for process providers in the local case.
		 */
		_handleProcessProviderContainerSelection : function() {
			if (this.processProviderContainerSelector.selectedIndex >= 0) { 
				this.selectedProcessProviderContainer = this.processProviderContainerSelector.options[this.processProviderContainerSelector.selectedIndex].container;
				this._setDialogEnabled(false, this.messages.ProcessProviderDialog_Loading);
			} else {
				this.selectedProcessProviderContainer = null;
				this._setDialogEnabled(false, this.messages.ProcessProviderDialog_0);
			}
			
			dojo.empty(this.processSelector);
			
			if (this.selectedProcessProviderContainer && this.selectedProcessProviderContainer.local) {
				// local case
				var srh = new ServiceResponseHandler(this, "_populateProcessProvidersFromLocalContainer", "_handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getAllProcessProviders({
            		owningApplicationKey: this._owningApplicationKey
            	}, srh);
			} else if (this.selectedProcessProviderContainer) {
				// remote case
				if (this.selectedProcessProviderContainer.errorMsg) {
					// already tried to contact this application
					this._setDialogEnabled(false, this.selectedProcessProviderContainer.errorMsg);
				} else {
					if (this.selectedProcessProviderContainer.providersServiceUrl) {
						// already have providers service url, GET it
						jazz.client.xhrGet({
							url: this.selectedProcessProviderContainer.providersServiceUrl,
							headers: {
								"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
							},
							handleAs: "xml", //$NON-NLS-1$
							load: dojo.hitch(this, this._populateProcessProvidersFromRemoteContainer),
							error: dojo.hitch(this, this._handleFetchError)
						});
					} else {
						// need to get the providers service and compatibility
						// of this application
						var handlerObj = {
							_handleGetInfo: dojo.hitch(this, function(info) {
								if (info.providersServiceUrl) {
									this.selectedProcessProviderContainer.providersServiceUrl = info.providersServiceUrl;									
								} else {
									this.selectedProcessProviderContainer.errorMsg = info.compatibilityMessage;									
								}
								this._handleProcessProviderContainerSelection();
							}),
							_handleError: dojo.hitch(this, function(error) {
								this.selectedProcessProviderContainer.errorMsg = this._handleFetchError(error);
							})
						};
						var srh = new ServiceResponseHandler(handlerObj, "_handleGetInfo", "_handleError"); //$NON-NLS-1$ //$NON-NLS-2$
						ProcessClient.getProcessProviderContainerInfo({
							'owningApplicationKey': this._owningApplicationKey, //$NON-NLS-1$
							'providerRootServicesUrl': this.selectedProcessProviderContainer.rootServicesUrl //$NON-NLS-1$
						}, srh);
					}					
				}
			}
		},
		
		_populateProcessProvidersFromLocalContainer: function(processes){
			if (processes) {
				this._populateOptionsFromLocalContainer(processes);
				this._setDialogEnabled(true, this.messages.ProcessProviderDialog_Prompt);
			} else {
				this._handleNoProcessProviders();
			}
		},
		
		_populateOptionsFromLocalContainer: function(processes){
			var processesToShow = new Array();
			//first get rid of the project area that this provider will provide for (if it is in the collection)
			dojo.forEach(processes, function(process){
				if (process.itemId !== this._projectAreaItemId) {
					processesToShow.push(process);
				}
			}, this);
			
			var newOptions = dojo.map(processesToShow, function(process){
                var option = new Option(process.name, process.itemId);
                option.process = process;
                return option;
			}, this);
			
			newOptions.sort(function(a, b){
				return a.text.localeCompare(b.text);
			});
			
			dojo.forEach(newOptions, function(option){
				if (dojo.isIE) {
					this.processSelector.options[this.processSelector.options.length] = option;
				} else {
					this.processSelector.appendChild(option);
				}
			}, this);			
		},
		
		_populateProcessProvidersFromRemoteContainer: function(processesXML){
			var remoteProcesses = this._parseRemoteProcesses(processesXML);
			if (remoteProcesses && remoteProcesses.length !== 0) {
				this._populateOptionsFromRemoteContainer(remoteProcesses);
				this._setDialogEnabled(true, this.messages.ProcessProviderDialog_Prompt);
			} else {
				this._handleNoProcessProviders();
			}
		},
		
		_parseRemoteProcesses: function(processesXML) {
			var remoteProcesses = [];
			
			var children = processesXML.childNodes;
			var projectAreasNode = UIHelper.getNode(children, "project-areas", ProcessConstants.NAMESPACE_JP06); //$NON-NLS-1$
			if (projectAreasNode) {
				var projectAreaNodes = UIHelper.getNodes(projectAreasNode.childNodes, "project-area", ProcessConstants.NAMESPACE_JP06); //$NON-NLS-1$
				dojo.forEach(projectAreaNodes, dojo.hitch(this, function(projectAreaNode){
					var urlNode = UIHelper.getNode(projectAreaNode.childNodes, "url", ProcessConstants.NAMESPACE_JP06); //$NON-NLS-1$
					var url = urlNode.firstChild.data;
					
					var titleNode = UIHelper.getAttributeNode(projectAreaNode, "name", ProcessConstants.NAMESPACE_JP06); //$NON-NLS-1$
					var title = titleNode.firstChild.data;
					
					remoteProcesses.push({
						url: url,
						name: title
					});
				}));
				
				return remoteProcesses;
			}
		},
		
		_populateOptionsFromRemoteContainer: function(remoteProcesses){
			var newOptions = dojo.map(remoteProcesses, function(remoteProcess){
                var option = new Option(remoteProcess.name, remoteProcess.name);
                option.process = remoteProcess;
                return option;
			}, this);
			
			newOptions.sort(function(a, b){
				return a.text.localeCompare(b.text);
			});
			
			dojo.forEach(newOptions, function(option){
				if (dojo.isIE) {
					this.processSelector.options[this.processSelector.options.length] = option;
				} else {
					this.processSelector.appendChild(option);
				}
			}, this);
		},
		
		
        /**
         * Sets the selected process information and enables the OK button
         */
		_handleProcessProviderSelection: function(){
			this.okButton.disabled = !this._shouldEnableOkButton(true);
			this.selectedProcess = this.processSelector.options[this.processSelector.selectedIndex].process;
			if (!this.selectedProcessProviderContainer.local) {
				this.selectedProcess.serverTitle = this.selectedProcessProviderContainer.name;
			}
		},
		
		_handleNoProcessProviders: function(){
			this._setDialogEnabled(false, this.messages.ProcessProviderDialogHTML_NoProcesses);
		},
		
		_handleFetchError: function(error){
			if (error.message && error.message !== "undefined") { //$NON-NLS-1$
				this._reportErrorFetchingRemote(error.message);
				return error.message;
			} else if (error.responseText && error.responseText !== "undefined") { //$NON-NLS-1$
				this._reportErrorFetchingRemote(error.responseText);
				return error.responseText;
			} else {
				this._reportErrorFetchingRemote(this.messages.ProcessProviderDialog_NoErrorDetails);
				return this.messages.ProcessProviderDialog_NoErrorDetails;
			}			
		},
		
		_reportErrorFetchingRemote: function(errorMessage, contextHeaderMessage){
			var iconPath = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/warning.gif"); //$NON-NLS-1$ //$NON-NLS-2$ 
			var imgHtml = "<img src=\"" + iconPath + "\"\/>"; //$NON-NLS-1$ //$NON-NLS-2$
			var brHtml = "<br />"; //$NON-NLS-1$
			if (!contextHeaderMessage) {
				contextHeaderMessage = this.messages.ProcessProviderDialog_ErrorFetchingRemote;
			}
			
			this.dialogHeading.innerHTML = imgHtml + " " + contextHeaderMessage + brHtml + brHtml + errorMessage; //$NON-NLS-1$
			this.okButton.disabled = true;
		},

		/**
		 * Adds the application search box to the dialog. A search will be
		 * triggered after typing or clicking the search button.
		 */
		_createSearchBox: function(){
			this._searchBox = new jazz.ui.SearchBox({
				scopes: [{
					id: "", //$NON-NLS-1$
					label: this.messages.ProcessProviderDialog_1,
					icon: "" //$NON-NLS-1$
				}],
				onUpdate: dojo.hitch(this, function(filter){
					this._getProcessProviderContainers(filter);
				}),
				onExecute: dojo.hitch(this, function(filter){
					this._getProcessProviderContainers(filter);
				})
			});
			this.processProviderContainerHeading.appendChild(this._searchBox.domNode);
		}
	});
})();
