/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaRecentlyViewed"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("jazz.app.viewHistory"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

(function(){
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var viewHistory = jazz.app.viewHistory;
	var JTS_APP = jazz.app.internal.applicationAbout.findAboutUriFromContext(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL);
	var PROJECTAREAS_TYPE = "com.ibm.team.process.projectareas"; //$NON-NLS-1$
	var viewHistoryQuery = {
			application: JTS_APP,
			type: PROJECTAREAS_TYPE 
		};
	
	com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaRecentlyViewed = {
	
		addRecentlyViewedProjectArea: function(recentlyViewedProjectArea){
			this._doAdd(recentlyViewedProjectArea);
		},
		
		/**
		 * Add the recently viewed project area into the cache, keep the elements in the cache to be in descending order by the last visit time
		 * @param {Object} recentlyViewedProjectArea recently viewed project area to be show in the menu item
		 */
		_doAdd: function(recent){
			recent.href = "#action=com.ibm.team.process.editProjectArea&itemId=" + recent.projectAreaItemId; //$NON-NLS-1$
			dojo.publish(ProcessConstants.PROJECT_AREA_RECENTLY_VIEWED, [recent]);
			viewHistory.put(viewHistoryQuery, this._createHistoryEntry(recent))
		},
		
		/**
		 * Create an history entry, which will be put into jazz.app.viewHistory
		 * @param {Object} recentlyViewedProjectArea
		 */
		_createHistoryEntry: function(recent) {
			var uri;
			var icon = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/projectarea_obj.gif").toString(); //$NON-NLS-1$ //$NON-NLS-2$
			if (!recent.isInitialized) {
				icon = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/uninitialized_pa_obj.gif").toString(); //$NON-NLS-1$ //$NON-NLS-2$
			}
			if (recent.isArchived) {
				icon = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/projectAreaIcon_archived.png").toString(); //$NON-NLS-1$ //$NON-NLS-2$
			}
			var root = new dojo._Url(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL);
			if(root) {
				icon = root.toString() + icon.split(root.path)[1];
				uri = root.toString() + "/process/project-areas/" + recent.projectAreaItemId; //$NON-NLS-1$
			}
			return {
				label: this._escape(recent.projectAreaName),
				iconUri: icon,
				data: recent.href,
				uri: uri
			};
		},
		
		/**
		 * Escape the special html characters in the text
		 * @param {String} text
		 */
		_escape: function(text) {
			return jazz.util.html.escape(text);
		}
		
	};
	
})();
