/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PromptDialog"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.require("jazz.ui.StyledBox"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var TrimStyles = jazz.ui.StyledBox.TrimStyles;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PromptDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/PromptDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/**
		 * Creates a simple dialog that displays a message to the user and
		 * offers ok/cancel buttons whose text is configurable
		 * 
		 * @param args {Object} that contains the following:
		 * 
		 * <code>message</code> [required], a string that will be displayed
		 *                      in the dialog
		 * 
		 * <code>okCallback</code> [optional], a function with no arguments
		 *                         that will be invoked if the user clicks
		 *                         the OK button
		 * 
		 * <code>cancelCallback</code> [optional], a function with no arguments
		 *                             that will be invoked if the user clicks
		 *                             the Cancel button
		 * 
		 * <code>okText</code> [optional], if specified, will replace "OK"
		 *                     as the label for the OK button
		 * 
		 * <code>cancelText</code> [optional], if specified, will replace
		 *                         "Cancel" as the label for the Cancel button
		 * 
		 * <code>titleText</code> [optional], if specified, will be the title
		 *                        in the dialog's window header. If not
		 *                        specified, the dialog will have no window
		 *                        header
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	
		},
		
		postCreate: function() {
			var args = {
				contentNode: this.domNode,
				width: "30em", //$NON-NLS-1$
				destroyContents: true,
				closable: false
			};
			if (this.titleText) {
				args.primaryTitle = this.titleText;
			} else {
				args.headerless = true;
			}
			this.dialog = new jazz.ui.Dialog(args);
			
			var htmlText = this.message;
			var index = htmlText.indexOf("\n"); //$NON-NLS-1$
			while (index !== -1) {
				htmlText = htmlText.replace("\n", "<br/>"); //$NON-NLS-1$ //$NON-NLS-2$
				index = htmlText.indexOf("\n"); //$NON-NLS-1$
			}
			this.messageContainer.innerHTML = htmlText;
			
			this.okButton.innerHTML = this.okText || this.messages.PromptDialog_0;
			this.cancelButton.innerHTML = this.cancelText || this.messages.PromptDialog_1;
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			
			this.connect(this.okButton, "onclick", this._okCallback); //$NON-NLS-1$
			this.connect(this.cancelButton, "onclick", this._cancelCallback); //$NON-NLS-1$
		},
		
		_okCallback: function(e) {
			this.close(e);
			if (this.okCallback) {
				this.okCallback();
			}
		},
		
		_cancelCallback: function(e) {
			this.close(e);
			if (this.cancelCallback) {
				this.cancelCallback();
			}
		},
		
		close: function(e) {
			dojo.stopEvent(e);
			this.dialog.close();
		}
	});
})();
