/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.SelectBehaviorConfigurationWidget"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTable"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var CONFIGURATION_TABLE_PAGE_SIZE = 5;
	
	/**
	 * A general-purpose widget that presents a table of operation behavior
	 * configurations in a given project/team area to select from.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.SelectBehaviorConfigurationWidget", //$NON-NLS-1$
			[dijit._Widget, dijit._Templated], {
		
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/SelectBehaviorConfigurationWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/**
		 * The argument must be an object with the following properties:
		 * 
		 * configurationStore
		 *     An instance of a OperationBehaviorConfigurationStore that
		 *     contains configurations that are to be presented
		 * 
		 * extensionStore
		 *     An instance of OperationBehaviorExtensionStore, typically one per
		 *     process editing session
		 * 
		 * selectionChangedCallback
		 *     A function that will be invoked when a configuration is selected.
		 *     A single argument will be passed: the configuration item that was
		 *     selected
		 * 
		 * pageSize (optional)
		 *     The page size for the table of configurations (the default is
		 * 	   CONFIGURATION_TABLE_PAGE_SIZE in this module)
		 * 	
		 * summaryText (optional)
		 *     A string to display at the top of the widget
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			if (this.summaryText !== undefined) {
				this.summaryTextNode.innerHTML = jazz.util.html.removeScript(this.summaryText);
			}
			
			this._operationBehaviorTable = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTable({
				configurationStore: this.configurationStore,
				extensionStore: this.extensionStore,
				pageSize: this.pageSize || CONFIGURATION_TABLE_PAGE_SIZE
			}).placeAt(this.domNode);
			this._operationBehaviorTable.populateTable();
			
			this.connect(this._operationBehaviorTable, "rowClicked", "_configurationRowClicked"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._operationBehaviorTable, "showPage", "_pageChanged"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_configurationRowClicked: function(rowNode, configurationItem) {
			if (rowNode !== this._selectedConfigurationRow) {
				if (this._selectedConfigurationRow) {
					dojo.removeClass(this._selectedConfigurationRow, "selectedConfigurationRow"); //$NON-NLS-1$
				}
				
				this._selectedConfigurationRow = rowNode;
				dojo.addClass(this._selectedConfigurationRow, "selectedConfigurationRow"); //$NON-NLS-1$
				this._selectedConfiguration = configurationItem;
				
				this.selectionChangedCallback({
					selectedConfigurationItem: this._selectedConfiguration 
				});
			}
		},
		
		_pageChanged: function() {
			if (undefined !== this._selectedConfigurationRow) {
				var rowToSelect = this._operationBehaviorTable.findRow(this._selectedConfiguration);
				if (null === rowToSelect) {
					//The selected row is no longer visible. Clear our
					//selected row state and notify our listener
					this._selectedConfigurationRow = undefined;
					this._selectedConfiguration = undefined;
					this.selectionChangedCallback({
						selectedConfigurationItem: this._selectedConfiguration 
					});	
				} else {
					//There is a row for the item that was selected. Remove
					//the selected style from the previous row, and add it to
					//the new one
					dojo.removeClass(this._selectedConfigurationRow, "selectedConfigurationRow"); //$NON-NLS-1$
					this._selectedConfigurationRow = rowToSelect;
					dojo.addClass(this._selectedConfigurationRow, "selectedConfigurationRow"); //$NON-NLS-1$
				}
			}
		},
		
		uninitialize: function(){
			this._operationBehaviorTable.destroyRecursive();
			this.inherited(arguments);
		}
	});
})();
