/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2012. All
 * Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWidget"); //$NON-NLS-1$

	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Provides an interface for selection of one or more users. This widget
	 * uses the
	 * com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog to
	 * let the users select one or more from a list of all available users, but
	 * letting you to construct and edit the list of selected users before
	 * completing the operation.
	 * 
	 * @constructor new
	 *              com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWidget({arguments});
	 * @arguments: disableUsers:[] :An array of objects containing the itemId of
	 *             users that should appear disabled in the list. Example:{
	 *             itemId="_zQADspleEd-joaYjvz6g2w"}
	 * @since 4.0.1
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWidget", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath : dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/SelectUsersWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$

		constructor : function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._removeButtonImg = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/left-arrow.gif").toString(); //$NON-NLS-1$ //$NON-NLS-2$
			this._addButtonImg = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/right-arrow.gif").toString(); //$NON-NLS-1$ //$NON-NLS-2$
		},

		postCreate : function() {
			this._userPicker = new com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog({
				destroyContents : true,
				disableUsers : this.disableUsers,
				useInternalDialog : false,
				onOK : dojo.hitch(this, this._onUserPickerOK),
				onChange : dojo.hitch(this, this._onUserPickerChange)
			});
			this._availableUsers.appendChild(this._userPicker.domNode);

			this.connect(this._selectedUsers, "onchange", "_onSelectedMembersChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._selectedUsers, "ondblclick", "_handleRemoveButton"); //$NON-NLS-1$ //$NON-NLS-2$

			this.connect(this._addButton, "onclick", "_handleAddButton"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._removeButton, "onclick", "_handleRemoveButton"); //$NON-NLS-1$ //$NON-NLS-2$

			this._updateAllButtonsState();
		},

		_onUserPickerOK : function(newUsers) {
			if (newUsers) {
				if (newUsers.length > 0) {
					// Add selected by the userPicker users to the
					// selectedMembers list.
					for ( var i = 0; i < newUsers.length; i++) {

						var user = newUsers[i];

						var option = document.createElement("option"); //$NON-NLS-1$
						option.text = user.text;
						option.label = user.text;// This is needed for IE8.
						option._itemId = user.itemId;
						option.name = user.name;
						option.userId = user.userId;
						option.emailAddress = user.emailAddress;
						option.archived = user.archived;

						this._selectedUsers.appendChild(option);
					}
				}
				this._updateAllButtonsState();
			}

		},

		_handleAddButton : function(e) {
			dojo.stopEvent(e);
			this._userPicker.onAdd(this._userPicker.selectedUsers);
		},

		_handleRemoveButton : function(e) {
			var userPicker = this._userPicker;

			var indexes = this._getSelectedIndexes(this._selectedUsers);
			for ( var i = indexes.length - 1; i >= 0; i--) {
				var option = this._selectedUsers.options[indexes[i]];

				// ENABLE THE USER IN THE USER PICKER				
				userPicker.enableUser(option._itemId);
				
				// REMOVE FROM SELECTED USERS LIST
				this._selectedUsers.removeChild(option);
			}

			this._updateAllButtonsState();
		},

		/**
		 * Returns an array of indexes of selected options in the provided list.
		 * 
		 * @param list -
		 *            The list, which will be searched for selected options.
		 * @returns {Array} of selected indexes.
		 */
		_getSelectedIndexes : function(list) {
			var selected = [];
			for ( var i = 0; i < list.options.length; i++) {
				if (list.options[i].selected) {
					selected.push(i);
				}
			}
			return selected;
		},

		/**
		 * Allow the user to advance if they have selected any users
		 */
		hasSelectedMembers : function() {
			return (this._selectedUsers.options.length > 0);
		},

		/**
		 * Public. Returns an array of objects with info about currently
		 * selected users.
		 * 
		 * @returns {Array} of object with the following contents ...
		 */
		getSelectedUsers : function() {
			var selectedUsers = [];
			var members = this._selectedUsers;
			for ( var i = 0; i < members.options.length; i++) {
				var user = members.options[i];
				var newMember = {
					archived : user.archived,
					emailAddress : user.emailAddress,
					itemId : user._itemId,
					name : user.name,
					userId : user.userId
				};
				selectedUsers.push(newMember);
			}

			return selectedUsers;
		},

		_onUserPickerChange : function(selectedUsers) {
			// If any user is selected on the right, unselect all users on the
			// left and update the state of the dialog buttons.
			if (selectedUsers.length) {
				for ( var i = this._selectedUsers.options.length; i--;) {

					this._selectedUsers.options[i].selected = false;
				}
			}
			this._updateAllButtonsState();
		},

		_onSelectedMembersChange : function() {
			// If any user is selected on the right, unselect all users on the
			// left.
			this._userPicker.unselectAllUsers();
		},

		/**
		 * Update the state of all dialog owned buttons.
		 */
		_updateAllButtonsState : function() {
			this._addButton.disabled = this._userPicker.selectedUsers.length === 0;
			dojo.toggleClass(this._addButton, "disabled", this._addButton.disabled); //$NON-NLS-1$

			this._removeButton.disabled = this._selectedUsers.selectedIndex === -1;
			dojo.toggleClass(this._removeButton, "disabled", this._removeButton.disabled); //$NON-NLS-1$

			this.updateAllButtonsState();
		},

		/**
		 * Public. Inform the caller that the state of dialog buttons was
		 * updated.
		 */
		updateAllButtonsState : function() {
		},

		/**
		 * Public. Sets the default focus for the widget.
		 */
		setDefaultFocus: function(){
			this._userPicker.setDefaultFocus();
		},
		
		uninitialize : function() {
			this._userPicker.destroyRecursive();
			delete this._userPicker;

			this.inherited(arguments);
		}
	});
})();