/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2012. All
 * Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWizardStep"); //$NON-NLS-1$

	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWidget"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Provides a jazz.ui.wizard.WizardStep for the SelectUsersWidget.
	 * 
	 * @constructor new new
	 *              com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWizardStep({arguments});
	 * @arguments: disableUsers:[] :An array of objects containing the itemId of
	 *             users that should appear disabled in the list. Example:{
	 *             itemId="_zQADspleEd-joaYjvz6g2w"}
	 * @since 4.0.1
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWizardStep", //$NON-NLS-1$
	[jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {

		templatePath : dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/SelectUsersWizardStep.html"), //$NON-NLS-1$ //$NON-NLS-2$

		constructor : function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stepTitle = this.messages.SelectUsersWizardStep_Title;
		},

		postCreate : function() {
			var userPickerArgs = {
				destroyContents : true,
				disableUsers : this.disableUsers
			};
			this._userPicker = new com.ibm.team.process.web.ui.internal.admin.projects.SelectUsersWidget(userPickerArgs);
			this._userSelector.appendChild(this._userPicker.domNode);

			// Update the WizardStep buttons when the userPicker buttons are
			// updated.
			this.connect(this._userPicker, "updateAllButtonsState", "triggerButtonStateUpdate"); //$NON-NLS-1$ //$NON-NLS-2$
		},

		/**
		 * Overridden. Allow the user to advance if they have selected any users
		 */
		canFlipToNextPage : function() {
			return this._userPicker.hasSelectedMembers();
		},

		/**
		 * Public. Returns an array of selected users.
		 */
		getSelectedUsers : function() {
			return this._userPicker.getSelectedUsers();
		},
		
		focusStep: function() {
			this.inherited(arguments);
			
			// SET FOCUS TO THE USER PICKER SEARCH FIELD
			// We need to use setTimeout() because the focus changes after this call returns 
			setTimeout (dojo.hitch(this,function(){this._userPicker.setDefaultFocus();}), 310);// 10 x min working timeout 
		},

		uninitialize : function() {
			this._userPicker.destroyRecursive();
			delete this._userPicker;

			this.inherited(arguments);
		}
	});
})();
