/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameStatus");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.ProgressBar");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledBox");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameVerificationStep");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameSummaryStep");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.rename", "ServerRenameStatusMessages");

var SummaryStep = com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameSummaryStep;
var VerificationStep = com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameVerificationStep;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.rename", "ServerRenameStatusMessages");
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var REFRESH_DELAY = 3000;
var LOCAL_STORAGE_KEY = "com-ibm-team-repository-server-rename-verification";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameStatus", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/rename/templates/ServerRenameStatus.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function() {
		this.messages = messages;
		this.stepTitle = this.messages.boxTitle;
		this.renameInProgress = true;
		this.allApplications = {};	//keep a list of all apps (+ jts) which will have a verification step.
	},
	
	postCreate: function() {
		this.header = new WizardStepHeader({stepTitle: this.stepTitle});
		this.headerArea.appendChild(this.header.domNode);
			
		var _queryParams = dojo.queryToObject(window.location.search.slice(1));
		if (_queryParams.internal && _queryParams.internal == "true") {
			dojo.removeClass(this._internalResetStatusButton, "noDisplay");
			this.connect(this._internalResetStatusButton, "onclick", this._resetRenameStatus);
		}
		else {
			this._scheduleRefresh(0);
			this.exitLink.href = net.jazz.ajax.contextRoot() + "/admin";
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.renameInProgress
			});
		}		
	},
	
	_resetRenameStatus : function() {
		var confirmation = confirm(this.messages.renameStatusResetWarning);
		if (confirmation) {
			var srh = new ServiceResponseHandler(this, "_handleReset", "_errorResetingStatus");
       		ServerStatusClient.postServerRenameOverallStatus(srh);
		}
	},
	
	_handleReset: function() {
		this.header.showMessage({
			type: "OK",
			summary: this.messages.renameStatusReset
		});
	},
	
	_errorResetingStatus : function() {
		this.header.showMessage({
			type: "ERROR",
			summary: this.messages.renameStatusResetError
		});
	},
	
	_scheduleRefresh: function(time) {
		this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh), time);
	},
	
	refresh: function() {
		var srh = new ServiceResponseHandler(this, "handleProgressInfo", "errorFetchingProgress");
        ServerStatusClient.getServerRenameOverallStatus(srh);
	},
	
	handleProgressInfo: function(data) {
		if(this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;	
		}
		
		if(!data) {
			//No Rename in progress
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.renameNotStarted
			});
			dojo.removeClass(this.exitLink, "noDisplay");
			this.renameInProgress = false;
			this.triggerButtonStateUpdate();
			return;	
		}
		
		//Delete the DOM content
		dojo.empty(this.progressContainer);
		
		//Track overall rename status
		this.someAppsRunning = false;
		this.someAppsHaveErrors = false;
		
		this._progressBars = [];
		this._messageBoxes = [];
		dojo.forEach(data, function(app) {
			var isRunning = app.state === "IN_PROGRESS";
			if(isRunning) {
				this.someAppsRunning = true;
			}
			var div = dojo.create("div", { className: "application" }, this.progressContainer);
			dojo.create("h3", { className: "displayName", innerHTML: app.name }, div);
			
			var percent = app.percentComplete || 0;
			this.progressBar = new dijit.ProgressBar({ progress: percent + "%" });
			dojo.addClass(this.progressBar.domNode, "progressBar");
			div.appendChild(this.progressBar.domNode);
			this.progressBar.startup();
			this._progressBars.push(this.progressBar);
			
			div._errDiv = dojo.create("div", { className: "applicationError noDisplay" }, div);
			div._errorMsgBox = new jazz.ui.MessageBox();
			this._messageBoxes.push(div._errorMsgBox);
			div._errDiv.appendChild(div._errorMsgBox.domNode);
			
			//Show the inline message for Errors/Warnings even if the rename is still in progress
			if(app.resultMessage && (app.result == "ERROR" || app.result == "WARNING")) {
				this._showInlineError(div, app);
			}
			
			if(!isRunning) {
				dojo.addClass(div, app.result);
				if(app.result == "ERROR") {
					this.someAppsHaveErrors = true;
				}
			}

		}, this);
		
		if(this.someAppsRunning) {
			this._scheduleRefresh(REFRESH_DELAY);
		} else {
			//Rename complete
			if(this.someAppsHaveErrors) {
				this.header.showMessage({
					type: "ERROR",
					summary: this.messages.renameFailure
				});	
				dojo.removeClass(this.exitLink, "noDisplay");
			} else {
				this._renameSucceeded();
			}
		}
	},
	
	_showInlineError: function(node, app) {
		dojo.removeClass(node._errDiv, "noDisplay");
		node._errorMsgBox.showMessage({
			type: app.result,
			summary: app.resultMessage
		});
	},
	
	errorFetchingProgress: function(errObject) {
		this.header.showMessage({
			errorObject: errObject,
			summary: this.messages.errorFetchingStatus
		});	
	},
	
	_renameSucceeded: function() {
		this.header.showMessage({
			type: "OK",
			summary: this.messages.renameSuccess
		});
		
		//Push a verification step for the JTS
		if(this.wizard) {
			var jtsRoot = net.jazz.ajax.contextRoot();
			this.allApplications[jtsRoot] = {};
			this.wizard.addStep(new VerificationStep({ appUrl: jtsRoot + "/", title: jtsRoot }));	
		}
		
		this.renameInProgress = false;
		this.triggerButtonStateUpdate();
		
		//Now add verification steps for the registered applications
		this._getFriends();
	},
	
	_getFriends: function (callback) {
		var srh = new ServiceResponseHandler(this, "_processFriends", "_errorFetchingFriends");
		OAuthAdminClient.getFriends(srh, { includeInternal: true });
	},
	
	_errorFetchingFriends: function(errorObject) {
		this.header.showMessage({
			errorObject: errorObject,
			summary: this.messages.errorFetchingApps
		});
	},
	
	_processFriends: function (friends) {
		if(friends) {
			friends = this.sortFriends(friends);
			dojo.forEach(friends, function(friend){
				if(friend.internal) {
					var scrURL = friend.scrURL;
					var contextRoot = scrURL.substring(0,scrURL.lastIndexOf("/scr") + 1);
					//push a new verification step into the wizard
					if(this.wizard) {
						this.allApplications[contextRoot] = {};
						this.wizard.addStep(new VerificationStep({ appUrl: contextRoot, title: friend.title }));
					}
				}
	        }, this);
		}
		//Push the summary step
		this.wizard.addStep(new SummaryStep());
	},
	
	sortFriends: function (friends) {	
		return friends.sort(function(a, b){
			var aTitle = a.title.toLowerCase();
			var bTitle = b.title.toLowerCase();
			return aTitle > bTitle ? 1 : -1;
		});
	},
	
	
	customOnNext: function() {
		try {
			localStorage[LOCAL_STORAGE_KEY] = dojo.toJson(this.allApplications);
		}
		catch (e) {
			console.error("This browser does not support localStorage. Unable to save flagged links.");
		}
	},
	
	canFlipToNextPage: function() {
		return !this.renameInProgress;
	},
	
	canFinish: function() {
		return false;
	},
	
	uninitialize: function () {
		this.header.destroyRecursive();
		if(this._messageBoxes) {
			dojo.forEach(this._messageBoxes, function(mb) {
				mb.destroyRecursive();
			});	
		}
		if(this._progressBars) {
			dojo.forEach(this._progressBars, function(pb) {
				pb.destroyRecursive();
			});	
		}
		this.inherited(arguments);
    }

});

})();