/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameVerificationStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.rename.schema.ServerRenameSchema");
dojo.require("jazz.util.binding");
dojo.require("jazz.app.internal.util.DiscoveredService");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.rename", "ServerRenameStatusMessages");

var ServerRenameSchema = com.ibm.team.repository.web.ui.internal.admin.rename.schema.ServerRenameSchema;
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.rename", "ServerRenameStatusMessages");
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var binding = jazz.util.binding;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var LOCAL_STORAGE_KEY = "com-ibm-team-repository-server-rename-verification";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameVerificationStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/rename/templates/ServerRenameVerificationStep.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function(args) {
		this.messages = messages;
		this.appTitle = args && args.title ? " (" + args.title + ")": "";
		this.stepTitle = this.messages.verificationStepTitle + this.appTitle;
		if(args && args.appUrl) {
			this.appUrl = args.appUrl;
		}
		this.totalFlagged = 0;
	},
	
	postCreate: function() {
		this.header = new WizardStepHeader({stepTitle: this.stepTitle});
		this.headerArea.appendChild(this.header.domNode);
		
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		this.headers[HttpConstants.ACCEPT] = HttpConstants.CT_APPLICATION_RDF_XML;
	
		this.buildTable();
		this.getVerificationEndpoint();
	},
	
	getVerificationEndpoint: function() {
		var d = DiscoveredService.loadDiscoveryServices(NAMESPACES.jfs + "RenameVerificationURLProvider", VERIFICATION_PROVIDER_DISCOVERY_BINDING, true);
		d.addCallback(dojo.hitch(this, function(services){
			this.verificationEndpoint = null;
			if(services && services.RenameVerificationURLProvider) {
				for(var i=0; i < services.RenameVerificationURLProvider.length; i++) {
					var url = services.RenameVerificationURLProvider[i].service;
					if(url.indexOf(this.appUrl) !== -1) {
						this.verificationEndpoint = url;
						break;	
					}
				}
			}
			if(!this.verificationEndpoint) {
				this.header.showMessage({
					type: "INFO",
					closable: true,
					summary: this.messages.noVerificationUrl
				});
				dojo.addClass(this.loading, "noDisplay");
				console.error("Unable to get the Server Rename Verification URL for: "  + this.appUrl);
			} else {
				this.fetchUrls();
			}
		}));
	},
	
	focusStep: function() {
		this.allApplications = dojo.fromJson(localStorage[LOCAL_STORAGE_KEY]);	
	},
	
	/*
	 * 	+ Application { }
	 * 		+ Title
	 * 		+ Base URL
	 * 		+ Categories[ ]
	 * 			+ [0] categoryObject { }
	 * 				+ name
	 * 				+ links [ ]
	 * 					+ [0] linkObject {}
	 * 						+ url
	 * 						+ label
	 * 						+ flagged (optional)
	 */
	
	fetchUrls: function() {
		
		this.categories = [];
		
		jazz.client.xhrGet({
			url: this.verificationEndpoint,
			handleAs: "xml",
			headers: this.headers,
			load: dojo.hitch(this, function(doc) {
				if(doc) {
					var parsed = binding.read(doc.documentElement, ServerRenameSchema.verificationUrls);	
				}
				if(parsed && parsed.publicUrl) {
					this.publicUrl = parsed.publicUrl;
					var pindex = this.getOrSetCategory(this.messages.publicUrl);
					this.categories[pindex].links.push({
						url: this.publicUrl,
						category: this.messages.publicUrl
					});
				}
				if(parsed && parsed.links) {
					links = this.sortByCategory(parsed.links);
					var last_category = "";
					var cat_index = null;
					dojo.forEach(links, function(link) {
						if(link.category === last_category) {
							//we already know the index, no need to search again
						} else {
							cat_index = this.getOrSetCategory(link.category);
							last_category = link.category;
						}
						var category = 	this.categories[cat_index];
						category.links.push(link);

					}, this);
				}
				this.displayAllLinks();

			}),
			error: dojo.hitch(this, function(errObject) {
				this.header.showMessage({
					errorObject: errObject,
					summary: this.messages.errorFetchingUrls
				});
				dojo.addClass(this.loading, "noDisplay");
			})
		});
	},
	
	
	buildTable: function() {
		var columns = [
			{
				width: "20%",
				label: this.messages.h_category,
				sortable: false
			}, {
				width: "26%",
				label: this.messages.h_label,
				sortable: false
			}, {
				width: "50%",
				label: this.messages.h_url,
				sortable: false
			}, {
				width: "4%",
				label: dojo.create("span", { className: "flagHeading", title: this.messages.h_flag }),
				sortable: false
			}
		];
		
		this.styledTable = new jazz.ui.StyledTable({
			columns: columns
		});
		this.linkContainer.appendChild(this.styledTable.domNode);
	},
	
	displayAllLinks: function() {
		dojo.addClass(this.loading, "noDisplay");
		if(this.categories && this.categories.length > 0) {
			dojo.removeClass(this.linkContainer, "noDisplay");
		} else {
			//no links found
			return;
		}
		for (var i = 0; i < this.categories.length; i++) {
			var cat = this.categories[i];
			
			//sort all of the links within this category
			if(cat && cat.links && cat.links.length) {
				cat.links = this.sortByLabelThenURL(cat.links);
			} else {
				//no links in this category (highly unlikely)
				return;
			}
			
			for(var x = 0; x < cat.links.length; x++) {
				var tr = dojo.create("tr", {}, this.styledTable.tbody);
				var link = cat.links[x];
				dojo.create("td", { innerHTML: cat.name },tr);
				dojo.create("td", { innerHTML: link.label || "", className: "linkLabel" },tr);
				var urlTd = dojo.create("td", { },tr);
				dojo.create("a", { href: link.url, innerHTML: link.url, target: "_blank" }, urlTd);
				var flagTd = dojo.create("td", { },tr);
				var flagLink = dojo.create("a", { href: "#", className: "flag", title: this.messages.flagHover }, flagTd);
				//Keep track of the indexes
				flagLink.callbackData = {
					url: link.url,
					cat_index: i,
					link_index: x
				};
				this.connect(flagLink, "onclick", function(e){
					dojo.stopEvent(e);
					var data = e.target.callbackData;
					var isFlaggedAlready = this.categories[data.cat_index].links[data.link_index].flagged;
					if(isFlaggedAlready) {
						//remove the flag
						this.categories[data.cat_index].links[data.link_index].flagged = false;
						dojo.removeClass(e.target, "selected");
						this.header.hideMessage();
						this.totalFlagged--;
					} else {
						this.categories[data.cat_index].links[data.link_index].flagged = true;
						dojo.addClass(e.target, "selected");
						this.totalFlagged++;
						this.header.showMessage({
							type: "INFO",
							closable: true,
							summary: dojo.string.substitute(messages.urlMarked, [ "<strong>" + this.totalFlagged + "</strong>" ]),
						});
					}
				});
			}
		}
	},
	
	// Returns the index of the category object. Creates new category if index is not found.
	getOrSetCategory: function(catName) {
		if(!catName) return null;
		
		for(var i=0; i < this.categories.length; i++) {
			if(this.categories[i].name === catName) {
				return i;
			}
		}
		//Category not found, add it.
		this.categories.push({
			name: catName,
			links: []
		});
		return this.categories.length - 1; //will be the index of our new category
	},
	
	customOnNext: function() {
		//print out the updated array
		this.allApplications[this.appUrl] = {
			title: this.appTitle,
			publicUrl: this.publicUrl,
			categories: this.categories
		};
		try {
			localStorage[LOCAL_STORAGE_KEY] = dojo.toJson(this.allApplications);
		}
		catch (e) {
			console.error("This browser does not support localStorage. Unable to save flagged links.");
		}
	},
	
	sortByCategory: function(links) {
		return links.sort(function(a, b){
			if(!a || !a.category) return 1;
			if(!b || !b.category) return -1;
			var aCat = a.category.toLowerCase();
			var bCat = b.category.toLowerCase();
			return aCat > bCat ? 1 : -1;
		});
	},
	
	sortByLabelThenURL: function(links) {
		return links.sort(function(a, b){
			if(!a || !a.label) return 1;
			if(!b || !b.label) return -1;
			var aLabel = a.label.toLowerCase();
			var bLabel = b.label.toLowerCase();
			if(aLabel === bLabel) {
				//then sort by URL
				var aUrl = a.url.toLowerCase();
				var bUrl = b.url.toLowerCase();
				return aUrl > bUrl ? 1 : -1;
			}
			return aLabel > bLabel ? 1 : -1;
		});
	},

	uninitialize: function () {
		this.header.destroyRecursive();
		this.inherited(arguments);
    }

});

var NAMESPACES = {
	jd: "http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
	jfs: "http://jazz.net/xmlns/foundation/1.0/",
	rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
};
var RENAME_INFO = {
	namespaces: NAMESPACES,
	service: ["string", "jd:service/@rdf:resource", "1"]
};
var VERIFICATION_PROVIDER_DISCOVERY_BINDING = {
	namespaces: NAMESPACES,
	RenameVerificationURLProvider: [RENAME_INFO, "jfs:RenameVerificationURLProvider", "*"]
};

})();