/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.AuthorizeOAuth");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("jazz.ui.MessageBox");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin", "NotAuthorizedViewMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");

var messages;
var setupMessages;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var MessageBox = jazz.ui.MessageBox
var NotAuthorizedView = com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView;


dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.AuthorizeOAuth", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/AuthorizeOAuth.html"),
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
		this.setupMessages = setupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	
	
	postCreate: function() {
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		this.getUserRoles();
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "serviceError");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this._userIsAdmin = true;
				break;
			}
		}
		if (this._userIsAdmin) {
			this.showAuthroizedView();
		} else {
			this.showNonAuthorizedView();
		}
	},
			
	showAuthroizedView: function () {
		this.evaluateSearchParams();
		this.getProvisionalRegistration();
	},
	
	showNonAuthorizedView: function () {
		var notAuth= new NotAuthorizedView({applicationName: this.messages.moduleHeading, showLogout: true});
		this.approvalResultsArea.appendChild(notAuth.domNode);

		dojo.toggleClass(this.keyApprovalArea, "noDisplay", true);
		dojo.toggleClass(this.approvalResultsArea, "noDisplay", false);
	},

	getProvisionalRegistration: function () {
		if (this.key) {
			var srh = new ServiceResponseHandler(this, "displayProvisionalRegistrations", "serviceError");
			OAuthAdminClient.getProvisionalRegistrationByKey(srh, this.key);
		}
	},
	
	displayProvisionalRegistrations: function (provisional) {
		if (provisional){	
			var summary = dojo.string.substitute(this.messages.moduleSummary, [ window.location.hostname, provisional.name]);
			this.summaryArea.appendChild(document.createTextNode(summary));

			var name = document.createElement("input");
			name.value = provisional.name;
			name.type = "text";
			this.nameValueArea.appendChild(name);
			this.nameInput = name;
			
			this.keyValueArea.appendChild(document.createTextNode(provisional.key));
			
			var request = provisional.trusted ? messages.trustedValue : messages.untrustedValue
			this.requestValueArea.appendChild(document.createTextNode(request));
			
			grantTrusted = document.createElement("input");
			grantTrusted.type = "checkbox";
			grantTrusted.className = "checkBox"
			this.grantArea.appendChild(grantTrusted);
			this.grantTrusted = grantTrusted;
			this.grantTrusted.checked = provisional.trusted;
			
			dojo.toggleClass(this.keyApprovalArea, "noDisplay", false);
			dojo.toggleClass(this.approvalResultsArea, "noDisplay", true);
		}
	},
	
	approveProvisional: function () {
		this.saveProvisional(true);
		this.approved = true;
	},
	
	rejectProvisional: function () {
		this.saveProvisional(false);
	},
			
	saveProvisional: function (approve) {
		var srh = new ServiceResponseHandler(this, "successSaveRegistration", "serviceError");
		OAuthAdminClient.acceptProvisionalRegistration(srh, this.nameInput.value, this.key, this.grantTrusted.checked, approve);
	},
	
	successSaveRegistration: function(response){
		dojo.toggleClass(this.keyApprovalArea, "noDisplay", true);
		dojo.toggleClass(this.approvalResultsArea, "noDisplay", false);
		dojo.addClass(this.approvalResultsArea, this.approved ? "icon-ok" : "icon-error");
		var status = this.approved ? messages.approvedLabel : messages.rejectedLabel;		
		this.approvalResultsArea.appendChild(document.createTextNode(status));		
	},
	
	serviceError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	}, 
	
	evaluateSearchParams: function(){
		var qstr = window.location.search.substr(1);
		if (qstr.length) {
			var qparts = qstr.split("&");
			for (var x = 0; x < qparts.length; x++) {
				var tp = qparts[x].split("=");
				if (tp[0] == "key") {
					this.key = decodeURIComponent(tp[1]);  // oauth key to be approved
				}
			}
		}
	}

});

})();