/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureMailSettings");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.SendTestEmailDialog");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");

(function() {

var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var MessageBox = jazz.ui.MessageBox;
var Util = com.ibm.team.repository.web.ui.internal.Util;
var ENABLE_EMAIL_NOTIFICATIONS_PROPERTY = "notification.mail.enabled";
var MAILER_SERVICE = "com.ibm.team.repository.service.jts.internal.mailer.MailerService";
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureMailSettings", [ConfigUtility, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ConfigureMailSettings.html"),
	widgetsInTemplate: true,
	_allServices: {
		"com.ibm.team.repository.service.jts.internal.mailer.MailerService": "mailTableHeading"
	},
	_propertySortOrder: [
		"notification.mail.enabled",
		"notification.mail.smtp.server",
		"notification.mail.smtp.server.port",
		"notification.mail.smtp.user",
		"notification.mail.smtp.password",
		"notification.mail.restrict_to_whitelist",
		"notification.mail.whitelist"
	],

	// widget initialization
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");
	},

	postCreate: function () {
		com.ibm.team.repository.web.ui.internal.admin.server.ConfigureMailSettings.superclass.postCreate.apply(this);
		this.header = new MessageBox();
		this.headerText.innerHTML = this.messages.mailHeading;
		this.headerArea.appendChild(this.header.domNode);
		this._initializeContent();
		this._saveButton.disabled = true;
		this._createSaveHandler(this._saveButton);
		
		//Append the context sensitive help
		this._pageHelpHover = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.emailSettings" });
		this.helpHover.appendChild(this._pageHelpHover.domNode);

		this.btnTestEmail.onClick = dojo.hitch(this, "_sendTestEmail");
		this.btnTestEmail.set("enabled", false);
		this._doesServerSupportEmail();
	},

	_sendTestEmail: function() {
		this.sendTestEmailDialog = new com.ibm.team.repository.web.ui.internal.admin.server.SendTestEmailDialog({
			onSend: dojo.hitch(this, "_handleTestSendEmail")
		});
	},

	_handleTestSendEmail: function() {
		var srh = new ServiceResponseHandler(this,"sendTestEmailCallback","sendTestEmailError");
		AdminClient.postSendTestEmail({
			emailAddress: this.sendTestEmailDialog.emailAddress.value
		}, srh);
	},

	sendTestEmailCallback: function() {
		this.sendTestEmailDialog.close();
		this.header.showMessage({ 
			type: "OK",
			closable: true,
			summary: dojo.string.substitute(this.messages.testEmailSent, [this.sendTestEmailDialog.emailAddress.value])
		});
	},

	sendTestEmailError: function(err) {
		this.sendTestEmailDialog.close();
		this.header.showMessage({ 
			type: STATUS_TYPES.ERROR,
			closable: false,
			summary: err.message
		});
	},

	_doesServerSupportEmail: function() {
		var hasEmailEnabled = false;
		var handler = {
			success: dojo.hitch(this,function(serviceInfo) {
				if(!this.domNode) return;
				if (serviceInfo.properties && serviceInfo.properties.length > 0) {
					dojo.forEach(serviceInfo.properties, function(prop){
						if (prop.name == ENABLE_EMAIL_NOTIFICATIONS_PROPERTY && prop.overrideValue === "true") {
							hasEmailEnabled = true;
							this.btnTestEmail.set("enabled", true);
							return;
						}
					}, this);
				}
			}),
			error: function(errorObj) {
				//don't show error
			}
		};
		var srh= new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getServiceConfiguration(srh, MAILER_SERVICE);
	},

	_initializeContent: function () {
		this._services = new Array();
		for (var serviceName in this._allServices) {
			var curService = new ServerPropertyService({
				"serviceInfo": {
					"implementationClassName": serviceName
				},
				"enableEdits": true,
				"serviceHeaderClass": ["header1"],
				"propertiesRowClass": [],
				"propertySortOrder": this._propertySortOrder,
				"customTitle": this.messages[this._allServices[serviceName]],
				"displayIn": this._configContainer,
				"forceShortFields": true
			});
			this._services.push(curService);
			this.connect(curService, "_markChanged", function () {
				this._enableSaveIfValid(this._saveButton);
			});
			this.connect(curService, "_checkUnchanged", function () {
				var change = false;
				for (var j=0; j < this._services.length; j++) {
					if (this._services[j]._changed) {
						change = true;
						break;
					}
				}
				if (!change) {
					this._saveButton.disabled = true;
				}
			});
		}
	},
	
	_getChanges: function () {
		var changes = {
			"changes": [],
			"properties": []
		};
		for (var i=0; i < this._services.length; i++) {
			var obj = this._services[i]._getChanges();
			changes["changes"] = changes["changes"].concat(obj["changes"]);
			changes["properties"] = changes["properties"].concat(obj["properties"]);
		}
		return changes;
	},
	
	_destroyServices: function() {
		for (var i = 0; i < this._services.length; i++) {
			this._services[i].destroyRecursive();
		}
		this._services = [];
	},
	
	_saveHandler: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ConfigureMailSettings.superclass._saveHandler.apply(this);
		for (var i=0; i < this._services.length; i++) {
			var changeList = this._services[i]._getChanges()["changes"];
			for (var j=0; j < changeList.length; j++) {
				if (changeList[j]["name"] == "notification.mail.enabled") {
					if (changeList[j]["value"]  == "true") {
						this.btnTestEmail.set("enabled", true);
					} else {
						this.btnTestEmail.set("enabled", false);
					}
				}
			}
		}
	},
	
	uninitialize: function(){
		this._destroyServices();
		
		// Destroy the message box
		this.header.destroyRecursive();
		
		this._pageHelpHover.destroyRecursive();

		if(this.sendTestEmailDialog) {
			this.sendTestEmailDialog.destroyRecursive();
		} 

		this.inherited(arguments);
	}
});

})();