/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
	
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.DiagnosticsUtil");
	
    dojo.require("dojo.i18n");
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerDiagnosticsMessages");

	var messages;
	var STATUS_OK_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/okay_obj.gif");
	var STATUS_ERR_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/error_obj.gif");
	var STATUS_WARN_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/warn_obj.gif");
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.DiagnosticsUtil", null, {
        
        constructor: function(){
            this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerDiagnosticsMessages");
        },
		
		displayDiagnosticStatus: function(response, nodeId) {
			if(response) {
				var warnings = 0;
				var errors = 0;
				dojo.forEach(response, function(test) {
					if(!test || test.disabled) return; //continue
					if(test.nodesResults && test.nodeScopedTest && nodeId) {
						var results = test.nodesResults;
						for (var key in results) { //Loop results for every node
							if(results[key].nodeID === nodeId) {
								//Found our node
								if(results[key].error) {
									errors++;
									return;
								}
								if(results[key].warning) {
									warnings++;
									return;
								}
								break;
							} 
						}
					}
					else {
						if(test.error) {
							errors++;
							return;
						}
						if(test.warning) {
							warnings++;
							return;
						}
					}
				}, this);
				
				var statusString = "";
				if(errors > 1) {
					statusString += errors + " " + this.messages.multipleErrors;
				} 
				else if(errors == 1) {
					statusString += errors + " " + this.messages.error;	
				}				
				if(warnings > 1) {
					if(statusString != "") statusString += ", ";
					statusString += warnings + " " + this.messages.multipleWarnings;
				} 
				else if(warnings == 1) {
					if(statusString != "") statusString += ", ";
					statusString += warnings + " " + this.messages.warning;	
				}
				
				if(!statusString) {
					statusString = this.messages.allOkay;
				}
				
				var node = dojo.create("div");
				var img = dojo.create("img", { className: "status-icon" }, node);
				node._displayLabel = statusString;
				
				var diagURL = "#action=com.ibm.team.repository.admin.serverDiagnostics";
				if(nodeId) {
					diagURL += "&nodeId=" + nodeId;
				}
				var detailsLink = dojo.create("a", { href: diagURL, innerHTML: statusString, style: { marginLeft: "5px" } }, node);
				
				//Set the correct status icon
				if(errors > 0) {
					this._setStatusIcon(img, "error");
				} else if(warnings > 0) {
					this._setStatusIcon(img, "warning");
				} else {
					this._setStatusIcon(img, "success");
				}
								
				return node;
			}
		},
		
		_setStatusIcon: function(image, status) {
			if(!image) return;
			switch (status) {
				case "error": 
					image.src = STATUS_ERR_IMG_SRC;
					image.alt = this.messages.status_error;
					break;
				case "warning": 
					image.src = STATUS_WARN_IMG_SRC;
					image.alt = this.messages.status_warning;
					break;
				case "success": 
					image.src = STATUS_OK_IMG_SRC;
					image.alt = this.messages.status_ok;
					break;
				default: 
					//Set the default to OK
					image.src = STATUS_OK_IMG_SRC;
					image.alt = this.messages.status_ok;
			}
			dojo.removeClass(image, "noDisplay");
		}
		
    });

})();
