/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseUsageView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date");
dojo.require("dojo.date.stamp");
dojo.require("dojox.collections.ArrayList");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseUsageViewMessages");

var ArrayList = dojox.collections.ArrayList;
var util = com.ibm.team.repository.web.ui.internal.admin.util;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var FloatingLicenseTypeCombo = com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo;
var formatTimeFromMilliseconds = com.ibm.team.repository.web.ui.internal.admin.util.formatTimeFromMilliseconds;
var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;
var MessageBox = jazz.ui.MessageBox;
var FLOATING_LICENSE_SERVICE_URL = "/service/com.ibm.team.repository.service.internal.license.IFloatingLicenseService/licenses";
var FLOATING_LICENSE_SERVICE_URL_METHOD = "/in-use";

function isNode(o){
	return o && o.nodeType && o.cloneNode;
}

function setText(e, text) {
	e.innerHTML = "";
	if (!isNode(text)) {
		text = document.createTextNode(text);
	}
	e.appendChild(text);
}

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseUsageView", com.ibm.team.repository.web.ui.internal.admin.ItemList, {

	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseUsageViewMessages");
		this.licenseId = args.licenseId;
	},
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseUsageView.superclass.postCreate.apply(this);
		this.header = new MessageBox(); 
		this.headerArea.appendChild(this.header.domNode);
		setText(this.heading,this.messages.heading);
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		this._refresh();
	},
	
	_handleTypeListLoad: function(licenseId) {
		if(!this.tableContents) return; //DOM not loaded
		if (!licenseId) {
			// No floating licenses are installed in the server
			var descNode = dojo.create("span");
			var floatinglink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses'>";
			descNode.innerHTML = dojo.string.substitute(this.messages.installFloatingLink, [ floatinglink, "</a>" ]);
			this.updateDescription(descNode);
		} else {
			this.licenseId = licenseId;
			this._fetchAndDisplayLeaseInfo();
		}
		dojo.toggleClass(this.tableContents,"noDisplay", !licenseId);
	},
	
	_handleTypeChanged: function(licenseId) {
		//Changing the action will force the widget to reload and call _refresh();
		document.location.href = "#action=com.ibm.team.repository.admin.floatingLicenseUsage&licenseId=" + licenseId;
	},
	
	_refresh: function() {
		this.header.hideMessage();
		var description = dojo.create("div", { innerHTML: this.messages.description });
		var licenseSelectDiv = dojo.create("div", { style: { "margin": "8px 0"}}, description);
		this.updateDescription(description);

		var self = this;
		var floatHelper = {
			yes: function(status) {
				var params = {
					licenseId: self.licenseId,
					onLoad: dojo.hitch(self, "_handleTypeListLoad"),
					onChange: dojo.hitch(self, "_handleTypeChanged"), 
					onError: dojo.hitch(self, "handleError")
				}	
				self.typeCombo = new FloatingLicenseTypeCombo(params);
				licenseSelectDiv.appendChild(self.typeCombo.domNode);
			},
			no: function(s) {
				self.header.showMessage({
					summary: self.messages.notEnabled
				}); 
			},
			error: function(errorObj) {
				self.handleError(errorObj);
			}
		}
		LicenseClient.checkFloatingLicensesSupported(floatHelper.yes, floatHelper.no, floatHelper.error);
	},
	
	updateDescription: function(description) {
		// using the newItemButtonArea instead of "description" since we're adding <div> tags that shouldn't 
		// be appended to a <p> tag.  The name "newItemButtonArea" doesn't apply here... 
		// just needed another <div> without changing the template now.
		dojo.removeClass(this.newItemButtonArea,"buttonArea");
		setText(this.newItemButtonArea, description);
	},
	
	_fetchAndDisplayLeaseInfo: function() {
		var self = this;
		jazz.client.xhrGet({
			url: net.jazz.ajax.contextRoot() + FLOATING_LICENSE_SERVICE_URL + "/" + self.licenseId + FLOATING_LICENSE_SERVICE_URL_METHOD,
			handleAs: "json",
			headers: this.headers,
			load: function(response) {
				self._displayActiveLeaseInfo(response);
			},
			error: function(errorObj) {
				self.handleError(errorObj);
			}
		});
	},

	_displayActiveLeaseInfo: function(info) {
		dojox.xml.parser.removeChildren(this.tbody);
		this.setHeaderRow([this.messages.server, this.messages.userId, this.messages.operation, this.messages.timeout]);
		if (info.used) {
			if (info.checkouts) {
				// Sort the leases so that the ones expiring soonest are put
				info.checkouts.sort(function(lease1, lease2){
					var date1 = dojo.date.stamp.fromISOString(lease1.endTime);
					var date2 = dojo.date.stamp.fromISOString(lease2.endTime);
					return date1.getTime() - date2.getTime();
				});
				// Add the sorted leases to the table
				dojo.forEach(info.checkouts, function(lease){
					var timeLeft = util.convertDateStringToRelativeTimeString(lease.endTime);
					var row = this.addRow(null, [lease.server, lease.userId, lease.operationId, timeLeft]);
					dojo.addClass(row.parentNode,"userTable");
				}, this);
			}
		} else {
			var row = dojo.create("tr", {}, this.tbody);
			dojo.create("td", { className: "noRowsYet", colSpan: 4, innerHTML: this.messages.noIssued }, row);
		}
	},
	
	uninitialize: function () {
		this.header.destroyRecursive();
		
		if(this.typeCombo) {
			this.typeCombo.destroyRecursive();
		}
		this.inherited(arguments);
    }

});

})();