/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.Friends");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.SetupOAuth");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("dojox.xml.parser");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");


(function() {
var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.admin.server.";
var messages;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var WizardForm = com.ibm.team.repository.web.ui.internal.wizard.WizardForm;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var SetupOAuth = com.ibm.team.repository.web.ui.internal.wizard.SetupOAuth;
var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var WizardStep = jazz.ui.wizard.WizardStep;


/*
 * Jazz Foundation 1.0 version of this code:
 * - asked the user to supply the JFS server and rootservices
 * - requested provisional key from JFS server input with hard-coded URI
 * - stored the input for JFS server as the OAuth domain (single entry)
 * 
 * Changes for 1.0.0.1:
 * - don't ask for JFS server
 * - look up the OAuth domain and provisional URI from root services
 * - use the retrieved provisional URI to request provisional key
 * - store the data retrieved from root services as the OAuth domain list
 * 
 */

dojo.declare(PACKAGE_PREFIX + "Friends", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/Friends.html"),
	_actionToolbars: [],
		
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		this.stepTitle = messages.stepTitle; 
		this.oauthFormData = {};
	},
	
	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		
		//Append the context sensitive help
		this._pageHelpHover = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.extFriends" });
		this.helpHover.appendChild(this._pageHelpHover.domNode);
		
		//Add *dynamic* page description
		this.pageDescription.innerHTML = this.isJTS ? this.messages.description : this.messages.description_app;
		this.friendsTableDescription.innerHTML = this.isJTS ? this.messages.friendsTableDescription_jts : this.messages.friendsTableDescription_app;
		
		this.reloadFriends();
	}, 
	
	reload: function () {
		this.header.hideMessage();
		dojo.toggleClass(this.provisionalStepArea, "noDisplay", true);
		this.reloadFriends();
	},
	
	reloadFriends: function () {
		dojox.xml.parser.removeChildren(this.friendsTableBody);	
		this.getFriends();
	},
		
	errorCallback: function (errorObj, contextMsg) {
		this.header.showMessage({
			errorObject: errorObj,
			summary: contextMsg
		});
	},

	setFormStatus: function (status) {
		if (status === -1) {
			this.header.hideMessage();
		}
		else {
			this.header.showMessage({
				summary: status.message,
				type: status.type
			});
		}
	}, 

	getFriends: function () {
		var srh = new SRH(this, "displayFriends", "errorFetchingFriends");
		//If the ?internal=true flag is set in the URL, we should show all friends. This will admins to edit the consumer secret for Apps.
		var args = { includeInternal: location.search.indexOf("internal") !== -1 };
		OAuthAdminClient.getFriends(srh,args);
	},
	
	displayFriends: function (friends) {		
		var self = this;
		if (friends){
			this.friendList = friends;
			if(this.friendList.length > 1) {
				this.sortFriends();
			}	
		} 

		if(!this.friendList || this.friendList.length <= 0) {
			this.showNoFriendsRow();
		} else {
			dojo.forEach(this.friendList, function(friend){
                self.addFriendListRow(friend, self.friendsTableBody, /*allowDelete*/ true);
            });
		}
	},
	
	errorFetchingFriends: function (errorObj) {
		this.errorCallback(errorObj, this.messages.fetchFriendsError);
	},
	
	sortFriends: function () {	
		return this.friendList.sort(function(a, b){
			var aTitle = a.title.toLowerCase();
			var bTitle = b.title.toLowerCase();
			return aTitle > bTitle ? 1 : -1;
		});
	},
	
	addFriendListRow: function(friend, targetTable, allowDelete) {
		if(!targetTable) return;
		
		var row = dojo.create("tr", { id: "row-" + friend.id, className: "hasAction" }, targetTable);
		var cell1 = dojo.create("td", {}, row);
		cell1.appendChild(document.createTextNode(friend.title));
		dojo.create("td", { innerHTML: friend.rootServicesURL}, row);
		dojo.create("td", { innerHTML: friend.oAuthConsumerKey}, row);
		var actions = dojo.create("td", {}, row);				
		var toolbar = new Toolbar();
		this._actionToolbars.push(toolbar);
		if (allowDelete) {
			var deleteAction = {};
			deleteAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/delete_hov.gif");
			deleteAction.label = this.messages.removeFriendButton;
			deleteAction.callbackData = {
				id: friend.id
			};
			var callback = function(e){
				dojo.stopEvent(e);
				this._deleteFriend(e.currentTarget.callbackData.id);
			};
			toolbar.addButton(this, callback, deleteAction);
		}
		var editAction = {};
		editAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit-ellipses.gif");
		editAction.label = this.messages.editFriend;
		editAction.callbackData = {
			friend: friend
		};
		var callback = function(e){
			dojo.stopEvent(e);
			this._launchEditFriend(e.currentTarget.callbackData.friend);
		};
		toolbar.addButton(this, callback, editAction);
		actions.appendChild(toolbar.domNode);
	}, 

	_deleteFriend: function(id) {
		var verifyDelete = confirm(this.messages.confirmDelete);
        if (verifyDelete) {
			this._friendIdPendingDeletion = id;
			var friend = this._getFriendForId(id);
			if (friend) {
				OAuthAdminClient.deleteFriend(new SRH(this, "removeFriendEntry", "errorDeletingFriend"), {title: friend.title});
			}
		}
	},
	
	_getFriendForId: function(id) {
		if (this.friendList) {
			for (friend in this.friendList) {
				if (this.friendList[friend].id === id) {
					return this.friendList[friend];
				}
			}
		}
		return null;
	},
	
	removeFriendEntry: function() {
		var elem = dojo.byId("row-" + this._friendIdPendingDeletion);
		this.friendsTableBody.removeChild(elem);
		if (this.friendsTableBody.childNodes.length == 0) {
			this.showNoFriendsRow();
		}
	}, 
	
	showNoFriendsRow: function() {
		var url = "<a id=\"_addFriendLink\" class=\"editLink\">" + this.messages.addFriendLink + "</a>";
		var msg = dojo.string.substitute(this.messages.noFriendsYet, [url]);
		var row = dojo.create("tr");
		dojo.create("td", {className: "noRowsYet", colSpan: 4, innerHTML: msg} , row);
		this.friendsTableBody.appendChild(row);
		this.connect(dojo.byId("_addFriendLink"), "onclick", "_launchAddFriendWizard");		
	},
	
	errorDeletingFriend: function (errorObj) {
		var contextMsg = dojo.string.substitute(this.messages.deleteFriendError, [this._friendIdPendingDeletion]);
		this.errorCallback(errorObj, contextMsg);
	}, 
	
	_launchEditFriend: function (friend) {
		this._destroyWizards();
		var caller = this;
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.EditFriendStep({
			stepTitle: this.messages.editFriendDialogTitle,
			friend: friend, 
			caller: caller
		});

		var wizardArgs = {
			steps: [step1],
			primaryTitle: this.messages.editFriendHeading,
			showStepSummary: false,
			processFinish: function(){
				this._steps[0].saveUpdates();
			},
			processCancel: function() {	
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(caller.messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}				
			}, 	
			saveSuccess: function () {
				caller.reloadFriends();
				caller.header.hideMessage();
				this.container.viewBox.close();
			}
		}
		this.wizard = new jazz.ui.Wizard(wizardArgs);
					
	},	
	
	_launchAddFriendWizard: function() {	
		this._destroyWizards();
		
		var caller = this;
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.AddFriendStep({
			stepTitle: this.messages.addFriendStepHeader,
			caller: caller
		});	
		var step2 = new com.ibm.team.repository.web.ui.internal.admin.server.AcceptProvisionalKeyStep({
			stepTitle: this.messages.step2Title,
			caller: caller
		});

		var wizardArgs = {
			steps: [step1, step2],
			primaryTitle: this.messages.addFriendStepHeader,
			width: "870px",
			showStepSummary: true,
			processFinish: function() {
				caller.reloadFriends();
				this.container.viewBox.close();
			},			
			processCancel: function() {		
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(caller.messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}		
			}
		}
		this.addFriendWizard = new jazz.ui.Wizard(wizardArgs);
	},
	
	_destroyActionToolbars: function(){
		for (var i = 0; i < this._actionToolbars.length; i++) {
			this._actionToolbars[i].destroyRecursive();
		}
		this._actionToolbars = [];
	},
	
	_destroyWizards: function() {
		if (this.wizard) {
			this.wizard.destroyRecursive();
			delete this.wizard;
		}
		if (this.addFriendWizard) {
			this.addFriendWizard.destroyRecursive();
			delete this.addFriendWizard;
		}
	},
	
	uninitialize: function(){
		this._destroyActionToolbars();
		
		// Destroy the message box
		this.header.destroyRecursive();
		
		this._pageHelpHover.destroyRecursive();		
		this._destroyWizards();
			
		this.inherited(arguments);
	}
	
});

var FORM_HEADER_DEF = [
	{type: "label", labelKey: "propertyLabel"},
	{type: "data", labelKey: "valueLabel"}
];

dojo.declare(PACKAGE_PREFIX + "OAuthRequest", WizardForm, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		
		// Create the RootServices description + example code
		this.rootServicesDesc = dojo.create("div");
		dojo.create("span", { innerHTML: this.messages.rootServicesDescription }, this.rootServicesDesc );
		var format = dojo.create("div", { innerHTML: this.messages.rootServicesFormat + " " }, this.rootServicesDesc);
		format.appendChild(document.createTextNode("https://<hostname><:port-number>/<context>/rootservices")); //NON NLS
	
		this.heading =  "";
		this.step = "";
		this.description = "";
		this.tableContent = {
			header: FORM_HEADER_DEF,
			rows: [
				{name: "title", labelKey: "titleLabel", exampleKey: "titleDescrption"},
				{name: "rootServices", labelKey: "rootServicesLabel", descriptionKey: "rootServicesExample", exampleKey: this.rootServicesDesc},
				{name: "consumerSecret", labelKey: "secretLabel", type: "password", exampleKey: "secretDescrption"},
				{name: "repeatSecret", labelKey: "secretLabel2", type: "password", exampleKey: "secretDescrption2", mustMatch: {
					matchName: "consumerSecret",
					errorMessageKey: "secretMismatch"
				}},
				{name: "trusted", labelKey: "trustedLabel", type: "checkbox", exampleKey: "trustedDescription", defaultValue: true}
			]
		};
	},
	
	postCreate: function () {
		this.inherited(arguments);
		this.validate();
	},
		
	getDataObject: function() {
		var valueMap = this.getData();
		var args = new Object();
		for (i in valueMap) {
			if (i == "repeatSecret") {
				continue;
			}
			args[i] = valueMap[i];
		}
		return args;		
	},
	
	getRootServices: function (){
		var valueMap = this.getData();
		for (i in valueMap) {
			if (i == "rootServices") {
				return valueMap[i];
			}
		}
		return -1
	}

});

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.EditFriendStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/EditFriend.html"),
	secretPlaceholder: "**********",
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	

	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		// Not using handleKeyUp with timeout because the validate is a simple check that there is content.
		// This way the user gets the Finish button much faster, but there are no errors to worry about.
		this.connect(this._editFriendTable,"onkeyup","validateChange");
		this.requiredInputs = [
			this._rootServicesInput, 
			this._titleInput,
			this._secretInput,
			this._retypeSecretInput
		];
		var self = this;
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem) {
				self.connect(elem,"onchange","validateChange");
			}
		});
		this._titleInput.value = this.friend.title;
		if (this.friend.internal) {
			this._internalValue.innerHTML = this.messages.internalLabel;
		}
	}, 
		
	validate: function() {
		//Make sure none of the required input fields is blank
		var self = this;
		var isValid = true;
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem && elem.value ==  "") {
				self.header.showMessage({
					type: STATUS_TYPES.ERROR,
					summary: self.messages.someFieldsEmpty
				});
				isValid = false;
				return; //doesn't return out of the function, only the forEach
			} 
		});
		//Check to see if the secrets match
		if(this._secretInput.value != this._retypeSecretInput.value) {
			dojo.addClass(this.mismatchErr,"inlineError");
			this.mismatchErr.innerHTML = this.messages.secretMismatch;
			isValid = false;
		} else {
			this.mismatchErr.innerHTML = "";
			dojo.removeClass(this.mismatchErr,"inlineError");
		}
		if(isValid) {
			this.header.hideMessage();			
			return true;	
		} else {
			return false;
		}		
	},
	
	validateChange: function () {
		this.triggerButtonStateUpdate();
	},
	
	saveUpdates: function () {
		var isValid = this.validate();
		if(!isValid) return;
		
		var args = {
			id: this.friend.id
		};
		
		if (this.friend.rootServicesURL != this._rootServicesInput.value) {
			args.rootServices = this._rootServicesInput.value;
		}
		if (this.friend.title != this._titleInput.value) {
			args.title = this._titleInput.value;
		}	
		if(this._secretInput.value != this.secretPlaceholder) {
			args.oauthConsumerSecret = this._secretInput.value;
		}
		
		OAuthAdminClient.updateFriend(new SRH(this, "handleFriendUpdateSuccess", "serviceError"), args);
	},
		
	handleFriendUpdateSuccess: function () {
		this.wizard.saveSuccess();		
	},
	
	serviceError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	canFinish: function(){
		var isValid = this.validate();
		return isValid && this.hasUpdates();
	}, 
	
	hasUpdates: function () {
		if (this.friend.rootServicesURL != this._rootServicesInput.value) {
			return true;
		}
		if (this.friend.title != this._titleInput.value) {
			return true;
		}
		if (this._secretInput.value != this.secretPlaceholder) {
			return true;
		}
		return false;
	},
	
	uninitialize: function(){
		// Destroy the message box
		this.header.destroyRecursive();
					
		this.inherited(arguments);
	}

});

/*
 * Wizard Step for adding new External Applications
 */

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.AddFriendStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/AddFriend.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	

	postCreate: function() {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		var constructor = dojo.getObject(PACKAGE_PREFIX + "OAuthRequest");
		var args = {
			setStatusCallback: dojo.hitch(this, this.setFormStatus), 
			templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SimpleWizardForm.html")
		};
		this.oauthForm = new constructor(args);
		this.oauthRequestArea.appendChild(this.oauthForm.domNode);
		
		this.connect(this.oauthForm.propertyTableBody,"onkeyup","updateCreateButton");
		
		this.createButton.disabled = true;
		this.connect(this.createButton, "onclick", "saveUpdates");
		this.isCompleted = false;
	},
	
	canFlipToNextPage: function() {
		return this.isCompleted;
	},
	
	updateCreateButton: function() {	
		var status = this.oauthForm.validate();
		this.createButton.disabled = !status.overrideNext;
	},
	
	saveUpdates: function() {
		var status = this.oauthForm.validate();
		if(!status.overrideNext) return;
		
		this.oauthForm.disableFields(true);
		this.createButton.disabled = true;
		this.header.showMessage({summary: this.messages.requestingAccess, type: STATUS_TYPES.INFO});
		this.testNewFriendProperties();
	},
	
	hasUpdates: function() {
		return !this.oauthForm.isFormEmpty();
	},
	
	testNewFriendProperties: function() {
		this.data = this.oauthForm.getData();		
		this.caller.oauthFormData = this.data;
		var args = {
			title: this.data.title, 
        	rootServices: this.data.rootServices, 
			oauthConsumerKey: "TEST_KEY", 
			oauthConsumerSecret: "TEST_SECRET"
		};
		OAuthAdminClient.testNewFriend(new SRH(this, "successTestingNewFriendProperties", "errorTestingFriend"), args);	
	},
	
	successTestingNewFriendProperties: function(result) {
		if (result.code && result.code === "OK") {
			this.fetchOauthProvisionalRequestUri();
		} else {
			var status = {
				type: result.code,
				summary: result.message
			}
			if (result.stacktrace) {
				if (dojo.isArray(result.stackwasFriendAddedtrace)) {
					var detail = result.stacktrace.join("<br>");
				} else {
					detail = result.stacktrace;
				}
				status.details = [{message: detail, code: result.code}];
			}
			this.header.showMessage(status);
			this.updateUIOnError();
		}
	},
	
	errorTestingFriend: function (errorObj) {
		this.header.showMessage({
			errorObject: errorObj,
			summary: this.messages.testFriendError
		});
		this.updateUIOnError();
	},

	fetchOauthProvisionalRequestUri: function () {
		var rootServicesURL = this.oauthForm.getRootServices();
		OAuthAdminClient.getOauthProvisionalRequestUri(new SRH(this, "requestProvsionalKey", "errorFetchingRootServices"), {url: rootServicesURL});
	},
		
	requestProvsionalKey: function(oauthRequestUrl) {
		if (!oauthRequestUrl) {
			this.header.showMessage({summary: this._getfetchRootServicesErrorMsg(), type: STATUS_TYPES.ERROR});
			this.updateUIOnError();
			return;
		}
		var args = this.oauthForm.getDataObject();
		args.url = oauthRequestUrl;
		OAuthAdminClient.requestRemoteProvisionalKey(new SRH(this, "acquiredProvisionalKey", "errorRequestingKey"), args);
	},
	
	errorFetchingRootServices: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj,
			summary: this._getfetchRootServicesErrorMsg()
		});
		this.updateUIOnError();
	},
	
	_getfetchRootServicesErrorMsg: function () {
		var serverUrl = this.oauthForm.getRootServices();
		var contextMsg = dojo.string.substitute(this.messages.fetchRootServicesError, [serverUrl]);
		this.updateUIOnError();
		return contextMsg;
	},
	
	acquiredProvisionalKey: function(key) {
		if(key && key.key) {
			this.provisionalKey = key.key;
			this.storeProvisionalKey();
		} else {
			this.errorRequestingKey();
		}
	}, 
	
	errorRequestingKey: function (errorObj) {
		this.header.showMessage({
			errorObject: errorObj,
			type: "ERROR",
			summary: dojo.string.substitute(this.messages.requestKeyError, [ this.data.rootServices ])
		});
		this.updateUIOnError();
	},
		
	storeProvisionalKey: function() {
		var args = {
			title: this.data.title, 
        	rootServices: this.data.rootServices,
        	oauthConsumerKey: this.provisionalKey,
        	oauthConsumerSecret: this.data.consumerSecret
		};
		OAuthAdminClient.postNewFriend(new SRH(this, "successStoredProvisionalKey", "errorPostingFriend"), args);	
	},
	
	successStoredProvisionalKey: function () {
		this.isCompleted = true;
		this.header.showMessage({
			type: STATUS_TYPES.OK,
			summary: this.messages.storedProvisionKeyOK
		});	
		this.wizard._steps[1].showApprovalStep(this.provisionalKey, this.data.rootServices);
		this.triggerButtonStateUpdate();
		this.createButton.disabled = true;
		this.provisionalKey = null;	
	},
	
	errorPostingFriend: function (errorObj) {
		this.header.showMessage({
			errorObject: errorObj,
			summary: this.messages.postFriendError
		});
		this.updateUIOnError();
	},
	
	updateUIOnError: function() {
		this.oauthForm.disableFields(false);
		this.createButton.disabled = false;
	},
	
	uninitialize: function(){
		// Destroy the message box
		this.header.destroyRecursive();
		this.oauthForm.destroyRecursive();
		this.inherited(arguments);
	}
});

/*
 * Wizard Step for accepting the provisional key after adding a friend
 */

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.AcceptProvisionalKeyStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/AcceptProvisionalKeyStep.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		this.approveOAuthMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
	},	

	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
	},
	
	showApprovalStep: function(provisionalKey, rootServices) {
		this.triggerButtonStateUpdate();
		this.provisionalKeyContainer.innerHTML = provisionalKey;

		var self = this;
		var provisionalKeyHandler = {
			success: function (approvalUri) {
				if (!approvalUri) {
					// The other server might not have the approval URI in its root services.
					// Do nothing if we can't offer the approval link.
					return;
				}

				var server = rootServices.substring(0, rootServices.lastIndexOf('/'));
				var needsAuth = dojo.string.substitute(self.messages.authorizationLink, [server]);
				var div = dojo.create("div", {className: "spaced-all border", innerHTML: needsAuth}, self.provisionalContentArea);
				var alink = dojo.create("a",{ id: 'approvelink', innerHTML: self.messages.grantAccessLink }, div);

				self.connect(alink,"onclick",function(e) {
					dojo.stopEvent(e);
					self.addApprovalIFrame(provisionalKey, approvalUri);
				});
			},
			failure: function (errorObj) {
				// The other server might not have the approval URI in its root services.
				// Do nothing if we can't offer the approval link.
			}
		};
		OAuthAdminClient.getFriendsOauthApprovalUri(new SRH(provisionalKeyHandler, "success", "failure"), {url: rootServices});	
	},
	
	addApprovalIFrame: function (provisionalKey, approvalUri) {		
		if(provisionalKey && approvalUri && this.iFrameContentArea.innerHTML == "") {
			var iframe = document.createElement("iframe");
			iframe.src = approvalUri + "?key=" + provisionalKey;
			iframe.marginWidth="0"
			iframe.marginHeight="0";
			iframe.style.width = "100%";
			iframe.style.height = "300px";
			iframe.style.border = "none";
			this.iFrameContentArea.appendChild(iframe);
		}
	},

	canFinish: function() {
		return true;
	},
	
	uninitialize: function(){
		// Destroy the message box
		this.header.destroyRecursive();
							
		this.inherited(arguments);
	}
	
});

})();