/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenReportContributor");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojox.xml.parser");
    
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportContributor");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportDateAndType");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenServiceConfigExtension");
    
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    dojo.require("dojo.i18n");
    
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");
    
    var ReportDateAndType = com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportDateAndType;
    var LicenseReportContributor = com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportContributor;
    var LicenseTokenProviderConfigExt = com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenServiceConfigExtension;
    
    var REPO_BUNDLE = "com.ibm.team.repository.service";
    var USAGE_PATH = "licenseReports/LicenseTokenUsage.rptdesign";
    var DENIAL_PATH = "licenseReports/LicenseTokenDenials.rptdesign";
    var USAGE_PER_LICENSE = "licenseReports/LicenseTokenUsageByLicense.rptdesign";
    
    
    var USAGE_CALLBACK = "processUsageReport";
    var DENIAL_CALLBACK = "processDenialReport";
    var USAGE_PER_LICENSE_CALLBACK = "processUsagePerLicenseReport";
    
    
    var FROM_TIME = "FROM_TIME";
    var TO_TIME = "TO_TIME";
    var START_KEY = 'startDate';
    var END_KEY = 'endDate';
    var PROVIDERS_KEY = 'providers';
    var FACET = "FACET";
	var NAME = "NAME";
	var LICENSE_KEY = "licenseName";
    var REPORTS = [{
        bundle: REPO_BUNDLE,
        path: USAGE_PATH,
        callback: USAGE_CALLBACK,
        params: [{
            name: FROM_TIME,
            formValue: START_KEY
        }, {
            name: TO_TIME,
            formValue: END_KEY
        }]
    },{
        bundle: REPO_BUNDLE,
        path: DENIAL_PATH,
        callback: DENIAL_CALLBACK,
        params: [{
            name: NAME,
            formValue: LICENSE_KEY
        },{
            name: FROM_TIME,
            formValue: START_KEY
        }, {
            name: TO_TIME,
            formValue: END_KEY
        }]
    }, {
        bundle: REPO_BUNDLE,
        path: USAGE_PER_LICENSE,
        callback: USAGE_PER_LICENSE_CALLBACK,
        params: [
		{
            name: NAME,
            formValue: LICENSE_KEY
        }, {
            name: FROM_TIME,
            formValue: START_KEY
        }, {
            name: TO_TIME,
            formValue: END_KEY
        }]
    
    
    }];
    
    var REPORT_VIEW_STYLE = "reportView";
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenReportContributor", LicenseReportContributor, {
    
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/LicenseTokenReportContributor.html"),
        constructor: function(){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");
            
        },
        postCreate: function(){
            com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportContributor.superclass.postCreate.apply(this);
            var args = {                
				allowAllLicensesChoice : true				
            }
            this.dateAndLicenseWidget = new ReportDateAndType(args, this.dateAndLicenseArea);
			
			this.connect(this.dateAndLicenseWidget, "handleDateSelected", "onChange");
			this.connect(this.dateAndLicenseWidget, "onLicenseLoad", "handleLicenseLoad");
			this.connect(this.dateAndLicenseWidget, "onError", "handleServiceError");
        },
        
        _createReportViews: function(container){
        	this.usagePerLicenseView = document.createElement("div");
            this.usagePerLicenseView.className = REPORT_VIEW_STYLE;
            container.appendChild(this.usagePerLicenseView);
        
        	this.denialReportView = document.createElement("div");
            this.denialReportView.className = REPORT_VIEW_STYLE;
            container.appendChild(this.denialReportView);       
        
            this.usageReportView = document.createElement("div");
            this.usageReportView.className = REPORT_VIEW_STYLE;
            container.appendChild(this.usageReportView);                             
        },
        
        clearReports: function(){
            dojox.xml.parser.removeChildren(this.usageReportView);
            dojox.xml.parser.removeChildren(this.usagePerLicenseView);
            
        },
        
        _fetchReports: function(){
            this.reportResponses = 0;
            var self = this;
            this.licenseName = this.dateAndLicenseWidget.getLicenseName();
            this.startDate = this.dateAndLicenseWidget.getStartDate();
            this.endDate = this.dateAndLicenseWidget.getEndDate();            
            dojo.forEach(REPORTS, function(report){
                var names = [];
                var values = [];
                if (report.params) {
                    dojo.forEach(report.params, function(param){						
                        if (param.name && param.formValue && self[param.formValue]) {
                            names.push(param.name);
                            values.push(self[param.formValue]);
                        }
                    });
                }
                var args = this._prepareReportArgs(report.bundle, report.path, names, values, "false");
                self._invokeReport(args, report.callback);
            }, this);
            
        },
        
        
        VIEWS: {
            processUsageReport: "usageReportView",
            processDenialReport: "denialReportView",
            processUsagePerLicenseReport: "usagePerLicenseView"            
        },
        
        showReport: function(htmlReport,view){
        	this.handleResponseCount();
            this._displayReportInfo(htmlReport, this[this.VIEWS[view]]);
        },
        
        processUsageReport: function(htmlReport){
        	this.showReport(htmlReport,"processUsageReport");
            
        },
        
        processDenialReport: function(htmlReport){
        	this.showReport(htmlReport,"processDenialReport");
        },
        
        processUsagePerLicenseReport: function(htmlReport){
        	this.showReport(htmlReport,"processUsagePerLicenseReport");
        },
        

        
        handleResponseCount: function(){
            this.reportResponses++;
            if (this.reportResponses == REPORTS.length) {
                this.onFetchComplete();
            }
        },
		
		isValid : function(){
			return this.dateAndLicenseWidget.isValid();
		},
		
		uninitialize: function () {
			if(this.dateAndLicenseWidget) {
				this.dateAndLicenseWidget.destroyRecursive();
			}
			this.inherited(arguments);
	    }
		
		
    });
    
    
    
})();
