/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenServiceProvidersView");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
	dojo.require("jazz.ui.MessageBox");
    dojo.require("dojox.xml.parser");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.client.internal.ExternalLicenseClient");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.PropertiesWizard");
    dojo.require("com.ibm.team.repository.web.ui.internal.common.StyledTable");
    dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
	dojo.require("dojo.i18n");
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");
    
    var MessageBox = jazz.ui.MessageBox;
	var ExternalLicenseClient = com.ibm.team.repository.web.client.internal.ExternalLicenseClient;
    var StyledTable = com.ibm.team.repository.web.ui.internal.common.StyledTable;
    var PropertiesWizard = com.ibm.team.repository.web.ui.internal.admin.server.PropertiesWizard;
    var ToolBar = com.ibm.team.repository.web.ui.internal.common.Toolbar;   
    var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
	var LICENSE_TOKEN_REST_SVC = "com.ibm.team.repository.service.internal.license.token.ILicenseTokenRestService";
	var PROVIDER_STATUS_URL = "com.ibm.team.repository.service.internal.license.token.ILicenseTokenRestService/provider-status";
	
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenServiceProvidersView", [dijit._Widget, dijit._Templated], {
        
		templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/LicenseTokenServiceProvidersView.html"),
		_actionToolbars: [],

        constructor: function(args){
            this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");
        },
        
        postCreate: function() {
            this.refresh();
            this.providerTable = new StyledTable({
                columns: [{
                    label: this.messages.tokenProviderName,
                    width: "20%"
                }, {
                    label: this.messages.tokenProviderDesc
                }, {
                    label: this.messages.capabilities
                }, {
					label: "",
                    width: "2%"
                }, {
                    label: this.messages.status,
                    width: "10%"
                }, {
                    label: this.messages.actions,
					className: "actions"
                }],
                highlight: true
            });
            
            dojo.addClass(this.providerTable.domNode, "actions-table");
            this.providerTableArea.appendChild(this.providerTable.domNode);
            
            dojo.addClass(this.providerTable.tbody, "com-ibm-team-repository-license-tbody");
            
            this.header = new MessageBox();
            this.headerArea.appendChild(this.header.domNode);
        },

        refresh: function(){
            var srh = new SRH(this, "_displayTokenServiceProviders", "handleError");
			ExternalLicenseClient.getRegisteredProviders(srh);   
        },

        _displayTokenServiceProviders: function(providers){
        	if (providers && this.domNode) {
            	dojox.xml.parser.removeChildren(this.providerTable.tbody);
        		if (providers.length == 0) {
            		//Show no providers row
					var tr = dojo.create("tr", {}, this.providerTable.tbody);
					dojo.create("td", { className: "noRowsYet", colSpan: 6, innerHTML: this.messages.noTokenProviders }, tr);          
        		} else {
					dojo.forEach(providers, function(provider) {
						this.providerTable.tbody.appendChild(this._createRow(provider));	
					}, this);
        		}
            }
        },

        _createRow: function(config){
			if(!config) return;
            var id = config.id;
			var row = dojo.create("tr", { className: "hasAction" } );
			dojo.create("td", { innerHTML: config.name }, row);
			dojo.create("td", { innerHTML: config.description }, row);
			var capabilitiesTd = dojo.create("td", {}, row);
			var iconTd = dojo.create("td", {}, row);
			var icon = dojo.create("div", {}, iconTd);
			var statusTd = dojo.create("td", {}, row);
			var status = dojo.create("span", {}, statusTd);
			var actionsTd = dojo.create("td", {}, row);
			
			//Determine provider capabilities
			var capabilities = "";
			if(config.providesTokens) {
				capabilities = this.messages.capabilities_tokens;
			}
			if(config.providesLicenses) {
				if(capabilities != "") capabilities += ", ";
				capabilities += this.messages.capabilities_licenses;
			}
			if(capabilities != "") {
				dojo.create("span", { innerHTML: capabilities }, capabilitiesTd);
			}
                            
            // Actions
            var toolBar = new ToolBar();
			this._actionToolbars.push(toolBar);
            if(id) {
            	this._fetchStatus(id, icon, status);
                if (config.hasProps) {
                    var editAction = {};
                    editAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit-ellipses.gif");
                    editAction.label = messages.edit;
                    editAction.callbackData = {
                        providerId: id,
                        serviceImpl: config.serviceImpl,
                        svcName: config.name
                    };
                    var callback = dojo.hitch(this, function(e){
                        this._editProviderConfig(e.currentTarget.callbackData);
                    });
                    toolBar.addButton(this, callback, editAction);
                }
				actionsTd.appendChild(toolBar.domNode);
            }
            return row;
        },
		
		_fetchStatus: function(providerId , statusIcon , statusMsg){
        	if(providerId && this.domNode) {
        		statusMsg.innerHTML = this.messages.pending;
        		var handler = {
        			success: function(response) {
        				if(response && response.code) {
							statusMsg.innerHTML = "";
							var status;
							switch(response.code) {
								case "OK":
									status = PROVIDER_STATUS.OK;
									break;
								case "WARNING":
									status = PROVIDER_STATUS.WARNING;
									break;
								case "ERROR":
									status = PROVIDER_STATUS.ERROR;
							};
                			if(status) {
								dojo.addClass(statusIcon, status.icon);
							}
                			if(response.message) {
                    			statusMsg.innerHTML = response.message;
                    			if(status) {
									dojo.addClass(statusMsg, status.color);	
								}
                			}
                		}
        			},
        			error: function(errorObj){
        				var status = PROVIDER_STATUS.ERROR;             		             		
                    	dojo.addClass(statusIcon, status.icon);
						dojo.addClass(statusMsg, status.color);
            			statusMsg.innerHTML = dojo.string.substitute(messages.errorFetchingStatus, [errorObj.status]);       
        			}
        		};
				var srh = new SRH(handler, "success", "error");
				ExternalLicenseClient.getStatus(srh, { "providerId": providerId });
			}
        },
		
        _editProviderConfig: function(config) {
			if(config) {
				this._destroyPropertiesWizards();
				this.propWizard = new PropertiesWizard({
	            	implClass: config.serviceImpl,
	                serviceName: config.svcName,
	                propConfigMessages: {
	                    headerTitle: dojo.string.substitute(this.messages.editTokenProviderPropertiesTitle, [ config.svcName ]),
	                    dialogTitle: this.messages.editTokenProviderPropertiesDialogTitle
	                },
	                configSaveCallback: dojo.hitch(this, "refresh")
	            });	
			}
        },

        handleError: function(errObject){
            this.header.showMessage({
				errorObject: errObject
			});
            if (this.parentPointer && this.parentPointer.header) {
                this.parentPointer.header.showMessage({
					errorObject: errObject
				});
            }
        },
		
		
		_destroyPropertiesWizards: function() {
			if (this.propWizard) {
				this.propWizard.destroyRecursive();
				delete this.propWizard;
			}
		},

		_destroyActionToolbars: function() {
			for (var i = 0; i < this._actionToolbars.length; i++) {
				this._actionToolbars[i].destroyRecursive();
			}
			this._actionToolbars = [];
		},
		
		uninitialize: function () {
			if(this.providerTable) {
				this.providerTable.destroyRecursive();
			}
			this.header.destroyRecursive();
			this._destroyPropertiesWizards();
			this._destroyActionToolbars();
			this.inherited(arguments);
	    }
		
    });
	
    PROVIDER_STATUS = {
        OK: {
            icon: "icon-ok",
            color: ""
        },
        WARNING: {
            icon: "icon-warning",
            color: "WARNINGcolor"
        },
        ERROR: {
            icon: "icon-error",
            color: "ERRORcolor"
        }
    };    
})();
