/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ProxyConfiguration");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dojo.i18n");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ProxyConfigurationMessages");

(function() {
var messages;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var AUTH_CONFIG_SERVICE = "com.ibm.team.repository.internal.service.auth.impl.AuthConfigurationProperties";
var WHITELIST_PROPERTY_NAME = "whitelist";
var ENTER_KEY_CODE = 13;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ProxyConfiguration", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ProxyConfiguration.html"),
	_actionToolbars: [],
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ProxyConfigurationMessages");
		this.configUtil = new ConfigUtility();
		this.whitelist = new Array();
		this.pendingWhitelist = new Array();
	},
	
	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		this.connect(this.domainInput, "onkeypress", "_handleKeyPress");
		this.connect(this.domainInput, "onkeyup", "_checkForInputChange");
		this.connect(this.domainInput, "onchange", "_checkForInputChange");
		this.exampleDomain.innerHTML = this.messages.example +  ": https://myserver.ibm.com:9443/"; 		//Example string $NON NLS$
		
		//Append the context sensitive help
		this._pageHelpHover = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.urlWhitelist" });
		this.helpHover.appendChild(this._pageHelpHover.domNode);
		
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		
		//Add *dynamic* description
		this.description.innerHTML = this.isJTS ? this.messages.description_jts : this.messages.description_app;
		
		this.reload();
	},
	
	reload: function () {
		this.header.hideMessage();
		this.domainInput.value = "";
		this.addButton.disabled = false;
		dojox.xml.parser.removeChildren(this.whitelistTableBody);	
		this._getWhitelist();
	},
	
	_handleKeyPress: function(e) {
		if(e.keyCode == ENTER_KEY_CODE) {
			dojo.stopEvent(e);
			this.addButton.disabled = true;
			this._addNewDomain();
		}
	},
	
	_getWhitelist: function() {
		var srh= new ServiceResponseHandler(this, "_handleServiceProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, AUTH_CONFIG_SERVICE);
	},
	
	_handleServiceProps: function(serviceInfo) {		
		if (serviceInfo.properties && serviceInfo.properties.length > 0) {
			var whitelist;
			//Find the whitelist property that we want
			dojo.forEach(serviceInfo.properties, function(prop) {
				if(prop.name == WHITELIST_PROPERTY_NAME) {
					whitelist = prop;
					return;
				}
			});
			//Build up an array of the whitelist contents
			if(whitelist && whitelist.overrideValue) {
				this.whitelist = whitelist.overrideValue.split(",");
				this._sortWhitelist();
				var self = this;
				var genId = 0;
				dojo.forEach(this.whitelist, function(domain) {
					var entry = {};
					entry.id = "domain-" + genId;
					entry.url = domain;
					self._createWhitelistRow(entry, self.whitelistTableBody);
					genId++;
				});
			} else {
				this._showNoDomainsRow();
			}
		} else {
			this.header.showMessage({
				type: STATUS_TYPES.ERROR,
				summary: this.messages.noProperties
			});
		}
	},
	
	_handleServiceFetchError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	_showNoDomainsRow: function() {
		var row = dojo.create("tr",{}, this.whitelistTableBody);
		dojo.create("td", {className: "noRowsYet", colSpan: 2, innerHTML: this.messages.noDomainsYet }, row);
	},
	
	_createWhitelistRow: function(entry, targetTable) {
		var row = dojo.create("tr",{id: "row-" + entry.id, className: "hasAction" }, targetTable);
		var url = dojo.create("td", {id: "url-" + entry.id }, row);
		url.appendChild(document.createTextNode(entry.url));
		var actions = dojo.create("td", {}, row);
		var toolbar = new Toolbar();
		this._actionToolbars.push(toolbar);
		var deleteAction = {};
		deleteAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/delete_hov.gif");
		deleteAction.label = this.messages.deleteButton;
		deleteAction.callbackData = {
			id: entry.id
		};
		var callback = function(e){
			dojo.stopEvent(e);
			this._deleteWhitelistEntry(e.currentTarget.callbackData.id);
		};
		toolbar.addButton(this, callback, deleteAction);
		
		var editAction = {};
		editAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit.gif");
		editAction.label = this.messages.editButton;
		editAction.callbackData = {
			id: entry.id
		};
		var callback = function(e){
			dojo.stopEvent(e);
			this._editWhitelistEntry(e.currentTarget.callbackData.id);
		};
		toolbar.addButton(this, callback, editAction);
		actions.appendChild(toolbar.domNode);
	},
	
	_deleteWhitelistEntry: function(id) {
		var verifyDelete = confirm(this.messages.confirmDelete);
        if (verifyDelete) {		
			//Remove it from the whitelist array
			this.pendingWhitelist = dojo.clone(this.whitelist);
			var url = dojo.byId("url-" + id).innerHTML;
			var index = dojo.indexOf(this.pendingWhitelist,url);
			if(index !== -1) this.pendingWhitelist.splice(index,1);
			
			//Now save the changes
			this._saveChanges();
			
			var elem = dojo.byId("row-" + id);
			this.whitelistTableBody.removeChild(elem);
			if (this.whitelistTableBody.childNodes.length == 0) {
				this._showNoDomainsRow();
			}
		}
	},
	
	_editWhitelistEntry: function(id) {		
		var td = dojo.byId("url-" + id);
		var startUrl = td.innerHTML;
		td.innerHTML = "";
		var input = dojo.create("input", {type: "text", value: startUrl, className: "domainInput" }, td);
		var save = dojo.create("button",{ className: "addButton", name: "saveButton", innerHTML: this.messages.saveButton }, td);
		var cancel = dojo.create("button",{ className: "addButton", name: "cancelButton", innerHTML: this.messages.cancelButton }, td);
		this.connect(save,"onclick", function(e) {
			dojo.stopEvent(e);
			save.disabled = true;
			this.activeSaveButton = save;
			if(this.warningPending) {
				//Use wants to save despite the warning
				this._saveChanges();
			} else {
				var index = dojo.indexOf(this.whitelist,startUrl);
				this.pendingWhitelist = dojo.clone(this.whitelist);
				this.pendingWhitelist[index] = input.value;
				this._validateWhitelist();
			}
		});
		this.connect(cancel,"onclick", function(e) {
			dojo.stopEvent(e);
			td.innerHTML = startUrl;
			this.header.hideMessage();
			this.activeSaveButton = null;
		});
	},
	
	_addNewDomain: function() {
		if(this.domainInput.value === "") return;
		if(this.warningPending) {
			//Use wants to save despite the warning
			this._saveChanges();
		} else {
			this.pendingWhitelist = dojo.clone(this.whitelist);
			this.pendingWhitelist.push(this.domainInput.value);
			this._validateWhitelist();
		}
	},
	
	_validateWhitelist: function() {
		this.header.hideMessage();
		var changeList = [{
			"name": WHITELIST_PROPERTY_NAME,
			"value": this.pendingWhitelist.join(","),
			"serviceImplementationName": AUTH_CONFIG_SERVICE
		}];
		var changeArgs = {
			updates: dojo.toJson(changeList), 
			fast: false
		};
		this.header.showMessage({
			type: "INFO",
			summary: this.messages.pendingValidation
		});
		this.domainInput._validationValue = this.domainInput.value;
		var srh = new ServiceResponseHandler(this, "_handleValidationResults", "_showGenericInvalidError");
		ServerStatusClient.postConfigurationValidation(srh, changeArgs);
	},
	
	_handleValidationResults: function(response) {
		if(response && response[0]) {
			//Either an error or a warning occurred
			this.addButton.disabled = false;
			if(this.activeSaveButton) {
				this.activeSaveButton.disabled = false;	
			}
			this.header.showMessage({
				type: response[0].code || "ERROR",
				summary: response[0].message
			});
			//Provide the user with an "Add/Save Anyway" button for warnings
			if(response[0].code == "WARNING") {
				this._enableWarning();
			}
		} else {
			//Validation found no issues
			this._saveChanges();
		}
	},
	
	_enableWarning: function() {
		this.warningPending = true;
		if(this.activeSaveButton) {
			this.activeSaveButton.innerHTML = this.messages.saveAnywayButton;
		} else {
			this.addButton.innerHTML = this.messages.addAnywayButton;
		}
	},
	
	_checkForInputChange: function() {
		if (this.warningPending && this.domainInput._validationValue != this.domainInput.value) {
			this._disableWarning();
		}
	},
	
	_disableWarning: function() {
		this.warningPending = false;
		if(this.activeSaveButton) {
			this.activeSaveButton.innerHTML = this.messages.saveButton;
		} else {
			this.addButton.innerHTML = this.messages.addButton;
		}
	},
	
	_showGenericInvalidError: function(errObject) {
		this.header.showMessage({
			errorObject: errObject,
			summary: this.messages.validationFailed
		});
	},
	
	_sortWhitelist: function () {	
		return this.whitelist.sort(function(a, b){
			var aTitle = a.toLowerCase();
			var bTitle = b.toLowerCase();
			return aTitle > bTitle ? 1 : -1;
		});
	},
	
	//Saves the contents of the current this.whitelist array back into the config property
	_saveChanges: function() {
		//Copy the pending whitelist (that has been validated) to the whitelist
		this.whitelist = dojo.clone(this.pendingWhitelist);
		var changeList = [{
			"name": WHITELIST_PROPERTY_NAME,
			"value": this.whitelist.join(","),
			"serviceImplementationName": AUTH_CONFIG_SERVICE
		}];
		var changes = {
			"changes": changeList,
			"properties": []
		};
		this.configUtil.save(changes, this, "_savedOK", "_saveFailed");
	},
	
	_savedOK: function(response) {
		this.pendingWhitelist = null;
		this.warningPending = null;
		this.activeSaveButton = null;
		this.addButton.innerHTML = this.messages.addButton;
		this.reload();
		this.header.showMessage({
			type: STATUS_TYPES.OK,
			summary: this.messages.savedOK,
			closable: true
		});
	},
	
	_saveFailed: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj,
			summary: this.messages.saveFailed
		});
	},
	
	_destroyActionToolbars: function(){
		for (var i = 0; i < this._actionToolbars.length; i++) {
			this._actionToolbars[i].destroyRecursive();
		}
		this._actionToolbars = [];
	},
	
	uninitialize: function(){
		this._destroyActionToolbars();
		
		// Destroy the message box
		this.header.destroyRecursive();
		
		this._pageHelpHover.destroyRecursive();		
			
		this.inherited(arguments);
	}
	
});

})();