/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerPage");

dojo.require("dijit._Widget");
dojo.require("dojox.xml.parser");
dojo.require("net.jazz.ajax.queryService");
dojo.require("net.jazz.ajax.storageService");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.SideNavigationLayout");

dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureAdvanced");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureTheming");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureDatabaseConnection");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureFeedSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureMailSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseUsageView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.OAuthView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.Friends");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ReloadLogSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerStatus");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.Statistics");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ProvisionStatus");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerReset");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.RegisteredApplications");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ProxyConfiguration");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerDiagnostics");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ClusterNodes");
dojo.require("com.ibm.team.repository.web.ui.internal.events");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerPageMessages");


var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var LIC_P = com.ibm.team.repository.web.client.internal.licensing;

var INTERNAL_TOOLS_NAV_KEY = 'internalTools';
var LICENSING_NAV_KEY = 'licensing';
var ACTION_SETS = [
	{ labelKey: 'status', actions: [
		{labelKey: 'statusSummary', shortId: "serverStatus", widget: "ServerStatus"},
		{labelKey: 'clusterNodes', shortId: "clusterNodes", widget: "ClusterNodes"},
		{labelKey: 'serverDiagnostics', shortId: "serverDiagnostics", widget: "ServerDiagnostics"},
		{labelKey: 'activeServices', shortId: "activeServices", widget: "ActiveServicesView"}
	]},
	{ labelKey: 'configuration', actions: [
		{labelKey: 'registeredApplications', shortId: "registeredApplications", widget: "RegisteredApplications", jtsOnly: true},
		{labelKey: 'emailSettings', shortId: "configureMailSettings", widget: "ConfigureMailSettings", jtsOnly: true},
		{labelKey: 'databaseConnection', shortId: "configureDatabaseConnection", widget: "ConfigureDatabaseConnection"},
		{labelKey: 'feedSettings', shortId: "configureFeedSettings", widget: "ConfigureFeedSettings"},
		{labelKey: 'advancedProperties', shortId: "configureAdvanced", widget: "ConfigureAdvanced"}
	]},	
	{ labelKey: LICENSING_NAV_KEY, jtsOnly: true, actions: [
		{labelKey: 'licenseKeyManagement', shortId: "manageLicenses", widget: "LicenseEditor"},
		{labelKey: 'reports', shortId: "floatingLicenseReports", widget: "FloatingLicenseReportView"},
		{labelKey: 'issuedLeases', shortId: "floatingLicenseUsage", widget: "FloatingLicenseUsageView"}
	]},
	{ labelKey: 'crossServerCommunication', actions: [
		{labelKey: 'oauthConfiguration', shortId: "configureOAuth", widget: "OAuthView"},
		{labelKey: 'friends', shortId: "friends", widget: "Friends"},	
		{labelKey: 'proxyConfiguration', shortId: "proxyConfiguration", widget: "ProxyConfiguration"}
	]},
	{ labelKey: INTERNAL_TOOLS_NAV_KEY, actions: [
		{labelKey: 'counters', shortId: "viewCounters", widget: "ViewCounters", cached: true},
		{labelKey: 'reloadLogSettings', shortId: "reloadLoggingSettings", widget: "ReloadLogSettings"},
		{labelKey: 'componentStatus', shortId: "componentStatus", widget: "ComponentsView"},
		{labelKey: 'statistics', shortId: "statistics", widget: "Statistics"},
		{labelKey: 'provisionStatus', shortId: "provisionStatus", widget: "ProvisionStatus"},
		{labelKey: 'serverReset', shortId: "serverReset", widget: "ServerReset"}
	]}
];

// suppress the theming config page if no storage or query services are available
if (net.jazz.ajax.storageService() && net.jazz.ajax.queryService()) {
	for (var i=0; i < ACTION_SETS.length; i++) {
		if (ACTION_SETS[i].labelKey === "configuration") {
			for (var j=0; j < ACTION_SETS[i].actions.length; j++) {
				if (ACTION_SETS[i].actions[j].labelKey === "feedSettings") {
					ACTION_SETS[i].actions.splice(j+1, 0, {labelKey: 'theming', shortId: 'configureTheming', widget: "ConfigureTheming", cached: true});
				}
			}
		}
	}
}

//
// aliases, constants, and module-private variables
//

var NoAuth         = com.ibm.team.repository.web.ui.internal.admin.NoAuth;
var PlatformUI     = net.jazz.ajax.ui.PlatformUI;
var ViewCounters   = com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters;

var ActionRegistry = null;
var PREFIX         = "com.ibm.team.repository.admin.";
var SERVER_PACKAGE = "com.ibm.team.repository.web.ui.internal.admin.server.";
var ACTION_MAP     = {};

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerPage", dijit._Widget, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerPageMessages");
	},
	
	//
	// Methods intended to be called by child pages
	//
	
	reloadCurrent: function() {
		this._previousActionEntry = this._currentActionEntry || null;
		this._showCurrentWidget(/*isReload*/true);
	},
	
	//
	// Dojo lifecycle and internal methods
	//
	
	postCreate: function() {
		//Call superclass's postCreate method
		this.inherited(arguments);
		
		//Initialize widget cache
		this._widgetCache = {};
		
		//Set the root style for the Server page and all of its sub-pages
		this.domNode.className = "com-ibm-team-repository-admin";
		
		//Instantiate the side-nav layout UI and append it to the page
		this._appView = new jazz.ui.SideNavigationLayout();
		this.domNode.appendChild(this._appView.domNode);
		
		//Register default action and create side-nav hyperlinks
		ActionRegistry = getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction");
		
		//Register the remaining actions upfront before we can fire any XHR requests (prevents action=null errors)
		var self = this;
		dojo.forEach(ACTION_SETS, function(actionSet) {
			dojo.forEach(actionSet.actions, function(action){
				self._registerAction(action);
			});
		});

		//Check to see if we are running on the JTS. If we are not, we should hide some of the navigation items.
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		this._processActionSets();
	},
	
	_processActionSets: function() {
		var self = this;
		this.allLinks = [];
		dojo.forEach(ACTION_SETS, function(actionSet) {
			if((actionSet.labelKey !== INTERNAL_TOOLS_NAV_KEY) || (actionSet.labelKey === INTERNAL_TOOLS_NAV_KEY && location.search.indexOf("internal") !== -1)) {
				if(self.isJTS || !actionSet.jtsOnly) {
					self._createNavSection(actionSet);
				}
			}
		});
		
		this._fetchLicenseInfo();
		this._getClusterStatus();
	},
	
	_createNavSection: function(actionSet) {
		var self = this;
		var navSection = this._appView.addNavigationSection(document.createElement("div"), this.messages[actionSet.labelKey], true);
		dojo.forEach(actionSet.actions, function(action){
			if(!self.isJTS && action.jtsOnly) {
				return; //continue
			}
			var id = PREFIX + action.shortId;
			var link = navSection.addSectionRow(self.messages[action.labelKey], "#action=" + id);
			link.__jazzActionId = action.shortId;
			self.allLinks.push(link);
			navSection.addArtifactCommand(link,self.messages.refresh,dojo.hitch(self,self._onRefreshClick),"admin-refresh-command");
		});
	},
	
	_registerAction: function(action){
		var id = PREFIX + action.shortId;
		ACTION_MAP[action.shortId] = action;
		ActionRegistry.registerAction(id, this, "_handleAction");
	},
	
	_defaultAction: function() {
		//By default, show the first section from the left-nav
		this._handleAction({action: PREFIX + ACTION_SETS[0].actions[0].shortId});
	},
	
	_handleAction: function(args) {
		if (!isAdmin()) {
			this._showUnauthorized();
			return;
		}

		if(!args.action) {
			console.debug("ServerPage.js#_handleAction: missing required param 'args.action'");
			return;
		}
		
		// cache the previous entry to see if the widget should be destroyed or
		// not
		this._previousActionEntry = this._currentActionEntry || null;
		
		// parse out the short id from the fully-qualified id
		var shortId = args.action.substring(args.action.lastIndexOf(".") + 1);
		this._currentActionEntry = ACTION_MAP[shortId];
		this._currentArgs = args;
		this._currentArgs.parent = this;

		// set the title for the action
		getWorkbench().setTitle(this.messages[this._currentActionEntry.labelKey]);
		
		// highlight the selected link and unhighlight the others
		dojo.forEach(this.allLinks, function(link){
			if (link.__jazzActionId === shortId)
				dojo.addClass(link, "selected");
			else
				dojo.removeClass(link, "selected");
		});
		
		// call the actual method implementation
		this._showCurrentWidget();
	},
	
	_onRefreshClick: function(e) {
		dojo.stopEvent(e);
		this.reloadCurrent();
	},
	
	_showCurrentWidget: function(isReload) {
		// save a reference to the prior content widget
		this._previousContentWidget = this._currentContentWidget || null;
		
		if(this._currentActionEntry.cached) {
			this._showCachedWidget(isReload);
		} else {
			this._showUncachedWidget();
		}
		
		// help the garbage collector do its job
		if(this._previousContentWidget && !this._previousActionEntry.cached) {
			this._previousContentWidget.destroyRecursive();
		}
	},
	
	_showCachedWidget: function(isReload) {
		var className = this._currentActionEntry.widget;
		if(this._widgetCache[className]) {
			// get widget from the cache if it exists
			this._currentContentWidget = this._widgetCache[className];
		} else {
			// otherwise instantiate it and add it to the cache
			var constructor = dojo.getObject(SERVER_PACKAGE + className);
			this._currentContentWidget = new constructor(this._currentArgs);
			dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
			this._widgetCache[className] = this._currentContentWidget;
		}
		if(isReload) {
			this._currentContentWidget.reload(this._currentArgs);
		} else {
			this._currentContentWidget.show(this._currentArgs);
			this._appView.setMainContent(this._currentContentWidget.domNode);
		}
	},
	
	_showUncachedWidget: function() {
		var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
		this._currentContentWidget = new constructor(this._currentArgs);
		dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
		this._appView.setMainContent(this._currentContentWidget.domNode);
	},
	
	_showUnauthorized: function(){
		dojox.xml.parser.removeChildren(this.domNode);
		if (!this._notAuth) {
			this._notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
		}	
		this.domNode.appendChild(this._notAuth.domNode);
	}, 
	
	_getClusterStatus: function() {
		var srh = new ServiceResponseHandler(this, "_processClusterDetails", "_errorFetchingNodeList");
		ServerStatusClient.getIsInCluster(srh);
	},
	
	_processClusterDetails: function(response) {
		dojo.setObject("com.ibm.team.repository.isInCluster", response === "false" ? false : true);
		if(response === "false") {
			//Server is not running in a cluster. Hide page.
			removeAction("clusterNodes");
		}
	},
	
	_errorFetchingNodeList: function(error) {
		console.info("Unable to determine if the server is running in a cluster. Hiding Cluster Nodes page.");
		removeAction("clusterNodes");
	},

	_fetchLicenseInfo: function () {
		var srh = new ServiceResponseHandler(this, "processLicenseInfoResponse", "handleLicenseErrorReponse");
		LIC_P.getLicenseInfo(srh);
	},
	
	processLicenseInfoResponse: function(licenseInfo) {
		var isFloating = dojo.getObject("floatingSetupStatus.server", false, licenseInfo);
		this._configureLicensingNavigation(isFloating);
		dojo.subscribe(com.ibm.team.repository.web.ui.internal.events.floatingLicenseChange, this, "receiveFloatingLicenseNotification");
	},
	
	
	handleLicenseErrorReponse: function(error) {
		console.info("Unable to determine if the server is a Floating License Server.");
		dojo.subscribe(com.ibm.team.repository.web.ui.internal.events.floatingLicenseChange, this, "receiveFloatingLicenseNotification");
	},
	
	receiveFloatingLicenseNotification: function (licenseInfo) {
		if (licenseInfo.actingAsFloating) {
			this._configureLicensingNavigation(licenseInfo.actingAsFloating);
		}
	},
	
	_configureLicensingNavigation: function (isFloatingServer) {
		if (isFloatingServer && !this.licensingNavInitialized) {
			this._checkReportsServices();
			this.licensingNavInitialized = true;
		}
	},
	
	_checkReportsServices: function() {
		var services= {
			serviceNames: ["com.ibm.team.reports.common.internal.service.IReportRestService"]
		};
		var srh= new ServiceResponseHandler(this, "_handleServiceCheckSuccess", "_handleServiceCheckFailure");
		ServerStatusClient.postStatusForServiceList(srh, services);
	},
	
	_handleServiceCheckSuccess: function(response) {
		response = response[0];
		if(response.code && response.code == "ERROR") {
			removeAction("floatingLicenseReports");
		}
	},
	
	_handleServiceCheckFailure: function(errorObj) {
		console.info("Unable to determine if the server has support for Reports.");
		this._createNavSection(this.licensingNav);
	}
});

function removeAction(actionId){
	var foundNode = null;
	dojo.query('a.section-entry').forEach(function(node, index, arr){
		if (node.__jazzActionId == actionId) {
			foundNode = node;
		}		
	});
	if (foundNode) {
  		foundNode.parentNode.removeChild(foundNode);
 	}
}

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

})();
