/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.SearchProvider");

dojo.require("jazz.ui._SearchProvider")
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("jazz.util.html");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "SearchProviderMessages");

(function() {

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "SearchProviderMessages");
var SCOPE_ALL = {id: "all", label: messages.searchScopeAll, icon: dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/all-users_search.png")};

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.SearchProvider", [jazz.ui._SearchProvider], {
	scopes: [SCOPE_ALL],
	tooltip: messages.searchTooltip,
	noResultsLabel: messages.searchNoResultsLabel,
	onUpdate: function(args) {	
		if (this._cachedResult && this._cachedResult.query === args.query && this._cachedResult.scope === args.scope) {
			args.callback(this._createResults(this._cachedResult.data, args.maxResults, args.page));
			return;
		}
		var serviceObj = {
			_self: this,
			_query: args.query,
			_callback: args.callback,
			_errorCallback: args.errback,
			success: function(response) {		
				var data = (response && response.elements) ? response.elements : [];
				this._self._cachedResult = {
					query: this._query,
					data: data,
					scope: args.scope
				};
				this._callback(this._self._createResults(data, args.maxResults, args.page));
			},
			failure: function(error) {
				this._errorCallback(error.message);
			}
		};
		var queryArgs = {
			searchTerm: args.query,
			sortBy: "name", 
			sortOrder: "asc"
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure"); 
		AdminClient.getContributors(queryArgs, srh);
	},
	
	_createResults: function(allResults, maxResults, page) {
		var results = [];
		var offset = (page-1) * maxResults;
		if(allResults) {
			for (var i = offset; i < allResults.length && results.length < maxResults; i++) {
				var newResult = this._createResult(allResults[i]);
				if(newResult) {
					results.push(newResult);
				}
			}
		}
		return {
			paging: true,
			totalResults: allResults.length,
			results: results
		};
	},

	_createResult: function(user) {
		if(!user) return null;
		return { 
			label: jazz.util.html.escape(user.name), 
			presentationUri: "#action=com.ibm.team.repository.viewUserProfile&itemId=" + user.itemId,
			uri: "users/" + user.userId,
			icon: dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/all-users_search.png").toString()
		};
	}
	
});

})();
