/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.dialogs.SelectionDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("dojo.html"); //$NON-NLS-1$ 
dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
dojo.require("dijit.tree.ForestStoreModel"); //$NON-NLS-1$
dojo.require("dijit.Tree"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTree"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTreePane"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.PracticeNavigatorData"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.WorkingCopy"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.FieldViewer");  //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.dialogs", "Messages");//$NON-NLS-1$//$NON-NLS-2$

(function() {

	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var PracticeTreePane = com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTreePane;
	var PracticeTree = com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTree;
	var FieldViewer = com.ibm.team.process.authoring.web.common.ui.internal.viewlets.FieldViewer;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var PracticeNavigatorData = com.ibm.team.process.authoring.web.common.common.internal.PracticeNavigatorData;
	var WorkingCopy = com.ibm.team.process.authoring.web.common.common.internal.WorkingCopy;
	var DEFAULT_WIDTH_IN_EM = 60;
	var _HEIGHT_IN_EM = 35;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.dialogs.SelectionDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
	// summary:
	//		SelectionDialog provides a dialog for selecting practice, practice element, section or field 
	//      in the process description. There is a practice/practice element tree on the left and a section/field
	//      tree on the right. We need to pass the process description, then the SelectionDialog will build practice tree
	//      and section tree for selection.
	//
	// parameters:
	//      scope: (String) The URI of the scope (project area, team area...) to select the process description.
	//		primaryTitle: (String) the primary title of the dialog.
	//		(optional) selectedUrl: (String) the initial selection url of the selected practice or practice element.
	//		(optional) selectedFragmentId: (String) the initial selection url of the selected section or field.
	//		(optional) width: (String) the width of the dialog. Default to 60em.
	//		(optional) showSectionTree: (boolean) indicates whether show the section tree or not. Default to true.
	//
	// methods:
	//		onOK: the function is called after clicking OK, one can override this function to add his own behavior.
	//      getTopText: override this function to provide top text in the dialog.
	//      onProcessDescriptionLoaded:  update dialog after process description is loaded.
	//
	// usage:
	//		 this.selectionDialog = new SelectionDialog({
	//          scope: ${projectAreaUrl},
	//          primaryTitle: "MyTitle",
    //        	selectedUrl: "https://127.0.0.1:9444/jazz/process-authoring/practices/_vxZnZ8_xEeG4MJPllocT3Q", //url of practice or practice element
    //        	selectedFragmentId: ${selectedFragmentId}, //section or field Id
	//	        width: "60em",
	//	        showSectionTree: true|false,
    //        	onOk: dojo.hitch(this, this.dialogOk, this._callback)
    //        });
	
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "ui/internal/dialogs/templates/SelectionDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
		widgetsInTemplate: true,
		currentLanguage: "en", //$NON-NLS-1$
		scope: null,
		sharedSupport: false,
		showSectionTree: true,
		primaryTitle: "", //$NON-NLS-1$
		width: DEFAULT_WIDTH_IN_EM + "em",  //$NON-NLS-1$
		constructor: function(args) {
			if (!args) {
				args = {};
			}
			this.workingCopy = new WorkingCopy();
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.dialogs", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
			this.selectedUrl = args.selectedUrl;
			this.selectedFragmentId = args.selectedFragmentId;
		},
		
		_updateDialog: function(){
			// update the dialog after process description is loaded. e.g. add the customized top text 
			this.onProcessDescriptionLoaded();
			
			//load practice tree
			this.treeContainer = new PracticeTreePane({
				isPlainTree: true,
				controller: this
			});
			
			this.currentLanguage = UIHelper.getLanguage(this.processDescriptionXML);
			
			this.practiceInfoList = UIHelper.getAllPracticesFromProcessDescription(this.processDescriptionXML);
			
			// build plain practice tree and calculate bulk fetching practice URLs 
			var practiceURLs = new Array();
			for (var i = 0; i < this.practiceInfoList.length; i++) {
				var practiceInfo = this.practiceInfoList[i];
				if (practiceInfo.shared === "true") { //$NON-NLS-1$
				    if (this.sharedSupport) {
						this.buildPlainPracticeTree(practiceInfo, this.treeContainer, true);
						practiceURLs.push(practiceInfo.url);
					}
				} else {
					this.buildPlainPracticeTree(practiceInfo, this.treeContainer, false);
					practiceURLs.push(practiceInfo.url);
				}
			}
			
			dojo.empty(this.practiceTreeArea);
			this.practiceTreeArea.appendChild(this.treeContainer.domNode);
			
			if (practiceURLs.length > 0) {
				// show 'loading' message
				this._showMessage(this.messages.loading, "NormalStatus"); //$NON-NLS-1$ 
				// fetch the tree data and build the real practice trees
				this.bulkFetchPracticeTree(practiceURLs);
			} else {
				this.practiceTreeArea.appendChild(document.createTextNode(this.messages.SelectionDialog_2));
			}
		},
		
		postCreate: function() {
			if (this.scope) {
				this.dialog = new jazz.ui.Dialog({
					contentNode: this.contentNode,
					primaryTitle: this.primaryTitle,
					width: this.width,
					destroyContents: true
				});
				
				// add the top text in the dialog
				var topText = this.getTopText();
				if(topText){
					this.topText.appendChild(document.createTextNode(topText));
				}
			
				UIHelper.resizeI18NButtons(this.dialogFooter, [this.clearSelectionButton, this.okButton, this.cancelButton]);
				this.connect(this.cancelButton, "onclick", "onCancel"); //$NON-NLS-1$ //$NON-NLS-2$ 
				this.connect(this.okButton, "onclick", "onOk"); //$NON-NLS-1$ //$NON-NLS-2$ 
				this.connect(this.clearSelectionButton, "onclick", "onClearSelection"); //$NON-NLS-1$ //$NON-NLS-2$ 
				if (!this.showSectionTree) {
					dojo.style(this.sectionTreeArea, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
					dojo.style(this.sectionTreeHeader, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
				}
				
				var height = this.showSectionTree ? _HEIGHT_IN_EM + "em" : _HEIGHT_IN_EM * 2 / 3 + "em"; //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(this.practiceTreeArea, "height", height); //$NON-NLS-1$ 
				dojo.empty(this.practiceTreeArea);
				
				this.practiceTreeArea.appendChild(document.createTextNode(this.messages.loading));
				
				this._fetchProcessDescription(this.scope);
			} else {
				alert(this.messages.SelectionDialog_3);
			}
		},
		
		/**
		 * Check license and then fetch process description.
		 * 
		 * @param {String} processAreaUrl the url of project area or team area that maybe contain a process description.
		 */
		_fetchProcessDescription: function(processAreaUrl) {
			// check license
			var readSharedId = ProcessAuthoringConstants.PROCESS_AUTHORING_OPERATION_ID_PREFIX + ProcessAuthoringConstants.PROCESS_AUTHORING_READ_SHARED_SUPPORT;
			ProcessAuthoringClient.getPracticeLibraryLicenseInfo({
				operationIds: [readSharedId],
				load: dojo.hitch(this, function(data, ioArgs) {
					this.sharedSupport = data[readSharedId];
					this._getPrivateProcessDescription(processAreaUrl);
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					this._handleError(error);
				})
			});
		},
		
		/**
		 * Get private process description from a project area or team area.
		 * 
		 * @param {String} processAreaUrl the url of project area or team area that maybe contain a private process description.
		 */
		_getPrivateProcessDescription: function(processAreaUrl) {
			var self = this;
			ProcessAuthoringClient.getProcessDescriptionsForOwner({
				ownerUrl: processAreaUrl,
				load: function(data, ioArgs) {
					var descriptionsNode = UIHelper.getNode(data.childNodes, ProcessAuthoringConstants.XMLELEMENT_RDF, ProcessAuthoringConstants.NAMESPACE_RDF);
					if (descriptionsNode) {
						var allProcessDescriptionNodes = UIHelper.getNodes(descriptionsNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_PROCESSDESCRIPTION_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
						if (allProcessDescriptionNodes && allProcessDescriptionNodes.length > 0) {
							self.processDescriptionXML = allProcessDescriptionNodes[0];
							self._updateDialog();
						} else {
							// no private process description, check associated process description
							self._getAssociatedProcessDescription(processAreaUrl);
						}
					}
				},
				error: function(error) {
					self._handleError(error);
				}
			});
		},
	
		/**
		 * Get associated process description from a project area or team area.
		 * 
		 * @param {String} processAreaUrl the url of project area or team area that may be associated with a shared process description.
		 */
		_getAssociatedProcessDescription: function(processAreaUrl) {
			var self = this;
			ProcessAuthoringClient.checkAssociateSharedProcessDescriptionForProcessArea({
				targetUrl: processAreaUrl,
				load: function(data, ioArgs) {
					var associationURL = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
					ProcessAuthoringClient.getAssociation({
						url: associationURL,
						load: function(data, ioArgs) {
							var descriptionURL = UIHelper.getProcessDescriptionUrlFromAssociation(data);
							if (descriptionURL) {
								ProcessAuthoringClient.getProcessDescription({
									resourceURL: descriptionURL,
									load: function(data, ioArgs) {
										self.processDescriptionXML = data;
										self._updateDialog();
									},
									error: function(error) {
										self._handleError(error);
									}
								});
							}
						},
						
						error: function(error) {
							self._handleError(error);
						}
					});
				},
				error: function(error) { //no associate shared process description
					self._handleError(error);
				}
			});
			
		},
		
		_handleError: function(error) {
			if (error.status && error.status == ProcessAuthoringConstants.SC_NOT_FOUND) {
				if (UIHelper.getDescriptionOwnerType(this.scope) === ProcessAuthoringConstants.PROJECT_AREA) {
					this._displayMessageInPracticeTreeArea(this.messages.noDescProject, "");  //$NON-NLS-1$
				} else if (UIHelper.getDescriptionOwnerType(this.scope) === ProcessAuthoringConstants.TEAM_AREA) {
					this._displayMessageInPracticeTreeArea(this.messages.noDescTeam, "");  //$NON-NLS-1$
				}
			} else {
				var message = error.responseText ? error.responseText : error.message;
				if (error.status === ProcessAuthoringConstants.SC_FORBIDDEN) {
					this._displayMessageInPracticeTreeArea(this.messages.permissionDenied, "ErrorStatus2");  //$NON-NLS-1$
				} else {
					this._displayMessageInPracticeTreeArea(message, "ErrorStatus2");  //$NON-NLS-1$
				}
			}
		},
		
		_displayMessageInPracticeTreeArea: function(message, cssClass) {
			dojo.empty(this.practiceTreeArea);
			var div = dojo.create("div"); //$NON-NLS-1$
			dojo.addClass(div, cssClass);
			div.appendChild(document.createTextNode(message));
			this.practiceTreeArea.appendChild(div);
		},
		
		/**
		 * Override this function to provide top text in the dialog.
		 * example: 
		 *   getTopText: function() {
		 *      return "My Top Text.";
		 *   },
		 * 
		 */
		getTopText: function() {
			
		},
		
		/**
		 * Override this function to update dialog after process description is loaded. 
		 * E.g. Provide customized top text which need the information from process description.
		 * 
		 */
		onProcessDescriptionLoaded: function() {
			
		},
		
		bulkFetchPracticeTree: function(practiceURLs) {
			// bulk fetching practices
			if (practiceURLs.length > 0) {
				var successCallback = dojo.hitch(this, function(responses, ioArgs) {
					for (var i = 0; i < this.practiceInfoList.length; i++) {
						var practiceInfo = this.practiceInfoList[i];
						var idx = dojo.indexOf(practiceURLs, practiceInfo.url);
						this.buildPracticeTree(responses[idx], this.treeContainer, practiceInfo.shared === "true");  //$NON-NLS-1$
					}
					this._selectInitialPracticeTreeItem();
					this._hideMessage();
				});
				var errorCallback = dojo.hitch(this, function(error, ioArgs) {
					this.displayErrorMessage(error);
				});
				var practiceURLsWithLang = ProcessAuthoringClient.addLanguageParameterInURLs(practiceURLs, this.currentLanguage);
				ProcessAuthoringClient.fetchResources(practiceURLsWithLang, null, successCallback, errorCallback);
			}
		},
		
		/**
		 *  Select the initial choice in the practice tree.
		 */
		_selectInitialPracticeTreeItem: function() {
			if(!this.selectedUrl){
				return;
			}
			
			var url = this.selectedUrl;
			var resourceId = url.substring(url.lastIndexOf("/") + 1); //$NON-NLS-1$
			if (ProcessAuthoringClient.isPracticeURL(url)) { //practices
				this.treeContainer.getItem(url, null, dojo.hitch(this, function(item, tree) {
					this.populatePracticeDescription(item);
					this.treeContainer.selectTree(tree);
					var path = this.workingCopy.getNodePath(url);
					setTimeout(dojo.hitch(this, function() {
						tree.set("selectedItem", path[0]); //$NON-NLS-1$
					}), 500);
				}));
			} else if (UIHelper.isPracticeElementURL(url)) { //practice elements
				dojo.some(this.treeContainer.trees, function(tree) {
					if (resourceId) {
						tree.model.store.fetchItemByIdentity({
							identity: resourceId,
							onItem: dojo.hitch(this, function(item) {
								if (item) {
									this.populatePracticeElement(item);
									this.treeContainer.selectTree(tree);
									var path = this.workingCopy.getNodePath(resourceId);
									setTimeout(dojo.hitch(this, function() {
										tree.set("path", path); //$NON-NLS-1$
									}), 500);
									return true;
								}
							})
						});
					}
					return false;
				}, this);
			}
		},
		
		/**
		 * Get index of the tree in practice tree pane.
		 * 
		 * @param {Object} url the practice url in the tree node.
		 */
		getTreeIndex: function(url) {
			for (var i = 0; i < this.practiceInfoList.length; i++) {
				var practiceUrl = this.practiceInfoList[i].url;
				if (practiceUrl === url) {
					return i;
				}
			}
		},
		
		/**
		 * Build the practice tree.
		 * 
		 * @param {Object} data the data that is used to create the tree.
		 * @param {PracticeTreePane} treeContainer the PracticeTreePane that contains this tree.
		 * @param {bool} isShared indicate if the practice is shared or not.
		 */
		buildPracticeTree: function(data, treeContainer, isShared) {
			var doc = UIHelper.parseXmlTextToDOMDocument(data.body);
			var practiceETag = data.etag;
			
			var navigatorData = new PracticeNavigatorData({
				practiceXMLData: doc,
				ETag: practiceETag
			});
			var practiceJsonData = navigatorData.getJsonData();
			
			if (isShared) {
				practiceJsonData.isShared = true;
			}
			this.workingCopy.addPractice(dojo.clone(practiceJsonData));
			
			var tree = this.createPracticeTree(practiceJsonData, treeContainer, false, isShared);
			tree.indexInParent = this.getTreeIndex(tree.practiceResource);
			if (treeContainer.exist(tree.practiceResource)) {
				// remove the old tree in treeContainer
				treeContainer.removePracticeTree(tree.practiceResource);
			}
			treeContainer.addPracticeTree(tree);			
		},
		
		/**
		 * Build the plain practice tree which only uses the plain data.
		 * 
		 * @param {Object} practiceInfo the plain data that is used to create the tree.
		 * @param {PracticeTreePane} treeContainer the PracticeTreePane that contains this tree.
		 * @param {bool} isShared indicate if the practice is shared or not.
		 */
		buildPlainPracticeTree: function (practiceInfo, treeContainer, isShared){
			var id = UIHelper.getResourceId(practiceInfo.url);
			var languagesObject = {};
			languagesObject[this.currentLanguage] = {};
			languagesObject[this.currentLanguage].ETag = "";  //$NON-NLS-1$
			languagesObject[this.currentLanguage].textContent = {};
			languagesObject[this.currentLanguage].basicInformation = {};
			languagesObject[this.currentLanguage].basicInformation.label = practiceInfo.title;
			languagesObject[this.currentLanguage].basicInformation.description = ""; //$NON-NLS-1$
			var model = {};
			model.ETag = ""; //$NON-NLS-1$
			model.structure = UIHelper.getEmptyPresentation();
			var practiceJsonData = {
				identifier: 'id', //$NON-NLS-1$
				label: 'label', //$NON-NLS-1$
				resource: practiceInfo.url,
				id: id,
				title: practiceInfo.title,
				practiceId: id,
				model: model,
				languages: languagesObject,
				attachmentsUrl: null,
				defaultLanguage: this.currentLanguage, 
				items: []
			};
			
			if (isShared) {
				practiceJsonData.isShared = true;
			}
			var tree = this.createPracticeTree(practiceJsonData, treeContainer, false, isShared);
			tree.indexInParent = this.getTreeIndex(tree.practiceResource);
			treeContainer.addPracticeTree(tree);
		},
		
		checkResponse: function(response) {
			if (response.status == 200) {
				return true;
			} else {
				var error = new Error();
				error.status = response.status;
				error.message = response.body;
				this.displayErrorMessage(error);
				return false;
			}
		},
		
		createPracticeTree: function(practiceJsonData, treeContainer, hideRoot, isShared) {
			var dataStore = new dojo.data.ItemFileWriteStore({
				data: practiceJsonData
			});
			var model = new dijit.tree.ForestStoreModel({
				rootLabel: practiceJsonData.title,
				store: dataStore,
				childrenAttr: ["children"] //$NON-NLS-1$
			});
			var practiceTree = new com.ibm.team.process.authoring.web.common.ui.internal.dialogs._PracticeTree({
				isPlainTree: treeContainer.isPlainTree,
				showRoot: !hideRoot,
				treeContainer: treeContainer,
				isSharedTree: isShared, //shared practice tree pane is view only
				model: model,
				practiceResource: practiceJsonData.resource,
				onClick: dojo.hitch(this, this._onPracticeTreeClick),
			}, document.createElement("div")); //$NON-NLS-1$
			return practiceTree;
		},
		
		_onPracticeTreeClick: function(/* dojo.data */item, /*TreeNode*/ node) {
			if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
				this.okButton.disabled = true;
				this.clearSelectionButton.disabled = true;
			} else {
				this.okButton.disabled = false;
				this.clearSelectionButton.disabled = false;
				this._showMessage(this.messages.loading, "NormalStatus"); //$NON-NLS-1$ 
				if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
					this.populatePracticeElement(item);
				} else if (item.root === true) {
					this.populatePracticeDescription(item);
				} else if (item.type == ProcessAuthoringConstants.XMLELEMENT_PRACTICE) {
					this.populatePracticeDescription(item);
				}
			}
			this.treeContainer.selectTree(node.tree);
		},
		
		/**
		 * Clears the data for the currently selected content.
		 */
		_clearFragmentSelection: function() {
			this.selectingFragmentId = ""; //$NON-NLS-1$
			this.selectingFragmentLabel = ""; //$NON-NLS-1$
		},
				
		 /**
         * Populates the description presentation of the specified practice for constructing section tree.
         * 
         * @param {Object} item the model data of the tree node representing the practice in the practice tree
         */
		populatePracticeDescription: function(item) {
			this.selectingLabel = item.label;
			this.selectingUrl = item.id;
			
			if (this.showSectionTree) {
				// clear previous selection on section tree
				this._clearFragmentSelection();
				// build section tree
				this.buildSectionTree(item.id);
			} else {
				this._hideMessage();
			}
		},
		
		/**
		 * Populates the description presentation of the specified practice element for constructing section tree.
		 * 
		 * @param {Object} item the model data of the tree node representing the practice element in the practice tree
		 */
		populatePracticeElement: function(item) {
			this.selectingLabel = item.label[0];
			this.selectingUrl = item.resource[0];
			
			if (this.showSectionTree) {
				// clear previous selection on section tree
				this._clearFragmentSelection();
				// build section tree
				this.buildSectionTree(item.resource[0]);
			} else {
				this._hideMessage();
			}
		},
		
		/**
		 * Construct the section tree.
		 * 
		 * @param {Object} practiceUrl the url of practice or practice element which is the parent of the sections in the tree.
		 */
		buildSectionTree: function(practiceUrl) {
			jazz.client.xhrGet({
				preventCache: true,
				url: practiceUrl + "?lang=" + this.currentLanguage + "&view=description", //$NON-NLS-1$ //$NON-NLS-2$
				headers: {
					"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
				},
				handleAs: "xml", //$NON-NLS-1$
				error: dojo.hitch(this, function(error) {
					this.displayErrorMessage(error);
				}),
				load: dojo.hitch(this, function(data) {
					this.sectionMap = this._createSectionMap(data);
					if (this.sectionMap) {
						var practiceJsonData = this._createSectionTreeData(this.sectionMap);
						this.sectionTree = this.createSectionTree(practiceJsonData);
						
						dojo.empty(this.sectionTreeArea);
						this.sectionTreeArea.appendChild(this.sectionTree.domNode);
						
						this._selectInitialSectionTreeItem();
					}
					this._hideMessage();
				})
			});
		},
		
		/**
		 * Select the initial choice in section tree.
		 */
		_selectInitialSectionTreeItem: function() {
			if (!this.selectedFragmentId) {
				return;
			}
			this.sectionTree.model.store.fetchItemByIdentity({
				identity: this.selectedFragmentId,
				onItem: dojo.hitch(this, function(item) {
					if (item) {
						var path = ["root"]; //$NON-NLS-1$
						if (item.type == ProcessAuthoringConstants.FRAGMENT_TYPE_SECTION) {
							this._onOpenSection(item.id[0]);
							path.push(item.id[0]);
						} else if (item.type == ProcessAuthoringConstants.FRAGMENT_TYPE_FIELD) {
							this._onOpenField(item.id[0], item.parentId[0]);
							path.push(item.parentId[0]);
							path.push(item.id[0]);
						}
						setTimeout(dojo.hitch(this, function() {
							this.sectionTree.set("path", path); //$NON-NLS-1$
						}), 500);
					}
				})
			});
		},
		
        /**
         * create a section map to cache the section and field information for current practice.
         * sectionMap data format:
         * {
         *   sectionId1: {"fieldMap": fieldMapsInSection1, "name": sectionLabel1},
         *   sectionId2: {"fieldMap": fieldMapsInSection2, "name": sectionLabel2},
         *   .....,
         *   sectionIdn: {"fieldMap": fieldMapsInSectionn, "name": sectionLabeln},
         * }
         *
         * fieldMap data format:
         * {
         *   fieldId1: {"value": fieldText1, "name": fieldLabel1},
         *   fieldId2: {"value": fieldText2, "name": fieldLabel2},
         *   .....,
         *   fieldIdn: {"value": fieldTextn, "name": fieldLabeln},
         * }
         * 
         * @param practice the data that returned from xhrGet request
         * @return an section map object
         */
		_createSectionMap: function(practice) {
			var sectionMap = {};
			var sections = UIHelper.getAllSections(practice);
			var hasSection = false;
			var hasField = false;
			dojo.forEach(sections, function(section) {
				var sectionId = UIHelper.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
				if (sectionId && sectionId.value) {
					hasSection = true;
					var fieldMap = {};
					var fields = UIHelper.getAllFields(section);
					dojo.forEach(fields, function(field) {
						var idAttr = UIHelper.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
						if (idAttr && idAttr.value) {
							hasField = true;
							var fieldLabel = UIHelper.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
							var value = UIHelper.getFieldText(field);
							fieldMap[idAttr.value] = {
								value: UIHelper.unescapeXML(value),
								name: fieldLabel ? fieldLabel : this.messages.NoFieldLabel
							};
						}
					}, this);
					
					var sectionLabel = UIHelper.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
					sectionMap[sectionId.value] = {
						fieldMap: fieldMap,
						name: sectionLabel ? sectionLabel : this.messages.NoSectionLabel
					};
				}
			}, this);
			
			if (!hasField && !hasSection) {
				dojo.empty(this.sectionTreeArea);
				this.sectionTreeArea.appendChild(document.createTextNode(this.messages.SelectionDialog_1));
				return;
			}
			return sectionMap;
		},
		
		/**
		 * Create tree data for section tree from section map. 
		 * 
		 * @param {Object} sectionMap the map that contains all the sections, fields.
		 */
		_createSectionTreeData: function(sectionMap) {
			var practiceJsonData = {
				identifier: 'id', //$NON-NLS-1$
				label: 'label', //$NON-NLS-1$
				items: []
			};
			
			var i = 0;
			for (var sectionId in this.sectionMap) {
				practiceJsonData.items.push({
					"label": this.sectionMap[sectionId].name, //$NON-NLS-1$
					"id": sectionId, //$NON-NLS-1$
					"type": ProcessAuthoringConstants.FRAGMENT_TYPE_SECTION, //$NON-NLS-1$
					children: []
				});
				
				var fieldMap = this.sectionMap[sectionId].fieldMap;
				for (var fieldId in fieldMap) {
					practiceJsonData.items[i].children.push({
						"label": fieldMap[fieldId].name, //$NON-NLS-1$
						"id": fieldId, //$NON-NLS-1$
						"parentId": sectionId, //$NON-NLS-1$
						"type": ProcessAuthoringConstants.FRAGMENT_TYPE_FIELD //$NON-NLS-1$ 
					});
				}
				i++;
			}
			
			return practiceJsonData;
		},
		
		/**
		 * Create a section tree.
		 * 
		 * @param {Object} practiceJsonData the practice data (includes sections and fields) that is used to create section tree.
		 */
		createSectionTree: function(practiceJsonData) {
			var dataStore = new dojo.data.ItemFileReadStore({
				data: practiceJsonData
			});
			var model = new dijit.tree.ForestStoreModel({
				rootLabel: practiceJsonData.title,
				rootId: "root", //$NON-NLS-1$
				store: dataStore,
				childrenAttr: ["children"] //$NON-NLS-1$
			});
			var practiceContentTree = new com.ibm.team.process.authoring.web.common.ui.internal.dialogs._SectionTree({
				sectionMap: this.sectionMap,
				showRoot: false,
				model: model,
				onClick: dojo.hitch(this, this._onSectionTreeClick),
				getIconClass: dojo.hitch(this, this._getIconClass),
			}, document.createElement("div")); //$NON-NLS-1$
			return practiceContentTree;
		},
		
		_onSectionTreeClick: function (/* dojo.data */item, /*TreeNode*/ node){
			if (item.type == ProcessAuthoringConstants.FRAGMENT_TYPE_SECTION) {
				this._onOpenSection(item.id[0]);
			} else if (item.type == ProcessAuthoringConstants.FRAGMENT_TYPE_FIELD) {
				this._onOpenField(item.id[0], item.parentId[0]);
			}
		},
		
		_onOpenSection: function(sectionId) {
			this.selectingFragmentLabel = this.sectionMap[sectionId].name;
			this.selectingFragmentId = sectionId;
		},
		
		_onOpenField: function(fieldId, sectionId) {
			var fieldMap = this.sectionMap[sectionId].fieldMap;
			var fieldContent = fieldMap[fieldId].value;
			this.selectingFragmentLabel = fieldMap[fieldId].name;
			this.selectingFragmentId = fieldId;
		},
		
		_getIconClass: function(item, opened) {
			if (item && item.type == ProcessAuthoringConstants.FRAGMENT_TYPE_SECTION) {
				return "sectionIcon"; //$NON-NLS-1$ 
			} else if (item && item.type == ProcessAuthoringConstants.FRAGMENT_TYPE_FIELD) {
				return "fieldIcon"; //$NON-NLS-1$
			} else {
				return this.inherited(arguments);
			}
		},
		
		displayErrorMessage: function(error) {
			var message = error.responseText ? error.responseText : error.message;
			if (error.status === ProcessAuthoringConstants.SC_FORBIDDEN) {
				this._showMessage(this.messages.permissionDenied, "ErrorStatus"); //$NON-NLS-1$
			} else {
				this._showMessage(message, "ErrorStatus"); //$NON-NLS-1$
			}
			
		},
		
		_showMessage: function(message, cssClass) {
			dojo.empty(this.messageArea);
			this.messageArea.appendChild(document.createTextNode(message));
			dojo.removeClass(this.messageArea);
			dojo.addClass(this.messageArea, cssClass);
			dojo.style(this.messageArea, "visibility", "visible"); //$NON-NLS-1$  //$NON-NLS-2$
		},
		
		_hideMessage: function() {
			dojo.style(this.messageArea, "visibility", "hidden"); //$NON-NLS-1$  //$NON-NLS-2$
		},
		
		/**
		 * Get label and url of the selected practice or practice element. 
		 * 
		 * @return an object that contains the label of the selected practice or practice element as well as its url.
		 */
		getSelectedPracticeOrPracticeElement: function() {
			if (this.selectingUrl) {
				return {
					"label": this.selectingLabel, //$NON-NLS-1$
					"url": this.selectingUrl //$NON-NLS-1$
				};
			} else {
				return {
					"label": "", //$NON-NLS-1$  //$NON-NLS-2$
					"url": "" //$NON-NLS-1$   //$NON-NLS-2$
				};
			}
		},
		
		/**
		 * Get label of the selected section or field.
		 * 
		 * @return an object that contains the label of the selected section or field as well as its id.
		 */
		getSelectedFragment: function() {
			if (this.selectingFragmentId) {
				return {
					"label": this.selectingFragmentLabel, //$NON-NLS-1$
					"id": this.selectingFragmentId //$NON-NLS-1$
				};
			} else {
				return {
					"label": "", //$NON-NLS-1$  //$NON-NLS-2$
					"url": "" //$NON-NLS-1$  //$NON-NLS-2$
				};
			}
		},
		
		onCancel: function(e) {
			this.close();
		},
		
		onOk: function(e) {
			this.close();
		},
		
		/**
		 * Deselect the choice in the practice tree and section tree. 
		 */
		onClearSelection: function(e) {
			// clear selection in practice tree
			if (this.selectingUrl) {
				var url = this.selectingUrl;
				var resourceId = url.substring(url.lastIndexOf("/") + 1); //$NON-NLS-1$
				// clear tree selection
				if (ProcessAuthoringClient.isPracticeURL(url)) { //practices
					this.treeContainer.getItem(url, null, dojo.hitch(this, function(item, tree) {
						tree.set("selectedItem", []); //$NON-NLS-1$
					}));
				} else if (UIHelper.isPracticeElementURL(url)) { //practice elements
					dojo.some(this.treeContainer.trees, function(tree) {
						if (resourceId) {
							tree.model.store.fetchItemByIdentity({
								identity: resourceId,
								onItem: dojo.hitch(this, function(item) {
									if (item) {
										tree.set("path", []); //$NON-NLS-1$
									}
									return true;
								})
							});
						}
						return false;
					}, this);
				}
				
				// clear selection
				this.selectingLabel = ""; //$NON-NLS-1$
				this.selectingUrl = ""; //$NON-NLS-1$
				this.selectedUrl = ""; //$NON-NLS-1$
			}
			
			// clear selection in section tree
			if (this.selectingFragmentId) {
				this.sectionTree.model.store.fetchItemByIdentity({
					identity: this.selectingFragmentId,
					onItem: dojo.hitch(this, function(item) {
						if (item) {
							this.sectionTree.set("path", []); //$NON-NLS-1$
						}
					})
				});
				
				// clear selection
				this._clearFragmentSelection();
				this.selectedFragmentId = ""; //$NON-NLS-1$
			}
			
		},
		
		close: function() {
			this.dialog.close();
		}
	});
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.dialogs._RichHover", jazz.ui.ResourceLink, { //$NON-NLS-1$
		postCreate: function() {
			this.lazyFetch = true;			
			this.inherited(arguments);
		},
		
		// buildRendering() is overridden to show nothing
		buildRendering: function() {
			this.domNode = dojo.create("div"); //$NON-NLS-1$
		}
	});
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.dialogs._PracticeTree", PracticeTree, { //$NON-NLS-1$
		createActions: function(item, treeNode, practiceTree) {
			var resourceUri;
			if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT) {
				resourceUri = item.resource[0];
			} else if (item.root) {
				resourceUri = practiceTree.practiceResource;
			}
			if (resourceUri) {
				var richHover = new com.ibm.team.process.authoring.web.common.ui.internal.dialogs._RichHover({
					uri: resourceUri,
					around: treeNode.contentNode
				});
			}
		}
	});
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.dialogs._SectionTree", dijit.Tree, { //$NON-NLS-1$
		_createTreeNode: function(/*Object*/args) {
			var item = args.item;
			var content = ""; //$NON-NLS-1$
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.dialogs", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
			if (item.type == ProcessAuthoringConstants.FRAGMENT_TYPE_SECTION) {
				var fieldMap = this.sectionMap[item.id[0]].fieldMap;
				var sectionName = ""; //$NON-NLS-1$
				if (this.sectionMap[item.id[0]].name !== this.messages.NoSectionLabel) {
					sectionName = this.sectionMap[item.id[0]].name;
				}
			
				var parentDiv = document.createElement("div"); //$NON-NLS-1$
				var div = document.createElement("div"); //$NON-NLS-1$
				
				// add the style programmatically since the css class can not be indentified in rich hover.
				dojo.style(div, "margin", "3px 3px 5px 3px"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(div, "color", "#3087B3"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(div, "fontSize", "11pt"); //$NON-NLS-1$ //$NON-NLS-2$
				div.appendChild(document.createTextNode(sectionName));
				parentDiv.appendChild(div);
				var sectionText = parentDiv.innerHTML;
				
				for (var fieldId in fieldMap) {
					var fieldTitle = ""; //$NON-NLS-1$
					if (fieldMap[fieldId].name !== this.messages.NoFieldLabel) {
						fieldTitle = fieldMap[fieldId].name;
					}
					var fieldContent = fieldMap[fieldId].value;
					sectionText += this._getFieldNode(fieldTitle, fieldContent).domNode.innerHTML;
				}
				content = sectionText;
			} else if (item.type == ProcessAuthoringConstants.FRAGMENT_TYPE_FIELD) {
				var fieldId = item.id[0];
				var fieldMap = this.sectionMap[item.parentId[0]].fieldMap;
				var fieldContent = fieldMap[fieldId].value;
				var fieldTitle = ""; //$NON-NLS-1$
				if (fieldMap[fieldId].name !== this.messages.NoFieldLabel) {
					fieldTitle = fieldMap[fieldId].name;
				}
				content = this._getFieldNode(fieldTitle, fieldContent).domNode.innerHTML;
			}
			var tnode = new dijit._TreeNode(args);
			
			var hoverPopup = new jazz.ui.HoverPopup({
				around: tnode.contentNode,
				html: content
			});
			return tnode;
		},
		
		_getFieldNode: function(fieldTitle, fieldContent) {
			// add field content for preview
			var div = document.createElement("div"); //$NON-NLS-1$
			var fieldView = new FieldViewer({
				fieldTitle: fieldTitle,
				fieldContent: fieldContent
			}, div);
			
			// add the style programmatically since the css class can not be indentified in rich hover.
			dojo.style(fieldView.fieldBody, "margin", "3px"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(fieldView.titleArea, "color", "#666666"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(fieldView.titleArea, "fontWeight", "bold"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(fieldView.titleArea, "marginTop", "0"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(fieldView.titleArea, "marginBottom", "0.5em"); //$NON-NLS-1$ //$NON-NLS-2$
			if (fieldView.titleArea.className === "hide") { //$NON-NLS-1$ 
				dojo.style(fieldView.titleArea, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			dojo.style(fieldView.contentArea, "marginTop", "0"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(fieldView.contentArea, "marginBottom", "0.5em"); //$NON-NLS-1$ //$NON-NLS-2$
			if (fieldView.contentArea.className === "hide") { //$NON-NLS-1$ 
				dojo.style(fieldView.contentArea, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			return fieldView;
		}
	});
})();
