/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.AbstractResourcePickerDialog"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$ 
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var DEFAULTRESOURCETYPE = 0;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.AbstractResourcePickerDialog", null, { //$NON-NLS-1$
		resourceGroups: [],
		currentType: DEFAULTRESOURCETYPE,
		selectedResourcesMap: null,
		selectingResources: [],
		dialog: null,
		cachedSearchText: null,
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.selectedResourcesMap = {};
			dojo.forEach(args.selectedResources, function(resource) {
				this.selectedResourcesMap[resource.url] = true;
			}, this);
		},
		
		setProperties: function(properties, args) {
			if (!args) {
				args = {};
			}
			
			for (var prop in properties) {
				if (args[prop]) {
					this[prop] = args[prop];
				} else {
					this[prop] = properties[prop];
				}
			}
			
			if (!this.width) {
				var dialogWidthInIE = this.dialogTitle.length * 2;
				if (dialogWidthInIE < ProcessAuthoringConstants.DIALOG_MIN_WIDTH_ON_IE) {
					dialogWidthInIE = ProcessAuthoringConstants.DIALOG_MIN_WIDTH_ON_IE;
				}
				this.width = dojo.isIE ? dialogWidthInIE + "em" : ProcessAuthoringConstants.EMPTY; //$NON-NLS-1$ //$NON-NLS-2$				
			}
		},
		
		setupConnections: function() {
			this.connect(this.resourceSelector, "ondblclick", "_onAddClose"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.resourceTypeSelector, "onchange", "_onResourceTypeChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.searchText, "onkeyup", "handleSearchChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.searchText, "onchange", "handleSearchChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.resourceSelector, "onchange", "_onSelectionChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.addCloseButton, "onclick", "_onAddClose"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.addButton, "onclick", "_onAdd"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		init: function() {
			this.setupObjects();
			this.setupConnections();
			this.addResourceTypeOptions(this);
			this.setupButtons();
			this.resourceSelector.multiple = this.allowMultiple;
		},
		
		addResourceTypeOptions: function() {
			if (this.resourceGroups.length > 1) {
				dojo.style(this.resourceTypeSelector, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
				dojox.xml.parser.replaceChildren(this.selectionTitle, document.createTextNode(this.selectionTitleLabel));
				dojo.style(this.selectionTitle, "width", this.selectionTitleLabel.length + "em"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.forEach(this.resourceGroups, function(resourceGroup) {
					var i = 0;
					var option = new Option(resourceGroup.type, i++);
					if (dojo.isIE) {
						this.resourceTypeSelector.options.add(option);
					} else {
						this.resourceTypeSelector.appendChild(option);
					}
				}, this);
				this.resourceTypeSelector.options[this.currentType].selected = true;
				this._getResources(this.currentType);
			} else {
				dojo.style(this.resourceTypeSelector, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
				dojox.xml.parser.replaceChildren(this.selectionTitle, document.createTextNode(this.resourceGroups[this.currentType].type));
				dojo.style(this.selectionTitle, "width", this.resourceGroups[this.currentType].type.length + "em"); //$NON-NLS-1$ //$NON-NLS-2$
				this._getResources(this.currentType);
			}
		},
		
		error: function(err) {
			var errorMessage;
			if (err.responseText) {
				errorMessage = err.responseText;
			} else {
				errorMessage = err.message;
			}
			this._setStatusMessage(errorMessage, true);
		},
		
		_clearStatusMessage: function() {
			dojo.style(this.status, "visibility", "hidden"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_setStatusMessage: function(msg, isError) {
			dojo.style(this.status, "visibility", "visible"); //$NON-NLS-1$ //$NON-NLS-2$
			dojox.xml.parser.replaceChildren(this.status, document.createTextNode(msg));
			this.status.className = isError ? "ErrorStatus" : "NormalStatus"; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		loadResources: function(resources) {
			if (!this.resourceSelector) {
				return; //disposed
			}
			
			dojo.empty(this.resourceSelector);
			if (this.disableAfterSelection && this.allowMultiple) {
				this.showResourcesWithChecking(resources);
			} else {
				this.showResourcesWithNoChecking(resources);
				this.addButton.disabled = !(this.resourceSelector.options.length > 0);
				this.addCloseButton.disabled = !(this.resourceSelector.options.length > 0);
				
				if (!this.allowMultiple && this.resourceSelector.options.length > 0) {
					if (this.associateSharedProcessDescriptionURL != undefined && this.associateSharedProcessDescriptionURL != null && this.associateSharedProcessDescriptionURL != "") { //$NON-NLS-1$
						for (var i = 0; i < this.resourceSelector.options.length; i++) {
							if (this.resourceSelector.options[i].resourceUrl === this.associateSharedProcessDescriptionURL) {
								this.resourceSelector.options[i].selected = true;
							}
						}
					} else {
						this.resourceSelector.options[0].selected = true;
					}
				}
			}
			
			if (this.searchText.value != "") { //$NON-NLS-1$
				var optionLen = this.resourceSelector.children.length;
				for (var i = 0; i < optionLen; i++) {
					this.toggleOption(this.resourceSelector, i, this.matchCheck(this.searchText.value, this.resourceSelector.children[i].innerHTML));
				}
			}
			
			var selected = false;
			if (this.resourceSelector.options.length > 0) {
				if (this.resourceSelector.selectedIndex != -1) {
					selected = true;
				}
			}
			this.addCloseButton.disabled = !selected;
			this.addButton.disabled = !selected;
		},
		
		showResourcesWithNoChecking: function(resources) {
			var i = 0;
			dojo.forEach(resources, function(resource) {
				var option = new Option(resource.name, resource.url);
				option.resourceUrl = resource.resourceUrl;
				option.resourceIdx = i++;
				this.resourceSelector.options.add(option);
			}, this);
		},
		
		showResourcesWithChecking: function(resources) {
			var i = 0;
			dojo.forEach(resources, function(resource) {
				var option = new Option(resource.name, resource.url);
				option.resourceUrl = resource.resourceUrl;
				option.resourceIdx = i++;
				if (this.selectedResourcesMap[resource.url]) {
					option.disabled = true;
					option.style.color = ProcessAuthoringConstants.STYLE_GRAYTEXT;
				}
				if (dojo.isIE) {
					this.resourceSelector.options.add(option);
				} else {
					this.resourceSelector.appendChild(option);
				}
				
			}, this);
		},
		
		_onResourceTypeChange: function(e) {
			this._clearStatusMessage();
			var idx = this.resourceTypeSelector.selectedIndex;
			this.currentType = idx;
			this._getResources(idx);
		},
		
		_onAdd: function(e) {
			dojo.stopEvent(e);
			if (this.allowMultiple) {
				this.onAdd(this.selectingResources);
			} else {
				if (dojo.style(this.resourceTypeSelector, "display") !== "none") { //$NON-NLS-1$ //$NON-NLS-2$
					var resourceList = this.resourceGroups[this.resourceTypeSelector.selectedIndex].list;
				} else {
					resourceList = this.resourceGroups[DEFAULTRESOURCETYPE].list;
				}
				var resource = resourceList[this.resourceSelector.options[this.resourceSelector.selectedIndex].resourceIdx];
				this.onAdd(resource);
			}
			
			if (this.disableAfterSelection && this.allowMultiple) {
				for (var i in this.selectingResources) {
					resource = this.selectingResources[i];
					this.resourceSelector.options[resource.optionIdx].disabled = true;
					this.resourceSelector.options[resource.optionIdx].selected = false;
					this.resourceSelector.options[resource.optionIdx].style.color = ProcessAuthoringConstants.STYLE_GRAYTEXT;
				}
			}
			this._onSelectionChange();
		},
		
		handleSearchChange: function(e) {
			//to avoid duplicated searching
			if (this.cachedSearchText === this.searchText.value) {
				return;
			}
			
			this.cachedSearchText = this.searchText.value;
			
			if (this.searchText.value == "") //$NON-NLS-1$
			{
				this._getResources(this.currentType);
				return;
			}
			
			var optionLen = this.resourceSelector.children.length;
			for (var i = 0; i < optionLen; i++) {
				this.toggleOption(this.resourceSelector, i, this.matchCheck(this.searchText.value, this.resourceSelector.children[i].innerHTML));
			}
			
			var selected = false;
			if (!this.allowMultiple && this.resourceSelector.options.length > 0) {
				this.resourceSelector.options[0].selected = true;
				selected = true;
			}
			
			if (this.allowMultiple && this.resourceSelector.options.length > 0) {
				if (this.resourceSelector.selectedIndex != -1) {
					selected = true;
				}
			}
			
			this.addCloseButton.disabled = !selected;
			this.addButton.disabled = !selected;
		},
		
		matchCheck: function(searchText, text) {
			if (!searchText) {
				return true;
			}
			
			var reg = new RegExp("^" + searchText.replace(/\*/g, "[\\w\\s:]*").replace(/\?/g, "."), "gi"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			return reg.exec(text) !== null;
		},
		
		//option.style.display='none' doesn't work for IE, so use following function for IE/Firefox compatibility purpose 
		toggleOption: function(theSelector, index, showOption) {
			var oldOption = theSelector.children[index];
			var newTagName = showOption ? 'option' : 'div'; //$NON-NLS-1$ //$NON-NLS-2$
			var oldTagName = showOption ? 'DIV' : 'OPTION'; //$NON-NLS-1$ //$NON-NLS-2$
			if (oldOption.tagName === oldTagName) {
				var oldStr = oldOption.innerHTML;
				var newOption = document.createElement(newTagName);
				newOption.innerHTML = oldStr;
				dojo.attr(newOption, 'value', dojo.attr(oldOption, 'value')); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.attr(newOption, 'resourceIdx', dojo.attr(oldOption, 'resourceIdx')); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.attr(newOption, 'resourceUrl', dojo.attr(oldOption, 'resourceUrl')); //$NON-NLS-1$ //$NON-NLS-2$
				theSelector.appendChild(newOption);
				theSelector.replaceChild(newOption, oldOption);
				if (!showOption) {
					dojo.style(newOption, 'display', 'none'); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		},
		
		_onSelectionChange: function(e) {
			if (this.allowMultiple) {
				var selected = false;
				this.selectingResources = [];
				
				for (var i = 0; i < this.resourceSelector.options.length; i++) {
					if ((this.resourceSelector.options[i].selected) && (!this.resourceSelector.options[i].disabled)) {
						selected = true;
						this.selectingResources.push({
							url: this.resourceSelector.options[i].value,
							name: this.resourceSelector.options[i].text,
							resourceUrl: this.resourceSelector.options[i].resourceUrl,
							optionIdx: i
						});
					}
				}
				this.addCloseButton.disabled = !selected;
				this.addButton.disabled = !selected;
			}
		},
		
		_onClose: function() {
		},
		
		//abstract functions
		_getResources: function(type) {
		},
		
		setupButtons: function() {
		},
		
		onAdd: function(resource) {
		
		}
		
	});
})();
