/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.Toolbar"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dijit.tree.dndSource"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var Toolbar = com.ibm.team.process.authoring.web.common.ui.internal.editors.Toolbar;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTree", dijit.Tree, { //$NON-NLS-1$
		practiceResource: null,
		persist: false,
		isNew: false,
		viewOnly: false,
		rootOnly: false, // if true, shows only the root node(s)
		indexInParent: -1, // the index of the tree in the tree container that contains many trees
		treeContainer: null,
		toolBarList: null,
		addWidth: -1,
		dndController: null,
		viewMode: true, // this is irrelevant if viewOnly is true
		isSharedTree: false,
		rootIconClass: null, // CSS class for the icon of the root node in the practice tree
		
		constructor: function(args) {
			this.toolBarList = [];
			this.betweenThreshold = 5;
			this.dndController = "dijit.tree.dndSource"; //$NON-NLS-1$
		},
		
		postCreate: function() {
			this.inherited(arguments);
			this.dndController.singular = true;  // Only allow single drag of items
			this.dndController.isSource = !this.viewOnly && !this.viewMode;
		},
		
		checkAcceptance: function(source, nodes) {
			// Can not drag shared practices and its children
			var practiceTree = source.tree;
			if (practiceTree.isSharedTree) {
				return false;
			}
			// Can not drag practice
			if (nodes.length > 0) {
				var treeNode = dijit.getEnclosingWidget(nodes[0]);
				var item = treeNode.item;
				if (item.root === true) {
					return false;
				}
			}
			return true;
		},
		
		checkItemAcceptance: function(target, source, position) {
			// Compute the new parent item
			this.targetAnchorNode = target;
			var targetNode = dijit.getEnclosingWidget(target);
			var targetItem = targetNode.item;
			
			this.position = position;
			// Can not drop over practice element node 
			if (targetItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT && position === "over") { //$NON-NLS-1$
				return false;
			}
			
			var practiceTree = source.tree;
			if (practiceTree.practiceResource != targetNode.tree.practiceResource) {
				return false;
			}
			return true;
		},
		
		onDndDrop: function(source, nodes, copy) {
			if (this.containerState == "Over") { //$NON-NLS-1$
				// remember the source item (item to be moved)
				var oldTreeNode, oldItem, oldParentItem;
				if (nodes.length > 0) {
					oldTreeNode = dijit.getEnclosingWidget(nodes[0]);
					oldItem = oldTreeNode.item;
					oldParentItem = oldTreeNode.getParent().item;
				}
				var practiceTree = source.tree;
				var target = this.targetAnchorNode;
				
				// Compute the new parent item
				var targetNode = dijit.getEnclosingWidget(target);
				var newParentItem;
				if (this.position == "over") { //$NON-NLS-1$
					newParentItem = (targetNode && targetNode.item) || practiceTree.item;
				} else if (this.position == "before" || this.position == "after") { //$NON-NLS-1$  //$NON-NLS-2$
					newParentItem = (targetNode.getParent() && targetNode.getParent().item) || practiceTree.item;
				}
				
				var targetIndex = targetNode.getIndexInParent();
				if (newParentItem.root === true && targetIndex == -1) {
					targetIndex = 0;
				}
				
				if (this.position == "before" && newParentItem.id == oldParentItem.id && oldTreeNode.getIndexInParent() < targetIndex) { //$NON-NLS-1$
					targetIndex = targetIndex - 1;
				}
				
				if (this.position == "after" && !(newParentItem.id == oldParentItem.id && oldTreeNode.getIndexInParent() < targetIndex)) { //$NON-NLS-1$
					targetIndex = targetIndex + 1;
				}
				
				if (this.position == "over") { //$NON-NLS-1$
					if (newParentItem.children) {
						targetIndex = targetIndex + newParentItem.children.length;
					}
				}
				if (oldItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_ELEMENT ||
					oldItem.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
					practiceTree.onMoveNode(oldItem, oldParentItem, newParentItem, targetIndex, practiceTree);
					practiceTree.selectNode(oldItem);
				} else if (oldItem.root === true) {
					// Currently, do not support DnD for practice
				}
			}
			this.onDndCancel();
		},
		
		onMoveNode: function(sourceItem, oldParentItem, newParentItem, position, isDnD, sourceTree) {
		
		},
		
		_load: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
			this.model.getRoot(dojo.hitch(this, function(item) {
				item.isSharedElement = this.isSharedTree;
				item.id = this.practiceResource;
			}), function(err) {
			});
			this.inherited(arguments);
			if (!this.viewOnly) {
				this.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_VIEWMODE, this._previewMode); //$NON-NLS-1$
				this.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_EDITMODE, this._editMode); //$NON-NLS-1$
			}
		},
		
		_previewMode: function() {
			this.viewMode = true;
			if (this.dndController && !dojo.isString(this.dndController)) {
				this.dndController.isSource = false;
			}
		},
		
		_editMode: function() {
			if (this.dndController && !dojo.isString(this.dndController)) {
				this.dndController.isSource = true;
			}
			this.viewMode = false;
		},
		
		/**
		 * This method is called when a tree node is selected. It should be overridden to handle the tree node selection event.
		 * 
		 * @param {Object} {dojo.data} item the model data of the selected tree node
		 * @param {dijit._TreeNode} treeNode the selected tree node
		 * @param {PracticeTree} practiceTree this practice tree
		 */
		onSelectNode: function(/* dojo.data */ item, treeNode, practiceTree) {
		},
		
		onClick: function(/* dojo.data */item, /*TreeNode*/ node) {
			this.treeContainer.selectTree(this);
			this.onSelectNode(item, node, this);
		},
		
		getIconClass: function(item, opened) {
			if (!item) {
				return null;
			}
			
			if (item.root && this.rootIconClass) {
				return this.rootIconClass;
			}
			
			var sharedPractice = item.root ? false : Boolean(item._S.getValue(item, "shared")); //$NON-NLS-1$
			
			if (item.root || item.type == ProcessAuthoringConstants.NAVIGATORTYPE_PRACTICE) {
				return (this.isSharedTree || sharedPractice) ? ProcessAuthoringConstants.SHARED_PRACTICE_ICON_CLASS : ProcessAuthoringConstants.PRIVATE_PRACTICE_ICON_CLASS;
			} else if (item.type == ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
				return opened ? ProcessAuthoringConstants.CATEGORY_OPEN_ICON_CLASS : ProcessAuthoringConstants.CATEGORY_CLOSED_ICON_CLASS;
			} else {
				return ProcessAuthoringConstants.PRACTICE_ELEMENT_ICON_CLASS;
			}
		},
		
		pageX: function(elem) {
			if (dojo.style(elem, "visibility") === "hidden") { //$NON-NLS-1$ //$NON-NLS-2$
				return this.pageX(elem.previousSibling);
			} else {
				return elem.offsetParent ? elem.offsetLeft + this.pageX(elem.offsetParent) : elem.offsetLeft;
			}
		},
		
		pageRight: function(elem) {
		
			if (!elem) {
				return 0;
			}
			if (dojo.style(elem, "visibility") === "hidden") { //$NON-NLS-1$ //$NON-NLS-2$
				return this.pageRight(elem.previousSibling);
			} else {
				return this.pageX(elem) + elem.offsetWidth;
			}
		},
		
		/**
		 * Show/Hide the actions when the editor is in view mode.
		 * 
		 * @param {Widget} editToolBar - The ToolBar that contains all the actions 
		 * 
		 */
		handleButtonShowup: function(editToolBar) {
			dojo.forEach(editToolBar.getChildren(), dojo.hitch(this, function(btn) {
				if (btn.hideInViewMode) {
					dojo.toggleClass(btn.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, this.viewMode);
				}
			}));
		},
		
		/**
		 * Show the actions when mouse hovering on and adjust the width of practice tree pane if needed.
		 * 
		 * @param {Widget} node - The tree node that the mouse hovers on 
		 * 
		 */
		_onNodeMouseEnter: function(/*Widget*/node) {
			if (node.editToolBar) {
				this.addWidth = -1;
				dojo.toggleClass(node.editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, false);
				this.handleButtonShowup(node.editToolBar);
				if (node.editToolBar.domNode.lastChild) {
					var nodeRight = this.pageRight(node.editToolBar.domNode.lastChild);
					var treePanelRight = this.pageRight(this.treeContainer.domNode);
					if (treePanelRight < nodeRight) {
						this.addWidth = nodeRight - treePanelRight;
						dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESIZE_NAVIGATOR, [this.addWidth]);
					}
				}
			}
		},
		
		/**
		 * Hide the actions after mouse leaving and adjust the width of practice tree pane if needed.
		 * 
		 * @param {Widget} node - The tree node that the mouse leaves from 
		 * 
		 */
		_onNodeMouseLeave: function(/*Widget*/node) {
		
			if (node.editToolBar) {
				dojo.toggleClass(node.editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, true);
				if (this.addWidth !== -1) {
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESIZE_NAVIGATOR, [0 - this.addWidth]);
				}
			}
		},
		
		/**
		 * Creates the action buttons for the specified item.
		 * 
		 * This method should be overridden to provide actions for the specified tree item.
		 * 
		 * @param {Object} item the item in the tree to create actions for
		 * @param {dijit._TreeNode} treeNode the tree node of the specified item
		 * @param {PracticeTree} practiceTree this practice tree
		 * @return an array of toolbar buttons (jazz.ui.toolbar.Button)
		 */
		createActions: function(item, treeNode, practiceTree) {
		},
		
		/**
		 * Create a tree node with several actions. 
		 * Actions examples: copy URL, export practice, delete practice/element/category, etc.
		 * 
		 * @param {Object} args  The parameters needed by a tree node. For example: label, tooltip, etc.
		 * @return the newly created tree node
		 */
		_createTreeNode: function(/*Object*/ args) {
			var item = args.item;
			var tnode = new dijit._TreeNode(args);
			
			if (!item.root && this.isSharedTree) {
				item._S.setValue(item, "isSharedElement", true); //$NON-NLS-1$
			}
			
			if (this.rootOnly && !item.root) {
				return null;
			}
			
			var actionButtons = this.createActions(item, tnode, this);
			if (actionButtons && actionButtons.length > 0) {
				var editToolBar = new Toolbar();
				dojo.forEach(actionButtons, function(button) {
					editToolBar.addChild(button);	
				});
				dojo.toggleClass(editToolBar.domNode, ProcessAuthoringConstants.CSS_HIDE_OCCUPY, true);
				tnode.editToolBar = editToolBar;
				tnode.contentNode.appendChild(editToolBar.domNode);
				this.toolBarList.push(editToolBar);
			}
			
			return tnode;
		},
		
		selectNode: function(item) {
			var nodes = this.getNodesByItem(item);
			if(nodes.length > 0) {
				var node = nodes[0];
				this.treeContainer.selectTree(this);
				setTimeout(dojo.hitch(this, function() {
					this.set("path", node.getTreePath()); //$NON-NLS-1$
				}), 200);
			}
		},
				
		onError: function() {
		},
		
		destroy: function() {
			dojo.forEach(this.toolBarList, function(toolBar) {
				toolBar.destroyRecursive();
			});
			
			this.inherited(arguments);
		}
	});
	
})();
