/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.frontingapp.ui.internal.editors.ProcessAreaDescriptionEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.ProcessDescriptionEditor"); //$NON-NLS-1$
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Separator"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.TwoStateButton"); //$NON-NLS-1$
	
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	
	dojo.declare("com.ibm.team.process.authoring.web.frontingapp.ui.internal.editors.ProcessAreaDescriptionEditor", [com.ibm.team.process.authoring.web.common.ui.internal.editors.ProcessDescriptionEditor], { //$NON-NLS-1$ 
		discardType: null,
		
		//constants definition
		constructor: function(args) {
            this._discardIconSrc = "com.ibm.team.process.authoring.web.common/graphics/icons/delete-button.gif"; //$NON-NLS-1$
            this._dissociateIconSrc =  "com.ibm.team.process.authoring.web.common/graphics/icons/dis-proc-description_co.gif"; //$NON-NLS-1$
		},
		
		save: function(needRefresh) {
			if (typeof needRefresh != "undefined") { //$NON-NLS-1$
				this.refreshAfterSave = needRefresh;
			}
			this.doSave();
		},
		
		_initialize: function(args) {
			this.inherited(arguments);
			this.enableLock = parent.location.search.toLowerCase().indexOf(ProcessAuthoringConstants.ENABLELOCK) !== -1;
			this.viewOnly = args.shared; //shared process description is view only in the foundation
			if (this.isNew) {
				this.workingCopy.currentLanguage = args.currentLanguage;
			}
		},
		
		loadProcessDescription: function() {
			var self = this;
			ProcessAuthoringClient.getProcessDescription({
				lastModified: this.workingCopy.lastModified,
				resourceURL: this.resourceURL + "?lang=" + this.workingCopy.currentLanguage, //$NON-NLS-1$
				load: function(data, ioArgs) {
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_PERMISSTION_CHECK_DONE, [true]);
					self.processDescriptionETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
					self.populateData(data);
					self.resizeEditor();
				},
				error: function(error) {
					if (error.responseText && error.status == ProcessAuthoringConstants.SC_FORBIDDEN) {
						error.message = error.responseText;
						error.headerMessage = self.messages.ProcessDescriptionError_3;
						self.displayErrorMessage(error);
					} else {
						self.displayErrorMessage(error);
					}
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_PERMISSTION_CHECK_DONE, [false]);
				}
			});
		},
		
		_hide: function(isReadable) {
			if (isReadable) {
				this.primaryFieldContainer.style.display = "block"; //$NON-NLS-1$
				this.detailWrapper.style.display = "block"; //$NON-NLS-1$
				dojo.toggleClass(this.contentWrapper, ProcessAuthoringConstants.CSS_HIDE, false);
			}
		},
		
		loadData: function() {
			if (this.isRefreshCalled) {
				this.loadProcessDescription();
			}
		},
		
		postCreate: function() {
			if (!this.isNew) {
				this.checkPermissionDoneHandle = dojo.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_PERMISSTION_CHECK_DONE, this, this._hide);
				this.loadProcessDescription();
			} else {
				this._hide(true);
			}
			this.inherited(arguments);
		},
		
		toggleDirty: function(isDirty) {
			this.editorContainer.onEditorDirty(isDirty, this.name.value);
		},
		
		// override AbsractPracticeEditor.confirmNavigate() to handle process area editor navigate handlers
		confirmNavigate: function() {
			if (this.editorContainer.containerChangesPending()) {
				if (!confirm(this.messages.AbstractPracticesEditor_6)) {
					return false;
				}
			}
			this._unregisterHandlers();
			
			this.editorContainer.setDirty(false);
			this.editorContainer.unregisterHandlers();
			return true;
		},
		
		navigateToPractice: function(practiceResource) {
			window.location.href = practiceResource;
		},
		
		clearLoadingMessage: function() {
			//notice the container to remove the "loading..." message
			this.editorContainer.onEditorLoaded();
		},
		
		setupCommandBar: function() {
			this.commandBar = new jazz.ui.Toolbar();
            
            if (!this.viewOnly) {
            	this.editButton = new jazz.ui.toolbar.TwoStateButton({
            		href: "#", //$NON-NLS-1$
            		imgSrc: this._editModeIconSrc,
            		title: this.messages.AbstractPracticesEditor_12,
            		onClick: dojo.hitch(this, this.toggleViewEditMode)
            	});
            	this.commandBar.addChild(this.editButton);
			}
			
			
			this.copyUrlButton = new jazz.ui.toolbar.Button({
    			imgSrc: this._copyUrlIconSrc, //$NON-NLS-1$ $
    			title: this.messages.ProcessDescriptionEditor_30,
    			onClick: dojo.hitch(this, function(evt) {
    				dojo.stopEvent(evt);
    				this.onCopyResourceUrl(this.resourceURL); //$NON-NLS-1$
    			})
    		});
			this.commandBar.addChild(this.copyUrlButton);
			
			this.exportProcessDescriptionButton = new jazz.ui.toolbar.Button({
				href: "#", //$NON-NLS-1$
               	imgSrc: this._exportIconSrc,
               	title: this.messages.ProcessDescriptionEditorHTML_4,
				onClick: dojo.hitch(this, this.exportProcessDescription)
			});
			this.commandBar.addChild(this.exportProcessDescriptionButton);
            
            var splitter = new jazz.ui.toolbar.Separator();
            this.commandBar.addChild(splitter);
			this.commandsContainer.appendChild(this.commandBar.domNode);
			this.showDiscardLink(this.discardType);			
		},
		
		discardProcessDescription: function(event) {
            dojo.stopEvent(event);
			this.editorContainer.onEditorDiscardProcessDescription(this.resourceURL);
		},
		
		backToStart: function(event) {
			dojo.stopEvent(event);
			this.editorContainer.onEditorBackToStart(this.resourceURL);
		},
		
		showDiscardLink: function(linkType) {

			if (!linkType) {
				return;
			}
            
            //remove previous button first
            if (this.discardButton) {
            	this.discardButton.destroyRecursive();
            	this.discardButton = null;
            }
            
            switch (linkType) {
            	case ProcessAuthoringConstants.DISCARDTYPE_BACKTOSTART:
            		this.discardButton = new jazz.ui.toolbar.Button({
            			href: "#", //$NON-NLS-1$
            			imgSrc: this._discardIconSrc,
            			title: this.messages.ProcessDescriptionEditor_28,
            			onClick: dojo.hitch(this, this.backToStart)
            		});
            		this.commandBar.addChild(this.discardButton);
            		break;
            	case ProcessAuthoringConstants.DISCARDTYPE_OWNER:
            		this.discardButton = new jazz.ui.toolbar.Button({
            			href: "#", //$NON-NLS-1$
            			imgSrc: this._discardIconSrc,
            			title: this.messages.ProcessDescriptionEditor_28,
            			onClick: dojo.hitch(this, this.discardProcessDescription)
            		});
            		this.commandBar.addChild(this.discardButton);
            		break;
            	default: //discardassociatelink
            		this.discardButton = new jazz.ui.toolbar.Button({
            			href: "#", //$NON-NLS-1$
            			imgSrc: this._dissociateIconSrc,
            			title: this.messages.ProcessDescriptionEditor_29,
            			onClick: dojo.hitch(this, this.discardProcessDescription)
            		});
            		this.commandBar.addChild(this.discardButton);
            		
            		break;
            }
		},
		
		setupBreadcrumbs: function() {
		},
		
		resetAllFields: function() {
			this.inherited(arguments);
		},
		
		toggleViewEditMode: function(e) {
			this.inherited(arguments);			
		},
		
		getResult: function(results) {
			if (this.inherited(arguments)) {
				this.showDiscardLink(ProcessAuthoringConstants.DISCARDTYPE_OWNER);
				this.editorContainer.onEditorSaveSuccess(this.resourceURL);
			} else {
				for (var i = 0; i < results.length; i++) {
					var result = results[i];
					if (!result.success) {
						this.editorContainer.onEditorSaveError(result);
						break;
					}
				}
			}
		},
		
		saveResultCallback: function(errObject) {
			if (errObject) {
				this.editorContainer.onEditorSaveError(errObject);
			} else {
				this.showDiscardLink(ProcessAuthoringConstants.DISCARDTYPE_OWNER);
				this.editorContainer.onEditorSaveSuccess(this.resourceURL);
			}
		},
		
		displaySuccessMessage: function(msg) {
			this.displayMessageInProcessAreaEditor(msg, "OK"); //$NON-NLS-1$
		},
		
		displayErrorMessage: function(msg) {
			this.displayMessageInProcessAreaEditor(msg, "ERROR"); //$NON-NLS-1$
		},
		
		displayWaitingMessage: function(msg) {
			this.displayMessageInProcessAreaEditor(msg, "INFO"); //$NON-NLS-1$
		},
		
		displayMessageInProcessAreaEditor: function(msg, cssClass) {
			this.editorContainer.onEditorDisplayMessage(msg, cssClass);
		},
		
		uninitialize: function() {
		
			if (this.checkPermissionDoneHandle) {
				dojo.unsubscribe(this.checkPermissionDoneHandle);
				this.checkPermissionDoneHandle = null;
			}
			
			if (this.commandBar) {
				this.commandBar.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
})();
