/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.ResourceLinkPickerDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$ 
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.AbstractResourcePickerDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	var AbstractResourcePickerDialog = com.ibm.team.process.authoring.web.common.ui.internal.editors.AbstractResourcePickerDialog;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	
	var SHAREDPROCESSDESCRIPTIONS = 0;
	var PRACTICES = 1;
	var PRACTICEELEMENTS = 2;
	var PROCESSDESCRIPTIONURLTEMPLATE = "#action=com.ibm.team.process.authoring.editProcessDescription&resource=${0}"; //$NON-NLS-1$
	var PRACTICEURLTEMPLATE = "#action=com.ibm.team.process.authoring.editPractice&resource=${0}"; //$NON-NLS-1$
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.ResourceLinkPickerDialog", [AbstractResourcePickerDialog], { //$NON-NLS-1$
		sharedSupport: true, //if it's true, should also support shared practice/process description selection, otherwise, for private practice selection only
		privatePracticeList: null, //if it has value, it's from process description editor, need to check private practice 
		
		constructor: function(args) {
			if ( (typeof args.sharedSupport) === "undefined" ) { //$NON-NLS-1$
				args.sharedSupport = false;
			}

			if ( (typeof args.privatePracticeList) === "undefined" ) { //$NON-NLS-1$
				args.privatePracticeList = null;
			}

			//practice editor supports shared practice only
			if ( !args.privatePracticeList ) {
				args.sharedSupport = true;
			}
			
			this.sharedSupport = args.sharedSupport;
			this.privatePracticeList = args.privatePracticeList;
			

			if ( !this.sharedSupport ) {
				this.SHAREDPROCESSDESCRIPTIONS_IDX = -1;
				this.PRACTICES_IDX = 0;
				this.PRACTICEELEMENTS_IDX = 1;
				this["resourceGroups"]= [{ //$NON-NLS-1$
					type: this.messages["ResourceLinkPickerDialogGroup_" + PRACTICES], //$NON-NLS-1$
					list: []
				}, {
					type: this.messages["ResourceLinkPickerDialogGroup_" + PRACTICEELEMENTS], //$NON-NLS-1$
					list: []
				}];
			} else {
				this.SHAREDPROCESSDESCRIPTIONS_IDX = 0;
				this.PRACTICES_IDX = 1;
				this.PRACTICEELEMENTS_IDX = 2;
				this["resourceGroups"]= [{ //$NON-NLS-1$
					type: this.messages["ResourceLinkPickerDialogGroup_" + SHAREDPROCESSDESCRIPTIONS], //$NON-NLS-1$
					list: []
				}, {
					type: this.messages["ResourceLinkPickerDialogGroup_" + PRACTICES], //$NON-NLS-1$
					list: []
				}, {
					type: this.messages["ResourceLinkPickerDialogGroup_" + PRACTICEELEMENTS], //$NON-NLS-1$
					list: []
				}];
			}
			
			this.setProperties({
				"modal": true, //$NON-NLS-1$ 
				"dialogTitle": this.messages.ResourceLinkPickerDialog_0, //$NON-NLS-1$
				"selectionTitleLabel": this.messages.ResourceLinkPickerDialogHTML_0, //$NON-NLS-1$
				"allowMultiple": true, //$NON-NLS-1$
				"disableAfterSelection": false, //$NON-NLS-1$
				"AddCloseButtonLabel": this.messages.ResourceLinkPickerDialog_ADD, //$NON-NLS-1$
				"CloseButtonLabel": this.messages.ResourceLinkPickerDialog_CLOSE //$NON-NLS-1$
			}, args);
		},
		
		setupButtons: function() {
			this.addCloseButton.appendChild(document.createTextNode(this.AddCloseButtonLabel));
			this.closeButton.appendChild(document.createTextNode(this.CloseButtonLabel));
			dojo.style(this.addCloseButton, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(this.closeButton, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
			try {
				UIHelper.resizeI18NButtons(this.dialogFooter, [this.addCloseButton, this.closeButton], -1, 3);
			} catch (e) {
				//failed to resize on IE for tooltipDialog, have to catch it. This is logged in workitem 111790
			}
		},
		
		loadPrivatePractices: function() {
			for (var i in this.privatePracticeList) {
				var practiceList = this.resourceGroups[this.PRACTICES_IDX].list;
				practiceList.push({
					name: this.privatePracticeList[i].name,
					url: dojo.string.substitute(PRACTICEURLTEMPLATE, [this.privatePracticeList[i].url]),
					resourceUrl: this.privatePracticeList[i].url
				});
			}
			if (!practiceList) {
				return;
			}
			practiceList.sort(function(practice1, practice2) {
				return practice1.name.toLocaleLowerCase().localeCompare(practice2.name.toLocaleLowerCase());
			});
		},
				
		loadAllSharedPractices: function(data) {
			var children = data.childNodes;
			var practices = UIHelper.getNode(children, ProcessAuthoringConstants.XMLELEMENT_RDF, ProcessAuthoringConstants.NAMESPACE_RDF);
			if (practices) {
				var allPractices = UIHelper.getNodes(practices.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRACTICE_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
				var practiceList = this.resourceGroups[this.PRACTICES_IDX].list;
				dojo.forEach(allPractices, function(practice) {
					var aboutAttribute = UIHelper.getAttributeNode(practice, ProcessAuthoringConstants.XMLATTRIBUTE_ABOUT, ProcessAuthoringConstants.NAMESPACE_RDF);
					var practiceName = UIHelper.getNodeTitle(practice);
					if (aboutAttribute && practiceName) { //$NON-NLS-1$ //we only display the practice that has about attribute and title
						practiceList.push({
							name: practiceName,
							url: dojo.string.substitute(PRACTICEURLTEMPLATE, [aboutAttribute.value]),
							resourceUrl: aboutAttribute.value
						});
					}
				}, this);
				practiceList.sort(function(practice1, practice2) {
					return practice1.name.toLocaleLowerCase().localeCompare(practice2.name.toLocaleLowerCase());
				});
			}
		},
		
		loadProcessDescriptions: function(data) {
			var descriptionsNode = UIHelper.getNode(data.childNodes, ProcessAuthoringConstants.XMLELEMENT_RDF, ProcessAuthoringConstants.NAMESPACE_RDF);
			if (descriptionsNode) {
				var allprocessDescriptionNodes = UIHelper.getNodes(descriptionsNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_PROCESSDESCRIPTION_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
				var sharedProcessDescription = this.resourceGroups[this.SHAREDPROCESSDESCRIPTIONS_IDX].list;
				
				dojo.forEach(allprocessDescriptionNodes, function(processDescriptionNode) {
					var value = UIHelper.getProcessDescriptionValues(processDescriptionNode);
					sharedProcessDescription.push({
						name: value.name,
						url: dojo.string.substitute(PROCESSDESCRIPTIONURLTEMPLATE, [value.url]),
						resourceUrl: value.url
					});
				}, this);
				sharedProcessDescription.sort(function(description1, description2) {
					return description1.name.toLocaleLowerCase().localeCompare(description2.name.toLocaleLowerCase());
				});
			}
		},
		
		loadAllPracticeElements: function(practiceList) {
			var idx = 0;
			var practiceNum = this.resourceGroups[this.PRACTICES_IDX].list.length;
			
			dojo.forEach(practiceList, function(practice) {
				ProcessAuthoringClient.getPractice({
					url: practice.resourceUrl,
					load: dojo.hitch(this, function(data) {
						this.loadPracticeElements(data, practice.url);
						//show practice element only when all elements are loaded
						if (++idx >= practiceNum) {
							var allPracticeElements = this.resourceGroups[this.PRACTICEELEMENTS_IDX].list;
							allPracticeElements.sort(function(element1, element2) {
								return element1.name.toLocaleLowerCase().localeCompare(element2.name.toLocaleLowerCase());
							});
							this.loadResources(this.resourceGroups[this.PRACTICEELEMENTS_IDX].list);
							this._clearStatusMessage();
						}
					}),
					error: dojo.hitch(this, this.error)
				});
			}, this);
		},
		
		loadPracticeElements: function(data, practiceURL) {
			var presentation = UIHelper.getPracticePresentation(data);
			if (presentation) {
				var allElements = presentation.childNodes;
				var allPracticeElements = this.resourceGroups[this.PRACTICEELEMENTS_IDX].list;
				dojo.forEach(allElements, function(element) {
					if (UIHelper.isCategoryNode(element)) {
						this.nestedLoadPracticeElements(element, practiceURL); //nested call
					} else if (UIHelper.isPracticeElement(element)) {
						var resourceAttr = UIHelper.getAttributeNode(element, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
						var elementName = UIHelper.getNodeLabel(element);
						if (resourceAttr && elementName) { //$NON-NLS-1$ //we only display the practice that has about attribute and title
							allPracticeElements.push({
								name: elementName,
								url: practiceURL,
								resourceUrl: resourceAttr.value
							});
						}
					}
				}, this);
			}
		},
		
		nestedLoadPracticeElements: function(grouping, practiceURL) {
			var elementList = this.resourceGroups[this.PRACTICEELEMENTS_IDX].list;
			var allElements = grouping.childNodes;
			dojo.forEach(allElements, function(element) {
				if (UIHelper.isCategoryNode(element)) {
					this.nestedLoadPracticeElements(element); //nested call
				} else if (UIHelper.isPracticeElement(element)) {
					var resourceAttr = UIHelper.getAttributeNode(element, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
					var elementName = UIHelper.getNodeLabel(element);
					if (resourceAttr && elementName) { //$NON-NLS-1$ //we only display the practice that has about attribute and title
						elementList.push({
							name: elementName,
							url: practiceURL,
							resourceUrl: resourceAttr.value
						});
					}
				}
			}, this);
			
		},
		
		_getResources: function(type) {
			switch (type) {
				case this.PRACTICES_IDX:
					//load all practices		
					if (this.resourceGroups[this.PRACTICES_IDX].list.length === 0) {
						this._setStatusMessage(this.messages.ResourceLinkPickerDialog_1);
						//load private practices
						if ( this.privatePracticeList ) {
							this.loadPrivatePractices(this.privatePracticeList);
							this.loadResources(this.resourceGroups[this.PRACTICES_IDX].list);
						}
						//load shared practices
						if ( this.sharedSupport ) {
							ProcessAuthoringClient.getAllSharedPractices({
								load: dojo.hitch(this, function(data) {
									this.loadAllSharedPractices(data);
									this.loadResources(this.resourceGroups[this.PRACTICES_IDX].list);
									this._clearStatusMessage();
								}),
								error: dojo.hitch(this, this.error)
							});
						} else {
							this._clearStatusMessage();
						}
					} else {
						this.loadResources(this.resourceGroups[this.PRACTICES_IDX].list);
					}
					break;
				case this.SHAREDPROCESSDESCRIPTIONS_IDX:
					if (this.resourceGroups[this.SHAREDPROCESSDESCRIPTIONS_IDX].list.length === 0) {
						this._setStatusMessage(this.messages.ResourceLinkPickerDialog_1);
						
						ProcessAuthoringClient.getAllProcessDescriptions({
							load: dojo.hitch(this, function(data) {
								this.loadProcessDescriptions(data);
								this.loadResources(this.resourceGroups[this.SHAREDPROCESSDESCRIPTIONS_IDX].list);
								this._clearStatusMessage();
							}),
							error: dojo.hitch(this, this.error)
						});
					} else {
						this.loadResources(this.resourceGroups[this.SHAREDPROCESSDESCRIPTIONS_IDX].list);
					}
					break;
				case this.PRACTICEELEMENTS_IDX:
					if (this.resourceGroups[this.PRACTICEELEMENTS_IDX].list.length === 0) {
						this._setStatusMessage(this.messages.ResourceLinkPickerDialog_1);
						var practiceNum = this.resourceGroups[this.PRACTICES_IDX].list.length;
						if (practiceNum > 0) {
							this.loadAllPracticeElements(this.resourceGroups[this.PRACTICES_IDX].list);
						} else {
							//load private practices if it's in process description editor
							if (this.privatePracticeList) {
								this.loadPrivatePractices(this.privatePracticeList);
								if (!this.sharedSupport) {
									this.loadAllPracticeElements(this.resourceGroups[this.PRACTICES_IDX].list);
								}
							}
							//load shared practices
							if (this.sharedSupport) {
								ProcessAuthoringClient.getAllSharedPractices({
									load: dojo.hitch(this, function(data) {
										this.loadAllSharedPractices(data);
										this.loadAllPracticeElements(this.resourceGroups[this.PRACTICES_IDX].list);
									}),
									error: dojo.hitch(this, this.error)
								});
							} else {
								this._clearStatusMessage();
							}
						}
					} else {
						this.loadResources(this.resourceGroups[this.PRACTICEELEMENTS_IDX].list);
					}
					break;
				default:
					break;
			}
		}
	});
})();
