/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.SharedPracticeSelectionDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$ 
	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.AbstractResourcePickerDialog"); //$NON-NLS-1$
	var AbstractResourcePickerDialog = com.ibm.team.process.authoring.web.common.ui.internal.editors.AbstractResourcePickerDialog;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.SharedPracticeSelectionDialog", [AbstractResourcePickerDialog, dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "ui/internal/editors/templates/ResourcePickerDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args) {
			this.inherited(arguments);
			
			this.setProperties({
				"dialogTitle": this.messages.SharedPracticeSelectionDialog_0, //$NON-NLS-1$
				"width": dojo.isIE ? "28em" : ProcessAuthoringConstants.EMPTY, //$NON-NLS-1$ //$NON-NLS-2$
				"allowMultiple": true, //$NON-NLS-1$
				"disableAfterSelection": true, //$NON-NLS-1$
				"modal": true //$NON-NLS-1$
			}, args);
			
			this.resourceGroups = [{
				type: this.messages.SharedPracticeSelectionDialog_2,
				list: []
			}];
		},
		
		setupObjects: function() {
		
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.contentNode,
				primaryTitle: this.dialogTitle,
				destroyContents: true,
				width: this.width
			});
		},
		
		setupConnections: function() {
			this.connect(this.closeButton, "onclick", function(e) { //$NON-NLS-1$
				dojo.stopEvent(e);
				this.dialog.close();
			});
			
			this.connect(this.dialog, "onClose", "_onClose"); //$NON-NLS-1$ //$NON-NLS-2$
			this.inherited(arguments);
		},
		
		
		postCreate: function() {
			this.inherited(arguments);
			this.init();
		},
		
		setupButtons: function() {
			this.addCloseButton.appendChild(document.createTextNode(this.messages.SharedPracticesSelectionDialog_ADDCLOSE));
			this.addButton.appendChild(document.createTextNode(this.messages.SharedPracticesSelectionDialog_ADD));
			this.closeButton.appendChild(document.createTextNode(this.messages.SharedPracticesSelectionDialog_CLOSE));
			dojo.style(this.addButton, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.addCloseButton, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.closeButton, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
		},
		
		_onAddClose: function(e) {
			this._onAdd(e);
			this.dialog.close();
		},
		
		_getResources: function(type) {
			var option = new Option(this.messages.SharedPracticeSelectionDialog_1);
			option.disabled = true;
			option.style.color = ProcessAuthoringConstants.STYLE_GRAYTEXT;
			this.resourceSelector.options.add(option);
			if (this.resourceGroups[0].list.length === 0) {
				this._setStatusMessage(this.messages.SharedPracticeSelectionDialog_1);
				ProcessAuthoringClient.getAllSharedPractices({
					load: dojo.hitch(this, function(data) {
						this.loadAllSharedPractices(data);
						this.loadResources(this.resourceGroups[0].list);
						this._clearStatusMessage();
					}),
					error: dojo.hitch(this, this.error)
				});
			} else {
				this.loadResources(this.resourceGroups[0].list);
			}
		},
		
		loadAllSharedPractices: function(data) {
			var children = data.childNodes;
			var practices = UIHelper.getNode(children, ProcessAuthoringConstants.XMLELEMENT_RDF, ProcessAuthoringConstants.NAMESPACE_RDF); 
			if (practices) {
				var allPractices = UIHelper.getNodes(practices.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRACTICE_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
				var sharedPracticeList = this.resourceGroups[0].list;
				
				dojo.forEach(allPractices, function(practice) {
					var aboutAttribute = UIHelper.getAttributeNode(practice, ProcessAuthoringConstants.XMLATTRIBUTE_ABOUT, ProcessAuthoringConstants.NAMESPACE_RDF);
					var practiceName = UIHelper.getNodeTitle(practice);
					if (aboutAttribute && practiceName) { //$NON-NLS-1$ //we only display the practice that has about attribute and title
						sharedPracticeList.push({
							name: practiceName,
							url: aboutAttribute.value,
							shared: "true" //$NON-NLS-1$
						});
					}
				}, this);
				sharedPracticeList.sort(function(practice1, practice2) {
					return practice1.name.toLocaleLowerCase().localeCompare(practice2.name.toLocaleLowerCase());
				});
			}
		},
		
		uninitialize: function() {
			dojo.forEach(this.resourceGroups, function(resourceGroup) {
				resourceGroup.list = [];
			}, this);
			this.inherited(arguments);
		}
	});
})();
