/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.frontingapp.ui.internal.editors.TemplateDescriptionEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.ProcessDescriptionEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.contextRoot");//$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.process_templates", "Messages");//$NON-NLS-1$//$NON-NLS-2$

	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var processConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var PROCESS_DESCRIPTION_CREATE_ACTION = "#action=com.ibm.team.process.templateAction&actionId=com.ibm.team.process.authoring.manageTemplateDescription&itemName=${0}&ownerURL=${1}"; //$NON-NLS-1$
	var PROCESS_DESCRIPTION_EDIT_ACTION = "#action=com.ibm.team.process.templateAction&actionId=com.ibm.team.process.authoring.manageTemplateDescription&resource=${0}"; //$NON-NLS-1$
	var messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_templates", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
	dojo.declare("com.ibm.team.process.authoring.web.frontingapp.ui.internal.editors.TemplateDescriptionEditor", [com.ibm.team.process.authoring.web.common.ui.internal.editors.ProcessDescriptionEditor], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.frontingapp", "ui/internal/editors/templates/TemplateDescriptionEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		templateListLink: "#action=com.ibm.team.process.manageTemplates", //$NON-NLS-1$
		waiting: false,
		
		constructor: function(args) {
			this.inherited(arguments);
		},
		
		_toggleHiddenButtons: function(isHide) {
			this.inherited(arguments);
			if (this.discardButton) {
				dojo.toggleClass(this.discardButton.domNode, ProcessAuthoringConstants.CSS_HIDE, isHide);
			}
		},
		
		/**
		 * Check if the template has already owned a process description, if yes, redirect to the description creating page,
		 * otherwise, redirect to the description editing page
		 * 		 
		 * @param {String} itemId template UUID passed in url
		 */
		// TODO: the code in this method is almost identical to TemplateButtonHandler.doAction()
		// 198321: Share common code in TemplateDescriptionEditor.redirectToEditor() and TemplateButtonHandler.doAction()
		redirectToEditor: function(itemId) {
			var templateUrl = document.location.protocol + "//" + document.location.host + net.jazz.ajax.contextRoot() + "/process/templates/" + itemId; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			var ownerUrl = UIHelper.getProcesTemplateUrl(itemId);
			ProcessAuthoringClient.getProcessDescriptionsForOwner({
				ownerUrl: ownerUrl,
				viewType: ProcessAuthoringConstants.VIEWTYPE_REFERENCE,
				load: dojo.hitch(this, function(data, ioArgs) {
					var descriptionsNode = UIHelper.getNode(data.childNodes, ProcessAuthoringConstants.XMLELEMENT_RDF, ProcessAuthoringConstants.NAMESPACE_RDF);
					if (descriptionsNode) {
						var allProcessDescriptionNodes = UIHelper.getNodes(descriptionsNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_PROCESSDESCRIPTION_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
						if (allProcessDescriptionNodes && allProcessDescriptionNodes.length > 0) {
							var processDescriptionNode = allProcessDescriptionNodes[0];
							var aboutAttribute = UIHelper.getAttributeNode(processDescriptionNode, ProcessAuthoringConstants.XMLATTRIBUTE_ABOUT, ProcessAuthoringConstants.NAMESPACE_RDF);
							var resourceUrl = aboutAttribute.value;
							window.location.href = dojo.string.substitute(PROCESS_DESCRIPTION_EDIT_ACTION, [resourceUrl]);
						} else {
							// No process description for template.
							// Because template name is not passed from the eclipse client for now, need to always fetch it so that it can be displayed in breadcrumb
							ProcessAuthoringClient.fetchTemplateInfo({
								url: templateUrl,
								load: dojo.hitch(this, function(data) {
									var templateName = this.getLocaleTemplateName(data.documentElement);
									//if there's no locale name, use the name defined in the doc element
									if (templateName == null) {
										templateName = UIHelper.getAttributeNode(data.documentElement, ProcessAuthoringConstants.XMLATTRIBUTE_NAME, processConstants.NAMESPACE_JP06).value;
									}
									window.location.href = dojo.string.substitute(PROCESS_DESCRIPTION_CREATE_ACTION, [encodeURIComponent(templateName), ownerUrl]);
								})
							});
						}
					}
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
                    this.displayErrorMessage(error);
				})
			});
		},
		
		_initialize: function(args) {
			//The request is from the eclipse rich client. It's rendered when a user clicks on the process description link of the process template editor
			if (!args.ownerURL && !args.resource && args.itemId) {
				//This is for showing blank page before the description editor is loaded
				this.waiting = true;
				this.templatePath = null;
				this.templateString = "<div>Loading...</div>"; //$NON-NLS-1$
				this.redirectToEditor(args.itemId);
			} else {
				// check license
				var readSharedId = ProcessAuthoringConstants.PROCESS_AUTHORING_OPERATION_ID_PREFIX + ProcessAuthoringConstants.PROCESS_AUTHORING_READ_SHARED_SUPPORT;
				var writeSharedId = ProcessAuthoringConstants.PROCESS_AUTHORING_OPERATION_ID_PREFIX + ProcessAuthoringConstants.PROCESS_AUTHORING_WRITE_SHARED_SUPPORT;
				ProcessAuthoringClient.getPracticeLibraryLicenseInfo({
					operationIds: [readSharedId, writeSharedId],
					load: dojo.hitch(this, function(data, ioArgs) {
						this.sharedSupport = data[readSharedId];
						this.canWriteShared = data[writeSharedId];
					}),
					error: dojo.hitch(this, function(error, ioArgs) {
						if (error.responseText) {
							error.message = error.responseText;
						}
						this.displayErrorMessage(error);
					})
				});
				
				this.inherited(arguments);
				this.templateName = args.itemName;
				this.isShared = false;
				this._refreshIconSrc = "com.ibm.team.process.authoring.web.common/graphics/icons/refresh16.gif"; //$NON-NLS-1$ 
				this._discardIconSrc = "com.ibm.team.process.authoring.web.common/graphics/icons/delete-button.gif"; //$NON-NLS-1$
			}
		},
		
		getLinks: function() {
			var links = [{
				href: this.templateListLink,
				label: messages.ManageTemplates_1
			}];
			
			if (this.templateName) {
				links.push({
					label: this.templateName
				});
			}
			
			return links;
		},
		
		/**
		 * Overload this function to fetch template name for displaying template name in breadscrumb
		 *
		 * @param XMLObject data
		 */
		populateData: function(data) {
			this.inherited(arguments);
			
			//don't check processDescriptionOwnerName, as it's set when the process description is created. 
			//Template name might be changed after the creation
			//and here we do nothing for any error.   
			if (!this.templateName) {
				ProcessAuthoringClient.fetchTemplateInfo({
					url: this.processDescriptionOwner,
					load: dojo.hitch(this, function(data) {
						this.templateName = this.getLocaleTemplateName(data.documentElement);
						//if there's no locale name, use the name defined in the doc element
						if (this.templateName == null) {
							this.templateName = UIHelper.getAttributeNode(data.documentElement, ProcessAuthoringConstants.XMLATTRIBUTE_NAME, processConstants.NAMESPACE_JP06).value;
						}
						this.breadcrumbs.segments = this.getLinks();
						this.breadcrumbs.refresh();
						
						this.checkForSuccessMsg();
					})
				});
			}
		},
		
		/**
		 * fetch locale specified template name
		 * @return null if there's no locale specified name
		 * @param {Element} document element
		 */
		getLocaleTemplateName: function(doc) {
			var localeTemplateName = null;
			
			var node = UIHelper.getNode(doc.childNodes, processConstants.XMLELEMENT_AVAILABLELOCALE, processConstants.NAMESPACE_JP06);
			if (node) {
				var locales = UIHelper.getNodes(node.childNodes, processConstants.XMLELEMENT_LOCALE, processConstants.NAMESPACE_JP06);
				//on IE&FF, the language with country name is like en-us, in xml, the format is like en_us
				//need to transform it to the same format   
				var browserLocale = dojo.locale.replace(/-/g, "_"); //$NON-NLS-1$
				var possibleName = null;
				
				dojo.some(locales, function(locale) {
					var lang = UIHelper.getAttributeNode(locale, processConstants.XMLELEMENT_ID, processConstants.NAMESPACE_JP06).value.toLowerCase();
					//sometimes, the browser's locale doesn't include country name, but the locale from xml has country name, 
					//and sometimes, on the contrary, need to check for both cases 
					if ((browserLocale.length > 2 && browserLocale.indexOf(lang) === 0) ||
					(lang.length > 2 && lang.indexOf(browserLocale) === 0)) {
						possibleName = UIHelper.getAttributeNode(locale, processConstants.XMLELEMENT_PROCESSNAME, processConstants.NAMESPACE_JP06).value;
					} else if (lang == browserLocale) {
						localeTemplateName = UIHelper.getAttributeNode(locale, processConstants.XMLELEMENT_PROCESSNAME, processConstants.NAMESPACE_JP06).value;
						return true;
					}
					return false;
				}, this);
			}
			
			if (localeTemplateName == null && possibleName != null) {
				localeTemplateName = possibleName;
			}
			
			return localeTemplateName;
		},
		
		postCreate: function() {
			if (this.waiting) { //show loading page before the description is loaded
				return;
			}
			this.inherited(arguments);
		},
		
		resetAllFields: function() {
			this.inherited(arguments);
		},
		
		doSave: function() {
			this.autoCommitEditor();
			
			var msg = this.validateFields();
			if (msg) {
				this.displayErrorMessage(msg);
				return;
			}
			
			this.displayWaitingMessage(this.messages.ProcessDescriptionEditor_13);
			
			if (this.isNew === true) {
				jazz.ui.underlay.show();
				var successCallback = dojo.hitch(this, function(location, ETag) {
					jazz.ui.underlay.hide();
					this.resourceURL = location;
					this.processDescriptionETag = ETag;
					this._changeToEditMode();
					// TODO: check if we really need saveResultCallback here. 
					// saveResultCallback is specific to ProcessAreaDescriptionEditor
					if (this.saveResultCallback) {
						this.saveResultCallback();
					} else {
						// set the save success message in cookie and change the URL of template description editor  
						dojo.cookie(this.msgCookieName, this.messages.ProcessDescriptionEditor_0);
				        this._unregisterHandlers();
				        document.location.href = dojo.string.substitute(PROCESS_DESCRIPTION_EDIT_ACTION, [this.resourceURL]);
					}
				});
				var errorCallback = dojo.hitch(this, function(error) {
					jazz.ui.underlay.hide();
					//TODO more details for save failure
					if (error.responseText) {
						error.message = error.responseText;
					}
					if (!error.headerMessage) {
						error.headerMessage = this.messages.ProcessDescriptionError_0;
					}
					if (this.saveResultCallback) {
						this.saveResultCallback(error);
					} else {
						this.displayErrorMessage(error);
					}
				});
				
				ProcessAuthoringClient.createProcessDescriptionForProcessTemplate({
					title: UIHelper.escapeXML(this.processDescriptionName),
					description: this.processDescription,
					language: this.workingCopy.currentLanguage ? this.workingCopy.currentLanguage : "en", //$NON-NLS-1$
					ownerUrl: this._getOwner(),
					successCallback: successCallback,
					errorCallback: errorCallback
				});
			} else {
				this.inherited(arguments);
			}
		},
		
		setupCommandBar: function() {
			this.commandBar = new jazz.ui.Toolbar();
			this.editButton = new jazz.ui.toolbar.TwoStateButton({
				href: "#", //$NON-NLS-1$
				imgSrc: this._editModeIconSrc,
				title: this.messages.AbstractPracticesEditor_12,
				onClick: dojo.hitch(this, this.toggleViewEditMode)
			});
			this.commandBar.addChild(this.editButton);
			
			this.copyUrlButton = new jazz.ui.toolbar.Button({
				imgSrc: this._copyUrlIconSrc, //$NON-NLS-1$ $
				title: this.messages.ProcessDescriptionEditor_30,
				onClick: dojo.hitch(this, function(evt) {
					dojo.stopEvent(evt);
					this.onCopyResourceUrl(this.resourceURL); //$NON-NLS-1$
				})
			});
			this.commandBar.addChild(this.copyUrlButton);
			
			this.exportProcessDescriptionButton = new jazz.ui.toolbar.Button({
				href: "#", //$NON-NLS-1$
				imgSrc: this._exportIconSrc,
				title: this.messages.ProcessDescriptionEditorHTML_4,
				onClick: dojo.hitch(this, this.exportProcessDescription)
			});
			this.commandBar.addChild(this.exportProcessDescriptionButton);
			
			
			this.discardButton = new jazz.ui.toolbar.Button({
				href: "#", //$NON-NLS-1$
				imgSrc: this._discardIconSrc,
				title: this.messages.ProcessDescriptionEditor_28,
				onClick: dojo.hitch(this, this.discardDescription)
			});
			this.commandBar.addChild(this.discardButton);
			
			this.commandBar.addChild(new jazz.ui.toolbar.Button({
				href: "#", //$NON-NLS-1$
				imgSrc: this._refreshIconSrc,
				title: this.messages.ProcessDescriptionEditorHTML_0,
				onClick: dojo.hitch(this, this.onRefresh)
			}));
			
			this.saveButton = dojo.create("button", { //$NON-NLS-1$
				"innerHTML": this.messages.ProcessDescriptionEditorHTML_1, //$NON-NLS-1$
				onClick: dojo.hitch(this, this.doSave),
				disabled: true,
				tabindex: 4
			}, this.commandBar.domNode);
			this.connect(this.saveButton, "onclick", this.doSave); //$NON-NLS-1$
			this.commandsContainer.appendChild(this.commandBar.domNode);
		},
		
		navigateToPractice: function(practiceResource) {
			window.location.href = practiceResource;
		},
		
		discardDescription: function() {
			if (this.resourceURL) {
				if (!confirm(this.messages.ProcessDescriptionEditor_31)) {
					return false; //veto the discard
				}
				jazz.ui.underlay.show();
				this.clearChangesPending();
				
				ProcessAuthoringClient.deleteProcessDescription({
					uri: this.resourceURL,
					load: dojo.hitch(this, function(data) {
						jazz.ui.underlay.hide();
						this._unregisterHandlers(); // release body click event handler
						window.location.href = this.templateListLink;
					}),
					error: dojo.hitch(this, function(error) {
						jazz.ui.underlay.hide();
						if (error.responseText) {
							error.message = error.responseText;
						}
						if (!error.headerMessage) {
							error.headerMessage = this.messages.ProcessDescriptionError_4;
						}
						this.displayErrorMessage(error);
					})
				});
			}
		},
		
		validateFields: function() {
			if (!this._getProcessDescriptionName() || dojo.trim(this._getProcessDescriptionName()).length === 0) {
				dojo.attr(this.saveButton, 'disabled', true); //$NON-NLS-1$
				return this.messages.ProcessDescriptionEditor_9;
			}
			if (this.workingCopy.processDescription && !this.workingCopy.processDescription.defaultLanguage) {
				return this.messages.ProcessDescriptionEditor_27;
			}
		}
	});
})();
