/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.FieldViewer"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ResourceLink = jazz.ui.ResourceLink;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.FieldViewer", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
	
	// summary:
	//		FieldViewer provides a widget for displaying the content of the field in the practice or practice element. 
	//      We need to pass the field title and field content, then the FieldViewer will render (call dojo.html.set()) 
	//      the content of field.
	//
	// parameters:
	//		(optional) fieldTitle: (String) the title of the field in the practice or practice element.
	//		(optional) fieldContent: (String) the content of the field in the practice or practice element.
	//
	// usage:
	//		var div = document.createElement("div"); //$NON-NLS-1$
	//		var fieldView = new FieldViewer({
	//			fieldTitle: ${title},
	//			fieldContent: ${content}
	//		}, div);

		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "ui/internal/viewlets/templates/FieldViewer.html"), //$NON-NLS-1$ //$NON-NLS-2$
		fieldTitle: null,
		fieldContent: null,
		
		constructor: function(args) {
			this.resourceLinks = [];
		},
		
		postCreate: function() {
			this.inherited(arguments);
			if (this.fieldTitle) {
				dojo.toggleClass(this.titleArea, "hide", false); //$NON-NLS-1$
				dojo.html.set(this.titleArea, this.fieldTitle);
			} else {
				dojo.toggleClass(this.titleArea, "hide", true); //$NON-NLS-1$
			}
			
			if (this.fieldContent) {
				dojo.toggleClass(this.contentArea, "hide", false); //$NON-NLS-1$
				dojo.html.set(this.contentArea, this.fieldContent);
				this._doCreateResouceLinks();
			} else {
				dojo.toggleClass(this.contentArea, "hide", true); //$NON-NLS-1$
			}
		},
		
		_doCreateResouceLinks: function() {
			var linkPrefix = ProcessAuthoringClient.getJtsProcessAuthoringRoot();
			if (linkPrefix) {
				var query = dojo.string.substitute("a[href*='${0}']", [linkPrefix]);//$NON-NLS-1$
				var links = dojo.query(query, this.contentArea);
				links.forEach(function(link) {
					var url = link.href;
					if (ProcessAuthoringClient.isPracticeURL(url) || UIHelper.isPracticeElementURL(url) || ProcessAuthoringClient.isProcessDescriptionURL(url)) {
						var rl = new ResourceLink({
							uri: url,
							presentationUri: url,
							'lazyFetch': true //$NON-NLS-1$
						}, link);
						this.resourceLinks.push(rl);
					}
				}, this);
			}
		},
		
		uninitialize: function() {
			dojo.forEach(this.resourceLinks, function(resourceLink) {
				resourceLink.destroyRecursive();
			}, this);
			this.inherited(arguments);
		}
	});
})();
