/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.ViewletSelectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.dialogs.SelectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.viewlets", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	var messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.viewlets", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var ResourceLink = jazz.ui.ResourceLink;
	var _WIDTH_IN_EM = 60;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.ViewletSelectionDialog", //$NON-NLS-1$
 com.ibm.team.process.authoring.web.common.ui.internal.dialogs.SelectionDialog, {
    // summary:
	//		ViewletSelectionDialog provides a dialog for selecting practice, practice element, section or field 
	//      in the process description for viewlet. There is a practice/practice element tree on the left and a section/field
	//      tree on the right. We need to pass the process description, then the ViewletSelectionDialog will build practice tree
	//      and section tree for selection.
	//
	// parameters:
	//		scope: (String) the url of process area for constructing the resource link which will display 
	//                       in this dialog.
	//		scopeItemName: (String) the name of process area that will display in this dialog.
	//		(optional) selectedUrl: (String) the initial selection url of the selected practice or practice element.
	//		(optional) selectedFragmentId: (String) the initial selection url of the selected section or field.
	//
	// methods:
	//		onOK: the function is called after clicking OK, one can override this function to add his own behavior.
	//      getTopText: provide top text in the dialog.
	//      onProcessDescriptionLoaded: update dialog after process description is loaded.
	//
	// usage:
	//		 this.selectionDialog = new ViewletSelectionDialog({
	//			scope: "https://localhost:9443/jazz/process/project-areas/_wSiF4s_uEeGeKcRB1ZnThg"
    //       	scopeItemName: "MyProject",
    //        	selectedUrl: "https://127.0.0.1:9444/jazz/process-authoring/practices/_vxZnZ8_xEeG4MJPllocT3Q", //url of practice or practice element
    //        	selectedFragmentId: ${selectedFragmentId}, //section or field Id
    //        	onOk: dojo.hitch(this, this.dialogOk, this._callback)
    //        });
		primaryTitle: messages.SelectionDialogPrimaryTitle,
		width: _WIDTH_IN_EM + "em", //$NON-NLS-1$
		showSectionTree: true,
		
		constructor: function(args) {
			this.scope = args.scope;
			this.scopeItemName = args.scopeItemName;
			this._resourceLinks = [];
		},
		
		getTopText: function(){
			return messages.SelectionDialogTopText_0;
		},
		
		onProcessDescriptionLoaded: function() {
			dojo.empty(this.topText);
			var topMessage = messages.SelectionDialogTopText;
			// add first part of the text
			var preText = topMessage.substring(0, dojo.indexOf(topMessage, "$")); //$NON-NLS-1$
			this.topText.appendChild(document.createTextNode(preText));
			
			// create resource link for process description
			var doc = this.processDescriptionXML.documentElement ? this.processDescriptionXML.documentElement : this.processDescriptionXML;
			var descriptionUrl = UIHelper.getAttributeNode(doc, ProcessAuthoringConstants.XMLATTRIBUTE_ABOUT, ProcessAuthoringConstants.NAMESPACE_RDF).value;
			var descriptionTitle = UIHelper.getProcessDescriptionTitle(this.processDescriptionXML);
			if (!descriptionTitle) {
				descriptionTitle = messages.NoValue;
			}
			var descLink = document.createElement("a"); //$NON-NLS-1$
			descLink.href = descriptionUrl;
			descLink.appendChild(document.createTextNode(descriptionTitle));
			var rl = new ResourceLink({
				'presentationUri': descriptionUrl, //$NON-NLS-1$
				'lazyFetch': true //$NON-NLS-1$
			}, descLink);
			this._resourceLinks.push(rl);
			
			// add resource link for process description
			var div = document.createElement("span"); //$NON-NLS-1$
			div.appendChild(descLink);
			this.topText.appendChild(div);
			
			// add middle part of the text
			var midText = topMessage.substring(dojo.indexOf(topMessage, "$") + 4, dojo.lastIndexOf(topMessage, "$")); //$NON-NLS-1$ //$NON-NLS-2$
			this.topText.appendChild(document.createTextNode(midText));
			
			// create resource link for process area
			if (!this.scopeItemName) {
				this.scopeItemName = messages.NoValue;
			}
			var processLink = document.createElement("a"); //$NON-NLS-1$
			processLink.href = this.scope;
			processLink.appendChild(document.createTextNode(this.scopeItemName));
			rl = new ResourceLink({
				'presentationUri': this.scope, //$NON-NLS-1$
				'lazyFetch': true //$NON-NLS-1$
			}, processLink);
			this._resourceLinks.push(rl);
			
			// add resource link for process area
			var div = document.createElement("span"); //$NON-NLS-1$
			div.appendChild(processLink);
			this.topText.appendChild(div);
			
			// add the last part of the text
			var postText = topMessage.substring(dojo.lastIndexOf(topMessage, "$") + 4); //$NON-NLS-1$
			this.topText.appendChild(document.createTextNode(postText));
		},
		
		_destroyResourceLinks: function(){
			for (var i = 0; i < this._resourceLinks.length; i++) {
				this._resourceLinks[i].destroyRecursive();
			}
			this._resourceLinks = [];
		},
		
		uninitialize: function(){
			this._destroyResourceLinks();
			this.inherited(arguments);
		}
	});
})();