/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.wizard.AbstractDashboardStep");

dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.wizard.WizardStep");

var WizardStep = jazz.ui.wizard.WizardStep;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;

/*
 * Common functionality for all dashboard wizard steps. Subclass steps must:
 * 
 * 1. Define this.messageBox: A jazz.ui.MessageBox.
 * 2. Call the super methods for button enablement tests.
 * 3. Add the com-ibm-team-dashboard-web-ui-internal-wizard-AbstractDashboardStep class to their root element.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.wizard.AbstractDashboardStep",
		[WizardStep], {
	
	_isInitialized: false,
	_isWorking: false,
	
	constructor: function(args) {
		this.data = args.wizard.data;
	},
	
	canFinish: function() {
		return !this._isWorking;
	},
	
	focusStep: function() {
		if (!this._isInitialized) {
			this.init();
			this._isInitialized = true;
		}
	},
	
	init: function() {
		// subclasses can override to perform one-time initialization, similar to postCreate
		// except only happens when the page is first viewed.
	},
	
	/*
	 * Sets a property on the wizard data object.
	 */
	setData: function(/* string */ property, /* any */ value) {
		this.data[property] = value;
		this.triggerButtonStateUpdate();
	},
	
	showMessage: function(/* object */ args) {
		this.messageBox.showMessage(args);
	},
	
	showError: function(/* string */ summary, /* Error */ e) {
		this.messageBox.showMessage({
			type: STATUS_TYPES.ERROR,
			summary: summary,
			errorObject: e,
			closable: true
		});
	},
	
	taskDone: function() {
		this._isWorking = false;
		this.messageBox.hideMessage(false);
		this.triggerButtonStateUpdate();
	},
	
	taskFailed: function(/* string */ summary, /* Error */ e) {
		this.taskDone();
		this.showError(summary, e);
	},
	
	taskStart: function(/* string */ summary) {
		this._isWorking = true;
		this.messageBox.showMessage({summary: summary});
		this.triggerButtonStateUpdate();
	}
});

})();
