/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDataWarehouse");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.client.internal.DBProviderClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");

dojo.require("dojo.i18n");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("com.ibm.team.repository.web.client.internal.ReportsClient");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDataWarehouseMessages");

(function() {

var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;
var WizardConfigurationStep = com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep;
var WizardForm = com.ibm.team.repository.web.ui.internal.wizard.WizardForm;
var licensing = com.ibm.team.repository.web.client.internal.licensing;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var DBProviderClient = com.ibm.team.repository.web.client.internal.DBProviderClient;
var ReportsClient = com.ibm.team.repository.web.client.internal.ReportsClient;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var UserRegistryClient = com.ibm.team.repository.web.client.internal.userRegistry;
var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;

var PERSISTENT_MAPPING_SERVICE = "com.ibm.team.repository.service.internal.schemagen.PersistentMappingService";
var DB_SERVICE_NAME = "com.ibm.team.datawarehouse.service.internal.RemoteDatabaseService";
var INTERNAL_DATAWAREHOUSE_SERVICE_NAME = "com.ibm.team.datawarehouse.service.internal.InternalDataWarehouseService";
var REGISTRY_REST_SERVICE_INTERFACE_NAME = "com.ibm.team.repository.service.internal.IExternalUserRegistryRestService";
var DB_VENDOR_NAME = "db.vendor";
var DB_TYPE_NAME = "db.type";
var JDBC_TYPE_VALUE = "JDBC";
var VENDOR_LICENSE_CHECK_PREFIX = "com.ibm.team.repository.support.";
var DB_BASE_FOLDER = "db.base.folder";
var REPORT_USER = "report.user";
var DB_AUTOMATIC_SETUP = "db.automatic.setup";
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
var PASSWORD = "********";
var JAZZ_ROOT = net.jazz.ajax.contextRoot();
	
var DISPLAY_PROPS = [];
DISPLAY_PROPS.J2EE = {
	ALL: [
		"db.j2ee.dataSource"
	]
};
DISPLAY_PROPS.JDBC = {
	ALL: [
		"db.jdbc.password",
		"db.jdbc.location"
	],
	derby: [
	    "db.net.port"
	],
	db2: [
	    "db.base.folder",
	    "report.user"
	],
	oracle: [
 	    "db.base.folder",
	    "report.user",
	    "db.schemaPrefix"
	],
	sqlserver: [
	   	"db.base.folder",
	 	"report.user"
	],
	db2i: [
	    "report.user",
	    "db.schemaPrefix"
	],
	db2z: [
        "report.user",
        "db.schemaPrefix",
		"db.db2.dsn.dbname"
	]
};

var VENDOR_MAP = {
	'db2': "DB2",
	'derby': "Derby",
	'derby_net': "Derby Client",
	'oracle': "Oracle",
	'sqlserver': "SQL Server",
	'db2z': "DB2 for z/OS",
	'db2i': "DB2 for i"
};

//Example location strings
var LOCATION_STRINGS = [
	{vendorName: "db2", locString: "//localhost:50000/JAZZ:user=db2inst1;password={password};"},  
	{vendorName: "oracle", locString: "thin:jazzDBuser/{password}@//localhost:1521/ORCL"}, 
	{vendorName: "sqlserver", locString: "//localhost:1433;databaseName=jazz;user=jazzDBuser;password={password}"},
	{vendorName: "db2i", locString: "//hostname;hold statements=true;user=jazzDBuser;password={password};"}, 
	{vendorName: "db2z", locString: "//ipAddress:ipPort/location:user=jazzDBuser;password={password};"},
	{vendorName: "derby_net", locString: "//localhost:1527/databaseName"},
	{vendorName: "derby", locString: "conf/jts/derby/warehouseDB"}
];

var dwMessages;

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDataWarehouse",  WizardConfigurationStep, {
	_state: null,
	_oldState: null,
	_oldCreateTablesStyle: null,
	_registryInfo: null,
	_testConnectionRequired: false,
	_dataCollectionLicenses: [],
	_savedUser: {
					oldUserId : "",
					oldPassword : PASSWORD,
					oldEmail : "",
					userId : "",
					password : "",
					email : "",
					_doUpdate: false
				},
	_canEditUser: true,
	_currentlyTesting: false,
	
	constructor: function(args) {
		this.messages = dwMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDataWarehouseMessages");
		this.implementationClassName = "com.ibm.team.datawarehouse.service.internal.RemoteDatabaseService";
		this.pendingNext = null;
		this.stepTitle = dwMessages.wizardPageHeading;
		this.isJTS = args.isJTS;
		this.description = this.isJTS ? dwMessages.wizardPageDescriptionJTS : dwMessages.wizardPageDescriptionFronting;
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		
		dojo.connect(this, "_testConnection", this._setCurrentlyTesting);
		dojo.connect(this, "_testConnectionError", this._unsetCurrentlyTesting);
		dojo.connect(this, "focusStep", this.fetchUserRegistry);
	},
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.ConfigureDataWarehouse.superclass.postCreate.apply(this);
		this.postEtlUserUpdatesHandle = dojo.connect(this, "onNext", this._postEtlUserUpdates);
		this.createUserHandle = dojo.connect(this, "onNext", this._createUser);
		dojo.connect(this, "onNext", this._setJobEnablement);
		ReportsClient.getIsDataWarehouseJobsEnabled(new ServiceResponseHandler(this, "_isEnabled", "_noop"));
		this.connect(this.dbConfigUI.btnNoWarehouse, "onclick", "onCheckboxClicked");		
	},
	
	fetchUserRegistry: function() {
		if (this.isJTS == true) {
			var srh = new ServiceResponseHandler(this, "setUserRegistryType", "_noop");
			UserRegistryClient.getUserRegistryType(srh);
		}
	},
	
	createInnerStepUI: function() {
		this.dbConfigUI = new DWConfigInnerSteps();
		this.serviceContainer = this.dbConfigUI._hiddenTableBody;
		this.configArea.appendChild(this.dbConfigUI.domNode);
		this.connect(this.dbConfigUI.createDBLink, "onclick", "_createDatabases");
		this._createUserSection();
		
		//We'll need this later
		this.STATUS.DW_NEEDS_TABLES = {
				test: false,
				revert: false,
				next: false,
				edit: true,
				code: "INFO",
				resolvedMessage: dwMessages.DBneedsTables
		};
		
		this.STATUS.CREATE_USER = {
				test: false,
				revert: false,
				next: false,
				edit: true,
				code: "INFO",
				resolvedMessage: dwMessages.completEtlUserForm
		};
		
		this.STATUS.USER_INVALID = {
				test: false,
				revert: false,
				next: false,
				edit: true,
				code: "ERROR",
				resolvedMessage: dwMessages.etlUserFormInvalid
		};
		
		this.STATUS.APP = {
				test: false,
				revert: false,
				next: true,
				edit: true,
				code: "OK",
				resolvedMessage: dwMessages.applicationOk
		};
		
		this.STATUS.APP_TEST = {
				test: true,
				revert: false,
				next: false,
				edit: true,
				code: "INFO",
				resolvedMessage: dwMessages.applicationTest
		};
		
		this.STATUS.NO_WAREHOUSE = {
				test: false,
				revert: false,
				next: true,
				edit: false,
				code: "WARNING",
				resolvedMessage: dwMessages.noWarehouseMessage
		};
		
		if (!this.isJTS) {
			this._initialLoadedState = this.STATUS.APP;
			this._state = this.STATUS.APP;
			this._setState(this.STATUS.APP);
			this._enablePasswordEdit();
		}
	},

	_isEnabled: function (retVal) {
		this.dbConfigUI.btnNoWarehouse.checked = (retVal == "true") ? false : true;
	},
	
	onCheckboxClicked: function() {
		if (this.dbConfigUI.btnNoWarehouse.checked === true) {
			this._oldState = this._state;
			this._state = this.STATUS.NO_WAREHOUSE;
			this._setState(this.STATUS.NO_WAREHOUSE);
		
			// Disconnect the Next handlers
			dojo.disconnect(this.postEtlUserUpdatesHandle);
			this.postEtlUserUpdatesHandle = null;
			dojo.disconnect(this.createUserHandle);
			this.createUserHandle = null;
			
			// Hide the "Create Tables" section, if it's showing
			this._oldCreateTablesStyle = this.dbConfigUI._createTablesSection.style.display;
			this.dbConfigUI._createTablesSection.style.display = "none";	

			// Disable the ETL user fields
			this._userSection.setFieldEnabled("userId", false);
			this._userSection.setFieldEnabled("password", false);
			this._userSection.setFieldEnabled("repeatPassword", false);
			this._userSection.setFieldEnabled("emailAddress", false);
		} else {
			if (this._oldState == null) this._oldState = this.STATUS.CONNECTED_OK;
			this._state = this._oldState;
			this._setState(this._oldState);
			
			// Reconnect the Next handlers if needed
			if (this.postEtlUserUpdatesHandle == null) {
				this.postEtlUserUpdatesHandle = dojo.connect(this, "onNext", this._postEtlUserUpdates);
			}
			if (this.createUserHandle == null) {
				this.createUserHandle = dojo.connect(this, "onNext", this._createUser);
			}

			this._userSection.setFieldEnabled("userId", true);
			this._userSection.setFieldEnabled("password", true);
			this._userSection.setFieldEnabled("repeatPassword", true);
			this._userSection.setFieldEnabled("emailAddress", true);

			this._displayRelevantSections();
			if (this._oldCreateTablesStyle != null) {
				this.dbConfigUI._createTablesSection.style.display = this._oldCreateTablesStyle;
			}
		}
	},
	
	_setJobEnablement: function() {
		var val = (this.dbConfigUI.btnNoWarehouse.checked === true) ? "false" : "true";
		
		var argArray = new Array();
		argArray
			.push(this
				._getPropertyArguments("datawarehouse.jobs.enabled",
						val,
						INTERNAL_DATAWAREHOUSE_SERVICE_NAME));
		var args = {
			'updates' : dojo.toJson(argArray)
		};
		var srh = new ServiceResponseHandler(this, "_noop", "_noop");
		ServerStatusClient.postConfigurationUpdates(srh, args);
	},

	requestLicenseRestrictions: function() {
		this.licenseRequestPending = true;
		this.licenseCheckResponse = 0;
		this.licenseCheckRequests = 0;
		var self = this;
		dojo.forEach(this._supportedVendors, function(vendor) { 
			//Don't do the checks anymore, currently all vendors are always supported
			vendor.licenseAllows = "OK";
			
			/* Don't delete this code in case DB/Licensing requirements are enabled again. (WI 121764)
			
			self.licenseCheckRequests++;
			var checkingLicense = {
				vendor: vendor,
				
				handleLicenseCheck: function(response) {
					vendor.licenseAllows = response.code;
					self.incrementLicenseCheckCount();
				},
				handleAcceptError: function(err) {
					self.saveError(err);
				}
			}
			var srh = new ServiceResponseHandler(checkingLicense, "handleLicenseCheck", "handleAcceptError");
			licensing.checkLicense(srh, {operationId: VENDOR_LICENSE_CHECK_PREFIX + vendor.licenseCheck} );
			
			*/
			
		});	
		this.incrementLicenseCheckCount(); //ADDED
	},
	
	incrementLicenseCheckCount: function() {
		//this.licenseCheckResponse++;
		if (this.licenseCheckResponse === this.licenseCheckRequests) {
			// All license checks are back. Load the vendors if the UI already loaded 
			this.licenseRequestPending = false;
			if (this.displayloaded && !this.vendorsDisplayed) {
				this._displayVendorsAllowedByLicense();
			}
		}	
	},
		
	_initializePropertiesAfterLoaded: function() {
		if (this._currentlyTesting == true) {
			setTimeout(dojo.hitch(this, this._initializePropertiesAfterLoaded), 100);
			return;
		}
		this._initializeHiddenProperty(DB_VENDOR_NAME);
		this._initializeHiddenProperty(DB_TYPE_NAME);
		
		var automaticSetup = this._config.getEditedValueForProperty(DB_AUTOMATIC_SETUP);
		if (!automaticSetup) {
			for (var key in DISPLAY_PROPS.JDBC) {
				DISPLAY_PROPS.JDBC[key].splice(DB_BASE_FOLDER, 1);
			}
		}
		
		this._initializePropertyDisplay(this.dbConfigUI._jdbcEditPropertiesBody, DISPLAY_PROPS.JDBC);
		this._initializePropertyDisplay(this.dbConfigUI._j2eeEditPropertiesBody, DISPLAY_PROPS.J2EE);
				
		this._displayRelevantSections();
		
		this.connect(this.dbConfigUI._selectVendor, "onchange", "_handleVendorSelectChange");
		this.connect(this.dbConfigUI._selectType, "onchange", "_handleTypeSelectChange");
		
		this.inherited(arguments);
		if (!this.licenseRequestPending) {
			this._displayVendorsAllowedByLicense();
		}
		this.displayloaded = true;
		if (!this.isJTS) {
			var srh = new ServiceResponseHandler(this, "_handleGotJTSURI", "_handleAcceptError");
			ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
		} else {
		    this._getEtlUser(JAZZ_ROOT, this._handleGetJtsEtlUser);
		}
	},	
	
	_handleGotJTSURI: function(url) {
		this._jtsUrl = url;
		// If there is no configured JTS, don't do this.
		if (url == null || url == "" || url == undefined) {
			return;
		}
		var lastChar = url.charAt(url.length-1);
		if (lastChar != '/') url += "/";
		var self = this;
		var xhrArgs = {
	        url: url + "service/com.ibm.team.repository.service.internal.IServerConfigurationRestService/serviceConfiguration?implementationClassName=" + DB_SERVICE_NAME,
	        content: { },
			handleAs: "text",
			headers: this.headers,
			handle: dojo.hitch(this, function(response, ioArgs) {
				if (response instanceof Error) {
					self.saveError(err);
				} else {
					self._handleGotJTSResponse(response);
				}
	        })
	    };
	    jazz.client.xhrGet(xhrArgs);
	    this._getEtlUser(url, this._handleGetJtsEtlUser);
	},
	
	// We are a fronting app and just received the JTS values for the DW settings.
	// Where applicable, substitute the JTS values - but only if the fronting app
	// values are still set to their defaults (i.e. no overrides).
	_handleGotJTSResponse: function(response) {
		if (this._config && this._config.serviceCallsPending > 0) {
			//make sure the service properties have fetched their own local values otherwise they will overwrite us
			setTimeout(dojo.hitch(this, this._handleGotJTSResponse, response), 100);
			return;
		} else if (this._config && this._config.serviceCallsPending == 0) {
			/*
			 * We use setTimeout here to yield, and allow the service property components to catch up before continuing to update the property values.
			 * 
			 * Service properties fetch the local app values while we are simultaneously fetching the JTS values and trying to override them
			 */
			this._config.serviceCallsPending--;
			setTimeout(dojo.hitch(this, this._handleGotJTSResponse, response), 100);
			return;
		}
		if (this._currentlyTesting == true) {
			setTimeout(dojo.hitch(this, this._handleGotJTSResponse, response), 100);
			return;
		}
		var dbVendorProperties = this._getProperties(response, "db.vendor");
		var dbVendor = this._getElementValue(dbVendorProperties, "overrideValue");
		
		var dbTypeProperties = this._getProperties(response, "db.type");
		var dbType = this._getElementValue(dbTypeProperties, "overrideValue");
		
		var jdbcLocationProperties = this._getProperties(response, "db.jdbc.location");
		var jdbcLocation = this._getElementValue(jdbcLocationProperties, "overrideValue");
		
		var j2eeDataSourceProperties = this._getProperties(response, "db.j2ee.dataSource");
		var j2eeDataSource = this._getElementValue(j2eeDataSourceProperties, "overrideValue");
				
		var dbBaseFolderProperties = this._getProperties(response, "db.base.folder");
		var dbBaseFolder = this._getElementValue(dbBaseFolderProperties, "overrideValue");
		
		var reportUserProperties = this._getProperties(response, "report.user");
		var reportUser = this._getElementValue(reportUserProperties, "overrideValue");
		
		var dbNetPortProperties = this._getProperties(response, "db.net.port");
		var dbNetPort = this._getElementValue(dbNetPortProperties, "overrideValue");
		if (dbNetPort != null) {
			dbNetPort = this._trimString(dbNetPort);
			if (dbNetPort.length == 0) {
				dbNetPort = null;
			}
		}
		
		var dbNameProperties = this._getProperties(response, "db.db2.dsn.dbname");
		var dbName = this._getElementValue(dbNameProperties, "overrideValue");
		
		var isDerby = ((dbVendor == null) || (dbVendor.toLowerCase() == "derby"));
		var isDerbyClient = isDerby && (dbNetPort != null);
		
		if (isDerbyClient) {
			this.dbConfigUI._selectVendor.value = "derby_net";
			this._handleChange("derby_net", "db.vendor");
		} else if ((dbVendor != null) && (!isDerby)) {
			this.dbConfigUI._selectVendor.value = dbVendor;
			this._handleChange(dbVendor, "db.vendor");
		}
		
		if (dbType != null) {
			this.dbConfigUI._selectType.value = dbType;
			this._handleChange(dbType, "db.type");
		}
		
		if (jdbcLocation != null) {
			if (isDerbyClient) {
				jdbcLocation = "//localhost:" + dbNetPort + '/' + jdbcLocation;
			}
			this._handleChange(jdbcLocation, "db.jdbc.location");
		}
		
		if (j2eeDataSource != null) {
			this._handleChange(j2eeDataSource, "db.j2ee.dataSource");
		}
		
		if (dbBaseFolder != null) {
			this._handleChange(dbBaseFolder, "db.base.folder");
		}
		
		if (reportUser != null) {
			this._handleChange(reportUser, "report.user");
		}
		
		if (dbName != null) {
			this._handleChange(dbName, "db.db2.dsn.dbname");
		}
		this._enablePasswordEdit();
		this._hideAndTest();
	},
	
	_hideAndTest: function() {
		if (this._currentlyTesting == true) {
			setTimeout(dojo.hitch(this, this._hideAndTest), 100);
		} else {
			this.header.hideMessage();
			this._testConnection();
		}
	},
	
	_trimString: function(stringToTrim) {
		return stringToTrim.replace(/^\s+|\s+$/g,"");
	},
	
	_getProperties: function(xmlText, propertyName) {
		var indexOfProperty = xmlText.indexOf("<name>" + propertyName + "</name>");
		if (indexOfProperty == -1) return null;
		var propertiesStart = xmlText.substring(0, indexOfProperty).lastIndexOf("<properties>");
		var propertiesEnd = indexOfProperty + xmlText.substring(indexOfProperty).indexOf("</properties>");
		return xmlText.substring(propertiesStart, propertiesEnd);
	},
	
	_getElementValue: function(xmlText, elementName) {
		var elementIndex = xmlText.indexOf("<" + elementName + ">");
		if (elementIndex == -1) return null;
		xmlText = xmlText.substring(elementIndex + (elementName.length + 2));
		elementIndex = xmlText.indexOf("</" + elementName+ ">");
		xmlText = xmlText.substring(0, elementIndex);
		return xmlText;
	},	
	
	_displayVendorsAllowedByLicense: function() {
		this.vendorsDisplayed = true;
		var self = this;
		dojo.forEach(this._supportedVendors, function(vendor) { 
	  		if (vendor.licenseAllows && vendor.licenseAllows === "OK") {
				var newOption = new Option(vendor.displayValue, vendor.propertyValue);
				self.dbConfigUI._selectVendor.options[self.dbConfigUI._selectVendor.options.length] = newOption;
			} else {
				self._displayLicenseLimitedMessage();
			}
		});	
		this._synchronizeSelectValue();	
	},

	_displayLicenseLimitedMessage: function() {
		dojo.toggleClass(this.dbConfigUI._licenseLimitedMessage, "noDisplay", false);
	}, 
	
	_initializeHiddenProperty: function(propertyName) {
		var targetProperty = this._config.getServerPropertyElement(propertyName);
		this.connect(targetProperty, "_markInvalid", function(msg) {
			this._handleInvalidInput(msg);
			this._showHiddenError(msg);
		});
	},
	
	_showHiddenError: function(msg) {
		if (msg.code === "ERROR") {
			this._state = this.STATUS.PENDING_ERR;
			this._setState(this.STATUS.PENDING_ERR, [msg]);
		} else {
			this._state = this.STATUS.PENDING_WARN;
			this._setState(this.STATUS.PENDING_WARN, [msg]);
		}
	},
	
	_initializePropertyDisplay: function(attachPoint, displayProps) {
		
		var targetPropertyServiceRow = new ServerPropertyServiceRow({
			_enableToggle: false,
			rowClass: ["indentNone editProperties"],
			_descriptionMode: "table"
		});
		attachPoint.appendChild(targetPropertyServiceRow.domNode);
		attachPoint._elements = [];
		var longExists = false;
		var shortExists = false;
		for (var type in displayProps) {
			for (var i=0; i < displayProps[type].length; i++) {
				var propertyElement = this._config.getServerPropertyElement(displayProps[type][i]);
				if (!propertyElement) continue;
				var vendors = propertyElement._vendors;
				if (!vendors) {
					vendors = {};
					propertyElement._vendors = vendors;
				}
				vendors[type] = true;
				attachPoint._elements.push(propertyElement);
				if (propertyElement.elementInfo.propertyType === "STRING") {
					longExists = true;
					var table = targetPropertyServiceRow._longPropertiesTableBody;
				} else {
					shortExists = true;
					var table = targetPropertyServiceRow._shortPropertiesTableBody;
				}
				table.appendChild(propertyElement.domNode);
			}
		}
		if (!longExists) {
			targetPropertyServiceRow._longPropertiesTableHeader.style.display = "none";
		}
		if (!shortExists) {
			targetPropertyServiceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
		
	_handleVendorSelectChange: function(event) {
		this._handleSelectChange(event, DB_VENDOR_NAME);
	},
	
	_handleTypeSelectChange: function(event) {
		this._handleSelectChange(event, DB_TYPE_NAME);
	}, 
	
	_handleSelectChange: function(event, propertyName) {
		var newSelectValue = event.target.value;
		this._handleChange(newSelectValue, propertyName);
	},
	
	_handleChange: function(newValue, propertyName) {
		this._testConnectionRequired = true;
		var targetProperty = this._config.getServerPropertyElement(propertyName);
		
		targetProperty._editValueNode.value = newValue;
		
		if (targetProperty._savedValue != newValue) {
			targetProperty._markChanged();
		} else {
			targetProperty._markUnchanged();
		}
		targetProperty._checkValidity();
		this._displayRelevantSections();
		this.triggerButtonStateUpdate();
	},
	
	_revert: function() {
		this.inherited(arguments);
		this._synchronizeSelectValue();
		this._displayRelevantSections();
		this._userSection.setDetails(this._savedUser.userId, this._savedUser.email, PASSWORD);
	},
			
	// UPDATE VIEW METHODS	
	_displayRelevantSections: function() {
		// KMB TODO need to clear out values on hide
		var vendor = this._config.getEditedValueForProperty(DB_VENDOR_NAME);
		if (vendor) {
			vendor = vendor.toLowerCase();
		}
		if (this._config.getEditedValueForProperty(DB_TYPE_NAME) === JDBC_TYPE_VALUE) {
			this.dbConfigUI._j2eePropertiesSection.style.display = "none";
			this.dbConfigUI._jdbcPropertiesSection.style.display = "block";
			
			for (var i=0; i < this.dbConfigUI._jdbcEditPropertiesBody._elements.length; i++) {
				var elem = this.dbConfigUI._jdbcEditPropertiesBody._elements[i];
				var vendors = elem._vendors;
				if (vendors && (vendors["ALL"] || vendors[vendor])) {
					elem.domNode.style.display = "";
				} else {
					elem.domNode.style.display = "none";
				}
			}
		} else {
			this.dbConfigUI._j2eePropertiesSection.style.display = "block";
			this.dbConfigUI._jdbcPropertiesSection.style.display = "none";
		}
		
		this._updateLocationStringExample();
		
		if (vendor == "derby") {
			var messageString = dojo.string.substitute(this.messages.jdbcLocExample, [ '1527' ]);
			this._config.setServerPropertyExample("db.net.port", messageString);
		}
		
		this.dbConfigUI._createTablesSection.style.display = "none";	
		this._enablePasswordEdit();
	}, 
	
	_allowPropertyEdit: function(allowEdits) {
		this.inherited(arguments);
		this.dbConfigUI._selectVendor.disabled = !allowEdits;
		this.dbConfigUI._selectType.disabled = !allowEdits;
	},
	
	_synchronizeSelectValue: function() {
		var vendorProp = this._config.getServerPropertyElement(DB_VENDOR_NAME);
		this.dbConfigUI._selectVendor.value = vendorProp._editValueNode.value.toLowerCase();
		var typeProp = this._config.getServerPropertyElement(DB_TYPE_NAME);
		this.dbConfigUI._selectType.value = typeProp._editValueNode.value;
		this._displayRelevantSections();
	},
	
	_testConnectionCallback: function(msg) {
		this._testConnectionRequired = false;
		this._currentlyTesting = false;
		var setRemote = false;
		var connected = null;
		this.dbConfigUI._createTablesSection.style.display = "none";
		if (msg != null) {
			connected = this.STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = this.STATUS.CONNECTED_ERR;
					//Check to see if we need to create Tables (i.e. error code 3)
					if (msg[i].id == 3) {
						connected = this.STATUS.DW_NEEDS_TABLES;
						msg = null; //Don't include 'show details'
						this.dbConfigUI.createDBLink.disabled = false;
						this.dbConfigUI._createTablesSection.style.display = "block";
						break;
					}
				}
			}
		} else {
			setRemote = true;
			var userStatus = this._userSection.validate();
			msg = null;
			if (userStatus.isComplete && userStatus.isValid) {
				connected = this.isJTS ? this.STATUS.CONNECTED_OK : this.STATUS.APP;
			} else if (!userStatus.isValid) {
				connected = this.STATUS.USER_INVALID;
			} else {
				connected = this.STATUS.CREATE_USER;
			}
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		if (this.dbConfigUI.btnNoWarehouse.checked === true) {
			this._state = this.STATUS.NO_WAREHOUSE;
			this._setState(this.STATUS.NO_WAREHOUSE);
		} else {
			this._state = connected;
			this._setState(connected, msg);
		}
		// Connection was successfully tested. Set the server's DW flag to remote.
		if (setRemote == true) {
			this._setRemote("_noop", "_handleAcceptError");
		}
		this._enablePasswordEdit();
	},
	
	_setRemote: function(success, failure) {
		var argArray = new Array();
		argArray
			.push(this
				._getPropertyArguments(
						"datawarehouse.provider",
						"Remote",
						INTERNAL_DATAWAREHOUSE_SERVICE_NAME));
		var args = {
			'updates' : dojo.toJson(argArray)
		};
		var srh = new ServiceResponseHandler(this, success, failure);
		ServerStatusClient.postConfigurationUpdates(srh, args);
	},
		
	_noop: function() {
	},
	
	_createDatabases: function() {
		this.dbConfigUI.createDBLink.disabled = true;

		this.STATUS.DW_SUCCESS = {
				test: false,
				revert: false,
				next: true,
				edit: true,
				code: "OK",
				resolvedMessage: dwMessages.creationSuccess
		};
		this.STATUS.DW_FAIL = {
				test: true,
				revert: true,
				next: false,
				edit: true,
				code: "ERROR",
				resolvedMessage: dwMessages.creationFail
		};
		this.STATUS.DW_PROGRESS = {
				test: false,
				revert: false,
				next: false,
				edit: false,
				code: "INFO",
				resolvedMessage: dwMessages.creationProgress
		};
		
		//First we need to save the properties before we can call createTables
		this.invokeSave(this, "saveOK", "saveError");
	},
	
	saveOK: function() {
		this._setRemote("_handleSetRemote", "_handleAcceptError");
	},
	
	_handleSetRemote: function() {
		this.header.showMessage({
			type: "OK",
			summary: this.messages.settingsSaved
		});
		var srh = new ServiceResponseHandler(this, "_handleCreateTables", "_handleAcceptError");
		ReportsClient.postInitializeDataWarehouse(srh);
	},
	
	_handleCreateTables: function(response) {	
		if (response == "true") { //Response passed back as string
			this._state = this.STATUS.DW_PROGRESS;
			//Database tables are not created.
			this._setState(this.STATUS.DW_PROGRESS);
			//Begin to poll for progress
			this.pollRequests = 0;
			this.pollID = setInterval(dojo.hitch(this,"checkProgress"),500);
		} else {
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.tablesExist
			});
		}
	},
	
	_handleAcceptError: function(err) {
		this.saveError(err);
	},
	
	_getPropertyArguments : function(name, value,
			implementationName) {
		var args = {
			'name' : name,
			'value' : value,
			'serviceImplementationName' : implementationName
		}
		return args;
	},
	
	checkProgress: function() {
		
		//Don't want to keep sending out requests unless they are coming back...
		if (this.pollRequests < 1) {
			var self = this;
			var pollWorker = {
				handleSuccess: function(response) {
					self.pollRequests--;
					if (response.id == 6) {
						//The database creation has completed sucessfully
						clearInterval(self.pollID);
						self.dbConfigUI.createDBLink.disabled = true;
						self._state = self.STATUS.DW_SUCCESS;
						self._setState(self.STATUS.DW_SUCCESS);
						
					} else if (response.id == 2) {
						//The database creation failed with an exception
						clearInterval(self.pollID);
						self.dbConfigUI.createDBLink.disabled = false;
						
						//Convert to the expected errorObj format
						var errorObj = {
							message: response.message,
							exception: response.stacktrace
						};
						self._state = self.STATUS.DW_FAIL;
						self._setState(self.STATUS.DW_FAIL,errorObj,true);
					} 
				},
				handleError: function(err) {
					self.pollRequests--;
					clearInterval(self.pollID);
					self.saveError(err);
				}
			}
			this.pollRequests++;
			var srh = new ServiceResponseHandler(pollWorker, "handleSuccess", "handleError");
			ReportsClient.getDataWarehouseInitializationStatus(srh);
		}

	},
	
	_updateLocationStringExample: function() {
		var locString = "";
		var selectedVendor = this._config.getEditedValueForProperty(DB_VENDOR_NAME).toLowerCase();
		
		dojo.some(LOCATION_STRINGS,function(vendor) {
			if (selectedVendor == vendor.vendorName) {
				locString = vendor.locString;
				return;
			}
		});
		var messageString = dojo.string.substitute(this.messages.jdbcLocExample, [ locString ]);
		this._config.setServerPropertyExample("db.jdbc.location", messageString);
		
	},
	
	_initializeContent: function() {
		var dbProviderSrh = new ServiceResponseHandler({
			handleIds: dojo.hitch(this, this._populateDBProviders),
			handleIdsError: dojo.hitch(this, function(err) {
				this.saveError(err);
			})
		}, "handleIds", "handleIdsError");
		DBProviderClient.getIds(dbProviderSrh);
	},
	
	_populateDBProviders: function(ids) {
		this._supportedVendors = [];
		this._vendorInfo = {};
		dojo.forEach(ids, dojo.hitch(this, function(id) {
			this._supportedVendors.push({
				propertyValue: id,
				displayValue: VENDOR_MAP[id] || id,
				licenseCheck: id
			});
			var srh = new ServiceResponseHandler({
				handleInfo: dojo.hitch(this, function(id, info) {
					this._vendorInfo[id] = info;
					this._initConfigIfReady();
				}, id),
				handleInfoError: dojo.hitch(this, function(id, error) {
					this._vendorInfo[id] = "ERROR";
					this._initConfigIfReady();
				}, id)
			}, "handleInfo", "handleInfoError");
			DBProviderClient.getInfo(srh, {id: id});
		}));
		this.requestLicenseRestrictions();
	},
	
	_initConfigIfReady: function() {
		var serviceInfo = [];
		for (var i=0; i < this._supportedVendors.length; i++) {
			var info = this._vendorInfo[this._supportedVendors[i].propertyValue];
			if (!this._vendorInfo[this._supportedVendors[i].propertyValue]) {
				return;
			} else {
				if (info !== "ERROR") {
					serviceInfo.push({
						implementationClassName: info.implementationClass
					});
				}
			}
		}
		serviceInfo.unshift({
			implementationClassName: PERSISTENT_MAPPING_SERVICE
		});
		serviceInfo.unshift({
			implementationClassName: DB_SERVICE_NAME
		});
		
		
		var serviceConfigProps = {
			serviceInfo: serviceInfo,
			enableEdits: true,
			serviceHeaderClass: ["noDisplay"], 
			displayIn: this.serviceContainer,
			descriptionMode: "table", 
			initialMode: "edit"
		};
		if (this.extraServiceConfigArgs) {
			dojo.mixin(serviceConfigProps, this.extraServiceConfigArgs);
		}

		this._config = new DWServerPropertyService(serviceConfigProps);
		this.connect(this._config, "_show", "_initializePropertiesAfterLoaded");
	},
	
	setStatus: function(status, errorObj, hideBottomStatus) {
		if (this.dbConfigUI.btnNoWarehouse.checked === true) {
			this._state = this.STATUS.NO_WAREHOUSE;
			this._setState(this.STATUS.NO_WAREHOUSE);
			return;
		}
		if (this._testConnectionRequired) {
			this._setState(this.STATUS.APP_TEST);
			this._enablePasswordEdit();
			return;
		}
		if ((this._state != this.STATUS.CONNECTED_OK && this._state != this.STATUS.APP && this._state != this.STATUS.CREATE_USER && this._state != this.STATUS.USER_INVALID) || !this._config._isValid()) {
			this._enablePasswordEdit();
			return; //steps 1- 3 aren't good yet so lets get them resolved first
		} 
		if (!status.isValid) {
			this._state = this.STATUS.USER_INVALID;
			this._setState(this.STATUS.USER_INVALID, null);
			if (status.useMessage) {
				this.header.showMessage({
					type: "ERROR",
					summary: status.message
				});
				this.statusMsgHandler.showMessage({ type: "ERROR", summary: status.message, details: '', errorObject: null });
			}
		} else if (!status.isComplete) {
			this._state = this.STATUS.CREATE_USER;
			this._setState(this.STATUS.CREATE_USER, null);
		} else {
			this._state = this.isJTS ? this.STATUS.CONNECTED_OK : this.STATUS.APP;
			this._setState(this._state);
		}
		this._enablePasswordEdit();
	},
	
	_enablePasswordEdit: function() {
		if (!this._config) {
			return;
		}
		var props = this._config._properties;
		for (var i=0; i < props.length; i++) {
			if (props[i].elementInfo.name == "db.jdbc.password") {
				props[i]._editValueNode.disabled = false;
			}
		};	
	},
	
	_getEtlUser: function(url, handler) {
		var lastChar = url.charAt(url.length-1);
		if (lastChar != '/') url += "/";
		var self = this;
	    var xhrArgs = {
		        url: url+ "service/com.ibm.team.repository.service.internal.IServerConfigurationRestService/serviceConfiguration?implementationClassName=" + INTERNAL_DATAWAREHOUSE_SERVICE_NAME,
		        content: { },
				handleAs: "text",
				headers: this.headers,
				handle: dojo.hitch(this, function(response, ioArgs) {
					if (response instanceof Error) {
						self.saveError(err);
					} else {
						handler(response, self);
					}
		        })
		    };
		jazz.client.xhrGet(xhrArgs);
	},
	
	_handleGetJtsEtlUser: function(response, context) {
		var etlUserIdProperties = context._getProperties(response, "auth.userId");
		var etlUserId = context._getElementValue(etlUserIdProperties, "overrideValue");
		var authTypeProperties = context._getProperties(response, "auth.type");
		var authType = context._getElementValue(authTypeProperties, "overrideValue");
		context._etlUserId = etlUserId;
		context._authType = authType;
		if (etlUserId && etlUserId != "ADMIN") {
			if (context.isJTS) {
				if (authType == "JTS") {
					// Disable the ETL user fields
					context._userSection.setFieldEnabled("userId", false);
					context._userSection.setFieldEnabled("password", false);
					context._userSection.setFieldEnabled("repeatPassword", false);
					context._userSection.setFieldEnabled("emailAddress", false);
				} else {
					var srh = new ServiceResponseHandler(context, "_handleGetUser", "_noop");
					RepositoryClient.getContributors(srh, {userId : etlUserId});
				}
			} else {
				context._getEtlUser(JAZZ_ROOT, dojo.hitch(context, context._handleGetAppEtlUser));
			}
		}
	},
	
	_handleGetAppEtlUser: function(response) {
		var etlUserIdProperties = this._getProperties(response, "auth.userId");
		var etlUserId = this._getElementValue(etlUserIdProperties, "overrideValue");
		var authTypeProperties = this._getProperties(response, "auth.type");
		var authType = this._getElementValue(authTypeProperties, "overrideValue");
		if (etlUserId && etlUserId != "ADMIN") {
			if (authType == "JTS") {
				// Disable the ETL user fields
				this._userSection.setFieldEnabled("userId", false);
				this._userSection.setFieldEnabled("password", false);
				this._userSection.setFieldEnabled("repeatPassword", false);
				this._userSection.setFieldEnabled("emailAddress", false);
			} else {
				var srh = new ServiceResponseHandler(this, "_handleGetUser", "_noop");
				RepositoryClient.getContributors(srh, {userId : etlUserId});
			}
		}
		else {
			if (this._authType == "JTS") {
				this._generateSecret = true;
				authType = this._authType;
				// Disable the ETL user fields
				this._userSection.setFieldEnabled("userId", false);
				this._userSection.setFieldEnabled("password", false);
				this._userSection.setFieldEnabled("repeatPassword", false);
				this._userSection.setFieldEnabled("emailAddress", false);
			} else {
				etlUserId = this._etlUserId;
				this._userSection.setDetails(etlUserId, "", "");
				this._userSection.validate();
			}
		}
		this._etlUserId = etlUserId;
		this._authType = authType;
	},

	_handleGetUser: function(users) {
		if (users != null && users.length > 0) {
			var user = users[0];
			this._savedUser.oldEmail = user.emailAddress;
			this._savedUser.oldUserId = user.userId;
			this._userSection.setDetails(user.userId, user.emailAddress, PASSWORD);
			this._userSection.validate();
			// Disable the ETL user fields
			this._userSection.setFieldEnabled("userId", false);
			this._userSection.setFieldEnabled("password", false);
			this._userSection.setFieldEnabled("repeatPassword", false);
			this._userSection.setFieldEnabled("emailAddress", false);
		}
	},
	
	_createUserSection: function() {	
		var userSection = this.dbConfigUI._createEtlUserSection;
		if (!this.isJTS && EtlUserFormTableContent.rows.length == 4) {
			EtlUserFormTableContent.rows.pop(); //remove email address field on applications
		}
		var widget = this._userSection = new EtlUserForm({
				setStatusCallback: dojo.hitch(this, this.setStatus),
				tableContent: this.isJTS ? EtlUserFormTableContent : EtlUserFormTableContentNotJts,
				description: (this.isJTS == true ? this.messages.createEtlUserDescriptionJts : this.messages.createEtlUserDescriptionNotJts) + 
					' ' + this.messages.createEtlUserLeftBlank
			});
		dojo.connect(this._userSection, "validate", dojo.hitch(this, function() {
			var data = this._userSection.getData();
			this._savedUser.email = data.emailAddress;
			this._savedUser.password = data.password;
			this._savedUser.userId = data.userId;
			this._savedUser._doUpdate = 
					this._savedUser.password != this._savedUser.oldPassword ||
					this._savedUser.userId != this._savedUser.oldUserId;
		}));
		if (userSection.firstChild) {
			if (userSection.firstChild !== widget.domNode) {
				userSection.replaceChild(widget.domNode, userSection.firstChild);
			}
		} else {
			userSection.appendChild(widget.domNode);
		}
	},
	
	_postEtlUserUpdates: function() {
		if (!this._savedUser._doUpdate) {
			return;
		}
		var data = this._userSection.getData();
		var argArray = new Array();
		if (data.password && data.password != PASSWORD) {
			argArray
				.push(this
					._getPropertyArguments("auth.password",
							data.password,
							INTERNAL_DATAWAREHOUSE_SERVICE_NAME));
		}
		if (data.userId) {
			argArray
				.push(this
						._getPropertyArguments("auth.userId",
								data.userId,
								INTERNAL_DATAWAREHOUSE_SERVICE_NAME));
		}
		if (argArray.length > 0) {
			var args = {
				'updates' : dojo.toJson(argArray)
			};
			var srh = new ServiceResponseHandler(this, "_noop", "_noop");
			ServerStatusClient.postConfigurationUpdates(srh, args);
		}
	},
	
	_createUser: function() {
		licensing.getLicenseInfo(new ServiceResponseHandler(this, "_getLicensingInfoCallback", "_doCreateUser"));
	},
	
	_getLicensingInfoCallback: function(data) {
		if (data && data.types) {
			var appCals = [];
			var jtsCal = [];
			for (var i = 0; i < data.types.length; i++) {
				var type = data.types[i];
				if (type.id == "com.ibm.team.clm.datacollector") {
					if (dojo.indexOf(jtsCal, type.id) == -1) {
						jtsCal.push(type.id);
					}
				} else if (type.id.indexOf("datacollector") != -1) {
					if (dojo.indexOf(appCals, type.id) == -1) {
						appCals.push(type.id);
					}
				}
			}
			if ((appCals.length < 3) && (jtsCal.length == 1)) {
				this._dataCollectionLicenses = jtsCal;
            } else {
            	this._dataCollectionLicenses = appCals;
            }
		}
		this._doCreateUser();
	},
	
	_doCreateUser: function() {
		if (this._savedUser.userId && this._etlUserId == this._savedUser.userId) {
			return;
		}
		if (this.isJTS) {	
			if (this._savedUser.userId) {
				//create contributor record
				var valueMap = this._userSection.getData();
				var args = new Object();
				for (i in valueMap) {
					if (i == "repeatPassword") {
						continue;
					}
					args[i] = valueMap[i];
				}
				if (this._registryInfo && this._registryInfo != null) {
					if ((this._registryInfo.writable) || (this.isExternalRegistry)) {
						args["repoOnly"] = !this._registryInfo.writable;
						args["name"] = this.messages.etlUserName;
						args["itemId"] = "new";
						args["jsonRoles"] = dojo.toJson(["JazzAdmins"]);
						args["jsonLicenses"] = dojo.toJson({
							add: this._dataCollectionLicenses,
							remove: []
						});
						AdminClient.postContributor(args, new ServiceResponseHandler(this, "_handleCreateUser", "_handleCreateUserError"));
					} else {
						this._createUserById();
					}
				}
			} else if (this._authType != "JTS") {
				this._createFunctionalUser();
			}
		} else {
			if (this._generateSecret) {
				this._createFunctionalUser();
			}
		}
	},
	
	_createUserById: function() {
		var srh = new ServiceResponseHandler(this, "_handleCreateUserById", "_handleCreateUserByIdError");
		var args = new Object();
		args["userId"] = this._userSection.getData().userId;
		TeamServerClient.invokeService(new ServiceRequest(REGISTRY_REST_SERVICE_INTERFACE_NAME, "postCreateUserById", args), srh);	
	},
	
	_createFunctionalUser: function() {
		var args = new Object();
		args["repoOnly"] = true;
		args["name"] = this.messages.etlUserName;
		args["userId"] = "etl_user";
		args["emailAddress"] = "etl_user";
		args["itemId"] = "new";
		args["jsonRoles"] = dojo.toJson(["JazzAdmins"]);
		args["jsonLicenses"] = dojo.toJson({
			add: this._dataCollectionLicenses,
			remove: []
		});
		AdminClient.postContributor(args, new ServiceResponseHandler(this, "_handleCreateFunctionalUser", "_handleCreateFunctionalUserError"));
	},
	
	_handleCreateUser: function() {
		this._etlUserId = this._userSection.getData().userId;
	},
	
	_handleCreateUserError: function(error) {
		if (this._dataCollectionLicenses.length > 0 || error.errorClass != "com.ibm.team.repository.service.internal.license.UnknownLicenseId") {
			alert(error.message + "\n\n" + this.messages.updatesStillApplied);
			this._getContributors();
		}
	},
	
	_handleCreateUserById: function() {
		this._etlUserId = this._userSection.getData().userId;
		if (this._dataCollectionLicenses.length > 0) {
			this._getContributors();
		}
	},
	
	_handleCreateUserByIdError: function(error) {
		alert(error.message + "\n\n" + this.messages.updatesStillApplied);
		if (this._dataCollectionLicenses.length > 0) {
			this._getContributors();
		}
	},
	
	_getContributors: function() {
		var args = new Object();
		args["searchField"] = "userId";
		args["searchTerm"] = this._userSection.getData().userId;
		AdminClient.getContributors(args, new ServiceResponseHandler(this, "_handleGetContributors", "_noop"));
	},
	
	_handleGetContributors: function(resultSet) {
		if (resultSet != null && resultSet.elements != null) {
			for (var i=0; i < resultSet.elements.length; i++) {
				var args = new Object();
				args["repoOnly"] = true;
				args["itemId"] = resultSet.elements[i].itemId;
				args["stateId"] = resultSet.elements[i].stateId;
				args["jsonLicenses"] = dojo.toJson({
						add: this._dataCollectionLicenses,
						remove: []
					});
				AdminClient.postContributor(args, new ServiceResponseHandler(this, "_noop", "_noop"));
				break;
			}
		}
	},
	
	_handleCreateFunctionalUser: function() {
		this._requestSecret();
	},
	
	_handleCreateFunctionalUserError: function(error) {
		this._requestSecret();
	},
	
	_requestSecret: function () {
		var srh = new ServiceResponseHandler(this, "_registerConsumerWithSecret", "_handleServiceError");
		OAuthAdminClient.generateSecret(srh);
	},
	
	_registerConsumerWithSecret: function (secret) {
		this._consumerSecret = secret;
		var args = {consumerSecret: secret,
		            secretType: "STRING",
				    name: "Jazz Reporting"};
		args.preAuthorizedUserId = "etl_user";
		var srh = new ServiceResponseHandler(this, "_handleConsumerRegistrationSuccess", "_handleServiceError");
		OAuthAdminClient.registerNewConsumer(srh, args);
	},
	
	_handleConsumerRegistrationSuccess: function (consumerKey) {
		var srh = new ServiceResponseHandler(this, "_noop", "_handleServiceError");
		OAuthAdminClient.setConsumerTrusted(srh, consumerKey, true);
		var argArray = new Array();
		argArray
			.push(this
				._getPropertyArguments("auth.password",
						this._consumerSecret,
						INTERNAL_DATAWAREHOUSE_SERVICE_NAME));
		argArray
			.push(this
				._getPropertyArguments("auth.userId",
						consumerKey,
						INTERNAL_DATAWAREHOUSE_SERVICE_NAME));
		argArray
			.push(this
				._getPropertyArguments("auth.type",
						"JTS",
						INTERNAL_DATAWAREHOUSE_SERVICE_NAME));
		var args = {
			'updates' : dojo.toJson(argArray)
		};
		srh = new ServiceResponseHandler(this, "_noop", "_handleServiceError");
		ServerStatusClient.postConfigurationUpdates(srh, args);
	},
	
	setUserRegistryType: function(value) {
		if (!value) {
			console.error("Unable to determine user registry type.");
		} else {
			value = value.toUpperCase();	
		}
		this.isExternalRegistry = (value == "UNSUPPORTED");
		var srh = new ServiceResponseHandler(this, "getUserRegistryInfoCallback", "_noop");
		AdminClient.getUserRegistryInfo(srh);
	},
	
	getUserRegistryInfoCallback: function(registryInfo) {
		this._registryInfo = registryInfo;
	},
	
	_handleServiceError: function(error) {
		alert(error.message);
	},

	_setCurrentlyTesting: function() {
		this._currentlyTesting = true;
	},
	
	_unsetCurrentlyTesting: function() {
		this._currentlyTesting = false;
		this._testConnectionRequired = false;
	},

	useFinishButton: function() {
		return !this.isEmbedded && this.finalStep();
	},
	
	canFinish: function() {
		return this.enableNext && this.useFinishButton();
	}
	
});

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.DWServerPropertyService", [dijit._Widget], {
	enableToggle: true,
	initialMode: "view",		// valid values are "view" or "edit"
	descriptionMode: "tooltip", // valid values are "tooltip" or "table"
	serviceCallsPending: 0,
	
	constructor: function() {
	},
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.DWServerPropertyService.superclass.postCreate.apply(this);
		checkNotNull(this.serviceInfo, "this.serviceInfo");
		checkNotNull(this.displayIn, "this.displayIn");
		this._serverPropertyServices = [];
		for (var i=0; i < this.serviceInfo.length; i++) {
			var props = {
				serviceInfo: {
					implementationClassName: this.serviceInfo[i].implementationClassName
				},
				enableEdits: this.enableEdits,
				serviceHeaderClass: this.serviceHeaderClass, 
				displayIn: this.displayIn,
				descriptionMode: this.descriptionMode, 
				initialMode: this.initialMode
			}
			var service = new ServerPropertyService(props);
			this._serverPropertyServices.push(service);
			this.connect(service, "_onServiceResponse", "_onServiceResponse");
			this.connect(service, "_markChanged", "_markChanged");
			this.connect(service, "_checkUnchanged", "_checkUnchanged");
			this.serviceCallsPending++;
		}
		this._initialize();
	},
	
	_onServiceResponse: function() {
		this.serviceCallsPending--;
	},
	
	_initialize: function() {
		this._showingMap = {};
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._properties = [];
			this.connect(this._serverPropertyServices[i], "_handleServiceProps", dojo.hitch(this, function(index) {
				for (var j=0; j < this._serverPropertyServices[index]._properties.length; j++) {
					this._properties.push(this._serverPropertyServices[index]._properties[j]);
				}
				this._showingMap[index] = true;
				if (this._showing) {
					return;
				}
				for (var j=0; j < this._serverPropertyServices.length; j++) {
					if (!this._showingMap[j]) {
						return;
					}
				}
				this._showing = true;
				this._show();
			}, i));
		}
	},
	
	_show: function() {
		// empty method that can be connected to in order to run tasks after
		// the config properties are displayed in the UI
	},
	
	_markChanged: function() {
		this._changed = true;
	},
	
	_checkHide: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._checkHide();
		}
	},
	
	_checkHideLongHeaders: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._checkHideLongHeaders();
		}
	},
	
	_checkHideShortHeaders: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._checkHideShortHeaders();
		}
	},
	
	_showLongHeaders: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._showLongHeaders();
		}
	},
	
	_showShortHeaders: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._showShortHeaders();
		}
	},
	
	_hide: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._hide();
		}
	},
	
	_checkUnchanged: function() {
		var changed = false;
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			if (this._serverPropertyServices[i]._changed) {
				changed = true;
				break;
			}
		}
		this._changed = changed;
	},
	
	_checkValid: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._checkValid();
		}
	},
	
	_isValid: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			if (this._serverPropertyServices[i]._invalid == true) {
				return false;
			}
		}
		return true;
	},
		
	_fetchValidity: function() {
		var pendingUpdates = this._getAllValuesExceptSensitiveReplacements();
		if (pendingUpdates.length === 0) {
			return;
		}
		var changeArgs = {
			updates: dojo.toJson(pendingUpdates), 
			fast: true
		};
		var srh = new ServiceResponseHandler(this, "_checkValidityCallback", "_checkValidityError");
		ServerStatusClient.postConfigurationValidation(srh, changeArgs);
	},
	
	_getAllValuesExceptSensitiveReplacements: function() {
		//don't send sensitive unless it's changed
		var changes = [];
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			var curChanges = this._serverPropertyServices[i]._getAllValuesExceptSensitiveReplacements();
			for (var j=0; j < curChanges.length; j++) {
				changes.push(curChanges[j]);
			}
		}
		return changes;
	},
	
	_checkValidityCallback: function(msg) {
		for (var i=0; i < this._properties.length; i++) {
			this._properties[i]._markValid();
		}
		if (msg != null) {
			// sort by reverse alpha (warning, ok, error) to apply errors last
			if (msg.length > 1) {
				msg.sort(
					function(a, b) {
						return a.code < b.code ? 1 : -1;
					});
			}
			// loop through all the messages		
			for (var i = 0; i < msg.length; i++) {			
				for (var j = 0; j < this._properties.length; j++) {
					// apply message to the correct property name
					if (msg[i].name === this._properties[j]._name) {
						// send whole msg object so element can display according to type 
						this._properties[j]._markInvalid(msg[i]);
					}
				}
			}
		}
		for (var i = 0; i < this._properties.length; i++) {
			this._properties[i]._validationPending = false;
		}
	},
	
	_checkValidityError: function() {
		for (var i = 0; i < this._properties.length; i++) {
			this._properties[i]._validationPending = false;
		}
	},
		
	_getChanges: function() {
		var changes = [];
		var properties = [];
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			var curChanges = this._serverPropertyServices[i]._getChanges();
			for (var j=0; j < curChanges.changes.length; j++) {
				changes.push(curChanges.changes[j]);
			}
			for (var j=0; j < curChanges.properties.length; j++) {
				properties.push(curChanges.properties[j]);
			}
		}
		return {
			"changes": changes,
			"properties": properties
		};
	},
	
	_toggleEdit: function(editMode, target) {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._toggleEdit(editMode, target);
		}
	}, 
	
	// public methods
	revertDefault: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i].revertDefault();
		}
	},
	
	restoreSaved: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i].restoreSaved();
		}
	},
	
	testRobustValidity: function(handler) {
		var pendingUpdates = this._getAllValuesExceptSensitiveReplacements();
		if (pendingUpdates.length === 0) {
			return;
		}
		var changeArgs = {
			updates: dojo.toJson(pendingUpdates), 
			fast: false
		};
		ServerStatusClient.postConfigurationValidation(handler, changeArgs);		
	},

	hasOverrideValues: function() {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			if (this._serverPropertyServices[i].hasOverrideValues()) {
				return true;
			}
		}
		return false;
	},
	
	getEditedValueForProperty: function(propertyName) {
		return this.getServerPropertyElement(propertyName)._editValueNode.value;		
	},	
	
	getServerPropertyElement: function(propertyName) {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			var propElem = this._serverPropertyServices[i].getServerPropertyElement(propertyName);
			if (propElem != null) {
				return propElem;
			}
		}
		return null;
	},
	
	getPendingServiceCalls: function() {
		var pending = 0;
		for (var i = 0; i < this._serverPropertyServices.length; i++) {
			pending += this._serverPropertyServices.getPendingServiceCalls();
		}
		return pending;
	},
	
	setServerPropertyExample: function(propertyName,exampleString) {
		for (var i = 0; i < this._serverPropertyServices.length; i++) {
			var propElem = this._serverPropertyServices[i].getServerPropertyElement(propertyName);
			if (propElem != null) {
				this._serverPropertyServices[i].setServerPropertyExample(propertyName, exampleString);
			}
		}	
	}
});

var DWServerPropertyService = com.ibm.team.repository.web.ui.internal.wizard.DWServerPropertyService;

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.DWConfigInnerSteps", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/ConfigureDataWarehouse.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function() {
		this.messages = dwMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDataWarehouseMessages");
		var JAZZ_ROOT = net.jazz.ajax.contextRoot();
		var adminLink = "<a href='" + JAZZ_ROOT + "/admin#action=com.ibm.team.repository.admin.manageLicenses'>";
		this.reviewAdminLink = dojo.string.substitute(this.messages.reviewAdminLink, [ adminLink, "</a>" ]);	
	},
	
	postCreate: function() {
		this.step1_header.innerHTML = "<span class=\"step\">" + this.messages.step1Label + "</span><span>" + this.messages.step1Title + "</span>";
		this.step2_header.innerHTML = "<span class=\"step\">" + this.messages.step2Label + "</span><span>" + this.messages.step2Title + "</span>";
		this.step3_header.innerHTML = "<span class=\"step\">" + this.messages.step3Label + "</span><span>" + this.messages.step3Title + "</span>";
	}
	
});

var DWConfigInnerSteps = com.ibm.team.repository.web.ui.internal.wizard.DWConfigInnerSteps;


dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.EtlUserForm", WizardForm, {
	
	_currentUserId: undefined,
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDataWarehouseMessages");
	}, 
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.EtlUserForm.superclass.postCreate.apply(this);
		RepositoryClient.getAuthenticatedContributor(new ServiceResponseHandler(this, "_getAuthenticatedContributorHandler", "_getAuthenticatedContributorHandlerError"));
	},
	
	_getAuthenticatedContributorHandler: function(contributor) {
		if (contributor && contributor.userId) {
			this._currentUserId = contributor.userId;
		} else {
			this._currentUserId = '';
		}
	},
	
	_getAuthenticatedContributorHandlerError: function(error) {
		this._currentUserId = '';
	},
	
	setDetails: function(userId, emailAddress, password) {
		for (var i=0; i<this.dataFields.length; i++) {
			var name = this.dataFields[i].name;
			if (name == "userId") {
				this.dataFields[i].value = userId;
			} else if (name == "emailAddress") {
				this.dataFields[i].value = emailAddress;
			} else if (name == "password" || name == "repeatPassword") {
				this.dataFields[i].value = password;
			}
		}
	},
	
	setFieldEnabled: function(field, enabled) {
		for (var i=0; i<this.dataFields.length; i++) {
			var name = this.dataFields[i].name;
			if (name == field) {
				this.dataFields[i].disabled = !enabled;
				break;
			}
		}
	},
	
	validate: function() {
		var isEmpty = true;
		for (var i=0; i<this.dataFields.length; i++) {
			var val = this.dataFields[i].value;
			if (val === "") {
				continue;
			}
			isEmpty = false;
			break;
		}
		if (isEmpty) {
			var status = {
				isComplete: true,
				isValid: true,
				type: "OK"
			};
			this.setStatusCallback(status);
			return status;
		}
		var status = this.inherited("validate", arguments);
		status.useMessage = undefined;
		if (!status.isValid) {
			status.message = this.messages.etlUserFormInvalid;
		} else if (!status.isComplete) {
			status.message = this.messages.completeEtlUserForm;
		} else {
			if (typeof this._currentUserId !== 'undefined') {
				var user = null;
				for (var i=0; i<this.dataFields.length; i++) {
					var name = this.dataFields[i].name;
					if (name == "userId") {
						user = this.dataFields[i].value;
						break;
					}
				}
				if (this._currentUserId != user) {
					status.message = this.messages.clickNext;
				} else {
					status.message = this.messages.etlUserSameAsAuthenticatedUser;
					status.isValid = false;
					status.useMessage = true;
				}
			} else {
				setTimeout(dojo.hitch(this, "validate"), 100);
			}
		}
		this.setStatusCallback(status);
		return status;
	}
});

var EtlUserFormTableContent = {
	header: [
		     	{type: "label", labelKey: "propertyLabel"},
		    	{type: "data", labelKey: "valueLabel"},
		    	{type: "description", labelKey: "descriptionLabel"}
		    ],
		rows: [
			{name: "userId", labelKey: "userIdLabel", descriptionKey: "userIdDescrption"},
			{name: "password", labelKey: "passwordLabel", type: "password", descriptionKey: "passwordDescrption"},
			{name: "repeatPassword", labelKey: "passwordLabel2", type: "password", descriptionKey: "passwordDescrption2", mustMatch: {
				matchName: "password",
				errorMessageKey: "passwordMismatch"
			}},
			{name: "emailAddress", labelKey: "emailLabel", dataType: "email", descriptionKey: "emailDescrption"}
		]
	};

var EtlUserFormTableContentNotJts = {
	header: [
		     	{type: "label", labelKey: "propertyLabel"},
		    	{type: "data", labelKey: "valueLabel"},
		    	{type: "description", labelKey: "descriptionLabel"}
		    ],
		rows: [
			{name: "userId", labelKey: "userIdLabel", descriptionKey: "userIdDescrption"},
			{name: "password", labelKey: "passwordLabel", type: "password", descriptionKey: "passwordDescriptionNotJts"},
			{name: "repeatPassword", labelKey: "passwordLabel2", type: "password", descriptionKey: "passwordDescrption2", mustMatch: {
				matchName: "password",
				errorMessageKey: "passwordMismatch"
			}}
		]
	};

var EtlUserForm = com.ibm.team.repository.web.ui.internal.wizard.EtlUserForm;
})();
