/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.client.internal.DBProviderClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.StyledBox");
dojo.require("net.jazz.ajax.contextRoot");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDatabaseMessages");

(function() {

var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;
var WizardConfigurationStep = com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep;
var licensing = com.ibm.team.repository.web.client.internal.licensing;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var DBProviderClient = com.ibm.team.repository.web.client.internal.DBProviderClient;

var PERSISTENT_MAPPING_SERVICE = "com.ibm.team.repository.service.internal.schemagen.PersistentMappingService";
var DB_SERVICE_NAME = "com.ibm.team.repository.service.internal.rdb.ConnectionPoolService"
var DB_VENDOR_NAME = "db.vendor";
var DB_TYPE_NAME = "db.type";
var JDBC_TYPE_VALUE = "JDBC";
var VENDOR_LICENSE_CHECK_PREFIX = "com.ibm.team.repository.support.";
	
var DISPLAY_PROPS = [];
DISPLAY_PROPS.J2EE = {
	ALL: [
		"db.j2ee.dataSource"
	]
};
DISPLAY_PROPS.JDBC = {
	ALL: [
		"db.jdbc.password",
		"db.jdbc.location"
	],
	db2i: [
		"db.db2i.schema.name"
	],
	db2z: [
		"db.schemaPrefix",
		"db.db2.dsn.dbname"
	]
};

var VENDOR_MAP = {
	'db2': "DB2",
	'derby': "Derby",
	'derby_net': "Derby Client",
	'oracle': "Oracle",
	'sqlserver': "SQL Server",
	'db2z': "DB2 for z/OS",
	'db2i': "DB2 for i"
};

//Example location strings
var LOCATION_STRINGS = [
	{vendorName: "db2", locString: "//localhost:50000/JTS:user=db2inst1;password={password};"},  
	{vendorName: "oracle", locString: "thin:jazzDBuser/{password}@//localhost:1521/ORCL"}, 
	{vendorName: "sqlserver", locString: "//localhost:1433;databaseName=jts;user=jazzDBuser;password={password}"},
	{vendorName: "db2i", locString: "//hostname;hold statements=true;user=jazzDBuser;password={password};metadata source=1;"}, 
	{vendorName: "db2z", locString: "//ipAddress:ipPort/location:user=jazzDBuser;password={password};"},
	{vendorName: "derby_net", locString: "//localhost:1527/databaseName"}
];

var messages;

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase",  WizardConfigurationStep, {

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDatabaseMessages");
		this.implementationClassName = "com.ibm.team.repository.service.internal.rdb.ConnectionPoolService"
		this.pendingNext = null;
		this.stepTitle = messages.wizardPageHeading;
		
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		
		//Add *dynamic* descriptions
		this.description = this.isJTS ? this.messages.wizardPageDescription_jts : this.messages.wizardPageDescription_app;
	},
	
	createInnerStepUI: function () {
		this.dbConfigUI = new DBConfigInnerSteps();
		this.serviceContainer = this.dbConfigUI._hiddenTableBody;
		this.configArea.appendChild(this.dbConfigUI.domNode);
		this.connect(this.dbConfigUI.createDBLink,"onclick","_createDatabases");
		
		//We'll need this later
		this.STATUS.DB_NEEDS_TABLES = {
				test: false,
				revert: false,
				next: false,
				edit: true,
				code: "INFO",
				resolvedMessage: messages.DBneedsTables
		};
		this.STATUS.DB_NEEDS_TABLES_PARTIALLY_CREATED = {
				test: false,
				revert: false,
				next: false,
				edit: true,
				code: "ERROR",
				resolvedMessage: messages.DBneedsTablesPartiallyCreated
		};
	},

	requestLicenseRestrictions: function () {
		this.licenseRequestPending = true;
		this.licenseCheckResponse = 0;
		this.licenseCheckRequests = 0;
		var self = this;
		dojo.forEach(this._supportedVendors, function(vendor){ 
			//Don't do the checks anymore, currently all vendors are always supported
			vendor.licenseAllows = "OK";
			
			/* Don't delete this code in case DB/Licensing requirements are enabled again. (WI 121764)
			
			self.licenseCheckRequests++;
			var checkingLicense = {
				vendor: vendor,
				
				handleLicenseCheck: function(response){
					vendor.licenseAllows = response.code;
					self.incrementLicenseCheckCount();
				},
				handleAcceptError: function(err){
					self.saveError(err);
				}
			}
			var srh = new ServiceResponseHandler(checkingLicense, "handleLicenseCheck", "handleAcceptError");
			licensing.checkLicense(srh, {operationId: VENDOR_LICENSE_CHECK_PREFIX + vendor.licenseCheck} );
			
			*/
			
		});	
		this.incrementLicenseCheckCount(); //ADDED
	},
	
	incrementLicenseCheckCount: function () {
		//this.licenseCheckResponse++;
		if (this.licenseCheckResponse === this.licenseCheckRequests){
			// All license checks are back. Load the vendors if the UI already loaded 
			this.licenseRequestPending = false;
			if (this.displayloaded && !this.vendorsDisplayed) {
				this._displayVendorsAllowedByLicense();
			}
		}	
	},
		
	_initializePropertiesAfterLoaded: function () {
		this._initializeHiddenProperty(DB_VENDOR_NAME);
		this._initializeHiddenProperty(DB_TYPE_NAME);
		
		this._initializePropertyDisplay(this.dbConfigUI._jdbcEditPropertiesBody, DISPLAY_PROPS.JDBC);
		this._initializePropertyDisplay(this.dbConfigUI._j2eeEditPropertiesBody, DISPLAY_PROPS.J2EE);
				
		this._displayRelevantSections();
		
		this.connect(this.dbConfigUI._selectVendor, "onchange", "_handleVendorSelectChange");
		this.connect(this.dbConfigUI._selectType, "onchange", "_handleTypeSelectChange");
		
		this.inherited(arguments);
		if (!this.licenseRequestPending) {
			this._displayVendorsAllowedByLicense();
		}
		this.displayloaded = true;
	},	
	
	_displayVendorsAllowedByLicense: function () {
		this.vendorsDisplayed = true;
		var self = this;
		dojo.forEach(this._supportedVendors, function(vendor){ 
	  		if (vendor.licenseAllows && vendor.licenseAllows === "OK"){
				var newOption = new Option(vendor.displayValue, vendor.propertyValue);
				self.dbConfigUI._selectVendor.options[self.dbConfigUI._selectVendor.options.length] = newOption;
			} else {
				self._displayLicenseLimitedMessage();
			}
		});	
		this._synchronizeSelectValue();	
	},

	_displayLicenseLimitedMessage: function() {
		dojo.toggleClass(this.dbConfigUI._licenseLimitedMessage, "noDisplay", false);
	}, 
	
	_initializeHiddenProperty: function (propertyName) {
		var targetProperty = this._config.getServerPropertyElement(propertyName);
		this.connect(targetProperty, "_markInvalid", function (msg) {
			this._handleInvalidInput(msg);
			this._showHiddenError(msg);
		});
	},
	
	_showHiddenError: function(msg){
		if (msg.code === "ERROR") {
			this._setState(this.STATUS.PENDING_ERR, [msg]);
		} else {
			this._setState(this.STATUS.PENDING_WARN, [msg]);
		}
	},
	
	_initializePropertyDisplay: function (attachPoint, displayProps) {
		
		var targetPropertyServiceRow = new ServerPropertyServiceRow({
			_enableToggle: false,
			rowClass: ["indentNone editProperties"],
			_descriptionMode: "table"
		});
		attachPoint.appendChild(targetPropertyServiceRow.domNode);
		attachPoint._elements = [];
		var longExists = false;
		var shortExists = false;
		for (var type in displayProps) {
			for (var i=0; i < displayProps[type].length; i++) {
				var propertyElement = this._config.getServerPropertyElement(displayProps[type][i]);
				if(!propertyElement) continue;
				var vendors = propertyElement._vendors;
				if (!vendors) {
					vendors = {};
					propertyElement._vendors = vendors;
				}
				vendors[type] = true;
				attachPoint._elements.push(propertyElement);
				if (propertyElement.elementInfo.propertyType === "STRING") {
					longExists = true;
					var table = targetPropertyServiceRow._longPropertiesTableBody;
				} else {
					shortExists = true;
					var table = targetPropertyServiceRow._shortPropertiesTableBody;
				}
				table.appendChild(propertyElement.domNode);
			}
		}
		if (!longExists) {
			targetPropertyServiceRow._longPropertiesTableHeader.style.display = "none";
		}
		if (!shortExists) {
			targetPropertyServiceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
		
	_handleVendorSelectChange: function (event) {
		this._handleSelectChange(event, DB_VENDOR_NAME);
	},
	
	_handleTypeSelectChange: function(event){
		this._handleSelectChange(event, DB_TYPE_NAME);
	}, 
	
	_handleSelectChange: function (event, propertyName) {
		var newSelectValue = event.target.value;
		this._handleChange(newSelectValue, propertyName);
	},
	
	_handleChange: function (newValue, propertyName) {
		var targetProperty = this._config.getServerPropertyElement(propertyName);
		
		targetProperty._editValueNode.value = newValue;
		
		if (targetProperty._savedValue != newValue) {
			targetProperty._markChanged();
		} else {
			targetProperty._markUnchanged();
		}
		targetProperty._checkValidity();
		this._displayRelevantSections();
		this.triggerButtonStateUpdate();
	},
	
	_revert: function() {
		this.inherited(arguments);
		this._synchronizeSelectValue();
		this._displayRelevantSections();

	},
			
	// UPDATE VIEW METHODS	
	_displayRelevantSections: function() {
		// KMB TODO need to clear out values on hide
		var vendor = this._config.getEditedValueForProperty(DB_VENDOR_NAME);
		if (vendor) {
			vendor = vendor.toLowerCase();
		}
		if (this._config.getEditedValueForProperty(DB_TYPE_NAME) === JDBC_TYPE_VALUE){
			this.dbConfigUI._j2eePropertiesSection.style.display = "none";
			this.dbConfigUI._jdbcPropertiesSection.style.display = "block";
			
			for (var i=0; i < this.dbConfigUI._jdbcEditPropertiesBody._elements.length; i++) {
				var elem = this.dbConfigUI._jdbcEditPropertiesBody._elements[i];
				var vendors = elem._vendors;
				if (vendors && (vendors["ALL"] || vendors[vendor])) {
					elem.domNode.style.display = "";
				} else {
					elem.domNode.style.display = "none";
				}
			}
		} else {
			this.dbConfigUI._j2eePropertiesSection.style.display = "block";
			this.dbConfigUI._jdbcPropertiesSection.style.display = "none";
		}
		
		this._updateLocationStringExample();
		
		this.dbConfigUI._createTablesSection.style.display = "none";	
		
	}, 
	
	_allowPropertyEdit: function (allowEdits){
		this.inherited(arguments); 
		this.dbConfigUI._selectVendor.disabled = !allowEdits;
		this.dbConfigUI._selectType.disabled = !allowEdits;
	},
	
	_synchronizeSelectValue: function() {
		var vendorProp = this._config.getServerPropertyElement(DB_VENDOR_NAME);
		this.dbConfigUI._selectVendor.value = vendorProp._editValueNode.value.toLowerCase();
		var typeProp = this._config.getServerPropertyElement(DB_TYPE_NAME);
		this.dbConfigUI._selectType.value = typeProp._editValueNode.value;
		this._displayRelevantSections();
	},
	
	_testConnectionCallback: function(msg) {
		var connected = null;
		this.dbConfigUI._createTablesSection.style.display = "none";
		if (msg != null) {
			connected = this.STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = this.STATUS.CONNECTED_ERR;
					//Check to see if we need to create Tables (i.e. error code 3)
					if(msg[i].id == 3 || msg[i].id == 7) {
						connected = (msg[i].id == 3) ? this.STATUS.DB_NEEDS_TABLES : this.STATUS.DB_NEEDS_TABLES_PARTIALLY_CREATED;						                                                                  
						msg = null; //Don't include 'show details'
						this.dbConfigUI.createDBLink.disabled = false;
						this.dbConfigUI._createTablesSection.style.display = "block";
						break;
					}
				}
			}
		} else {
			connected = this.STATUS.CONNECTED_OK;
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		this._setState(connected, msg);
	},
	
	_createDatabases: function() {
		this.dbConfigUI.createDBLink.disabled = true;

		this.STATUS.DB_SUCCESS = {
				test: false,
				revert: false,
				next: true,
				edit: true,
				code: "OK",
				resolvedMessage: messages.creationSuccess
		};
		this.STATUS.DB_FAIL = {
				test: true,
				revert: true,
				next: false,
				edit: true,
				code: "ERROR",
				resolvedMessage: messages.creationFail
		};
		this.STATUS.DB_PROGRESS = {
				test: false,
				revert: false,
				next: false,
				edit: false,
				code: "INFO",
				resolvedMessage: messages.creationProgress
		};
		
		//First we need to save the properties before we can call createTables
		this.invokeSave(this, "saveOK", "saveError");
	},
	
	saveOK: function() {
		this.header.showMessage({
			type: "OK",
			summary: this.messages.settingsSaved
		});
		var srh = new ServiceResponseHandler(this, "_handleCreateTables", "_handleAcceptError");
		ServerStatusClient.postInitalizeDatabase(srh);
	},
	
	_handleCreateTables: function(response){	
		if(response == "true") { //Response passed back as string
		
			//Database tables are not created.
			this._setState(this.STATUS.DB_PROGRESS);
			//Begin to poll for progress
			this.pollRequests = 0;
			this.pollID = setInterval(dojo.hitch(this,"checkProgress"),500);
		} else {
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.tablesExist
			});
		}
	},
	
	_handleAcceptError: function(err){
		this.saveError(err);
	},
	
	checkProgress: function() {
		
		//Don't want to keep sending out requests unless they are coming back...
		if(this.pollRequests < 1) {
			var self = this;
			var pollWorker = {
				handleSuccess: function(response){
					self.pollRequests--;
					if(response.id == 6) {
						//The database creation has completed sucessfully
						clearInterval(self.pollID);
						self.dbConfigUI.createDBLink.disabled = true;
						self._setState(self.STATUS.DB_SUCCESS);
						
					} else if(response.id == 2) {
						//The database creation failed with an exception
						clearInterval(self.pollID);
						self.dbConfigUI.createDBLink.disabled = false;
						
						//Convert to the expected errorObj format
						var errorObj = {
							message: response.message,
							exception: response.stacktrace
						};
						
						self._setState(self.STATUS.DB_FAIL,errorObj,true);
					} 
				},
				handleError: function(err){
					self.pollRequests--;
					clearInterval(self.pollID);
					self.saveError(err);
				}
			}
			this.pollRequests++;
			var srh = new ServiceResponseHandler(pollWorker, "handleSuccess", "handleError");
			ServerStatusClient.getCreationStatus(srh);
		}

	},
	
	_updateLocationStringExample: function() {
		var locString = "";
		var selectedVendor = this._config.getEditedValueForProperty(DB_VENDOR_NAME).toLowerCase();
		if(selectedVendor != "derby") {
			dojo.some(LOCATION_STRINGS,function(vendor) {
				if(selectedVendor == vendor.vendorName) {
					locString = vendor.locString;
					return;
				}
			});
			var messageString = dojo.string.substitute(this.messages.jdbcLocExample, [ locString ]);
			this._config.setServerPropertyExample("db.jdbc.location", messageString);
		} else {
			this._config.setServerPropertyExample("db.jdbc.location","");
		}
	},
	
	_initializeContent: function () {
		var dbProviderSrh = new ServiceResponseHandler({
			handleIds: dojo.hitch(this, this._populateDBProviders),
			handleIdsError: dojo.hitch(this, function(err) {
				this.saveError(err);
			})
		}, "handleIds", "handleIdsError");
		DBProviderClient.getIds(dbProviderSrh);
	},
	
	_populateDBProviders: function(ids) {
		this._supportedVendors = [];
		this._vendorInfo = {};
		dojo.forEach(ids, dojo.hitch(this, function(id) {
			this._supportedVendors.push({
				propertyValue: id,
				displayValue: VENDOR_MAP[id] || id,
				licenseCheck: id
			});
			var srh = new ServiceResponseHandler({
				handleInfo: dojo.hitch(this, function(id, info) {
					this._vendorInfo[id] = info;
					this._initConfigIfReady();
				}, id),
				handleInfoError: dojo.hitch(this, function(id, error) {
					this._vendorInfo[id] = "ERROR";
					this._initConfigIfReady();
				}, id)
			}, "handleInfo", "handleInfoError");
			DBProviderClient.getInfo(srh, {id: id});
		}));
		this.requestLicenseRestrictions();
	},
	
	_initConfigIfReady: function() {
		var serviceInfo = [];
		for (var i=0; i < this._supportedVendors.length; i++) {
			var info = this._vendorInfo[this._supportedVendors[i].propertyValue];
			if (!this._vendorInfo[this._supportedVendors[i].propertyValue]) {
				return;
			} else {
				if (info !== "ERROR") {
					serviceInfo.push({
						implementationClassName: info.implementationClass
					});
				}
			}
		}
		serviceInfo.unshift({
			implementationClassName: PERSISTENT_MAPPING_SERVICE
		});
		serviceInfo.unshift({
			implementationClassName: DB_SERVICE_NAME
		});
		
		
		var serviceConfigProps = {
			serviceInfo: serviceInfo,
			enableEdits: true,
			serviceHeaderClass: ["noDisplay"], 
			displayIn: this.serviceContainer,
			descriptionMode: "table", 
			initialMode: "edit"
		};
		if (this.extraServiceConfigArgs){
			dojo.mixin(serviceConfigProps, this.extraServiceConfigArgs);
		}

		this._config = new DBServerPropertyService(serviceConfigProps);
		this.connect(this._config, "_show", "_initializePropertiesAfterLoaded");
	}
	
});

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.DBServerPropertyService", [dijit._Widget], {
	enableToggle: true,
	initialMode: "view",		// valid values are "view" or "edit"
	descriptionMode: "tooltip", // valid values are "tooltip" or "table"
	serviceCallsPending: 0,
	
	constructor: function() {
	},
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.DBServerPropertyService.superclass.postCreate.apply(this);
		checkNotNull(this.serviceInfo, "this.serviceInfo");
		checkNotNull(this.displayIn, "this.displayIn");
		this._serverPropertyServices = [];
		for (var i=0; i < this.serviceInfo.length; i++) {
			var props = {
				serviceInfo: {
					implementationClassName: this.serviceInfo[i].implementationClassName
				},
				enableEdits: this.enableEdits,
				serviceHeaderClass: this.serviceHeaderClass, 
				displayIn: this.displayIn,
				descriptionMode: this.descriptionMode, 
				initialMode: this.initialMode
			}
			var service = new ServerPropertyService(props);
			this._serverPropertyServices.push(service);
			this.connect(service, "_onServiceResponse", "_onServiceResponse");
			this.connect(service, "_markChanged", "_markChanged");
			this.connect(service, "_checkUnchanged", "_checkUnchanged");
		}
		this._initialize();
	},
	
	_onServiceResponse: function () {},
	
	_initialize: function () {
		this._showingMap = {};
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._properties = [];
			this.connect(this._serverPropertyServices[i], "_handleServiceProps", dojo.hitch(this, function(index) {
				for (var j=0; j < this._serverPropertyServices[index]._properties.length; j++) {
					this._properties.push(this._serverPropertyServices[index]._properties[j]);
				}
				this._showingMap[index] = true;
				if (this._showing) {
					return;
				}
				for (var j=0; j < this._serverPropertyServices.length; j++) {
					if (!this._showingMap[j]) {
						return;
					}
				}
				this._showing = true;
				this._show();
			}, i));
		}
	},
	
	_show: function () {
		// empty method that can be connected to in order to run tasks after
		// the config properties are displayed in the UI
	},
	
	_markChanged: function () {
		this._changed = true;
	},
	
	_checkHide: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._checkHide();
		}
	},
	
	_checkHideLongHeaders: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._checkHideLongHeaders();
		}
	},
	
	_checkHideShortHeaders: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._checkHideShortHeaders();
		}
	},
	
	_showLongHeaders: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._showLongHeaders();
		}
	},
	
	_showShortHeaders: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._showShortHeaders();
		}
	},
	
	_hide: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._hide();
		}
	},
	
	_checkUnchanged: function () {
		var changed = false;
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			if (this._serverPropertyServices[i]._changed) {
				changed = true;
				break;
			}
		}
		this._changed = changed;
	},
	
	_checkValid: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._checkValid();
		}
	},
		
	_fetchValidity: function () {
		var pendingUpdates = this._getAllValuesExceptSensitiveReplacements();
		if (pendingUpdates.length === 0) {
			return;
		}
		var changeArgs = {
			updates: dojo.toJson(pendingUpdates), 
			fast: true
		};
		var srh = new ServiceResponseHandler(this, "_checkValidityCallback", "_checkValidityError");
		ServerStatusClient.postConfigurationValidation(srh, changeArgs);
	},
	
	_getAllValuesExceptSensitiveReplacements: function () {
		//don't send sensitive unless it's changed
		var changes = [];
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			var curChanges = this._serverPropertyServices[i]._getAllValuesExceptSensitiveReplacements();
			for (var j=0; j < curChanges.length; j++) {
				changes.push(curChanges[j]);
			}
		}
		return changes;
	},
	
	_checkValidityCallback: function (msg) {
		for (var i=0; i < this._properties.length; i++) {
			this._properties[i]._markValid();
		}
		if (msg != null) {
			// sort by reverse alpha (warning, ok, error) to apply errors last
			if (msg.length > 1) {
				msg.sort(
					function(a, b) {
						return a.code < b.code ? 1 : -1;
					});
			}
			// loop through all the messages		
			for (var i = 0; i < msg.length; i++) {			
				for (var j = 0; j < this._properties.length; j++) {
					// apply message to the correct property name
					if (msg[i].name === this._properties[j]._name) {
						// send whole msg object so element can display according to type 
						this._properties[j]._markInvalid(msg[i]);
					}
				}
			}
		}
		for (var i = 0; i < this._properties.length; i++) {
			this._properties[i]._validationPending = false;
		}
	},
	
	_checkValidityError: function () {
		for (var i = 0; i < this._properties.length; i++) {
			this._properties[i]._validationPending = false;
		}
	},
		
	_getChanges: function () {
		var changes = [];
		var properties = [];
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			var curChanges = this._serverPropertyServices[i]._getChanges();
			for (var j=0; j < curChanges.changes.length; j++) {
				changes.push(curChanges.changes[j]);
			}
			for (var j=0; j < curChanges.properties.length; j++) {
				properties.push(curChanges.properties[j]);
			}
		}
		return {
			"changes": changes,
			"properties": properties
		};
	},
	
	_toggleEdit: function (editMode, target) {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i]._toggleEdit(editMode, target);
		}
	}, 
	
	// public methods
	revertDefault: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i].revertDefault();
		}
	},
	
	restoreSaved: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			this._serverPropertyServices[i].restoreSaved();
		}
	},
	
	testRobustValidity: function (handler) {
		var pendingUpdates = this._getAllValuesExceptSensitiveReplacements();
		if (pendingUpdates.length === 0) {
			return;
		}
		var changeArgs = {
			updates: dojo.toJson(pendingUpdates), 
			fast: false
		};
		ServerStatusClient.postConfigurationValidation(handler, changeArgs);		
	},

	hasOverrideValues: function () {
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			if (this._serverPropertyServices[i].hasOverrideValues()){
				return true;
			}
		}
		return false;
	},
	
	getEditedValueForProperty: function(propertyName){
		return this.getServerPropertyElement(propertyName)._editValueNode.value;		
	},	
	
	getServerPropertyElement: function(propertyName){
		for (var i=0; i < this._serverPropertyServices.length; i++) {
			var propElem = this._serverPropertyServices[i].getServerPropertyElement(propertyName);
			if (propElem != null) {
				return propElem;
			}
		}
		return null;
	},
	
	getPendingServiceCalls: function() {
		var pending = 0;
		for (var i = 0; i < this._serverPropertyServices.length; i++) {
			pending += this._serverPropertyServices.getPendingServiceCalls();
		}
		return pending;
	},
	
	setServerPropertyExample: function(propertyName,exampleString) {
		for (var i = 0; i < this._serverPropertyServices.length; i++) {
			var propElem = this._serverPropertyServices[i].getServerPropertyElement(propertyName);
			if (propElem != null) {
				this._serverPropertyServices[i].setServerPropertyExample(propertyName, exampleString);
			}
		}	
	}
});

var DBServerPropertyService = com.ibm.team.repository.web.ui.internal.wizard.DBServerPropertyService;

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.DBConfigInnerSteps", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/ConfigureDatabase.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function(){
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigureDatabaseMessages");
		var JAZZ_ROOT = net.jazz.ajax.contextRoot();
		var adminLink = "<a href='" + JAZZ_ROOT + "/admin#action=com.ibm.team.repository.admin.manageLicenses'>";
		this.reviewAdminLink = dojo.string.substitute(this.messages.reviewAdminLink, [ adminLink, "</a>" ]);	
	},
	
	postCreate: function() {
		this.step1_header.innerHTML = "<span class=\"step\">" + this.messages.step1Label + "</span><span>" + this.messages.step1Title + "</span>";
		this.step2_header.innerHTML = "<span class=\"step\">" + this.messages.step2Label + "</span><span>" + this.messages.step2Title + "</span>";
		this.step3_header.innerHTML = "<span class=\"step\">" + this.messages.step3Label + "</span><span>" + this.messages.step3Title + "</span>";
	}
	
});

var DBConfigInnerSteps = com.ibm.team.repository.web.ui.internal.wizard.DBConfigInnerSteps;

})();