/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("net.jazz.ajax.contextRoot");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "FinalizeStepMessages");

(function() {
var messages;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var arrayIndexWithPropertyOfObject = com.ibm.team.repository.web.ui.internal.admin.util.arrayIndexWithPropertyOfObject;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;
var ALLOW_ACCCESS_PROPERTY_NAME = "ws.allow.admin.access";
var REGISTRATION_HANDLER_SERVICE = "com.ibm.team.repository.service.internal.setup.IRegistrationHandlerService";
var SETUP_COMPLETION_SERVICE = "com.ibm.team.repository.service.internal.setup.ISetupCompletionService";
var SERVLET_CONFIG_SERVICE_NAME = "com.ibm.team.repository.service.internal.ServletConfigurationService";

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/FinalizeStep.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "FinalizeStepMessages");
		this.stepTitle = messages.stepTitle;
	},

	postCreate: function() {
		this.header = new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		//Set StyledBox titles
		this.step1_header.innerHTML = "<span class=\"step\">" + this.messages.step1Label + "</span><span>" + this.messages.step1Title + "</span>";
		this.step2_header.innerHTML = "<span class=\"step\">" + this.messages.step2Label + "</span><span>" + this.messages.step2Title + "</span>";
		
		//Prepare for raw XHR requests
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		
		//Could improve this to be more robust...
		this.servicePrefix = net.jazz.ajax.contextRoot() + "/service/";
		
		//Controls whether or not the step can go next
		this._setCompletedState(false);
		
		this.hasProcessed = false;
		this.startButton.disabled = true;
		
		//Create DOM for the individual finalize operations
		this.domFinalize = this._addOperationDiv(this.messages.operation1title, this.messages.operation1desc);
		this.domUserSync = this._addOperationDiv(this.messages.operation2title, this.messages.operation2desc);

		this.connect(this.startButton, "onclick", "start");
	},
	
	focusStep: function() {
		this.isFinalizationNeeded();
	},
	
	isFinalizationNeeded: function() {
		var self = this;
		var dFinalize = this.finalizationIsNeeded();
		dFinalize.addCallback(function(response){
			if(response === "true") {
				//Finalize needed
				var srh = new ServiceResponseHandler(self, "setJTSUrl", "handleAppNotRegistered");
				ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
			} else {
				self.header.showMessage({type: "OK", summary: self.messages.noFinalizeNeeded });
				self._setCompletedState(true);
				dojo.addClass(self.progressSection, "noDisplay");
			}
		});
		dFinalize.addErrback(function(errObject){
			//Allow the user to try to finalize since we don't know if they have already done so.
			self._handleError(self.messages.finalizeCheckError, errObject);
			self.startButton.disabled = false;
		});
	},
	
	setJTSUrl: function(response) {
		this.configuredJTS = response;
		this.startButton.disabled = false;
		this.triggerButtonStateUpdate();
	},
	
	handleAppNotRegistered: function(error) {
		this.header.showMessage({
			errorObject: error, 
			summary: this.messages.appNotRegistered 
		});
	},
	
	//STEP 1: REGISTRATION HANDLER, FINALIZE
	start: function(e) {
		dojo.stopEvent(e);
		this.startButton.disabled = true;
		if (!this.hasProcessed) {	
			var self = this;
			
			//Show Pending Message
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.finalizePending
			});	
			this.triggerButtonStateUpdate();
			
			//Create DOM to display the progress
			var domNode = this.domFinalize;
			var dFinalize = this.finalize();
			dFinalize.addCallback(function(response){
				self.hasProcessed = true;
				dojo.addClass(domNode, "ok");
				self.userSync();
			});
			dFinalize.addErrback(function(errObject){
				self._showInlineError(domNode, self.messages.finalizeError, errObject);
				self.startButton.disabled = false;
			});
		}
	},
	
	//STEP 2: USER SYNC
	userSync: function() {	
		var self = this;
		var domNode = this.domUserSync;
		var dUserSync = this.setupCompletion("userSync");
		dUserSync.addCallback(function(response){
			dojo.addClass(domNode, "ok");
			
			//Check if the user is logged in as ADMIN. If so, tell them that they will be disabled and do a logout
			if(self.configuredJTS) {
				//Proxy to the JTS to see if ADMIN is disabled
				var deferred = self.getJTSAdminDisabled(self.configuredJTS);
				deferred.addCallback(function(response){
					var index = arrayIndexWithPropertyOfObject(response.properties, "name", ALLOW_ACCCESS_PROPERTY_NAME);
					var allowAdmin = response.properties[index];
					var allowAdminValue = allowAdmin.overrideValue ? allowAdmin.overrideValue : allowAdmin.defaultValue;
					if (allowAdminValue === "false") {
						if(self.isDefaultAdmin()) {
							self.header.showMessage({
								type: "INFO",
								summary: self.messages.disablingAdmin
							});
							self._setCompletedState(false); //hide scrollbar
							self.forceLogout = true;
						}
						//Make the call to actually disable ADMIN on the application
						var dFinalizeAdmin = self.finalizeAdminUser();
						dFinalizeAdmin.addCallback(function(response){
							self._setCompletedState(true);
							self.showCompletedMessage();
						});
						dFinalizeAdmin.addErrback(function(errObj){
							console.error(errObj); //TODO: Add a custom error message here
							self._setCompletedState(true);
						});
					} else {
						self.showCompletedMessage();
					}
				});
				deferred.addErrback(function(errObj){
					console.error(errObj);
					self.showCompletedMessage();
				});
			} else {
				self.showCompletedMessage();
			}
		});
		dUserSync.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.userSyncError, errObject);
		});
	},
	
	showCompletedMessage: function() {
		var buttonLabel = this.useFinishButton() === true ? this.messages.finish : this.messages.next;
		this.header.showMessage({
			type: "OK",
			summary: dojo.string.substitute(this.messages.allComplete, [ buttonLabel ])
		});	
		this._setCompletedState(true);
	},
	
	finalizationIsNeeded: function() {
		var deferred = new dojo.Deferred();
		jazz.client.xhrGet({
			url: this.servicePrefix + REGISTRATION_HANDLER_SERVICE + "?finalizationNeeded=true",
			handleAs: "text",
			headers: this.headers,
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		return deferred;
	},

	finalize: function() {
		var deferred = new dojo.Deferred();
		jazz.client.rawXhrPost({
			url: this.servicePrefix + REGISTRATION_HANDLER_SERVICE + "?finalize=true",
			handleAs: "json",	 //$NON-NLS-1$
			headers: this.headers,
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);		
			}
		});	
		return deferred;
	},
	
	finalizeAdminUser: function() {
		var deferred = new dojo.Deferred();
		jazz.client.rawXhrPost({
			url: this.servicePrefix + REGISTRATION_HANDLER_SERVICE + "?finalizeAdminUser=true",
			handleAs: "json",
			headers: this.headers,
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);		
			}
		});	
		return deferred;
	},
	
	setupCompletion: function(operation) {
		var deferred = new dojo.Deferred();
		jazz.client.rawXhrPost({
			url: this.servicePrefix + SETUP_COMPLETION_SERVICE,
			handleAs: "json",
			headers: this.headers,
			content: {
				operation: operation
			},
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		return deferred;
	},
	
	_showInlineError: function(node, message, errObject) {
		dojo.addClass(node,"error");
		var errorDiv = node.getElementsByTagName("div");
		dojo.removeClass(errorDiv[0], "noDisplay");
		var errorObj = this._getErrorObject(errObject);
		node._errorMsgBox.showMessage({
			errorObject: errorObj,
			summary: message
		});
		this._setCompletedState(false);
	},
	
	_hideInlineError: function(node) {
		var errorDiv = node.getElementsByTagName("div");
		dojo.addClass(errorDiv[0], "noDisplay");
	},	
	
	_handleError: function(message, response) {
		var errorObj = this._getErrorObject(response);
		this.header.showMessage({
			errorObject: errorObj,
			summary: message
		});
		this._setCompletedState(false);
	},
	
	_getErrorObject: function(response) {
		var errorObj = {};
		if (response && response.responseText) {
			//TODO: Code from TeamServerClient.js......
			try {
				var responseObj = dojo.fromJson(response.responseText);
				dojo.mixin(errorObj, {
					errorClass: responseObj.errorClass,
					message: responseObj.errorMessage,
					exception: responseObj.errorTrace,
					errorData: responseObj.errorData,
					nestedError: responseObj.nestedError
				});
			} 
			catch (/* Error */ e) {
				// for non-JSON responses, include it as a message
				errorObj.message = response.responseText;
			}
		}
		return errorObj;
	},
	
	//Creates a new DIV element for the operation and appends it to the DOM
	//Returns: a handle to the DIV element
	_addOperationDiv: function(title, description, customClass) {
		var div = dojo.create("div", { className: "operation"}, this.operations);
		if(customClass) {
			dojo.addClass(div, customClass);
		}
		dojo.create("h3", { className: "operationTitle", innerHTML: title}, div);
		dojo.create("p", { className: "operationDesc", innerHTML: description}, div);
		var errDiv = dojo.create("div", { className: "operationError noDisplay"}, div);
		div._errorMsgBox = new jazz.ui.MessageBox({
			onToggle: dojo.hitch(this, this._handleMsgBoxToggle)
		});
		errDiv.appendChild(div._errorMsgBox.domNode);
		return div;
	},
	
	_handleMsgBoxToggle: function() {
		this.triggerButtonStateUpdate();
	},
	
	_setCompletedState: function(state) {
		this.isCompleted = state;
		this.triggerButtonStateUpdate();
	},
	
	getJTSAdminDisabled: function(jtsURL) {
		var endpoint = "service/com.ibm.team.repository.service.internal.IServerConfigurationRestService/serviceConfiguration";
		endpoint += "?implementationClassName=" + SERVLET_CONFIG_SERVICE_NAME;
		var deferred = new dojo.Deferred();
		var self = this;
		//Tweak the headers a bit for this call
		var headers = this.headers;
		headers[HttpConstants.ACCEPT] = HttpConstants.CT_TEXT_JSON;
		if(jtsURL) {
			jazz.client.xhrGet({
				url: this._buildServiceURL(jtsURL, endpoint),
				handleAs: "json",
				headers: headers,
				load: function(response) {
					var response = self._demarshallResponse(response);
					deferred.callback(response);
				},
				error: function(errObject) {
					deferred.errback(errObject);
				}
			});	
		}
		return deferred;
	},
	
	_buildServiceURL: function(jts, endpoint) {	
		var lastChar = jts.charAt(jts.length-1);
		if(lastChar != '/') jts += "/";
		return jts + endpoint;
	},
	
	_demarshallResponse: function(evaldObj) {
		return com.ibm.team.repository.web.transport.TeamServerClient._demarshallResponse(evaldObj);
	},
	
	isDefaultAdmin: function() {
		var authInSession = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		return authInSession && authInSession == "ADMIN";
	},
	
	customOnNext: function() {
		if(this.forceLogout) {
			jazz.app.auth.logout();	
		}
	},
	
	customOnFinish: function() {
		this.customOnNext();
	},

	canFlipToNextPage: function() {
		return this.isCompleted;
	},
	
	useFinishButton: function() {
		return !this.isEmbedded && this.finalStep();
	},
	
	canFinish: function() {
		return this.isCompleted && this.useFinishButton();
	}

});

})();