/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.SetupApplication");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("jazz.ui.Wizard");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.util.browserSupport");
dojo.require("jazz.ui.UnsupportedBrowserWarning");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.app.auth");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupApplicationMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupApplicationMessages");

var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var UserRegistryClient = com.ibm.team.repository.web.client.internal.userRegistry;
var Wizard = jazz.ui.Wizard;
var WizardStep = jazz.ui.wizard.WizardStep;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var NotAuthorizedView = com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView;

var EXPRESS_SETUP_URL = "/expressSetup";
var CUSTOM_SETUP_URL = "/customSetup";

var SETUP_HAS_RUN_PROPERTY = "completed.setup";
var SETUP_CONFIGURATION_SERVICE = "com.ibm.team.repository.service.internal.setup.SetupConfigurationService";

var LOCAL_STORAGE_KEY = "com-ibm-team-repository-web-SetupWizardChoice";

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.SetupApplication", dijit._Widget, {
	
	constructor: function() {
		this.messages = messages;
	},
	
	postCreate: function() {
		this._fetchInitData();
	},
	
	_fetchInitData: function (){
		var srh = new ServiceResponseHandler(this, "_handleInitData", "_fetchInitDataFailed");
		RepositoryClient.getInitializationData(srh);
	},
	
	_handleInitData: function(initData) {
		com.ibm.team.repository.web.client.session.init(initData);
		this.getUserRoles();
	},
	
	_fetchInitDataFailed: function(errorObject) {
		//If this fails, we can still continue
		this.getUserRoles();
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			this.loggedInUser = response.userId;
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this._startApp();
	},
	
	authRequestFailed: function() {
		this._startApp();
	},
	
	_startApp: function () {
		if(this.authorized) {
			ServerStatusClient.getServerModeString(new ServiceResponseHandler(this, "_storeIsJTS", "_handleIsJTSError"));
		} else {
			this.isJTS = true;
			this._checkBrowser();
		}
	},
	
	_storeIsJTS: function(response) {
		this.isJTS = (response == "APPLICATION") ? false : true;
		dojo.setObject("com.ibm.team.repository.server.mode", response);
		this._checkBrowser();
	},
	
	_handleIsJTSError: function(errObject) {
		// Forbidden users see unauthorized message
		if (errObject.status === 403) {
			// this is probably an archived admin user
			this.authorized = false;
		}
		this.isJTS = true;
		this._checkBrowser();
	},

	_checkBrowser: function() {
		//Check to see if we are using a supported Browser
		var support = jazz.util.browserSupport.getSupportLevel();
		if (support && (support == jazz.util.browserSupport.NOT_SUPPORTED)) {
			var warn = new jazz.ui.UnsupportedBrowserWarning();
			this.connect(warn, "onContinue", function() {
				warn.destroy();
				this._getClusterStatus();
			});
			warn.placeAt(document.body);
		} else {
			this._getClusterStatus();
		}
	},
	
	_getClusterStatus: function() {
		var handler = {
			success: dojo.hitch(this, function(response) {
				dojo.setObject("com.ibm.team.repository.isInCluster", response === "false" ? false : true);
				this._createUI();	
			}),
			error: dojo.hitch(this, function(err) {
				console.info("Unable to determine if the server is running in a cluster.");	
				this._createUI();
			})
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getIsInCluster(srh);
	},
	
	createLogoutArea: function() {
		var logoutNode = dojo.create("div", { className: "logout-node" });
		dojo.create("span", { innerHTML: this.loggedInUser, className: "userName" }, logoutNode);
		dojo.create("span", { innerHTML: "|", style: { color: "#999", padding: "0 4px" } }, logoutNode);
		this.logoutLink = dojo.create("a", { innerHTML: this.messages.logout, href: "#", className: "logoutLink" }, logoutNode);
		this.connect(this.logoutLink, "onclick", function(e) {
			dojo.stopEvent(e);
			var dfd = jazz.app.auth.logout();
			dfd.addCallback(function() {
				window.location.reload();
			});
		});
		return logoutNode;
	},
	
	_createUI: function() {
		document.body.style.background = "none";
		document.title = this.messages.setupName;
		this.content = dojo.create("div", { className: "com-ibm-team-repository-wizard setup-application-wizard" });
		if (this.loggedInUser) {
			this.content.appendChild(this.createLogoutArea());
		}
		dojo.addClass(this.domNode, "com-ibm-team-repository-web-setup-RootNode");
		var setupTitle = this.messages.serverSetup;
		dojo.create("div", { innerHTML: setupTitle.toUpperCase(), className: "setup-header"}, this.content);
		var jazzIcon = dojo.create("img", { className: "jazz-icon" }, this.content);
		jazzIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/jazz-identR.png");
		this.wizardBox = document.createElement("div");
		this.content.appendChild(this.wizardBox);
		var wizardParams = {
			fullPageContainer: this.wizardBox,
			showStepSummary: false,
			updateNavigationControls: this._updateNavigationControls
		};
		if (this.authorized) {
			var SetupApplicationStep = new com.ibm.team.repository.web.ui.internal.wizard.SetupApplicationStep();
			wizardParams.steps = [ SetupApplicationStep ];
		} else {
			var notAuthStep = new NotAuthorizedView({
					applicationName: this.messages.setupName,
					customNoUserMessage: this.messages.userNotRecognizable,
					showLogout: true });
			dojo.mixin(notAuthStep, new WizardStep());
			notAuthStep.includeNavigationUI = false;
			wizardParams.stepListLayout = "horizontal";
			wizardParams.steps = [notAuthStep];
			wizardParams.showStepSummary = false;
		}
		this.wizard = new Wizard(wizardParams);
		var outerFooter = new com.ibm.team.repository.web.ui.internal.wizard.LogoFooter();
		this.content.appendChild(outerFooter.domNode);
		this.domNode.appendChild(this.content);
	},
	
	_updateNavigationControls: function() {
		//override this.wizard.updateNavigationControls to control our own button behavior
	}
	
});

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.SetupApplicationStep", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SetupApplicationStep.html"),
	widgetsInTemplate: true,
	
	constructor: function() {
		this.messages = messages;
		this.stepTitle = this.messages.stepTitle;
	},

	postCreate: function() {
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		this.connect(this.expressOption, "onclick", "_selectionMade");
		this.connect(this.customOption, "onclick", "_selectionMade");
		
		//The Express Setup should be disabled if the server is configured for LDAP. For the pre-setup case, we can detect this
		//by looking to see if the loggedInUser exists in the repository. If not, the contrib will be "ADMIN".
		this.ldapEnabled = false;
		var loggedInUser = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		var contrib = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR");
		if (loggedInUser && contrib && contrib.userId) {
			if(loggedInUser != contrib.userId) {
				this.ldapEnabled = true;
			}
		}

		var serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (serverMode == "APPLICATION") ? false : true;
		
		this.isCluster = dojo.getObject("com.ibm.team.repository.isInCluster");
		
		if(!this.isCluster && this.isJTS && !this.ldapEnabled) {
			this.isExpressEnabled();
		}
	},
	
	focusStep: function() {
		//Manually show the Next button and hide the Finish button
		dojo.removeClass(this.wizard.wizardNextButton, "noDisplay");
		dojo.addClass(this.wizard.wizardFinishButton, "noDisplay");
		
		//Disable Express Setup if we are running in APPLICATION mode
		if (!this.isJTS) {
			this.disableExpress(this.messages.disabledInAppMode);
			return;
		}
		
		//Disable Express Setup if we are running LDAP
		if(this.ldapEnabled) {
			this.disableExpress(this.messages.disabledLDAP);
			return;
		}
		
		//Disable Express Setup if we are running in a Cluster
		if(this.isCluster) {
			this.disableExpress(this.messages.disabledCluster);
		}
	},
	
	isExpressEnabled: function() {
		var srh = new ServiceResponseHandler(this, "_handleServiceProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, SETUP_CONFIGURATION_SERVICE);
	},
	
	_handleServiceProps: function(serviceInfo) {
		if (serviceInfo.properties && serviceInfo.properties.length > 0) {
			dojo.forEach(serviceInfo.properties, function(prop){
				if (prop.name == SETUP_HAS_RUN_PROPERTY) {
					this.hasSetupRun = prop.overrideValue;
					return;
				}
			}, this);
		}
		this.updateOptions();
	},
	
	updateOptions: function() {
		if(this.hasSetupRun === "true") {
			this.disableExpress(this.messages.disabledMessage);
		} 
		else {
			//See if we have a saved setup preference in localStorage
			var data = dojo.fromJson(localStorage[LOCAL_STORAGE_KEY]);
			if(data && data.choice) {
				val = data.choice;
				if(val === "express") {
					this.expressOption.checked = true;
					this._selectionMade();
				} else if(val === "custom") {
					this.customOption.checked = true;
					this._selectionMade();
				}	
			}
					
			// Setup is marked as not completed & loggedInUser exists in the repository.
			// It's _still_ possible that LDAP is enabled but setup was not marked as complete. (WI #206179)
			// We should fire off an async xhr request to check.
			var handler = {
				success: dojo.hitch(this, function(value) {
					if (value == "LDAP" || value == "LDAPLocalGroup") {
						this.ldapEnabled = true;
						this.disableExpress(this.messages.disabledLDAP);
					}
				}),
				error: dojo.hitch(this, function(errObj) {
					this.disableExpress(this.messages.disabledDatabase); //possibly relative derby path on WAS?
				})
			};
			var srh = new ServiceResponseHandler(handler, "success", "error");
			UserRegistryClient.getUserRegistryType(srh);
		}
	},
	
	disableExpress: function(msg) {
		this.expressOption.disabled = true;
		this.customOption.checked = true;
		dojo.addClass(this.expressRow, "disabledRow");
		if(msg) {
			this.header.showMessage({
				summary: msg,
				type: "INFO"
			});
		}
		this._selectionMade();	
	},
	
	_handleServiceFetchError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},

	onNext: function() {
		var choice = this._getWizardSelection();
		var data = {
			choice: choice,
			mainSetupURL: location.href 
		};
		
		//Save the choice so that we can remember it if the user visits the page again later
		try {
			localStorage[LOCAL_STORAGE_KEY] = dojo.toJson(data);
		}
		catch (e) {
			// browser doesn't support localStorage
		}
		
		var url = net.jazz.ajax.contextRoot();
		if(choice === "express") {
			url += EXPRESS_SETUP_URL;
		} else if(choice === "custom") {
			url += CUSTOM_SETUP_URL;
		} else {
			return;
		}
		if(location.search) {
			url += location.search;
		}
		location.href = url;
	},

	_selectionMade: function() {
		if(this.wizard) {
			this.wizard.wizardNextButton.disabled = false;
		}
	},
	
	_getWizardSelection: function() {
		if (this.expressOption.checked) {
			return this.expressOption.value;
		} else if (this.customOption.checked) {
			return this.customOption.value;
		} else {
			return null;
		}
	}

});

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.LogoFooter", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/LogoFooter.html"),
	
	constructor: function() {
		this.messages = messages;
	}
});

})();