/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.WizardApplication");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.ui.Wizard");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.wizard.ExternalWizardStep");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardIntro");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardReview");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDataWarehouse");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.EnableEmail");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.SetupUsers");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.RegisteredAppsStep");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ContributedStepsController");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.schema.SetupWizardDescriptor");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.WhitelistUtility");
dojo.require("jazz.app.entryPoints");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.util.browserSupport");
dojo.require("jazz.ui.UnsupportedBrowserWarning");

dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ConfigureDatabase = com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase;
var ConfigureDataWarehouse = com.ibm.team.repository.web.ui.internal.wizard.ConfigureDataWarehouse;
var EnableEmail = com.ibm.team.repository.web.ui.internal.wizard.EnableEmail;
var SetupUsers = com.ibm.team.repository.web.ui.internal.wizard.SetupUsers;
var FinalizeStep = com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep;
var RegisteredApps = com.ibm.team.repository.web.ui.internal.wizard.RegisteredAppsStep;
var Wizard = jazz.ui.Wizard;
var WizardStep = jazz.ui.wizard.WizardStep;
var Intro = com.ibm.team.repository.web.ui.internal.wizard.WizardIntro;
var Review = com.ibm.team.repository.web.ui.internal.wizard.WizardReview;
var ConfigurePublicURI = com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI;
var NotAuthorizedView = com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView;
var ContributedStepsController = com.ibm.team.repository.web.ui.internal.wizard.ContributedStepsController;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var BootstrapProperties = net.jazz.ajax.BootstrapProperties;
var ExternalWizardStep = jazz.ui.wizard.ExternalWizardStep;
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var WizardDescriptorSchema = com.ibm.team.repository.web.ui.internal.wizard.schema.SetupWizardDescriptor;
var WhitelistUtility = com.ibm.team.repository.web.ui.internal.admin.server.WhitelistUtility;
var STEPS_URI_PREFIX = "/steps/";

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.WizardApplication", dijit._Widget, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	},
	
	postCreate: function() {
		this.getUserRoles();
	},

	_startApp: function () {
		this.isEmbedded = isEmbedded();
		if(this.authorized) {
			ServerStatusClient.getServerModeString(new ServiceResponseHandler(this, "_storeIsJTS", "_handleIsJTSError"));
		} else {
			this.isJTS = true;
			this._checkBrowser();
		}
	},
	
	_storeIsJTS: function(response) {
		this.isJTS = (response == "APPLICATION") ? false : true;
		
		//Set as a global variable so the steps have access to it.
		dojo.setObject("com.ibm.team.repository.server.mode", response);
		
		//If this is the JTS, we should fire off a request to "warm-up" any installed applications we have detected.
		//This task will run in the background and we don't really care about the results of this operation.
		if(this.isJTS) {
			OAuthAdminClient.postWarmUpLocallyInstalledApps(new ServiceResponseHandler(this, "doNothing", "doNothing"));
		}
		this._checkBrowser();
	},
	
	_handleIsJTSError: function(errObject) {
		// Forbidden users see unauthorized message
		if (errObject.status === 403) {
			// this is probably an archived admin user
			this.authorized = false;
		} else {
			console.info(this.messages.isJTSUnknown);
		}
		this.isJTS = true;
		this._checkBrowser();
	},
	
	_checkBrowser: function() {
		//Check to see if we are using a supported Browser
		var support = jazz.util.browserSupport.getSupportLevel();
		if (support && (support == jazz.util.browserSupport.NOT_SUPPORTED)) {
			var warn = new jazz.ui.UnsupportedBrowserWarning();
			this.connect(warn, "onContinue", function() {
				warn.destroy();
				this._createUI();
			});
			warn.placeAt(document.body);
		} else {
			this._createUI()
		}
	},
	
	createLogoutArea: function() {
		var logoutNode = dojo.create("div", { className: "logout-node" });
		this.loggedInUserNode = dojo.create("span", { innerHTML: this.loggedInUser, className: "userName" }, logoutNode);
		dojo.create("span", { innerHTML: "|", style: { color: "#999", padding: "0 4px" } }, logoutNode);
		this.logoutLink = dojo.create("a", { innerHTML: this.messages.logout, href: "#", className: "logoutLink" }, logoutNode);
		this.connect(this.logoutLink, "onclick", function(e) {
			dojo.stopEvent(e);
			var dfd = jazz.app.auth.logout();
			dfd.addCallback(function() {
				window.location.reload();
			});
		});
		return logoutNode;
	},
	
	_createUI: function() {
		document.body.style.background = "none";
		document.title = this.messages.setupName;
		this.content = dojo.create("div", { className: "com-ibm-team-repository-wizard" });
		
		if(!this.isEmbedded) {
			if (this.loggedInUser) {
				this.content.appendChild(this.createLogoutArea());
			}
			dojo.addClass(this.domNode, "com-ibm-team-repository-web-setup-RootNode");
			var setupTitle = this.isJTS ? this.messages.serverSetup : this.messages.applicationSetup;
			dojo.create("div", { innerHTML: setupTitle.toUpperCase(), className: "setup-header"}, this.content);
			var jazzIcon = dojo.create("img", { className: "jazz-icon" }, this.content);
			jazzIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/jazz-identR.png");
		//	var setupGraphic = dojo.create("img", { className: "wizard-graphic" }, this.content);
		//	setupGraphic.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/JTS-wizard-graphic.png");
			
		} else {
			dojo.addClass(this.content, "wizard-embedded");
		}

		this.wizardBox = document.createElement("div");
		this.content.appendChild(this.wizardBox);		
		var wizardParams = {
			fullPageContainer: this.wizardBox,
			stepListLayout: "vertical",
			processFinish:  function(){
				try {
					localStorage["com-ibm-team-repository-web-SetupHasFinished"] = dojo.toJson({ "fromSetup": true });
				} catch (e) {}
				document.location.href = net.jazz.ajax.contextRoot() + "/admin";
            }
		};
		
		if (this.authorized) { 
			wizardParams.steps = [];
		} else {
			var notAuthStep = new NotAuthorizedView({
					applicationName: this.messages.setupName,
					customNoUserMessage: this.messages.userNotRecognizable,
					showLogout: true,
					redirectUrl: window.location.href
			});
			dojo.mixin(notAuthStep, new WizardStep());
			notAuthStep.includeNavigationUI = false;
			wizardParams.stepListLayout = "horizontal";
			wizardParams.steps = [notAuthStep];
			wizardParams.showStepSummary = false;
		}
		this.wizard = new Wizard(wizardParams);
		if (this.authorized) { 
			this.initServerSetupSteps();
			this.wizard.contentWidget.updateURI(null);
		}

		if (!this.isEmbedded) {
			var outerFooter = new com.ibm.team.repository.web.ui.internal.wizard.LogoFooter();
			this.content.appendChild(outerFooter.domNode);
		}

		this.domNode.appendChild(this.content);
		
		if (this.isJTS && this.authorized) {
			//Inspect the registered applications for step contributions
			var controller = new ContributedStepsController();
			controller.initParams(this, "addSummaryStep", "_handleGetApplicationsFailed");
		}
		
		//0.6 FRONTING APP (Not Embedded)
		if(!this.isJTS && !this.isEmbedded) {
			var handler = {
				success: dojo.hitch(this, function (response) {
					if(response) {
						this.jtsURL = response;
						this.whitelister = new WhitelistUtility();
						this.whitelister.onSuccess = dojo.hitch(this, function() {
							this.isLPAInstalled();
						});
						this.whitelister.addURL(this.jtsURL);
					}
				}),
				failure: dojo.hitch(this, "doNothing")
			};
			
			var srh = new ServiceResponseHandler(handler, "success", "failure");
			ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
		}
		
		
	},
	
	initServerSetupSteps: function () {
		// Compute the name of our initial step group
		if(BootstrapProperties.FRONTSIDE_URL) {
			var url = BootstrapProperties.FRONTSIDE_URL;
			var index = url.lastIndexOf("/");
			var instanceName = url.substr(index);
		}
		if(BootstrapProperties.APP_NAME) {
			var groupName = BootstrapProperties.APP_NAME;
			if(instanceName) {
				groupName += "<span class=\"instanceName\">(" + instanceName + ")</span>";
			}
		} else {
			var groupName = this.messages.jazzTeamServer;
		}
		
		// CLM/JTS Setup Wizard
		if(this.isJTS) {
			var group = { name: groupName, steps: [
				new Intro(),
				new ConfigurePublicURI(),
				new ConfigureDatabase(),
				new EnableEmail(),
				new RegisteredApps({ myParent: this}), //so it can call this.addSummaryStep()
				new SetupUsers({ myParent: this }),
				new ConfigureDataWarehouse({isJTS: true})
			]};
		// 0.6 FRONTING APPS	
		} else {
			var group = { name: groupName, steps: [
				new ConfigureDatabase(),
			    new FinalizeStep({ isEmbedded: this.isEmbedded }),
			    new ConfigureDataWarehouse({isJTS: false, isEmbedded: this.isEmbedded })
			]};
		}
		var node = this.wizard.addStepGroup(group);
		dojo.addClass(node, "first"); //hides the top border and adds extra padding
	},
	
	addSummaryStep: function(response) {
		//If we already have a summary group, we have to remove it before adding another.
		if(this.summaryGroup) {
			this.wizard.removeGroup(this.summaryGroup);
		}
		var group = { 
			name: this.messages.summaryGroupTitle,
			steps: [ new Review() ]
		};
		this.summaryGroup = this.wizard.addStepGroup(group);
		
		//If the user tried to refresh an embedded step, we need to wait
		//until they are loaded, and then try to load that step for them
		this.wizard.checkForPendingStepRequest();
		
		//This fixes an issue in IE7 where the summary step shows up below the intro step on first load
		//It's because the summary step is added after the call to hideChildren()
		//This fix uses private API of the wizard to force the step content to hide/show correct steps again
		this.wizard.contentWidget.displayStepContent();
	},
	
	//0.6 FRONTING APP (Not Embedded)
	isLPAInstalled: function() {
		var deferred = jazz.app.entryPoints.get("LifecycleProjectAdmin");
		deferred.addCallback(dojo.hitch(this,function(url) {
			if(url) {
				this.lpaURL = url;
				var type = "http://jazz.net/xmlns/prod/jazz/jfs/1.0/setupWizardDescriptor";
				var deferred = DiscoveredService.loadDiscoveryServices(type, WizardDescriptorSchema.descriptorService);
				deferred.addBoth(dojo.hitch(this, function(doc){
					if(doc && doc.setupWizardDescriptor) {
						dojo.forEach(doc.setupWizardDescriptor, function(elem) {
							var url = elem.url;
							if(url.indexOf(this.lpaURL) != -1) {
								this.lpaWizardDescriptorURL = url + "?minimal=true";
								this.whitelister.onSuccess = dojo.hitch(this, function() {
									this._getStepsForApplication(this.lpaWizardDescriptorURL);
								});
								this.whitelister.addURL(this.lpaWizardDescriptorURL);
								return;
							}
						}, this);
					}
				}));
			}
		}));
		deferred.addErrback(function(err) {
			//BootstrapProperties may be returning null for DISCOVERY_URL
			console.error("Unable to compute the Discovery URL. Error: " + err);
		});
	},

	//0.6 FRONTING APP (Not Embedded)
	_getStepsForApplication: function(wizardDescriptorURL) {
		if(wizardDescriptorURL) {
			jazz.client.xhrGet({
				url: wizardDescriptorURL,
				handleAs: "json",
				headers: this.headers,
				load: dojo.hitch(this, function(response) {
					this._addSteps(response);
					//Remove the URLs we temporarily added to the proxy whitelist
					this.whitelister.onSuccess = null;
					this.whitelister.removeURL([this.jtsURL, this.lpaWizardDescriptorURL]);
				})
			});
		}
	},
	
	//0.6 FRONTING APP (Not Embedded)
	_addSteps: function(appSteps) {
		var baseURL = appSteps.baseSetupURL;
		var steps = new Array();
		var count = appSteps.steps.length;
		dojo.forEach(appSteps.steps, function(step) {
			if(!step.stepURL) {
				step.stepURL = baseURL + "?embed=true#" + STEPS_URI_PREFIX + step.stepNumber;
			}
			count--;
			step.overrideFinish = (count === 0); //allow finish on last step
			steps.push(new ExternalWizardStep(step));
		});
		var group = { name: appSteps.application, steps: steps};
		this.wizard.addStepGroup(group);
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			this.loggedInUser = response.userId;
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this._startApp();
	},
	
	authRequestFailed: function(errorObject) {
		this._startApp();
	},
	
	_handleGetApplicationsFailed: function(errObject) {
		this.addSummaryStep();
		console.error(messages.getAppsFailed);
	},
	
	doNothing: function () {
		// do nothing
	}
});
var messages;
dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.LogoFooter", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/LogoFooter.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	}
});

function isEmbedded() {
	var qstr = window.location.search.substr(1);
	var params = dojo.queryToObject(qstr);
	return params && params.embed === "true";
}

})();