/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.WizardReview");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardReviewMessages");

(function() {

var WizardStep = jazz.ui.wizard.WizardStep;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var arrayIndexWithPropertyOfObject = com.ibm.team.repository.web.ui.internal.admin.util.arrayIndexWithPropertyOfObject;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var DW_SERVICE_NAME = "com.ibm.team.datawarehouse.service.internal.RemoteDatabaseService";
var DB_SERVICE_NAME = "com.ibm.team.repository.service.internal.rdb.ConnectionPoolService";
var DB_VENDOR_NAME = "db.vendor";
var EMAIL_SERVICE_NAME = "com.ibm.team.repository.service.jts.internal.mailer.MailerService";
var EMAIL_ENABLED_NAME = "notification.mail.enabled";
var PUBLIC_URL_SERVICE_NAME = "com.ibm.team.repository.service.internal.BaseServerDescriptionService";
var PUBLIC_URL_NAME = "server.webapp.url";
var TRIGGER_DIAGNOSTICS_SERVICE = "com.ibm.team.repository.service.internal.diagnostic.IBasicDiagnosticRestService";
var USER_REGISTRY_SERVICE_NAME = "com.ibm.team.repository.service.jts.internal.userregistry.ExternalUserRegistryInternalService";
var USER_REGISTRY_NAME = "user.registry.type";
var LDAP_SERVICE_NAME = "com.ibm.team.repository.service.jts.internal.userregistry.ldap.LDAPUserRegistryProvider";
var SETUP_HAS_RUN_PROPERTY = "completed.setup";
var SETUP_CONFIGURATION_SERVICE = "com.ibm.team.repository.service.internal.setup.SetupConfigurationService";

var messages;

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.WizardReview",  [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/WizardReview.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardReviewMessages");
		this.stepTitle = messages.stepTitle;
		this._callPending = 0;
		this._callReturned = 0;
	},

	postCreate: function() {
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		//Prepare for raw XHR requests
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		
		//Could improve this to be more robust...
		this.servicePrefix = net.jazz.ajax.contextRoot() + "/service/";
	},

	_refresh: function () {
		//Don't allow the same call to be pending more than once.
		if(this._callPending == this._callReturned) {
			this._summaryList.innerHTML = "";
			this._showConnectionStatus("INFO", messages.loading);
			this._callPending = 0;
			this._callReturned = 0;
			this._fetchSummary();
			this._triggerDiagnosticExecution();
			this.saveSetupRunStatus();
		}
	},
	
	_fetchSummary: function () {
		// db config
		var srh= new ServiceResponseHandler(this, "_handleDbProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, DB_SERVICE_NAME);
		this._callPending++;
		
		// public URL
		var srh= new ServiceResponseHandler(this, "_handlePublicUrlProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, PUBLIC_URL_SERVICE_NAME);
		this._callPending++;
		
		// data warehouse config
		var srh = new ServiceResponseHandler(this, "_handleDataWarehouseProps", "doNothing");
		ServerStatusClient.postStatusForServiceList(srh, {serviceNames: ["com.ibm.team.reports.common.internal.service.IReportRestService"]});
		this._callPending++;
		
		// email config
		var srh= new ServiceResponseHandler(this, "_handleEmailProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, EMAIL_SERVICE_NAME);
		this._callPending++;
		
		// user registry config
		var srh= new ServiceResponseHandler(this, "_handleUserRegistryProps", "_handleServiceFetchError");
		ServerStatusClient.getServiceConfiguration(srh, USER_REGISTRY_SERVICE_NAME);
		this._callPending++;
		
		// registered applications
		var srh = new ServiceResponseHandler(this, "_handleRegisteredApps", "_handleServiceFetchError");
		OAuthAdminClient.getFriends(srh, { includeInternal: true });
		this._callPending++;

	},
	
	_callReturnedCheckFinished: function () {
		this._callReturned++;
		if (this._callPending == this._callReturned) {
			this._showConnectionStatus("OK", this.messages.success);
		}
	},
	
	_handleDbProps: function(serviceInfo) {
		this._callReturnedCheckFinished();
		var vendor = "DB";
		for (var i=0; i<serviceInfo.properties.length; i++) {
			if (serviceInfo.properties[i].name === DB_VENDOR_NAME) {
				vendorProp = serviceInfo.properties[i]
				vendor = vendorProp.overrideValue != null ? vendorProp.overrideValue : vendorProp.defaultValue;
			}	
		};
		var dbStatus = vendor + messages.connectionSuccess; 
		var dbSummary = this._createConfigSummaryItem(messages.databaseConfiguration, dbStatus, serviceInfo.properties)
		this._summaryList.appendChild(dbSummary);
	},
	
	_handleDataWarehouseProps: function(response) {
		this._callReturnedCheckFinished();
		response = response[0];
		var hasReports = response.code == "OK" ? true : false;
		if (hasReports == true) {
			var srh = new ServiceResponseHandler(this, "_handleDataWarehouseCallback", "_handleServiceFetchError");
			ServerStatusClient.getServiceConfiguration(srh, DW_SERVICE_NAME);
			this._callPending++;
		}
	},
		
	_handleDataWarehouseCallback: function(serviceInfo) {
		this._callReturnedCheckFinished();
		var vendor = "DB";
		for (var i=0; i<serviceInfo.properties.length; i++) {
			if (serviceInfo.properties[i].name === DB_VENDOR_NAME) {
				vendorProp = serviceInfo.properties[i]
				vendor = vendorProp.overrideValue != null ? vendorProp.overrideValue : vendorProp.defaultValue;
			}	
		};
		var dbStatus = vendor + messages.connectionSuccess; 
		var dataWarehouseSummary = this._createConfigSummaryItem(messages.dataWarehouseConfiguration, dbStatus, serviceInfo.properties);
		this._summaryList.appendChild(dataWarehouseSummary);
	},
	
	_handlePublicUrlProps: function(serviceInfo) {
		this._callReturnedCheckFinished();
		var summaryLabel;
		for (var i=0; i<serviceInfo.properties.length; i++) {
			if (serviceInfo.properties[i].name === PUBLIC_URL_NAME) {
				var prop = serviceInfo.properties[i]
				summaryLabel = prop.overrideValue != null ? prop.overrideValue : prop.defaultValue;
			}	
		};
		
		if(summaryLabel == "") {
			summaryLabel = messages.notSpecified;
		}

		var urlSummary = this._createConfigSummaryItem(messages.jtsPublicUrl, summaryLabel, null)
		this._summaryList.appendChild(urlSummary);
	}, 
	
	_handleEmailProps: function(serviceInfo) {
		this._callReturnedCheckFinished();
		var emailDisabled = true;
		for (var i=0; i<serviceInfo.properties.length; i++) {
			if (serviceInfo.properties[i].name === EMAIL_ENABLED_NAME) {
				var emailProp = serviceInfo.properties[i]
				emailSetting = emailProp.overrideValue != null ? emailProp.overrideValue : emailProp.defaultValue;
				if (emailSetting === "true") {
					emailDisabled = false;
				}
			}	
		};
		var summaryLabel = messages.enabled;
		if (emailDisabled) {
			serviceInfo.properties = null;
			summaryLabel = messages.disabled;
		}
		var emailSummary = this._createConfigSummaryItem(messages.emailNotification, summaryLabel, serviceInfo.properties)
		this._summaryList.appendChild(emailSummary);
	}, 
	
	_handleUserRegistryProps: function(serviceInfo) {
		var index = arrayIndexWithPropertyOfObject(serviceInfo.properties, "name", "user.registry.type");
		var registryType = serviceInfo.properties[index];
		var registryTypeValue = registryType.overrideValue != null ? registryType.overrideValue : registryType.defaultValue;

		if (registryTypeValue === "LDAP") {
			var srh = new ServiceResponseHandler(this, "_handleLdapProps", "_handleServiceFetchError");
			ServerStatusClient.getServiceConfiguration(srh, LDAP_SERVICE_NAME);
			this._callPending++;
		} else {
			registryTypeLabel = messages[registryTypeValue];
			var userRegistrySummary = this._createConfigSummaryItem(messages.userRegistry, registryTypeLabel, null);
			this._summaryList.appendChild(userRegistrySummary);
		}
		this._callReturnedCheckFinished();
	},
	
	_handleLdapProps: function(serviceInfo) {
		this._callReturnedCheckFinished();
		var userRegistrySummary = this._createConfigSummaryItem(messages.userRegistry, "LDAP", serviceInfo.properties);
		this._summaryList.appendChild(userRegistrySummary);
	}, 
	
	_handleRegisteredApps: function(apps) {
		this._callReturnedCheckFinished();
		var numApps = 0;
		var properties = new Array();
		properties.apps = true;
		dojo.forEach(apps, function(app) {
			if(app.internal && app.internal == true) {
				var prop = {};
				prop.title = app.title;
				prop.url = app.scrURL;
				properties.push(prop);
				numApps++;		
			}
		});
		
		var label = dojo.string.substitute(this.messages.numAppsRegistered, [numApps]); 
		var appsSummary = this._createConfigSummaryItem(messages.registeredApps, label, properties);
		this._summaryList.appendChild(appsSummary);
	}, 
	
	_createConfigSummaryItem: function(label, summary, props){
		if (props != null && props.length > 0) {
			var item = dojo.create("a", { href: "#",  className: "summaryItem"});
		} else {
			var item = dojo.create("div", { className: "summaryItem" });
		}
		dojo.create("div", { className: "summaryItemLabel", innerHTML: label }, item);
		dojo.create("div", { className: "summaryItemStatus", innerHTML: summary }, item);
		var itemClear = dojo.create("div", { className: "summaryItemClear"}, item);		
		if (props != null && props.length > 0) {
			var itemProperties = props.apps ?  this._createAppsSection(props) : this._createDetailedProperties(props);
			itemClear.appendChild(itemProperties);
			dojo.addClass(itemProperties, "summaryItemProperties noDisplay");
			this.connect(item, "onclick", function(event){
				dojo.stopEvent(event);
				dojo.toggleClass(item, "expanded");
				dojo.toggleClass(itemProperties, "noDisplay");
			});
		} 		
		return item;
	}, 
	
	_createAppsSection: function(properties) {
		var tableDiv = document.createElement("table");
		var tableBody = dojo.create("tbody", {}, tableDiv);	
		var header = dojo.create("tr", {}, tableBody);
		dojo.create("th", { innerHTML: this.messages.applicationName }, header);
		dojo.create("th", { innerHTML: this.messages.scrURL }, header);	
		dojo.forEach(properties, function(app) {		
			var row = dojo.create("tr", {}, tableBody);
			var title = dojo.create("td", {}, row);
			title.appendChild(document.createTextNode(app.title));
			dojo.create("td", { innerHTML: app.url }, row);	
		});
		return tableDiv;	
	},
	
	_createDetailedProperties: function (properties) {
		var tableDiv = document.createElement("table");
		var tableBody = dojo.create("tbody", {}, tableDiv);
		var header = dojo.create("tr", {}, tableBody);
		dojo.create("th", { innerHTML: this.messages.property }, header);
		dojo.create("th", { innerHTML: this.messages.current }, header);					
		for (var i=0; i<properties.length; i++) {
			if (properties[i].overrideValue != null || properties[i].defaultValue != null) {
				var row = dojo.create("tr", {}, tableBody);
				dojo.create("td", { innerHTML: properties[i].displayName }, row);
				var value = properties[i].overrideValue != null ? properties[i].overrideValue : properties[i].defaultValue;
				if (properties[i].name === "db.jdbc.location"){
					// replace ; with <WBR> for wrapping
					value = value.replace(/;/g,";<wbr>");
					value = value.replace(/:/g,":<wbr>");
				}
				var valueCell = dojo.create("td", {}, row);
				dojo.create("div", { innerHTML: value }, valueCell);
			}	
		};	
		return tableDiv;
	},
	
	_showConnectionStatus: function(type, summary) {
		this.header.showMessage({type: type, summary: summary})
	}, 
	
	_handleServiceFetchError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	}, 
	
	_triggerDiagnosticExecution: function() {
		jazz.client.xhrGet({
			url: this.servicePrefix + TRIGGER_DIAGNOSTICS_SERVICE + "/diagnosticExecution",
			handleAs: "text",
			headers: this.headers,
			load: function(response) {
				//do nothing
			}
		});
	},
	
	//The server needs to know that setup has already been run so that Express Setup can be disabled
	saveSetupRunStatus: function() {
		var updates = [{
			"name" : SETUP_HAS_RUN_PROPERTY,
			"value" : "true",
			"serviceImplementationName" : SETUP_CONFIGURATION_SERVICE
		}];
		var changeArgs = {
			updates: dojo.toJson(updates)
		};
		var srh = new ServiceResponseHandler(this, "doNothing", "_handleServiceFetchError");
		ServerStatusClient.postConfigurationUpdates(srh, changeArgs);	
	},
	
	doNothing: function() {
		//nothing to do here
	},
	
	// SUPER WizardStep functions
	focusStep: function () {
		this._refresh();
	}, 
	
	canFinish: function() {
		return true;
	}
	
});

})();