#!/bin/sh
#*******************************************************************************
# Licensed Materials - Property of IBM
# (c) Copyright IBM Corporation 2011. All Rights Reserved.
#
# Note to U.S. Government Users Restricted Rights:  
# Use, duplication or disclosure restricted by GSA ADP Schedule 
# Contract with IBM Corp. 
#*******************************************************************************

#############
# Functions #
#############

# Help
help() {
	echo "Usage: tvt.sh {-ccm | -qm | -rm} -oldAppli <2.x install dir> -newAppli <3.0.1 install dir> [-jts <JTS 3.0.1 install dir>]"
	echo "Examples:"
	echo " - RTC: tvt.sh -ccm -old /opt/IBM.2x/RTC2002if5/jazz -new /data/IBM/JazzTeamServer"
	echo " - RQM: tvt.sh -qm -old /opt/IBM.2x/RQM201 -new /data/IBM/RQM.301 -jts /data/IBM/JazzTeamServer -tar /data/IBM/import/new_POT_rqm.tar"
	exit 1
}

# Run the current step
runStep() {
	echo ""
	echo "Commande: $STEP_CMD"
	echo ""
	confirm_step
	CONFIRMED=$?
	if [ $CONFIRMED -eq 1 ]; then
		cd $1
		$STEP_CMD
		ERROR_REPOTOOLS=$?
	else
		ERROR_REPOTOOLS=0
	fi
	return $CONFIRMED
}

# Confirm step
confirm_step() {
	answer=
	read -n 1 -p "[E] to execute, [S] to skip or [Enter] to cancel the TVT scenario execution: " answer
	echo ""
	if [ "$answer" = "s" ] || [ "$answer" = "S" ]; then
		return 0
	elif [ "$answer" = "" ]; then
		exit 2
	elif [ "$answer" != "e" ] && [ "$answer" != "E" ]; then
		echo "'$answer' is an invalid entry, only 'E', 'S' or 'Enter' are allowed"
		confirm_step
		return $?
	fi
	return 1
}

if [ $# -eq 0 ]; then
	echo "Missing parameter: expected -ccm, -qm or -rm!"
	help
fi
case $1 in
	-ccm | -rm | -qm)
		appli=`echo $1 | cut -d'-' -f2`
		shift
		;;
	*)
		echo "$1 is an invalid parameter. Expected -ccm, -qm or -rm!"
		help
		;;
esac

# Remaining parameters
while [ $# -ne 0 ];
do
	case $1 in
		-old)
			shift
			if [ ! \(  -d $1 \) ] ; then
				echo "$1 is an invalid directory."
				help
			fi
			if [ ! \(  -d $1/server \) ] ; then
				echo "$1 seems not to point on the $appli 2.x installation directory as sub-dir server was not found."
				help
			fi
			if [ ! \(  -f $1/server/repotools.sh \) ] ; then
				echo "$1 seems not to point on the $appli 2.x installation directory as server/repotools.sh was not found."
				help
			fi
			OLD_DIR="$1/server"
			;;
		-new)
			shift
			if [ ! \(  -d $1 \) ] ; then
				echo "$1 is an invalid directory."
				help
			fi
			if [ ! \(  -d $1/server \) ] ; then
				echo "$1 seems not to point on the $appli installation directory as sub-dir server was not found."
				help
			fi
			if [ $appli = rm ]; then
				ctxr=jts
			else
				ctxr=jazz
			fi
			if [ ! \(  -f $1/server/repotools-$ctxr.sh \) ] ; then
				echo "$1 seems not to point on the $appli installation directory as server/repotools-$ctxr.sh was not found."
				help
			fi
			APP_DIR="$1/server"
			;;
		-jts)
			shift
			if [ ! \(  -d $1 \) ] ; then
				echo "$1 is an invalid directory."
				help
			fi
			if [ ! \(  -d $1/server \) ] ; then
				echo "$1 seems not to point on the JTS installation directory as sub-dir server was not found."
				help
			fi
			if [ ! \(  -f $1/server/repotools-jts.sh \) ] ; then
				echo "$1 seems not to point on the JTS installation directory as server/repotools-jts.sh was not found."
				help
			fi
			JTS_DIR="$1/server"
			;;
		-tar)
			shift
			if [ ! \(  -f $1 \) ] ; then
				echo "$1 is an invalid file."
				help
			fi
			TAR_FILE="$1"
			;;
		-lang)
			export LC_ALL=
			LANG=$2
			shift
			locale
			read -n 1 -p "Hit any key to continue..."
			;;
		-lc_all)
			LANG=
			export LC_ALL=$2
			shift
			locale
			read -n 1 -p "Hit any key to continue..."
			;;
		*)
			echo "Ignoring unknown parameter $1..."
			;;
	esac
	shift
done

# Old application install dir
if [ "$OLD_DIR" = "" ]; then
	echo "Missing 2.x $appli installation directory!"
	help
fi

# New application install dir
if [ "$APP_DIR" = "" ]; then
	echo "Missing $appli installation directory!"
	help
fi
cd $APP_DIR

# Run TVT scenario
case $appli in
	ccm)
		STEP_CMD="app_upgrade.sh"
		runStep $APP_DIR/upgrade
		STEP_CMD="${appli}_upgrade.sh"
		runStep $APP_DIR/upgrade
		STEP_CMD="${appli}_upgrade.sh -help"
		runStep $APP_DIR/upgrade
		STEP_CMD="upgrade/${appli}_upgrade.sh -help"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -unknown"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplication /tmp"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -newJTSHome ."
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -newApplicationHome /data/IBM/JazzTeamServer/server"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome .."
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome unknownDir"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -noPrompt"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -noPrompt -overwrite -createJTSTables"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -noPrompt"
		runStep $APP_DIR
		;;
	qm)
		STEP_CMD="upgrade/${appli}_upgrade.sh -help"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -importFromFile"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -importFromFile unknownFile"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -importFromFile $TAR_FILE"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -newJTSHome $JTS_DIR/conf -importFromFile $TAR_FILE"
		runStep $APP_DIR
		STEP_CMD="upgrade/${appli}_upgrade.sh -oldApplicationHome $OLD_DIR/conf -newJTSHome $JTS_DIR/conf -importFromFile $TAR_FILE"
		runStep $APP_DIR
		;;
	rm)
		echo "Not supported yet!"
		;;
esac

	
